# -*- coding: utf-8 -*-
"""
vcolorcboxplugin.py

A color comboBox custom widget plugin for Qt Designer.

Copyright (C) 2012-2016 Vincent Vande Vyvre. <vincent.vandevyvre@oqapy.eu>

GNU General Public License Usage
This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
"""

from PyQt5 import QtGui

from PyQt5.QtDesigner import QPyDesignerCustomWidgetPlugin
from VWidgets_PyQt5.vcolorcombobox import VColorComboBox


class VColorComboBoxPlugin(QPyDesignerCustomWidgetPlugin):
    """Provides a Python custom plugin for Qt Designer by implementing the
    QDesignerCustomWidgetPlugin using a PyQt-specific custom plugin class.
    """

    # The __init__() method is only used to set up the plugin and define its
    # initialized variable.
    def __init__(self, parent = None):
    
        QPyDesignerCustomWidgetPlugin.__init__(self)
        self.initialized = False

    # The initialize() and isInitialized() methods allow the plugin to set up
    # any required resources, ensuring that this can only happen once for each
    # plugin.
    def initialize(self, formEditor):

        if self.initialized:
            return
        
        self.initialized = True

    def isInitialized(self):
        return self.initialized

    # This factory method creates new instances of our custom widget with the
    # appropriate parent.
    def createWidget(self, parent):
        return VColorComboBox(parent)

    # This method returns the name of the custom widget class that is provided
    # by this plugin.
    def name(self):
        return "VColorComboBox"

    # Returns the name of the group in Qt Designer's widget box that this
    # widget belongs to.
    def group(self):
        return "VWidgets PyQt5"

    # Returns the icon used to represent the custom widget in Qt Designer's
    # widget box.
    def icon(self):
        return QtGui.QIcon(_logo_pixmap)

    # Returns a short description of the custom widget for use in a tool tip.
    def toolTip(self):
        return "Color ComboBox"

    # Returns a short description of the custom widget for use in a "What's
    # This?" help message for the widget.
    def whatsThis(self):
        return "ComboBox wich present a color palette"

    # Returns True if the custom widget acts as a container for other widgets;
    # otherwise returns False. Note that plugins for custom containers also
    # need to provide an implementation of the QDesignerContainerExtension
    # interface if they need to add custom editing support to Qt Designer.
    def isContainer(self):
        return False

    # Returns an XML description of a custom widget instance that describes
    # default values for its properties. Each custom widget created by this
    # plugin will be configured using this description.
    def domXml(self):
        return '<widget class="VColorComboBox" name="colorComboBox" />\n'

    # Returns the module containing the custom widget class. It may include
    # a module path.
    def includeFile(self):
        return "VWidgets_PyQt5.vcolorcombobox"


# Define the image used for the icon.
colorcbbox_32x44_xpm = [ "32 44 127 2",
"#8 c #4d4d4e", ".B c #8d8d8f", ".J c #2e2e2f", "Qt c #121212",
".v c #1f1f1f", ".e c #1a1a1b", ".# c #161616", ".a c #181818",
".b c #000000", "#Q c #0000e7", "#R c #0000ff", "#g c #00e700",
"#h c #00ff00", ".d c #020202", ".z c #040404", ".c c #060606",
".y c #080808", ".p c #101010", ".i c #121212", "#M c #1313a5",
"#N c #1515b6", ".A c #1b1b1b", ".j c #1f1f1f", ".x c #1f1f20",
".w c #242424", "#H c #2ba02c", "#I c #2fad30", "#U c #303031",
".s c #343434", "#X c #3a3a3a", ".0 c #3d3d3d", "#t c #434344",
".I c #444445", ".t c #474747", ".m c #494949", ".F c #4f4f50",
"#u c #515152", ".2 c #545455", ".8 c #555556", "#w c #575759",
"#Z c #585859", ".H c #5a5a5b", "#3 c #5c5c5d", ".n c #5d5d5d",
"#c c #60a561", ".f c #626262", ".1 c #646465", "#d c #64ab65",
".6 c #666667", "#O c #696999", "#S c #6969a4", "#i c #69a36a",
"#2 c #6a6a6b", "#v c #6c6c6d", "#j c #6d6d6e", "#1 c #747476",
".5 c #757577", "#V c #767678", "#J c #79a27b", ".u c #7d7d7c",
"#5 c #7d7da9", "#6 c #7e7eab", "#r c #828283", "#0 c #878789",
".7 c #8c8c8e", "#W c #909092", "#s c #929294", "#B c #979799",
"#p c #99999b", ".Q c #9a6768", "#m c #9a9a9c", ".o c #a1a1a0",
".W c #a1a1a3", "#e c #a1bca3", ".h c #a2a2a2", ".U c #a3696a",
"#y c #a4a4a6", "#q c #a8a8aa", "## c #a9888a", "#l c #a9a9ab",
".Z c #acacae", "#x c #afafb1", ".3 c #b2b2b4", "#D c #b4b4b6",
"#P c #b4b4c2", "#f c #b4c0b6", ".O c #b50b0b", "#k c #b7b7b9",
"#L c #b7b7c4", "#7 c #b7b7cb", "#o c #b8b8ba", ".4 c #b8b8bb",
"#z c #bababd", "#T c #bbbbbe", ".9 c #bcbcbf", "#E c #bfbfc2",
".R c #c0b4b6", ".N c #c1b5b7", ".q c #c1c1c0", "#G c #c1cac3",
".E c #c2c2c4", ".P c #c70c0c", "#n c #c7c7c9", "#C c #c8c8ca",
"#a c #cec1c3", ".G c #d0d0d3", "#Y c #d5d5d8", "#b c #d5dad8",
".Y c #d6d6d9", ".g c #d7d7d6", ".V c #d7d7da", "#F c #d9d9dc",
".X c #dadadd", ".L c #dedee1", "#4 c #dfdfe6", ".k c #e3e3e2",
"#. c #e6e3e6", ".S c #e70000", "#A c #e9e9eb", ".M c #ececef",
"#K c #ededf0", ".K c #f3f3f6", ".D c #f4f4f7", ".C c #f5f5f8",
".r c #f9f9f8", ".T c #ff0000", ".l c #fffffe",
"Qt.#.#.#.#.#.#.#.#.#.#.#.#.#.#.#.#.#.#.#.#.#.#.#.#.#.#.#.#.#.#Qt",
".a.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.c.c.c.c.c.c.d.b.e",
".a.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.f.g.g.g.g.h.i.b.e",
".a.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.j.k.l.l.l.m.b.b.e",
".a.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.n.l.l.o.c.b.b.e",
".a.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.p.q.r.s.b.b.b.e",
".a.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.t.u.b.b.b.b.e",
".v.w.w.w.w.w.w.w.w.w.w.w.w.w.w.w.w.w.w.w.x.y.b.b.b.z.A.b.b.b.b.e",
".B.C.C.C.C.C.C.C.C.C.C.C.C.C.C.C.C.C.C.C.D.E.i.b.b.b.b.b.b.b.b.e",
".B.C.C.C.C.C.C.C.C.C.C.C.C.C.C.C.C.C.C.C.C.C.F.b.b.b.b.b.b.b.b.e",
".B.C.C.C.C.C.C.C.C.C.C.C.C.C.C.C.C.C.C.C.C.C.G.H.t.I.I.I.I.I.I.J",
".B.C.C.C.C.C.C.C.C.C.C.C.C.C.C.C.C.C.C.C.C.C.C.C.C.C.C.C.C.C.C.B",
".B.K.L.L.L.L.L.L.L.L.L.M.C.C.C.C.C.C.C.C.C.C.C.C.C.C.C.C.C.C.C.B",
".B.N.O.P.P.P.P.P.P.P.P.Q.C.C.C.C.C.C.C.C.C.C.C.C.C.C.C.C.C.C.C.B",
".B.R.S.T.T.T.T.T.T.T.T.U.C.V.W.C.C.X.Y.C.C.C.C.C.C.C.C.C.C.C.C.B",
".B.R.S.T.T.T.T.T.T.T.T.U.C.Z.0.1.1.2.3.C.C.C.C.C.C.C.C.C.C.C.C.B",
".B.R.S.T.T.T.T.T.T.T.T.U.C.4.5.6.7.8.9.C.C.C.C.C.C.C.C.C.C.C.C.B",
".B.R.S.T.T.T.T.T.T.T.T.U.C.C.C.C.C.C.C.C.C.C.C.C.C.C.C.C.C.C.C.B",
".B#.###################a.C.C.C.C.C.C.C.C.C.C.C.C.C.C.C.C.C.C.C.B",
".B.C.C.C.C.C.C.C.C.C.C.C.C.C.C.C.C.C.C.C.C.C.C.C.C.C.C.C.C.C.C.B",
".B.C.C.C.C.C.C.C.C.C.C.C.C.C.C.C.C.C.C.C.C.C.C.C.C.C.C.C.C.C.C.B",
".B.C.C.C.C.C.C.C.C.C.C.C.C.C.C.C.C.C.C.C.C.C.C.C.C.C.C.C.C.C.C.B",
".B#b#c#d#d#d#d#d#d#d#d#e.C.C.C.C.C.C.C.C.C.C.C.C.C.C.C.C.C.C.C.B",
".B#f#g#h#h#h#h#h#h#h#h#i.C.C.C.C.C.C.C.C.C.C.C.C.C.C.C.C.C.C.C.B",
".B#f#g#h#h#h#h#h#h#h#h#i.C.G#j#k#l#m#n#o#p.C.C.C.C.C.C.C.C.C.C.B",
".B#f#g#h#h#h#h#h#h#h#h#i.C#q#r.8#s#t#u#v#w#x.C.C.C.C.C.C.C.C.C.B",
".B#f#g#h#h#h#h#h#h#h#h#i.C.D#y#z#A#B#C#D#E#F.C.C.C.C.C.C.C.C.C.B",
".B#G#H#I#I#I#I#I#I#I#I#J.C.C.C.C.C.C.C.C.C.C.C.C.C.C.C.C.C.C.C.B",
".B.C.C.C.C.C.C.C.C.C.C.C.C.C.C.C.C.C.C.C.C.C.C.C.C.C.C.C.C.C.C.B",
".B.C.C.C.C.C.C.C.C.C.C.C.C.C.C.C.C.C.C.C.C.C.C.C.C.C.C.C.C.C.C.B",
".B.C.C.C.C.C.C.C.C.C.C.C.C.C.C.C.C.C.C.C.C.C.C.C.C.C.C.C.C.C.C.B",
".B.C#K#K#K#K#K#K#K#K#K.K.C.C.C.C.C.C.C.C.C.C.C.C.C.C.C.C.C.C.C.B",
".B#L#M#N#N#N#N#N#N#N#N#O.C.C.C.C.C.C.C.C.C.C.C.C.C.C.C.C.C.C.C.B",
".B#P#Q#R#R#R#R#R#R#R#R#S.C.V.7#T.C.C.C.C.C.C.C.C.C.C.C.C.C.C.C.B",
".B#P#Q#R#R#R#R#R#R#R#R#S.C.Z#U#V#W#V#X#Y.C.C.C.C.C.C.C.C.C.C.C.B",
".B#P#Q#R#R#R#R#R#R#R#R#S.C#o#Z#0#1#2#3#K.C.C.C.C.C.C.C.C.C.C.C.B",
".B#P#Q#R#R#R#R#R#R#R#R#S.C.C.C.C.C.C.C.C.C.C.C.C.C.C.C.C.C.C.C.B",
".B#4#5#6#6#6#6#6#6#6#6#7.C.C.C.C.C.C.C.C.C.C.C.C.C.C.C.C.C.C.C.B",
".B.C.C.C.C.C.C.C.C.C.C.C.C.C.C.C.C.C.C.C.C.C.C.C.C.C.C.C.C.C.C.B",
".B.C.C.C.C.C.C.C.C.C.C.C.C.C.C.C.C.C.C.C.C.C.C.C.C.C.C.C.C.C.C.B",
".B.C.C.C.C.C.C.C.C.C.C.C.C.C.C.C.C.C.C.C.C.C.C.C.C.C.C.C.C.C.C.B",
".B.C.C.C.C.C.C.C.C.C.C.C.C.C.C.C.C.C.C.C.C.C.C.C.C.C.C.C.C.C.C.B",
".B.C.C.C.C.C.C.C.C.C.C.C.C.C.C.C.C.C.C.C.C.C.C.C.C.C.C.C.C.C.C.B",
"#8.B.B.B.B.B.B.B.B.B.B.B.B.B.B.B.B.B.B.B.B.B.B.B.B.B.B.B.B.B.B#8"]


_logo_pixmap = QtGui.QPixmap(colorcbbox_32x44_xpm)
