# -*- coding: utf-8 -*-

"""
vlineeditplugin.py

A line edit custom widget plugin for Qt Designer.

Copyright (C) 2012-2016 Vincent Vande Vyvre. <vincent.vandevyvre@oqapy.eu>

GNU General Public License Usage
This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
"""

from PyQt5 import QtGui

from PyQt5.QtDesigner import QPyDesignerCustomWidgetPlugin
from VWidgets_PyQt5.vlineedit import VLineEdit


class VLineEditPlugin(QPyDesignerCustomWidgetPlugin):
    """Provides a Python custom plugin for Qt Designer by implementing the
    QDesignerCustomWidgetPlugin using a PyQt-specific custom plugin class.
    """

    # The __init__() method is only used to set up the plugin and define its
    # initialized variable.
    def __init__(self, parent = None):
    
        QPyDesignerCustomWidgetPlugin.__init__(self)
        self.initialized = False

    # The initialize() and isInitialized() methods allow the plugin to set up
    # any required resources, ensuring that this can only happen once for each
    # plugin.
    def initialize(self, formEditor):

        if self.initialized:
            return
        
        self.initialized = True

    def isInitialized(self):
        return self.initialized

    # This factory method creates new instances of our custom widget with the
    # appropriate parent.
    def createWidget(self, parent):
        return VLineEdit(parent)

    # This method returns the name of the custom widget class that is provided
    # by this plugin.
    def name(self):
        return "VLineEdit"

    # Returns the name of the group in Qt Designer's widget box that this
    # widget belongs to.
    def group(self):
        return "VWidgets PyQt5"

    # Returns the icon used to represent the custom widget in Qt Designer's
    # widget box.
    def icon(self):
        return QtGui.QIcon(_logo_pixmap)

    # Returns a short description of the custom widget for use in a tool tip.
    def toolTip(self):
        return "Line edit"

    # Returns a short description of the custom widget for use in a "What's
    # This?" help message for the widget.
    def whatsThis(self):
        return "Single line editor with custom buttons"

    # Returns True if the custom widget acts as a container for other widgets;
    # otherwise returns False. Note that plugins for custom containers also
    # need to provide an implementation of the QDesignerContainerExtension
    # interface if they need to add custom editing support to Qt Designer.
    def isContainer(self):
        return False

    # Returns an XML description of a custom widget instance that describes
    # default values for its properties. Each custom widget created by this
    # plugin will be configured using this description.
    def domXml(self):
        return '<widget class="VLineEdit" name="lineEdit" />\n'

    # Returns the module containing the custom widget class. It may include
    # a module path.
    def includeFile(self):
        return "VWidgets_PyQt5.vlineedit"

lineEdit_15x75_xpm = [ "75 15 131 2",
"Qt c None", ".# c #000000", ".i c #0a0a0a", ".d c #121212",
".a c #151515", ".o c #121212", ".b c #0c0c0c", ".e c #232323",
".j c #353535", ".c c #080808", ".F c #070707", "#9 c #010101",
".W c #0d0d0d", ".U c #121212", ".V c #141414", "#U c #1b1b1b",
".X c #1d1d1d", "#J c #202020", "#5 c #242424", "#j c #2e2e2e",
"#x c #303030", "a. c #393939", "a# c #3e3e3e", "#y c #515151",
"#i c #535353", "#3 c #555555", "#f c #575757", "#6 c #626262",
"#K c #6a6a6a", "#T c #878787", "#V c #8e8e8d", "#4 c #959594",
".Y c #989897", ".u c #9b9b9b", ".6 c #9c9c9c", ".C c #9d9d9c",
".2 c #9d9d9d", ".3 c #9e9e9e", ".8 c #9f9f9f", ".5 c #a0a0a0",
".1 c #a1a1a1", ".t c #a2a2a2", ".v c #a3a3a3", ".9 c #a4a4a4",
".R c #a5a5a5", "#u c #a6a6a6", ".0 c #a7a7a7", "#t c #a8a8a8",
".7 c #a9a9a9", "#v c #aaaaaa", ".z c #ababab", "#m c #acacac",
"#E c #adadad", "#b c #aeaeae", "#H c #afafaf", "#Q c #b0b0b0",
"#c c #b1b1b1", ".f c #b3b3b2", ".4 c #b3b3b3", "#W c #b5b5b5",
"#q c #b6b6b6", ".T c #b7b7b6", "#N c #b7b7b7", "#n c #b8b8b8",
"#8 c #b9b9b8", "#S c #b9b9b9", ".P c #bababa", "#0 c #bbbbbb",
".J c #bcbcbc", "#k c #bdbdbc", "#l c #bdbdbd", "#G c #bebebe",
"#A c #bfbfbf", "#w c #c0c0bf", "#r c #c0c0c0", "#g c #c1c1c0",
"#p c #c1c1c1", "#O c #c2c2c2", ".H c #c3c3c3", "#B c #c4c4c4",
".Q c #c5c5c5", ".Z c #c7c7c7", ".q c #c8c8c8", "#X c #c9c9c9",
"#D c #cacaca", "#s c #cbcbcb", ".K c #cccccc", "aa c #cdcdcc",
"#o c #cdcdcd", "#L c #cececd", "#R c #cecece", ".p c #cfcfce",
"#d c #d0d0cf", "#a c #d2d2d1", "#e c #d3d3d2", "#F c #d4d4d3",
".y c #d5d5d4", "#M c #d7d7d6", "#2 c #d9d9d8", "#P c #dbdbda",
"## c #dcdcdb", "#1 c #dddddc", "#h c #dededd", "#z c #dfdfde",
"#Z c #e1e1e0", "#I c #e2e2e1", ".w c #e4e4e3", "#C c #e6e6e5",
".m c #e9e9e8", "#Y c #eaeae9", "#7 c #ebebea", ".x c #ececeb",
".s c #ededec", ".g c #eeeeed", ".D c #efefee", ".E c #f0f0ef",
".B c #f1f1f0", ".n c #f2f2f1", ".L c #f3f3f2", "#. c #f4f4f3",
".O c #f5f5f4", ".I c #f6f6f5", ".S c #f7f7f6", ".A c #f8f8f7",
".l c #f9f9f8", ".r c #fafaf9", ".N c #fbfbfa", ".k c #fcfcfb",
".G c #fdfdfc", ".M c #fefefd", ".h c #fffffe",
"QtQt.#.a.b.c.c.c.c.c.c.c.c.c.c.c.c.c.c.c.c.c.c.c.c.c.c.c.c.c.c.c.c.c.c.c.c.c.c.c.c.c.c.c.c.c.c.c.c.c.c.c.c.c.c.c.c.c.c.c.c.c.c.c.c.c.c.c.c.c.b.a.#QtQt",
"Qt.d.e.f.g.h.h.h.h.h.h.h.h.h.h.h.h.h.h.h.h.h.h.h.h.h.h.h.h.h.h.h.h.h.h.h.h.h.h.h.h.h.h.h.h.h.h.h.h.h.h.h.h.h.h.h.h.h.h.h.h.h.h.h.h.h.h.h.h.h.g.f.e.dQt",
".i.j.k.h.h.h.h.h.h.h.h.h.l.m.n.h.h.h.h.h.h.h.h.h.h.h.h.h.h.h.h.h.h.h.h.h.h.h.h.h.h.h.h.h.h.h.h.h.h.h.h.h.h.h.h.h.h.h.h.h.h.h.h.h.h.h.h.h.h.h.h.h.k.j.i",
".o.p.h.h.g.q.r.h.h.h.h.s.t.u.v.h.h.h.h.h.h.h.h.h.h.h.h.h.h.h.h.h.h.h.w.x.h.y.z.A.h.h.h.h.h.h.h.h.h.h.h.h.h.h.h.h.h.h.h.h.h.h.h.B.C.D.C.E.h.h.h.h.h.p.o",
".F.G.h.h.H.u.g.I.D.k.h.J.u.K.w.k.B.h.h.G.D.L.M.n.n.G.D.N.h.h.M.B.O.I.u.P.r.Q.R.E.h.h.S.h.G.E.k.h.h.h.h.h.h.h.h.h.h.h.h.h.h.h.h.T.U.V.W.X.Y.k.h.h.h.G.F",
".c.h.h.h.z.t.Z.u.u.0.I.1.2.u.K.3.u.4.O.5.u.6.7.u.u.8.u.9#.##.t.u.6#a.3.u#b.v#c.u.w#d.u#e.5.u.0.l.h.h.h.h.h.h.h.h.h.h.h.h.h.h.h.T#f#d#g#h#i#j#k.h.h.h.c",
".c.h.h.S.u.P.0.R#l.u.p.u.4.Z.u#m#n.u#o.u#p.K.u#q.9.5#r.u#s.u.7#t.u.H.u.Z#a.u#n.u.y.R#u.3.t#v.u.m.h.h.h.h.h.h.h.h.h.h.h.h.h.h.h.T#f.h.h.h.h#w#x#y#h.h.c",
".c.h.h#z.u.K.u#A#d.u#B.u#z#s.u#C#D.u.P.u.D.y.u##.v#E#F.u#q.u.g.R#m#b.t.h#a.u.p.u#u.u#G.2.u.u#H.G.h.h.h.h.h.h.h.h.h.h.h.h.h.h.h.T#f.h.h.h.h.h#I#J#K.h.c",
".c.h.h.Z.u#L.u#M#n.6.P.u.S.Z.u#d.t.R#v.t.h#l.u.w.u.H#l.u#N.u#F.u#O.8.z#P#n.u.E.u.u#Q#R.u#m#S.E.h.h.h.h.h.h.h.h.h.h.h.h.h.h.h.h.T#f.h.h.h.A#T#U#V.N.h.c",
".c.h.h#W.1#X.u.D#u#Q.v.z.h#Y.2.u.8#Z.2#0.h.z.7#h.u#1.z#m#2.u.u.u.H.4.u.u.R#m.h.0.9.D.g.5.u.u.y.h.h.h.h.h.h.h.h.h.h.h.h.h.h.h.h.T#3.u#4#b#5#6.m.h.h.h.c",
".F.G.h#..n.r.D.h.n#M.u.K.h.h.S.w.l.h.D.l.h.L.O.M#7.h.n.I.h.s#C.D.l.k.w#7.n.I.h#..n.h.h.I#I#7.h.h.h.h.h.h.h.h.h.h.h.h.h.h.h.h.h#8#9a.#9a#aa.h.h.h.h.G.F",
".o.p.h.h.h.h.h.h.h#P#S.r.h.h.h.h.h.h.h.h.h.h.h.h.h.h.h.h.h.h.h.h.h.h.h.h.h.h.h.h.h.h.h.h.h.h.h.h.h.h.h.h.h.h.h.h.h.h.h.h.h.h.h.h.y.h.y.h.h.h.h.h.h.p.o",
".i.j.k.h.h.h.h.h.h.h.h.h.h.h.h.h.h.h.h.h.h.h.h.h.h.h.h.h.h.h.h.h.h.h.h.h.h.h.h.h.h.h.h.h.h.h.h.h.h.h.h.h.h.h.h.h.h.h.h.h.h.h.h.h.h.h.h.h.h.h.h.h.k.j.i",
"Qt.d.e.f.g.h.h.h.h.h.h.h.h.h.h.h.h.h.h.h.h.h.h.h.h.h.h.h.h.h.h.h.h.h.h.h.h.h.h.h.h.h.h.h.h.h.h.h.h.h.h.h.h.h.h.h.h.h.h.h.h.h.h.h.h.h.h.h.h.h.g.f.e.dQt",
"QtQt.#.a.b.c.c.c.c.c.c.c.c.c.c.c.c.c.c.c.c.c.c.c.c.c.c.c.c.c.c.c.c.c.c.c.c.c.c.c.c.c.c.c.c.c.c.c.c.c.c.c.c.c.c.c.c.c.c.c.c.c.c.c.c.c.c.c.c.c.b.a.#QtQt"]

_logo_pixmap = QtGui.QPixmap(lineEdit_15x75_xpm)
