# -*- coding: utf-8 -*-

"""
vtoolheaderplugin.py

A header bar custom widget plugin for Qt Designer.

Copyright (C) 2012-2016 Vincent Vande Vyvre. <vincent.vandevyvre@oqapy.eu>

GNU General Public License Usage
This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
"""


from PyQt5.QtGui import QPixmap, QIcon
from PyQt5.QtDesigner import QPyDesignerCustomWidgetPlugin

from VWidgets_PyQt5.vtoolheader import VToolHeader


class VToolHeaderPlugin(QPyDesignerCustomWidgetPlugin):
    """Provides a Python custom plugin for Qt Designer by implementing the
    QDesignerCustomWidgetPlugin using a PyQt-specific custom plugin class.
    """

    # The __init__() method is only used to set up the plugin and define its
    # initialized variable.
    def __init__(self, parent=None):
    
        QPyDesignerCustomWidgetPlugin.__init__(self)
        self.initialized = False

    # The initialize() and isInitialized() methods allow the plugin to set up
    # any required resources, ensuring that this can only happen once for each
    # plugin.
    def initialize(self, formEditor):

        if self.initialized:
            return

        manager = formEditor.extensionManager()
        
        self.initialized = True

    def isInitialized(self):
        return self.initialized

    # This factory method creates new instances of our custom widget with the
    # appropriate parent.
    def createWidget(self, parent):
        return VToolHeader(parent)

    # This method returns the name of the custom widget class that is provided
    # by this plugin.
    def name(self):
        return "VToolHeader"

    # Returns the name of the group in Qt Designer's widget box that this
    # widget belongs to.
    def group(self):
        return "VWidgets PyQt5"

    # Returns the icon used to represent the custom widget in Qt Designer's
    # widget box.
    def icon(self):
        print('header', _logo_pixmap, _logo_pixmap.isNull())
        return QIcon(_logo_pixmap)

    # Returns a short description of the custom widget for use in a tool tip.
    def toolTip(self):
        return "Tool Header"

    # Returns a short description of the custom widget for use in a "What's
    # This?" help message for the widget.
    def whatsThis(self):
        return "Header bar for tool box"

    # Returns True if the custom widget acts as a container for other widgets;
    # otherwise returns False. Note that plugins for custom containers also
    # need to provide an implementation of the QDesignerContainerExtension
    # interface if they need to add custom editing support to Qt Designer.
    def isContainer(self):
        return False

    # Returns an XML description of a custom widget instance that describes
    # default values for its properties. Each custom widget created by this
    # plugin will be configured using this description.
    def domXml(self):
        return '<widget class="VToolHeader" name="toolHeader" />\n'

    # Returns the module containing the custom widget class. It may include
    # a module path.
    def includeFile(self):
        return "VWidgets_PyQt5.vtoolheader"


# Define the image used for the icon.
vtoolheader_xpm = [ "118 20 299 2",
"Qt c #a2a2a1", ".a c #bebebd", ".# c #bfbfbe", "#d c #000000",
"#y c #020202", "#4 c #0b0b0b", "#c c #212221", "#D c #222322",
"ad c #242423", "#e c #262626", "a6 c #272727", ".0 c #313130",
"#K c #373837", "bx c #383938", "ba c #414140", "b. c #424241",
"aB c #434342", "#z c #444443", "a7 c #464645", "be c #464746",
"aQ c #474846", "aN c #474847", "aG c #4a4a49", "#9 c #50504f",
"aU c #515150", "aD c #515251", "ao c #525352", "bg c #535352",
"am c #545453", "bh c #585857", "#i c #5b5c5a", "aL c #5d5e5c",
"#5 c #5e5e5d", "ag c #5e5f5e", "aa c #5f5f5e", "bH c #5f605f",
"bK c #606160", "a8 c #626261", "bz c #656564", "aj c #656665",
".1 c #666765", "bA c #676866", "#o c #696a68", "#7 c #6a6a69",
"an c #6b6b6a", "ak c #6b6c6a", "ab c #6b6c6b", "ae c #6e6e6d",
"bJ c #6f6f6d", "ai c #6f706e", "bI c #70706f", "b4 c #717170",
"b6 c #737271", "#E c #737372", "#8 c #737472", "#f c #747574",
"bE c #757574", "#l c #757674", "a# c #767775", "bu c #767776",
"bC c #797978", "bB c #797a78", "ac c #7a7a79", "b1 c #7a7b79",
"cc c #7b7b7a", "a5 c #7b7c7a", "#O c #7c7c7b", "c# c #7c7d7c",
"ca c #7d7e7c", "ce c #7e7e7c", "b9 c #7e7f7d", "#N c #7f7f7e",
"#L c #7f807e", "cs c #807b74", "aK c #818180", "b3 c #828180",
"#n c #828281", "#W c #828378", "#A c #828382", "aE c #848583",
"#J c #858685", "bw c #878886", "bO c #88816f", "#H c #888886",
".K c #888987", "#U c #898987", "aS c #898a88", "cm c #8a717d",
".T c #8a7679", "#S c #8a8b8a", ".N c #8b7a86", "#C c #8b8b8a",
"aV c #8b8c8b", "#q c #8c7188", "bF c #8e8d8c", "b7 c #8e8e8d",
".w c #8e8f79", "b5 c #8e8f8e", "cg c #8f7c8f", ".f c #8f887d",
".x c #8f8c6a", "af c #8f908f", ".F c #906589", "cE c #917396",
"cu c #917398", ".Q c #92836f", "bX c #937499", "bV c #937e73",
".o c #938171", "bN c #945d9e", "#3 c #946c9c", "## c #94737d",
".p c #947978", "bS c #947b76", "b2 c #949493", ".t c #95629f",
"cC c #95639e", "bT c #956887", "bk c #956a85", "aw c #956d83",
"cI c #95709b", ".g c #957c9a", "#x c #957d9a", "aC c #959493",
"#u c #96529a", "cz c #9659a2", "bs c #9661a1", "#r c #966c83",
"as c #967080", "bP c #967879", "bR c #967c76", ".H c #96809a",
".E c #97618d", "cL c #9762a2", "cM c #97759d", "#Q c #979796",
"cq c #9958a7", "aH c #999998", "#P c #999a98", "bf c #999a99",
"cJ c #9a59a8", "ci c #9a5f8f", ".8 c #9a8172", "b8 c #9a9a99",
".n c #9b4eaa", "#2 c #9b5a93", "cH c #9b859f", ".e c #9b978d",
"#a c #9c4eae", "br c #9c5598", "bv c #9c9d9c", ".y c #9d48a4",
"ay c #9d51ae", "#1 c #9d856e", "#X c #9d9c5a", "bc c #9d9e9d",
"cf c #9e9f9e", "ar c #9f3cae", "ck c #9f48a4", ".m c #9f8ba2",
"#j c #9f9f9e", ".Y c #a08fa4", "a1 c #a133b6", "cK c #a142b6",
"by c #a1a1a0", "cy c #a23bb1", "#s c #a29066", ".h c #a290a6",
"aR c #a2a1a0", "bQ c #a33dae", "a3 c #a33eb9", ".2 c #a3a2a1",
"cd c #a4a5a4", "cw c #a533bf", "cN c #a593a7", "co c #a594a8",
".z c #a62dbe", ".s c #a727c2", ".G c #a730c0", ".6 c #a797aa",
"a9 c #a7a8a7", "cr c #a826c2", "bj c #a829c4", ".W c #a830ba",
".B c #a89762", "aM c #a8a8a7", "aZ c #a99960", "cv c #a999aa",
"#M c #a9a9a8", "ch c #aa23c6", ".7 c #aa23c9", "aW c #aa98ac",
"aI c #aaaaaa", "cl c #ad1fc9", ".3 c #adadac", ".D c #af18d0",
"#G c #afafae", "#T c #afb0af", "aY c #b015d2", "cn c #b019d1",
".X c #b01ad0", "cB c #b0a6b3", "ax c #b11ccd", "a0 c #b1a357",
".V c #b1a951", "b# c #b1b1b0", "bU c #b2a456", ".U c #b2a457",
".u c #b2a6b5", "aO c #b2b2b1", "aA c #b3b2b2", "aX c #b40ed9",
"aq c #b4a9b5", "bZ c #b4b3b2", ".r c #b50ddb", ".C c #b50eda",
"ct c #b70add", "cD c #b70ade", ".Z c #b8b8b7", "aP c #b8b9b8",
"#v c #b906e1", "#I c #b9b8b7", "#V c #b9b9b8", "bW c #bb03e4",
"a2 c #bb04e4", "cx c #bb05e3", "at c #bbac51", "#w c #bc03e5",
"bb c #bcbbba", "c. c #bcbbbb", "cb c #bcbcbc", ".q c #bd00e7",
".i c #bdb6bd", "bY c #bdbdbc", "aJ c #bebfbe", "aF c #bfbebe",
".k c #bfbfb8", "bd c #c0bfbf", ".A c #c1b946", "av c #c1bb45",
"bi c #c1bfc1", "#g c #c1c1c0", "ap c #c3c2c1", "bD c #c4c3c2",
"a. c #c4c4c3", ".J c #c7c6c6", "#m c #c7c7c6", ".P c #c8c240",
"#t c #c8c43e", "bL c #c8c8c7", ".O c #c9c43e", "#b c #c9c4c8",
"cA c #c9c4c9", "cF c #cac5c9", "#k c #cac9c9", "al c #cbcaca",
"cO c #cccbcb", "cj c #cdcd38", "a4 c #cdcdcc", "bG c #cdcecd",
"cp c #cecccf", "#h c #cecdcd", ".d c #d1d0d0", "aT c #d2d1d0",
"bn c #d3cb3a", "bm c #d3d134", "#0 c #d3d135", ".5 c #d3d2d2",
"cG c #d4d4d3", "bt c #d5d4d4", "ah c #d6d5d4", "#B c #d6d5d5",
"az c #d6d6d6", ".S c #d7d531", "bM c #d8d7d7", "#p c #d9d8d8",
"b0 c #dad9d9", ".j c #dadad9", ".L c #dbdbda", ".l c #dcdbdb",
".4 c #dedddd", "#F c #e0dfdf", "#R c #e0e0df", ".I c #e1e0e0",
"bo c #e2e129", "#6 c #e2e1e1", ".M c #e4e3e3", ".c c #e5e4e4",
"bq c #e5e525", ".v c #e6e5e5", ".b c #e7e6e6", "bp c #ebec21",
"au c #eeee1f", "bl c #eeee20", "#Y c #f0f01d", "#. c #f3f31b",
".R c #f3f31c", "#Z c #f8f817", ".9 c #f9f917",
"Qt.#.#.#.#.#.#.#.#.#.#.#.#.#.#.#.#.#.#.#.#.#.#.#.#.#.#.#.#.#.#.#.#.#.#.#.#.#.#.#.#.#.#.#.#.#.#.#.#.#.#.#.#.#.#.#.#.#.#.#.#.#.#.#.#.#.#.#.#.#.#.#.#.#.#.#.#.#.#.#.#.#.#.#.#.#.#.#.#.#.#.#.#.#.#.#.#.#.#.#.#.#.#.#.#.#.#.#.#.#.#.#.#.#.#.#.#Qt",
".a.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.c.c.b.b.b.b.b.b.b.b.b.b.a",
".#.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.d.e.f.g.h.i.j.b.b.b.b.b.b.#",
".#.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.k.l.m.n.o.p.q.r.s.t.u.v.b.b.b.b.#",
".#.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.w.x.y.z.A.B.C.D.E.F.G.H.I.b.b.b.#",
".#.b.b.b.b.b.b.J.K.L.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.M.N.O.P.Q.R.S.T.U.V.W.q.X.Y.b.b.b.#",
".#.b.b.b.b.b.b.Z.0.1.2.c.b.b.b.b.b.b.b.b.b.b.b.b.b.b.3.4.b.b.b.b.b.b.b.b.b.b.5.Z.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.6.7.8.9.9.9.9.9#.##.q.q.q#a#b.b.b.#",
".#.b.b.b.b.b.b.Z#c#d#e#f#g.b.b.b.b.b.b.b.b.b.b.b.b#h#i#j.b.b.b.b.b.b#k#l#m.b#n#o.4.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b#p#q#r#s.9.9.9.9.9#t#u#v.q.q#w#x.b.b.#",
".#.b.b.b.b.b.b.Z#c#d#d#y#z#A#B.b.b.b.b.b.b.b.b.b.b#C#D#E.v.b#F#G#H#I#J#K#L#M#N#O.M#p#P#Q#B#R#S#T#U#V.v.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.v#W#X#Y.9.9.9.9.9.9#Z#0#1#2.r.q#3.v.b.#",
".#.b.b.b.b.b.b.Z#c#d#d#d#d#4#5#Q.b.b.b.b.b.b.b.b#6#7#8#9a..ca#aaab#lacadaeaf.1agahaiajajakalam#7anaoap.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.baqarasat.9.9.9.9.9auav#1awax.qayaz.b.#",
".#.b.b.b.b.b.b.Z#c#d#d#d#d#4#5#Q.b.b.b.b.b.b.b.baAaBaCaDaEaFaGaH.M.LaIaBaJ.baKaLaMaNaOaPaQaRaoaSaTaUaV.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.baWaXaYaZ.9.9.9.9.9a0a1a2.q.q.qa3a4.b.#",
".#.b.b.b.b.b.b.Z#c#d#d#y#z#A#B.b.b.b.b.b.b.b.b.ba5a6#oa7a8a9b.b#.b.vaIbabb.baKaLbca7bdaJbebfbgaV.jbhaE.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.bbibjbkblbmbn.9bobpbqbr.q.q.q.qbs.I.b.#",
".#.b.b.b.b.b.b.Z#c#d#e#f#g.b.b.b.b.b.b.b.b.b.bbtambubvbwbxbybzbAbBbCbDa7bEbFaKaLbGbHbIbJbKbLbgaV.jbhaE.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.bbMbNbObPbQbR.9bSbTbUbVbW.q.q.qbX.b.b.#",
".#.b.b.b.b.b.b.Z.0.1.2.c.b.b.b.b.b.b.b.b.b.b.bbY#lbZ.bb0b1b2#gb3b4#C#6b5b6b7b8b9#6c.c#cacb.4cccd#Fcecf.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.bcgch.q.qcicjck.qclcmcl.q.qcnco.b.b.#",
".#.b.b.b.b.b.b.J.K.L.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.bcpcq.q.qcrcsct.q.q.q.q.qbWcu#6.b.b.#",
".#.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.bcvcw.qcxcy.q.q.q.q.q#wczcA.b.b.b.#",
".#.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.vcBcCcD.q.q.q.q.q.XcEcF.b.b.b.b.#",
".#.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.bcGcHcIcJcKcLcMcN#R.b.b.b.b.b.#",
".a.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.b.jcO#F.b.b.b.b.b.b.b.b.a",
"Qt.#.#.#.#.#.#.#.#.#.#.#.#.#.#.#.#.#.#.#.#.#.#.#.#.#.#.#.#.#.#.#.#.#.#.#.#.#.#.#.#.#.#.#.#.#.#.#.#.#.#.#.#.#.#.#.#.#.#.#.#.#.#.#.#.#.#.#.#.#.#.#.#.#.#.#.#.#.#.#.#.#.#.#.#.#.#.#.#.#.#.#.#.#.#.#.#.#.#.#.#.#.#.#.#.#.#.#.#.#.#.#.#.#.#.#.#Qt"]


_logo_pixmap = QPixmap(vtoolheader_xpm)
