# -*- coding: utf-8 -*-

"""
vtoolpanelplugin.py

A tool boxes container custom widget plugin for Qt Designer.

Copyright (C) 2013-2016 Vincent Vande Vyvre. <vincent.vandevyvre@oqapy.eu>

GNU General Public License Usage
This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
"""


from PyQt5.QtGui import QPixmap, QIcon
from PyQt5.QtDesigner import QPyDesignerCustomWidgetPlugin

from VWidgets_PyQt5.vtoolpanel import VToolPanel


class VToolPanelPlugin(QPyDesignerCustomWidgetPlugin):
    """Provides a Python custom plugin for Qt Designer by implementing the
    QDesignerCustomWidgetPlugin using a PyQt-specific custom plugin class.
    """

    # The __init__() method is only used to set up the plugin and define its
    # initialized variable.
    def __init__(self, parent=None):
    
        QPyDesignerCustomWidgetPlugin.__init__(self)
        self.initialized = False

    # The initialize() and isInitialized() methods allow the plugin to set up
    # any required resources, ensuring that this can only happen once for each
    # plugin.
    def initialize(self, formEditor):

        if self.initialized:
            return

        manager = formEditor.extensionManager()
        
        self.initialized = True

    def isInitialized(self):
        return self.initialized

    # This factory method creates new instances of our custom widget with the
    # appropriate parent.
    def createWidget(self, parent):
        return VToolPanel(parent)

    # This method returns the name of the custom widget class that is provided
    # by this plugin.
    def name(self):
        return "VToolPanel"

    # Returns the name of the group in Qt Designer's widget box that this
    # widget belongs to.
    def group(self):
        return "VWidgets PyQt5"

    # Returns the icon used to represent the custom widget in Qt Designer's
    # widget box.
    def icon(self):
        return QIcon(_logo_pixmap)

    # Returns a short description of the custom widget for use in a tool tip.
    def toolTip(self):
        return "Tool panel"

    # Returns a short description of the custom widget for use in a "What's
    # This?" help message for the widget.
    def whatsThis(self):
        return "Vertical container for tool boxes"

    # Returns True if the custom widget acts as a container for other widgets;
    # otherwise returns False. Note that plugins for custom containers also
    # need to provide an implementation of the QDesignerContainerExtension
    # interface if they need to add custom editing support to Qt Designer.
    def isContainer(self):
        return True

    # Returns an XML description of a custom widget instance that describes
    # default values for its properties. Each custom widget created by this
    # plugin will be configured using this description.
    def domXml(self):
        return '<widget class="VToolPanel" name="toolPanel" />\n'

    # Returns the module containing the custom widget class. It may include
    # a module path.
    def includeFile(self):
        return "VWidgets_PyQt5.vtoolpanel"


# Define the image used for the icon.
vtoolpanel_xpm = [ "37 44 158 2",
"Qt c #191919", ".c c #171717", "aB c #191919", "ay c #1c1c1c",
"#u c #252525", ".Q c #232323", ".T c #252525", "#x c #242424",
"#7 c #242424", "#0 c #232323", "#5 c #1f1f1f", "#r c #212121",
".P c #222222", ".E c #202020", "#y c #1b1b1b", "#4 c #202020",
"#W c #212121", "aA c #222222", ".b c #232323", ".a c #252525",
"#p c #2a2a2a", "#n c #2b2b2b", "#q c #2c2c2c", ".q c #202020",
".# c #222222", "az c #232323", ".D c #262626", ".r c #2e2e2e",
"#M c #191919", ".U c #1d1d1d", ".d c #212121", "aw c #292929",
"au c #2e2e2e", "#Y c #303030", "#Z c #323232", "#F c #191919",
"at c #1b1b1b", "ad c #1c1c1c", "#a c #232323", "am c #1d1d1d",
"#E c #1f1f1f", "#m c #222222", "## c #242424", ".2 c #272727",
"#V c #252525", "an c #272727", "#8 c #282828", "ae c #292929",
".S c #393939", ".R c #3f3f3f", "#L c #242424", "#v c #303030",
"#w c #333333", "#1 c #939393", "#H c #0c0c0c", ".4 c #101010",
"ag c #111111", ".x c #303030", "#6 c #313131", "#I c #333333",
"af c #3c3c3c", "#z c #3e3e3e", "#O c #424242", ".8 c #464646",
".V c #484848", "#d c #494949", "#K c #4a4a4a", ".1 c #4c4c4c",
".H c #4d4d4d", "av c #4e4e4e", ".t c #505050", ".B c #515151",
"#9 c #525252", ".W c #535353", ".3 c #575757", "#B c #595959",
"a. c #5a5a5a", ".5 c #5c5c5c", ".J c #5e5e5e", "ak c #606060",
".7 c #616161", ".Z c #636363", ".L c #646464", ".v c #656565",
"ai c #666666", ".u c #676767", "al c #686868", ".y c #696969",
"#C c #6a6a6a", ".0 c #6b6b6b", "#c c #6d6d6d", ".K c #6e6e6e",
"#N c #6f6f6f", "#S c #727272", ".6 c #757575", ".w c #767676",
".z c #787878", ".I c #7b7b7b", ".A c #7d7d7d", "#A c #7e7e7e",
"#G c #7f7f7f", ".X c #808080", ".Y c #818181", "aq c #828282",
"#o c #848484", "#T c #878787", "#D c #8a8a8a", ".h c #8b8b8b",
".9 c #8f8f8f", ".m c #909090", "aj c #919191", "#k c #939393",
"#b c #949494", ".g c #959595", ".s c #979797", ".N c #989898",
"ap c #999999", ".M c #9b9b9b", "#j c #9c9c9c", "#Q c #9d9d9d",
"#X c #9e9e9e", "#i c #9f9f9f", "#l c #a0a0a0", "ar c #a1a1a1",
".O c #a4a4a4", "#3 c #a7a7a7", ".F c #a8a8a8", "#h c #aaaaaa",
"#s c #ababab", "#2 c #adadad", "#R c #afafaf", "#U c #b0b0b0",
".G c #b2b2b2", "#g c #b5b5b5", ".n c #b8b8b8", ".i c #b9b9b9",
"#t c #bababa", ".C c #bcbcbc", "#P c #c2c2c2", ".o c #c9c9c9",
"#e c #cbcbcb", "#J c #cdcdcd", "ac c #d0d0d0", "a# c #d3d3d3",
"aa c #d6d6d6", "ab c #d7d7d7", "#. c #d9d9d9", "#f c #e0e0e0",
"ah c #e1e1e1", "ao c #e9e9e9", ".j c #f0f0f0", ".p c #f1f1f1",
".k c #f2f2f2", ".e c #f5f5f5", ".l c #f7f7f7", "as c #fcfcfc",
"ax c #fefefe", ".f c #ffffff",
"Qt.#.a.a.a.a.a.a.a.a.a.a.a.a.a.a.a.a.a.a.a.a.a.a.a.a.a.a.a.a.a.a.a.a.a.b.c",
".d.e.f.f.g.h.i.j.k.l.i.m.n.o.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.p.q",
".r.f.f.s.t.u.v.w.x.y.z.A.B.C.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.D",
".E.F.G.s.H.w.I.g.J.K.L.M.t.N.G.G.G.G.G.G.G.G.G.G.G.G.G.G.G.G.G.G.G.G.G.O.P",
".Q.R.S.S.S.S.S.S.S.S.S.S.S.S.S.S.S.S.S.S.S.S.S.S.S.S.S.S.S.S.S.S.S.S.S.R.T",
".U.V.W.t.X.Y.Y.Y.Y.Z.Y.Y.0.Y.Y.Y.Y.Y.Y.Y.Y.Y.Y.Y.Y.Y.Y.Y.Y.Y.Y.Y.Y.Y.Y.1.U",
".2.o.3.4.3.f.5.L.6.1.7.8.9.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f#.##",
"#a#b#c#d#e#f#g#b.O#h#i#j#k#f#f#f#f#f#f#f#f#f#f#f#f#f#f#f#f#f#f#f#f#f#f#l#m",
"#n#o#c#c#c#c#c#c#c#c#c#c#c#c#c#c#c#c#c#c#c#c#c#c#c#c#c#c#c#c#c#c#c#c#c.X#p",
"#q.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f#n",
"#q.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f#n",
"#q.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f#n",
"#q.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f#n",
"#q.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f#n",
"#q.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f#n",
"#q.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f#n",
"#q.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f#n",
"#q.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f#n",
"#r#s#t#t#t#t#t#t#t#t#t#t#t#t#t#t#t#t#t#t#t#t#t#t#t#t#t#t#t#t#t#t#t#t#t.F.P",
"#u.S#v#v#v#v#v#v#v#v#v#v#v#v#v#v#v#v#v#v#v#v#v#v#v#v#v#v#v#v#v#v#v#v#v#w#x",
"#y.8.Z#z#o#A.w#B#C#D#D#D#D#D#D#D#D#D#D#D#D#D#D#D#D#D#D#D#D#D#D#D#D#D#D.u#E",
"#F#h#G#H#I#J.V.B#K.J#G.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.l#L",
"#M#N.9#O#P#J#Q.u#R#S#T#.#.#.#.#.#.#.#.#.#.#.#.#.#.#.#.#.#.#.#.#.#.#.#.#U#V",
"#W#X.6.6.6.6.6.6.6.6.6.6.6.6.6.6.6.6.6.6.6.6.6.6.6.6.6.6.6.6.6.6.6.6.6#G#Y",
"#W.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f#Z",
"#W.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f#Z",
"#W.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f#Z",
"#W.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f#Z",
"#W.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f#Z",
"#W.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f#Z",
"#W.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f#Z",
"#W.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f#Z",
"#W.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f#Z",
"#0#1#2#2#2#2#2#2#2#2#2#2#2#2#2#2#2#2#2#2#2#2#2#2#2#2#2#2#2#2#2#2#2#2#2#3#4",
"#5#6#6#6#6#6#6#6#6#6#6#6#6#6#6#6#6#6#6#6#6#6#6#6#6#6#6#6#6#6#6#6#6#6#6#v#7",
"#8.N#9a.a##.#.#e#.#.aa#.ab#.#.#Qac#.#.#.#.#.#.#.#.#.#.#.#.#.#.#.#.#.#..zad",
"ae#fafag.uahaiaiaj#cakal.5#A#N#dai.y.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f#tam",
"an#Pa..7.eao.wap.z.v#T.X#G#caqar#G.zas.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.Mat",
"au.3avavavavavavavavavavavavavavavavavavavavavavavavavavavavavavavavav.v#4",
"#Z.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f#W",
"#Z.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f#W",
"#Z.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f#W",
"awax.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.f.l.q",
"ayaz.a.a.a.a.a.a.a.a.a.a.a.a.a.a.a.a.a.a.a.a.a.a.a.a.a.a.a.a.a.a.a.a.aaAaB"]

_logo_pixmap = QPixmap(vtoolpanel_xpm)
