# -*- coding: utf-8 -*-

# about.py
# This file is part of Oqapy serie 3
#    
# Author: Vincent Vande Vyvre <vincent.vandevyvre@oqapy.eu>
# Home: http://www.oqapy.eu
# Copyright: 2011-2017 Vincent Vande Vyvre
# Licence: GPL3
#
# About Oqapy dialog box


import sys
import platform
import subprocess
import webbrowser

from PyQt5.QtGui import QIcon, QTextCharFormat, QFont, QCursor
from PyQt5.QtWidgets import (QApplication, QDialog, QGridLayout, QVBoxLayout,
                             QHBoxLayout, QSizePolicy, QSpacerItem,
                             QPlainTextEdit, QLabel, QLayout, QWidget, QMenu,
                             QAction, QFrame, QTabWidget, QTextBrowser)
from PyQt5.QtCore import (Qt, QT_VERSION_STR, PYQT_VERSION_STR, QSize,
                          QMetaObject, QUrl, pyqtSignal)
from sip import SIP_VERSION_STR

import stylesheets as style

class About:
    def __init__(self, vers):
        """Show a dialog box with version and licence infos
        
        Keyword arguments:
        vers -- version of Oqapy
        """
        VERSION__STR = vers
        PY_VERSION = sys.version.split()[0]
        PLATFORM = platform.platform()
        from oqapy_iproc import OqapyIProc
        oi = OqapyIProc()
        IPROC_VERS = str(oi.version(), 'utf-8')
        try:
            import pyexiv2
            EXIV2 = pyexiv2.version_info
            del pyexiv2
        except:
            EXIV2 = "< 2.x"
        if isinstance(EXIV2, tuple):
            EXIV2 = ".".join([str(x) for x in EXIV2])

        rep = subprocess.Popen(["gphoto2", "--version"], universal_newlines=True,
                                    stdout=subprocess.PIPE).communicate()
        try:
            gph = rep[0].split('\n')[0]
            GPHOTO2_VERS = gph.split(" ")[1]
        except:
            GPHOTO2_VERS = "Unknow"

        self.versions = {"platform": PLATFORM,
                        "Oqapy": VERSION__STR,
                        "python": PY_VERSION,
                        "qt": QT_VERSION_STR,
                        "pyqt": PYQT_VERSION_STR,
                        "sip": SIP_VERSION_STR,
                        "Pyexiv2": EXIV2,
                        "gPhoto2": GPHOTO2_VERS,
                        "liboqapy-iproc": IPROC_VERS}

        self.win = UiAbout()
        self.win.label.setText("Oqapy {0}".format(vers))
        self.win.show()
        self.set_versions_text(self.win.vers_tab.editor)
        self.win.lic_tab.editor.insertPlainText(self.get_licence_text())
        self.set_translation_text(self.win.trans_tab)
        self.win.vers_tab.copyRequested.connect(self.copy_versions)
        self.win.lic_tab.copyRequested.connect(self.copy_licence)
        self.win.trans_tab.anchorClicked.connect(self.on_link_activated)
        self.clip_board = QApplication.clipboard()
        self.win.exec_()

    def set_versions_text(self, edit):
        font = QFont("Mono", 10)
        c_form = QTextCharFormat()
        c_form.setFont(font)
        font1 = QFont("sans", 10)
        font1.setWeight(56)
        c_form1 = QTextCharFormat()
        c_form1.setFont(font1)

        vers = self.versions
        l = 25 - len(vers["Oqapy"])
        edit.setCurrentCharFormat(c_form1)
        edit.appendPlainText("Oqapy")
        txt = "\t" + " " * l + vers["Oqapy"]
        edit.setCurrentCharFormat(c_form)
        edit.insertPlainText(txt)

        l = 25 - len(vers["python"])
        edit.setCurrentCharFormat(c_form1)
        edit.appendPlainText("Python")
        txt = "\t" + " " * l + vers["python"]
        edit.setCurrentCharFormat(c_form)
        edit.insertPlainText(txt)

        l = 25 - len(vers["qt"])
        edit.setCurrentCharFormat(c_form1)
        edit.appendPlainText("Qt")
        txt = "\t" + " " * l + vers["qt"]
        edit.setCurrentCharFormat(c_form)
        edit.insertPlainText(txt)

        l = 25 - len(vers["pyqt"])
        edit.setCurrentCharFormat(c_form1)
        edit.appendPlainText("PyQt")
        txt = "\t" + " " * l + vers["pyqt"]
        edit.setCurrentCharFormat(c_form)
        edit.insertPlainText(txt)

        l = 25 - len(vers["sip"])
        edit.setCurrentCharFormat(c_form1)
        edit.appendPlainText("Sip")
        txt = "\t" + " " * l + vers["sip"]
        edit.setCurrentCharFormat(c_form)
        edit.insertPlainText(txt)

        l = 25 - len(vers["Pyexiv2"])
        edit.setCurrentCharFormat(c_form1)
        edit.appendPlainText("Pyexiv2")
        txt = "\t" + " " * l + vers["Pyexiv2"]
        edit.setCurrentCharFormat(c_form)
        edit.insertPlainText(txt)

        l = 25 - len(vers["gPhoto2"])
        edit.setCurrentCharFormat(c_form1)
        edit.appendPlainText("gPhoto2")
        txt = "\t" + " " * l + vers["gPhoto2"]
        edit.setCurrentCharFormat(c_form)
        edit.insertPlainText(txt)

        l = 15 - len(vers["liboqapy-iproc"])
        edit.setCurrentCharFormat(c_form1)
        edit.appendPlainText("liboqapy-iproc")
        txt = "\t" + " " * l + vers["liboqapy-iproc"]
        edit.setCurrentCharFormat(c_form)
        edit.insertPlainText(txt)

    def get_licence_text(self):
        try:
            with open('medias/copyright', 'r') as inf:
                txt = inf.read()
        except Exception as why:
                txt = '\n\tLicence text not found!\n%s' % why

        return txt

    def set_translation_text(self, edit):
        url = QUrl('medias/howToTranslate.html')
        edit.setSource(url)

    def copy_versions(self):
        """Copy all versions nummers into the clipboard.

        The method copy() of QPlainTextEdit don't save the formatting of the
        text, so we re-create it.
        """
        txt = """Oqapy\t\t%(Oqapy)s\nPlatform\t%(platform)s\nQt\t\t\t%(qt)s
Python\t\t%(python)s\nPyQt\t\t%(pyqt)s\nSip\t\t\t%(sip)s\nPyexiv2\t\t%(Pyexiv2)s
gPhoto2\t\t%(gPhoto2)s
""" % self.versions
        self.clip_board.setText(txt)

    def copy_licence(self):
        """Copy the licence text into the clipboard.

        """
        self.clip_board.setText(self.get_licence_text())

    def on_link_activated(self, link):
        """Called when a web link is clicked into the translation tab.

        """
        self.win.trans_tab.reload()
        webbrowser.open_new_tab(link.toString())

class UiAbout(QDialog):
    def __init__(self, parent=None):
        super().__init__(parent)
        self.resize(650, 551)
        self.setSizeGripEnabled(True)
        self.setWindowTitle('About - Oqapy')
        pal = style.get_palette()
        self.setPalette(pal)
        self.gridLayout_2 = QGridLayout(self)
        self.verticalLayout_5 = QVBoxLayout()
        self.horizontalLayout = QHBoxLayout()
        self.horizontalLayout.setSpacing(0)
        self.horizontalLayout.setSizeConstraint(QLayout.SetMinimumSize)
        self.verticalLayout = QVBoxLayout()
        spacerItem = QSpacerItem(20, 17, QSizePolicy.Minimum, 
                                        QSizePolicy.Fixed)
        self.verticalLayout.addItem(spacerItem)
        self.line = QFrame(self)
        sizePolicy = QSizePolicy(QSizePolicy.Fixed, 
                                        QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.line.sizePolicy().hasHeightForWidth())
        self.line.setSizePolicy(sizePolicy)
        self.line.setMinimumSize(QSize(25, 0))
        self.line.setMaximumSize(QSize(25, 16777215))
        self.line.setLineWidth(1)
        self.line.setFrameShape(QFrame.HLine)
        self.line.setFrameShadow(QFrame.Plain)
        self.line.setObjectName("line")
        self.line.setStyleSheet(style.UNDERLINE)
        self.verticalLayout.addWidget(self.line)
        self.horizontalLayout.addLayout(self.verticalLayout)
        self.label = QLabel(self)
        sizePolicy = QSizePolicy(QSizePolicy.Fixed, 
                                        QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.label.sizePolicy().hasHeightForWidth())
        self.label.setSizePolicy(sizePolicy)
        font = QFont()
        font.setFamily("Ubuntu")
        font.setPointSize(20)
        font.setBold(True)
        font.setWeight(75)
        self.label.setFont(font)
        self.label.setStyleSheet("QLabel {color: #758088;}")
        self.horizontalLayout.addWidget(self.label)
        self.verticalLayout_2 = QVBoxLayout()
        spacerItem1 = QSpacerItem(20, 17, QSizePolicy.Minimum, 
                                        QSizePolicy.Fixed)
        self.verticalLayout_2.addItem(spacerItem1)
        self.line_2 = QFrame(self)
        sizePolicy = QSizePolicy(QSizePolicy.Preferred, 
                                        QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.line_2.sizePolicy().hasHeightForWidth())
        self.line_2.setSizePolicy(sizePolicy)
        self.line_2.setLineWidth(1)
        self.line_2.setFrameShape(QFrame.HLine)
        self.line_2.setFrameShadow(QFrame.Plain)
        self.line_2.setObjectName("line_2")
        self.line_2.setStyleSheet(style.UNDERLINE)
        self.verticalLayout_2.addWidget(self.line_2)
        self.horizontalLayout.addLayout(self.verticalLayout_2)
        self.verticalLayout_3 = QVBoxLayout()
        spacerItem2 = QSpacerItem(20, 13, QSizePolicy.Minimum, 
                                        QSizePolicy.Fixed)
        self.verticalLayout_3.addItem(spacerItem2)
        self.label_2 = QLabel(self)
        sizePolicy = QSizePolicy(QSizePolicy.Fixed, 
                                        QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.label_2.sizePolicy().hasHeightForWidth())
        self.label_2.setSizePolicy(sizePolicy)
        font = QFont()
        font.setFamily("DejaVu Sans")
        font.setItalic(True)
        self.label_2.setFont(font)
        self.label_2.setText("Photographic Workflow Software ")
        self.label_2.setAlignment(Qt.AlignRight|Qt.AlignTrailing|
                                        Qt.AlignVCenter)
        self.verticalLayout_3.addWidget(self.label_2)
        self.horizontalLayout.addLayout(self.verticalLayout_3)
        self.verticalLayout_5.addLayout(self.horizontalLayout)

        self.horizontalLayout_16 = QHBoxLayout()
        spacerItem3 = QSpacerItem(20, 20, QSizePolicy.Fixed, 
                                        QSizePolicy.Minimum)
        self.horizontalLayout_16.addItem(spacerItem3)
        self.verticalLayout_6 = QVBoxLayout()
        self.verticalLayout_4 = QVBoxLayout()
        spacerItem4 = QSpacerItem(20, 20, QSizePolicy.Minimum, 
                                        QSizePolicy.Fixed)
        self.verticalLayout_4.addItem(spacerItem4)
        self.horizontalLayout_3 = QHBoxLayout()
        self.horizontalLayout_3.setObjectName("horizontalLayout_3")
        spacerItem6 = QSpacerItem(20, 20, QSizePolicy.Expanding, 
                                        QSizePolicy.Minimum)
        self.horizontalLayout_3.addItem(spacerItem6)
        self.label_3 = QLabel('Author:', self)
        self.label_3.setAlignment(Qt.AlignRight|Qt.AlignTrailing|
                                        Qt.AlignVCenter)
        self.label_3.setObjectName("label_3")
        self.horizontalLayout_3.addWidget(self.label_3)
        self.label_4 = QLabel(self)
        self.label_4.setText("Vincent Vande Vyvre")
        self.label_4.setObjectName("label_4")
        self.horizontalLayout_3.addWidget(self.label_4)
        spacerItem7 = QSpacerItem(40, 20, QSizePolicy.Expanding, 
                                        QSizePolicy.Minimum)
        self.horizontalLayout_3.addItem(spacerItem7)
        self.verticalLayout_4.addLayout(self.horizontalLayout_3)
        self.horizontalLayout_4 = QHBoxLayout()
        self.horizontalLayout_4.setObjectName("horizontalLayout_4")
        spacerItem8 = QSpacerItem(40, 20, QSizePolicy.Expanding, 
                                        QSizePolicy.Minimum)
        self.horizontalLayout_4.addItem(spacerItem8)
        self.label_5 = QLabel('Licence:', self)
        self.label_5.setAlignment(Qt.AlignRight|Qt.AlignTrailing|
                                        Qt.AlignVCenter)
        self.label_5.setObjectName("label_5")
        self.horizontalLayout_4.addWidget(self.label_5)
        self.label_6 = QLabel('GPL-3', self)
        self.horizontalLayout_4.addWidget(self.label_6)
        spacerItem9 = QSpacerItem(40, 20, QSizePolicy.Expanding, 
                                        QSizePolicy.Minimum)
        self.horizontalLayout_4.addItem(spacerItem9)
        self.verticalLayout_4.addLayout(self.horizontalLayout_4)
        self.horizontalLayout_5 = QHBoxLayout()
        self.horizontalLayout_5.setObjectName("horizontalLayout_5")
        spacerItem10 = QSpacerItem(40, 20, QSizePolicy.Expanding, 
                                        QSizePolicy.Minimum)
        self.horizontalLayout_5.addItem(spacerItem10)
        self.label_7 = QLabel('Home page:', self)
        self.label_7.setAlignment(Qt.AlignRight|Qt.AlignTrailing|
                                        Qt.AlignVCenter)
        self.label_7.setObjectName("label_7")
        self.horizontalLayout_5.addWidget(self.label_7)
        self.label_8 = UrlLabel('http://www.oqapy.eu', self)
        self.horizontalLayout_5.addWidget(self.label_8)
        spacerItem11 = QSpacerItem(40, 20, QSizePolicy.Expanding, 
                                        QSizePolicy.Minimum)
        self.horizontalLayout_5.addItem(spacerItem11)
        self.verticalLayout_4.addLayout(self.horizontalLayout_5)
        self.horizontalLayout_6 = QHBoxLayout()
        self.horizontalLayout_6.setObjectName("horizontalLayout_6")
        spacerItem12 = QSpacerItem(40, 20, QSizePolicy.Expanding, 
                                        QSizePolicy.Minimum)
        self.horizontalLayout_6.addItem(spacerItem12)
        self.label_9 = QLabel('Forge:', self)
        self.label_9.setAlignment(Qt.AlignRight|Qt.AlignTrailing|
                                        Qt.AlignVCenter)
        self.label_9.setObjectName("label_9")
        self.horizontalLayout_6.addWidget(self.label_9)
        self.label_10 = UrlLabel('https://launchpad.net/oqapy', self)
        self.horizontalLayout_6.addWidget(self.label_10)
        spacerItem13 = QSpacerItem(40, 20, QSizePolicy.Expanding, 
                                        QSizePolicy.Minimum)
        self.horizontalLayout_6.addItem(spacerItem13)
        self.verticalLayout_4.addLayout(self.horizontalLayout_6)
        self.horizontalLayout_7 = QHBoxLayout()
        self.horizontalLayout_7.setObjectName("horizontalLayout_7")
        spacerItem14 = QSpacerItem(40, 20, QSizePolicy.Expanding, 
                                        QSizePolicy.Minimum)
        self.horizontalLayout_7.addItem(spacerItem14)
        self.label_11 = QLabel('Bugs:', self)
        self.label_11.setAlignment(Qt.AlignRight|Qt.AlignTrailing|
                                        Qt.AlignVCenter)
        self.label_11.setObjectName("label_11")
        self.horizontalLayout_7.addWidget(self.label_11)
        self.label_15 = UrlLabel('https://bugs.launchpad.net/oqapy', self)
        self.horizontalLayout_7.addWidget(self.label_15)
        spacerItem15 = QSpacerItem(40, 20, QSizePolicy.Expanding, 
                                        QSizePolicy.Minimum)
        self.horizontalLayout_7.addItem(spacerItem15)
        self.verticalLayout_4.addLayout(self.horizontalLayout_7)
        self.horizontalLayout_8 = QHBoxLayout()
        self.horizontalLayout_8.setObjectName("horizontalLayout_8")
        spacerItem16 = QSpacerItem(40, 20, QSizePolicy.Expanding, 
                                        QSizePolicy.Minimum)
        self.horizontalLayout_8.addItem(spacerItem16)
        self.label_14 = QLabel('Translations:', self)
        self.label_14.setAlignment(Qt.AlignRight|Qt.AlignTrailing|
                                        Qt.AlignVCenter)
        self.label_14.setObjectName("label_14")
        self.horizontalLayout_8.addWidget(self.label_14)
        self.label_12 = UrlLabel('https://translations.launchpad.net/oqapy', 
                                        self)
        self.horizontalLayout_8.addWidget(self.label_12)
        spacerItem17 = QSpacerItem(40, 20, QSizePolicy.Expanding, 
                                        QSizePolicy.Minimum)
        self.horizontalLayout_8.addItem(spacerItem17)
        self.verticalLayout_4.addLayout(self.horizontalLayout_8)
        self.verticalLayout_6.addLayout(self.verticalLayout_4)
        spacerItem4 = QSpacerItem(20, 20, QSizePolicy.Minimum, 
                                        QSizePolicy.Fixed)
        self.verticalLayout_6.addItem(spacerItem4)
        self.tab_wdg = QTabWidget(self)
        self.tab_wdg.setStyleSheet(style.TABDIAL)
        self.tab_wdg.setObjectName("tab_wdg")
        self.verticalLayout_6.addWidget(self.tab_wdg)
        self.horizontalLayout_16.addLayout(self.verticalLayout_6)
        spacerItem8 = QSpacerItem(20, 20, QSizePolicy.Fixed, 
                                        QSizePolicy.Minimum)
        self.horizontalLayout_16.addItem(spacerItem8)
        self.verticalLayout_5.addLayout(self.horizontalLayout_16)
        self.gridLayout_2.addLayout(self.verticalLayout_5, 0, 0, 1, 1)
        self.vers_tab = TextWidget()
        self.tab_wdg.addTab(self.vers_tab, 'Versions')
        self.lic_tab = TextWidget()
        self.tab_wdg.addTab(self.lic_tab, 'Licence')
        self.trans_tab = QTextBrowser()
        self.tab_wdg.addTab(self.trans_tab, 'How To Translate Oqapy')
        self.tab_wdg.currentChanged.connect(self.on_tab_changed)

        QMetaObject.connectSlotsByName(self)

    def on_tab_changed(self, idx):
        if idx != 2:
            self.tab_wdg.currentWidget().editor.verticalScrollBar().setValue(0)
            

class UrlLabel(QLabel):
    def __init__(self, text='', parent=None):
        super().__init__(parent)
        self.url = text
        font = QFont()
        font.setUnderline(True)
        self.setFont(font)
        self.setStyleSheet("color: #0000FF")
        self.set_text(text)
        self.setCursor(QCursor(Qt.PointingHandCursor))
        self.clip_board = QApplication.clipboard()

    def set_text(self, text):
        self.url = text
        self.setText(text)

    def mousePressEvent(self, event):
        if event.button() == 1:
            self.open_in_webbroser()

        elif event.button() == 2:
            self.show_context_menu(event.globalPos())

    def show_context_menu(self, pos):
        self.menu = QMenu()
        self.copy = QAction(QIcon('medias/copy.png'), 'Copy', None)
        self.web = QAction(QIcon('medias/web.png'), 'Open', None)
        self.menu.addAction(self.copy)
        self.menu.addAction(self.web)
        self.copy.triggered.connect(self.copy_url)
        self.web.triggered.connect(self.open_in_webbroser)
        self.menu.popup(pos)

    def copy_url(self):
        self.clip_board.setText(self.url)

    def open_in_webbroser(self):
        webbrowser.open_new_tab(self.url)

class TextWidget(QWidget):
    copyRequested = pyqtSignal()
    def __init__(self, parent=None):
        super().__init__(parent)
        self.vlayout = QVBoxLayout()
        self.vlayout.setContentsMargins(0, 0, 0, 0)
        self.hlayout = QHBoxLayout()
        spacer = QSpacerItem(40, 20, QSizePolicy.Expanding, 
                                        QSizePolicy.Minimum)
        self.hlayout.addItem(spacer)
        self.copy_lbl = QLabel(self)
        font = QFont()
        font.setItalic(True)
        self.copy_lbl.setFont(font)
        self.copy_lbl.setStyleSheet("color: #A0A0A0")
        self.copy_lbl.setText('Copy to the clipboard')
        self.copy_lbl.setCursor(QCursor(Qt.PointingHandCursor))
        self.copy_lbl.mousePressEvent = self.on_copy_clicked
        self.hlayout.addWidget(self.copy_lbl)
        self.vlayout.addLayout(self.hlayout)
        self.editor = QPlainTextEdit(self)
        self.editor.setReadOnly(True)
        self.vlayout.addWidget(self.editor)
        self.setLayout(self.vlayout)

    def on_copy_clicked(self, event):
        self.copyRequested.emit()

