# -*- coding: utf-8 -*-

# IOgPhoto2.py
# This file is part of Oqapy serie 3
#
# Author: Vincent Vande Vyvre <vincent.vandevyvre@oqapy.eu>
# Home: http://www.oqapy.eu
# Copyright: 2010-2017 Vincent Vande Vyvre
# Licence: GPL3
#
# This script is just a layer for use gPhoto2 with Python,
# gPhoto2 is a command-line application for libgphoto2.
# See: http://www.gphoto.org/proj/gphoto2/
# ! Be care to maintain the compatibility Python 2 & Python 3


import os
import subprocess
import time

from PyQt5 import QtCore

class IOcamera(QtCore.QObject):
    fileDownloaded = QtCore.pyqtSignal(str)
    downloadingFinished = QtCore.pyqtSignal(str)
    ports = []
    ports_error = ""
    cameras = []
    cameras_error = ""
    camera = ""
    d_folder = ""   # target directory for download
    s_folder = ""   # source directory, into the camera
    vers = ""
    proc = None

    def __init__(self):
        """Initialise support for digital camera.

        List all computer's ports and search if camera(s) is (are) pluged
        """
        super().__init__()

        self.vers = subprocess.Popen(["gphoto2", "--version"], 
                                    universal_newlines=True, 
                                    stdout=subprocess.PIPE).communicate()

        reply = self.read_ports()

        if reply[0]:
            for p in reply[0]:
                self.ports.append(p)

        else:
            self.ports_error = reply[1]

        reply = self.search_cameras()

        if reply[0]:
            for c in reply[0]:
                self.cameras.append(c)

        else:
            self.cameras_error = reply[1]        
        
    def infos(self):
        """Export basic infos.

        Returns infos collected by __init__ in a list
        structured like this:
        ['Version',
        [Supported ports],
        'Error in ports detection' or None,
        [List of detected cameras],
        'Error in cameras detection' or None]
        """
        pack = []
        pack.append(self.vers)
        pack.append(self.ports)
        pack.append(self.ports_error)
        pack.append(self.cameras)
        pack.append(self.cameras_error)
        return pack

    def about_camera(self):
        """Returns general infos for the current camera.
        
        """
        return subprocess.Popen(["gphoto2", "--summary"], universal_newlines=True, 
                                stdout=subprocess.PIPE).communicate()

    def read_ports(self):
        """Detect ports on pc.

        Returns: 
        list of ports and/or eventually error.
        """
        lp, er = subprocess.Popen(["gphoto2", "--list-ports"], 
                                    universal_newlines=True, 
                                    stdout=subprocess.PIPE).communicate()

        if lp:
            ports = []
            for p in lp.split("\n"):
                ports.append(p)
            return ports, er

        return None, er

    def search_cameras(self):
        """Detect plugged cameras.

        The camera must be ON.
        This only works with USB, RS232 ports are dumb.

        Returns: 
        list of cameras and/or eventually error.
        """
        lc, er = subprocess.Popen(["gphoto2", "--auto-detect"], 
                                universal_newlines=True, 
                                stdout=subprocess.PIPE).communicate()
        
        if lc:
            cameras = []
            for c in lc.split("\n"):
                cameras.append(c)
            return cameras, er

        return None, er

    def get_abilities(self, translate=True):
        """Read abilities of current camera.

        Args:
        translate --  if True (default), the locale langage will be used, if
                      the translation is available.

        Returns:
        list of abilities.
        """
        if translate:
            return subprocess.Popen(["gphoto2", "--abilities"], 
                                universal_newlines=True, 
                                stdout=subprocess.PIPE).communicate()

        return subprocess.Popen(["env", "LANG=C", "gphoto2", "--abilities"], 
                                universal_newlines=True, 
                                stdout=subprocess.PIPE).communicate()

    def get_memory_card_info(self, translate=True):
        if translate:
           return subprocess.Popen(["gphoto2", "--storage-info"], 
                                universal_newlines=True, 
                                stdout=subprocess.PIPE).communicate()

        return subprocess.Popen(["env", "LANG=C", "gphoto2", "--storage-info"], 
                                universal_newlines=True, 
                                stdout=subprocess.PIPE).communicate()     

    def list_files(self):
        """Make a list of files found into the current camera.

        Normally, gPhoto2 returns a list in this form:

            There is no file in folder '/'.
            There is no file in folder '/store_00010001'.
            There is no file in folder '/store_00010001/DCIM'.
            There are 22 files in folder '/store_00010001/DCIM/100NIKON'.
            #1     DSCN0890.JPG                 2335 KB 3072x2304 image/jpeg
            #2     DSCN0891.JPG                 2109 KB 3072x2304 image/jpeg
            ...
            There is 1 file in folder '/store_00010001/MISC'.
            #23    TRANSMRK.NJB                    4 KB application/x-unknown

        returns:
        tuple that content one list of files or an error message if exist.
        """
        return subprocess.Popen(["env", "LANG=C", "gphoto2", "--list-files"], 
                                    universal_newlines=True, 
                                    stdout=subprocess.PIPE).communicate()

    def get_exif(self, r):
        """Get the image metadata.

        Since we need to identify the string "EXIF tags:", Lang=C is used
        in parameter.

        Args:
        r -- range of files
        """

        cmd = ["env", "LANG=C", "gphoto2", "--show-exif", r]
        reply = subprocess.Popen(cmd, universal_newlines=True, 
                                stdout=subprocess.PIPE).communicate()

        return reply


    def get_thumbnails(self, rep, r=None):
        """Download all thumbnails.

        Download all thumbnails found into the current camera.
        Thumbnails are always downloaded into the current folder.

        Args:
        rep -- destination folder
        r -- range of files, beginning at 1
                it can be a list of numbers separated by a comma or
                spans ("first-last"). 
                Ranges are XOR (exclusive or)
                i.e. "5-11,7,9,15" download files 5, 6, 8, 10, 11 and 15
        """
        old_rep = os.getcwd()
        os.chdir(rep)
        if r is not None:
            reply = subprocess.Popen(["env", "LANG=C", "gphoto2",
                                        "--get-thumbnail", r], 
                                        universal_newlines=True, 
                                        stdout=subprocess.PIPE).communicate()

        else:
            reply = subprocess.Popen(["env", "LANG=C", "gphoto2", 
                                        "--get-all-thumbnails"], 
                                        universal_newlines=True, 
                                        stdout=subprocess.PIPE).communicate()

        os.chdir(old_rep)
        return reply

    def get_thumbnails_replace(self, rep, r=None):
        """Download all the thumbnails and replace existing files.

        Download all the thumbnails found into the current camera,
        if same files exist they're replaced,
        thumbnails are allways downloaded into the current folder.

        Args:
        rep -- destination folder
        r -- range of files, beginning at 1
                it can be a list of numbers separated by a comma or
                spans ("first-last"). 
                Ranges are XOR (exclusive or)
                i.e. "5-11,7,9,15" download files 5, 6, 8, 10, 11 and 15
        """
        old_rep = os.getcwd()
        os.chdir(rep)
        if r is not None:
            reply = subprocess.Popen(["gphoto2", "--get-thumbnails", r,
                                        "--force-overwrite"], 
                                        universal_newlines=True, 
                                        stdout=subprocess.PIPE).communicate()

        else:
            reply = subprocess.Popen(["gphoto2", "--get-all-thumbnails",
                                        "--force-overwrite"], 
                                        universal_newlines=True, 
                                        stdout=subprocess.PIPE).communicate()

        os.chdir(old_rep)
        return reply

    def kill_process(self):
        """Cancel the current subprocess.

        """
        if self.proc is not None:
            self.proc.kill()

    def get_images(self, rep, r=None):
        """Download files from current camera.

        Download images from current camera to the current directory,
        if r = None all files (jpeg, avi, wave, ...) are downloaded.
        Note: if a file with same name is already in directory, gphoto2
        may interrupt the transfert without return. 
        It's important to verify this case before all and use 
        get_images_replace() if necessary.

        Args:
        rep -- directory to copy files.
        r -- range of files, beginning at 1
                it can be a list of numbers separated by a comma or
                spans ("first-last"). 
                Ranges are XOR (exclusive or)
                i.e. "5-11,7,9,15" download files 5, 6, 8, 10, 11 and 15

        Emit a pyqtSignal 'fileDownloaded(file name)' for each file downloaded.
        Emit a pyqtSignal 'downloadingFinished(range of files)' at the end of
            the downloading.
        """
        old_rep = os.getcwd()
        os.chdir(rep)
        if r is not None:
            self.proc = subprocess.Popen(["env", "LANG=C", "gphoto2", 
                                        "--get-file", r], 
                                        universal_newlines=True,
                                        stdout=subprocess.PIPE)

            while 1:
                text = self.proc.stdout.readline()[:-1]
                if type(text) != str or text == '' and self.proc.poll() != None: 
                    break

                elif type(text) == str and len(text) > 6:
                    self.fileDownloaded.emit(text)

            self.proc = None
            self.downloadingFinished.emit(r)

        else:
            self.proc = subprocess.Popen(["gphoto2", "--get-all-files"], 
                                        universal_newlines=True, 
                                        stdout=subprocess.PIPE)

            while 1:
                text = self.proc.stdout.readline()[:-1]
                if type(text) != str or text == '' and self.proc.poll() != None: 
                    break

                elif type(text) == str and len(text) > 6:
                    self.fileDownloaded.emit(text)

            self.proc = None
            self.downloadingFinished.emit(r)

        os.chdir(old_rep)

    def get_images_replace(self, rep, r=None):
        """Download the files from current camera and replace existing files.

        Download the images from current camera to the current directory,
        if same files exists they're replaced.

        Args:
        rep -- directory to copy files.
        r -- range of files ; beginning at 1
                it can be a list of numbers separated by a comma or
                spans ("first-last"). 
                Ranges are XOR (exclusive or)
                i.e. "5-11,7,9,15" download files 5, 6, 8, 10, 11 and 15
                if r == None, all files (jpeg, raw, avi, wave) are downloaded.

        Emit a pyqtSignal 'fileDownloaded(file name)' for each file downloaded.
        Emit a pyqtSignal 'downloadingFinished(range of files)' at the end of
            the downloading.
        """
        old_rep = os.getcwd()
        os.chdir(rep)
        if r is not None:
            self.proc = subprocess.Popen(["gphoto2", "--get-file", r, 
                                        "--force-overwrite"], 
                                        universal_newlines=True, 
                                        stdout=subprocess.PIPE)

            while 1:
                text = self.proc.stdout.readline()[:-1]
                if type(text) != str or text == '' and self.proc.poll() != None: 
                    break

                elif type(text) == str and len(text) > 6:
                    self.fileDownloaded.emit(text)

            self.proc = None
            self.downloadingFinished.emit(r)

        else:
            self.proc = subprocess.Popen(["gphoto2", "--get-all-files", 
                                        "--force-overwrite"], 
                                        universal_newlines=True, 
                                        stdout=subprocess.PIPE)

            while 1:
                text = self.proc.stdout.readline()[:-1]
                if type(text) != str or text == '' and self.proc.poll() != None: 
                    break

                elif type(text) == str and len(text) > 6:
                    self.fileDownloaded.emit(text)

            self.proc = None
            self.downloadingFinished.emit(r)

        os.chdir(old_rep)


    def get_capture_preview(self, rep):
        """Capture a quick preview.

        Args:
        rep -- folder where the capture mut be saved
        """
        old_rep = os.getcwd()
        os.chdir(rep)
        reply = subprocess.Popen(["gphoto2", "--capture-preview"], 
                                    universal_newlines=True, 
                                    stdout=subprocess.PIPE).communicate()
        os.chdir(old_rep)
        return reply

    def capture(self, t):
        """Take a nice picture.

        NOT YET TESTED

        Args:
        t -- time in second
        """
        #print "Smile ..."
        #TODO Place the timer into a thread and make possible to cancel
        # the capture.
        time.sleep(t)
        reply = subprocess.Popen(["gphoto2", "--capture-image"], 
                                    universal_newlines=True, 
                                    stdout=subprocess.PIPE).communicate()
        #TODO make a multi-capture function

    def set_camera(self, c, p):
        """Set the current camera.

        Needed only if it's more than one camera.
        
        Args:
        c -- name of camera.
        p -- port where this camera is connected
        """
        subprocess.Popen(["gphoto2", "--camera", c, "--port", p], 
                                universal_newlines=True, 
                                stdout=subprocess.PIPE).communicate()

    def supported_cameras(self, folder=None):
        """Export a list of cameras supported by libgphoto2.

        Since this list is too long for the console we create a file
        named 'supportedCameras.txt'.

        Args:
        folder -- folder to save text file
        """
        if folder is None:
            folder = os.getcwd()

        f = os.path.join(folder, "supportedCameras.txt")
        reply = subprocess.Popen(["gphoto2", "--list-cameras"], 
                                    universal_newlines=True, 
                                    stdout=subprocess.PIPE).communicate()

        try:
            with open(f, "w") as objfile:
                objfile.write(reply[0])
                print("file %s created." %(f))
        except IOError as why:
            print("Error I/O:", why)

    def supported_ports(self):
        """Export a list of supported ports.

        """
        return subprocess.Popen(["gphoto2", "--list-ports"], 
                                    universal_newlines=True, 
                                    stdout=subprocess.PIPE).communicate() 

    def delete_images(self, bind, r=None):
        """Delete photo(s) into the camera.
        
        Args:
        r -- nummer or range of photo(s), if r = None, delete all photos.
        bind -- instance of text editor for viewing the progress of process.
        """
        if not r:
            reply = subprocess.Popen(["gphoto2", "--delete-all-files", 
                                        "--recurse"],  universal_newlines=True, 
                                        stdout=subprocess.PIPE)

            while 1:
                #The last character is '\n'
                text = reply.stdout.readline()[:-1]
                if text == '' and reply.poll() != None: 
                    break

                if len(text):
                    #The next two lines are specific to Qt widget
                    bind.appendPlainText(text)
                    QtCore.QCoreApplication.processEvents() 
    
        else:
            reply = subprocess.Popen(["gphoto2", "--delete-file", r, 
                                        "--recurse"], universal_newlines=True, 
                                        stdout=subprocess.PIPE)

            while 1:
                #The last character is '\n'
                text = reply.stdout.readline()[:-1]
                if text == '' and reply.poll() != None: 
                    break

                if len(text):
                    #The next two lines are specific to Qt widget
                    bind.appendPlainText(text)
                    QtCore.QCoreApplication.processEvents()

        return reply
