# -*- coding: utf-8 -*-

# importfromcamera.py
# this file is part of Oqapy serie 3
#
# Author: Vincent Vande Vyvre <vincent.vandevyvre@oqapy.eu>
# Home: http://www.oqapy.eu
# Copyright: 2010-2017 Vincent Vande Vyvre
# Licence: GPL3
#
# Dialog box for importing images from digital camera.
#
# Mapping PTP Format Codes to WIA Format GUIDs:
# http://msdn.microsoft.com/en-us/library/windows/hardware/ff545221%28v=vs.85%29.aspx


import sys
import os
import time
import shutil
import logging
lgg = logging.getLogger(__name__)

from threading import Thread

from PyQt5 import QtCore, QtGui, QtWidgets


import stylesheets as styles
from camerahandler.ui_camera import Ui_Camera
from camerahandler.IOgPhoto2 import IOcamera
from camerahandler.data import CAMERA_NAMES
from tools.path import Path
from warningmessages import WarningMessages

RAWFILES = ('3RF', 'ARI', 'ARW', 'BAY', 'CAP', 'CRW', 'CR2', 'DCS', 'DCR',
            'DNG', 'DRF', 'EIP', 'ERF', 'FFF', 'IIQ', 'K25', 'KDC', 'MEF', 
            'MOS', 'MRW', 'NEF', 'NRW', 'OBM', 'ORF', 'PEF', 'PTX', 'PXN',
            'R3D', 'RAF', 'RAW', 'RWL', 'RW2', 'RWZ', 'SR2', 'SRF', 'SRW',
            'X3F')

class ImportFromCamera(QtCore.QObject):
    searchFinished = QtCore.pyqtSignal()
    findThumbnail = QtCore.pyqtSignal(str)
    thumbnailsCreationFinished = QtCore.pyqtSignal()
    readError = QtCore.pyqtSignal(int)

    cameras = []
    curr_camera = None
    images = []
    raws = []
    thumbnails = []
    file_types = set()
    selected = []
    ico_size = 150
    is_importing = False

    def __init__(self, main):
        """Instanciate the ImportFromCamera class.

        """
        super().__init__()
        self.main = main
        self.cfg = main.cfg
        self.ui = Ui_Camera(self)
        self.cambox = self.ui.camera_box
        self.setbox = self.ui.importsett_box
        self.impbox = self.ui.import_box
        self.picbox = self.ui.picture_box
        self.cam = IOcamera()
        self.utils = main.utils

        self.cambox.cam_cmb.currentIndexChanged.connect(self.change_camera)
        self.cambox.refresh_btn.clicked.connect(self.set_camera_finder)
        self.cambox.info_btn.clicked.connect(self.get_camera_infos)
        self.cambox.show_content_btn.clicked.connect(self.show_camera_content)
        self.setbox.browse_btn.clicked.connect(self.get_import_folder)
        self.impbox.only_rdo.toggled.connect(self.on_category_selected)
        self.impbox.ext_cmb.currentIndexChanged.connect(
                                            self.on_selected_type_changed)
        self.impbox.select_all_btn.clicked.connect(self.select_all_items)
        self.impbox.clear_btn.clicked.connect(self.unselect_all)
        self.impbox.import_btn.clicked.connect(self.import_files)
        self.impbox.cancel_btn.clicked.connect(self.cancel_importing)
        self.ui.content_view.itemClicked.connect(self.on_item_clicked)

        self.searchFinished.connect(self._set_camera_toolbox)
        self.findThumbnail.connect(self._add_thumbnail)
        self.thumbnailsCreationFinished.connect(self._on_thumbnails_complete)
        self.readError.connect(self._show_error)
        self.cam.fileDownloaded.connect(self._message_handler)
        self.cam.downloadingFinished.connect(self.on_downloading_finished)

        try:
            self.current_fld = self.cfg.get(("folders", "camera"))
            self.setbox.imp_fld_led.setText(self.current_fld)
        except:
            self.current_fld = os.path.expanduser('~')

        self.temp_fld = self.main.workspace['camera']
        lgg.info('Startup')

        self.clean_temp_folder(self.temp_fld.path)
        self.set_camera_finder()
        self.all_processed = False

    def set_camera_finder(self):
        """Prepare the camera finder.

        """
        self.cambox.cam_com_lbl.setText(_("Searching cameras..."))
        self.cambox.cam_cmb.setEnabled(False)
        self.cambox.refresh_btn.setEnabled(False)
        self.cambox.cam_pgb.setMaximum(0)
        QtCore.QCoreApplication.processEvents()
        Thread(target=self.get_cameras).start()

    def get_cameras(self):
        """Search for camera(s) plugged.

        Emit the signal 'searchFinished' when done.
        """
        t, d = 0, False

        while t < 10:
            lgg.info('Get camera, run {0}'.format(t))
            reply = self.cam.search_cameras()
            if len(reply[0]) > 2 and len(reply[0][2:]) > 1:
                self.cameras = []

                for c in reply[0][2:]:
                    # Two first elements are title
                    if c:
                        c = c.split("  ")
                        cam = Camera(c[0], self)
                        cam.port = c[1].lstrip(" ")
                        self.cameras.append(cam)

                self.curr_camera = self.cameras[0]
                self.text = ""
                for i, c in enumerate(self.cameras):
                    self.text += "Camera:\n  {0}\nModel:\n  {1}\n  on port:"\
                                    " {2}".format(c.maker, c.model, c.port)

                d = True

                if not i:
                    self.cambox.cam_com_lbl.setText(_("Found 1 camera"))

                else:
                    self.cambox.cam_com_lbl.setText(_("Found {0} cameras".format
                                    (str(i +1))))

                break

            time.sleep(0.2)
            t += 1

        if not d:
            if reply[1]:
                self.text = reply[1]

            else:
                self.text = _("*** No camera detected. ***")

        else:
            reply = self.cam.about_camera()
            if "<gphoto-devel@lists.sourceforge.net>" in reply[0]:
                self.text = _("The camera seems already open\n"
                                "by an other application.\n\n"
                                "Please, unmount the camera and\n" 
                                "click on the button 'Refresh'.")

        lgg.info('Found {0} camera(s)'.format(len(self.cameras)))
        self.searchFinished.emit()

    def _set_camera_toolbox(self):
        """Sets enabled the camera toolbox widgets.

        """
        self.cambox.cam_cmb.clear()
        self.cambox.cam_pgb.setMaximum(10)
        self.cambox.cam_pgb.setValue(10)
        self.cambox.refresh_btn.setEnabled(True)
        self.cambox.camera_edit.clear()
        self.cambox.camera_edit.appendPlainText(self.text)

        if self.cameras:
            self.cambox.cam_cmb.blockSignals(True)
            for c in self.cameras:
                self.cambox.cam_cmb.addItem(c.name)

            self.cambox.cam_cmb.setCurrentIndex(0)
            self.cambox.cam_cmb.blockSignals(False)
            self.cambox.cam_cmb.setEnabled(True)
            self.cambox.info_btn.setEnabled(True)
            self.get_file_list()

    def get_camera_infos(self):
        """Get and launch the camera infos dialog box.

        """
        idx = self.cambox.cam_cmb.currentIndex()
        self.info_dialog = CameraInquiries()
        self.info_dialog.set_data(self.cameras, idx)
        self.info_dialog.exec_()

    def show_camera_content(self):
        """Load thumbnails found into the current camera to the preview area.

        When finished, emit the signal 'thumbnailsCreationFinished'.
        """
        self.begin = time.time()
        self._prepare_viewer()
        self.make_dirs()
        self.ui.editor.clear()
        self.ui.editor.appendPlainText(_("Loading thumbnails ..."))
        QtCore.QCoreApplication.processEvents()

        self.thumbnails = []
        self.ui.setCursor(QtCore.Qt.BusyCursor)

        def get_thumbs():
            keys = list(self.files_tree.keys())
            keys.sort(key=str.lower)
            for key in keys:
                fld = os.path.join(self.curr_camera.temp, key)
                range_ = ",".join([v[0] for v in self.files_tree[key]])

                reply = self.cam.get_thumbnails(fld, range_)

                try:
                    if reply[0].startswith('For debugging messages'):
                        self.readError.emit(1)
                except Exception as why:
                    lgg.warning('get_thumbs error: {0}'.format(why))

                else:
                    lgg.info('All thumbnails loaded from {0}'.format(key))
                    self.build_viewlist(key)

            tm = time.time() - self.begin
            lgg.info('Import thumbnails finished at: {0} sec.'.format(tm))
            self.thumbnailsCreationFinished.emit()

        Thread(target=get_thumbs).start()

    def build_viewlist(self, fld):
        """Seek, in a thread, into the temp folder. 

        When a new file appears, emit the signal 'findThumbnail'. 

        Args:
        fld -- folder where the thumbnails are loaded.
        """
        dr = os.path.join(self.curr_camera.temp, fld)
        fls = os.listdir(dr)
        fls.sort(key=str.lower)
        count = 1

        for f in fls:
            if f.startswith('thu'):
                i = os.path.join(dr, f)
                lgg.info('Found thumbnail {0}'.format(f))
                self.findThumbnail.emit(i)
                count += 1

        lgg.info('Found {0} thumbnails'.format(count))

    def _add_thumbnail(self, fl):
        """Create a new thumbnail instance when loaded.

        This function is a slot of the signal 'findThumbnail'.

        Args:
        thumb -- file's full path
        """
        # Need to keep the parent folder with the file basename
        name = str(fl).replace(self.curr_camera.temp, '')
        line = "".join(["* find thumb: ", name])
        self.ui.editor.appendPlainText(line)
        thumb = ThumbnailItem(self.curr_camera.temp, name[1:], 
                                self.ui.content_view)
        self.thumbnails.append(thumb)
        QtCore.QCoreApplication.processEvents()

    def _prepare_viewer(self):
        """Clean the preview area and the temp folder.

        """
        lgg.info('Reset thumbnails area')
        self.ui.content_view.clear()
        self.ui.content_view.setIconSize(QtCore.QSize(self.ico_size, 
                                        self.ico_size))

        for root, dirs, files in os.walk(self.curr_camera.temp, topdown=False):
            for name in files:
                try:
                    os.remove(os.path.join(root, name))
                except Exception as why:
                    lgg.warning('Clean folder {0} error: {1}'.format(
                                    self.curr_camera.temp, why))

            for name in dirs:
                try:
                    os.rmdir(os.path.join(root, name))
                except Exception as why:
                    lgg.warning('Clean folder {0} error: {1}'.format(
                                    self.curr_camera.temp, why))

    def make_dirs(self):
        for k in self.files_tree.keys():
            try:
                os.mkdir(os.path.join(self.curr_camera.temp, k))
            except Exception as why:
                lgg.warning('Create folder {0} error: {1}'.format(k, why))

    def _on_thumbnails_complete(self):
        """When thumbnails are complete, get and update the image metadata.

        This function is a slot of the signal 'thumbnailsCreationFinished'.
        """
        self.get_metadata()
        self.update_images_data()
        self.ui.setCursor(QtCore.Qt.ArrowCursor)

        if self.raws:
            self.set_raw_files()

        self.set_import_toolbox()

    def _show_error(self, idx):
        """Show error messages into the camera toolbox editor.

        Args:
        idx -- index of message
        """
        if idx == 1:
            self.ui.editor.appendPlainText(_("*** Read Error ***\n\n"
                                            "Please check your camera."))

        elif idx == 2:
            self.ui.editor.appendPlainText(_("*** Metadata read Error ***\n\n"
                                            "No picture info available.\n"
                                            "Please check your camera"))

    def get_metadata(self):
        """Get the image metadata.

        """
        if not self.thumbnails:
            return

        self.ui.editor.appendPlainText(_("Read metadata..."))
        QtCore.QCoreApplication.processEvents()
        lgg.info('Get metadata')

        rang = "-".join(["1", str(len(self.thumbnails))])
        data = self.cam.get_exif(rang)

        if '<gphoto-devel@lists.sourceforge.net>' in data[0]:
            self._show_error(2)
            return

        infos = data[0].split('EXIF tags:')
        infos.pop(0)

        for idx, thumb in enumerate(self.thumbnails):
            thumb.set_metadata(infos[idx])

    def change_camera(self, c):
        """Change the current camera.

        Args:
        c -- index of new camera
        """
        lgg.info('Change camera to: {0}'.format(self.cameras[c]))
        self.curr_camera = self.cameras[c]
        self.cam.set_camera(self.curr_camera.name, self.curr_camera.port)
        self.ui.content_view.clear()
        if not self.curr_camera.content:
            self.get_file_list()

        else:
            for item in self.curr_camera.content:
                self._add_thumbnail(item[1])

    def on_item_clicked(self, item):
        """A thumbnail has been clicked in the preview area.

        If the item is already selected, he's unselected, otherwise, he's
        displayed into the preview toolbox

        Args:
        item -- item instance
        """
        if item not in self.selected:
            self.selected.append(item)

        else:
            self.selected.remove(item)
            item.setSelected(False)

        if self.selected:
            # Always the last selected item is displayed
            self.picbox.set_picture(self.selected[-1])
            lgg.info('Thumbnails #{0} edited'.format(self.selected[-1].idx))

        else:
            self.picbox.clear_all()

    def get_file_list(self):
        """Get a list of pictures from camera.

        """
        self.ui.editor.appendPlainText("Reading content of camera...")
        QtCore.QCoreApplication.processEvents()
        reply = self.cam.list_files()

        if not reply[0]:
            self.cambox.show_content_btn.setEnabled(False)
            if reply[1]:
                self.ui.editor.appendPlainText(reply[1])

            else:
                self.ui.editor.appendPlainText("*** The memory card is empty ***")
            return

        self.cambox.show_content_btn.setEnabled(True)
        self.sort_files(reply[0])

    def sort_files(self, lst):
        """Sort the file list and update the thumbnail's data.

        Args:
        lst -- file list
        """
        self.files_tree = {}
        self.camera_files = []
        st = set()
        lines = lst.split("\n")
        for i in range(len(lines)):

            if 'There is no file' in lines[i]:
                continue

            elif 'There is 1 file' in lines[i] or 'There are' in lines[i]:
                # From the line
                # "There are 5 files in folder '/store_00010001/DCIM/103P_003'"
                # we use "103P_003" as folder name
                fld = lines[i].split(' ')[-1][1:-2].split('/')[-1]
                self.files_tree[fld] = []

            else:
                words = lines[i].split(' ')
                elem = [e for e in words if e]
                if not elem:
                    continue

                if elem[-1] == 'application/x-unknown':
                    if not self.compare_raw_file(elem[1]):
                        continue

                    self.raws.append(elem[0].strip().replace('#', ''))
                    elem[-1] = 'image/raw'

                elif elem[-1].split('/')[0] not in ('image', 'video'):
                    continue

                idx = elem[0].strip().replace('#', '')
                name = elem[1]
                size = elem[3]
                type_ = elem[-1]

                self.files_tree[fld].append((idx, name, size, type_))
                self.camera_files.append((idx, name, size, type_))

                t, m = type_.split('/')
                if t == 'video':
                    s = [t]

                else:
                    s = [t, m]

                st = st.union(set(s))

        for k in list(self.files_tree.keys()):
            # Remove empty folders
            if not len(self.files_tree[k]):
                del self.files_tree[k]

        self.curr_camera.content = self.camera_files
        self.file_types = st
        chain = "The camera has {0} files.".format(len(self.camera_files))
        lgg.info('Found {0} files into {1}'.format(len(self.camera_files),
                                        str(self.cambox.cam_cmb.currentText())))
        self.cambox.camera_edit.appendPlainText(chain)
        QtCore.QCoreApplication.processEvents()

    def update_images_data(self):
        """Sets the thumbnail's metadata.

        """
        cfl = self.camera_files[:]

        for th in self.thumbnails:
            name = th.fname.split('/')[1].replace('thumb_', '')

            for idx, item in enumerate(cfl):
                # Extension of thumbnail may be in lower case
                if item[1].lower() == name.lower():
                    th.update_data(cfl.pop(idx))
                    break

    def compare_raw_file(self, name):
        """Verify if the file is a raw file image.

        Args:
        name -- file name
        """
        lgg.info('compare_raw_file: {0}'.format(name))
        try:
            n, e = name.split('.')
        except:
            return False

        if e.upper() in RAWFILES:
            return True

        return False

    def set_raw_files(self):
        """Handle raw files.

        For some raw files libgphoto2 can't extract the thumbnail from the 
        file. So we use a 'No preview' image.
        """
        for raw in self.raws:
            # self.raws is the list of raw files
            is_shown = False
            for th in self.thumbnails:
                if th.idx == raw:
                    # the thumbnail has been extracted
                    is_shown = True
                    break

            if not is_shown:
                # no thumbnail, we create one
                name = 'RAWS/' + self.camera_files[int(raw)-1][1]
                target = os.path.join(self.curr_camera.temp, name)
                shutil.copy('medias/No_Preview_200x150.png', target)
                thumb = ThumbnailItem(self.curr_camera.temp, name, 
                                        self.ui.content_view)
                thumb.idx = raw
                thumb.fsize = self.camera_files[int(raw)-1][2]
                thumb.type_ = self.camera_files[int(raw)-1][3]
                self.thumbnails.append(thumb)

    def set_import_toolbox(self):
        """Set the import toolbox.

        """
        self.impbox.set_widgets_enabled(len(self.thumbnails) > 0)
        ext = list(self.file_types)
        ext.sort(key=str.lower)

        if 'image' in ext:
            ext.remove('image')
            ext.insert(0, 'Images')

        if 'video' in ext:
            ext.remove('video')
            ext.insert(1, 'Videos')

        self.impbox.ext_cmb.blockSignals(True)
        self.impbox.ext_cmb.clear()
        self.impbox.ext_cmb.addItems(ext)
        self.impbox.ext_cmb.blockSignals(False)
        self.impbox.all_rdo.setChecked(True)

    def select_all_items(self):
        """Select all items in the preview area.

        """
        self.ui.content_view.selectAll()
        self.selected = self.ui.content_view.selectedItems()
        lgg.info('Select all items')

    def unselect_all(self):
        """Unselect all items in the preview area.

        """
        self.ui.content_view.clearSelection()
        self.selected = []
        lgg.info('All items unselected')

    def on_selected_type_changed(self, idx):
        typ = str(self.impbox.ext_cmb.currentText())
        self.ui.content_view.blockSignals(True)

        if typ == 'Images':
            typ = 'image'

        elif typ == 'Videos':
            typ = 'video'

        for th in self.thumbnails:
            if typ in th.type_:
                th.setSelected(True)

            else:
                th.setSelected(False)

        self.ui.content_view.blockSignals(False)
        self.selected = self.ui.content_view.selectedItems()
        self.ui.on_selection_changed(len(self.selected))
        lgg.info('{0} item(s) selected'.format(len(self.selected)))

    def on_category_selected(self, b):
        """The radioButton 'Import only:' has been checked.

        Args:
        b -- boolean
        """
        self.impbox.ext_cmb.setEnabled(b)

        if b:
            self.on_selected_type_changed(None)

        else:
            self.ui.content_view.clearSelection()
            self.selected = []

    def get_import_folder(self):
        """Change the download folder.

        """
        d = QtWidgets.QFileDialog.getExistingDirectory(None, 
                    _("Open folder"), self.current_fld,
                    QtWidgets.QFileDialog.DontResolveSymlinks
                    | QtWidgets.QFileDialog.ShowDirsOnly)

        if not d:
            return

        else:
            self.current_fld = d
            self.setbox.imp_fld_led.setText(d)
            self.cfg.update(("folders", "camera"), d)
            lgg.info('Set downloading folder {0}'.format(d))

    def import_files(self):
        """Import files from the camera to a temp folder.

        """
        self.dest = self.get_destination_folder()
        if not self.dest:
            return

        self.swap = self.get_temp_folders()
        if not self.swap:
            return

        self.range_ = self._get_files_range()
        packs = self._get_packs()

        self.files_to_download = []
        self.files_saved = []
        self.impbox.import_pgb.setValue(0)
        self.impbox.import_pgb.setRange(0, len(self.range_))

        for i in self.range_:
            self.files_to_download.append(self.camera_files[int(i)-1][1])
        self.ui.setCursor(QtCore.Qt.BusyCursor)

        def get_files(*args):
            keys = list(args[0].keys())
            keys.sort(key=str.lower)

            for key in keys:
                dest = os.path.join(self.swap, key)
                rng = args[0][key]
                lgg.info('Download files {0}'.format(rng))
                lgg.info('to: {0}'.format(dest))
                reply = self.cam.get_images(dest, rng)

        lgg.info('Prepare downloading files: {0}'.format(self.range_))
        Thread(target=get_files, args=(packs,)).start()

    def _message_handler(self, mess):
        """Show a message for each file downloaded.

        This is a slot of the signal 'fileDownloaded' emit by the instance
        of IOgPhoto2.

        Args:
        mess -- message wich contains the basename file or eventually error
        """
        mess = str(mess)
        lgg.info('Receive message: {0}'.format(mess))

        if mess.startswith('For debugging messages'):
            self.impbox.report_lbl.setText('Cancel on error.')
            self._show_error(1)
            return

        fname = mess.split(' ')[-1]
        if fname in self.files_to_download:
            self.files_saved.append(fname)
            self.on_file_downloaded(fname)

        else:
            self.ui.editor.appendPlainText('Unexpected message: {0}'
                                            .format(mess))

    def on_downloading_finished(self, val):
        self.rename_files(val)
        if len(self.files_saved) == len(self.files_to_download):
            if self.setbox.remove_after_chb.isChecked():
                self.impbox.report_lbl.setText('Delete files...')
                self.delete_files()
            self.impbox.report_lbl.setText('Finished.')
            self.ui.setCursor(QtCore.Qt.ArrowCursor)

    def on_file_downloaded(self, fl):
        fl = str(fl)
        ln = len(self.files_saved)
        self.ui.editor.appendPlainText('File {0} saved'.format(fl))
        self.impbox.import_pgb.setValue(ln)
        self.impbox.report_lbl.setText('File {0} to {1}'.format(ln, 
                                        len(self.files_to_download)))

        self._mark_has_downloaded(int(self.range_[ln-1])-1)
        lgg.info('File {0} imported'.format(fl))

    def rename_files(self, fls):
        """Move imported files from the swap folder to the destination folder.

        """
        indexes = [int(i) for i in fls.split(',')]
        path = os.path.join(self.swap, self.thumbnails[indexes[0]-1].root)
        lgg.info('Rename from: {0}'.format(path))

        for fname in os.listdir(path):
            orig = os.path.join(path, fname)
            dest = self.get_filename(fname)
            os.rename(orig, dest)

        lgg.info('{0} files renamed'.format(len(indexes)))

    def _get_packs(self):
        packs = {}
        for k in self.files_tree.keys():
            packs[k] = []

        for i in self.range_:
            packs[self.thumbnails[int(i) - 1].root].append(i)

        for k in list(packs.keys()):
            if not packs[k]:
                del packs[k]

            else:
                packs[k] = ",".join(packs[k])

        return packs

    def get_filename(self, name):
        """Return the full path of destination file.

        If the file already exist in the destination folder a suffix (number)
        is appended to the filename.

        Args:
        name -- basename of the file
        """
        count = 1
        new = os.path.join(self.dest, name)
        n, e = os.path.splitext(name)

        while 1:
            if os.path.isfile(new):
                fl = "".join([n, "(%s)" % count, e])
                new = os.path.join(self.dest, fl)
                count += 1

            else:
                break

        return new

    def cancel_importing(self):
        """Stop the downloading process.

        """
        # In case of aborting the process, don't erase the content of the camera
        self.setbox.remove_after_chb.setChecked(False)
        self.cam.kill_process()
        lgg.info('Stop downloading process')

    def _get_files_range(self):
        fls = self.thumbnails

        if self.impbox.all_rdo.isChecked():
            range_ = [item.idx for item in fls]

        elif self.impbox.select_rdo.isChecked():
            range_ = [item.idx for item in fls if item.isSelected()]

        elif self.impbox.unselect_rdo.isChecked():
            range_ = [item.idx for item in fls if not item.isSelected()]

        else:
            self.on_selected_type_changed(None)
            range_ = [item.idx for item in fls if item.isSelected()]

        return range_

    def delete_files(self):
        """Delete picture(s) into the camera.

        """
        lgg.info('Delete files into the camera')
        self.ui.editor.appendPlainText(_("Delete images ..."))
        QtCore.QCoreApplication.processEvents()
        rng = ','.join(self.range_)

        reply = self.cam.delete_images(self.ui.editor, rng)

        if reply:
            self.ui.editor.appendPlainText(_("... Finished."))
            self.remove_thumbnails()

        else:
            self.ui.editor.appendPlainText(reply[1])
            lgg.warning('Deletion error: {0}'.format(reply[1]))

        QtCore.QCoreApplication.processEvents()

    def _mark_has_downloaded(self, idx):
        """Marks the thumbnails of the file downloaded.

        """
        self.thumbnails[idx].reduce_opacity()
        QtCore.QCoreApplication.processEvents()

    def remove_thumbnails(self):
        """Remove the thumbnails of files downloaded and erased into the camera.

        """
        for th in self.thumbnails:
            if th.idx in self.range_:
                self.ui.content_view.takeItem(self.ui.content_view.row(th))
                del th

    def _on_page_changed(self, group):
        if group == 1:
            self.set_capture_tool()

    def clean_temp_folder(self, fld):
        """Remove all files and subfolder in the temporary folder.

        """
        for root, dirs, files in os.walk(fld, topdown=False):
            for name in files:
                try:
                    os.remove(os.path.join(root, name))
                except Exception as why:
                    lgg.warning('Remove file {0} error in temp: {1}'
                                    .format(name, why))

            for name in dirs:
                try:
                    os.rmdir(os.path.join(root, name))
                except Exception as why:
                    lgg.warning('Remove folder {0} error: {1}'
                                    .format(name, why))

    def get_destination_folder(self):
        """Returns the destination folder.

        If the folder doesn't exists, it will be create.
        Call warning dialog box if the folder is read only.
        """
        dest = str(self.setbox.imp_fld_led.text())

        if not os.path.isdir(dest):
            try:
                os.makedirs(dest)
            except Exception as why:
                lgg.warning('Unable to create the directory: %s' % why)
                mssg = {"context": _("Directory creation"),
                        'warn': _("Unable to create folder:"),
                        'data': dest,
                        'msg': _('Reason:'),
                        'data1': str(why),
                        'btn': _('Close')}
                warn = WarningMessages(0, mssg)
                warn.build_dialog()
                return False

        else:
            if not os.access(dest, os.W_OK):
                lgg.warning('Directory: %s is read only' % dest)
                mssg = {"context": _("Import directory"),
                        'warn': _("Unable to use folder:"),
                        'data': dest,
                        'msg': _('Reason:'),
                        'data1': _("This directory is read only"),
                        'btn': _('Close')}
                warn = WarningMessages(0, mssg)
                warn.build_dialog()
                return False

        return dest

    def get_temp_folders(self):
        """Return a swap folder path for the downloading.

        """
        tmp = os.path.join(self.temp_fld.path, 'swap')
        if not os.path.isdir(tmp):
            os.mkdir(tmp)

        self.clean_temp_folder(tmp)
        for key in self.files_tree.keys():
            dest = os.path.join(tmp, key)
            try:
                os.mkdir(dest)
            except Exception as why:
                lgg.warning('Unable to create temp folder {0}\n  Reason: {1}'
                                 .format(key, why))
                mssg = {"context": _("Directory creation"),
                        'warn': _("Unable to create temporary folder:"),
                        'data': dest,
                        'msg': _('Reason:'),
                        'data1': str(why),
                        'btn': _('Close')}
                warn = WarningMessages(0, mssg)
                warn.build_dialog()
                return False

        lgg.info('Using swap: {0}'.format(tmp))
        return tmp

    # -+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-
    # Capture
    # -+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-

    def set_capture_tool(self):
        reply = self.cam.get_memory_card_info(False)
        if reply[0]:
            lines = reply[0].split('\n')

            for line in lines:
                if 'totalcapacity' in line:
                    tot = line.split('=')[1].split(' ')
                    cap = self.utils.format_number(tot[0])
                    self.capbox.cap_lbl.setText(" ".join([str(cap), tot[1]]))

                elif 'free=' in line:
                    tot = line.split('=')[1].split(' ')
                    free = self.utils.format_number(tot[0])
                    self.capbox.mem_lbl.setText(" ".join([str(free), tot[1]]))

    def get_quick_preview(self):
        lgg.info('Get quick preview')
        fld = self.get_temp_folder()
        reply = self.cam.get_capture_preview(fld)

        self.capbox.display_preview(fld)

    def start_capture(self):
        pass

    def cancel_capture(self):
        pass
        

class Camera:
    def __init__(self, name, parent=None):
        """Instanciate a camera object.

        Args:
        name -- device name
        """
        super().__init__()
        self.name = name
        self.maker = ''
        self.model = ''
        self.port = None
        self.capture = False
        self.temp = ''
        self.thumbs = ''
        self.content = []
        self.capacity = 0
        self.get_name()
        self.get_temp_folder(parent.temp_fld)

    def get_name(self):
        """Split the manufacturer and model names.

        """
        for name in CAMERA_NAMES:
            if self.name.startswith(name):
                self.maker = name.replace(',', '')
                self.model = self.name[len(name):]
                return

        sp = self.name.split(' ')
        self.maker = sp[0]
        self.model = ' '.join(sp[1:])

    def get_temp_folder(self, path):
        """Return the temp folder for this camera.

        Args:
        path -- usually ~/.oqapy/camera/
        """
        self.temp = path.path + self.name.replace(' ', '_')
        if os.path.isdir(self.temp):
            count = 0

            while 1:
                suffix = "".join(['(', str(count), ')'])
                if not os.path.isdir(self.temp + suffix):
                    break
                count += 1

            self.temp = self.temp + suffix

        self.thumbs = os.path.join(self.temp, 'items')

        try:
            os.makedirs(self.thumbs)
        except Exception as why:
            lgg.warning('Camera temp folder error: {0}'.format(why))


class ThumbnailItem(QtWidgets.QListWidgetItem):
    def __init__(self, orig, fl, parent=None):
        """Instanciate a thumbnail item.

        Args:
        orig -- path where the thumbnail has been downloaded
        fl -- parentfolder/basename of file
        parent -- the QListWidget instance
        """
        super().__init__(parent)
        self.parent = parent,
        self.idx = None
        self.type_ = ''
        self.orig = orig
        self.fname = fl
        self.root = fl.split('/')[0]
        self.path = os.path.join(orig, 'items', fl)
        self.text = ''
        self.data = None
        self.mimetype = os.path.splitext(fl)[1]
        self.date = ''
        self.size = ''
        self.orientation = ''
        self.iso = ''
        self.fnumber = ''
        self.speed = ''
        self.resolution = ''
        self.fsize = 0
        self.set_picture()

    def set_picture(self):
        th = self.create_thumbnail()
        txt = self.create_text()
        self.setIcon(QtGui.QIcon(th))
        self.setText(txt)
        self.setTextAlignment(QtCore.Qt.AlignHCenter)
        self.setFlags(QtCore.Qt.ItemIsSelectable | QtCore.Qt.ItemIsEnabled)

    def create_thumbnail(self, mark=False):
        """Create thumbnail for the viewer.
    
        Args:
        fn -- file name
        """
        img = self.get_pixmap()
        w, h = img.width(), img.height()
        # Black border
        brect = QtCore.QRect(0, 0, w+3, h+3)
        # White background
        bckgrnd = QtGui.QPixmap(QtCore.QSize(w+4, h+4))
        bckgrnd.fill(QtGui.QColor('white'))
        painter = QtGui.QPainter()
        painter.begin(bckgrnd)
        painter.setPen(QtGui.QPen(QtCore.Qt.black, 1,
                              QtCore.Qt.SolidLine, QtCore.Qt.RoundCap,
                              QtCore.Qt.MiterJoin))
        painter.drawRect(brect)
        # Destination rect
        drect = QtCore.QRect(2, 2, w, h)
        # Source rect
        srect = QtCore.QRect(0.0, 0.0, w, h)

        if mark:
            # Used to mark the file wich has been downloaded
            painter.setOpacity(0.5)

        painter.drawPixmap(drect, img, srect)
        painter.end()
        bckgrnd.save(self.path)

        return bckgrnd

    def get_pixmap(self):
        pix = QtGui.QPixmap(os.path.join(self.orig, self.fname))
        w, h = pix.width(), pix.height()

        if w > 250 or h > 250:
            return pix.scaled(250, 250,QtCore.Qt.KeepAspectRatio,
                                        QtCore.Qt.SmoothTransformation)

        return pix
    
    def create_text(self):
        """Create text for thumbnail in viewer.

        """
        txt = self.fname.split('/')[1]
        if txt.startswith("thumb_"):
            txt = txt[6:]

        return txt

    def set_metadata(self, data):
        width, height, xres, yres = 0, 0, 0, 0
        lines = data.split('\n')

        for line in lines:
            if line.startswith('Date'):
                self.date = line.split('|')[1].strip()
            elif line.startswith('PixelX'):
                width = line.split('|')[1].strip()
            elif line.startswith('PixelY'):
                height = line.split('|')[1].strip()
            elif line.startswith('Ori'):
                self.orientation = line.split('|')[1].strip()
            elif line.startswith('ISO'):
                self.iso = line.split('|')[1].strip()
            elif line.startswith('FNum'):
                self.fnumber = line.split('|')[1].strip()
            elif line.startswith('ExposureTime'):
                self.speed = line.split('|')[1].strip()
            elif line.startswith('XRe'):
                xres = line.split('|')[1].strip()
            elif line.startswith('YRe'):
                yres = line.split('|')[1].strip()

        self.size = (width, height)
        self.resolution = (xres, yres)

    def update_data(self, data):
        # data = (idx, name, size, type_)
        self.idx = data[0]
        self.fsize = data[2]
        self.type_ = data[3]

    def set_orientation(self, val=None):
        if val is None:
            if self.orientation is None or self.orientation == 'top - left':
                return

            a = {'bottom - right': 180, 'top - right': -90, 'bottom - left': 90}

            try:
                val = a[self.orientation]
            except Exception as why:
                lgg.warning('Orientation value error: {0}'
                                .format(self.orientation))

        self.parent.setRowHidden(self.parent.row(self), True)
        img = QtGui.QImage(os.path.join(self.orig, self.fname))
        matrix = QtGui.QTransform()
        img = img.transformed(matrix.rotate(val), 
                             QtCore.Qt.SmoothTransformation)
        img.save(os.path.join(self.orig, self.fname))

        self.set_picture()
        self.parent.setRowHidden(self.parent.row(self), False)
        self.parent.updateGeometries()

    def reduce_opacity(self):
        self.setSelected(False)
        th = self.create_thumbnail(True)
        self.setIcon(QtGui.QIcon(th))


class CameraInquiries(QtWidgets.QDialog):
    def __init__(self, parent=None):
        """Instanciate the dialog box for the camera abilities.

        """
        super().__init__(parent)
        self.resize(600, 650)
        self.setWindowTitle("Camera inquiries - Oqapy")
        self.gridLayout = QtWidgets.QGridLayout(self)
        self.gridLayout.setContentsMargins(2, 4, 2, 4)
        self.verticalLayout = QtWidgets.QVBoxLayout()
        self.editor = QtWidgets.QPlainTextEdit(self)
        self.editor.setReadOnly(True)
        self.editor.setHorizontalScrollBarPolicy(QtCore.Qt.ScrollBarAlwaysOff)
        self.verticalLayout.addWidget(self.editor)
        self.horizontalLayout = QtWidgets.QHBoxLayout()
        spacerItem = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, 
                                        QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout.addItem(spacerItem)
        self.camera_cmb = QtWidgets.QComboBox(self)
        self.horizontalLayout.addWidget(self.camera_cmb)
        self.save_btn = QtWidgets.QPushButton(self)
        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap("medias/save.png"), QtGui.QIcon.Normal, 
                                        QtGui.QIcon.Off)
        self.save_btn.setIcon(icon)
        self.horizontalLayout.addWidget(self.save_btn)
        self.close_btn = QtWidgets.QPushButton(self)
        icon1 = QtGui.QIcon()
        icon1.addPixmap(QtGui.QPixmap("medias/close.png"), QtGui.QIcon.Normal, 
                                        QtGui.QIcon.Off)
        self.close_btn.setIcon(icon1)
        self.horizontalLayout.addWidget(self.close_btn)
        self.verticalLayout.addLayout(self.horizontalLayout)
        self.gridLayout.addLayout(self.verticalLayout, 0, 0, 1, 1)
        self.save_btn.setText(QtCore.QCoreApplication.translate("self", 
                                "Save", None))
        self.close_btn.setText(QtCore.QCoreApplication.translate("self", 
                                "Close", None))

        self.camera_cmb.currentIndexChanged.connect(self.on_camera_changed)
        self.save_btn.clicked.connect(self.save_infos)
        self.close_btn.clicked.connect(self.accept)
        self.iocam = IOcamera()

        self.setStyleSheet(styles.WIDGET)
        self.editor.setStyleSheet(styles.TEXTEDIT)
        self.editor.verticalScrollBar().setStyleSheet(
                                                    styles.VSCROLLBAR)
        self.camera_cmb.setStyleSheet(styles.COMBO)
        self.save_btn.setStyleSheet(styles.PUSHBUTTON)
        self.close_btn.setStyleSheet(styles.PUSHBUTTON)

    def set_data(self, cams, idx):
        self.cameras = cams
        for c in self.cameras:
            self.camera_cmb.addItem(c.name)

        self.camera_cmb.setCurrentIndex(idx)

    def on_camera_changed(self, idx):
        self.editor.clear()
        self.current = self.cameras[idx]
        self.iocam.set_camera(self.current.name, self.current.port)
        reply = self.iocam.about_camera()

        if reply[0]:
            self.text = reply[0].decode('utf8', 'replace')
            self.editor.appendPlainText(self.text)

        elif reply[1] is not None:
            self.editor.appendPlainText(reply[1])

        else:
            self.editor.appendPlainText("** No information available. **"
                                            "\n\n  This camera seems not"
                                            " support this feature.")

    def save_infos(self):
        """Save the camera info into a text file.

        """
        fld = os.path.expanduser('~')
        cmr = self.current.name.replace(' ', '_')
        fl = os.path.join(fld, cmr)
        format = 'txt'
        name = QtWidgets.QFileDialog.getSaveFileName(
                        None, "Save File", fl,
                        "Files (*.%s)" % format)[0]

        if not len(name):
            return

        try:
            with open(name, 'w') as outf:
                outf.write(self.text.encode('utf8', 'replace'))
        except Exception as why:
            self.editor.appendPlainText("\n*** Write error ***\n\n"
                                            "Reason: {0}".format(why))

