# -*- coding: utf-8 -*-

# horizontalslider.py
# this file is part of Oqapy serie 3
#
# Author: Vincent Vande Vyvre <vincent.vandevyvre@oqapy.eu>
# Home: http://www.oqapy.eu
# Copyright: 2013-2016 Vincent Vande Vyvre
# Licence: GPL3
#
# A horizontal slider with three buttons; Minus, Plus and Reset

import sys

from PyQt5 import QtCore, QtGui, QtWidgets


class HorizontalSlider(QtWidgets.QWidget):
    valueChanged = QtCore.pyqtSignal(int)
    def __init__(self, parent=None):
        super(HorizontalSlider, self).__init__(parent)
        self.layout = QtWidgets.QHBoxLayout(self)
        self.layout.setContentsMargins(0, 0, 0, 0)
        self.layout.setSpacing(0)
        font = QtGui.QFont()
        font.setPointSize(10)
        font.setBold(True)
        font.setWeight(75)
        self.minus = QtWidgets.QToolButton(self)
        self.minus.setFont(font)
        self.minus.setText(u'-')
        self.slider = QtWidgets.QSlider(self)
        self.slider.setOrientation(QtCore.Qt.Horizontal)
        self.plus = QtWidgets.QToolButton(self)
        self.plus.setFont(font)
        self.plus.setText(u'+')
        self.reset = QtWidgets.QToolButton(self)
        self.reset.setFont(font)
        self.reset.setText(u'←')

        self.set_normal_style()
        self.set_range((0, 99))
        self.single_step = 1
        self.default_value = 0

        self.minus.clicked.connect(self.decrement_value)
        self.plus.clicked.connect(self.increment_value)
        self.slider.valueChanged.connect(self.on_value_changed)
        self.reset.clicked.connect(self.reset_value)
        self.slider.enterEvent = self.enter_event

    def set_range(self, rng):
        self._min = rng[0]
        self._max = rng[1]
        self.slider.setMinimum(rng[0])
        self.slider.setMaximum(rng[1])

    def set_single_step(self, value):
        if value >  self._max / 3:
            raise ValueError(
                   "HorizontalSlider.set_single_step arg too large: %s" % value)

        self.single_step = value

    def set_value(self, value, blocksignal=False):
        self.slider.blockSignals(blocksignal)
        self.slider.setValue(value)
        self.slider.blockSignals(False)

    def set_default_value(self, value):
        self.default_value = value

    def decrement_value(self):
        self.set_value(self.slider.value() - self.single_step)

    def increment_value(self):
        self.set_value(self.slider.value() + self.single_step)

    def on_value_changed(self, val):
        self.minus.setEnabled(val > self._min)
        self.plus.setEnabled(val < self._max)
        self.valueChanged.emit(val)

    def get_value(self):
        return self.slider.value()

    def reset_value(self):
        self.slider.setValue(self.default_value)

    def set_layout_style(self, style):
        if style == self.style:
            return

        self.remove_widgets()

        if style == 'normal':
            self.set_normal_style()

        elif style == 'left':
            self.set_left_style()

        elif style == 'right':
            self.set_right_style()

        else:
            sys.stderr.write('HorizontalSlider.set_layout_style Error: '
                             'Invalid style: %s' % style)

    def remove_widgets(self):
        self.layout.removeWidget(self.minus)
        self.layout.removeWidget(self.slider)
        self.layout.removeWidget(self.plus)
        self.layout.removeWidget(self.reset)

    def set_normal_style(self):
        self.layout.addWidget(self.minus)
        self.layout.addWidget(self.slider)
        self.layout.addWidget(self.plus)
        self.layout.addWidget(self.reset)

    def set_left_style(self):
        self.layout.addWidget(self.reset)
        self.layout.addWidget(self.minus)
        self.layout.addWidget(self.plus)
        self.layout.addWidget(self.slider)

    def set_right_style(self):
        self.layout.addWidget(self.slider)
        self.layout.addWidget(self.minus)
        self.layout.addWidget(self.plus)
        self.layout.addWidget(self.reset)

    def enter_event(self, event):
        self.setToolTip(str(self.slider.value()))
        QtWidgets.QSlider.enterEvent(self.slider, event)
        
