# imageformatcombobox.py
# This file is part of Oqapy serie 3
#
# Author: Vincent Vande Vyvre <vincent.vandevyvre@oqapy.eu>
# Home: http://www.oqapy.eu
# Copyright: 2013-2016 Vincent Vande Vyvre
# Licence: GPL3
#
# Combo box for extensions of image's files

from PyQt5 import QtGui, QtWidgets

from ioimage import IOImage

class ImageFormatComboBox(QtWidgets.QComboBox):
    """Define a comboBox widget designed for images file formats.

    """
    def __init__(self, parent=None):
        super().__init__(parent)
        self.setDuplicatesEnabled(False)
        self.icon = QtGui.QIcon(QtGui.QPixmap("medias/read_only.png"))
        self.set_writable_formats()
        self.last_is_readonly = False
        self.build_view()

    def set_current_item(self, t):
        """Set the current text.

        Args:
        t -- str(format), the dot will be added if not present
        """
        if not t.startswith("."):
            t = ".{0}".format(t)

        if self.last_is_readonly:
            self.removeItem(self.count() - 1)
            self.last_is_readonly = False

        idx = self.findText(t)
        if idx == -1:
            idx = self.add_new_item(t)

        self.setCurrentIndex(idx)

    def set_writable_formats(self):
        """Create the list of writable formats.

        """
        imgProc = IOImage()
        f = imgProc.get_write_format_support()
        s = set([])
        for i in f:
            # Add the dot and reject duplicate
            s.add("." + i.lower())

        # Add the .pgm format wich is not returned by 
        # QImageWriter.supportedImageFormats()
        # https://bugreports.qt.nokia.com/browse/QTBUG-22958
        s.add(".pgm")
        l = list(s)
        l.sort()
        self.writable_formats = l
        
    def build_view(self):
        """Populate the popup menu.

        """
        self.clear()
        self.addItems(self.writable_formats)

    def add_new_item(self, item):
        """Add a new item in popup menu.

        The popup menu contains only the formats wich are writable, if a format
        non writable is given, it will be appended at the popup menu with
        an appropriate icon.

        Args:
        item -- Extension file (included dot)
        """
        # Check for synonym, by default, popup menu contains only lowercase
        idx = self.findText(item.lower())
        if idx == -1:
            idx = self.findText(item.upper())

        if idx != -1:
            # The new item replace an old one
            self.removeItem(idx)
            self.insertItem(idx, item)

        else:
            idx = self.count()
            self.addItem(self.icon, item)
            self.last_is_readonly = True

        return idx
