# -*- coding: utf-8 -*-

# config.py
# This file is part of Oqapy serie 3
#
# Author: Vincent Vande Vyvre <vincent.vandevyvre@oqapy.eu>
# Home: http://www.oqapy.eu
# Copyright: 2012-2018 Vincent Vande Vyvre
# Licence: GPL3

# Configuration and user's preferences handler
# http://seriot.ch/parsing_json.php

import os
import json
import logging
lgg = logging.getLogger(__name__)


class Config:
    def __init__(self, core):
        self.core = core
        wspace = core.workspace['root'].path
        self.block_update = True
        self.cfg_file = os.path.join(wspace, 'user_config')
        if not os.path.isfile(self.cfg_file):
            # Old config is not readable due to a conflict PyQt4 ~ PyQt5
            self.data = self.create_config()

        else:
            self.data = self.load_config()

        self.shrtct_file = os.path.join(wspace, 'shortcuts')
        if not os.path.isfile(self.shrtct_file):
            self.shortcuts = self.create_shortcuts()
            self.save_shortcuts()

        else:
            self.shortcuts = self.load_shortcuts()

        self.update_config()
        self.block_update = False

    def load_config(self):
        lgg.info('Load config from: %s' % self.cfg_file)
        try:
            with open(self.cfg_file, 'r') as objfile:
                cnt = objfile.read()
                return json.loads(cnt)
        except Exception as why:
            lgg.warning("Read error from: {0},".format(self.cfg_file))
            lgg.warning("reason: {0}.".format(why))
            return self.create_config()

    def load_shortcuts(self):
        lgg.info('Load shortcuts from: %s' % self.shrtct_file)
        try:
            with open(self.shrtct_file, 'r') as objfile:
                cnt = objfile.read()
                return json.loads(cnt)
        except Exception as why:
            lgg.warning("Read error from: {0},".format(self.shrtct_file))
            lgg.warning("reason: {0}.".format(why))
            return self.create_shortcuts()

    def save_config(self):
        lgg.info('Save config')
        self.update_recents()
        jsn = json.dumps(self.data, sort_keys=True, indent=4, 
                         separators=(',', ': '), ensure_ascii=False)
        try:
            with open(self.cfg_file, 'wb') as outfile:
                outfile.write(jsn.encode('utf-8', 'replace'))
                lgg.info("Configuration saved")
        except Exception as why:
            lgg.warning("Write error file: {0}".format(self.cfg_file))
            lgg.warning("reason: {0}".format(why))

    def save_shortcuts(self):
        lgg.info('Save shortcuts')
        jsn = json.dumps(self.shortcuts, sort_keys=True, indent=4, 
                         separators=(',', ': '), ensure_ascii=False)
        try:
            with open(self.shrtct_file, 'wb') as outfile:
                outfile.write(jsn.encode('utf-8', 'replace'))
                lgg.info("Configuration saved")
        except Exception as why:
            lgg.warning("Write error file: {0}".format(self.shrtct_file))
            lgg.warning("reason: {0}".format(why))

    def on_config_changed(self):
        """Save the config when updated.

        """
        if not self.block_update:
            self.save_config()

    def create_config(self):
        """Return the default configuration.

        """
        lgg.info('Create config default')
        from data import CONFIG_DEFAULT
        return CONFIG_DEFAULT

    def create_shortcuts(self):
        """Return the default shortcuts.

        """
        lgg.info('Create shortcuts default')
        from data import SHORTCUTS
        return SHORTCUTS

    def save_on_exit(self):
        """Save the config when the application is closed.

        """
        self.block_update = True
        self.data['startup']['trailer'] = self.core.ui.stackedtrailers.isVisible()
        targets = [p.path.path for p in self.core.targets if p is not None]
        self.data['sorting']['paths'] = targets
        w, h = self.core.ui.width(), self.core.ui.height()
        self.data['startup']['size'] = w, h
        self.save_config()

    def update_recents(self):
        """Update the list of recent folders.

        """
        self.data['folders']['recents'] = self.core.recents.path_strs

    def find_subvalue(self, items):
        """return the key-subkeys path of a configuration item.

        Args:
        items -- list of keys
        """
        member = self.data
        for item in items:
            member = self.get_member(member, item)

        return member

    def get_member(self, base, key):
        """Return a configuration item.

        If the key does not exists,it is created with the value None.

        Args:
        base -- keys path 
        key -- key of the item
        """
        if not key in base:
            base[key] = None

        return base[key]

    def get(self, keys):
        """Return the value of a configuration item.

        Args:
        keys -- list of key-subkeys
        """
        member = self.find_subvalue(keys[:-1])
        return member[keys[-1]]

    def update(self, keys, value):
        """Update the configuration.

        Args:
        keys -- list of key-subkeys
        value -- the value of the item
        """
        member = self.find_subvalue(keys[:-1])
        member[keys[-1]] = value
        self.on_config_changed()

    def append(self, keys, value):
        """Append a new value of a configuration item.

        Args:
        keys -- list of key-subkeys
        value -- the value to append
        """
        member = self.find_subvalue(keys[:-1])
        if not isinstance(member[keys[-1]], list):
            raise TypeError("Config.append: value of '%s' is not a list"
                            % " ".join(keys))

        member[keys[-1]].append(value)
        self.on_config_changed()

    def remove(self, keys, value):
        """Remove a value of a configuration item.

        Args:
        keys -- list of key-subkeys
        value -- the value to remove
        """
        member = self.find_subvalue(keys[:-1])
        if not isinstance(member[keys[-1]], list):
            raise TypeError("Config.remove: value of '%s' is not a list"
                            % " ".join(keys))

        member[keys[-1]].remove(value)

    def pop(self, keys, place):
        """Remove and return a value of a configuration item.

        Args:
        keys -- list of key-subkeys
        place -- int() if the item is a list, str() if the item is a dict
        """
        member = self.find_subvalue(keys[:-1])
        if not isinstance(member[keys[-1]], (list, dict)):
            raise TypeError("Config.pop: value of '%s' is not a list"
                            % " ".join(keys))

        value = member[keys[-1]].pop(place)
        self.on_config_changed()
        return value

    def exists(self, keys):
        """Return True if the item exists.

        Args:
        keys -- list of key-subkeys
        """
        member = self.find_subvalue(keys[:-1])
        return keys[-1] in member

    def set_source_folder(self, folder):
        """Set the source folder at startup.

        Args:
        folder -- the full path of the folder
        """
        if self.get(('startup', 'open')):
            if isinstance(folder, str):
                self.update(('startup', 'open'), folder)

            else:
                try:
                    self.update(('startup', 'open'), folder.path)
                except:
                    pass

    def update_config(self):
        from data import VERSION, VERSION_STR
        self.update(('app', 'vers'), VERSION)
        self.update(('app', 'svers'), VERSION_STR)
        if not self.exists(("multiviews",)):
            # new in version 3.1.0
            self.data["multiviews"] = {"rows": 2, "columns": 4}
