# -*- coding: utf-8 -*-

# core.py
# This file is part of Oqapy serie 3
#
# Author: Vincent Vande Vyvre <vincent.vandevyvre@oqapy.eu>
# Home: http://www.oqapy.eu
# Copyright: 2014-2018 Vincent Vande Vyvre
# Licence: GPL3
#
# Oqapy is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Oqapy is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Oqapy.  If not, see <http://www.gnu.org/licenses/>.
#
# Oqapy main class


import os
import sys
import datetime
import time
import glob
import shutil

# devel usage
#import tracemalloc
#tracemalloc.start()

import logging
lgg = logging.getLogger(__name__)

from threading import Thread, Timer
try:
    from shutil import disk_usage
except ImportError:
    from tools.diskusage import disk_usage

from PyQt5.QtCore import (Qt, QLocale, QObject, pyqtSignal, QCoreApplication,
                          QT_VERSION_STR, PYQT_VERSION_STR, QPoint, QSize, )
from PyQt5.QtGui import QImageReader
from PyQt5.QtWidgets import (QApplication, QDesktopWidget, QFileDialog,
                             QGraphicsPixmapItem, QGraphicsView)

from config import Config
from contextmenuscene import ContextMenuScene
from croptool import CropTool
from data import VERSION_STR
from demosaicing.rawviewer import RawViewer
from filtering import Filtering
from globalmenu import GlobalMenu
from grid import Grid
from ioimage import IOImage
from magnifier import Magnifier
from metadataeditor import MetadataEditor
from picture import Picture
from previewer import PreviewHandler
from shredder import Shredder
from state import State
from thumbnailer import Thumbnailer
from trailers import Trailer
from utils import Utils
from viewing import Viewing
from warningmessages import WarningMessages
from gui.aerialview import AerialView
from gui.altscreen import AltScreen
from gui.main_ui import MainUi
from metadata.metadata import Metadata
from tools.path import Path
from tools.rectsize import RectSize
from multiviews import MultiViews

import gettext
LOC_PATH = '/usr/share/'
gettext.install('oqapy', LOC_PATH, True)
# For testing a specific translation, comment the previous line,
# uncomment the two next lines and set the language code.
#tr = gettext.translation('oqapy', LOC_PATH, languages=['de'])
#tr.install()

LOC = QLocale.system().name()
LANG = QLocale(LOC)

# https://rhettinger.wordpress.com/2011/05/26/super-considered-super/
# http://pythonhosted.org/psutil
# pyreverse -o svg -p oqapy_uml -A -my -S core.py


class OqapyCore:
    """Define the main class of oqapy.

    """
    def __init__(self, appli, args):
        # instance of QApplication
        self.appli = appli
        self.source_fld = None
        self.first_image = 0
        self.current_index = 0
        self.raws = []
        self.handle_raw_files = False
        self.on_startup = True
        self.trash_enabled = True
        self.multiviewing = False
        self.utils = Utils(self)
        self.install_environment()
        self.setup_targets()
        self.ui = MainUi(self)
        self.ui.show()
        QCoreApplication.processEvents()
        self.ui.targetarea.build_targets()
        self.state = State(self)
        self.globalmenu = GlobalMenu(self)
        self.globalmenu.configure()
        self.state.set_current_state(0, True)
        v = self.state.current_viewer
        self.viewing = Viewing(self, v)
        self.trailer = Trailer(self)
        self.trailer.configure(self.cfg.get(('trailer', )))
        self.ui.configure()
        self.ui.toolbar.configure()
        self.ui.toolpanel.configure_toolboxes()
        self.shredder = Shredder(self)
        self.config_shredder()
        self.filtering = Filtering(self)
        self.create_targets_tabs()
        self.aerial_view = False
        self.magnifier = False
        self.filtered_mode = False
        self.old_context = 0
        self.metadataeditor = MetadataEditor(self)
        self.geo_view = self.ui.toolpanel.g9n_view.webView
        self.geo_view._setup((50.8313, 4.3850, 16), self.workspace['g9n'].path)
        self.state.stateChanged.connect(self.ui.toolbar.on_state_changed)
        self.grid = None
        self.crop = False
        self.rawviewer = RawViewer(self)
        self.ui.toolpanel.configure_signals()

        self.handle_args(args)
        if self.alt_screen and self.cfg.get(('startup', '2screens')):
            # https://specifications.freedesktop.org/wm-spec/wm-spec-1.3.html
            self.state.is_dual_screen = True

        if self.source_fld is None:
            st = self.cfg.get(('startup', ))
            if st['always_open']:
                self.open_source_folder(st['always_open'])

            elif st['open']:
                self.open_source_folder(st['open'])

        else:
            self.open_source_folder(self.source_fld, True)

    def handle_args(self, args):
        lgg.info('Handle_args %s' % str(args))
        if args:
            if isinstance(args, str):
                self.set_source_folder(args)

            elif isinstance(args, list):
                self.set_source_folder(args[0])

    def set_source_folder(self, path):
        """Define the source folder.

        args:
        path -- the full path of a folder or an image file
        """
        lgg.info('Set source folder: %s' % str(path))
        try:
            f = Path(path)
        except Exception as why:
            lgg.warning('Bad argument, error: %s' % why)
            self.source_fld = None

        else:
            if f.exists:
                if f.is_dir:
                    self.source_fld = SourceFolder(self, f)

                else:
                    f = Path(f.parents)
                    self.source_fld = SourceFolder(self, f)
                    self.first_image = path

                self.recents.add_path(f)

            else:
                lgg.info('No such file `%s`' % path)
                self.source_fld = None

    def install_environment(self):
        """Install the working environment.

        """
        self.workspace = self.utils.define_workspace()
        self.cfg = Config(self)
        self.set_recent_folders()
        # check dual-screen
        self.detect_screens()

    def reload_config(self):
        self.cfg = Config(self)
        self.cfg.block_update = True
        self.ui.configure()
        self.ui.toggle_scroll_bars(self.cfg.get(('style', 'scrlbars')))
        self.ui.toolpanel.configure_toolboxes()
        if self.trailer.check_config(self.cfg.get(('trailer',))):
            self.trailer.clean_view()
            self.build_trailer()
        self.rawviewer.viewer.step_zoom = self.cfg.get(('zoom', 'step'))
        self.cfg.block_update = False

    def config_shredder(self):
        """Configure the trash can.

        """
        clean = self.cfg.get(('startup', 'empty_trash'))
        self.shredder.setup(clean)

    def set_recent_folders(self):
        """Build the recent folder's list.

        """
        number = self.cfg.get(('startup', 'numb_recents'))
        self.recents = Recents(self, number, 
                               self.cfg.get(('folders', 'recents')))

    def setup_targets(self):
        """Buid the list of target folders.

        """
        discarded = []
        lgg.info('Set target folders')
        for path in self.cfg.get(('sorting', 'paths')):
            if path is not None:
                if not os.path.isdir(path):
                    discarded.append(path)

        for d in discarded:
            # Remove the folder which no longer exists
            self.cfg.remove(('sorting', 'paths'), d)

        paths = self.cfg.get(('sorting', 'paths'))
        self.targets = [Repertory(p) for p in paths]
        # Complete to 24 items
        self.targets.extend([None]*(24-len(self.targets)))

    def detect_screens(self):
        """Search for screens.

        If more than one screen are found, the screen 0 is defined as main 
        screen and the screen 1 is defined as alternate screen.
        """
        self.alt_screen = False
        dt = QDesktopWidget()
        ns = dt.screenCount()
        self.main_screen = dt.primaryScreen()
        geom = dt.screenGeometry(0)
        lgg.info('Found screen: %s' % geom)
        if ns > 1:
            geom = dt.screenGeometry(1)
            size = (geom.width(), geom.height())
            offset = (geom.x(), geom.y())
            lgg.info('Found second screen: %s x %s' % size)
            self.alt_screen = AltScreen(self, dt.screen(1), size, offset) 

    def create_targets_tabs(self):
        """Build the tab widget for the sorting mode.

        """
        self.targets_tabs = self.ui.stackedviews.sorting_tab
        for i, t in enumerate(self.targets):
            if t is None:
                break
            self.set_target_tab(i)

    def close_app(self, from_menu=False):
        self.cfg.save_on_exit()
        if self.state.context == 6:
            self.state.close_multiviewing()

        self.state.set_current_state(0)
        QCoreApplication.processEvents()
        self.globalmenu.widget.setParent(None)
        self.globalmenu.widget.deleteLater()
        del self.globalmenu
        self.utils.clean_all_temp_folders()
        if self.alt_screen:
            self.alt_screen.scene_.clear()
            del self.alt_screen.scene_
            del self.alt_screen

        view = self.state.current_viewer
        del view
        if from_menu:
            QCoreApplication.quit()
        # quiet exit
        QCoreApplication.processEvents()
        #self.appli.quit()

    def link_new_target(self, place, path=None):
        """Link an new target to a target's button.

        Args:
        place -- the index of the target, 0 to 23
        path -- the path of the target, not provided if called from a button
        """
        lgg.info('Link target at: %s' % place)
        if path is None:
            path = self.get_dir()

        # Check if the folder isn't already linked
        if path and path not in self.get_targets_paths_list():
            rep = Repertory(path)
            if not rep.check_validity():
                return

            button = self.ui.targetarea.targets_buttons[place]
            self.targets[place] = rep
            self.targets[place].button = button
            button.set_text(os.path.basename(path))
            self.state.current_viewer.setFocus(True)
            self.set_target_tab(place)
            tmp = self.get_target_temp_folder(path)
            self.targets_tabs.set_temp_folder(place+1, tmp)
            self.recents.add_path(Path(path))

    def get_targets_paths_list(self):
        """Return the list of the target's paths.

        """
        return [p.path.path for p in self.targets if p is not None]

    def set_target_tab(self, index):
        """Insert a new target tab.

        Args:
        index -- The index of the target
        """
        lgg.info('Set target tab: %s' % index)
        # Tab 0 is reserved for the source folder
        self.targets_tabs.insert_tab(index+1, self.targets[index])

    def unlink_target(self, button):
        """Remove an existing target.

        Args -- the target button instance
        """
        lgg.info('Unlink target: %s' % button)
        self.ui.targetarea.targets_buttons[button].disable_folder()
        self.targets[button] = None
        self.targets_tabs.remove_tab(button+1)

    def get_target_temp_folder(self, target):
        return

    def get_target_index(self, path):
        """Returns the index of a target folder.

        Args:
        path -- path of the target
        """
        try:
            return self.get_targets_paths_list().index(path)
        except (IndexError, ValueError):
            return -1

    def apply_geotagging(self, lat, lon):
        """Apply the geolocation to the current picture.

        This called by the mapviewer.

        Args:
        lat -- the latitude value as string
        lon -- the longitude value as string
        """
        self.current_picture.write_geotag(lat, lon) 

    #------------------------------------------------------------------------
    # Slots
    #------------------------------------------------------------------------
    def open_new_source(self):
        """Browse to a new source folder and open it.

        Called from the global menu.
        """
        fld = self.browse_to_source_folder()
        QCoreApplication.processEvents()
        if fld is not None:
            self.open_source_folder(fld)

    def on_target_clicked(self, index, button):
        """Called when a target button has been clicked.

        args:
        index -- the target id
        button -- the mouse button
        """
        if button != 2:
            lgg.info('Target %s pressed with button %s' %(index, button))
            if self.targets[index] is None:
                # No folder linked to this target button
                self.link_new_target(index)
                return

            if self.state.context == 6 \
                    and not self.ui.stackedviews.sorting_tab.currentIndex():
                if self.multiviewing.selected:
                    self.multi_image_to_target(index, button)

                return

            self.send_to_target(index, button) 

    def import_from_camera(self):
        """Display the Import from Camera dialog box.

        """
        lgg.info('Launch Import from Camera dialog')
        from camerahandler.importfromcamera import ImportFromCamera
        ifc = ImportFromCamera(self)

    def launch_printing_dialog(self):
        """Display the Printing dialog box.

        """
        if self.source_fld is None:
            return

        lgg.info('Launch Printing dialog')
        from printing.printing import Printing
        printing = Printing(self, self.source_fld.images, self.viewing.counter)

    def show_about_dialog(self):
        """Display the About dialog box.

        """
        lgg.info('Launch About dialog')
        from about import About
        about = About(VERSION_STR)

    def show_manual(self):
        """Display th Help Viewer.

        """
        lgg.info('Launch Manual dialog')
        from helpviewer.helpviewer import HelpViewer
        manual = HelpViewer(self)

    def launch_setting_dialog(self):
        """Display the Settings dialog box.

        """
        lgg.info('Launch Settings dialog')
        from settings.settings import Configuration
        self.cfg.save_config()
        config = Configuration(self)

    def launch_search_and_find_dialog(self):
        """Display the Search and Find dialog box.

        """
        lgg.info('Launch Search and Find dialog')
        from sortingtools.imagefinder import ImageFinder
        fld = self.source_fld
        if fld is None:
            return

        if fld.name == "Selection":
            fld = self.pictures_fld

        self.selection = None

        self.imgf = ImageFinder(self, fld)
        if self.selection is not None:
            self.state.close_source_folder()
            self.trailer.clean_view()
            self.source_fld = SelectionList(self, self.selection)
            self.ui.set_window_title("Selection")
            self.start = 0
            self.thumbs = []
            self.state.set_current_state(1)
            self.initiate_viewing()
            self.build_trailer()

        del self.selection
        
    def launch_geotagging_dialog(self):
        fld = self.source_fld
        if not fld.path.is_writable:
            warn = WarningMessage(self)
            warn.setup(0, {"context": _("Geolocation"),
                           "msg": _("Folder:"), 
                           "dat": self.folders[0].where,
                           "msg_1": _("is read only!")})
            return

        writables = []
        exts = ['.jpg', '.jpeg', '.JPG', '.JPEG', 
                '.tif', '.tiff', '.TIF', '.TIFF']
        for img in fld.images:
            if os.path.splitext(img)[-1] in exts:
                writables.append(img)

        if not writables:
            warn = WarningMessage(self)
            warn.setup(0, {"context": _("Geolocation"),
                           "msg": _("There's no writable images into "
                                        "the current folder.")})
            return

        from g9n.geolocation import Geolocation
        Geolocation(self, writables, self.workspace['g9n'].path, 
                    self.cfg.get(('g9n', 'default')), 'gmt')


    def toggle_magnifier(self, state=None):
        """Show or hide the magnifying glass tool.

        Args:
        state -- True or False if called by the globalmenu or None if called
                 from the toolbar
        """
        if self.magnifier:
            lgg.info('Close magnifier')
            self.magnifier.close()
            self.magnifier = False

        else:
            lgg.info('Activate magnifier')
            view = self.state.current_viewer
            qimage = view.get_qimage()
            zoom = self.viewing.current_zoom
            self.magnifier = Magnifier(self, view, qimage, zoom)
            self.magnifier.pixmapitem.setFocus(True)

        # QToolButton.setChecked require a boolean
        self.globalmenu.widget.magnifier.setChecked(self.magnifier and 1)

    def set_focus_on_magnifier(self):
        if self.magnifier:
            self.magnifier.pixmapitem.setFocus(True)

    def remove_magnifier(self):
        """Close the magnifier.

        """
        if self.magnifier:
            if self.magnifier.pixmapitem:
                self.toggle_magnifier()

            else:
                # Called by the magnifier himself
                self.magnifier = False
                self.globalmenu.widget.magnifier.setChecked(False)

    def _escape(self):
        lgg.info("Escape triggered state: %s" % self.state.context)
        if self.state.context == 3:
            self.state.exit_fullscreen()

    def rotate_to_right(self):
        """Rotate the current image to the right.

        """
        lgg.info('Rotate to right')
        context = self.state.context
        if context in (1, 2, 3):
            self.viewing.rotate_to_right()
            self.on_image_rotated(90)

        elif context == 5:
            self.rawviewer.rotate_image(90)

    def previous_image(self):
        """Display the previous image.

        """
        lgg.info('Back to previous image')
        context = self.state.context
        if context in (1, 2, 3):
            self.viewing.previous_view()

        elif context == 5:
            self.rawviewer.previous_image()

        elif context == 6:
            self.multiviewing.move_to_left()

    def next_image(self):
        """Display the next image.

        """
        lgg.info('Display next image')
        context = self.state.context
        if context in (1, 2, 3):
            self.viewing.next_view()
            #self.set_mem_stat()

        elif context == 5:
            self.rawviewer.next_image()

        elif context == 6:
            self.multiviewing.move_to_right()

    def rotate_to_left(self):
        """Rotate the current image to the left.

        """
        lgg.info('Rotate to left')
        context = self.state.context
        if context in (1, 2, 3):
            self.viewing.rotate_to_left()
            self.on_image_rotated(-90)

        elif context == 5:
            self.rawviewer.rotate_image(-90)

    def get_zoom_area(self):
        """Activate the zoom on area draw with the mouse.

        """
        lgg.info('Zoom on area')
        context = self.state.context
        if context in (1, 2, 3):
            self.viewing.zoom_on_area()

        elif context == 5:
            self.rawviewer.zoom_on_area()

    def on_zoom_full_size(self):
        lgg.info('Zoom full size')
        context = self.state.context
        if context in (1, 2, 3):
            self.viewing.set_full_size()

        elif context == 5:
            self.rawviewer.set_full_size()

    def on_zoom_up(self):
        lgg.info('Zoom up')
        context = self.state.context
        if context in (1, 2, 3):
            self.viewing.zoom_up()

        elif context == 5:
            self.rawviewer.zoom_up()

    def on_zoom_edited(self):
        value = self.ui.toolbar.get_zoom_value()
        lgg.info('Zoom at {} %'.format(value))
        context = self.state.context
        if context in (1, 2):
            self.viewing.zoom_at_value(value)

        elif context == 5:
            self.rawviewer.zoom_at_value(value)

    def on_zoom_down(self):
        lgg.info('Zoom down')
        context = self.state.context
        if context in (1, 2, 3):
            self.viewing.zoom_down()

        elif context == 5:
            self.rawviewer.zoom_down()

    def on_fit_in_view(self):
        lgg.info('Fit in view')
        context = self.state.context
        if context in (1, 2, 3):
            self.viewing.fit_in_view()

        elif context == 5:
            self.rawviewer.fit_in_view()

    def zoom_on_cursor_pos(self):
        if self.state.context == 5:
            self.raw_zoom_on_cursor_pos()
            return

        fact = self.viewing.current_zoom
        pos = self.clic_image_pos
        lgg.info('Zoom on cursor pos %s x %s' %(pos.x(), pos.y()))
        self.viewing.zoom_up()
        self.state.current_viewer.centerOn(pos.x(), pos.y())

    def raw_zoom_on_cursor_pos(self):
        fact = self.rawviewer.viewer.current_zoom
        pos = self.clic_image_pos
        lgg.info('Zoom on cursor pos %s x %s' %(pos.x(), pos.y()))
        self.rawviewer.zoom_up()
        self.rawviewer.viewer.centerOn(pos.x(), pos.y())

    def on_zoom_factor_changed(self, factor):
        if self.state.context == 5:
            self.raw_zoom_factor_changed(factor)
            return

        lgg.info('Zoom factor: %s' % factor)
        self.viewing.current_zoom = factor
        self.ui.toolbar.set_zoom_value(factor*100)
        self.update_navigation()
        self.ui.stackedviews.update_dualview_zoom(factor)
        if self.crop:
            view = self.state.current_viewer
            qpixmap = view.get_qpixmap()
            self.crop.resize_by_zooming(view.scene(), qpixmap, factor)

        elif self.filtering.is_previewed:
            self.filtering.redraw_preview()

    def raw_zoom_factor_changed(self, factor):
        lgg.info('Zoom factor: %s' % factor)
        self.rawviewer.viewer.current_zoom = factor
        self.ui.toolbar.set_zoom_value(factor*100)
        self.update_navigation()
        self.ui.stackedviews.update_dualview_zoom(factor)

    def on_viewer_size_changed(self):
        lgg.info('Viewer size changed')

    def on_image_rotated(self, angle):
        """Update some widget after the image rotation.

        Args:
        angle -- the angle of rotation
        """
        self.current_picture.set_rotation(angle)
        self.update_navigation()
        val = (self.current_picture.rotation != 0)
        self.ui.stackedviews.update_file_name(val)
        qimage = self.state.current_viewer.get_qimage()
        self.update_navigation_viewer(qimage)
        self.ui.toolpanel.resizing.change_size(qimage.width(), qimage.height())

    def on_viewer_context_menu(self, pos, scene_pos):
        """Called by contextMenuEvent on the current image.

        Args:
        pos -- Coordinate of the click in global pos
        scene_pos -- Coordinate of the click in scene pos
        """
        # Keep the clic position for the option `Zoom here`
        self.clic_image_pos = scene_pos
        lgg.info('Viewer context menu event at: %s' % pos)
        self.set_scene_context_menu(pos)

    def set_viewing_state(self, menu=False):
        """Change to the mode Viewing.

        Args:
        menu -- True if called by the menu View
        """
        if self.multiviewing:
            self.close_multiviewing(False)
        self.state.set_current_state(1)
        self.viewing.load_picture()

    def set_sorting_state(self, menu=False):
        """Change to the mode Sorting.

        Args:
        menu -- True if called by the menu Sorting
        """
        self.state.set_current_state(2)
        self.viewing.load_picture()

    def set_multi_viewing(self, menu=False):
        """Change to the mode Multi views.

        Args:
        menu -- True if called by the menu Multi Views
        """
        context = self.state.context
        if context in (1, 2):
            fld = self.source_fld.images
            raw = False

        elif context == 5:
            fld = self.rawviewer.raw_previews
            raw = True

        else:
            return

        self.old_context = context
        self.state.set_current_state(6)
        QCoreApplication.processEvents()
        conf = {"config": self.cfg.get(("multiviews",))}
        conf["images"] = fld
        conf["read_only"] = not self.source_fld.path.is_writable
        conf["idx"] = self.viewing.counter
        conf["graphics"] = self.state.current_viewer
        conf["trailer"] = self.trailer
        conf["raw_views"] = raw
        self.multiviewing = MultiViews(self, conf)
        self.multiviewing.build_views()
        if menu:
            self.update_multiviews_navigation(conf["idx"], len(fld))
            self.ui.toolbar.widget.multiviews_state_btn.update_state()

    def configure_multiviews(self, arg):
        # called by the spinBox of the headerBar
        if self.multiviewing:
            self.multiviewing.reset_layout()
            self.cfg.update(("multiviews",), self.multiviewing.config)

    def unselect_multiviews(self):
        if self.multiviewing:
            self.multiviewing.clear_selection()

    def update_multiviews_navigation(self, index, total):
        self.ui.toolbar.enable_multiviews_navigation(index, total)

    def close_multiviewing(self, reset=True, update=False):
        pict, idx = self.multiviewing.get_current_picture()
        lgg.info("Close multiviewing at index: %s" % idx)
        self.state.close_multiviewing()
        if update:
            self.ui.toolbar.widget.multiviews_state_btn.update_state()

        if reset: 
            self.state.set_current_state(self.old_context)
            if self.state.context == 5:
                self.rawviewer.set_current_image(idx)

            else:
                self.viewing.counter = idx
                self.viewing.load_picture()

        self.multiviewing = False

    def display_trash_tab(self):
        self.set_sorting_state()
        self.ui.stackedviews.sorting_tab.show_trash_tab()

    def activate_crop_tool(self):
        if self.crop:
            return

        lgg.info('Begin crop tool')
        viewer = self.state.current_viewer
        viewer.set_mask(False)
        self.crop = CropTool(self, viewer)
        #self.state.set_crop_tool_active(True)
        viewer.crop = self.crop
        self.crop_flag = True
        viewer.setDragMode(QGraphicsView.RubberBandDrag)
        viewer.setCursor(Qt.CrossCursor)

    def abort_crop(self):
        lgg.info('Abort crop tool')
        self.viewing.viewer.exit_tool_mode()
        #self.state.set_crop_tool_active(False)
        self.crop.remove_crop_area()
        self.crop = False
        self.state.request_scrollBar(self.cfg.get(('style', 'scrlbars')))

    def set_straighten_mode(self, tool, orientation):
        self.set_filtered_mode(True)
        self.filtering.set_freehand_rotation_mode(tool, orientation)
        self.ui.stackedviews.viewing_header.enable_layout_buttons(False)
        self.state.freeze_tool_boxes(tool, True)

    def set_grid(self, state=None):
        if self.grid is not None:
            return

        conf = self.cfg.get(('grid',))
        self.grid = Grid(self, self.state.current_viewer, conf)
        self.grid.initialize()

    def show_grid(self):
        if self.grid is None:
            self.set_grid()

        # Update the grid before show it
        self.grid.apply_ratio(True)
        self.grid.show_grid(True)
        self.globalmenu.widget.grid.setChecked(True)

    def hide_grid(self):
        if self.grid is None:
            return

        self.grid.show_grid(False)
        self.globalmenu.widget.grid.setChecked(False)
        self.grid = None

    def grid_is_visible(self):
        if self.grid is None:
            return False

        return self.grid.is_visible

    def toggle_grid(self, b=True):
        if self.grid_is_visible():
            self.hide_grid()

        else:
            self.show_grid()

    def set_filtered_mode(self, mode):
        """Set the filtered viewing mode.

        Args:
        mode -- boolean, if True the filtered mode is actived
        """
        if self.filtered_mode == mode:
            return

        if mode:
            pict = self.current_picture
            viewer = self.state.current_viewer
            self.filtering.set_image(pict, viewer)
            self.ui.stackedviews.viewing_header.set_quick_save_enabled(False)

        self.state.set_filtering_state(mode)
        self.filtered_mode = mode

    def set_filter_previewing_mode(self, mode):
        """Set the filter previewing mode.

        Args:
        mode -- boolean, if True the filtered mode is actived
        """
        if mode:
            pict = self.current_picture
            viewer = self.state.current_viewer
            self.filtering.set_image(pict, viewer)
            self.ui.stackedviews.viewing_header.set_quick_save_enabled(False)

    def filtering_finished(self, state, tool, backup):
        """Called whenever a transformation is finished.

        Args:
        state -- boolean if False, the transformation has failed
        tool -- the toolbox instance of the filter
        backup -- instance of Backup
        """
        if state:
            self.ui.stackedviews.update_file_name(True)
            pict = self.filtering.origin
            self.ui.stackedviews.set_dual_view(True, pict, backup)
            qimage = self.viewing.viewer.get_qimage()
            self.ui.toolpanel.export.set_data(pict, qimage, backup)
            dest = self.ui.toolpanel.export.name_led.text() + pict.ext
            self.ui.toolpanel.export.setEnabled(True)
            self.ui.toolpanel.export.on_extension_changed(pict.ext, False)
            self.ui.stackedviews.update_target_name(dest)
            self.current_picture = backup
            self.viewing.viewer.exit_tool_mode()
            self.state.freeze_tool_boxes(None, False)

    def undo_filtering(self, last):
        self.current_picture = last
        self.viewing.viewer.set_source_image(last)
        self.viewing.viewer.set_qimage_from_source()
        self.viewing.viewer.refresh_qpixmap()
        self.update_navigation_viewer(self.viewing.viewer.get_qimage())
        self.ui.stackedviews.dual_view.undo_backup()

    def set_preview_window(self, tool):
        if self.state.filter_preview:
            self.close_preview_window()

        lgg.info('Display filter preview')
        vals = tool.get_values()
        config = {'name': tool.i18n_name, 'params': vals}
        size = self.cfg.get(('filtering', 'preview', 'size'))
        viewer = self.state.current_viewer
        qimage = viewer.get_qimage()
        zoom = self.viewing.current_zoom
        self.state.filter_preview = PreviewHandler(self, size, viewer, 
                                                   qimage, zoom, tool)
        self.state.filter_preview.config_filter(config)
        self.state.filter_preview.update()

    def close_preview_window(self):
        if self.state.filter_preview:
            try:
                self.state.filter_preview.close()
                lgg.info('Filter preview closed')
            except RuntimeError:
                # The proxy widget has been deleted (i.e. zoom action)
                pass

        self.state.filter_preview = False

    def hide_proxy_tool(self):
        lgg.info('Hide proxy tool')
        self.hidden_tool = False
        if self.state.filter_preview:
            self.hidden_tool = self.state.filter_preview.toolbox
            self.close_preview_window()

    def reload_proxy_tool(self):
        lgg.info('Reload proxy tool')
        if self.hidden_tool:
            if self.hidden_tool.name == 'ContrastBrightnessBox':
                self.set_preview_window(self.hidden_tool)

            self.hidden_tool = False
        
    def display_histogram(self, fname):
        if not fname:
            self.ui.toolpanel.histogram.set_no_histogram()
            return

        name = os.path.basename(fname)
        lgg.info('Histogram created: %s' % name)
        self.ui.toolpanel.histogram.show_histo(fname)

    def on_trashcan_clicked(self, fname=None):
        lgg.info('Delete current image')
        if fname is None:
            if self.state.context in(1, 2):
                img = self.current_picture
                self.shredder.add_file_from_source(img)
                self.remove_image(img)

            elif self.state.context == 6:
                for idx in self.multiviewing.selected:
                    name = self.source_fld.images[idx]
                    #idx = self.source_fld.images.index(i)
                    pict = Picture(self, name, idx, False)
                    self.shredder.add_file_from_source(pict)
                    self.remove_image(pict)

                self.multiviewing.remove_selection()

    def display_global_menu(self):
        lgg.info('Call global menu')
        g = self.ui.toolbar.widget.geometry()
        pos = self.ui.toolbar.widget.mapToGlobal(QPoint(g.x()+g.width(), 0))
        self.globalmenu.display_menu(pos.x(), pos.y(), self.state.context)

    def display_aerial_view(self, state=None):
        """Display or hide the aerial view.

        Args:
        state -- the state of the button on the global menu
        """
        if not self.state.context:
            return

        if self.aerial_view:
            lgg.info('Hide aerial view')
            self.aerial_view.setParent(None)
            self.aerial_view.deleteLater()
            self.aerial_view = False

        else:
            viewer = self.state.current_viewer
            qimage = viewer.get_qimage()
            size = RectSize(qimage.size())
            self.aerial_view = AerialView(self, viewer)
            self.aerial_view.show()
            QCoreApplication.processEvents()
            name = self.current_picture.base
            self.aerial_view.set_image(viewer, qimage, name, size)

        self.globalmenu.widget.aerial.setChecked(self.aerial_view and 1)

    def set_raw_files_viewing(self, from_menu=False):
        if not self.raw_folder_empty():
            lgg.info('Set RAW files viewing')
            self.state.set_current_state(5)
            if from_menu:
                self.ui.statusbar.set_current_mode(4)

            idx = self.rawviewer.current_index
            fls = self.rawviewer.count()
            self.ui.statusbar.set_image_index(idx+1, fls)
            if fls:
                self.rawviewer.set_current_image(idx)
            self.ui.toolbar.set_trailer_slider_max(fls-1)

    def raw_folder_empty(self):
        if not self.rawviewer.raw_previews:
            if not self.source_fld.raws:
                return True

        return False

    #------------------------------------------------------------------------

    def browse_to_source_folder(self):
        """Browse to a new source folder in the property panel.
        
        """
        rep = self.get_dir(True)
        QCoreApplication.processEvents()
        if not rep:
            return

        if self.is_source_folder(rep):
            # Folder already open
            return

        #self.update_source_tool(rep)
        return rep

    def open_source_folder(self, path, force=False):
        """Open the source folder.

        Args:
        path -- the full path of the folder
        force -- if True the same source can be reloaded
        """
        if not force and self.is_source_folder(path):
            # Folder already open
            return

        if not isinstance(path, SourceFolder):
            self.set_source_folder(path)

        state = 0
        # Check if path was not rejected
        if self.source_fld is not None:
            self.utils.clean_all_temp_folders()
            if not self.trailer.is_complete:
                self.trailer.stop_request = True

            if not self.on_startup:
                if self.state.context:
                    self.state.close_source_folder()
                    self.trailer.clean_view()

            self.source_fld.build_image_list()
            if self.source_fld.images:
                state = 1
                self.state.set_current_state(1, True)
                self.source_fld.set_subfolders()
                self.ui.set_source_folder_name(self.source_fld)
                self.initiate_viewing()
                self.build_trailer()

            if self.source_fld.raws:
                self.source_fld.set_subfolders()
                self.prepare_raw()
                if not state:
                    state = 1
                    self.state.set_current_state(5, True)
                    self.ui.set_source_folder_name(self.source_fld)
                    self.initiate_viewing(True)
                    self.build_trailer()
                    self.state.set_raw_only(True)

            if not state:
                self.state.set_current_state(0)

            self.on_startup = False
            if isinstance(path, SourceFolder):
                self.cfg.set_source_folder(path.path)

            else:
                self.cfg.set_source_folder(path)

    def is_source_folder(self, path):
        """Check if a path is the path of the source folder.

        Args:
        path -- the path of the folder
        """
        if self.source_fld is not None:
            return os.path.samefile(self.source_fld.where, path)

        return False

    def set_raw_files_handling(self, state):
        ignore = True
        self.ui.statusbar.enable_raw_viewing(False)
        self.globalmenu.accept_raw_files = False
        if not state:
            ignore = False
            lgg.info('Handle RAW files: True')
            if self.source_fld is not None:
                self.source_fld.accept_raw_files(True)

        self.cfg.update(('rawfiles', 'ignore'), ignore)
        self.prepare_raw()

    def prepare_raw(self):
        if self.cfg.get(('rawfiles', 'ignore')):
            self.ui.statusbar.enable_raw_viewing(False)
            self.globalmenu.accept_raw_files = False
            return

        if self.source_fld.raws:
            self.rawviewer.set_raw_files(self.source_fld)
            self.ui.statusbar.enable_raw_viewing(True)

    def build_trailer(self):
        self.trailer.stop_request = False
        self.trailer.load_images()

    def initiate_viewing(self, raw_only=False):
        lgg.info('Initiate viewing')
        viewer = self.state.current_viewer
        self.viewing.set_viewer(viewer)
        if raw_only:
            self.set_raw_files_viewing()
            return

        else:
            self.viewing.images = self.source_fld.images

        self.viewing.first = self.first_image
        self.ui.statusbar.enable_index(True)
        self.ui.statusbar.set_image_index(self.first_image+1,
                                          self.viewing.count())

        self.viewing.run_viewing()

    def set_current_picture(self, path, index):
        lgg.info('Set current picture: %s' % index)
        self.current_index = index
        self.current_picture = Picture(self, path, index)
        self.trailer.set_current_item(index)
        self.ui.statusbar.set_image_index(index+1, self.viewing.count())
        self.metadataeditor.set_data(self.current_picture.metadata)
        self.ui.toolpanel.update_with_picture(self.current_picture)
        self.ui.stackedviews.change_file_name(self.current_picture.base)
        self.filtering.set_original_picture(self.current_picture)

    def set_mem_stat(self):
        # Debugging function
        snapshot = tracemalloc.take_snapshot() 
        top_stats = snapshot.statistics('lineno') 

        print("----[ Top 10 ]-----------------------------------") 
        for stat in top_stats[:10]: 
            print(stat)
        print("-------------------------------------------------")

    def reject_current_image(self):
        """Remove an unreadable image file from the list.

        """
        lgg.info('Remove unreadable file: %s' 
                 % self.source_fld.images[self.current_index])
        try:
            self.source_fld.images.pop(self.current_index)
        except IndexError:
            pass

        if not self.source_fld.images:
            self.state.set_current_state(0)

        else:
            self.viewing.load_picture()

    def remove_image(self, image):
        """Remove an image file from the list.

        Args:
        image -- Picture instance or image file path
        """
        if isinstance(image, Picture):
            image = image.uri
        lgg.info('Remove file: %s' % image)
        idx = self.source_fld.images.index(image)
        is_current = idx == self.current_index
        try:
            self.source_fld.images.pop(idx)
        except IndexError:
            pass

        self.trailer.remove_at_index(idx)

        if not self.source_fld.images:
            self.state.set_current_state(0)

        elif is_current:
            if idx == self.source_fld.images_count():
                self.viewing.previous_view()

            else:
                self.viewing.load_picture()

        elif idx < self.current_index:
            self.current_index -= 1

    def update_navigation(self):
        """Update the navigation tool box. Called by the graphics area.

        """
        if not self.state.current_viewer.is_auto_scroll:
            self.ui.toolpanel.navigation.update_()
            if self.aerial_view:
                self.aerial_view.update_()

    def center_views(self, x, y):
        viewer = self.state.current_viewer
        viewer.centerOn(x, y)
        self.reset_aerial_view()

    def update_navigation_viewer(self, qimage):
        """Update the navigation tool box. 

        This method is called when the current image has changed.

        args:
        qimage -- the QImage instance
        """
        # Viewing area may be changed
        #if self.state.context == 6:
            #viewer = self.state.viewers.viewing_area

        viewer = self.state.current_viewer
        self.ui.toolpanel.navigation.set_image(viewer, qimage)
        if self.aerial_view:
            name = self.current_picture.base
            size = RectSize(qimage.size())
            self.aerial_view.set_image(viewer, qimage, name, size)

        if self.alt_screen:
            self.alt_screen.set_image(qimage, self.current_picture)
            QCoreApplication.processEvents()
            # Reset the focus on the main window, stolen by the second screen
            self.ui.activateWindow()

    def reset_navigation(self):
        """Update the frame position into the navigation toolbox.

        Called by the aerial view.
        """
        self.ui.toolpanel.navigation.update_()

    def reset_aerial_view(self):
        """Update the frame position into the aerial view.

        Called by the navigation toolbox.
        """
        if self.aerial_view:
            self.aerial_view.update_()

    #------------------------------------------------------------------------
    # Copy and export image
    #------------------------------------------------------------------------
    def get_temp_path(self, fld):
        """Return a temp file name builded with time.

        Args:
        fld -- the folder path
        """
        d = str(time.time()).replace(".", "")
        t = "".join(["temp", d])
        count = 1
        while 1:
            target = os.path.join(fld, t)
            if os.path.isfile(target):
                t = "".join(["temp", d, str(count)])
                count += 1

            else:
                break

        return target

    def save_current_image(self, fname, img=None):
        if img is None:
            img = self.state.current_viewer.get_qimage()

        ioimage = IOImage()
        ioimage.set_image_to_writer(img)
        ioimage.set_filename_to_writer(fname)
        save = ioimage.write_image()
        if not save[0]:
            err = ' '.join(str(save[1]), str(save[2]))
            lgg.warning("Save change error: %s" % err)
            message = {'context': _('Save current image'),
                'warn': _("Unable to write the image file:"),
                'data': fname,
                'msg': _('Reason:'),
                'data1': err,
                'btn': _('Close')}
            warn = WarningMessages(0, message)
            warn.build_dialog()
            return False

        lgg.info("Current saved")
        return True

    def create_thumbnail(self, image, folder=None):
        if folder is None:
            folder = self.workspace["temp"].path

        temp = self.get_temp_path(folder) + '.jpg'
        thumb = Thumbnailer(image, temp)
        thumb.draw_and_save()
        return temp

    def change_orientation(self, fname, size, thumbnail, data=None):
        if data is None:
            data = Metadata(fname)
            if not data.get_data():
                return

        data.update_orientation(1, size.width(), size.height())
        data.modify_thumbnail(thumbnail)
        data.save_data()

    def rename_file(self, old, new, remove=False):
        if remove:
            try:
                os.unlink(new)
            except Exception as why:
                lgg.warning('Remove file error: %s' % why)

        try:
            os.move(old, new)
        except Exception as why:
            lgg.warning('Move file error: %s' % why)

    def quick_save(self):
        """Save the current image into the same folder with the same name.

        """
        lgg.info("quick_save")
        pict = self.current_picture
        target = pict.uri
        qimage = self.state.current_viewer.get_qimage()
        size = qimage.size()
        tmp = self.get_temp_path(pict.folder) + pict.ext
        if not self.save_current_image(tmp, qimage):
            return

        thumb = self.create_thumbnail(qimage)

        if pict.metadata is not None:
            pict.metadata.export_data(tmp)
            self.change_orientation(tmp, size, thumb)
            data = pict.metadata.get_data_for_thumbnail()
            md = Metadata(thumb)
            if md.get_data():
                md.write_thumbnail_data(data)

        if self.copy_file(tmp, target):
            fld = self.source_fld.thumbs_fld.path
            tsrc = self.utils.get_thumb_name(target, fld)
            self.copy_file(thumb, tsrc)
            self.trailer.reload_current(target, tsrc)

        self.ui.stackedviews.update_file_name(False)

    def multi_image_to_target(self, target, action):
        dest = self.targets[target].path.path
        tab = self.ui.stackedviews.sorting_tab.currentIndex()
        if not tab:
            selection = self.multiviewing.get_images_selected()
            for idx, image, name in selection:
                print(image)
                fname = os.path.join(dest, name)
                pict = Picture(self, image, idx, False)
                if not self.check_copy_permission(fname, pict):
                    continue

                self.copy_from_viewer_to_target(pict, target, action)

            if action == 4:
                self.multiviewing.remove_selection()
        

    def send_to_target(self, target, action):
        """Send an image file to a target folder.

        Called by the target buttons.

        args:
        target -- target id (0~23)
        action -- mouse button
                    1: copy
                    4: copy and delete source
        """
        dest = self.targets[target].path.path
        if self.state.context == 2:
            # Sorting mode
            tab = self.ui.stackedviews.sorting_tab.currentIndex()
            if not tab:
                # Copy from viewer to target
                image = self.current_picture
                fname = os.path.join(dest, image.base)
                if not self.check_copy_permission(fname, image):
                    return

                if self.current_image_changed():
                    if isinstance(image, Picture):
                        self.save_to_target(target, fname)

                else:
                    self.copy_from_viewer_to_target(image, target, action)

            else:
                # Copy from target to target
                self.copy_from_target_to_target(self.targets[tab], 
                                                target, action)
                self.targets[tab].update_free_space()

        self.targets[target].update_free_space()

    def copy_from_viewer_to_target(self, picture, target, action=1):
        """Copy a file from the viewer to a target folder.

        Args:
        picture -- the Picture instance
        target -- target id (0~23)
        action -- int() if the value is 4 then the original image file will
                  be removed
        """
        lgg.info('copy %s to %s' % (picture.uri, target))
        dest = self.targets[target].path.path
        src = picture.uri
        fld = self.source_fld.thumbs_fld.path
        thumb = self.utils.get_thumb_name(picture.uri, fld)
        fname = os.path.join(dest, picture.base)
        thumb2 = self.utils.get_thumb_name(fname)
        if self.copy_file(src, fname):
            self.copy_file(thumb, thumb2)
            self.add_picture_to_target(picture, thumb2, fname, target)
            if action == 4:
                self.delete_image_file(picture, thumb)

    def save_to_target(self, target, pict):
        if isinstance(target, int):
            fld = self.targets[target].path.path

        else:
            fld = os.path.dirname(target)

        pict = self.current_picture
        qimage = self.state.current_viewer.get_qimage()
        size = qimage.size()
        if not self.save_current_image(target, qimage):
            return

        thumb = self.create_thumbnail(qimage, fld)

        if pict.metadata is not None:
            pict.metadata.export_data(target)
            self.change_orientation(target, size, thumb)
            data = pict.metadata.get_data_for_thumbnail()
            md = Metadata(thumb)
            if md.get_data():
                md.write_thumbnail_data(data)
            tsrc = self.utils.get_thumb_name(target)
            self.rename_file(thumb, tsrc, True)

    def save_from_export(self, obj=None):
        """Save the current image.

        Called from the export tool box or by the quick save button.

        Args:
        obj -- Dict {'orig': Picture instance of the original image
                     'backup': Backup instance
                     'dest': destination folder
                     'name': image name
                     'format': file extension
                     'quality': compression quality
                     'data': copy metadata (boolean)
                     'qimage': instance of the current QImage}
               obj is None if called by the button 'quick save'
        """
        if obj is None:
            obj = self.ui.toolpanel.export.get_export_config()

        lgg.info("Export image {0}".format(obj['name'] + obj['format']))
        if self.state.context == 5:
            self.rawviewer.export_current_image(obj)
            return

        pict = obj['orig']
        src = pict.uri
        lgg.info("Save %s to %s" %(src, obj['dest']))
        dest = os.path.join(obj['dest'], obj['name'] + obj['format'])
        thumb = self.utils.get_thumb_name(dest)
        exist = False
        if not self.check_copy_permission(dest, obj['orig']):
            # Cancelled by the user
            self.ui.toolpanel.export.set_progress_bar(False)
            return

        if obj['backup'] is None:
            if pict.rotation or obj['format'].lower() != pict.ext.lower():
                # Image rotated or image format changed
                self.save_to_target(dest, pict)

            else:
                if self.copy_file(src, dest):
                    fld = self.source_fld.thumbs_fld.path
                    tsrc = self.utils.get_thumb_name(src, fld)
                    self.copy_file(tsrc, thumb)

        else:
            obj['backup'].export(dest, thumb, obj)

        if self.is_source_folder(obj['dest']):
            # The folder is the source folder, the new image must be inserted
            #if dest in self.source_fld.images:
                #self.redraw_thumbnail(obj['orig'], obj['qimage'], True)
            self.state.set_filtering_state(False)
            self.insert_image(dest, thumb, reload_=True)

        else:
            idx = self.get_target_index(obj['dest'])
            if idx != -1:
                self.add_picture_to_target(pict, thumb, dest, idx)
            
        self.ui.toolpanel.export.set_progress_bar(False)
        self.ui.stackedviews.update_file_name(False)
        self.filtered_mode = False

    def copy_from_target_to_target(self, source, target, remove):
        """Copy a file from a target folder to an other target.

        Args:
        source -- index of the origin folder
        target -- index of the destination folder
        remove -- int() indicate if the file must be removed from the original 
                  folder
        """
        dest = self.targets[target].path.path
        items = self.define_selected_files()
        copied = []
        for item in items:
            data = dict(item.data)
            src = data['fpath']
            thumb = data['thumb']
            copy = os.path.join(dest, os.path.basename(src))
            thumb2 = self.utils.get_thumb_name(copy)
            if self.copy_file(src, copy):
                copied.append(item)
                self.copy_file(thumb, thumb2)
                data['fpath'] = copy
                data['thumb'] = thumb2
                self.targets[target].files.append(item.data)
                self.targets[target].tab.add_item(item.data)

        self.targets[target].update_free_space()
        if remove == 4:
            # Only the files copied will be removed
            self.delete_files_from_target(source, copied)

    def define_selected_files(self):
        """Returns the list of the selected files into the current target tab.

        """
        listwidget = self.ui.get_current_list_widget()
        return listwidget.selection

    def current_image_changed(self):
        if self.current_picture.rotation or self.filtered_mode:
            return True

    def get_dest_filename(self, fname, fld):
        """Build a destination filename.

        Args:
        fname -- basename of original file
        fld -- destination folder path
        """
        path = os.path.join(fld, fname)
        return path

    def get_compression(self, img):
        """Return the value of compression.

        Args:
        img -- instance of Picture
        """
        if img.ext.lower() in ['.tif', '.tiff']:
            return 'lzw'

    def export_current_image(self, dest):
        lgg.info('Save current image')
        qimage = self.state.current_viewer.get_qimage()
        qimage.save(dest)

    def copy_file(self, src, dest):
        """Copy one file in target folder.

        Args :
        src -- source name (full path)
        dest -- new name (full path)

        Returns:
        destination file path, destination file thumbnail
        """
        try:
            shutil.copyfile(src, dest)
            return True
        except EnvironmentError as why:
            lgg.warning("Copy file error: %s" % why)
            self.on_save_error(src, dest, why)
            return False

    def check_copy_permission(self, path, src):
        """Return True if the file can be copied into a folder.

        If a file with same name already exists into the target folder, the
        warning dialog box will be called.

        Args:
        path -- destination file name
        src -- instance of Picture
        """
        if not os.path.isfile(path):
            return True

        return self.display_copy_warning(path, src)
        
    def on_save_error(self, fname, to, reason):
        lgg.warning('Unable to copy %s\n\t\tto %s\n\t\tReason: %s'
                     %(fname, to, reason))
        message = {"context": _("Save image file:"),
                    "object": os.path.basename(fname),
                    "warn": _("Unable to copy the file in"),
                    "data": os.path.dirname(to),
                    "msg": _("Reason:"),
                    "data1": reason,
                    "size": QSize(280, 150)}
        warn = WarningMessages(0, message)
        warn.build_dialog()

    def add_picture_to_target(self, image, thumbnail, dest, idx):
        """Add an image file into the list of a target.

        Args:
        image -- instance of Picture
        thumbnail -- path of the thumbnail
        dest -- path of copied file
        idx -- index of the target
        """
        target = self.targets[idx]
        self.ui.statusbar.display_message(_(" {0} saved to {1}".format(
                                            image.base, target.name)), 2000)
        data = self.get_data_from_picture(image, thumbnail, dest)
        target.files.append(data)
        target.tab.add_item(data)
        target.get_free_space()
        target.button.sum += target.used

    def delete_image_file(self, picture, thumbnail):
        """Unlink an image file.

        Args:
        picture -- file path or Picture instance
        thumbnail -- file path
        """
        tab = self.ui.stackedviews.sorting_tab.currentIndex()
        path = picture
        if isinstance(picture, Picture):
            path = picture.uri

        try:
            os.unlink(path)
        except EnvironmentError as why:
            lgg.warning("Deleting file error: %s" % why)

        try:
            os.unlink(thumbnail)
        except EnvironmentError as why:
            lgg.warning("Deleting file error: %s" % why)

        if not tab:
            self.remove_image(picture)

    def delete_files_from_target(self, from_, files):
        listwidget = self.ui.get_current_list_widget()
        for item in files:
            try:
                os.unlink(item.data['fpath'])
                os.unlink(item.data['thumb'])
                listwidget.remove_item(item)
                from_.remove(item.data)
            except Exception as why:
                lgg.warning("Deleting file error: %s" % why)

    def insert_image(self, image, thumbnail, reload_=False):
        """Insert a new image in source folder.

        When an image is restored from the trash can or a new image is created 
        by a transformation of an original file and saved with a new name, 
        the new file must be inserted at the right place.

        Args:
        image -- the full path of the image
        thumbnail -- the full path of the thumbnail
        """
        lgg.info('Insert file: .../%s' % os.path.basename(image))
        idx = self.source_fld.insert_in_right_place(image)
        self.trailer.insert_thumbnail(image, idx, thumbnail)
        if len(self.source_fld.images) == 1:
            self.state.set_raw_only(False)
            self.viewing.images = self.source_fld.images

        if reload_:
            self.viewing.counter = idx
            self.viewing.load_picture()

        elif self.current_index > idx:
            self.current_index += 1
            self.viewing.counter += 1

        if self.state.context == 1:
            self.ui.statusbar.set_image_index(self.current_index,
                                          self.viewing.count())

    def redraw_thumbnail(self, picture, qimage, trailer=False):
        """Redraw an image thumbnail.

        Called when an image has been transformed.

        Args:
        picture -- the Picture instance
        qimage -- the QImage instance
        trailer -- if True, the current item in the trailer will be replaced
                   by the new thumbnail
        """
        tname = self.utils.get_thumb_name(picture.uri, 
                                          self.source_fld.thumbs_fld.path)
        self.trailer.create_thumbnail(picture.uri, tname)
        self.trailer.reload_current(picture.uri, tname)

    def get_data_from_picture(self, pict, tname, fname):
        """Return the essential data of an image for the target folder.

        Args:
        pict -- instance of Picture
        tname -- full path of the thumbnail
        fname -- path of copied file
        """
        data = {'date': None, 'speed': None, 'fnumber': None, 'iso': None}
        data['fpath'] = fname
        data['orig'] = pict.uri
        data['added'] = datetime.datetime.now().strftime('%a %d %b %Y, %H:%M')
        data['thumb'] = tname
        data['width'] = pict.width
        data['height'] = pict.height
        data['weight'] = pict.weight
        data['orient'] = pict.orientation
        if pict.metadata is not None:
            data['date'] = pict.metadata.get_date_and_id()[0]
            values = pict.metadata.get_exposure_infos()
            data['iso'] = values[0]
            data['speed'] = values[1]
            data['fnumber'] = values[2]

        return data

    def display_copy_warning(self, dest, pict):
        """Show copy warning dialog box.

        Args:
        dest -- destination file name
        pict -- instance of Picture
        """
        src = pict.uri
        sthumb = self.utils.get_thumb_name(src, self.source_fld.thumbs_fld.path)
        dthumb = self.utils.get_thumb_name(dest)
        size = "%s x %s pxl" %(pict.width, pict.height)
        dsize = "%s x %s pxl" % self.get_image_size(dest)
        data = {'context': 'Copy of:', 'object': pict.base, 
                'warn': 'A file with same name already exists in:',
                'data': os.path.dirname(dest), 
                'msg': 'Do you want to replace it ?', 
                'img': sthumb,
                'img_data': {'date': None, 'size': size,
                             'iso': None, 'speed': None, 'fnumber': None},
                'img1': dthumb,
                'img1_data': {'date': None, 'size': dsize,
                             'iso': None, 'speed': None, 'fnumber': None},
                'btn': _('Replace'),
                'btn_1': _('Ignore')}

        if pict.metadata is not None:
            data['img_data']['date'] = pict.metadata.get_date_and_id()[0]
            values = pict.metadata.get_exposure_infos()
            data['img_data']['iso'] = values[0]
            data['img_data']['speed'] = values[1]
            data['img_data']['fnumber'] = values[2]

        if pict.ext.lower() in [".jpg", ".jpeg", ".tif", ".tiff"]:
            values = self.get_image_data(dest)
            data['img1_data']['date'] = values['date']
            data['img1_data']['iso'] = values['iso']
            data['img1_data']['speed'] = values['speed']
            data['img1_data']['fnumber'] = values['fnumber']

        warn = WarningMessages(1, data)
        return warn.build_dialog()
        # search a numerical suffix as name_(1).tiff
        suffix = re.compile('(?<=_\()(\d+)(?=\)%s)' % ext)
        match = suffix.search(n)

    def get_image_size(self, path):
        reader = QImageReader(path)
        return reader.size().width(), reader.size().height()

    def get_image_data(self, path):
        metadata = Metadata(path)
        dt = {'date': None, 'iso': None, 'speed': None, 'fnumber': None}
        if metadata.get_data():
            dt['date'] = metadata.get_date_and_id()[0]
            values = metadata.get_exposure_infos()
            dt['iso'] = values[0]
            dt['speed'] = values[1]
            dt['fnumber'] = values[2]

        return dt

    #------------------------------------------------------------------------
    # Dialogs
    #------------------------------------------------------------------------

    def get_dir(self, source=False):
        """Launch QFileDialog to open a directory.

        Args:
        source -- True if directory is source

        Returns:
        directory's path
        """
        if self.source_fld and self.source_fld.path is not None:
            folder = self.source_fld.path.parents

        else:
            folder = self.cfg.get(('folders', 'last_dir'))\
                     or os.path.expanduser('~')

        if source:
            title = _("Open source folder")

        else:
            title = _("Open target folder")

        d = QFileDialog.getExistingDirectory(None, title, folder,
                                            QFileDialog.DontResolveSymlinks
                                            | QFileDialog.ShowDirsOnly)

        if d:
            if not source and not os.access(d, os.W_OK):
                message = {"context": 
                                _("Open destination folder:"),
                                "object": d,
                                "warn": _("This folder is read only"),
                                "size": QSize(280, 100)}
                warn = WarningMessages(0, message)
                warn.build_dialog()
                d = False

            else:
                # Save last directory
                self.cfg.update(('folders', 'last_dir'), os.path.split(d)[0])

        else :
            d = False

        return d

    def set_scene_context_menu(self, pos):
        """Build and show context menu for the current image.

        Args:
        pos -- event.pos() in screen coordinates
        """
        self.menu = ContextMenuScene(self, pos)

    #------------------------------------------------------------------------


class SourceFolder:
    """define the source folder.

    """
    def __init__(self, core, path):
        """Instanciate the source folder.

        Args:
        core -- instance of OqapyCore
        path -- instance of Path
        """
        self.core = core
        self.path = path
        self.where = path.path
        self.name = self.path.name
        self.handle_rawfiles = not core.cfg.get(('rawfiles', 'ignore')) 
        self.images = []
        self.raws = []
        self.thumbs_fld = None
        self.raw_previews_fld = None

    def set_subfolders(self):
        """Create, if not exists, the subfolders `thumbnails` & `raw_previews`.

        """
        if self.path.is_writable:
            self.thumbs_fld = self.path / 'thumbnails'
            # Space needed for the thumbnails: 10 Ko * images
            needed = (len(self.images) + len(self.raws)) * 10240
            stats = disk_usage(self.where)
            if not self.thumbs_fld.exists:
                lgg.info('Source folder free space: %s' % stats.free)
                if stats.free < needed:
                    self.set_temporary_folder('thumbs')

                self.thumbs_fld.ensure_folder_exists()

            if self.raws:
                # Space needed for the raw previews: 1 Mo * raw files
                needed += len(self.raws) * 1048576
                if stats.free < needed:
                    self.set_temporary_folder('rprev')

                else:
                    self.raw_previews_fld = self.path / 'raw_previews'

                self.raw_previews_fld.ensure_folder_exists()
                self.check_old_preview_folder()

        else:
            self.set_temporary_folder('thumbs')
            self.set_temporary_folder('rprev')

    def check_old_preview_folder(self):
        old = self.path / "raws"
        if old.exists and old.is_dir:
            fls = glob.glob(old.path + "/*.*")
            for f in fls:
                b = os.path.basename(f)
                if b.startswith("prev_") and b.endswith(".jpg"):
                    dest = os.path.join(self.raw_previews_fld.path, b)
                    try:
                        os.rename(f, dest)
                    except Exception as why:
                        pass

    def set_temporary_folder(self, name):
        """Set the temporary folder ~.oqapy/name.

        Args:
        name -- the name of the folder
        """
        lgg.info('Using temporary folder %s' % name)
        if name == 'thumbs':
            self.thumbs_fld = self.core.workspace[name]

        else:
            self.raw_previews_fld = self.core.workspace[name]

    def images_count(self):
        return len(self.images)

    def raws_count(self):
        return len(self.raws)

    def build_image_list(self):
        """Build the list of image files of the source folder.

        """
        lgg.info('Get the images list in %s' % self.path.name)
        self.images = []
        self.raws = []
        lst = glob.glob(os.path.join(self.where, '*'))
        if not lst:
            return

        for f in lst:
            if os.path.isfile(f):
                type_ = self.validate_image(f)
                if type_ == 'image':
                    self.images.append(f)

                elif type_ == 'raw':
                    self.raws.append(f)
                
        self.images.sort(key=str.lower)
        self.raws.sort(key=str.lower)
        lgg.info('Found %s images and %s raw files' %(len(self.images),
                                                      len(self.raws)))
        self.set_first_image()

    def set_first_image(self):
        """Convert the first image path by the index of the file.

        """
        if self.core.first_image:
            try:
                self.core.first_image = self.images.index(self.core.first_image)
            except IndexError:
                try:
                    self.core.first_image = self.raws.index(self.core.first_image)
                except IndexError:
                    self.core.first_image = 0

    def accept_raw_files(self, accept):
        self.handle_rawfiles = accept
        if self.raws:
            self.core.ui.statusbar.enable_raw_viewing(accept)
            self.core.globalmenu.accept_raw_files = accept

    def validate_image(self, fname):
        """Validate an image file.

        Args:
        fname -- path of file

        Returns
        str -- 'image' if valid, 'raw' if valid raw file or False
        """
        format = self.core.utils.valid_image(fname)
        if format is None:
            lgg.info('Check for raw: %s' % fname)
            if self.core.utils.valid_raw_image(fname):
                return 'raw'

            return False

        elif format == 'tiff':
            if os.path.splitext(fname)[1].lower() not in ('.tiff', '.tif'):
                # Various RAW files may appears as tiff files
                if self.core.utils.valid_raw_image(fname):
                    return 'raw'

        return 'image'

    def insert_in_right_place(self, path):
        self.images.append(path)
        self.images.sort(key=str.lower)
        return self.images.index(path)
        

class SelectionList:
    """Define the source folder as a list of files selected with the search
    and find tool.

    """
    def __init__(self, core, paths):
        """Instanciate the file selection list.

        Args:
        core -- instance of OqapyCore
        paths -- instances of Path
        """
        self.core = core
        self.path = None
        self.where = None
        self.name = 'Selection'
        self.handle_rawfiles = not core.cfg.get(('rawfiles', 'ignore'))
        self.sort_images(paths)
        self.current = 0
        self.thumbs_fld = core.workspace["thumbs"]
        self.raw_previews_fld = None

    def sort_images(self, images):
        self.images = []
        self.raws = []
        for f in images:
            if os.path.isfile(f):
                type_ = self.validate_image(f)
                if type_ == 'image':
                    self.images.append(f)

                elif type_ == 'raw':
                    self.raws.append(f)
                
        self.images.sort(key=str.lower)
        self.raws.sort(key=str.lower)
        lgg.info('Found %s images and %s raw files' %(len(self.images),
                                                      len(self.raws)))

    def validate_image(self, fname):
        """Validate an image file.

        Args:
        fname -- path of file

        Returns
        str -- 'image' if valid, 'raw' if valid raw file or False
        """
        format = self.core.utils.valid_image(fname)
        if format is None:
            lgg.info('Check for raw: %s' % fname)
            if self.core.utils.valid_raw_image(fname):
                return 'raw'

            return False

        elif format == 'tiff':
            if os.path.splitext(fname)[1].lower() not in ('.tiff', '.tif'):
                # Various RAW files may appears as tiff files
                if self.core.utils.valid_raw_image(fname):
                    return 'raw'

        return 'image'


class Recents:
    """Define the recents folders list.

    """
    def __init__(self, core, max_, paths):
        super().__init__()
        self.core = core
        self.max_folders = max_
        paths = self.clean_paths(paths)
        self.paths_ = [Path(p) for p in paths]

    @property
    def paths(self):
        """Returns the list of recent Path instances.

        """
        return self.paths_

    @property
    def path_strs(self):
        """Returns the list of Path.path.

        """
        return [p.path for p in self.paths_]

    @property
    def get_first(self):
        """Returns the path of the first item.

        """
        if self.paths_:
            return self.paths_[0].path

    def add_path(self, path):
        lgg.info("Recents add path %s" % path)
        for idx, p in enumerate(self.paths_):
            if path.path == p.path:
                self.paths_.pop(idx)
                break

        self.paths_.insert(0, path)
        if len(self.paths_) > self.max_folders:
            self.paths_.pop()

        self.core.ui.update_recent_folders()

    def remove_path(self, path):
        if isinstance(path, str):
            path = Path(path)

        if path in self.paths_:
            self.paths_.remove(path)
            self.core.ui.update_recent_folders()

    def clean_paths(self, fnames):
        folders = []
        for f in fnames:
            if f not in folders:
                folders.append(f)

        return folders


class Repertory:
    """Define a target folder.

    """
    def __init__(self, path):
        self.path = Path(path)
        self.name = self.path.fname
        self.button = None
        self.tab = None
        self.files = []
        self.thumbnails = None

    def check_validity(self):
        """Check the validity of the folder.

        Return False if the folder is read only or if the creation of the 
        thumbnails folder fail.
        """
        if not self.path.is_writable:
            return False

        th = self.path / 'thumbnails'
        if not th.ensure_folder_exists():
            return False

        self.thumbnails = th
        return True

    def get_free_space(self):
        stats = disk_usage(self.path.path)
        self.size = stats.total
        self.used = stats.used
        self.free = stats.free

    def update_free_space(self):
        self.get_free_space()
        self.button.sum += self.used
        

def get_main_screen_size():
    size = QDesktopWidget().availableGeometry(-1)
    return size.width(), size.height()

if __name__ == '__main__':
    lvl = logging.DEBUG
    LOG_FORMAT = "%(asctime)-6s %(levelname)s: %(name)s - %(message)s"
    logging.basicConfig(format=LOG_FORMAT, datefmt='%H:%M:%S', level=lvl)
    py_vers = sys.version.replace('\n', ' ')
    lgg.info('Run Oqapy-%s with Python-%s' %(VERSION_STR, py_vers))
    lgg.info('PyQt version %s' % PYQT_VERSION_STR)
    errors = 0
    try:
        from oqapy_iproc import OqapyIProc
        oi = OqapyIProc()
        vers = oi.version().decode('ascii')
        lgg.info('Using liboqapy_iproc-%s' % str(vers))
    except ImportError:
        lgg.critical('\x1b[1mliboqapy-iproc\x1b[0m not found,'
                     ' please install python3-oqapy-iproc')
        errors += 1
    except RuntimeError as why:
        lgg.critical('Importing \x1b[1mliboqapy-iproc\x1b[0m: %s' % why)
        errors += 1

    app = QApplication([])
    #import sip
    #sip.setdestroyonexit(True)
    f = ''
    if len(sys.argv) > 1:
        f = sys.argv[1]
    core = OqapyCore(app, f)
    sys.exit(app.exec_())

