# -*- coding: utf-8 -*-

# croptool.py
# This file is part of Oqapy serie 3
#
# Author: Vincent Vande Vyvre <vincent.vandevyvre@oqapy.eu>
# Home: http://www.oqapy.eu
# Copyright: 2013-2017 Vincent Vande Vyvre
# Licence: GPL3
#
# Crop tool

import logging
lgg = logging.getLogger(__name__)

from PyQt5 import QtCore, QtGui, QtWidgets
from tools.rectsize import RectSize

import stylesheets as theme

class CropTool:
    def __init__(self, core, viewer):
        super().__init__()
        self.core = core
        self.ui = core.ui
        self.viewer = viewer
        self.scene = viewer.scene_
        self.picture = viewer.get_qpixmap()
        self.zoom = viewer.get_zoom_factor()
        self.toolbox = self.ui.toolpanel.crop
        # posX, posY : start position in graphicsView coordinates
        self.posX = 0
        self.posY = 0
        # width, height : size of selection area
        self.width = 0
        self.height = 0
        # real_origin : start position in image 1/1
        self.real_origin = (0, 0)
        # real_size : ... yes, real size of cropped area
        self.real_size = (0, 0)
        self.maxX = self.scene.width()
        self.maxY = self.scene.height()
        self.area = False
        self.show_grid = False
        self.dial = False
        self.update_settings()

    def update_settings(self, reset=False):
        self.config = self.core.cfg.get(('crop', ))
        self.update_constrain_settings()
        if self.config['grid']:
            self.core.show_grid()

        else:
            self.core.hide_grid()

        if reset:
            self.toolbox.update_settings(self.config)

    def set_selection_area(self, start):
        """Build a selection area.

        Args:
        start -- mousePressEvent pos in scene coordinates
        """
        lgg.info('Begin crop area at: %s, %s' %(start.x(), start.y()))
        self.remove_crop_area()
        self.build_crop_area()

        if start.x() < 0:
            self.area.oldX = 0

        elif start.x() > self.maxX:
            self.area.oldX = self.maxX

        else:
            self.area.oldX = start.x()

        if start.y() < 0:
            self.area.oldY = 0

        elif start.y() > self.maxY:
            self.area.oldY = self.maxY

        else:
            self.area.oldY = start.y()

        self.area.show()

    def build_crop_area(self):
        self.set_opaque_layer()
        self.area = Area(self, self.picture, None, self.scene)
        self.area.geometryChanged.connect(self.update_geometry)
        self.area.processRequested.connect(self.prepare_crop)
        self.area.exitCroping.connect(self.on_abort_cropping)
        self.scene.addItem(self.area)

    def set_opaque_layer(self):
        self.op_layer = OpaqueLayer(self, self.scene)
        self.scene.addItem(self.op_layer)

    def draw_selection_area(self, end):
        """Modify the geometry of the QRubberBand.

        Args:
        end -- mouseMoveEvent pos in graphicsView coordinates
        """
        if end.x() < 0:
            x = 0

        elif end.x() > self.maxX:
            x = self.maxX

        else:
            x = end.x()

        if end.y() < 0:
            y = 0

        elif end.y() > self.maxY:
            y = self.maxY

        else:
            y = end.y()

        self.area.track_area(x, y)

    def fix_selection_area(self):
        """Set the selection area's size when mouse button is released.

        """
        if self.area is not None:
            self.area.fix_area()

    def update_constrain(self, constrain):
        self.core.cfg.update(('crop', 'constrain'), constrain)

    def update_geometry(self):
        self.real_origin = (self.area.posX / self.zoom, 
                            self.area.posY / self.zoom)
        self.real_size = (self.area._width / self.zoom, 
                            self.area._height / self.zoom)
        self.toolbox.set_size_label((int(self.real_size[0]),  
                                    int(self.real_size[1])))

    def modify_geometry(self, x, y, w, h):
        self.area.modify_geometry(x, y, w, h)

    def resize_by_zooming(self, scene, pix, factor):
        """Resize the crop area after zooming.

        Args:
        factor -- zoom's factor
        """
        lgg.info('Crop area resized by zooming, factor: %s' % factor)
        self.scene = scene
        self.picture = pix
        self.remove_crop_area()
        self.build_crop_area()
        self.area.posX = self.real_origin[0] * factor
        self.area.posY = self.real_origin[1] * factor
        self.area._width = self.real_size[0] * factor
        self.area._height = self.real_size[1] * factor
        self.zoom = factor
        self.area.call_redraw()
        self.update_geometry()

    def prepare_crop(self):
        """Get the parameters for process to crop..

        """
        lgg.info('Crop image: {0} x {1}'.format(
                    str(self.real_size[0]), str(self.real_size[1])))
        self.core.set_filtered_mode(True)
        left, upper =  int(self.real_origin[0]), int(self.real_origin[1])
        right, lower = int(self.real_size[0]), int(self.real_size[1])
        rect = QtCore.QRect(left, upper, right, lower)
        self.core.filtering.active_toolbox = self.toolbox
        self.core.abort_crop()
        self.core.filtering.crop_image(rect)
        self.toolbox.is_drawn = True
        # The settings are updated only if the cropping is achieved
        self.core.cfg.update(('crop', ), self.config)

    def close_extract(self):
        """Close the preview area.

        """
        lgg.info('Close the preview area')
        self.frame.deleteLater()
        self.frame.setParent(None)

        if self.core.state == "sorting":
            self.ui.main_tab.show()

        else:
            self.core.cur_viewer.show()

        self.core.abort_crop()

    def update_constrain_settings(self):
        if self.area:
            self.area.cfg = self.config
            self.area.fix_ratio()

    def remove_crop_area(self):
        lgg.info('Remove crop area')
        try:
            self.scene.removeItem(self.op_layer)
            del self.op_layer
        except Exception as why:
            pass
        
        try:
            self.scene.removeItem(self.area)
            del self.area
        except Exception as why:
            pass

    def apply_new_settings(self, cfg):
        """Sets the configuration values.

        Args:
        cfg -- dict of config
        """
        if self.area:
            self.area.apply_new_settings(cfg)

    def on_abort_cropping(self):
        self.toolbox.undo_crop()


class OpaqueLayer(QtWidgets.QGraphicsPixmapItem):
    def __init__(self, main, scene, parent=None):
        super().__init__(parent)
        self.viewer = main.viewer
        rsize = RectSize(scene.width(), scene.height()) 
        self.color = QtGui.QColor()
        self.color.setRgb(150, 150, 150, 255)
        self.cursor = self.viewer.cursor()
        self.pix = QtGui.QPixmap(rsize.as_qsize)
        self.pix.fill(self.color)
        self.setPixmap(self.pix)
        self.setPos(0, 0)
        self.show()
        self.setOpacity(0.5)

    def set_opacity(self, op=None, inc=None):
        if inc is not None:
            op = self.opacity() + (0.1 * inc)
        if op <= 0.0:
            op = 0.1
        elif op > 1.0:
            op = 1.0
        self.setOpacity(op)


class Area(QtWidgets.QGraphicsPixmapItem):
    def __init__(self, main, base, parent=None, scene=None):
        super().__init__(parent)
        self.main = main
        self.viewer = main.viewer
        self._scene = scene
        self.cfg = main.config
        self.posX = 0.0
        self.posY = 0.0
        self.oldX = 0.0
        self.oldY = 0.0
        self._width = 1.0
        self._height = 1.0
        self.maxX = scene.width()
        self.maxY = scene.height()
        self.w_width = 0
        self.w_height = 0
        self.grip_dragged = -1
        self.base = base
        self.setFlags(QtWidgets.QGraphicsItem.ItemIsMovable |
                        QtWidgets.QGraphicsItem.ItemSendsScenePositionChanges |
                        QtWidgets.QGraphicsItem.ItemIsFocusable)
        self.setAcceptHoverEvents(True)
        self.color = QtGui.QColor()
        self.color.setRgb(150, 220, 250, 100)
        self.border = QtGui.QColor()
        self.border.setRgb(30, 40, 220, 255)
        self.border_2 = QtGui.QColor()
        self.border_2.setRgb(150, 150, 255, 255)
        self.old_cursor = self.viewer.cursor()
        shape = QtGui.QPixmap("medias/cross_cursor_32.png")
        self.hovered_cursor = QtGui.QCursor(shape, -1, -1)
        self.cursors = []

        for f in ["medias/double_arrow_45.png", "medias/double_arrow_0.png",
                    "medias/double_arrow_135.png", "medias/double_arrow_90.png",
                    "medias/double_arrow_45.png", "medias/double_arrow_0.png",
                    "medias/double_arrow_135.png", "medias/double_arrow_90.png"]:
            shape = QtGui.QPixmap(f)
            self.cursors.append(QtGui.QCursor(shape, -1, -1))

        self.hlines = [100.0, 250, 400, 550]
        self.vlines = [100.0, 250, 400, 550, 700]

        self._modified = Modified()
        self._processCrop = ProcessCrop()
        self._closeCropTool = CloseCropTool()

    def geometryChanged():
        def fget(self):
            return self._modified.geometryChanged
        return locals()
    geometryChanged = property(**geometryChanged())

    def processRequested():
        def fget(self):
            return self._processCrop.processRequested
        return locals()
    processRequested = property(**processRequested())

    def exitCroping():
        def fget(self):
            return self._closeCropTool.exitCroping
        return locals()
    exitCroping = property(**exitCroping())

    def fix_ratio(self):
        """set the constrain's ratio if value is same as image.

        """
        if self.cfg['mode'][0] == "same_as":
            self.cfg['current_ratio'] = self.set_same_ratio_as_image()

        elif self.cfg['mode'][0] == "defined":
            w, h = self.cfg['mode'][1].split(':')
            if self.cfg['reverse']:
                self.cfg['current_ratio'] = float(h) / float(w)

            else:
                self.cfg['current_ratio'] = float(w) / float(h)

    def _constrain_new_area(self, b):
        self.cfg['constrain'] = b
        if b and self.cfg['current_ratio'] is not None:
            while 1:
                if self._width / self.cfg['current_ratio'] <= self.maxY:
                    # Keep width, resize height
                    self._height = self._width / self.cfg['current_ratio']
                    if self.posY + self._height > self.maxY:
                        self.posY -= (self.maxY - self._height)
                    break

                elif self._height * self.cfg['current_ratio'] <= self.maxX:
                    # Keep height, resize width
                    self._width = self._height * self.cfg['current_ratio']
                    if self.posX + self._width > self.maxX:
                        self.posX -= (self.maxX - self._width)
                    break

                else:
                    # No solution, reduce width
                    self._width -= 10

            self.remove_grips()
            self.redraw()
            self.set_grips()
            self.setFocus()
            self.geometryChanged.emit()

    def track_area(self, x, y):
        self.posX, self.posY, self._width, self._height = self.normalize(x, y)
        self.redraw()

    def redraw(self):
        x, y, w, h = self.posX, self.posY, self._width, self._height
        irect = QtCore.QRectF(x, y, w, h)
        prect = QtCore.QRectF(2.0, 2.0, self._width, self._height)
        self.selected_area = QtGui.QPixmap(QtCore.QSize(w + 4, h + 4))
        painter = QtGui.QPainter()
        painter.begin(self.selected_area)
        painter.setPen(QtGui.QPen(self.border_2, 1,
                              QtCore.Qt.SolidLine, QtCore.Qt.RoundCap,
                              QtCore.Qt.MiterJoin))
        painter.drawRect(1.0, 1.0, w + 1, h + 1)
        painter.setPen(QtGui.QPen(self.border, 1,
                              QtCore.Qt.SolidLine, QtCore.Qt.RoundCap,
                              QtCore.Qt.MiterJoin))
        painter.drawRect(0.0, 0.0, w + 3, h + 3)
        painter.drawPixmap(prect, self.base, irect)
        painter.end()
        self.setPixmap(self.selected_area)
        self.setPos(self.posX - 2, self.posY - 2)
        self.show()

    def normalize(self, x, y):
        if x == self.oldX:
            x, w = self.oldX, 1

        elif x < self.oldX:
            w = self.oldX - x

        else:
            x, w = self.oldX, x - self.oldX

        if y == self.oldY:
            y, h = self.oldY, 1

        elif y < self.oldY:
            h = self.oldY - y

        else:
            y, h = self.oldY, y - self.oldY

        return(x, y, w, h)

    def modify_geometry(self, x, y, w, h):
        """Modify the geometry by keyboard entries

        Keyword arguments:
        x, y -- top left corner
        w, h -- width, height
        """
        if self.cfg['constrain']:
            if w or h:
                # resize
                w, h = self.constrain_to_ratio(w, h)

        self.posX += x
        if self.posX < 0:
           self.posX = 0

        self.posY += y
        if self.posY < 0:
           self.posY = 0

        self._width += w
        if self.posX + self._width > self.maxX:
            self._width = self.maxX - self.posX

        self._height += h
        if self.posY + self._height > self.maxY:
            self._height = self.maxY - self.posY

        self.remove_grips()
        self.call_redraw()

    def call_redraw(self):
        self.redraw()
        self.set_grips()
        self.setFocus()
        self.geometryChanged.emit()

    def fix_area(self):
        self.set_grips()
        self.setFocus()
        if self.cfg['constrain']:
            self._constrain_new_area(True)

        self.geometryChanged.emit()

    def move_area(self, pos):
        """Move selection area with mouse.

        Keyword arguments:
        pos -- current mouse position
        """
        X, Y = pos.x(), pos.y()
        self.shift_area(X - self.old_X, Y - self.old_Y)
        self.old_X, self.old_Y = X, Y

    def shift_area(self, m_X, m_Y):
        """Move selection area.

        Keyword arguments:
        m_X -- delta x
        m_Y -- delta y
        """
        if self.posX + m_X <= 0 or self.posX + self._width + m_X >= self.maxX:
            m_X = 0

        if self.posY + m_Y <= 0 or self.posY + self._height + m_Y >= self.maxY:
            m_Y = 0    

        self.posX += m_X
        self.posY += m_Y
        self.call_redraw()

    def constrain_to_ratio(self, width, height):
        """constrain the size with ratio when resizing by keyboard.

        Keyword arguments:
        width, height -- delta width, delta height
        """
        if self.cfg['current_ratio'] is not None:
            if width:
                height = width / self.cfg['current_ratio']
                return width, height

            else:
                width = height * self.cfg['current_ratio']
                return width, height

    def _constrain_width(self, y, height):
        """Resize width when height is changed by mouse.

        Keyword arguments:
        y -- new pos y
        height -- new height
        """
        if self.cfg['current_ratio'] is not None:
            w = height * self.cfg['current_ratio']
            if self.posX + w >= self.maxX:
                back = self.posX - (w - self._width)
                if back < 0:
                    # can't enlarge anymore
                    return self.posX, self.posY, self._width, self._height

                else:
                    # width is max, move to left
                    return back, y, w, height

            else:
                return self.posX, y, w, height

        else:
            return self.posX, y, self._width, height

    def _constrain_height(self, x, w):
        if self.cfg['current_ratio'] is not None: 
            h = w / self.cfg['current_ratio']
            if self.posY + h > self.maxY:
                if self.maxY - h >= 0:
                    return x, self.maxY - h, w, h

                else:
                    # can't enlarge anymore
                    return self.posX, self.posY, self._width, self._height

            else:
                return x, self.posY, w, h

        else:
            return x, self.posY, w, self._height

    def get_left(self, left):
        deltaX = left - self.posX
        if left <= 0:
            left, w = 0, self._width

        elif left + self._width >= self.maxX:
            left, w = self.posX, 2

        else:
            w = self._width - deltaX
            if w <= 0:
                w = 2

        return left, w

    def get_top(self, top):
        deltaY = top - self.posY
        if top <= 0:
            top, h = 0, self._height

        elif top + self._height >= self.maxY:
            top, h = self.posY, 2

        else:
            h = self._height - deltaY
            if h <= 0:
                h = 2

        return top, h

    def get_right(self, right):
        deltaX = right - (self.posX + self._width)
        w = self._width + deltaX
        if w <= 0:
            return 2

        elif self.posX + w >= self.maxX:
            return self.maxX - self.posX

        return w

    def get_bottom(self, bottom):
        deltaY = bottom - (self.posY + self._height)
        h = self._height + deltaY
        if h <= 0 or self.posY + h >= self.maxY:
            return self._height

        return h

    def resize_by_top_left(self, pos):
        x, w = self.get_left(pos.x())
        y, h = self.get_top(pos.y())
        self.set_values(x, y, w, h)
        self.redraw()

    def resize_by_top(self, pos):
        y, h = self.get_top(pos.y())
        if self.cfg['constrain']:
            x, y, w, h = self._constrain_width(y, h)
            self.set_values(x, y, w, h)
            self.redraw()

        else:
            self.set_values(self.posX, y, self._width, h)
            self.redraw()

    def resize_by_top_right(self, pos):
        w = self.get_right(pos.x())
        y, h = self.get_top(pos.y())
        self.set_values(self.posX, y, w, h)
        self.redraw()

    def resize_by_right(self, pos):
        w = self.get_right(pos.x())
        if self.cfg['constrain']:
           self.posX, self.posY, w, self._height = self._constrain_height(
                                                            self.posX, w)
        self.set_values(self.posX, self.posY, w, self._height)
        self.redraw()

    def resize_by_bottom_right(self, pos):
        w = self.get_right(pos.x())
        h = self.get_bottom(pos.y())
        self.set_values(self.posX, self.posY, w, h)
        self.redraw()

    def resize_by_bottom(self, pos):
        h = self.get_bottom(pos.y())
        if self.cfg['constrain']:
            self.posX, self.posY, self._width, h = self._constrain_width(
                                                            self.posY, h)
        self.set_values(self.posX, self.posY, self._width, h)
        self.redraw()

    def resize_by_bottom_left(self, pos):
        x, w = self.get_left(pos.x())
        h = self.get_bottom(pos.y())
        self.set_values(x, self.posY, w, h)
        self.redraw()

    def resize_by_left(self, pos):
        x, w = self.get_left(pos.x())
        if self.cfg['constrain']:
            x, self.posY, w, self._height = self._constrain_height(x, w)
        self.set_values(x, self.posY, w, self._height)
        self.redraw()

    def set_values(self, left, top, right, bottom):
        self.posX, self.posY = left, top
        self._width, self._height = right, bottom
        self.geometryChanged.emit()

    def remove_grips(self):
        for child in self.childItems():
            child.setParentItem(None)
            self._scene.removeItem(child)

    #=====================================================================
    # EVENTS
    #=====================================================================

    def mousePressEvent(self, event):
        pos = event.scenePos()
        self.button = event.button()
        if event.button() == 4:
            self.old_X, self.old_Y = pos.x(), pos.y()
            if self.grip_dragged != -1:
                self.remove_grips()
            event.accept()

    def mouseReleaseEvent(self, event):
        self.button = False
        self.grip_dragged = -1
        self.fix_area()
        event.accept()

    def mouseMoveEvent(self, event):
        if self.button != 4: 
            event.ignore()
            return
        if self.grip_dragged == -1:
            self.move_area(event.scenePos())
        elif self.grip_dragged == 0:
            self.resize_by_top_left(event.scenePos())
        elif self.grip_dragged == 1:
            self.resize_by_top(event.scenePos())
        elif self.grip_dragged == 2:
            self.resize_by_top_right(event.scenePos())
        elif self.grip_dragged == 3:
            self.resize_by_right(event.scenePos())
        elif self.grip_dragged == 4:
            self.resize_by_bottom_right(event.scenePos())
        elif self.grip_dragged == 5:
            self.resize_by_bottom(event.scenePos())
        elif self.grip_dragged == 6:
            self.resize_by_bottom_left(event.scenePos())
        elif self.grip_dragged == 7:
            self.resize_by_left(event.scenePos())
        event.accept()

    def hoverEnterEvent(self, event):
        self.is_hovered = False
        self.setCursor(self.hovered_cursor)
        event.accept()

    def hoverLeaveEvent(self, event):
        self.is_hovered = False
        self.setCursor(self.old_cursor)
        event.accept()

    def hoverMoveEvent(self, event):
        event.accept()

    def contextMenuEvent(self, event):
        self.button = None
        self.show_menu(event.screenPos())
        event.accept()

    def keyPressEvent(self, event):
        mod = QtWidgets.QApplication.keyboardModifiers()
        l = 1
        h = 5
        sX = 0
        sY = 0
        eX = 0
        eY = 0
        if event.key() == QtCore.Qt.Key_Return or \
                event.key() == QtCore.Qt.Key_Enter:
            # Enter 16777221 & 16777220
            self.processRequested.emit()
            event.accept()

        elif mod == QtCore.Qt.NoModifier:             
            # Move 1 pxl 
            if event.key() == 16777234:
                sX -= l          # arrow L
            elif event.key() == 16777235:
                sY -= l          # arrow U
            elif event.key() == 16777236:
                sX += l          # arrow R
            elif event.key() == 16777237:
                sY += l          # arrow D
            else :
                if event.key() == QtCore.Qt.Key_F5 or event.key() == 16777268:
                    self.main.op_layer.set_opacity(None, 1)
                    event.accept()
                    return
                elif event.key() == QtCore.Qt.Key_F4 or event.key() == 16777267:
                    self.main.op_layer.set_opacity(None, -1)
                    event.accept()
                    return
                event.ignore()
                return
            self.shift_area(sX, sY)
            event.accept()

        elif mod == (QtCore.Qt.ShiftModifier | QtCore.Qt.ControlModifier):
            # Quick maximise 
            if event.key() == 16777234:
                sX -= h          # arrow L
                eX += h
            elif event.key() == 16777235:
                sY -= h          # arrow U
                eY += h
            elif event.key() == 16777236:
                eX += h            # arrow R
            elif event.key() == 16777237:
                eY += h            # arrow D           
            else :
                event.ignore()
                return
            self.modify_geometry(sX, sY, eX, eY)
            event.accept()

        elif mod == (QtCore.Qt.ShiftModifier | QtCore.Qt.AltModifier):
            # Quick minimise
            if event.key() == 16777234:
                eX -= h            # arrow L
            elif event.key() == 16777235:
                eY -= h            # arrow U
            elif event.key() == 16777236:
                sX += h          # arrow R
                eX -= h
            elif event.key() == 16777237:
                sY += h          # arrow D
                eY -= h
            else :
                event.ignore()
                return
            self.modify_geometry(sX, sY, eX, eY)
            event.accept()

        elif mod == QtCore.Qt.ShiftModifier:
            # Quick move 
            if event.key() == 16777234:
                sX -= h          # arrow L
            elif event.key() == 16777235:
                sY -= h          # arrow U
            elif event.key() == 16777236:
                sX += h          # arrow R
            elif event.key() == 16777237:
                sY += h          # arrow D
            else :
                event.ignore()
                return
            self.shift_area(sX, sY)
            event.accept()

        elif mod == QtCore.Qt.ControlModifier:
            # Maximise
            if event.key() == 16777234:
                sX -= l          # arrow L
                eX += l
            elif event.key() == 16777235:
                sY -= l          # arrow U
                eY += l
            elif event.key() == 16777236:
                eX += l            # arrow R
            elif event.key() == 16777237:
                eY += l            # arrow D      
            else :
                event.ignore()
                return
            self.modify_geometry(sX, sY, eX, eY)
            event.accept()

        elif mod == QtCore.Qt.AltModifier:
            # Minimise 
            if event.key() == 16777234:
                eX -= l            # arrow L
            elif event.key() == 16777235:
                eY -= l            # arrow U
            elif event.key() == 16777236:
                sX += l          # arrow R
                eX -= l
            elif event.key() == 16777237:
                sY += l          # arrow D
                eY -= l
            else :
                event.ignore()
                return
            self.modify_geometry(sX, sY, eX, eY)
            event.accept()

        else :
            event.ignore()

    #==============================
    # Grips
    #==============================

    def change_grip(self, grip):
        self.grip_dragged = grip

    def set_grips(self):
        icons = ['medias/grip_45.png', 'medias/grip_0.png',
                   'medias/grip_315.png', 'medias/grip_270.png',
                   'medias/grip_225.png', 'medias/grip_180.png',
                   'medias/grip_135.png', 'medias/grip_90.png']
 
        pos = [(-2, -2), 
                (self._width / 2 - 7, -2), 
                (self._width - 6, - 2),
                (self._width - 6, self._height / 2  - 7),
                (self._width -6, self._height - 6),
                (self._width / 2 - 7, self._height - 6), 
                (-2, self._height - 6), 
                (-2, self._height / 2 - 7)]

        for n, p in enumerate(pos):
            grip = Grip(n, p, icons[n], self, self._scene, self.cursors[n])
            grip.show()

    #=============================================================
    # Context menu
    #=============================================================

    def show_menu(self, pos):
        """Build and show context menu.

        Keyword arguments:
        pos -- event.pos() in screen coordinates
        """
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setWeight(75)
        font.setBold(True)
        self.menu = QtWidgets.QMenu()
        self.settings = QtWidgets.QAction(None)
        self.settings.setText(_("Settings"))
        self.menu.addAction(self.settings)
        ratio_group = QtWidgets.QActionGroup(self.menu)
        self.ratio_on = QtWidgets.QAction(None)
        self.ratio_on.setText(_("Constrain ratio"))
        self.ratio_on.setCheckable(True)
        self.ratio_on.setChecked(False)
        ratio_group.addAction(self.ratio_on)
        self.menu.addAction(self.ratio_on)
        self.ratio_out = QtWidgets.QAction(None)
        self.ratio_out.setText(_("Unconstrain ratio"))
        self.ratio_out.setCheckable(True)
        self.ratio_out.setChecked(True)
        ratio_group.addAction(self.ratio_out)
        self.menu.addAction(self.ratio_out)
        self.grid_on = QtWidgets.QAction(None)
        self.grid_on.setText(_("Show grid"))
        self.grid_on.setCheckable(True)
        #self.grid_on.setChecked(self.main.main.grid_is_visible())
        self.menu.addAction(self.grid_on)
        self.menu.addSeparator()      
        self.crop_act = QtWidgets.QAction(None) 
        self.crop_act.setText(_("Extract image"))
        self.menu.addAction(self.crop_act)
        self.abort_act = QtWidgets.QAction(None) 
        self.abort_act.setText(_("Close crop tool"))
        self.menu.addAction(self.abort_act)

        if self.cfg['constrain']:
            self.ratio_on.setChecked(True)

        if self.main.show_grid:
            self.grid_on.setChecked(True)

        self.settings.triggered.connect(self.set_settings)
        self.ratio_on.triggered.connect(self.set_ratio)
        self.ratio_out.triggered.connect(self.set_ratio)
        self.grid_on.toggled.connect(self.set_grid)
        self.crop_act.triggered.connect(self.extract_image)
        self.abort_act.triggered.connect(self.quit)

        self.menu.exec_(pos)

    def set_settings(self):
        ui = Settings(self)
        ui.set_values(self.main.config)
        ui.exec_()
        self.main.update_settings(reset=True)

    def set_ratio(self):
        if self.ratio_on.isChecked():
            if self.cfg['mode'][0] == "same_as":
                self.cfg['current_ratio'] = self.set_same_ratio_as_image()

        self.main.config["constrain"] = self.ratio_on.isChecked()
        self._constrain_new_area(self.ratio_on.isChecked())
        
    def set_grid(self, b):
        self.main.core.toggle_grid(b)

    def extract_image(self):
        self.main.prepare_crop()

    def get_ratio(self):
        if self.cfg['mode'][0] == "same_as":
            return self.set_same_ratio_as_image()

    def set_same_ratio_as_image(self):
        return self.main.picture.width() / float(self.main.picture.height())

    def apply_new_settings(self, cfg):
        lgg.info('New settings: {0}'.format(cfg['mode']))
        self.ratio = cfg["ratio"]
        self.reverse = cfg["reverse"]
        self.force_width = cfg["force_width"]
        self.grid = cfg["grid"]
        self.main.config = cfg

    def quit(self):
        """Quiet exit crop tool.

        """ 
        self._scene.removeItem(self)
        self.exitCroping.emit()


class Grip(QtWidgets.QGraphicsPixmapItem):
    def __init__(self, idx, pos, icon, parent=None, scene=None, cursor=None):
        super().__init__(parent)
        self.parent_ = parent
        self.idx = idx
        self._cursor = cursor
        pix = QtGui.QPixmap(icon)
        self.setPixmap(pix)
        self.setOpacity(0.5)
        self.setPos(pos[0], pos[1])
        self.setFlags(QtWidgets.QGraphicsItem.ItemIgnoresParentOpacity)
        self.setAcceptHoverEvents(True)

    def hoverEnterEvent(self, event):
        self.setCursor(self._cursor)
        self.setOpacity(1.0)
        self.parent_.change_grip(self.idx)

    def hoverLeaveEvent(self, event):
        self.parent_.change_grip(-1)
        self.setOpacity(0.5)
        event.accept()


class Settings(QtWidgets.QDialog):
    def __init__(self, parent):
        super().__init__()
        #pal = theme.get_palette()
        #self.setPalette(pal)
        self.parent = parent
        self.resize(500, 250)
        self.gridLayout = QtWidgets.QGridLayout(self)
        self.gridLayout.setContentsMargins(20, 4, 20, 4)
        self.verticalLayout = QtWidgets.QVBoxLayout()
        self.horizontalLayout = QtWidgets.QHBoxLayout()
        self.sub_1 = SubTitle(self)
        self.horizontalLayout.addWidget(self.sub_1)
        spacerItem = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, 
                                    QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout.addItem(spacerItem)
        self.verticalLayout.addLayout(self.horizontalLayout)
        self.sep_1 = QtWidgets.QFrame(self)
        self.sep_1.setFrameShape(QtWidgets.QFrame.HLine)
        self.sep_1.setFrameShadow(QtWidgets.QFrame.Plain)
        self.sep_1.setMidLineWidth(1)
        self.sep_1.setObjectName("sep_1")
        self.verticalLayout.addWidget(self.sep_1)
        self.horizontalLayout_2 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_2.setObjectName("horizontalLayout_2")
        self.same_as_rdo = QtWidgets.QRadioButton(self)
        self.same_as_rdo.setChecked(True)
        self.same_as_rdo.setObjectName("same_as_rdo")
        self.group = QtWidgets.QButtonGroup(self)
        self.group.addButton(self.same_as_rdo)
        self.horizontalLayout_2.addWidget(self.same_as_rdo)
        spacerItem1 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, 
                                    QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_2.addItem(spacerItem1)
        self.verticalLayout.addLayout(self.horizontalLayout_2)
        self.horizontalLayout_3 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_3.setObjectName("horizontalLayout_3")
        self.defined_rdo = QtWidgets.QRadioButton(self)
        self.defined_rdo.setObjectName("defined_rdo")
        self.group.addButton(self.defined_rdo)
        self.horizontalLayout_3.addWidget(self.defined_rdo)

        self.defined_cmb = QtWidgets.QComboBox(self)
        self.defined_cmb.setEnabled(False)
        self.defined_cmb.setMinimumSize(QtCore.QSize(120, 0))
        self.defined_cmb.setObjectName("defined_cmb")
        self.horizontalLayout_3.addWidget(self.defined_cmb)
        self.reverse_chb = QtWidgets.QCheckBox(self)
        self.reverse_chb.setEnabled(False)
        self.reverse_chb.setObjectName("reverse_chb")
        self.horizontalLayout_3.addWidget(self.reverse_chb)
        spacerItem2 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, 
                                    QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_3.addItem(spacerItem2)
        self.verticalLayout.addLayout(self.horizontalLayout_3)
        self.horizontalLayout_4 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_4.setObjectName("horizontalLayout_4")
        self.custom_rdo = QtWidgets.QRadioButton(self)
        self.custom_rdo.setObjectName("custom_rdo")
        self.group.addButton(self.custom_rdo)
        self.horizontalLayout_4.addWidget(self.custom_rdo)
        self.label_2 = QtWidgets.QLabel(self)
        self.label_2.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|
                                    QtCore.Qt.AlignVCenter)
        self.label_2.setObjectName("label_2")
        self.horizontalLayout_4.addWidget(self.label_2)

        self.width_spb = QtWidgets.QDoubleSpinBox(self)
        self.width_spb.setMinimum(0.01)
        self.width_spb.setMaximum(500.0)
        self.width_spb.setValue(1.0)
        self.width_spb.setEnabled(False)
        self.width_spb.setMinimumSize(QtCore.QSize(80, 0))
        self.width_spb.setObjectName("width_spb")
        self.horizontalLayout_4.addWidget(self.width_spb)
        self.label_3 = QtWidgets.QLabel(self)
        self.label_3.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|
                                    QtCore.Qt.AlignVCenter)
        self.label_3.setObjectName("label_3")
        self.horizontalLayout_4.addWidget(self.label_3)
        self.height_spb = QtWidgets.QDoubleSpinBox(self)
        self.height_spb.setMinimum(0.01)
        self.height_spb.setMaximum(500.0)
        self.height_spb.setValue(1.0)
        self.height_spb.setEnabled(False)
        self.height_spb.setMinimumSize(QtCore.QSize(80, 0))
        self.height_spb.setObjectName("height_spb")
        self.horizontalLayout_4.addWidget(self.height_spb)
        self.append_chb = QtWidgets.QCheckBox(self)
        self.append_chb.setEnabled(False)
        self.append_chb.setObjectName("append_chb")
        self.horizontalLayout_4.addWidget(self.append_chb)
        spacerItem3 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, 
                                    QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_4.addItem(spacerItem3)
        self.verticalLayout.addLayout(self.horizontalLayout_4)
        #self.horizontalLayout_8 = QtWidgets.QHBoxLayout()
        #self.horizontalLayout_8.setObjectName("horizontalLayout_8")
        #self.sub_2 = SubTitle(self)
        #self.horizontalLayout_8.addWidget(self.sub_2)
        #spacerItem = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, 
                                    #QtWidgets.QSizePolicy.Minimum)
        #self.horizontalLayout_8.addItem(spacerItem)
        #self.verticalLayout.addLayout(self.horizontalLayout_8)
        #self.sep_2 = QtWidgets.QFrame(self)
        #self.sep_2.setFrameShape(QtWidgets.QFrame.HLine)
        #self.sep_2.setFrameShadow(QtWidgets.QFrame.Plain)
        #self.sep_2.setMidLineWidth(1)
        #self.sep_2.setObjectName("sep_2")
        #self.verticalLayout.addWidget(self.sep_2)
        #self.horizontalLayout_5 = QtWidgets.QHBoxLayout()
        #self.horizontalLayout_5.setObjectName("horizontalLayout_5")
        #self.label_4 = QtWidgets.QLabel(self)
        #self.label_4.setObjectName("label_4")
        #self.horizontalLayout_5.addWidget(self.label_4)
        #spacerItem4 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, 
                                    #QtWidgets.QSizePolicy.Minimum)
        #self.horizontalLayout_5.addItem(spacerItem4)
        #self.verticalLayout.addLayout(self.horizontalLayout_5)
        #self.horizontalLayout_6 = QtWidgets.QHBoxLayout()
        #self.horizontalLayout_6.setObjectName("horizontalLayout_6")
        #spacerItem5 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Fixed, 
                                    #QtWidgets.QSizePolicy.Minimum)
        #self.horizontalLayout_6.addItem(spacerItem5)
        #self.constrain_width_rdo = QtWidgets.QRadioButton(self)
        #self.constrain_width_rdo.setChecked(True)
        #self.constrain_width_rdo.setObjectName("constrain_width_rdo")
        #self.group_1 = QtWidgets.QButtonGroup(self)
        #self.group_1.addButton(self.constrain_width_rdo)
        #self.horizontalLayout_6.addWidget(self.constrain_width_rdo)
        #spacerItem6 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, 
                                    #QtWidgets.QSizePolicy.Minimum)
        #self.horizontalLayout_6.addItem(spacerItem6)
        #self.verticalLayout.addLayout(self.horizontalLayout_6)
        #self.horizontalLayout_7 = QtWidgets.QHBoxLayout()
        #self.horizontalLayout_7.setObjectName("horizontalLayout_7")
        #spacerItem7 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Fixed, 
                                    #QtWidgets.QSizePolicy.Minimum)
        #self.horizontalLayout_7.addItem(spacerItem7)
        #self.constrain_height_rdo = QtWidgets.QRadioButton(self)
        #self.constrain_height_rdo.setObjectName("constrain_height_rdo")
        #self.group_1.addButton(self.constrain_height_rdo)
        #self.horizontalLayout_7.addWidget(self.constrain_height_rdo)
        #spacerItem8 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, 
                                    #QtWidgets.QSizePolicy.Minimum)
        #self.horizontalLayout_7.addItem(spacerItem8)
        #self.verticalLayout.addLayout(self.horizontalLayout_7)
        self.horizontalLayout_11 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_11.setObjectName("horizontalLayout_11")
        self.sub_3 = SubTitle(self)
        self.horizontalLayout_11.addWidget(self.sub_3)
        spacerItem = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, 
                                    QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_11.addItem(spacerItem)
        self.verticalLayout.addLayout(self.horizontalLayout_11)
        self.sep_3 = QtWidgets.QFrame(self)
        self.sep_3.setFrameShape(QtWidgets.QFrame.HLine)
        self.sep_3.setFrameShadow(QtWidgets.QFrame.Plain)
        self.sep_3.setMidLineWidth(1)
        self.sep_3.setObjectName("sep_3")
        self.verticalLayout.addWidget(self.sep_3)
        self.horizontalLayout_9 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_9.setObjectName("horizontalLayout_9")
        self.grid_chb = QtWidgets.QCheckBox(self)
        self.grid_chb.setObjectName("grid_chb")
        self.horizontalLayout_9.addWidget(self.grid_chb)
        spacerItem10 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, 
                                    QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_9.addItem(spacerItem10)
        self.verticalLayout.addLayout(self.horizontalLayout_9)
        spacerItem11 = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Minimum, 
                                    QtWidgets.QSizePolicy.Expanding)
        self.verticalLayout.addItem(spacerItem11)
        self.horizontalLayout_10 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_10.setObjectName("horizontalLayout_10")
        spacerItem12 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, 
                                    QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_10.addItem(spacerItem12)
        self.close_btn = QtWidgets.QPushButton(self)
        icon2 = QtGui.QIcon()
        icon2.addPixmap(QtGui.QPixmap("medias/close.png"), QtGui.QIcon.Normal, 
                                    QtGui.QIcon.Off)
        self.close_btn.setIcon(icon2)
        self.close_btn.setObjectName("close_btn")
        self.horizontalLayout_10.addWidget(self.close_btn)
        self.verticalLayout.addLayout(self.horizontalLayout_10)
        self.gridLayout.addLayout(self.verticalLayout, 0, 0, 1, 1)
        self.retranslate()

        self.same_as_rdo.toggled.connect(self.on_same_as_toggled)
        self.defined_rdo.toggled.connect(self.on_defined_toggled)
        self.custom_rdo.toggled.connect(self.on_custom_toggled)
        self.width_spb.valueChanged.connect(self.on_custom_value_changed)
        self.height_spb.valueChanged.connect(self.on_custom_value_changed)
        self.append_chb.stateChanged.connect(self.on_append_changed)
        self.close_btn.clicked.connect(self.apply_settings)
        self.cfg = None
        self.custom_width = 0.0
        self.custom_height = 0.0

    def apply_style(self):
        self.same_as_rdo.setStyleSheet(theme.RADIO)
        self.defined_rdo.setStyleSheet(theme.RADIO)
        self.defined_cmb.setStyleSheet(theme.COMBO_DIAL)
        self.reverse_chb.setStyleSheet(theme.CHECKBOX)
        self.custom_rdo.setStyleSheet(theme.RADIO)
        self.width_spb.setStyleSheet(theme.DSPINBOX)
        self.height_spb.setStyleSheet(theme.DSPINBOX)
        self.append_chb.setStyleSheet(theme.CHECKBOX)
        self.grid_chb.setStyleSheet(theme.CHECKBOX)
        self.sep_1.setStyleSheet(theme.UNDERLINE)
        self.sep_3.setStyleSheet(theme.UNDERLINE)
        self.close_btn.setStyleSheet(theme.PUSHBUTTONDIAL)

    def set_values(self, cfg):
        self.cfg = cfg
        idx = 0
        if self.cfg["mode"][0] == "same_as":
            self.same_as_rdo.setChecked(True)

        elif self.cfg["mode"][0] == "defined":
            self.defined_rdo.blockSignals(True)
            self.defined_rdo.setChecked(True)
            self.defined_rdo.blockSignals(False)

            if not self.cfg["mode"][1] in self.cfg["ratios"]:
                # Should not appears ...
                self.cfg["ratios"].append(self.cfg["mode"][1])
            idx = self.cfg["ratios"].index(self.cfg["mode"][1])

        else:
            self.custom_rdo.setChecked(True)
            self.width_spb.setValue(self.cfg["mode"][1])
            self.height_spb.setValue(self.cfg["mode"][2])

        self.defined_cmb.addItems(self.cfg["ratios"])
        self.defined_cmb.setCurrentIndex(idx)

        if self.cfg["reverse"]:
            self.reverse_chb.setChecked(True)

        if self.cfg["grid"]:
            self.grid_chb.setChecked(True)

    def on_same_as_toggled(self, b):
        if b:
            self.set_defined_widget(False)
            self.set_custom_widget(False)
            self.cfg["mode"] = ["same_as"]

    def on_defined_toggled(self, b):
        if b:
            self.set_defined_widget(True)
            self.set_custom_widget(False)
            self.cfg["mode"] = ["defined", self.defined_cmb.currentIndex()]

    def on_custom_toggled(self, b):
        if b:
            self.set_defined_widget(False)
            self.set_custom_widget(True)
            self.append_chb.setChecked(False)
            self.append_chb.setEnabled(self.compare_ratio())
            self.cfg["mode"] = ["custom", self.width_spb.value(), 
                                self.height_spb.value()]

    def on_custom_value_changed(self, value):
        self.append_chb.setChecked(False)
        if value == 0.0:
            self.append_chb.setEnabled(False)

        else:
            self.append_chb.setEnabled(self.compare_ratio())
            self.set_custom_values()

    def set_custom_values(self):
        self.custom_width = self.width_spb.value()
        self.custom_height = self.height_spb.value()
        if self.custom_width == 0 or self.custom_height == 0:
            self.append_chb.setEnabled(False)

        else:
            self.append_chb.setEnabled(True)

    def on_append_changed(self, b):
        if b is not False:
            r = self.get_ratio_string()
            self.defined_cmb.addItem(r)
            self.cfg["ratios"].append(r)

    def set_defined_widget(self, b):
        self.defined_cmb.setEnabled(b)
        self.reverse_chb.setEnabled(b)

    def set_custom_widget(self, b):
        self.width_spb.setEnabled(b)
        self.height_spb.setEnabled(b)
        self.append_chb.setEnabled(b)

    def apply_settings(self):
        self.cfg["reverse"] = self.reverse_chb.isChecked()
        if self.cfg["mode"][0] == "custom":
            self.set_custom_values()
            self.cfg["mode"] = ["custom", self.custom_width, 
                                self.custom_height]
            self.cfg["current_ratio"] = self.custom_width / self.custom_height

        elif self.cfg["mode"][0] == "defined":
            w, h = self.set_defined_number()
            if self.cfg["reverse"]:
                self.cfg["current_ratio"] = h / x

            else:
                self.cfg["current_ratio"] = w / h

        self.cfg["grid"] = self.grid_chb.isChecked()
        self.accept()

    def compare_ratio(self):
        r = self.get_ratio_string()
        if self.defined_cmb.findText(r) != -1:
            return False

        return True

    def get_ratio_string(self):
        w, h = self.width_spb.value(), self.height_spb.value()
        if w % 1 == 0:
            w = int(w)

        if h % 1 == 0:
            h = int(h)

        return ":".join([str(w), str(h)])

    def set_defined_number(self):
        s = str(self.defined_cmb.currentText())
        self.cfg["mode"] = ["defined", s]
        w, h = s.split(":")
        return float(w), float(h)

    def retranslate(self):
        self.setWindowTitle(_("Crop Tool Settings - Oqapy"))
        self.sub_1.set_text(_("Ratio"))
        self.same_as_rdo.setText(_("Same as image"))
        self.defined_rdo.setText(_("Defined"))
        self.reverse_chb.setToolTip(_("Reverse width / height"))
        self.reverse_chb.setText(_("Reverse"))
        self.custom_rdo.setText(_("Custom"))
        self.label_2.setText(_("width:"))
        self.label_3.setText(_("height:"))
        self.append_chb.setToolTip(_("Append this ratio into predefined list"))
        self.append_chb.setText(_("Append"))
        self.sub_3.setText(_("Grid"))
        self.grid_chb.setText(_("Always show grid"))
        self.close_btn.setText(_("Close"))

class SubTitle(QtWidgets.QLabel):
    def __init__(self, parent=None):
        super().__init__(parent)
        font = QtGui.QFont()
        font.setFamily("Trebuchet MS")
        font.setPointSize(11)
        self.setFont(font)
        self.setStyleSheet("QLabel {color: #787C84;}")

    def set_text(self, text):
        self.setText(text)

class Modified(QtCore.QObject):
    """Signal used by crop tool when selected area geometry has changed.

    """
    geometryChanged = QtCore.pyqtSignal()
    def __init__(self):
        super(Modified, self).__init__()

class ProcessCrop(QtCore.QObject):
    """Signal used by crop tool when extract processing is requested.

    """
    processRequested = QtCore.pyqtSignal()
    def __init__(self):
        super(ProcessCrop, self).__init__()

class CloseCropTool(QtCore.QObject):
    """Signal used by crop tool when close crop tool button is 
    pressed in context menu.

    """
    exitCroping = QtCore.pyqtSignal()
    def __init__(self):
        super(CloseCropTool, self).__init__()
