# -*- coding: utf-8 -*-

# iodcraw.py
# This file is part of Oqapy serie 3
#
# Author: Vincent Vande Vyvre <vincent.vandevyvre@oqapy.eu>
# Home: http://www.oqapy.eu
# Copyright: 2015-2017 Vincent Vande Vyvre
# Licence: GPL3
#
# Raw process with DCraw

import os 
import sys
import subprocess
import logging
lgg = logging.getLogger(__name__)

from PyQt5 import QtCore

class IOdcraw:
    def __init__(self):
        self.output_file = None
        self.pid = False

    def valid_image(self, path):
        if not os.path.isfile(path):
            return False

        reply = subprocess.Popen(["dcraw", "-i", path], 
                            universal_newlines=True, 
                            stdout=subprocess.PIPE)

        while 1:
            if reply.poll() != None: 
                break

        if reply.poll() == 1:
            # Unvalid raw file
            return False

        return True

    def valid_images(self, imgs):
        """Verify if each image in imgs is readable by dcraw.

        Keyword arguments :
        imgs -- list of images
        
        Returns :
        one list of raw files
        """
        return [i for i in imgs if self.valid_image(i)]

    def get_thumbnail(self, img):
        """Extract the thumbnail from a raw file.

        Keyword arguments :
        img -- valid raw file

        Returns :
        0 if the image has a thumbnail, else 1 
        """
        reply = subprocess.Popen(["dcraw", "-e", img], 
                            universal_newlines=True, 
                            stdout=subprocess.PIPE)
        while 1:
            p = reply.poll()
            if p != None:
                break

        return not p

    def get_metadata(self, fname, img=0):
        """Extract the metadata from a raw file.

        Keyword arguments :
        fname -- valid raw file
        img -- index of image in file

        Returns :
        Metadata
        """
        fl = os.path.basename(fname)
        fobj = subprocess.PIPE
        if not self.registered(fl):
            fobj = open('data_log', 'a')
            fobj.write('\n * %s *\n' % fl)
        subprocess.Popen(["dcraw", "-s", str(img), "-i", "-v", fname],
                            universal_newlines=True, 
                            stdout=fobj, stderr=subprocess.STDOUT)
        try:
            fobj.close()
        except AttributeError:
            pass

    def get_parameters(self, path):
        return subprocess.Popen(["dcraw", "-i", "-v", path],
                            universal_newlines=True, 
                            stdout=subprocess.PIPE, 
                            stderr=subprocess.STDOUT).communicate()

    def convert_image(self, img):
        fobj = open('process', 'a')
        fobj.write('\n * * %s\n' % img)
        cmd = ["dcraw", "-v", "-w", "-s", "all", "-T", img]
        reply = subprocess.Popen(cmd,
                            universal_newlines=True,
                            stdout=fobj, stderr=subprocess.STDOUT)

        while 1:
            if reply.poll() != None:
                break

        fobj.close()
        return reply.communicate()

    def convert_with_parameters(self, param):
        self.last_error = False
        self.output_file = None
        param.insert(0, "dcraw -v")
        cmd = " ".join(param)
        lgg.info("run cmd: `%s'" % cmd)
        reply = subprocess.Popen(cmd, universal_newlines=True, 
                                 stdout=subprocess.PIPE, 
                                 stderr=subprocess.STDOUT,
                                 shell=True)
        self.pid = reply.pid

        while 1:
            text = reply.stdout.readline()[:-1]
            if type(text) != str or text == '' and reply.poll() != None:
                break

            if '.pgm has the wrong dimensions!' in text:
                self.kill_process()
                self.last_error = 'Darkframe has the wrong dimensions'
            if 'Writing data' in text:
                self.output_file = text[16:]
        self.pid = False

    def kill_process(self):
        if self.pid:
            lgg.info('Cancel unraw process, pid: %s' % self.pid)
            try:
                os.kill(self.pid, 15)
                os.kill(self.pid+1, 15)
            except Exception as why:
                lgg.warning('Kill process error: %s' % why)

    def convert_darkframe(self, fname):
        cmd = " ".join(['dcraw', '-v', '-D', '-4', '-j', '-t 0', '"%s"' % fname])
        lgg.info("run cmd: `%s'" % cmd)
        reply = subprocess.Popen(cmd, universal_newlines=True, 
                                 stdout=subprocess.PIPE, 
                                 stderr=subprocess.STDOUT,
                                 shell=True)
        self.pid = reply.pid

        while 1:
            text = reply.stdout.readline()[:-1]
            if type(text) != str or text == '' and reply.poll() != None:
                break

        self.pid = False
