# -*- coding: utf-8 -*-

# rawimport.py
# This file is part of Oqapy serie 3
#
# Author: Vincent Vande Vyvre <vincent.vandevyvre@oqapy.eu>
# Home: http://www.oqapy.eu
# Copyright: 2015-2017 Vincent Vande Vyvre
# Licence: GPL3
#
# Raw images handler

# http://www.camerahacker.com/Digital/dcraw_by_example.shtml

import os
import shutil
import logging
lgg = logging.getLogger(__name__)

from datetime import datetime
from threading import Thread

from PyQt5.QtCore import Qt, QObject, pyqtSignal
from PyQt5.QtGui import QPixmap, QImage, QTransform
from pyexiv2.metadata import ImageMetadata
from pyexiv2 import xmp

from tools.path import Path
from metadata.metadata import Metadata
from demosaicing.iodcraw import IOdcraw
from warningmessages import WarningMessages
from data import VERSION_STR, XMP_VERSION_STR


class RawImport(QObject):
    processFinished = pyqtSignal(bool)
    thumbnailsCompleted = pyqtSignal()
    def __init__(self, core, image=None):
        super().__init__()
        self.core = core
        self.ui = core.ui
        self.image = image
        self.preview = None
        self.temp_fld = core.workspace['raws']
        self.raw_clipboard = None
        self.raw_process_canceled = False
        self.dcraw = IOdcraw()

    def is_raw_file(self, path=None):
        """Define if an image file is a RAW file.

        Args:
        path -- Full path of file

        Returns:
        True if the file is valid and if dcraw support this format
        """
        if path is None:
            if self.image is None:
                return False
            path = self.image

        if not os.path.isfile(path):
            return False

        return self.dcraw.valid_image(path)

    def import_raw_image(self, path, preview):
        """Prepare and run the unraw process.

        Args:
        path -- full path of original RAW file
        preview -- full path of preview
        dark -- full path of darkframe, it must be a .pgm file
        """
        def process(arg):
            self.dcraw.convert_with_parameters(arg)
            if self.dcraw.last_error:
                self.processFinished.emit(False)

            else:
                self.processFinished.emit(True)

        self.preview = preview
        params, data = self.get_parameters()
        if params:
            data['oqapyVersion'] = VERSION_STR
            data['version'] = XMP_VERSION_STR
            image = RawImage(path, preview, data)
            params.append('"%s"' % path)
            Thread(target=process, args=(params,)).start()
            return image

    def get_parameters(self):
        """Collect the unraw processing parameters.

        Returns:
        chain -- The dcraw command arguments
        data -- The dictionnary used for the image metadata
        """
        data = {}
        chain = []
        box = self.ui.toolpanel.raw_param
        white = ["-w", "-a", "-r"][box.white_balance_cmb.currentIndex()]
        chain.append(white)
        if white == "-w":
            # Camera's parameters
            data['whiteBalance'] = 'default'

        elif white == "-r":
            # User's modifiers
            values, factors = self.get_multiplier_values()
            data['whiteBalance'] = " ".join([str(v) for v in factors])
            for v in values:
                chain.append(str(v))

        else:
            # Average calculate on the entire image
            data['whiteBalance'] = 'average'

        interpolation = box.interpolation_cmb.currentIndex()
        data['interpolation'] = 'AHD'
        if interpolation != 3:
            if interpolation == 4:
                chain.append('-f')
            else:
                chain.append('-q %s' % interpolation)
            data['interpolation'] = str(box.interpolation_cmb.currentText())

        color = box.color_space_cmb.currentIndex()
        data['colorSpace'] = 'sRGB D65'
        if color != 1:
            chain.append("-o")
            chain.append(str(color))
            data['colorSpace'] = str(box.color_space_cmb.currentText())
        chain.append("-T")

        data['gamma'] = 'BT.709'
        if box.gamma_chb.isChecked():
            chain.append('-4')
            data['gamma'] = 'linear 16-bits'

        else:
            gamma = box.gamma_cmb.currentIndex()
            if gamma:
                values = [None, '-g 2.2 0', '-g 1.8 0', '-g 2.4 12.9']
                chain.append(values[gamma])
                data['gamma'] = str(box.gamma_cmb.currentText())

        data['cleanupPasses'] = '0'
        if box.cleanup_chb.isChecked():
            val = box.cleanup_spb.value()
            chain.append('-m %s' % val)
            data['cleanupPasses'] = str(val)

        try:
            val = int(box.n_image_cmb.currentText()) - 1
        except ValueError:
            val = 0
        data['imageIndex'] = val
        if val:
            chain.append('-s %s' % val)

        return chain, data

    def get_multiplier_values(self):
        """Returns the values of the multipliers modified by the user.

        Returns:
        values -- the default values (provided by the camera)
        (red, blue) -- the modified values
        """
        red = self.ui.toolpanel.raw_param.red_dsp.value() 
        blue = self.ui.toolpanel.raw_param.blue_dsp.value()
        mults = self.preview.unraw_settings['multipliers']
        if mults is None:
            return

        values = [float(i) for i in mults]
        values[0] = values[0] * red
        values[2] = values[2] * blue
        return values, (red, blue)

    def copy_settings(self):
        _, self.raw_clipboard = self.get_parameters()
        self.ui.toolpanel.paste_raw_settings_btn.setEnabled(True)

    def set_old_settings(self):
        if self.raw_clipboard is not None:
            self.ui.toolpanel.raw_param.set_values(self.raw_clipboard)
            

class RawImage(object):
    def __init__(self, path, preview, data):
        super(RawImage, self).__init__()
        # original raw file in temporary
        self.raw_path = Path(path)
        # original raw file in source folder
        self.uri = None
        self.preview = preview
        self.out_path = None
        self.format = None
        self.data = data
        self.unraw_setting = None
        self.is_rotated = False
        self.preview = None
        self.data['rawSourceFile'] = self.raw_path.fname
        self.base = self.raw_path.fname
        self.name = self.raw_path.name
        self.ext = 'TIFF'

    def set_backup_name(self, name):
        self.out_path = name
        self.name, self.ext = os.path.splitext(os.path.basename(name))

    def transpose_data(self):
        self.read_metadata()
        if self.metadata is not None:
            self.add_raw_process_metadata()
            self.write_metadata()

    def read_metadata(self):
        try:
            self.metadata = Metadata(self.uri.path)
            self.metadata.get_data()
        except Exception as why:
            lgg.info("Error in read metadata, reason:\n{0}".format(why))
            self.metadata = None

    def update_data(self):
        self.data['nameSpace'] = "www.oqapy.eu/xmp/1.0/"
        date = datetime.utcnow().strftime('%Y %m %d %H:%M:%S')
        self.data['lastEditDate'] = date
        self.data['unrawDate'] = date
        if not 'substractDarkFrame' in self.data:
            self.data['substractDarkFrame'] = 'No dark'
        self.data['lastEditorName'] = 'Unknow'
        self.data['lastChanges'] = 'Nihil'

    def add_raw_process_metadata(self):
        """Add the raw process parameters into the metadata.

        Domain name: www.oqapy.eu/xmp/1.0/
        ['oqapyVersion', 'lastEditDate', 'lastEditorName', 'lastChanges',
                'rawSourceFile', 'unrawDate', 'whiteBalance', 'interpolation', 
                'colorSpace', 'gamma', 'cleanupPasses', 'imageIndex', 
                'substractDarkFrame']
        """
        if self.metadata is None:
            # FIXME create a blank metadata
            return

        data = self.metadata.get_xmp_oqapy_tags()
        if data is None:
            try:
                xmp.register_namespace('www.oqapy.eu/xmp/1.0/', 'oqapy')
            except Exception as why:
                if not 'prefix already exists' in str(why):
                    lgg.info('register_namespace error: %s' % why)
                    return
                self._create_data()

        self.update_data()
        for key, value in self.data.items():
            key = 'Xmp.oqapy.%s' % key
            try:
                self.metadata.data[key] = str(value)
            except Exception as why:
                lgg.warning('update_data set value error key: %s value: %s'
                                %(key, value))

        try:
            self.metadata.data['Exif.Image.Orientation'] = 1
        except:
            pass

    def _create_data(self):
        tags = ['oqapyVersion', 'version']
        vals = [VERSION_STR, XMP_VERSION_STR]
        for tag, val in zip(tags, vals):
            tag_ = 'Xmp.oqapy.%s' % tag
            try:
                self.metadata.data[tag_] = val
            except KeyError:
                 self.metadata.data[tag_] = xmp.XmpTag(tag_, val)

    def write_metadata(self):
        if self.metadata is not None:
            try:
                new = ImageMetadata(self.out_path)
                new.read()
            except Exception as why:
                lgg.warning("Read error: {0}".format(why))
                message = {'context': _('Metadata error'),
                    'warn': _("Can't copy the metadata in"),
                    'data': self.out_path,
                    'msg': _('Reason:'),
                    # Be care, system errors are translated
                    'data1': why.strerror.decode('utf-8', 'ignore'),
                    'btn': _('Close')}
                warn = WarningMessages(0, message)
                warn.build_dialog()
                return

            self.metadata.data.copy(new, comment=False)

            try:
                new.write()
            except Exception as why:
                lgg.warning("Write metadata error: {0}".format(why))

    def print_data(self):
        # Debugging function
        for d in self.data:
            print('%-15s:%s' %(d, self.data[d]))


