# -*- coding: utf-8 -*-

# rawviewer.py
# this file is part of Oqapy serie 3
#
# Author: Vincent Vande Vyvre <vincent.vandevyvre@oqapy.eu>
# Home: http://www.oqapy.eu
# Copyright: 2015-2017 Vincent Vande Vyvre
# Licence: GPL3
#
# Viewing engine for RAW images

import os
import shutil
import time
from threading import Thread
import logging
lgg = logging.getLogger(__name__)

from PyQt5.QtCore import QObject, Qt, pyqtSignal, QCoreApplication
from PyQt5.QtGui import QPixmap, QImage, QTransform

from trailers import Trailer
from metadata.metadata import Metadata
from metadataeditor import MetadataEditor
from histogram import Histogram
from ioimage import IOImage
from tools.path import Path
from warningmessages import WarningMessages
from demosaicing.rawimport import RawImport
from demosaicing.iodcraw import IOdcraw

class RawViewer(QObject):
    """Define the raw images handler.

    """
    rawPreviewsCompleted = pyqtSignal()
    thumbnailsCompleted = pyqtSignal()
    def __init__(self, core):
        """Instanciate the raw images viewing.

        Args:
        core -- Oqapy Core instance
        """
        super().__init__()
        self.core = core
        self.ui = core.ui
        self.trailer = Trailer(core, True)
        self.trailer.view = self.ui.stackedtrailers.raw_trailer
        self.trailer.configure(self.core.cfg.get(('trailer', )))
        self.rawtemp_fld = self.core.workspace['raws']
        self.viewer = self.core.state.current_viewer
        self.toolboxes = self.ui.toolpanel
        self.header_bar = self.ui.stackedviews.raw_header
        self.current_index = 0
        self.raw_import = RawImport(core)
        self.current_preview = None
        self.current_raw = False
        self.raw_process_canceled = False
        self.backups = []
        self.raw_previews = []
        self.bck_count = 0
        self.toolboxes.raw_param.raw_viewer = self
        self.rawPreviewsCompleted.connect(self.on_raw_previews_ready)
        self.raw_import.processFinished.connect(self.on_raw_process_finished)
        self.trailer.trailerCompleted.connect(self.on_trailer_completed)
        self.header_bar.quick_save_btn.clicked.connect(self.on_quick_save)
        self.header_bar.fname_led.textChanged.connect(self.on_name_changed)

    def set_raw_files(self, dir_):
        """Set the list of raw files.

        Args:
        dir_ -- instance of Core.SourceFolder
        """
        self.source_dir = dir_
        self.thumbs_fld = dir_.thumbs_fld
        self.previews_fld = dir_.raw_previews_fld
        self.current_index = 0
        self.initiate_viewing()

    def initiate_viewing(self):
        def process(args):
            raws = []
            self.raw_previews = []
            for idx, r in enumerate(self.source_dir.raws):
                b = os.path.basename(r)
                raw = RawPreview(self.core, r, self.previews_fld, 
                                 self.thumbs_fld)
                self.raw_previews.append(raw)

            self.rawPreviewsCompleted.emit()
        Thread(target=process, args=(None,)).start()

    def close_rawfiles_viewing(self):
        if self.current_raw:
            # TODO display a warning message
            return

        self.viewer.clear()

    def get_thumbnail_name(self, fname, th_dir):
        fld, base = os.path.split(fname)
        name, ext = os.path.splitext(base)
        thumb = "thumb_%s<%s>.jpg" %(name, ext[1:])
        return os.path.join(th_dir, thumb)

    def on_raw_previews_ready(self):
        lgg.info("RAW previews completed %s files" % len(self.raw_previews))
        self.trailer.clean_view()
        self.trailer.load_raw_images(self.raw_previews)

    def on_trailer_completed(self):
        lgg.info("RAW trailer completed")

    def count(self):
        return len(self.raw_previews)

    def next_image(self):
        """Go to the next image.

        """
        idx = (self.current_index + 1) % self.count()
        lgg.info('Next raw file: %s' % idx)
        self.set_current_image(idx)

    def previous_image(self):
        """Go back to the previous image.

        """
        idx = (self.current_index - 1) % self.count()
        lgg.info('Previous raw file: %s' % idx)
        self.set_current_image(idx)

    def rotate_image(self, angle):
        """Rotate the current image.

        Args:
        angle -- 90 or -90 degrees
        """
        self.viewer.rotate_image(angle)
        self.viewer.refresh_qpixmap()
        if self.current_raw:
            # The current image has been demosaicated
            self.current_raw.is_rotated = True

    def set_current_image(self, idx=None):
        """Set the current image.

        Args:
        idx -- the index of the image
        """
        if idx is None:
            idx = self.current_index

        idx = min(idx, len(self.raw_previews)-1)
        self.current_preview = self.raw_previews[idx]
        lgg.info('Set current raw image: %s, %s' 
                    % (idx, self.current_preview.source.fname))
        self.viewer.set_qimage_from_backup(self.current_preview.preview)
        self.viewer.refresh_qpixmap()
        self.current_index = idx
        self.current_raw = False
        self.update_trailer()
        self.set_header_widgets(self.current_preview.source.fname, True)
        self.edit_raw_image()
        num = self.count()
        self.ui.statusbar.set_image_index(idx+1, num)

    def zoom_on_area(self):
        self.viewer.set_zoom_area(True)

    def set_full_size(self):
        self.viewer.full_size()

    def zoom_up(self):
        self.viewer.zoom_in()

    def zoom_at_value(self, zoom):
        self.viewer.current_zoom = zoom / 100.0
        self.viewer.refresh_qpixmap(0)

    def zoom_down(self):
        self.viewer.zoom_out()

    def fit_in_view(self):
        self.viewer.fit_in_view()

    def update_trailer(self):
        """Update the trailer feature with the current image.

        """
        lgg.info('Update raw trailer')
        self.trailer.blockSignals(True)
        self.trailer.set_current_item(self.current_index)
        self.trailer.blockSignals(False)
        self.ui.toolbar.widget.trailer_sld.blockSignals(True)
        self.ui.toolbar.set_trailer_slider_value(self.current_index)
        self.ui.toolbar.widget.trailer_sld.blockSignals(False)

    def edit_raw_image(self):
        """Display the current raw file.

        The image displayed into the viewer is a preview extracted from the
        header of the raw file.
        """
        src = self.current_preview.source
        self.set_properties(src)
        self.core.update_navigation_viewer(self.viewer.qimage)
        self.toolboxes.raw_param.reset_to_default()
        self.toolboxes.raw_histogram.clear()
        self.clear_temp()
        self.backups = []
        self.bck_count = 0
        self.temp_raw = os.path.join(self.rawtemp_fld.path, src.fname)
        try:
            shutil.copy(src.path, self.rawtemp_fld.path)
            self.backups.append(self.temp_raw)
        except IOError as why:
            lgg.warning('Raw copy error: %s' % why)
            message = {'context': _('Copy error'),
                'warn': _("Can't create safe copy of"),
                'data': self.temp_raw,
                'msg': _('Reason:'),
                'data1': why.strerror.decode('utf-8', 'ignore'),
                'btn': _('Close')}
            warn = WarningMessages(0, message)
            warn.build_dialog()
            self.raw_temp = None

    def set_header_widgets(self, fname, prev, safe=False):
        """Update the widget into the header bar of the viewer.

        Args:
        fname -- the file name
        prev -- True if the image is a preview, False if the image has been
                demosaicated
        safe -- True if the image has been saved
        """
        self.header_bar.set_file_name(fname)
        self.header_bar.set_preview_state(prev, safe)

    def set_properties(self, path):
        """Set the image properties into the Image Infos tool box.

        Args:
        fname -- Path instance of the file
        """
        self.toolboxes.image_infos.clear()
        self.toolboxes.info_bx.setTitle(path.fname)
        self.data = Metadata(path.path)
        data = self.data.get_data()
        if not data:
            self.data = None
            return

        if self.current_preview.unraw_settings is None:
            self.current_preview.get_camera_parameters()

        self.core.metadataeditor.current_image = path.path
        self.core.metadataeditor.set_data(self.data)
        self.create_histogram(self.current_preview.preview, True)
        weight = path.size()
        date = str(self.data.data['Exif.Photo.DateTimeOriginal'].value)
        settings = self.current_preview.unraw_settings
        width = settings['raw_size'][0]
        height = settings['raw_size'][1]
        self.toolboxes.image_infos.set_raw_image_data(width, height, weight, date)
        self.toolboxes.raw_inquiries.clear()
        self.toolboxes.raw_inquiries.set_values(path.fname, settings)
        self.toolboxes.raw_param.set_image_number(settings['n_images'])
        self.toolboxes.raw_param.darkframe_ext = path.ext
        self.toolboxes.export.clear()
        
    def apply_parameters(self):
        """Apply the user's parameters to the current image.

        """
        raw = self.temp_raw
        prev = self.current_preview
        self.set_buzzy_widgets(True)
        self.current_raw = self.raw_import.import_raw_image(raw, prev)
        self.toolboxes.export.save_btn.setEnabled(True)

    def set_buzzy_widgets(self, state):
        cursors = (Qt.ArrowCursor, Qt.BusyCursor)
        self.viewer.setCursor(cursors[state])
        QCoreApplication.processEvents()

    def cancel_process(self):
        """Abort the current unraw process.

        """
        self.raw_process_canceled = True
        self.raw_import.dcraw.kill_process()

    def show_view(self, fname):
        """Send the unrawed image to the viewer.

        Args:
        fname -- full path of the unrawed image
        """
        self.viewer.set_qimage_from_backup(fname)
        self.viewer.refresh_qpixmap()
        qimage = self.viewer.get_qimage()
        if self.core.alt_screen:
            self.core.alt_screen.set_image(qimage, self.current_raw)
        QCoreApplication.processEvents()


    def on_raw_process_finished(self, state):
        """Slot of the signal `processFinished'.

        Args:
        state -- True if the process has finished without error, False otherwise
        """
        self.set_buzzy_widgets(False)
        self.toolboxes.raw_param.set_progress_bar(False)
        if self.raw_process_canceled:
            self.raw_process_canceled = False
            return

        if not state:
            self.handle_raw_process_error()
            return

        img = self.get_new_files()
        if not img:
            return

        self.create_histogram(img)
        fname = os.path.basename(img)
        self.set_header_widgets(fname, False, False)
        self.toolboxes.clone_raw_settings_btn.setEnabled(True)
        self.current_raw.set_backup_name(img)
        self.current_raw.uri = self.current_preview.source
        self.current_raw.transpose_data()
        self.show_view(img)
        # devel function
        #self.current_raw.print_data()
        self.toolboxes.export.setEnabled(True)
        self.toolboxes.export.set_data(self.current_raw, 
                                       self.viewer.get_qimage())
        self.toolboxes.export.data_chb.setChecked(True)

    def on_quick_save(self):
        cfg = self.toolboxes.export.get_export_config()
        if self.current_raw.ext.lower() == cfg['format'].lower():
            self.save_current_image(cfg)

        else:
            self.export_current_image(cfg)

    def on_name_changed(self, text):
        try:
            name, ext = os.path.splitext(text)
        except:
            self.update_target_name(self.target_name)
            return

        self.toolboxes.export.update_filename(name)
        self.toolboxes.export.update_format(ext)
        self.toolboxes.export.on_extension_changed(ext, False)

    def create_histogram(self, fname, preview=False):
        """Build the histogram of the current image.

        Args:
        fname -- Full path of the current image (raw or unrawed)
        preview -- True if the current image is a preview
        """
        size = self.toolboxes.raw_histogram.get_histogram_size()
        bname = os.path.splitext(os.path.basename(fname))[0]

        hname = "".join(["histo_", bname, '.png'])
        self.fhisto = self.core.workspace['histo'] / hname
        vname = "".join(["values_", bname])
        #fval = self.core.workspace['histo'] / vname
        histogram = Histogram(fname, self.fhisto.path, size)
        if preview:
            histogram.histogramCreated.connect(self.update_histogram)

        else:
            histogram.histogramCreated.connect(self.update_raw_histogram)
        histogram.brightness = True
        histogram.build_histogram()

    def get_new_files(self):
        """Returns the name of the unrawed file.

        """
        img = self.raw_import.dcraw.output_file
        img = self.rename_backup(img)
        lgg.info('Rename backup: %s' % img)
        self.current_raw.set_backup_name(img)
        self.backups.append(img)
        return img

    def copy_settings(self):
        """Keep reference of the current parameters.

        This needed to process with the same parameters on several raw files.
        """
        self.raw_import.copy_settings()

    def set_old_settings(self):
        """Reset the old parameters.

        """
        self.raw_import.set_old_settings()

    def update_histogram(self, b):
        if b:
            self.toolboxes.histogram.show_histo(self.fhisto.path)

    def update_raw_histogram(self, b):
        if b:
            self.toolboxes.raw_histogram.show_histo(self.fhisto.path)

    def update_viewer(self):
        if self.current_preview is None:
            self.set_current_image()

    def clear_temp(self):
        for f in os.listdir(self.rawtemp_fld.path):
            fpath = self.rawtemp_fld / f
            try:
                os.remove(fpath.path)
            except:
                pass

    def rename_backup(self, bck):
        """Copy the unrawed file with an appropriated backup name.

        Since the user can do some demosaications of the same file with
        different parameters, the backup name is suffixed with a number.

        Args:
        bck -- the name used by DCRaw for the unrawed file
        """
        bck = bck.lstrip().rstrip(' .\n')
        path, ext = os.path.splitext(bck)
        self.bck_count += 1
        dest = "".join([path, "_(%s)" % self.bck_count, ext])
        shutil.copy(bck, dest)
        return dest

    def save_current_image(self, conf):
        """Save the current image.

        Called by the 'Quick Save' button.
        The image is always saved in TIFF format (compression LZW) and the
        metadata are written in the new file.

        Args:
        conf -- dict defining the current demosaicated image
        """
        orig = self.current_raw.out_path
        dest = self.check_file_name(conf['dest'],
                                    conf['name'], 
                                    conf['format'])
        th = self.get_thumbnail_name(dest, self.thumbs_fld.path)
        parent = os.path.basename(conf['dest'])
        fname = os.path.basename(dest)
        lgg.info('Copy %s to .../%s/%s' % (orig, parent, fname))
        try:
            shutil.copy(orig, dest)
        except IOError as why:
            lgg.warning('Raw copy error: %s' % why)
            message = {'context': _('Copy error'),
                'warn': _("Can't create safe copy of"),
                'data': dest,
                'msg': _('Reason:'),
                'data1': why.strerror.decode('utf-8', 'ignore'),
                'btn': _('Close')}
            warn = WarningMessages(0, message)
            warn.build_dialog()
            return

        if not os.path.isfile(th):
            self.create_thumbnail(th)

        if self.source_dir.where == conf['dest']:
            lgg.info("Insert: %s\n\t%s" %(dest, th))
            self.core.insert_image(dest, th)

    def export_current_image(self, param):
        """Save on disk the current image.

        Args:
        param -- dictionnary
        """
        fname = self.check_file_name(param['dest'], param['name'], 
                                     param['format'])
        self._export(fname, param['format'], param['quality'], param['data'])

    def create_thumbnail(self, fname):
        src = self.viewer.get_qimage()
        thumb = src.scaled(200, 200, Qt.KeepAspectRatio, 
                                   Qt.FastTransformation)
        thumb.save(fname, None, -1)

    def _export(self, fname, mime_type, quality, data):
        src = self.viewer.get_qimage()
        ioimage = IOImage()
        ioimage.set_image_to_writer(src)
        ioimage.set_filename_to_writer(fname)
        save = ioimage.write_image()
        if not save[0]:
            err = ' '.join(str(save[1]), str(save[2]))
            lgg.warning("Save change error: %s" % err)
            message = {'context': _('Export image'),
                'warn': _("Unable to write the file:"),
                'data': fname,
                'msg': _('Reason:'),
                'data1': err,
                'btn': _('Close')}
            warn = WarningMessages(0, message)
            warn.build_dialog()
            return False

        lgg.info("Current raw file saved to: %s" % fname)
        if data:
            self.write_metadata(fname)

        thumb = self.get_thumbnail_name(fname, self.thumbs_fld.path)
        if os.path.dirname(fname) == self.source_dir.where:
            if not os.path.isfile(thumb):
                self.core.trailer.create_thumbnail(fname, thumb)

            self.core.insert_image(fname, thumb)

        self.toolboxes.export.set_progress_bar(False)
        self.set_header_widgets(os.path.basename(fname), False, True)

    def write_metadata(self, fname):
        """Write the metadata into the unrawed image.

        Args:
        fname -- full path of the file
        """
        self.current_raw.out_path = fname
        self.current_raw.read_metadata()
        self.current_raw.add_raw_process_metadata()
        self.current_raw.write_metadata()

    def check_file_name(self, path, fname, ext):
        """Returns a safe file name.

        If a file with the same name already exists, an index is added at the
        new file name.

        Args:
        path -- the path of the destination folder
        fname -- the name without extension of the file
        ext -- the extension of the file

        Returns:
        full path of the file
        """
        dest = os.path.join(path, fname + ext)
        count = 0
        while os.path.isfile(dest):
            count += 1
            dest = os.path.join(path, fname + "_(%s)%s" %(count, ext))

        return dest

    def handle_raw_process_error(self):
        mssg = self.raw_import.dcraw.last_error
        if mssg is None:
            mssg = 'Unknow error'

        lgg.warning('Raw process error: %s' % mssg)
        message = {'context': _('Process error'),
            'warn': _("Can't convert RAW image"),
            'data': os.path.basename(self.temp_raw),
            'msg': _('Reason:'),
            'data1': mssg,
            'btn': _('Close')}
        warn = WarningMessages(0, message)
        warn.build_dialog()


class RawPreview:
    def __init__(self, core, source, folder, thumbs):
        self.core = core
        self.source = Path(source)
        self.base = self.source.fname
        self.name = self.source.name
        self.ext = self.source.ext
        self.preview = None
        self.size = None
        self.metadata = False
        self.previews_fld = folder
        self.thumbnails_fld = thumbs
        self.unraw_settings = None
        self.get_preview_filename()
        self.read_metadata()
        if not os.path.isfile(self.preview):
            self.extract_preview()

        if not os.path.isfile(self.thumbnail):
            self.create_thumbnail()

    def extract_preview(self):
        lgg.info("Extract preview %s" % self.source.path)
        dcraw = IOdcraw()
        preview = dcraw.get_thumbnail(self.source.path)
        if preview:
            fld, base = self.source.parents, self.source.fname
            prname = self.find_file_name(fld, self.source.name+".thumb")
            if prname is None:
                return False

            orig = os.path.join(fld, prname)
            orient = self.get_orientation()
            if orient != 1:
                orig = self.rotate_preview(orig, orient)

            elif os.path.splitext(prname)[1].lower() not in ('.jpg', '.jpeg'):
                orig = self.convert_to_jpeg(orig)
            try:
                shutil.move(orig, self.preview)
            except:
                pass
            else:
                try:
                    self.clone_data()
                    return
                except:
                    pass

        else:
            # dcraw can't extract the preview, try with pyexiv2
            self.read_metadata()
            self.metadata.get_thumbnails()
            if self.metadata.thumbnails:
                thumb = self.metadata.thumbnails[-1]
                fld = self.source.parents
                name = self.source.name
                dest = os.path.join(fld, name)
                thumb.write_to_file(str(dest))
                prname = self.find_file_name(fld, name)
                orig = os.path.join(fld, prname)
                orient = self.get_orientation()
                if orient != 1:
                    orig = self.rotate_preview(orig, orient)

                elif os.path.splitext(prname)[1].lower() not in ('.jpg', '.jpeg'):
                    orig = self.convert_to_jpeg(orig)

                try:
                    shutil.move(orig, self.preview)
                except:
                    # No more chance, use 'No preview' thumb
                    pass
                else:
                    try:
                        self.clone_data()
                        return
                    except:
                        pass

            else:
                shutil.copy("medias/No_Preview_200x150.jpg", self.preview)
                self.clone_data()

    def get_camera_parameters(self):
        dcraw = IOdcraw()
        reply = dcraw.get_parameters(self.source.path)
        if isinstance(reply, tuple):
            vals = reply[0].split('\n')
            sett = {}
            for val in vals:
                if val:
                    key, value = val.split(':', 1)
                    sett[key.strip()] = value.strip()
            self.set_default_settings(sett)
            return True
        else:
            return False

    def set_default_settings(self, dct):
        default = {}
        default["raw_name"] = self.source.fname
        keys = ["camera", "date", "shutter", "ISO", "aperture",
                "colors", "pattern", "ICC_profile"]
        labels = ["Camera", "Timestamp", "Shutter", "ISO speed", "Aperture", 
                  "Raw colors", "Filter pattern", "Embedded ICC profile"]
        for k, l in zip(keys, labels): 
            try:
                default[k] = dct[l]
            except KeyError:
                default[k] = None
        try:
            default["n_images"] = int(dct["Number of raw images"])
        except KeyError:
            default["n_images"] = None
        for k, l in [["raw_size", "Image size"], 
                     ["out_size", "Output size"],
                     ["thumb_size", "Thumb size"]]:
            try:
                default[k] = [int(i) for i in dct[l].split('x')]
            except KeyError:
                default[k] = None
        try:
            icc = dct["Embedded ICC profile"]
            default["ICC"] = icc if icc != 'no' else None
        except KeyError:
            default["ICC"] = None
        try:
            default["daylight"] = dct["Daylight multipliers"].split(' ')
        except KeyError:
            default["daylight"] = None
        try:
            default["multipliers"] = dct["Camera multipliers"].split()
        except KeyError:
            default["multipliers"] = None

        if abs(default["out_size"][0] - default["raw_size"][0]) > 10:
            self.is_rotated = True

        self.unraw_settings = default

    def read_metadata(self):
        metadata = Metadata(self.source.path)
        data = metadata.get_data()
        if not data:
            return

        self.metadata = metadata

    def get_orientation(self):
        try:
            return self.metadata.data['Exif.Image.Orientation'].value
        except Exception as why:
            return 1

    def clone_data(self):
        fn = os.path.basename(self.preview)
        try:
            self.metadata.export_data(self.preview)
        except Exception as why:
            lgg.warning("Can't copy data: %s" % why)
            return

        self.metadata = Metadata(self.preview)
        try:
            self.metadata.get_data()
        except Exception as why:
            lgg.warning("Can't read data: %s" % why)
            return

    def find_filename(self, base):
        for item in os.listdir(self.previews_fld.path):
            if base in item:
                return item

    def find_file_name(self, folder, name):
        for item in os.listdir(folder):
            if name in item:
                return item

    def rotate_preview(self, prev, orientation):
        angle = {3: 180, 6: 90, 8: -90}[orientation]
        dest = os.path.splitext(prev)[0] + ".jpg"
        img = QImage(prev)
        if img.isNull():
            img = QImage("medias/No_Preview_200x150.jpg")

        else:
            matrix = QTransform()
            img = img.transformed(matrix.rotate(angle), Qt.SmoothTransformation)

        img.save(dest, None, -1)
        if prev != dest:
            try:
                os.remove(prev)
            except Exception as why:
                lgg.warning("Can't remove %s %s" %(prev, why))

        try:
            self.metadata.data['Exif.Image.Orientation'] = 1
        except:
            pass
        return dest

    def create_thumbnail(self):
        img = QImage(self.preview)
        if img.isNull():
            img = QImage("medias/No_Preview_200x150.jpg")

        pix = img.scaled(200, 200, Qt.KeepAspectRatio, 
                                   Qt.FastTransformation)
        pix.save(self.thumbnail, None, -1)

    def convert_to_jpeg(self, fname):
        img = QImage(fname)
        if img.isNull():
            img = QImage("medias/No_Preview_200x150.jpg")
        dest = os.path.splitext(fname)[0] + ".jpg"
        img.save(dest, None, -1)
        try:
            os.remove(fname)
        except Exception as why:
            lgg.warning("Can't remove %s %s" %(fname, why))

        return dest

    def get_preview_filename(self):
        prev = "prev_%s<%s>.jpg" %(self.source.name, self.source.ext[1:])
        thumb = "thumb_%s<%s>.jpg" %(self.source.name, self.source.ext[1:])
        self.preview = os.path.join(self.previews_fld.path, prev)
        self.thumbnail = os.path.join(self.thumbnails_fld.path, thumb)

