# -*- coding: utf-8 -*-

# filtering.py
# This file is part of Oqapy serie 3
#
# Author: Vincent Vande Vyvre <vincent.vandevyvre@oqapy.eu>
# Home: http://www.oqapy.eu
# Copyright: 2016-2017 Vincent Vande Vyvre
# Licence: GPL3
#
# Image transformations module

# http://colour-science.org/posts/demosaicing-and-hdri-with-python/


import os
import time
import glob
import shutil
import logging
lgg = logging.getLogger(__name__)

from math import radians, sin, cos, tan
from threading import Thread, Timer

from PyQt5.QtCore import Qt, QObject, pyqtSignal, QRect, QSize
from PyQt5.QtGui import QTransform, QPixmap

from ioimage import IOImage
from thumbnailer import Thumbnailer
from histogram import Histogram
from metadata.metadata import Metadata
from warningmessages import WarningMessages
from filters.contrastbrightnessfilter import ContrastBrightnessFilter
from filters.contraststretchingfilter import ContrastStretchingFilter
from filters.denoisingfilter import DenoisingFilter
from filters.toning import Toning
from filters.laplace import Laplace


class Filtering(QObject):
    processFinished = pyqtSignal()
    processFailed = pyqtSignal()
    def __init__(self, core):
        super().__init__()
        self.core = core
        self.ui = core.ui
        self.header_view = self.ui.stackedviews.viewing_header
        self.historic_view = self.ui.toolpanel.historic
        self.temp_folder = core.workspace['bckups']
        self.historic = Historic(self.temp_folder, self.historic_view)
        self.ioimage = IOImage()
        self.backups = []
        self.preview = False
        # Instance of the current original Picture
        self.origin = None
        self.is_previewed = False
        self.is_rotated = False
        self.processFinished.connect(self.on_filtering_finished)
        self.processFailed.connect(self.on_filtering_failed)

    def set_original_picture(self, origin):
        """Set the original current Picture.

        Args:
        origin -- instance of Picture
        """
        self.origin = origin
        self.backups = []
        self.historic.reset()

    def set_image(self, picture, viewer):
        """Set the image.

        Args:
        picture -- the Picture or the Backup instance
        viewer -- the current GraphicsArea instance
        """
        self.image = picture
        self.viewer = viewer

    def resize_image(self, toolbox, width, height, prop=True):
        """Resize the current picture.

        Args:
        toolbox -- instance of resizing toolbox
        width -- the new width
        height -- the new height
        prop -- boolean, if True resize proportionnal
        """
        lgg.info('Resizing image to %s x %s' %(width, height))
        self.active_toolbox = toolbox
        backup = self.historic.create_new_change(['resize', (width, height)])
        self.viewer.resize_qimage(width, height, prop)
        self.viewer.refresh_qpixmap()
        self.viewer.setFocus()
        saved = self.set_backup(backup)
        self.historic.set_new_change(saved)
        if saved:
            self.add_backup(backup, self.image)

        self.core.filtering_finished(saved, toolbox, self.backups[-1])
        self.core.update_navigation_viewer(self.viewer.get_qimage())

    def crop_image(self, rect, rotation=False):
        """Crop the current image.

        Args:
        rect -- QRect instance
        rotation -- angle of rotation if called by Rotation-Straighten tool box
        """
        lgg.info('Crop image: {0} x {1}'.format(rect.width(), rect.height()))
        self.viewer.crop_qimage(rect)
        self.viewer.refresh_qpixmap()
        self.viewer.setFocus()
        if rotation:
            args = ['straight', (rotation, rect.width(), rect.height())]

        else:
            args = ['crop', (rect.width(), rect.height())]
            #try:
                # Not sure if selection area already exists
                #self.core.crop.remove_crop_area()
            #except Exception as why:
                #lgg.info("Crop tool can't remove the layer: {0}".format(why))

        img = self.viewer.get_qimage()
        backup = self.historic.create_new_change(args)
        saved = self.set_backup(backup)
        self.historic.set_new_change(saved)
        if not saved:
            self.historic.cancel_new_change()

        else:
            self.add_backup(backup, self.image)
            #self.ui.preview.set_image(img)
            # Update the resizing tool box
            self.on_last_backup_changed(img)
            #if rotation:
                #self.reset_widgets(backup, self.straight_tb)

            #else:
                #self.reset_widgets(backup, self.crop_tb)

            #self.crop_tb.is_drawn = True
        self.core.filtering_finished(saved, self.active_toolbox, 
                                     self.backups[-1])
        self.core.update_navigation_viewer(img)

    def rotate_image(self, angle):
        """Rotate the current image.

        Args:
        angle -- degrees, int or float
        """
        lgg.info('Rotate image angle: %s (%s rad)' % (angle, radians(angle)))
        #self.core.state.freeze_tool_boxes(self.straight_tb)
        if not self.is_rotated:
            # Keep the original view
            self.orig = os.path.join(self.temp_folder.path, 'orig.tiff')
            self.viewer.qimage.save(self.orig)
            self.is_rotated = True
            self.rotations = [angle]

        elif abs(angle) != 90:
            self.rotations.append(angle)
            self.is_rotated = True
            # To avoid additionnal loss of quality, we reset the original
            # image and apply the sum of the rotations
            self.viewer.set_qimage_from_backup(self.orig)

        if abs(angle) > 45 and abs(angle) != 90:
            lgg.warning('Angle must be in range -45, 45.  Get: %s' % angle)
            return

        w, h = self.viewer.get_qimage_size()
        w, h = float(w), float(h)
        img = self.viewer.get_qimage()
        matrix = QTransform()
        rimg = img.transformed(matrix.rotate(angle), Qt.SmoothTransformation)
        if rimg.isNull():
            self.show_warning('rotation', 'Unknow')
            return

        self.viewer.qimage = rimg
        self.viewer.refresh_qpixmap()

        if angle in (0, 90, -90):
            self.viewer.set_mask(False)
            return

        cr = CropRectangle(self, matrix, angle, w, h)
        rect = cr.compute_rect()
        self.viewer.set_mask(rect)
        self.straighten_mask = QRect(rect[0], rect[1], rect[2], rect[3])

    def flip_image(self, tool, direction):
        """Apply a flip transformation to the current image.

        Args:
        tool -- the Rotation-Straighten toolbox instance
        direction -- 0: vertical
                     1: horizontal
        """
        if self.is_rotated:
            return

        self.active_toolbox = tool
        img = self.viewer.get_qimage()
        rimg = img.mirrored(direction, not direction)
        self.viewer.qimage = rimg
        self.viewer.refresh_qpixmap()
        d = ("Vertical", "Horizontal")
        backup = self.historic.create_new_change(['flip', (d[direction],)])
        saved = self.set_backup(backup)
        self.historic.set_new_change(saved)
        if not saved:
            self.historic.cancel_new_change()

        else:
            self.add_backup(backup, self.image)
        self.core.filtering_finished(saved, self.active_toolbox, 
                                     self.backups[-1])
        self.core.update_navigation_viewer(rimg)

    def set_freehand_rotation_mode(self, tool, orientation):
        """Enter into the freehand straighten mode.

        Args:
        tool -- the Rotation-Straighten toolbox instance
        orientation -- 0: vertical
                       1: horizontal
        """
        self.active_toolbox = tool
        self.orientation = orientation
        self.viewer.set_freehand_linedrawer(self.apply_free_rotation)

    def apply_free_rotation(self, angle):
        lgg.info("Apply free rotation, angle: %s" %angle)
        if self.orientation:
            lgg.info('Straighten horizontal, angle: %s' % angle)
            if 90 > angle > 0:
                value = angle

            elif 270 > angle > 90:
                value = angle - 180

            else:
                 value = angle - 360

        else:
            lgg.info('Straighten vertical, angle: %s' % angle)
            if 180 > angle > 0:
                value = angle - 90

            else:
                value = angle - 270

        self.viewer.exit_tool_mode()
        self.rotate_image(value)
        self.active_toolbox.change_angle_value(value)
        self.active_toolbox.set_buttons_enabled(True)

    def apply_rotation(self):
        """Apply the rotation to the image.

        Called by the 'Apply' button of the Rotation-Straighten tool box.
        """
        self.viewer.set_mask(False)
        self.crop_image(self.straighten_mask, sum(self.rotations))
        self.is_rotated = False
        self.rotations = []
        self.core.state.freeze_tool_boxes(None, False)

    def cancel_rotation(self):
        """Abort the straigthen mode.

        Called only if the rotation was not yet saved into the historic.
        """
        self.viewer.exit_tool_mode()
        if self.is_rotated:
            self.viewer.set_qimage_from_backup(self.orig)
            self.viewer.mask = False
            self.viewer.refresh_qpixmap()

    def apply_contrast_brightness(self, values, tool):
        def process():
            ret = flt.apply_filter()
            if not ret:
                lgg.info('Contrast brightness filter Error: %s' 
                            % flt.last_error)
                self.show_warning('contrast', flt.last_error)
                self.processFailed.emit()

            else:
                self.processFinished.emit()

        self.active_toolbox = tool
        low, up, contrast, brightness = values
        source = self.get_current_image_path()
        flt = ContrastBrightnessFilter(bytes(source, 'utf-8'), None)
        gain, bias = flt.get_gain_bias(contrast, brightness)
        outf = self.historic.create_new_change(['contrast', (gain, bias)])
        flt.set_out_file_name(bytes(outf, 'utf-8'))
        flt.configure(cnt=contrast, brg=brightness, low=low, up=up)
        if not flt.load_image():
            lgg.info('Contrast filter loading image error: %s' 
                            % flt.last_error)
            self.show_warning('contrast', flt.last_error)
            self.processFailed.emit()
            return

        self.set_buzzy_widget(True)
        Thread(target=process).start()

    def apply_contrast_stretching(self, values, tool):
        lgg.info('Apply contrast stretching: lower: %s, upper: %s' % values)
        def process():
            ret = flt.apply_filter()
            if not ret:
                lgg.info('Contrast stretching filter Error: %s' 
                            % flt.last_error)
                self.show_warning('stretching', flt.last_error)
                self.processFailed.emit()

            else:
                self.processFinished.emit()

        self.active_toolbox = tool
        lower, upper = values
        width, height = self.viewer.get_qimage_size()
        pixels = width * height
        upper = (1 - upper / 100) * pixels
        lower = pixels * lower / 100
        source = self.get_current_image_path()
        outf = self.historic.create_new_change(['stretch', (lower, upper)])
        flt = ContrastStretchingFilter(bytes(source, 'utf-8'), 
                                       bytes(outf, 'utf-8'))
        flt.configure(min_=lower, max_=upper)
        if not flt.load_image():
            lgg.info('stretching filter loading image error: %s' 
                            % flt.last_error)
            self.show_warning('stretching', flt.last_error)
            self.processFailed.emit()
            return

        Thread(target=process).start()

    def apply_denoising_filter(self, args, tool):
        def process():
            if not flt.apply_filter():
                self.show_warning('Denoising', flt.last_error)
                self.processFailed.emit()

            else:
                self.processFinished.emit()

        self.active_toolbox = tool
        fname = self.get_current_image_path()
        dest = self.historic.create_new_change(['denoise', args])
        flt = DenoisingFilter(bytes(fname, 'utf-8'), 
                              bytes(dest, 'utf-8'))
        l, lc, t, s = args
        flt.configure(l_bw=l, l_color=lc, template=t, sample=s)
        if not flt.load_image():
            lgg.info('denoising filter loading image error: %s' 
                            % flt.last_error)
            self.show_warning('Denoising', flt.last_error)
            self.processFailed.emit()
            return

        Thread(target=process).start()

    def show_toning_preview(self, filter_, level):
        """Apply a filter effect to the current QPixmap.

        Args:
        filter_ -- the name of the filter
        level -- the level of the effect
        """
        lgg.info('Apply preview, filter: %s level: %s' %(filter_, level))
        if not self.preview:
            self.create_preview()

        if not os.path.isfile(self.preview):
            lgg.warning("File not found, can't show preview")
            return

        tmpfile = self.preview.replace('preview', 'toned')
        toning = Toning(bytes(self.preview, 'utf-8'), bytes(tmpfile, 'utf-8'))
        toning.configure(color=filter_, level=level)
        toning.load_image()
        pix = toning.get_preview()
        if pix:
            self.viewer.scene().clear()
            self.viewer.scene().addPixmap(QPixmap(tmpfile))
            self.is_previewed = True
            self.current_filter = (filter_, level)
            self.viewer.display_preview_label(True)
        else:
            lgg.warning("Toning preview error: %s" % toning.last_error)

    def redraw_preview(self):
        cf = self.current_filter
        lgg.info('redraw preview: %s, %s' % cf)
        if cf[0] in ('greymap', 'greyscale', 'sepia', 'blue'):
            self.preview = False
            self.show_toning_preview(cf[0], cf[1])
            self.viewer.display_preview_label(True)

    def cancel_toning_preview(self):
        lgg.info('cancel toning preview')
        self.preview = False
        self.current_filter = None
        self.is_previewed = False
        self.core.viewing.load_picture()

    def grayscale_filter(self, mode, value, tool):
        """Convert the current image to grey map.

        Args:
        mode -- 'greymap' use the common matrix
                'greyscale' use a custom matrix given by the arg value
        value -- the index of the matrix
        tool -- the instance of the toolbox
        """
        if mode == 'greymap':
            args = ('greymap', ('Standard',))

        else:
            args = ('greymap', ('Contrasted %s' % value,))

        def process_to_graymap(*args):
            toning = Toning(bytes(fname, 'utf-8'), bytes(outf, 'utf-8'))
            toning.configure(color=mode, level=value)
            toning.load_image()
            reply = toning.get_preview()
            self.processFinished.emit()

        self.active_toolbox = tool
        self.cancel_toning_preview()
        outf = self.historic.create_new_change(['greymap', (value,)])
        lgg.info('Convert to grey map, method: %s' % mode)
        fname = self.get_current_image_path()
        Thread(target=process_to_graymap, args=args).start()

    def apply_toning(self, color, contrast, tool):
        """Apply a toning on the current image.

        Args:
        color -- the color of the toning, at the moment, sepia or blue
        contrast -- boolean 0: normal
                            1: push the contrast
        tool -- the instance of the toolbox
        """
        def process_to_toning(*args):
            toning = Toning(bytes(fname, 'utf-8'), bytes(outf, 'utf-8'))
            toning.configure(color=color, level=contrast)
            toning.load_image()
            reply = toning.get_preview()
            self.processFinished.emit()

        lgg.info('Apply toning %s, contrast: %s' % (color, contrast))
        self.active_toolbox = tool
        self.cancel_toning_preview()
        outf = self.historic.create_new_change(['toning', (color, contrast)])
        fname = self.get_current_image_path()
        Thread(target=process_to_toning, args=(None,)).start()

    def apply_laplacian_filter(self, aperture, tool):
        def process():
            if not flt.apply_filter():
                self.show_warning('Denoising', flt.last_error)
                self.processFailed.emit()

            else:
                self.processFinished.emit()

        self.active_toolbox = tool
        fname = self.get_current_image_path()
        dest = self.historic.create_new_change(['laplace', (aperture,)])
        flt = Laplace(bytes(fname, 'utf-8'), bytes(dest, 'utf-8'))
        flt.configure(aperture=aperture)
        if not flt.load_image():
            lgg.info('laplacian filter loading image error: %s' 
                            % flt.last_error)
            self.show_warning('Denoising', flt.last_error)
            self.processFailed.emit()
            return

        Thread(target=process).start()

    def on_filtering_finished(self):
        self.set_buzzy_widget(False)
        self.update_current_image()

    def on_last_backup_changed(self, qimage):
        """Update some widgets when the active image has been changed.

        Args:
        qimage -- instance of the active QImage
        """
        # Reset the resizing tool box
        self.ui.toolpanel.resizing.setEnabled(True)
        self.ui.toolpanel.resizing.change_size(qimage.width(), qimage.height())
        self.ui.toolpanel.export.set_qimage(qimage)

    def update_current_image(self):
        chng, _, outf = self.historic.new
        self.active_toolbox.reset()
        #self.viewer.display_preview_label(False)

        lgg.info('Update current image: %s' % outf)
        if os.path.isfile(outf):
            self.viewer.set_qimage_from_backup(outf, 
                                self.core.current_picture.oriented)
            self.viewer.refresh_qpixmap()
            self.historic.set_new_change(True)
            self.add_backup(outf, self.image)
            self.core.filtering_finished(True, self.active_toolbox, 
                                        self.backups[-1])
            self.core.update_navigation_viewer(self.viewer.qimage)
            self.create_histogram(self.backups[-1])

    def on_filtering_failed(self):
        lgg.warning('Filtering failed!')
        self.active_toolbox.reset()
        self.historic.set_new_change(False)

    def set_backup(self, fname):
        """Create a backup of the transformed image.

        Args:
        fname -- path/name of the backup

        Return False if the backup writing has failed, True otherwise
        """
        self.ioimage.set_image_to_writer(self.viewer.get_qimage())
        self.ioimage.set_filename_to_writer(fname)
        save = self.ioimage.write_image()
        if not save[0]:
            err = ' '.join([str(save[1]), str(save[2])])
            lgg.warning("Save change error: %s" % err)
            message = {'context': _('Save current image'),
                'warn': _("Unable to write the image file:"),
                'data': fname,
                'msg': _('Reason:'),
                'data1': err,
                'btn': _('Close')}
            warn = WarningMessages(0, message)
            warn.build_dialog()
            return False

        lgg.info("Backup saved")
        return True

    def del_backup(self, path):
        try:
            os.unlink(path)
        except EnvironmentError as why:
            lgg.warning("Deleting file error: %s" % why)

        lgg.info("%s deleted" % path)

    def add_backup(self, fname, pict):
        new = Backup(self, fname, pict)
        new.change = self.historic.changes[-1]
        qimg = self.viewer.get_qimage()
        thumbnail = self.create_thumbnail(new, qimg)
        new.set_size(qimg)
        new.thumbnail = thumbnail
        self.backups.append(new)

    def create_preview(self):
        fname = self.temp_folder / 'preview.TIFF'
        self.viewer.qpixmap.save(fname.path)
        self.preview = fname.path

    def create_thumbnail(self, backup, image):
        path = self.core.utils.get_thumb_name(backup.uri)
        data = self.origin.metadata
        drawer = Thumbnailer(image, path, 1, data=data)
        drawer.draw_and_save()
        if data is not None:
            drawer.copy_metadata()

        return path

    def create_histogram(self, backup):
        def draw_histo():
            histogram.build_histogram()
        size = self.core.ui.get_histogram_size()
        fname = "".join(["histo_", backup.name, '.png'])
        file_histo = self.core.workspace['histo'] + fname
        histogram = Histogram(backup.uri, file_histo, size)
        if self.active_toolbox.name == 'LaplaceBox':
            histogram.is_grey = True
        histogram.histogramCreated.connect(self.core.display_histogram)
        histogram.brightness = True
        Thread(target=draw_histo).start()

    def undo_last_change(self, reload_=True):
        last = self.backups.pop()
        self.del_backup(last.uri)
        self.del_backup(last.thumbnail)
        self.historic.undo(1)
        if not self.backups:
            self.core.state.set_filtering_state(False)
            self.core.filtered_mode = False
            self.core.viewing.load_picture()

        elif reload_:
            self.reload_last_backup()

    def undo_changes(self, numbers):
        for i in range(numbers):
            self.undo_last_change(False)

        if self.backups:
            self.reload_last_backup()

    def undo_all_changes(self):
        self.undo_changes(len(self.backups))
        self.core.state.disable_undo_buttons(None)

    def reload_last_backup(self):
        self.core.undo_filtering(self.backups[-1])

    def get_current_image_path(self):
        return self.historic.get_last_backup() or self.origin.uri

    def set_buzzy_widget(self, state):
        cursors = (Qt.ArrowCursor, Qt.BusyCursor)
        self.viewer.setCursor(cursors[state])

    def show_warning(self, context, message):
        dct = {'context': 'Image transformation', 'msg': _('Reason:'),
               'btn': _('Close')}
        if context == "make backup":
            dct['warn'] = _('Save backup error')
            dct['data1'] = message[1]
            dct['size'] = QSize(350, 150)

        elif context == "rotation":
            dct['warn'] = 'Rotation error'
            dct['data1'] = 'Reason: %s' % message

        elif context == 'contrast':
            dct['warn'] = 'Contrast-brightness error'
            dct['data1'] = 'Reason: %s' % message

        elif context == 'stretching':
            dct['warn'] = 'Contrast stretching error'
            dct['data1'] = 'Reason: %s' % message

        warn = WarningMessages(0, dct)
        warn.build_dialog()


class Historic:
    def __init__(self, temp, view):
        """Define the image transformation historic.

        Args:
        temp -- the path of the backup folder
        view -- the historic toolbox
        """
        self.temp_fld = temp.path
        self.view = view.list_wdg
        self.changes = []
        self.last = 0
        self.new = None

    def create_new_change(self, change):
        """Prepare a new change.

        Args:
        change -- list(type of change, (param))
        """
        name = str(self.get_next_number()) + '.TIFF'
        name = os.path.join(self.temp_fld, name)
        change.append(name)
        self.new = change
        self.remove_file(name)
        return name

    def set_new_change(self, state):
        """Valid or invalid the last change.

        Args:
        state -- boolean, if True, the transformation was succeed
        """
        if state:
            self.apply_new_change()

        else:
            self.cancel_new_change()

    def apply_new_change(self):
        if self.new is not None:
            # The changes list is a stack
            self.changes.insert(0, self.new)
            self.view.add_change(self.new)
            self.new = None

    def count(self):
        return len(self.changes)

    def undo(self, numb=None):
        if numb is None:
            # Called from a tool box
            numb = 1

        for i in range(numb):
            ch = self.changes.pop(0)
            self.view.cancel_last_change()

    def reset(self):
        self.changes = []
        self.last = 0
        for i in glob.glob(self.temp_fld + '/*.*'):
            os.remove(i)

        self.view.clear_changes()

    def get_next_number(self):
        if self.last:
            self.last += 1

        else:
            self.last = int(time.time())

        return self.last

    def cancel_new_change(self):
        """Cancel the last change when the transformation has failed.

        """
        self.new = None

    def get_backups(self):
        """Returns the list of the backups.

        The list is ordered from most recent to oldest.
        Backups are full paths.
        """
        return [i[2] for i in self.changes]

    def get_last_backup(self):
        if self.changes:
            return self.changes[0][2]

        return False

    def remove_file(self, fname):
        """Remove a file from the disk.

        Args:
        fname -- the full path of the file

        Return:
        The error if the deletion failed, None otherwise or if the file
        doesn't exist.
        """
        if os.path.isfile(fname):
            try:
                os.remove(fname)
            except Exception as why:
                lgg.warning("Can't remove backup: %s" % why)


class Backup:
    def __init__(self, driver, path, origin):
        self.driver = driver
        b = os.path.basename(path)
        n, e = os.path.splitext(b)
        self.uri = path
        self.base = b
        self.name = n
        self.ext = e
        self.weight = os.path.getsize(path)
        self.origin = origin
        self.change = None
        self.thumbnail = None
        self.data = None
        self.metadata = None
        self.oriented = 0
        self.orientation = 1
        self.rotation = False

    def set_size(self, qimage):
        self.width = qimage.width()
        self.height = qimage.height()

    def set_metadata(self, dest):
        if self.origin.metadata is None:
            return

        # TODO Update metadata
        data = self.origin.metadata
        data.export_data(dest)
        d = Metadata(dest)
        if d.get_data():
            self.metadata = d

    def export(self, fname, thumbnail, data):
        ioimage = IOImage()
        ioimage.set_image_to_writer(data['qimage'])
        # Save the image with a temporary name
        temp = os.path.join(data['dest'], self.name + data['format'])
        ioimage.set_filename_to_writer(temp)
        lgg.info('Writing: %s' % temp)
        save = ioimage.write_image()
        if not save[0]:
            err = ' '.join(str(save[1]), str(save[2]))
            lgg.warning("Save change error: %s" % err)
            message = {'context': _('Save current image'),
                'warn': _("Unable to write the image file:"),
                'data': temp,
                'msg': _('Reason:'),
                'data1': err,
                'btn': _('Close')}
            warn = WarningMessages(0, message)
            warn.build_dialog()
            return False

        if data['data']:
            self.set_metadata(temp)

        try:
            shutil.copyfile(temp, fname)
        except EnvironmentError as why:
            lgg.warning("Copy file error: %s" % why)
            lgg.warning("Save change error: %s" % err)
            message = {'context': _('Save current image'),
                'warn': _("Unable to write the image file:"),
                'data': fname,
                'msg': _('Reason:'),
                'data1': why,
                'btn': _('Close')}
            warn = WarningMessages(0, message)
            warn.build_dialog()
            return False

        try:
            os.remove(temp)
        except EnvironmentError as why:
            lgg.warning("Remove temp file '%s' failed\n  Reason: %s" 
                        %(temp, why))

        try:
            shutil.copyfile(self.thumbnail, thumbnail)
        except Exception as why:
            lgg.warning("Copy thumbnail error: %s" % why)


    def set_rotation(self, angle):
        """Save the new orientation of the backup.

        This happens when the filtered image is rotated to left or right.

        Args:
        angle -- angle of rotation, unusued here
        """
        if self.driver.set_backup(self.uri):
            self.driver.create_thumbnail(self, self.uri)

class CropRectangle:
    """Define the geometry of a rectangle drawn into an other rectangle.

    """
    def __init__(self, changes, matrix, angle, width, height):
        """Init the instance.

        Args:
        changes -- the instance of ImageChanges
        matrix -- the instance of QMatrix used for the rotation of the image
        angle -- the angle of rotation
        width -- width of the original image
        height -- height of the original image
        """
        self.changes = changes
        self.matrix = matrix
        self.angle = angle
        self.width = width
        self.height = height

    def compute_rect(self):
        rad = radians(self.angle)
        if self.angle < 0:
            # Rotate counterclockwise
            if self.angle > -25:
                return self.compute(abs(rad))

            return self.compute_4(abs(rad))

        # Rotate clockwise
        if self.angle < 24:
            return self.compute_2(abs(rad))

        return self.compute_3(abs(rad))

    def compute(self, angle):
        """Compute the largest rectange inserted into a rectange.

        The outer rectangle is rotated into -1, -24 degrees

        Args:
        angle -- the angle value in radians
        """
        downx, downy = self.matrix.map(0, self.height)
        upx, upy = self.matrix.map(self.width, 0)
        dc = (self.width - downx) * tan(angle)
        c = downy - dc
        ax = c * tan(angle)
        a = ax * tan(angle)
        ay = abs(upy) - a
        cx = self.width - ax
        cy = c + a

        return ax, ay, cx, cy

    def compute_4(self, angle):
        """Compute the largest rectange inserted into a rectange.

        The outer rectangle is rotated into -25, -45 degrees

        Args:
        angle -- the angle value in radians
        """
        upx, upy = self.matrix.map(self.width, 0)
        downx, downy = self.matrix.map(0, self.height)

        if self.width >= self.height:
            f = self.width / (1 + self.width / self.height)
            ax = f * cos(angle)
            ay = f * sin(angle)
            f = self.width - f
            cx = f * cos(angle) + downx - ax
            cy = downy - (f * sin(angle)) + ay
            ay = abs(upy) - ay

        else:
            f = self.height / (1 + self.height / self.width)
            g = self.height - f
            dx = g * sin(angle)
            dy = g * cos(angle)
            ax = dx
            bx = upx + (f * sin(angle))
            ay = f * cos(angle)
            cx = bx - ax
            cy = dy + abs(upy) - ay

        return ax, ay, cx, cy

    def compute_2(self, angle):
        """Compute the largest rectange inserted into a rectange.

        The outer rectangle is rotated into 1, 24 degrees

        Args:
        angle -- the angle value in radians
        """
        leftx, lefty = self.matrix.map(0, self.height)
        ay = (self.width + leftx) * tan(angle)
        ax = abs(leftx) - (ay * tan(angle))
        cx = self.width - ax
        cy = lefty - ay + (abs(ax) * tan(angle))

        return ax, ay, cx, cy

    def compute_3(self, angle):
        """Compute the largest rectange inserted into a rectange.

        The outer rectangle is rotated into 24, 45 degrees

        Args:
        angle -- the angle value in radians
        """
        leftx, lefty = self.matrix.map(0, self.height)
        downx, downy = self.matrix.map(self.width, self.height)
        rightx, righty = self.matrix.map(self.width, 0)
        dwx = downx + abs(leftx)

        if self.width >= self.height:
            # Landscape
            f = self.width / (1 + self.width / self.height)
            g = self.width - f
            ax = f * cos(angle)
            ay = g * sin(angle)
            cx = abs(leftx) + (g * cos(angle)) - ax
            cy = lefty + (f * sin(angle)) - ay

        else:
            f = self.height / (1 + self.height / self.width)
            g = self.height - f
            ax = abs(leftx) - (f * sin(angle))
            ay = f * cos(angle)
            cx = abs(leftx) + rightx - (g * sin(angle)) - ax
            cy = righty + (g * cos(angle)) - ay

        return ax, ay, cx, cy      
