# -*- coding: utf-8 -*-

# geolocation.py
# 
# This file is part of Oqapy serie 3
#
# Author: Vincent Vande Vyvre <vincent.vandevyvre@oqapy.eu>
# Home: http://www.oqapy.eu
# Copyright: 2009-2017 Vincent Vande Vyvre
# Licence: GPL3
#
# Geolocation module

import os
import glob
import time as _time
import math
import json
import logging
lgg = logging.getLogger(__name__)

from xml.sax import handler
from xml.sax import make_parser
from xml.sax.handler import feature_namespaces
from fractions import Fraction
from threading import Thread, Timer
from datetime import datetime, timedelta, date, time

import pyexiv2

from PyQt5.QtCore import QCoreApplication, QObject, pyqtSignal
from PyQt5.QtWidgets import QFileDialog

from metadata.metadata import GeoTag
from g9n.ui_geolocation import Ui_Geolocation
from warningmessages import WarningMessages


class Geolocation(QObject):
    parsingFinished = pyqtSignal()
    def __init__(self, core, images, tmp, loc, utc):
        """class for handle geographical coordinates in picture's metadata.

        Args:
        core -- Oqapy instance
        images -- the list of full path of images
        tmp -- the path of the temp folder ~.oqapy/g9n/
        loc -- the default coordinates found in user's prefs
        utc -- the last setting of gmt offset of the camera
        """
        super().__init__()
        lgg.info('Instanciate g9n dialog')
        self.core = core
        self.image_files = images
        self.temp = tmp
        self.camera_utc = utc
        self._is_alive = True
        self.parsingFinished.connect(self.display_data)
        self.dial = Ui_Geolocation(self, loc, utc)
        self.gpx_file = None
        self.tracks = None
        self.current_track = None
        self.limits = None
        self.start_at = 0
        self.clock = Clock(self.dial)
        self.dial.show()
        QCoreApplication.processEvents()
        self.dial.setup_widgets()

        self.dial.exec_()
        self.clock._is_alive = False
        self.clean_temp_folder()

    def load_gpx(self):
        """Launch open file dialog box.

        """
        lgg.info('Browse to GPX file')
        self.set_message(1)
        title = _("Open gpx file")
        folder = self.get_last_gpx_dir()
        options = QFileDialog.Options()
        fname = QFileDialog.getOpenFileName(None, title, folder,
                                "GPX Files (*.gpx)", None, options)[0]

        if fname:
            self.dial.gpx_fname_led.setText(fname)
            self.update_config(os.path.dirname(fname))
            self.read_gpx()

    def get_last_gpx_dir(self):
        folder = None
        try:
            folder = self.core.cfg.get(("g9n", "gpx_dir"))
        except KeyError:
            pass

        if folder is None:
            folder = os.path.expanduser("~")
            self.update_config(folder)

        return folder

    def update_config(self, path):
        self.core.cfg.update(("g9n", "gpx_dir"), path)

    def read_gpx(self):
        """Set the gpx instance.

        Set the gpx instance, verify the validity and version of file.
        Versions 1.0 and 1.1 are supported.
        Show a message if file is not readable.
        """
        self.parsed = False
        def parse_data(fn):
            parser.parse(file_gpx)
            self.parsed = True

        file_gpx = self.dial.gpx_fname_led.text()
        lgg.info('Read gpx file')
        if not os.path.isfile(file_gpx):
            lgg.warning('No such file: {0}'.format(file_gpx))
            return

        if self.gpx_file == file_gpx:
            return

        self.dial.set_rolling_progress(True)
        self.set_message(2)
        parser = make_parser()
        parser.setFeature(feature_namespaces, 0)
        self.gp = GpxParser(self)
        ge = GpxError(self.gp)
        parser.setContentHandler(self.gp)
        parser.setErrorHandler(ge)
        Thread(target=parse_data, args=(file_gpx,)).start()

    def display_data(self):
        data = self.set_tracks_list(self.gp.tracks)
        self.dial.tracks_tbw.display_tracks(data)
        self.dial.message_lbl.setText('')
        self.dial.info_track_lbl.setText('Found %s tracks' % len(data))
        lgg.info('Found %s tracks' % len(data))
        self.dial.set_rolling_progress(False)

    def on_track_selected(self, track):
        if not track:
            self.dial.enable_time_widgets(False)
            self.dial.enable_other_time_widgets(False)
            self.dial.reset_preview(False)
            self.dial.info_track_lbl.setText('')
            self.dial.message_lbl.setText('')
            self.current_track = None
            return

        self.dial.enable_time_widgets(True)
        self.dial.reset_preview(True)
        self.current_track = track
        self.dial.info_track_lbl.setText("Track `%s` has %s coordinate's points"
                                            % (track, self.tracks[track][-1]))
        self.update_map(track)
        lgg.info('Track selected: %s' % track)
        self.set_message(6)

    def update_map(self, track):
        coord = self.tracks[track]
        self.dial.map.go_to(coord[1], coord[2], 18)

    def set_tracks_list(self, tracks):
        """Create the list of track for the tableView

        'track-1': (u'2012-12-19T09:46:11.837Z', u'50.831425815', 
                    u'4.385337634', u'165.877106')

        Returns:
        list[[name, desc, start date, lat, lon], [...]]
        """
        self.tracks = tracks
        lst = []
        self.names = self.sort_(tracks.keys())
        for idx, name in enumerate(self.names):
            tk = []
            tk.append(name)
            tk.append(None)
            tk.append(tracks[name][0])
            tk.append(tracks[name][1])
            tk.append(tracks[name][2])
            lst.append(tk)

        return lst

    def get_track_limits(self):
        """Define min & max datetime in gps trace.

        Returns:
        tuple of date (old, recent)
        """
        return (self.points[0][0], self.points[-1][0])

    def set_viewer_widgets(self, b):
        self.dial.label_1.setEnabled(b)
        self.dial.previous_btn.setEnabled(b)
        self.dial.next_btn.setEnabled(b)
        self.dial.label_2.setEnabled(b)
        self.dial.apply_pict_btn.setEnabled(b)

    def sort_images(self):
        """Create the list of valid images found into the current folder.

        A file is valid if his original date is into the range (start time -
        end time) of the track.
        """
        if self.current_track is None:
            self.on_track_selected(False)
            return

        self.points = self.get_trackpoints()['points']
        if not self.points:
            return

        self.dial.progress_pgb.setValue(0)
        self.set_message(8)
        lgg.info('Sort the valid images')
        time_dif = self.dial.time_delta
        if time_dif is None:
            limits = self.get_large_limits()

        else:
            limits = self.get_exact_limits(time_dif)

        self.select_files(limits)

    def get_large_limits(self):
        """Returns the time limits of a track.

        No utc was provided by the user, we add 12 hours before and after the
        first and the last date stamp of the track.

        Returns:
        (datetime start, datetime end)
        """
        l = self.get_track_limits()
        offset = timedelta(hours=12)
        start = self.iso_to_datetime(l[0]) - offset
        end = self.iso_to_datetime(l[1]) + offset
        return start, end

    def get_exact_limits(self, offset):
        """Returns the time limits of a track.

        The utc offset provided by the user is applied.

        Returns:
        (datetime start, datetime end)
        """
        l = self.get_track_limits()
        start = self.iso_to_datetime(l[0]) + offset
        end = self.iso_to_datetime(l[1]) + offset
        return start, end

    def sort_(self, lst):
        """Sort a list in alphabetical order.

        This method is needed for the lists that have items wich contents
        numerical characters.

        Arg:
        lst -- unordered list

        Returns:
        sorted list
        """
        lst = sorted(lst)
        dct = {}
        for item in lst:
            ln = len(item)
            if ln not in dct:
                dct[ln] = []
            dct[ln].append(item)

        keys = sorted(dct.keys())
        out = []
        for key in keys:
            out.extend(dct[key])

        return out

    def select_files(self, limits):
        valids = {}
        step = 100 / len(self.image_files)
        p_val = 0
        for image in self.image_files:
            geo = GeoTag(image)
            dat = geo.get_geo_data()
            # dat = [date, lat, latref, long, longref, alt, altref, comments]
            p_val += step
            self.dial.progress_pgb.setValue(p_val)
            if dat[0] is None:
                continue

            if dat[0] >= limits[0] and dat[0] <= limits[1]:
                valids[dat[0]] = image

        if not valids:
            self.on_empty_selection()
            return

        self.images = [(k, valids[k]) for k in sorted(valids.keys())]
        self.dial.viewer.images = [i[1] for i in self.images]
        self.dial.viewer.load_image(0)
        self.start_at = 0
        self.update_info_widgets(len(self.images))
        lgg.info('Found %s valid images' % len(self.images))

    def view_changed(self, idx):
        self.show_image_prop(idx)
        self.dial.select_btn.setEnabled(True)

    def file_rejected(self, idx):
        self.images.pop(idx)

    def on_empty_selection(self):
        self.dial.progress_pgb.setValue(0)
        self.dial.apply_btn.setEnabled(False)
        self.set_message(12)
        self.dial.viewer.display_no_images()

    def show_image_prop(self, i):
        """Display the properties of the current image.

        Args:
        p -- properties of image; name, date and time       
        """
        prop = self.get_image_prop(self.images[i])
        txt = prop[4] + ' - '
        txt += prop[1].strftime('%A %d %B %Y') + ' '
        txt += prop[2].strftime('%H:%M:%S')

        self.dial.picture_name_lbl.setText(txt)

    def get_image_prop(self, i):
        """Returns the properties of the current image shown into the preview.

        Args:
        i -- tuple(datetime, file path)

        Returns:
        tuple (datetime.datetime(), datetime.date(), 
                datetime.time(), path, basename)
        """
        dt = i[0]
        d = date(dt.year, dt.month, dt.day)
        t = time(dt.hour, dt.minute, dt.second)
        p = i[1]
        n = os.path.split(p)[1]
        return (dt, d, t, p, n)

    def set_first_image(self):
        """Select the first image.

        """
        self.start_at = self.dial.viewer.current_view
        self.dial.apply_btn.setEnabled(True)
        lgg.info('Set the first image: %s' % self.start_at)

    def tag_images(self):
        """Clean up the list of images and call tagging process.

        """
        if not self.images:
            return

        self.set_message(13)
        lgg.info('Process to the tagging')
        utc = self.get_utc(self.images[self.start_at][0], self.points[0][0])
        points = self.convert_date_points(utc)
        self.tag = Tagging(self, self.images[self.start_at:], points)
        self.set_message(14)
        self.tag.localise_images()
        self.dial.apply_btn.setEnabled(False)
        self.set_message(15)

    def get_delta_camera_to_utc(self):
        """Returns the difference from camera time to GMT.

        """
        return self.dial.time_delta

    def get_utc(self, di, ds):
        """Return the time offset camera to GMT.

        Args:
        di -- date of the image, type datetime
        ds -- stampdate of coordinate point, type ISO

        Returns:
        timedelta(di - ds)
        """
        if self.dial.time_delta is None:
            return di - self.iso_to_datetime(ds)

        return self.dial.time_delta

    def convert_date_points(self, utc):
        """Convert the datetime iso to Python date and apply the utc offset.

        Args:
        utc -- time offset camera to GMT

        Returns:
        list of points
        """
        news = []
        for pt in self.points:
            d = self.iso_to_datetime(pt[0])
            dt = d + utc
            news.append((dt, float(pt[1]), float(pt[2]), float(pt[3])))

        return news

    def iso_to_datetime(self, iso):
        """Convert a date ISO to datetime.datetime.

        Convert an ISO formatted string to datetime.datetime,
        i.e. 2009-04-21T14:14:37Z  >> datetime(2009, 04, 21, 14, 14, 37)

        Args:
        iso -- date ISO formatted

        Returns:
        datetime.datetime object.
        """
        yr = int(iso[0:4])
        mo = int(iso[5:7])
        da = int(iso[8:10])
        hr = int(iso[11:13])
        mi = int(iso[14:16])
        se = int(iso[17:19])
        return datetime(yr, mo, da, hr, mi, se)

    def get_trackpoints(self):
        fname = os.path.join(self.temp, self.current_track + '.trk')
        try:
            with open(fname, 'r') as inf:
                cnt = inf.read()
                return json.loads(cnt)
        except Exception as why:
            lgg.warning("Can't read track file: %s" % fname)
            message = {'context': _('Reading track file'),
                'warn': _("Can't read the track file:"),
                'data': fname,
                'msg': _('Reason:'),
                'data1': str(why),
                'btn': _('Close')}
            warn = WarningMessages(0, message)
            warn.build_dialog()
            return False

    def update_info_widgets(self, numb):
        self.dial.progress_pgb.setValue(0)
        self.set_message(10, numb)
        if not self.dial.choose_rdo.isChecked():
            self.dial.apply_btn.setEnabled(True)

        else:
            self.set_message(11, numb)

    def set_message(self, message=0, val=None):
        """Show a message at bottom of dialog.

        Args:
        message -- message
        """
        if message == 1:
            m = ("Browse to ...")

        if message == 2:
            m = ("Reading .gpx file ...")

        elif message == 5:
            m = ("Found %s tracks" % (val))

        elif message == 6:
            m = ("Choose the time setting and clik on the Sort button.")

        elif message == 8:
            m = ("Searching images ...")

        elif message == 10:
            m = ("Found %s images" % (val))

        elif message == 11:
            m = ("Found %s images. Select the first picture." % (val))

        elif message == 12:
            m = ('No images found')

        elif message == 13:
            m = ('Preparing geographical points ...')

        elif message == 14:
            m = ('Writing geotag ...')

        elif message == 15:
            m = ('Finished')

        else:
            return

        self.dial.message_lbl.setText(m)
        QCoreApplication.processEvents()

    def clean_temp_folder(self):
        pattern = os.path.join(self.temp, 'track*.trk')
        fls = glob.glob(pattern)
        for f in fls:
            try:
                os.remove(f)
            except:
                lgg.warning("Can't remove %s" % f)


class GpxParser(handler.ContentHandler):
    def __init__(self, gl):
        self.gl = gl
        self.temp = gl.temp
        self.gpx_version = None
        self.current = 0
        self.current_track_seg = "track_seg_0"
        self.points = []
        self.begin_track = False
        self.track_name = False
        self.track_desc = ''
        self.in_track_name = False
        self.in_track_desc = False
        self.in_track_point = False
        self.in_elev = False
        self.elev_value = False
        self.in_time = False
        self.time_value = False
        self.pos = False
        self.tracks = {}
        self.descs = {}

    def update_segment_name(self):
        self.current += 1
        self.current_track_seg = "track_seg_%s" % self.current

    def startElement(self, name, attrs):
        if name == 'trkseg':
            self.begin_track = True

        elif name == 'trkpt':
            self.in_track_point = True
            self.pos = (attrs.get('lat', None), attrs.get('lon', None))

        elif name == 'ele' and self.in_track_point:
            self.in_elev = True
            self.elev_value = ""

        elif name == 'time' and self.in_track_point:
            self.in_time = True
            self.time_value = ""

        if all([self.pos, self.elev_value, self.time_value]):
            self.points.append((self.time_value, self.pos[0], self.pos[1], 
                                self.elev_value))
            self.pos = self.elev_value = self.time_value = False

    def endElement(self, name):
        if name == 'trkseg':
            if self.points:
                curr = self.current_track_seg
                self.tracks[curr] = list(self.points[0])
                self.tracks[curr].extend([self.points[-1][0], 
                                                      len(self.points)])
                self.save_current_track()
                self.update_segment_name()
            self.points = []

        elif name == 'ele':
            self.in_elev = False

        elif name == 'time':
            self.in_time = False

    def endDocument(self):
        self.begin_track = False
        self.track_name = False
        self.in_track_name = False
        self.in_elev = False
        self.alt_value = False
        self.in_time = False
        self.time_value = False
        self.pos = False
        self.gl.parsingFinished.emit()

    def normalize_whitespace(self, text):
        return ' '.join(text.split())

    def characters(self, ch):

        if self.in_elev:
            self.elev_value += ch

        if self.in_time:
            self.time_value += ch

    def save_current_track(self):
        dct = {'name': self.current_track_seg, 'points': self.points}
        fname = self.get_track_filename()
        jsn = json.dumps(dct, indent=4, 
                         separators=(',', ': '), ensure_ascii=False)
        try:
            with open(fname, 'wb') as outfile:
                outfile.write(jsn.encode('utf-8', 'replace'))
        except Exception as why:
            lgg.warning('write error: %s' % why)
            message = {'context': _('Writing GPX file'),
                'warn': _("Can't write the .gpx file:"),
                'data': fname,
                'msg': _('Reason:'),
                'data1': str(why),
                'btn': _('Close')}
            warn = WarningMessages(0, message)
            warn.build_dialog()

    def get_track_filename(self):
        return os.path.join(self.temp, self.current_track_seg + '.trk')


class GpxError(handler.ErrorHandler):
    def __init__(self, parser):
        self.parser = parser

    def error(self, exception):
        if self.parser.in_track_point:
            self.parser.pos = False
            self.parser.endElement('trkpt')

    def fatalError(self, why):
        lgg.warning('Fatal error when reading GPX file: %s' % why)
        message = {'context': _('Reading GPX file'),
            'warn': _("Can't read the .gpx file"),
            'data': None,
            'msg': _('Reason:'),
            'data1': str(why),
            'btn': _('Close')}
        warn = WarningMessages(0, message)
        warn.build_dialog()


class Tagging:
    def __init__(self, gl, imgs, locs):
        self.images = imgs
        self.points = locs
        # Need one more point for the last image
        self.points.append(locs[-1])
        self.progress = gl.dial.progress_pgb
                
    def localise_images(self):
        """Main function of geotagging.

        """
        self.progress.setRange(0, len(self.images))
        npoints = len(self.points) - 1
        locate = 0
        mid = self.get_middle_point(0)
        for idx, image in enumerate(self.images):
            dt, fname = image
            while locate < npoints:
                if dt <= mid:
                    self.tag_image(fname, self.points[locate])
                    self.progress.setValue(idx)
                    break

                else:
                    locate += 1
                    mid = self.get_middle_point(locate)

        self.progress.setValue(idx+1)

    def get_middle_point(self, idx):
        """Returns the middle time of two datetime.

        Args:
        idx -- index of the first trackpoint

        Returns:
        datetime
        """
        t1 = self.points[idx][0]
        try:
            t2 = self.points[idx+1][0]
            return t1 + (t2 - t1) / 2
        except:
            return t1

    def tag_image(self, img, point):
        """Fit the data in exif format and write it in image file.

        """
        stamp, lat, lon, alt = point
        latref, lonref, altref = "N", "E", "0"

        # Fit data
        # Latitude & longitude are expressed as three RATIONAL values giving 
        # the degrees, minutes, and seconds, respectively.
        lt = self.decimal_to_dms(lat)
        if lt[0] < 0:
            lt[0] *= -1
            latref = "S"

        lat = [self.int_to_rational(lt[0]),
                    self.int_to_rational(lt[1]),
                    self.float_to_rational(lt[2])]

        lg = self.decimal_to_dms(lon)
        if lg[0] < 0:
            lg[0] *= -1
            lonref = "W"

        lon = [self.int_to_rational(lg[0]),
                    self.int_to_rational(lg[1]),
                    self.float_to_rational(lg[2])]

        # Altitude is expressed as one RATIONAL value.
        if alt < 0:
            alt *= -1
            altref = "1"

        elev = self.float_to_rational(alt)

        # DateStamp is expressed as string "YYYY:MM:DD"
        # TimeStamp is expressed as three RATIONAL values giving the hour, 
        # minute and second. 
        dt = stamp.date()
        date_ = "%s:%s:%s" %(dt.year, dt.month, dt.day)
        tm = stamp.time()
        time = [self.int_to_rational(tm.hour),
                    self.int_to_rational(tm.minute),
                    self.int_to_rational(tm.second)]

        geo = GeoTag(img)
        geo.write_geo_data([lat, latref, lon, lonref, elev, altref, 
                            date_, time])
        
    def decimal_to_dms(self, d):
        """Convert decimal degrees into degrees, minutes, seconds.

        Args:
        d -- float(degrees)
        """
        i, d = math.modf(d)
        j, m = math.modf(i*60)
        s = j*60
        return [int(d), int(abs(m)), abs(s)]

    def int_to_rational(self, val):
        """Convert integer to rational.

        Args:
        val -- int()
        """
        return Fraction(val, 1)
    
    def float_to_rational(self, val):
        """Create a pyexiv2.Rational with help from fractions.Fraction.

        Args:
        val -- float()
        """
        # pyexiv2 don't support too big denominator
        return Fraction.from_float(val).limit_denominator(9999)


class Clock:
    def __init__(self, gui):
        super().__init__()
        self.gui = gui
        self._is_alive = True

        def _set_time():
            utc = datetime.utcnow().strftime('%A %d %B %Y, %H:%M:%S')
            utc = " " + utc
            lcl = datetime.today().strftime('%A %d %B %Y, %H:%M:%S')
            lcl = " " + lcl
            self.gui.gmt_val_lbl.setText(utc)
            self.gui.local_val_lbl.setText(lcl)
            if self._is_alive:
                timer = Timer(1, _set_time)
                timer.start()

        _set_time()

