# mapviewer.py
# 
# This file is part of Oqapy serie 3
#
# Author: Vincent Vande Vyvre <vincent.vandevyvre@oqapy.eu>
# Home: http://www.oqapy.eu
# Copyright: 2009-2017 Vincent Vande Vyvre
# Licence: GPL3
#
# QWebView for geolocation

import os

from threading import Thread
import logging
lgg = logging.getLogger(__name__)

from PyQt5.QtCore import pyqtSignal, QUrl, Qt
from PyQt5.QtWebKit import QWebSettings
from PyQt5.QtWebKitWidgets import QWebView, QWebPage

# QtWebEngine is not available for Debian and derived
# https://marc.info/?l=kde-core-devel&m=142954900813235&w=2
# Ubuntu bug #1579265
#from PyQt5.QtWebEngineWidgets import QWebEngineSettings
#from PyQt5.QtWebEngineView import QWebEngineView, QWebEnginePage

#from g9n.htmlwriter import HtmlWriter


class MapViewer(QWebView):
    """Define a web page used for the image's geolocation.

    """
    coordinateChanged = pyqtSignal(list)
    mapServerChanged = pyqtSignal(int)
    def __init__(self, parent=None):
        super().__init__(parent)
        self.zoom = 8
        self.is_enable = False
        self.server = 1
        self.setMouseTracking(False)
        self.current_coord = (0, 0)
        self.page_ = WebPage()
        self.setPage(self.page_)
        self.settings().setAttribute(QWebSettings.PluginsEnabled, True)
        self.loadFinished.connect(self.on_mapserver_changed)
        self.page_.linkClicked.connect(self.link_clicked)

    def _setup(self, coord, fld):
        coord = [str(c) for c in coord]
        self.current_coord = coord
        here = os.getcwd()
        ol = os.path.join(here, 'g9n/osm.html')
        gm = os.path.join(here, 'g9n/gm.html')
        # Google Map removed because API Key
        self.urls = [QUrl.fromLocalFile(ol), QUrl.fromLocalFile(ol)]
        self.change_server()
        self.is_enable = True

    def set_url(self, url=None):
        if not self.isVisible():
            # At startup, the webView is not visible, this may cause a crash
            # of the program if we try to set the map.
            # With Google, this problem occurs only at startup,
            # with OpenStreetMap it occurs always.
            return

        try:
            # At startup the width must be fixed, now we can release it
            self.parent().webView.setMaximumSize(9999, 9999)
        except AttributeError:
            # Geolocation dialog case
            pass

        if url is None:
            url = self.url_

        self.setUrl(url)
        self.url_ = url

    def change_server(self):
        """Change the map server OpenStreetMap <==> GoogleMap.

        """
        lgg.info('Change map server: %s' % int(not self.server))
        self.page_.is_visible = False
        self.server = not self.server
        self.url_ = self.urls[self.server]
        self.set_url(self.url_)
        try:
            lat, lon = self.current_coord
            self.go_to(lat, lon, 16)
        except ValueError:
            # No coordinates at startup
            pass

    def set_coordinate_from_picture(self, picture):
        """Update the map's coordinates.

        Args:
        picture -- current Picture instance
        """
        loc = picture.locate
        if loc and not isinstance(loc[0], str):
            self.go_to(loc[0], loc[1], 16)
            self.parent().set_locked(True)

        else:
            self.parent().set_locked(False)

    def go_to(self, lat, lon, zoom=None):
        """Update the map's coordinates.

        Args:
        lat -- float(latitude)
        lon -- float(longitude)
        zoom -- int(zoom factor)
        """
        if zoom is not None:
            self.zoom = zoom

        if not self.url():
            # This is the case at startup
            self.set_url()

        if self.url() not in self.urls:
            self.set_url(self.url_)

        lgg.info('Set coordinate: {0}, {1}, {2}'.format(lat, lon, self.zoom))
        self.current_coord = (lat, lon)

        self.page_.mainFrame().evaluateJavaScript(
                'centerOnLatLon({0}, {1}, {2});'.format(lat, lon, self.zoom))

        self.get_coordinate()
        self.page_.is_visible = True

    def fly_to(self, lat, lon, zoom):
        """Update the map's coordinates with the effect 'Fly-To'.

        Used only with OpenLayers.

        Args:
        lat -- float(latitude)
        lon -- float(longitude)
        zoom -- int(zoom factor)
        """
        if not self.server:
            lgg.info('Fly to: {0}, {1}, {2}'.format(lat, lon, self.zoom))
            self.current_coord = (lat, lon)
            self.page_.mainFrame().evaluateJavaScript(
                'flyTo({0}, {1});'.format(lat, lon))

        else:
            self.go_to(lat, lon, zoom)

    def update_map(self):
        """Center the map to the current coordinate.

        """
        if self.current_coord is not None:
            lat, lon = self.current_coord[0], self.current_coord[1]
            if lat:
                self.go_to(lat, lon, self.zoom)

    def on_mapserver_changed(self):
        self.mapServerChanged.emit(self.server)
        self.update_map()

    def link_clicked(self, link):
        # Avoid inappropriate navigation
        if link not in self.urls:
            self.set_url(self.url_)

    def get_coordinate(self):
        frame = self.page_.mainFrame()
        evl = frame.evaluateJavaScript("updateLatLonFields();")
        if evl is not None:
            if self.server:
                if evl is not None:
                    i = evl.split(',')
                    if len(i) > 1:
                        self.set_coordinate(i)

            else:
                coord = [str(evl[1]), str(evl[0]), evl[2]]
                self.set_coordinate(coord)

    def set_coordinate(self, coord):
        lat = coord[0]
        if len(lat) > 8:
            lat = lat[:8]

        lon = coord[1]
        if len(lon) > 8:
            lon = lon[:8]

        try:
            self.zoom = coord[2]
        except IndexError:
            self.zoom = 8
            coord.append(self.zoom)

        self.current_coord = coord
        self.coordinateChanged.emit(coord)

    def mouseDoubleClickEvent(self, event):
        QWebView.mouseDoubleClickEvent(self, event)
        self.get_coordinate()

    def mouseReleaseEvent(self, event):
        QWebView.mouseReleaseEvent(self, event)
        self.get_coordinate()

    def wheelEvent(self, event):
        QWebView.wheelEvent(self, event)
        self.get_coordinate()

    def resizeEvent(self, event):
        QWebView.resizeEvent(self, event)

class WebPage(QWebPage):
    def __init__(self, parent=None):
        super().__init__(parent)
        self.LinkDelegationPolicy(QWebPage.DelegateExternalLinks)
        self.WebAction(QWebPage.NoWebAction)
        self.mainFrame().setScrollBarPolicy(Qt.Vertical, 
                                            Qt.ScrollBarAlwaysOff)
        self.settings().setAttribute(QWebSettings.LocalContentCanAccessRemoteUrls, True)
        self.settings().setAttribute(QWebSettings.JavascriptEnabled, True)
        self.settings().setAttribute(QWebSettings.JavaEnabled, True)
        self.is_visible = False

    def javaScriptConsoleMessage(self, msg, line, source):
        if self.is_visible:
            lgg.warning('%s line %d: %s' % (source, line, msg))
