# -*- coding: utf-8 -*-

# globalmenu.py
# This file is part of Oqapy serie 3
#
# Author: Vincent Vande Vyvre <vincent.vandevyvre@oqapy.eu>
# Home: http://www.oqapy.eu
# Copyright: 2009-2018 Vincent Vande Vyvre
# Licence: GPL3
#
# Application's menus

import logging
lgg = logging.getLogger(__name__)

from PyQt5.QtCore import Qt, QObject, pyqtSignal, QSize, QPoint, QTimer
from PyQt5.QtGui import QPixmap, QIcon
from PyQt5.QtWidgets import (QGridLayout, QVBoxLayout, QHBoxLayout, QToolButton, 
                             QLabel, QWidget, QMenu, QFrame, QSizePolicy, )

import stylesheets as styles

class GlobalMenu(QObject):
    """Define the global menu.

    """
    def __init__(self, core):
        """Instantiate the global menu.

        Args:
        core -- the main oqapy class instance
        """
        super().__init__()
        self.core = core
        self.ui = core.ui
        self.widget = PopupMenu(self, self.ui)
        l = self.widget.layout
        self.width = l.columnCount() * 85
        self.height = 7 * 50
        self.accept_raw_files = False
        self.raw_only = False

    def display_menu(self, posx, posy, context):
        """Show the menu.

        Args:
        posx -- coordinate x
        posy -- coordinate y
        """
        self.context = context
        anchor = QPoint(posx-self.width, posy-self.height)
        self.widget.set_context(context)
        self.widget.popup(anchor)

    def configure(self):
        """Create the signal's connections.

        """
        w = self.widget
        cfg = self.core.cfg.data
        w.source.clicked.connect(self.on_source_clicked)
        w.import_.clicked.connect(self.on_import_clicked)
        w.print_.clicked.connect(self.on_print_clicked)
        w.search.clicked.connect(self.on_search_clicked)
        w.geoloc.clicked.connect(self.on_geotag_clicked)
        w.magnifier.clicked.connect(self.on_magnifier_clicked)
        w.grid.clicked.connect(self.on_grid_clicked)
        w.aerial.clicked.connect(self.on_aerial_clicked)
        w.trailer.setChecked(cfg['startup']['trailer'])
        w.trailer.setEnabled(False)
        w.trailer.clicked.connect(self.on_trailer_clicked)
        w.toolpanel.setChecked(cfg['startup']['panel'])
        w.toolpanel.clicked.connect(self.on_toolpanel_clicked)
        w.scrollbars.setChecked(cfg['style']['scrlbars'])
        w.scrollbars.clicked.connect(self.on_scrollbars_clicked)
        w.doc.clicked.connect(self.on_doc_clicked)
        w.about.clicked.connect(self.on_about_clicked)
        w.settings.clicked.connect(self.on_settings_clicked)
        w.quit.clicked.connect(self.on_quit_clicked)
        self.on_source_folder_changed(False)

    def on_source_clicked(self):
        self.core.open_new_source()
        self.widget.hide()
    def on_import_clicked(self):
        self.core.import_from_camera()
        self.widget.hide()
    def on_print_clicked(self):
        self.core.launch_printing_dialog()
        self.widget.hide()
    def on_search_clicked(self):
        self.core.launch_search_and_find_dialog()
        self.widget.hide()
    def on_geotag_clicked(self):
        self.core.launch_geotagging_dialog()
        self.widget.hide()
    def on_magnifier_clicked(self, state):
        self.core.toggle_magnifier(state)
        self.widget.hide()
    def on_grid_clicked(self, state):
        self.core.toggle_grid(state)
        self.widget.hide()
    def on_aerial_clicked(self, state):
        self.core.display_aerial_view(state)
        self.widget.hide()
    def on_trailer_clicked(self, state):
        self.widget.hide()
        self.ui.toggle_trailer(state)
    def on_toolpanel_clicked(self, state):
        self.ui.toggle_tool_panel(state)
        self.widget.hide()
    def on_scrollbars_clicked(self, state):
        self.ui.toggle_scroll_bars(state)
        self.widget.hide()
    def on_doc_clicked(self):
        self.core.show_manual()
        self.widget.hide()
    def on_about_clicked(self):
        self.core.show_about_dialog()
        self.widget.hide()
    def on_settings_clicked(self):
        self.core.launch_setting_dialog()
        self.widget.hide()
    def on_quit_clicked(self):
        self.core.close_app(True)

    def display_menu_views(self):
        """Call the menu Views.

        """
        self.views_menu = ViewsMenu(self.context, self)
        g = self.widget.geometry()
        self.widget.hide()
        self.views_menu.popup(QPoint(g.x(), g.y()+110))
        self.views_menu.optionSelected.connect(self.on_view_selected)

    def display_menu_targets(self):
        """Call the menu Targets.

        """
        # TODO
        pass

    def set_enable_trailer(self, enable):
        """Enable or disable the Trailer's button.

        Args:
        enable -- boolean
        """
        self.widget.trailer.setEnabled(enable)

    def on_view_selected(self, option):
        """Called when an option was selected into the menu Views.

        Args:
        option -- int() the option selected.
                    1 -- viewing
                    2 -- sorting
                    3 -- full screen
                    5 -- RAW files viewing
                    6 -- multiviewing
        """
        self.views_menu.setParent(None)
        self.views_menu.deleteLater()
        del self.views_menu
        if option == 5:
            self.core.set_raw_files_viewing(True)

        elif option == 6:
            if self.core.multiviewing:
                self.core.close_multiviewing(True, True)

            else:
                self.core.set_multi_viewing(True)

        else:
            self.core.state.set_current_state(option)

    def set_context(self, context):
        """Called when the state of the application has changed.

        Args:
        context -- int
        """
        if not context:
            self.on_source_folder_changed(False)

        else:
            self.on_source_folder_changed(True)

    def set_raw_only(self, raw):
        self.raw_only = True

    def on_source_folder_changed(self, source):
        w = self.widget
        w.print_.setEnabled(source)
        w.search.setEnabled(source)
        w.geoloc.setEnabled(source)
        w.magnifier.setEnabled(source)
        w.grid.setEnabled(source)
        w.aerial.setEnabled(source)
        w.trailer.setEnabled(source)

class PopupMenu(QMenu):
    """Define the menu widget.

    """
    def __init__(self, driver, parent=None):
        super().__init__(parent)
        self.driver = driver
        self.layout = QGridLayout(self)
        self.layout.setContentsMargins(2, 2, 2, 2)
        self.layout.setSpacing(2)
        self.view_icons = ['open.png', 'viewing_mode_128x64.png',
                           'multifolder_128x111.png', 'full_btn_35x20.png',
                           'viewing_mode_128x64.png', 'raw_th_active.png',
                           'multiviews.png']
        tip = _('Open an image folder')
        self.source = MenuItem('Source', tip, 'source_36x32.png', False, self)
        self.targets = MenuItem('Targets', tip, 'open.png', False, self)
        tip = _('Import from camera')
        self.import_ = MenuItem('Import', tip, 'camera.svg', False, self)
        tip = _('Open the printing tool')
        self.print_ = MenuItem('Print', tip, 'printer.svg', False, self)
        tip = _('Open the research tool')
        self.search = MenuItem('Search', tip, 'find.png', False, self)
        tip = _('Open the geolocation tool')
        self.geoloc = MenuItem('Geolocation', tip, 'world_36.png', False, self)
        tip = _('Show-hide the magnifier')
        self.magnifier = MenuItem('Magnifier', tip, 'magnifier_36x30.png', 
                                    True,  self)
        tip = _('Show-hide the grid')
        self.grid = MenuItem('Grid', tip, 'grid_46', True, self)
        tip = _('Show-hide the aerial view')
        self.aerial = MenuItem('AerialView', tip, 'bird_2_48x24.png', True, self)
        tip = _('Show the menu Views')
        self.views = MenuItem('Views', tip, 'open.png', False, self)
        tip = _('Show-hide the trailer')
        self.trailer = MenuItem('Trailer', tip, 'trailer2.png', True, self)
        tip = _('Show-hide the tool panel')
        self.toolpanel = MenuItem('Tool panel', tip, 'toolboxpanel.png', True, self)
        tip = _('Show-hide the scroll-bars')
        self.scrollbars = MenuItem('Scroll bars', tip, 'scrollbar.png', True, self)
        tip = _('Open the manual')
        self.doc = MenuItem('Manual', tip, 'doc.svg', False, self)
        tip = _('Open the About box')
        self.about = MenuItem('About', tip, 'infos.png', False, self)
        tip = _('Configuration')
        self.settings = MenuItem('Settings', tip, 'settings.png', False, self)
        tip = _('Close Oqapy')
        self.quit = MenuItem('Quit', tip, 'close', False, self)

        self.layout.addWidget(self.source, 0, 0, 1, 1)
        self.layout.addWidget(self.targets, 0, 1, 1, 1)
        self.layout.addWidget(self.import_, 0, 2, 1, 1)
        line = Separator(self)
        self.layout.addWidget(line, 1, 0, 1, 3)
        self.layout.addWidget(self.print_, 2, 0, 1, 1)
        self.layout.addWidget(self.search, 2, 1, 1, 1)
        self.layout.addWidget(self.geoloc, 2, 2, 1, 1)
        self.layout.addWidget(self.magnifier, 3, 0, 1, 1)
        self.layout.addWidget(self.grid, 3, 1, 1, 1)
        self.layout.addWidget(self.aerial, 3, 2, 1, 1)
        line_1 = Separator(self)
        self.layout.addWidget(line_1, 4, 0, 1, 3)
        self.layout.addWidget(self.views, 5, 1, 1, 1)
        line_2 = Separator(self)
        self.layout.addWidget(line_2, 6, 0, 1,3)
        self.layout.addWidget(self.trailer, 7, 1, 1, 1)
        self.layout.addWidget(self.toolpanel, 7, 2, 1, 1)
        self.layout.addWidget(self.scrollbars, 7, 0, 1, 1)
        line_3 = Separator(self)
        self.layout.addWidget(line_3, 8, 0, 1, 3)
        self.layout.addWidget(self.doc, 9, 1, 1, 1)
        self.layout.addWidget(self.about, 9, 2, 1, 1)
        self.layout.addWidget(self.settings, 9, 0, 1, 1)
        line_4 = Separator(self)
        self.layout.addWidget(line_4, 10, 0, 1, 3)
        self.layout.addWidget(self.quit, 11, 1, 1, 1)
        line_5 = Separator(self)
        self.layout.addWidget(line_5, 12, 0, 1, 3)
        self.tip_lbl = QLabel(self)
        self.tip_lbl.setAlignment(Qt.AlignHCenter|Qt.AlignVCenter)
        self.layout.addWidget(self.tip_lbl, 13, 0, 1, 3)
        self.targets.hide()
        self.targets.clicked.connect(self.driver.display_menu_targets)
        self.views.clicked.connect(self.driver.display_menu_views)
        self.set_style()

    def set_context(self, state):
        lgg.info('Set context: %s' % state)
        self.views.setEnabled(state and 1)
        self.views.set_icon(self.view_icons[state])

    def set_style(self):
        self.setPalette(styles.get_palette())
        self.setStyleSheet("""QWidget 
                        {background-color: #F4F4F4;
                        color: #000000;}""")


class MenuItem(QToolButton):
    """Define a menu item.

    """
    def __init__(self, text, tip, pixmap, checkable, parent=None):
        super().__init__(parent)
        self.setMinimumSize(QSize(80, 50))
        self.tip = tip
        self.set_icon(pixmap)
        self.setText(text)
        self.setToolButtonStyle(Qt.ToolButtonTextUnderIcon)
        self.setAutoRaise(True)
        self.setMouseTracking(True)
        if checkable:
            self.setCheckable(True)

    def set_icon(self, path):
        icon = QIcon()
        icon.addPixmap(QPixmap('medias/%s' % path))
        self.setIcon(icon)
        self.setIconSize(QSize(30, 30))

    def mouseMoveEvent(self, event):
        self.update_tip_label(self.tip)
        #timer = QTimer()
        #timer.singleShot(3000, self.update_tip_label)

    def update_tip_label(self, tip=''):
        try:
            self.parent().tip_lbl.setText(tip)
        except AttributeError:
            # The menu is closed
            pass


class Separator(QFrame):
    """Define a separator of items.

    """
    def __init__(self, parent=None):
        super().__init__(parent)
        sizePolicy = QSizePolicy(QSizePolicy.Expanding, QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(1)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.sizePolicy().hasHeightForWidth())
        self.setSizePolicy(sizePolicy)
        self.setFrameShape(QFrame.HLine)
        self.setFrameShadow(QFrame.Sunken)


class ViewsMenu(QMenu):
    """Define the menu Views.

    """
    optionSelected = pyqtSignal(int)
    def __init__(self, idx, driver, parent=None):
        """Instantiate the menu Views.

        Args:
        idx -- the current state as integer
        driver -- the GlobalMenu instance
        """
        super().__init__(parent)
        self.setMinimumSize(140, 300)
        self.layout = QVBoxLayout(self)
        self.layout.setContentsMargins(2, 2, 2, 2)
        self.layout.setSpacing(2)
        icons = driver.widget.view_icons
        tip = 'Viewing'
        self.viewing = MenuItem('Viewing', tip, icons[1], True, self)
        tip = 'Multiviewing'
        self.multiviewing = MenuItem('Multiviewing', tip, icons[6], True, self)
        tip = 'Sorting'
        self.sorting = MenuItem('Sorting', tip, icons[2], True, self)
        tip = 'Full screen'
        self.fullscreen = MenuItem('Full screen', tip, icons[3], True, self)
        tip = 'Raw images'
        self.raw = MenuItem('Raw images', tip, icons[5], True, self)
        self.layout.addWidget(self.viewing)
        self.layout.addWidget(self.multiviewing)
        self.layout.addWidget(self.sorting)
        self.layout.addWidget(self.fullscreen)
        self.layout.addWidget(self.raw)
        line = Separator(self)
        self.layout.addWidget(line)
        self.tip_lbl = QLabel(self)
        self.tip_lbl.setAlignment(Qt.AlignHCenter|Qt.AlignVCenter)
        self.layout.addWidget(self.tip_lbl)
        if driver.raw_only:
            self.viewing.setEnabled(False)
            self.multiviewing.setEnabled(False)
            self.sorting.setEnabled(False)
            self.raw.setEnabled(False)

        elif idx in [0, 1, 4]:
            self.viewing.setEnabled(False)

        elif idx == 2:
            self.sorting.setEnabled(False)

        elif idx == 5 or not driver.accept_raw_files:
            self.raw.setEnabled(False)

        self.viewing.clicked.connect(self.on_viewing_selected)
        self.multiviewing.clicked.connect(self.on_multiviewing_selected)
        self.sorting.clicked.connect(self.on_sorting_selected)
        self.fullscreen.clicked.connect(self.on_fullscreen_selected)
        self.raw.clicked.connect(self.on_raw_selected)

    def on_viewing_selected(self):
        self.optionSelected.emit(1)

    def on_multiviewing_selected(self):
        self.optionSelected.emit(6)

    def on_sorting_selected(self):
        self.optionSelected.emit(2)

    def on_fullscreen_selected(self):
        self.optionSelected.emit(3)

    def on_raw_selected(self):
        self.optionSelected.emit(5)

