# -*- coding: utf-8 -*-

# grid.py
# This file is part of Oqapy serie 3
#
# Author: Vincent Vande Vyvre <vincent.vandevyvre@oqapy.eu>
# Home: http://www.oqapy.eu
# Copyright: 2012-2016 Vincent Vande Vyvre
# Licence: GPL3 
#
# Grid tool

import logging
logger = logging.getLogger(__name__)

from fractions import Fraction

from PyQt5.QtCore import (QObject, pyqtSignal, QSize, Qt, QMetaObject, )
from PyQt5.QtGui import (QColor, QPen, QIcon, QPixmap, QFont, )
from PyQt5.QtWidgets import (QGraphicsView, QDialog, QGraphicsLineItem,
                             QGridLayout, QVBoxLayout, QHBoxLayout, QSpacerItem, 
                             QSizePolicy, QFrame, QLabel, QRadioButton, 
                             QButtonGroup, QSpinBox, QCheckBox, QPushButton, 
                             QComboBox)

#import stylesheets as theme

from VWidgets_PyQt5.vcolorcombobox import VColorComboBox


class Grid(QObject):
    settingsChanged = pyqtSignal()
    def __init__(self, core, graphics, cfg, parent=None):
        super().__init__(parent)
        logger.info('Instanciate the grid')
        self.core = core
        self.graphics_view = graphics
        self.cfg = cfg
        self.graphic_lines = {'h': [], 'v': []}
        self.is_visible = False
        self.initialized = False
        self.ratio = 0
        self.ratios = core.cfg.get(("crop", "ratios"))

    def set_graphics_view(self, gvw):
        """Sets the QGraphicsView instance.

        Args:
        gvw -- QGraphicsView

        Raise TypeError if not valid
        """
        if not isinstance(gvw, QGraphicsView):
            raise TypeError("set_graphics_view() expect QGraphicsView(), got"
                                        " {0}".format(str(type(gvw))))

        self.graphics_view = gvw

    def set_line_style(self, style):
        """Sets the line style.

        Args:
        type_ -- str(line style), two styles are allowed 'solid' and 'dash'

        Raise a TypeError if argument is not str()
        Raise a ValueError if argument is not a valid style
        """
        if not isinstance(style, str):
            raise TypeError("set_line_style() expect str() got {0}"
                                        .format(str(type(style))))

        if style not in ("solid", "dash"):
            raise ValueError("set_line_style() unknow argument: '{0}'"
                                        .format(style))

        self.line_style = style
        self.update_()

    def set_color(self, color):
        """Sets the line color.

        Args:
        color -- QColor() instance or tuple(red, green, blue[, alpha])

        Raise a ValueError if argument not valid
        """
        if isinstance(color, QColor):
            self.cfg["color"] = color.name()

        elif isinstance(color, tuple):
            self.cfg["color"] = QColor(color[0], color[1], color[2], 255).name()

        elif isinstance(color, str):
            self.cfg["color"] = color

        else:
            raise ValueError("set_color() expect QColor or RGB values, got {0}"
                                        .format(color))

        self.update_()

    def set_thickness(self, thick):
        """Sets the line thickness.

        Args:
        thick -- int() in range (1, 5), if thick is out of range, value 1
                 will be setted.

        Raise a TypeError if argument is not int()
        """
        if not isinstance(thick, int):
            raise TypeError("set_thickness() expect int(), got '{0}'"
                                        .format(thick))

        if abs(thick) not in range(1, 5):
            thick = 1

        self.cfg["thickness"] = abs(thick)

    def set_ratio(self, ratio):
        """Sets the horizontal-vertical ratio constrain.

        Args:
        ratio -- False if no ratio is applied,
                 Fraction() instance,
                 'sameAs' if ratio is same as image,
                 'x:y' if values predefined or 
                 tuple(horizontal, vertical)
        """
        if isinstance(ratio, str):
            if ratio == "sameAs":
                self.cfg["type_ratio"] = ratio
                ratio = self.get_image_ratio()

            else:
                ratio = self.get_ratio_from_string(ratio)

        elif isinstance(ratio, Fraction):
            ratio = ratio

        elif isinstance(ratio, tuple):
            ratio = self.get_rational(ratio[0], ratio[1])

        else:
            ratio = False

        self.ratio = ratio

    def apply_ratio(self, b):
        """Apply the ratio. Called by menu setting.

        Args:
        b -- boolean
        """
        if not b:
            self.cfg["ratio"] = False

        if self.cfg["mode"] == 'lines':
            self.set_horizontal_line_number(self.cfg["numH"], True)

        else:
            self.set_horizontal_gap(self.cfg["offsetX"], True)

    def initialize(self):
        """Initialize the grid.

        """
        if self.cfg["type_ratio"] == 'sameAs':
            self.cfg["ratio"] = 'sameAs'

        self.set_ratio(self.cfg["ratio"])
        off = self.get_offset()
        if self.cfg["mode"] == 'gap' and off is not None:
            self.set_vertical_gap(off[0], True)

        else:
            self.set_horizontal_line_number(self.cfg["numH"], True)

        self.create_grid(True)


    def set_horizontal_line_number(self, num, force=False):
        """Sets the number of horizontal lines.

        If ratio is defined, the number of vertical lines will be updated.

        Args:
        num -- int()
        force -- boolean, if True, number of lines has been changed by the user,
                    we need to check the ratio.

        Raise a ValueError if argument is not int()
        """
        if not isinstance(num, int):
            raise TypeError("set_horizontal_line_number() expect int() got {0}"
                                        .format(str(type(num))))

        self.cfg["mode"] = 'lines'
        self.cfg["numH"] = num
        h = self.get_scene_size()[1]
        self.cfg["offsetY"] = int(h / (num + 1)) + 1
        if force:  
            self._update_gap_by_ratio(True)

        self.update_()

    def set_vertical_line_number(self, num, force=False):
        """Sets the number of vertical lines.

        If ratio is defined, the number of horizontal lines will be updated.

        Args:
        num -- int()
        force -- boolean, if True, number of lines has been changed by the user,
                    we need to check the ratio.

        Raise a ValueError if argument is not int()
        """
        if not isinstance(num, int):
            raise TypeError("set_vertical_line_number() expect int() got {0}"
                                        .format(str(type(num))))

        self.cfg["mode"] = 'lines'
        self.cfg["numV"] = num
        w = self.get_scene_size()[0]
        self.cfg["offsetX"] = int(w / (num + 1)) + 1
        if force:
            self._update_gap_by_ratio(False)

        self.update_()

    def set_vertical_gap(self, gap, gapX=None, force=False):
        """Sets the vertical line's gap.

        If ratio is defined, the horizontal gap will be updated.

        Args:
        gap -- int()
        force -- boolean, if True, gap has been changed by the user,
                    we need to check the ratio.

        Raise a TypeError if argument is not int()
        """
        if not isinstance(gap, int):
            raise TypeError("set_vertical_gap() expect int(), got {0}"
                                        .format(str(type(gap))))

        self.cfg["mode"] = 'gap'
        self.cfg["offsetY"] = gap
        if force:
            self._update_gap_by_ratio(True)

        self.update_()

    def set_horizontal_gap(self, gap, gapY=None, force=False):
        """Sets the horizontal line's gap.

        If ratio is defined, the vertical gap will be updated.

        Args:
        gap -- int()
        force -- boolean, if True, gap has been changed by the user,
                    we need to check the ratio.

        Raise a TypeError if argument is not int()
        """
        if not isinstance(gap, int):
            raise TypeError("set_horizontal_gap() expect int(), got {0}"
                                        .format(str(type(gap))))

        self.cfg["mode"] = 'gap'
        self.cfg["offsetX"] = gap
        if force:
            self._update_gap_by_ratio(False)

        self.update_()

    def _update_gap_by_ratio(self, hline):
        """Sets the gap of horizontal or vertical lines when value of one
        of both has been changed.

        Args:
        hline -- boolean, if True, horizontal lines are fixed
        """
        if not self.cfg["ratio"]:
            return

        r = self.ratio
        if hline:
            self.cfg["offsetX"] = int(self.cfg["offsetY"] * 
                                        (r.numerator / float(r.denominator)))

        else:
            self.cfg["offsetY"] = int(self.cfg["offsetX"] * 
                                        (r.denominator / float(r.numerator)))

    def set_image_size(self, size):
        """Sets the image's size.

        Args:
        size -- tuple(x, y) or QSize instance

        Raise ValueError if argument not valid
        """
        if isinstance(size, QSize):
            self.img_size = size.width(), size.height()

        else:
            try:
                self.img_size = size[0], size[1]
            except Exception:
                raise ValueError("set_image_size() expect QSize or tuple, got"
                                    " {0}".format(size))

    def show_grid(self, force=None):
        """Show - hide the grids.

        Args:
        force -- if None, toggle show - hide
                 if True, set visible
                 if False, set invisible
        """
        if force is None:
            force = not self.is_visible

        self._set_lines_visible(force)
            
    def get_scene(self):
        """Returns QGraphicsScene instance.

        """
        try:
            return self.graphics_view.scene()
        except:
            logger.info("No QGraphicsScene defined")

    def get_scene_size(self):
        """Returns the scene's size.

        """
        scene = self.get_scene()
        if scene is None:
            return

        rect = scene.sceneRect()

        return int(rect.width()), int(rect.height())

    def get_image_ratio(self):
        """Returns the image size ratio.

        """
        size = self.get_scene_size()
        if size is None:
            return

        return self.get_rational(size[0], size[1])

    def get_ratio_string(self, r):
        """Returns the ratio as string.

        Args:
        r -- Fraction() instance

        Returns:
        str("num:denom")
        """
        return ":".join([str(r.numerator), str(r.denominator)])

    def get_values(self):
        """Returns some values needed by the configuration dialog box.

        Returns:
        tuple(number horizontal lines, number vertical lines,
                horizontal gap, vertical gap, ratio)
        """
        ny = len(self.graphic_lines['h'])
        nx = len(self.graphic_lines['v'])
        return ny, nx, self.cfg

    def create_grid(self, show=False):
        """Create lines.

        Args:
        show -- if True, show the lines
        """
        logger.info('Draw lines')
        self.size = self.get_scene_size()
        if self.size is None:
            logger.info("No QGraphicsScene defined")
            return

        self.graphic_lines = {'h': [], 'v': []}

        if self.cfg["offsetX"]:
            self._set_vertical_lines()

        if self.cfg["offsetY"]:
            self._set_horizontal_lines()

        self.initialized = True

        if show:
            self._set_lines_visible(True)

    def _set_vertical_lines(self):
        """Draw vertical lines.

        """
        vl = []
        off = self.cfg["offsetX"]

        while off < self.size[0]:
            line = Line(self, 0, self.size[1])
            line.move_to(off, 0)
            vl.append(line)
            off += self.cfg["offsetX"]

        self.graphic_lines['v'] = vl
        self.cfg["numV"] = len(vl)

    def _set_horizontal_lines(self):
        """Draw horizontal lines.

        """
        hl = []
        off = self.cfg["offsetY"]

        while off < self.size[1]:
            line = Line(self, self.size[0], 0)
            line.move_to(0, off)
            hl.append(line)
            off += self.cfg["offsetY"]

        self.graphic_lines['h'] = hl
        self.cfg["numH"] = len(hl)

    def _set_lines_visible(self, show):
        """Show - hide the lines.

        Args:
        show -- boolean
        """
        scene = self.get_scene()
        if scene is None:
            return

        if show and not self.is_visible:
            logger.info('Show the grid')
            for key in self.graphic_lines:
                for line in self.graphic_lines[key]:
                    scene.addItem(line)
                self.is_visible = True

        elif not show and self.is_visible:
            logger.info('Hide the grid')
            for key in self.graphic_lines:
                try:
                    for line in self.graphic_lines[key]:
                        scene.removeItem(line)
                except:
                    # Grid has been deleted by a new pixmap
                    pass
                self.is_visible = False

    def update_(self):
        """Redraw the grid.

        """
        if self.initialized:
            logger.info('Update the grid')
            v = self.is_visible
            self._set_lines_visible(False)
            self.create_grid(v)


    def get_color(self):
        """Returns the current color.

        """
        return self.cfg["color"].getRgb()

    def get_style(self):
        """Returns the current style.

        """
        return self.line_style

    def get_offset(self):
        if self.cfg["offsetY"] and self.cfg["offsetX"]:
            return self.cfg["offsetY"], self.cfg["offsetX"]

        return None

    def get_rational(self, num, den):
        try:
            return Fraction(num, den)
        except ZeroDivisionError:
            return False

    def reverse_ratio(self, b):
        """Reverse the ratio values.

        Args:
        b -- boolean
        """
        n, d = self.cfg["ratio"].split(':')
        self.cfg["ratio"] = '%s:%s' %(d, n)
        self.cfg["offsetX"], self.cfg["offsetY"] = self.cfg["offsetY"], \
                                                   self.cfg["offsetX"]
        self.update_()

    def get_ratio_from_string(self, s):
        """Return a Fraction() instance from string.

        Args:
        s -- string "x:y"
        """
        try:
            x, y = s.split(":")
            return Fraction(int(x), int(y))
        except ValueError:
            logger.info('Ratio unsubscriptable: {0}'.format(s))
            return False 

    def float_to_rational(self, val):
        """Return a Fraction() instance from float().

        Args:
        val -- float()
        """
        return Fraction.from_float(val).limit_denominator(9999)

    def call_settings_menu(self):
        """Call the settings dialog box.

        """
        menu = SettingsMenu(self)
        menu.exec_()

class Line(QGraphicsLineItem):
    def __init__(self, grid, width_, height_, scene=None):
        """Line object.

        Args:
        grid -- Grid() instance
        width_ -- scene's width
        height_ -- scene's height
        """
        super().__init__()
        self.thick = grid.cfg["thickness"]
        self.color = QColor(grid.cfg["color"])
        self.style = Qt.SolidLine
        pen = QPen(self.color, self.thick, self.style, Qt.RoundCap,
                                                Qt.MiterJoin)
        if grid.cfg["line_style"] == 'dash':
            pen.setDashPattern([5.0, 10.0])

        self.setPen(pen)
        self.setLine(0.0, 0.0, width_, height_)

    def move_to(self, x, y):
        self.setPos(x, y)


class SettingsMenu(QDialog):
    def __init__(self, grid):
        super().__init__()
        self.grid = grid
        self.resize(413, 270)
        self.setModal(True)
        #pal = theme.get_palette()
        #Dialog.setPalette(pal)

        self.gridLayout_3 = QGridLayout(self)
        self.gridLayout_3.setContentsMargins(20, 4, 20, 4)
        self.verticalLayout_3 = QVBoxLayout()
        self.verticalLayout = QVBoxLayout()
        self.verticalLayout.setSpacing(0)
        self.horizontalLayout_5 = QHBoxLayout()
        self.sub_1 = SubTitle(self)
        self.sub_1.setMaximumSize(QSize(16777215, 18))
        self.horizontalLayout_5.addWidget(self.sub_1)
        spacerItem = QSpacerItem(40, 20, QSizePolicy.Expanding, 
                                        QSizePolicy.Minimum)
        self.horizontalLayout_5.addItem(spacerItem)
        self.verticalLayout.addLayout(self.horizontalLayout_5)
        self.sep_1 = QFrame(self)
        self.sep_1.setFrameShape(QFrame.HLine)
        self.sep_1.setFrameShadow(QFrame.Plain)
        self.sep_1.setMidLineWidth(1)
        self.verticalLayout.addWidget(self.sep_1)
        self.verticalLayout_3.addLayout(self.verticalLayout)
        spacerItem1 = QSpacerItem(20, 8, QSizePolicy.Minimum, 
                                        QSizePolicy.Fixed)
        self.verticalLayout_3.addItem(spacerItem1)
        self.horizontalLayout_8 = QHBoxLayout()
        self.gridLayout = QGridLayout()
        spacerItem2 = QSpacerItem(37, 19, QSizePolicy.Minimum, 
                                        QSizePolicy.Minimum)
        self.gridLayout.addItem(spacerItem2, 0, 0, 1, 1)
        self.style_lbl = QLabel(self)
        self.style_lbl.setAlignment(Qt.AlignRight|
                                        Qt.AlignTrailing|
                                        Qt.AlignVCenter)
        self.gridLayout.addWidget(self.style_lbl, 0, 1, 1, 1)
        self.style_cmb = QComboBox(self)
        self.style_cmb.setMinimumSize(QSize(100, 0))
        self.gridLayout.addWidget(self.style_cmb, 0, 2, 1, 1)
        spacerItem3 = QSpacerItem(88, 20, QSizePolicy.Expanding, 
                                        QSizePolicy.Minimum)
        self.gridLayout.addItem(spacerItem3, 0, 3, 1, 1)
        spacerItem4 = QSpacerItem(37, 19, QSizePolicy.Minimum, 
                                        QSizePolicy.Minimum)
        self.gridLayout.addItem(spacerItem4, 1, 0, 1, 1)
        self.color_lbl = QLabel(self)
        self.color_lbl.setAlignment(Qt.AlignRight|
                                        Qt.AlignTrailing|
                                        Qt.AlignVCenter)
        self.gridLayout.addWidget(self.color_lbl, 1, 1, 1, 1)
        self.color_cmb = VColorComboBox(self)
        self.gridLayout.addWidget(self.color_cmb, 1, 2, 1, 1)
        spacerItem5 = QSpacerItem(88, 20, QSizePolicy.Expanding, 
                                        QSizePolicy.Minimum)
        self.gridLayout.addItem(spacerItem5, 1, 3, 1, 1)
        self.horizontalLayout_8.addLayout(self.gridLayout)
        spacerItem6 = QSpacerItem(40, 20, QSizePolicy.Expanding, 
                                        QSizePolicy.Minimum)
        self.horizontalLayout_8.addItem(spacerItem6)
        self.verticalLayout_3.addLayout(self.horizontalLayout_8)
        spacerItem7 = QSpacerItem(20, 12, QSizePolicy.Minimum, 
                                        QSizePolicy.Fixed)
        self.verticalLayout_3.addItem(spacerItem7)
        self.verticalLayout_2 = QVBoxLayout()
        self.verticalLayout_2.setSpacing(0)
        self.horizontalLayout_7 = QHBoxLayout()
        self.sub_2 = SubTitle(self)
        self.sub_2.setMaximumSize(QSize(16777215, 18))
        self.horizontalLayout_7.addWidget(self.sub_2)
        spacerItem8 = QSpacerItem(40, 20, QSizePolicy.Expanding, 
                                        QSizePolicy.Minimum)
        self.horizontalLayout_7.addItem(spacerItem8)
        self.verticalLayout_2.addLayout(self.horizontalLayout_7)
        self.sep_2 = QFrame(self)
        self.sep_2.setFrameShape(QFrame.HLine)
        self.sep_2.setFrameShadow(QFrame.Plain)
        self.sep_2.setMidLineWidth(2)
        self.verticalLayout_2.addWidget(self.sep_2)
        self.verticalLayout_3.addLayout(self.verticalLayout_2)
        spacerItem9 = QSpacerItem(20, 8, QSizePolicy.Minimum, 
                                        QSizePolicy.Fixed)
        self.verticalLayout_3.addItem(spacerItem9)
        self.horizontalLayout_9 = QHBoxLayout()
        self.gridLayout_2 = QGridLayout()
        self.horizontalLayout = QHBoxLayout()
        spacerItem10 = QSpacerItem(40, 20, QSizePolicy.Minimum, 
                                        QSizePolicy.Minimum)
        self.horizontalLayout.addItem(spacerItem10)
        self.lines_rdo = QRadioButton(self)
        self.lines_rdo.setChecked(True)
        buttonGroup = QButtonGroup(self)
        buttonGroup.addButton(self.lines_rdo)
        self.horizontalLayout.addWidget(self.lines_rdo)
        self.gridLayout_2.addLayout(self.horizontalLayout, 0, 0, 1, 1)
        self.h_lines_spb = QSpinBox(self)
        self.h_lines_spb.setMinimumSize(QSize(140, 0))
        self.gridLayout_2.addWidget(self.h_lines_spb, 0, 1, 1, 1)
        self.v_lines_spb = QSpinBox(self)
        self.v_lines_spb.setMinimumSize(QSize(140, 0))
        self.gridLayout_2.addWidget(self.v_lines_spb, 0, 2, 1, 1)
        self.horizontalLayout_2 = QHBoxLayout()
        spacerItem11 = QSpacerItem(40, 20, QSizePolicy.Fixed, 
                                        QSizePolicy.Minimum)
        self.horizontalLayout_2.addItem(spacerItem11)
        self.gap_rdo = QRadioButton(self)
        self.gap_rdo.setChecked(False)
        buttonGroup.addButton(self.gap_rdo)
        self.horizontalLayout_2.addWidget(self.gap_rdo)
        self.gridLayout_2.addLayout(self.horizontalLayout_2, 1, 0, 1, 1)
        self.h_gap_spb = QSpinBox(self)
        self.h_gap_spb.setEnabled(False)
        self.h_gap_spb.setMinimum(10)
        self.h_gap_spb.setMaximum(990)
        self.h_gap_spb.setProperty("value", 200)
        self.gridLayout_2.addWidget(self.h_gap_spb, 1, 1, 1, 1)
        self.v_gap_spb = QSpinBox(self)
        self.v_gap_spb.setEnabled(False)
        self.v_gap_spb.setMinimum(10)
        self.v_gap_spb.setMaximum(990)
        self.v_gap_spb.setProperty("value", 200)
        self.gridLayout_2.addWidget(self.v_gap_spb, 1, 2, 1, 1)
        self.horizontalLayout_3 = QHBoxLayout()
        spacerItem12 = QSpacerItem(40, 20, QSizePolicy.Minimum, 
                                        QSizePolicy.Minimum)
        self.horizontalLayout_3.addItem(spacerItem12)
        self.ratio_chb = QCheckBox(self)
        self.horizontalLayout_3.addWidget(self.ratio_chb)
        self.gridLayout_2.addLayout(self.horizontalLayout_3, 2, 0, 1, 1)
        self.ratio_cmb = QComboBox(self)
        self.ratio_cmb.setEnabled(False)
        self.gridLayout_2.addWidget(self.ratio_cmb, 2, 1, 1, 1)
        self.reverse_chb = QCheckBox(self)
        self.reverse_chb.setEnabled(False)
        self.gridLayout_2.addWidget(self.reverse_chb, 2, 2, 1, 1)
        self.horizontalLayout_4 = QHBoxLayout()
        spacerItem13 = QSpacerItem(40, 20, QSizePolicy.Expanding, 
                                        QSizePolicy.Minimum)
        self.horizontalLayout_4.addItem(spacerItem13)
        spacerItem14 = QSpacerItem(40, 20, QSizePolicy.Expanding, 
                                        QSizePolicy.Minimum)
        self.horizontalLayout_4.addItem(spacerItem14)
        self.gridLayout_2.addLayout(self.horizontalLayout_4, 3, 0, 1, 1)
        self.custom_h_spb = QSpinBox(self)
        self.custom_h_spb.setEnabled(False)
        self.custom_h_spb.setMinimum(1.0)
        self.gridLayout_2.addWidget(self.custom_h_spb, 3, 1, 1, 1)
        self.custom_v_spb = QSpinBox(self)
        self.custom_v_spb.setEnabled(False)
        self.custom_v_spb.setMinimum(1.0)
        self.gridLayout_2.addWidget(self.custom_v_spb, 3, 2, 1, 1)
        self.horizontalLayout_9.addLayout(self.gridLayout_2)
        spacerItem15 = QSpacerItem(40, 20, QSizePolicy.Expanding, 
                                        QSizePolicy.Minimum)
        self.horizontalLayout_9.addItem(spacerItem15)
        self.verticalLayout_3.addLayout(self.horizontalLayout_9)
        self.horizontalLayout_6 = QHBoxLayout()
        spacerItem16 = QSpacerItem(40, 20, QSizePolicy.Expanding, 
                                        QSizePolicy.Minimum)
        self.horizontalLayout_6.addItem(spacerItem16)
        self.close_btn = QPushButton(self)
        icon = QIcon()
        icon.addPixmap(QPixmap("medias/dialog-apply.svg"), 
                                        QIcon.Normal, QIcon.Off)
        self.close_btn.setIcon(icon)
        self.horizontalLayout_6.addWidget(self.close_btn)
        self.verticalLayout_3.addLayout(self.horizontalLayout_6)
        self.gridLayout_3.addLayout(self.verticalLayout_3, 0, 0, 1, 1)

        st = _("Solid")
        self.style_cmb.addItem(st)
        st = _("Dash")
        self.style_cmb.addItem(st)
        self.color_cmb.set_palette()
        self.update_values()
        self.set_color()
        self.ratio_cmb.addItem('Same as Image')
        self.ratio_cmb.addItems(self.grid.ratios)
        self.ratio_cmb.addItem('Custom')
        self.ratios = self.grid.ratios
        ratio = self.grid.ratio

        if ratio:
            self.ratio_chb.setChecked(True)
            self.set_ratio_widgets(True)
            if self.grid.cfg['type_ratio'] == 'sameAs':
                self.ratio_cmb.setCurrentIndex(0)

            else:
                ratio = self.grid.get_ratio_string(ratio)
                if ratio in self.ratios:
                    self.ratio_cmb.setCurrentIndex(self.ratios.index(ratio))
                
                else:
                    self.ratio_cmb.setCurrentIndex(self.ratio_cmb.count() - 1)
                    h, v = ratio.split(':')
                    self.custom_h_spb.setValue(int(h))
                    self.custom_v_spb.setValue(int(v))
                    self.custom_h_spb.setEnabled(True)
                    self.custom_v_spb.setEnabled(True)

        self.close_btn.clicked.connect(self.accept)
        self.set_connections()

        if self.grid.cfg['mode'] == 'gap':
            self.gap_rdo.setChecked(True)              

        self.retranslateUi()
        #self.apply_style()

    def apply_style(self):
        self.sep_1.setStyleSheet(theme.UNDERLINE)
        self.sep_2.setStyleSheet(theme.UNDERLINE)
        self.style_lbl.setStyleSheet(theme.LABEL)
        self.color_lbl.setStyleSheet(theme.LABEL)
        self.style_cmb.setStyleSheet(theme.COMBO)
        self.color_cmb.setStyleSheet(theme.DARKCOMBO)
        self.ratio_cmb.setStyleSheet(theme.COMBO)
        self.lines_rdo.setStyleSheet(theme.RADIO)
        self.gap_rdo.setStyleSheet(theme.RADIO)
        self.ratio_chb.setStyleSheet(theme.CHECKBOX)
        self.reverse_chb.setStyleSheet(theme.CHECKBOX)
        self.custom_h_spb.setStyleSheet(theme.SPINBOX)
        self.custom_v_spb.setStyleSheet(theme.SPINBOX)
        self.h_lines_spb.setStyleSheet(theme.SPINBOX)
        self.v_lines_spb.setStyleSheet(theme.SPINBOX)
        self.h_gap_spb.setStyleSheet(theme.SPINBOX)
        self.v_gap_spb.setStyleSheet(theme.SPINBOX)

    def set_connections(self):
        self.style_cmb.currentIndexChanged.connect(self.on_style_changed)
        self.color_cmb.currentColorChanged.connect(self.on_color_changed)
        self.lines_rdo.toggled.connect(self.on_lines_checked)
        self.h_lines_spb.valueChanged.connect(self.on_h_lines_value_changed)
        self.v_lines_spb.valueChanged.connect(self.on_v_lines_value_changed)
        self.h_gap_spb.valueChanged.connect(self.on_h_gap_value_changed)
        self.v_gap_spb.valueChanged.connect(self.on_v_gap_value_changed)
        self.ratio_chb.stateChanged.connect(self.on_ratio_checked)
        self.ratio_cmb.currentIndexChanged.connect(self.on_ratio_defined)
        self.reverse_chb.stateChanged.connect(self.on_reverse_checked)
        self.custom_h_spb.valueChanged.connect(self.on_custom_values_changed)
        self.custom_v_spb.valueChanged.connect(self.on_custom_values_changed)

    def on_style_changed(self, style):
        s = ["solid", "dash"]
        self.grid.set_line_style(s[style])

    def on_color_changed(self, name, color):
        self.grid.set_color(color)

    def on_lines_checked(self, state):
        self.grid.mode = 'lines' if state else 'gap'
        self.h_lines_spb.setEnabled(state)
        self.v_lines_spb.setEnabled(state)
        self.h_gap_spb.setEnabled(not state)
        self.v_gap_spb.setEnabled(not state)

    def on_h_lines_value_changed(self, val):
        self.grid.set_horizontal_line_number(val, force=True)
        self.update_values()

    def on_v_lines_value_changed(self, val):
        self.grid.set_vertical_line_number(val, force=True)
        self.update_values()

    def on_h_gap_value_changed(self, val):
        self.grid.set_horizontal_gap(val, force=True)
        self.update_values()

    def on_v_gap_value_changed(self, val):
        self.grid.set_vertical_gap(val, force=True)
        self.update_values()

    def on_ratio_checked(self, state):
        self.set_ratio_widgets(state)
        self.grid.set_ratio('sameAs')
        self.grid.apply_ratio(state)
        self.update_values()

    def on_ratio_defined(self, ratio):
        self.reverse_chb.setChecked(False)
        if self.ratio_cmb.currentIndex == 0:
            self.grid.set_ratio('sameAs')
            self.grid.apply_ratio(True)

        if str(self.ratio_cmb.currentText()) == "Custom":
            self.custom_h_spb.setEnabled(True)
            self.custom_v_spb.setEnabled(True)
            self.grid.type_ratio = 'custom'

        else:
            self.grid.type_ratio = 'predefined'
            self.custom_h_spb.setValue(1)
            self.custom_v_spb.setValue(1)
            self.custom_h_spb.setEnabled(False)
            self.custom_v_spb.setEnabled(False)
            self.grid.set_ratio(str(self.ratio_cmb.currentText()))
            self.grid.apply_ratio(True)

    def on_reverse_checked(self, state):
        self.grid.reverse_ratio(state)

    def on_custom_values_changed(self, val):
        h = self.custom_h_spb.value()
        v = self.custom_v_spb.value()
        self.grid.set_ratio((h, v))
        self.grid.apply_ratio(True)

    def update_values(self):
        values = self.grid.get_values()
        self.block_signals(True)
        self.h_lines_spb.setValue(values[0])
        self.v_lines_spb.setValue(values[1])
        self.h_gap_spb.setValue(values[2]['offsetX'])
        self.v_gap_spb.setValue(values[2]['offsetY'])
        self.block_signals(False)

    def set_color(self):
        self.color_cmb.set_current_color(self.grid.cfg["color"])
      
    def block_signals(self, b):
        self.h_lines_spb.blockSignals(b)
        self.v_lines_spb.blockSignals(b)
        self.h_gap_spb.blockSignals(b)
        self.v_gap_spb.blockSignals(b)

    def set_ratio_widgets(self, state):
        self.ratio_cmb.setEnabled(state)
        self.ratio_cmb.setCurrentIndex(0)
        self.reverse_chb.setEnabled(state)
        self.reverse_chb.setChecked(False)

    def retranslateUi(self):
        self.setWindowTitle(_("Grid Settings - Oqapy"))
        self.sub_1.setText(_("Style"))
        self.style_lbl.setText(_("Lines: "))
        self.color_lbl.setText(_("Color: "))
        self.sub_2.setText(_("Lines"))
        self.lines_rdo.setText(_("Lines: "))
        self.h_lines_spb.setToolTip(_("Horizontal lines"))
        self.v_lines_spb.setToolTip(_("Vertical lines"))
        self.gap_rdo.setText(_("Gap: "))
        self.v_gap_spb.setToolTip(_("Vertical gap"))
        self.h_gap_spb.setToolTip(_("Horizontal gap"))
        self.ratio_chb.setText(_("Ratio: "))
        self.reverse_chb.setToolTip(_("Reverse the ratio"))
        self.reverse_chb.setText(_("Reverse"))
        self.custom_h_spb.setToolTip(_("Horizontal"))
        self.custom_v_spb.setToolTip(_("vertical"))
        self.close_btn.setText(_("Close"))

class SubTitle(QLabel):
    def __init__(self, parent=None):
        super().__init__(parent)
        font = QFont()
        font.setFamily("Trebuchet MS")
        font.setPointSize(11)
        self.setFont(font)
        self.setStyleSheet("QLabel {color: #787C84;}")

    def set_text(self, text):
        self.setText(text)
