# -*- coding: utf-8 -*-

# aerialview.py
# this file is part of Oqapy serie 3
#
# Author: Vincent Vande Vyvre <vincent.vandevyvre@oqapy.eu>
# Home: http://www.oqapy.eu
# Copyright: 2013-2017 Vincent Vande Vyvre
# Licence: GPL3
#
# Aerial view.

import logging
lgg = logging.getLogger(__name__)

from PyQt5.QtCore import (Qt, QObject, QSize, QRectF, QMetaObject, pyqtSignal, 
                          QPoint, )
from PyQt5.QtGui import (QIcon, QPixmap, QImage, QTransform, QPen, QCursor,
                         QColor, QPainter, )
from PyQt5.QtWidgets import (QWidget, QGridLayout, QVBoxLayout, QHBoxLayout,
                             QGraphicsView, QToolButton, QSpinBox, QSpacerItem,
                             QSizePolicy, QGraphicsScene, QPushButton,
                             QGraphicsItem, QGraphicsPixmapItem, QLabel, )

import stylesheets as styles
from VWidgets_PyQt5.vtoolheader import VToolHeader
from tools.rectsize import RectSize

class AerialView(QWidget):
    def __init__(self, core, graphicsview, parent=None):
        super().__init__(parent)
        self.setWindowFlags(Qt.Tool | Qt.WindowStaysOnTopHint | Qt.FramelessWindowHint)
        self.setWindowTitle("Aerial view")
        self.core = core
        self.cfg = core.cfg.get(('aerialview',))
        self.main_viewer = graphicsview
        self.size_ = RectSize(252, 224)
        #self.setMinimumSize(self.size_.as_qsize)
        self.glayout = QGridLayout(self)
        self.glayout.setContentsMargins(0, 0, 0, 0)
        self.glayout.setSpacing(0)
        self.vlayout = QVBoxLayout()
        self.vlayout.setSpacing(0)
        self.header = VToolHeader(self)
        self.header.setTitle("No image")
        self.header.setMaximumSize(1000, 22)
        self.header.setCollapseButton(False)
        self.header.mousePressEvent = self.on_press_event
        self.header.mouseMoveEvent = self.move_widget
        self.vlayout.addWidget(self.header)
        self.config_btn = self.header.add_button('medias/settings.png', 
                                                 'config_btn')
        self.config_btn.setToolTip('Configure')
        self.close_btn = self.header.add_button('medias/close.png', 'close_btn')
        self.close_btn.setToolTip('Close')
        self.viewer = Viewer(core, RectSize(250, 200), graphicsview)
        self.vlayout.addWidget(self.viewer)
        self.glayout.addLayout(self.vlayout, 0, 0, 1, 1)
        g = graphicsview.geometry()
        w, h = g.width(), g.height()
        end = graphicsview.mapToGlobal(QPoint(w, h))
        posx = end.x() - self.cfg['size']
        posy = end.y() - self.cfg['size'] - 40
        self.move(posx, posy)
        self.rectsize = RectSize(self.cfg['size'], self.cfg['size'])
        self.image_size = RectSize(0, 0)
        self.config_btn.clicked.connect(self.configure)
        self.close_btn.clicked.connect(self.close)
        self.set_style()

    def set_image(self, mainview, img, fname, rectsize):
        """Change the image into the view.

        args:
        mainview -- the main viewer
        img -- the current QImage instance, if None the view is just cleared
        fname -- the basename of the image file
        rectsize -- RectSize of the current QImage
        """
        if img is None:
            self.clear()

        else:
            self.main_viewer = mainview
            lgg.info("Set new image, %s" % rectsize)
            if rectsize is not None:
                lgg.info("Size: %s" % rectsize.as_string)
                if rectsize != self.image_size:
                    self.fix_size(rectsize)

            self.header.setTitle(fname)
            self.viewer.display_image(img)
            lgg.info("view size: %s" % self.viewer.size())

    def fix_size(self, size):
        """Fix the size of the graphics view to the image size.

        args:
        size -- RectSize instance
        """
        if size.is_square:
            self.rectsize = RectSize(self.cfg['size'], self.cfg['size'])
        
        elif size.is_horizontal:
            w, h = size.values
            height = self.cfg['size'] * (h / w)
            self.rectsize = RectSize(self.cfg['size'], height)

        else:
            w, h = size.values
            width = self.cfg['size'] * (h / w)
            self.rectsize = RectSize(width, self.cfg['size'])

        wsize = self.rectsize + RectSize(4, 4)
        self.resize(wsize[0], wsize[1])
        self.viewer.setMinimumSize(self.rectsize.as_qsize)
        #max_ = self.rectsize + RectSize(2, 2)
        #self.viewer.setMaximumSize(QSize(max_[0], max_[1]))

    def update_(self):
        """Update the mask.

        """
        if self.isVisible():
            self.viewer.reset_window()

    def clear(self):
        """Clear the view.

        """
        self.viewer.set_background()
        self.header.setTitle("No image")

    def configure(self):
        """Call the configuration widget.

        """
        self.config_widget = ConfigWidget(self)
        self.config_widget.show()

    def close_config(self, size):
        lgg.info("Close aerial view config, size: %s" % size)
        self.config_widget.setParent(None)
        self.config_widget.deleteLater()
        del self.config_widget
        if size != self.cfg['size']:
            self.cfg['size'] = size
            if self.rectsize.is_square:
                self.rectsize = RectSize(self.cfg['size'], self.cfg['size'])

            elif self.rectsize.is_horizontal:
                f = self.rectsize.height / self.rectsize.width
                self.rectsize = RectSize(size, size*f)

            else:
                f = self.rectsize.width / self.rectsize.height
                self.rectsize = RectSize(size*f, size)

            wsize = self.rectsize + RectSize(4, 26)
            self.setMinimumSize(wsize[0], wsize[1])
            self.viewer.setMaximumSize(self.rectsize.as_qsize)

    def on_press_event(self, event):
        self.last_pos = self.mapToGlobal(event.pos())

    def move_widget(self, event):
        new = self.mapToGlobal(event.pos())
        newx = self.pos().x() + (new.x() - self.last_pos.x())
        newy = self.pos().y() + (new.y() - self.last_pos.y())
        self.move(newx, newy)
        self.last_pos = new

    def close(self):
        self.core.cfg.update(('aerialview', 'size'), self.viewer.width())
        self.core.display_aerial_view()

    def set_style(self):
        self.setStyleSheet(styles.WIDGET)
        self.header.setStyleSheet(styles.HEADER)
        self.header.label.setStyleSheet(styles.TITLE)


class Viewer(QGraphicsView):
    def __init__(self, core, size, mainview, parent=None):
        super().__init__(parent)
        self.core = core
        self.mainview = mainview
        self.setMinimumSize(size.as_qsize)
        self.setVerticalScrollBarPolicy(Qt.ScrollBarAlwaysOff)
        self.setHorizontalScrollBarPolicy(Qt.ScrollBarAlwaysOff)
        self.scene_ = QGraphicsScene()
        self.setScene(self.scene_)
        self.src_image = None
        self.old_geometry = QRectF(0.0, 0.0, 1000.0, 563.0)
        # editing_mode is True when the viewer is the EditingArea instance
        self.editing_mode = False

    def get_geometry(self):
        try:
            self.p_rect = self.mainview.scene().sceneRect()
        except RuntimeError:
            # This can happens when the entire main window is resized
            self.p_rect = self.old_geometry

        if self.p_rect.width() < 5:
            return

        self.old_geometry = self.p_rect
        s_rect = self.scene_.sceneRect()
        self.factor = s_rect.width() / self.p_rect.width()
        u_offset = self.mainview.mapFromScene(float(self.p_rect.x()),
                                            float(self.p_rect.y()))

        x = u_offset.x()
        if x >= 0:
            x = 0

        y = u_offset.y()
        if y >= 0:
            y = 0

        width = self.mainview.width() * self.factor
        if width > s_rect.width():
            width = s_rect.width()

        height = self.mainview.height() * self.factor
        if height > s_rect.height():
            height = s_rect.height()

        # window
        g = [abs(x) * self.factor, abs(y) * self.factor, width, height]
        return g

    def display_image(self, qimage):
        self.src_image = qimage
        w, h = self.width(), self.height()
        self.base = w
        if qimage.width() < qimage.height():
            self.is_horizontal = False
            self.base = h

        self.pixmap = QPixmap(qimage.scaled(w, h, Qt.KeepAspectRatio, 
                                    Qt.SmoothTransformation))

        self.change_preview()

    def change_preview(self):
        self.scene_.clear()
        self.scene_.setSceneRect(QRectF(self.pixmap.rect()))
        self.scene_.addPixmap(self.pixmap)
        self.set_mask(self.get_geometry())
        
    def set_mask(self, r):
        self.window = Window(self, r, None, self.scene_)
        s_rect = self.scene_.sceneRect()
        self.window.w_width = s_rect.width()
        self.window.w_height = s_rect.height()
        self.window.positionChanged.connect(self._update_viewer)

    def _update_viewer(self, value):
        valX = value[0] / self.factor
        valY = value[1] / self.factor
        if self.editing_mode:
            self.block_sync(False)

        self.block_sync(True)
        self.mainview.centerOn(valX, valY)
        self.block_sync(False)
        self.core.reset_navigation()

        if self.editing_mode:
            self.block_sync(True)    

    def reset_window(self):
        if self.src_image is not None:
            try:
                self.scene_.removeItem(self.window)
            except:
                pass
            self.window = None
            self.set_mask(self.get_geometry())

    def resizeEvent(self, event):
        if self.src_image is not None:
            self.display_image(self.src_image)
        QGraphicsView.resizeEvent(self, event)

    def block_sync(self, b):
        self.mainview.is_auto_scroll = b

    def set_background(self):
        self.src_image = None
        pix = QPixmap('medias/pix_viewer_87x80.png')
        self.scene_.clear()
        self.scene_.setSceneRect(QRectF(pix.rect()))
        self.scene_.addPixmap(pix)

        
class Window(QGraphicsPixmapItem):
    def __init__(self, main, rect, parent=None, scene=None):
        super().__init__(parent)
        if scene is not None: 
            scene.addItem(self)
        self.main = main
        self.viewer = main.mainview
        self.posX = rect[0]
        self.posY = rect[1]
        self.width = rect[2]
        self.height = rect[3]
        self.w_width = 0
        self.w_height = 0
        self.setFlags(QGraphicsItem.ItemIsMovable | 
                        QGraphicsItem.ItemSendsScenePositionChanges |
                        QGraphicsItem.ItemIsFocusable)
        self.setAcceptHoverEvents(True)
        color = QColor()
        color.setRgb(100, 220, 255, 130)
        pix = QPixmap(QSize(self.width, self.height))
        pix.fill(color)
        color.setRgb(230, 240, 255, 255)
        painter = QPainter()
        painter.begin(pix)
        painter.setPen(QPen(color, 1, Qt.SolidLine, Qt.RoundCap, Qt.MiterJoin))
        painter.drawRect(0.0, 0.0, self.width - 2, self.height - 2)
        painter.end()
        self.setPixmap(pix)
        self.setPos(self.posX, self.posY)
        self._moved = Moved()

    def positionChanged():
        def fget(self):
            return self._moved.positionChanged
        return locals()
    positionChanged = property(**positionChanged())

    def mousePressEvent(self, event):
        pos = event.scenePos()
        if event.button() == 1:
            self.old_X, self.old_Y = pos.x(), pos.y()
        event.accept()

    def mouseMoveEvent(self, event):
        pos = event.scenePos()
        if event.button() == 2:
            return

        X, Y = pos.x(), pos.y()
        m_X, m_Y = X - self.old_X, Y - self.old_Y
        newX, newY = self.posX + m_X, self.posY + m_Y
        if newX <= 0 or newX + self.width >= self.w_width + 2:
            m_X = 0

        if newY <= 0 or newY + self.height >= self.w_height + 2:
            m_Y = 0
       
        self.old_X, self.old_Y = X, Y
        self.moveBy(m_X, m_Y)
        self._update()
        self.positionChanged.emit([self.posX + (self.width / 2), 
                                   self.posY + self.height / 2])
        event.accept()

    def hoverEnterEvent(self, event):
        self.old_shape = self.viewer.cursor().shape()
        shape = QPixmap("medias/cross_cursor_32.png")
        cursor = QCursor(shape, -1, -1)
        self.setCursor(cursor)

    def hoverLeaveEvent(self, event):
        cursor = QCursor(self.old_shape)
        self.viewer.setCursor(cursor)

    def _update(self):
        self.posX, self.posY = self.pos().x(), self.pos().y()


class Moved(QObject):
    """Signal used by aerial view when window's position changed.

    """
    positionChanged = pyqtSignal(list)
    def __init__(self):
        super().__init__()


class ConfigWidget(QWidget):
    def __init__(self, parent=None):
        super().__init__(parent)
        self.vlayout = QVBoxLayout(self)
        self.vlayout.setContentsMargins(0, 0, 0, 0)
        self.vlayout.setSpacing(0)
        self.header = VToolHeader(self)
        self.header.setTitle("Configure aerial view")
        self.header.setMaximumSize(1000, 22)
        self.header.setCollapseButton(False)
        self.vlayout.addWidget(self.header)
        self.close_btn = self.header.add_button('medias/close.png', 'close_btn')
        self.close_btn.setToolTip('Configure')
        self.vlayout.addWidget(self.header)
        self.hlayout = QHBoxLayout()
        self.hlayout.setContentsMargins(0, 2, 2, 2)
        space = QSpacerItem(18, 20, QSizePolicy.Expanding, QSizePolicy.Minimum)
        self.hlayout.addItem(space)
        self.label = QLabel(self)
        self.label.setText(_("Size:"))
        self.hlayout.addWidget(self.label)
        self.size_spb = QSpinBox(self)
        self.size_spb.setRange(200, 500)
        self.size_spb.setSingleStep(50)
        self.size_spb.setSuffix(' pixels')
        size = parent.cfg['size']
        self.size_spb.setValue(size)
        self.hlayout.addWidget(self.size_spb)
        self.vlayout.addLayout(self.hlayout)
        self.close_btn.clicked.connect(self.on_close)
        x = size / 2 - 100
        self.setGeometry(x, 40, 200, 50)
        self.setStyleSheet("""QWidget {background: #E8E8E8;
                              color: black;}""")
        self.size_spb.setStyleSheet(styles.SPINBOX)

    def on_close(self):
        self.parentWidget().close_config(self.size_spb.value())
        



