# -*- coding: utf-8 -*-

# altscreen.py
# This file is part of Oqapy serie 3
#
# Author: Vincent Vande Vyvre <vincent.vandevyvre@oqapy.eu>
# Home: http://www.oqapy.eu
# Copyright: 2017 Vincent Vande Vyvre
# Licence: GPL3
#
# Dual screen handler

import logging
lgg = logging.getLogger(__name__)

from PyQt5.QtCore import (Qt, QLocale, QObject, pyqtSignal, QPoint, QSize, )
from PyQt5.QtGui import QImage, QPixmap, QIcon, QFont
from PyQt5.QtWidgets import (QWidget, QGridLayout, QVBoxLayout, QHBoxLayout, 
                             QGraphicsPixmapItem, QGraphicsView, QGraphicsScene,
                             QToolButton, QLabel, QDoubleSpinBox, QSizePolicy, 
                             QSpacerItem, QMenu, QAction)


import stylesheets as styles
from gui.toolbar import ZoomTools
from contextmenuscene import ContextMenuScene

class AltScreen:
    def __init__(self, core, screen, size, pos):
        """Instanciate the second screen.

        Args:
        core -- instance of OqapyCore
        screen -- instance of QWidget from QDesktopWidget.screen(1)
        size -- the size of the screen
        pos -- coordinates of the left upper corner
        """
        self.core = core
        self.screen = screen
        self.width = size[0]
        self.height = size[1]
        self.scrollBars = self.core.cfg.get(('style', 'scrlbars'))
        self.zoom_step = self.core.cfg.get(('zoom', 'step'))
        self.set_viewer()
        if pos[1] < 100:
            pos = (pos[0], 0)
        self.wdg.move(pos[0], pos[1])
        self.wdg.setStyleSheet(styles.VIEWER)
        self.viewer.set_scrollBars(self.scrollBars)
        self.screen.keyPressEvent = self.keyboard_event

    def set_viewer(self):
        self.wdg = QWidget(self.screen)
        self.layout = QVBoxLayout(self.wdg)
        self.layout.setContentsMargins(2, 2, 2, 2)
        self.header = HeaderBar(self.wdg)
        self.layout.addWidget(self.header)
        self.viewer = Viewer(self, self.wdg)
        self.layout.addWidget(self.viewer)
        zoom = self.header.zoom_wdg
        zoom.zoom_full_btn.clicked.connect(self.full_size)
        zoom.zoom_up_btn.clicked.connect(self.zoom_in)
        zoom.zoom_spb.editingFinished.connect(self.on_zoom_edited)
        zoom.zoom_down_btn.clicked.connect(self.zoom_out)
        zoom.zoom_fit_btn.clicked.connect(self.fit_in_view)

    def set_image(self, image, picture):
        lgg.info("Set %s in alt screen" % type(picture))
        self.header.reset_labels()
        self.viewer.change_image(image)
        self.wdg.showFullScreen()
        try:
            # picture may be a filtering.Backup instance
            self.header.set_data(picture.origin)
        except AttributeError:
            self.header.set_data(picture)

    def update_zoom_widget(self, factor):
        self.header.zoom_wdg.zoom_spb.setValue(factor)

    def keyboard_event(self, event):
        print("Key: ", event.key())

    def zoom_in(self):
        lgg.info("Zoom in")
        self.viewer.set_zoom_step(self.zoom_step)

    def zoom_out(self):
        lgg.info("Zoom out")
        self.viewer.set_zoom_step(-self.zoom_step)

    def set_zoom_tool(self):
        pass

    def center_view(self, coord):
        lgg.info("Center here: %s" % coord)
        self.viewer.set_zoom_step(self.zoom_step)
        x = coord.x() + (coord.x() * self.zoom_step)
        y = coord.y() + (coord.y() * self.zoom_step)
        self.viewer.centerOn(x, y)

    def fit_in_view(self):
        lgg.info("Fit in view")
        self.viewer.fit_in_view()

    def full_size(self, coord=None):
        lgg.info("Full size, pos: %s" % coord)
        self.viewer.set_full_size(coord)

    def toggle_scrollbars(self):
        self.scrollBars = not self.scrollBars
        self.viewer.set_scrollBars(self.scrollBars)

    def on_zoom_edited(self):
        value = self.header.zoom_wdg.zoom_spb.value()
        lgg.info("Zoom: %s" % value)
        zoom = value / 100 - self.zoom_step
        self.viewer.current_zoom = value
        self.viewer.set_zoom_step(self.zoom_step)


class Viewer(QGraphicsView):
    def __init__(self, screen, parent=None):
        super().__init__(parent)
        self.screen = screen
        self.width_ = screen.width
        self.height_ = screen.height
        self.current_zoom = 1
        self.setDragMode(QGraphicsView.ScrollHandDrag)
        self.horizontalScrollBar().setStyleSheet(styles.HSCROLLBAR)
        self.verticalScrollBar().setStyleSheet(styles.VSCROLLBAR)
        self.scene_ = QGraphicsScene()
        self.setScene(self.scene_)

    def set_scrollBars(self, show):
        if show:
            self.setVerticalScrollBarPolicy(Qt.ScrollBarAsNeeded)
            self.setHorizontalScrollBarPolicy(Qt.ScrollBarAsNeeded)

        else:
            self.setVerticalScrollBarPolicy(Qt.ScrollBarAlwaysOff)
            self.setHorizontalScrollBarPolicy(Qt.ScrollBarAlwaysOff)

        self.screen.scrollBars = show

    def change_image(self, image):
        self.scene_.clear()
        if isinstance(image, QImage):
            self.qimage = image
            pix = QPixmap.fromImage(image.scaled(self.width_, self.height_,
                                    Qt.KeepAspectRatio,
                                    Qt.SmoothTransformation))

        self.scene_.setSceneRect(0, 0, pix.width(), pix.height())
        self.scene_.addPixmap(pix)
        self.current_zoom = pix.width() / image.width()
        self.screen.update_zoom_widget(self.current_zoom*100)

    def set_zoom_step(self, step):
        self.scene_.clear()
        zoom = self.current_zoom + step
        width = self.qimage.width() * zoom
        pix = QPixmap.fromImage(self.qimage.scaledToWidth(width, 
                                Qt.SmoothTransformation))
        self.scene_.setSceneRect(0, 0, pix.width(), pix.height())
        self.scene_.addPixmap(pix)
        self.current_zoom = pix.width() / self.qimage.width()
        self.screen.update_zoom_widget(self.current_zoom*100)

    def fit_in_view(self):
        self.change_image(self.qimage)

    def set_full_size(self, pos=False):
        self.scene_.clear()
        pix = QPixmap.fromImage(self.qimage)
        self.scene_.setSceneRect(0, 0, pix.width(), pix.height())
        self.scene_.addPixmap(pix)
        if pos:
            x = int(pos.x() / self.current_zoom)
            y = int(pos.y() / self.current_zoom)
            self.centerOn(x, y)

        self.current_zoom = 1
        self.screen.update_zoom_widget(100)

    def wheelEvent(self, event):
        step = event.angleDelta().y() / 120
        pos = event.pos()
        ipos = self.mapToScene(event.pos())
        old_zoom = self.current_zoom
        steps = step * self.screen.zoom_step
        new_zoom = old_zoom + steps
        if new_zoom < 0.002:
            return

        self.set_zoom_step(steps)
        delta_x = delta_y = 0
        imw, imh = self.qimage.width(), self.qimage.height()
        if imw * new_zoom > self.width():
            delta_x = pos.x() - (ipos.x() / old_zoom) * new_zoom

        if imh * new_zoom > self.height():
            delta_y = pos.y() - (ipos.y() / old_zoom) * new_zoom

        self.centerOn(-delta_x+self.width()/2, -delta_y+self.height()/2)

    def contextMenuEvent(self, event):
        gpos = event.globalPos()
        spos = self.mapToScene(event.pos())
        self.menu = ContextMenu(self.screen, gpos, spos)


class ContextMenu(QMenu):
    def __init__(self, screen, pos, scene_pos, parent=None):
        super().__init__(parent)
        self.screen = screen
        self.scene_pos = scene_pos
        self.scrollBars = self.screen.scrollBars
        self.header = self.screen.header.isVisible()
        self.font_1 = QFont()
        self.font_1.setFamily("Arial")
        self.font_1.setWeight(75)
        self.font_1.setBold(True)
        self.font_2 = QFont()
        self.font_2.setFamily("Trebuchet")
        self.font_2.setWeight(50)
        self.title_1 = QAction(None)
        self.title_1.setFont(self.font_1)
        self.title_1.setText(_("Zoom"))
        self.addAction(self.title_1)
        self.zoom_in = QAction(None)
        self.zoom_in.setText(_("Zoom in"))
        self.zoom_in.setFont(self.font_2)
        self.addAction(self.zoom_in)
        self.zoom_out = QAction(None)
        self.zoom_out.setText(_("Zoom out"))
        self.zoom_out.setFont(self.font_2)
        self.addAction(self.zoom_out)
        self.activ_zoom = QAction(None)
        self.activ_zoom.setText(_("Zoom on area"))
        self.activ_zoom.setFont(self.font_2)
        #self.addAction(self.activ_zoom)
        self.center = QAction(None)
        self.center.setText(_("Center here"))
        self.center.setFont(self.font_2)
        self.addAction(self.center)
        self.fit_in_view = QAction(None)
        self.fit_in_view.setText(_("Fit in view"))
        self.fit_in_view.setFont(self.font_2)
        self.addAction(self.fit_in_view)
        self.full_size = QAction(None)
        self.full_size.setText(_("Full size"))
        self.full_size.setFont(self.font_2)
        self.addAction(self.full_size)
        self.addSeparator()
        self.title_2 = QAction(None)
        self.title_2.setFont(self.font_1)
        self.title_2.setText(_("Window"))
        self.addAction(self.title_2)
        self.show_header = QAction(None)
        self.show_header.setText(_("Show the header bar"))
        if self.header:
            self.show_header.setText(_("Hide the header bar"))
        self.show_header.setFont(self.font_2)
        self.addAction(self.show_header)
        self.show_scrollbars = QAction(None)
        self.show_scrollbars.setText(_("Show the scroll bars"))
        if self.scrollBars:
            self.show_scrollbars.setText(_("Hide the scroll bars"))
        self.show_scrollbars.setFont(self.font_2)
        self.addAction(self.show_scrollbars)
        self.zoom_in.triggered.connect(self.screen.zoom_in)
        self.zoom_out.triggered.connect(self.screen.zoom_out)
        #self.activ_zoom.triggered.connect(self.screen.set_zoom_tool)
        self.center.triggered.connect(self.on_center_here)
        self.fit_in_view.triggered.connect(self.screen.fit_in_view)
        self.full_size.triggered.connect(self.on_full_size)
        self.show_header.triggered.connect(self.screen.header.toggle_headerbar)
        self.show_scrollbars.triggered.connect(self.screen.toggle_scrollbars)
        self.popup(pos)

    def on_center_here(self):
        self.screen.center_view(self.scene_pos)

    def on_full_size(self):
        self.screen.full_size(self.scene_pos)

class HeaderBar(QWidget):
    def __init__(self, parent=None):
        super().__init__(parent)
        self.setMaximumSize(QSize(16777215, 35))
        self.gl = QGridLayout(self)
        self.gl.setContentsMargins(0, 0, 0, 0)
        self.hl = QHBoxLayout()
        self.hl.setContentsMargins(15, -1, -1, -1)
        self.fname_lbl = QLabel(self)
        self.hl.addWidget(self.fname_lbl)
        sp = QSpacerItem(40, 20, QSizePolicy.Expanding, QSizePolicy.Minimum)
        self.hl.addItem(sp)
        self.iso_lbl = QLabel(self)
        self.hl.addWidget(self.iso_lbl)
        self.fnumber_lbl = QLabel(self)
        self.hl.addWidget(self.fnumber_lbl)
        self.speed_lbl = QLabel(self)
        self.hl.addWidget(self.speed_lbl)
        sp1 = QSpacerItem(40, 20, QSizePolicy.Expanding, QSizePolicy.Minimum)
        self.hl.addItem(sp1)
        self.zoom_wdg = ZoomTools(self)
        #self.zoom_wdg.setMaximumSize(QSize(16777215, 35))
        self.hl.addWidget(self.zoom_wdg)
        sp2 = QSpacerItem(40, 20, QSizePolicy.Expanding, QSizePolicy.Minimum)
        self.hl.addItem(sp2)
        self.setting_btn = QToolButton(self)
        icon = QIcon()
        icon.addPixmap(QPixmap("medias/settings.png"), QIcon.Normal, QIcon.Off)
        self.setting_btn.setIcon(icon)
        self.hl.addWidget(self.setting_btn)
        self.close_btn = QToolButton(self)
        icon1 = QIcon()
        icon1.addPixmap(QPixmap("medias/close.png"), QIcon.Normal, QIcon.Off)
        self.close_btn.setIcon(icon1)
        self.close_btn.setToolTip(_("Hide the header bar"))
        self.hl.addWidget(self.close_btn)
        self.gl.addLayout(self.hl, 0, 0, 1, 1)
        self.setStyleSheet(styles.WIDGET)
        self.fname_lbl.setStyleSheet(styles.TITLE)
        self.iso_lbl.setStyleSheet(styles.TITLE)
        self.fnumber_lbl.setStyleSheet(styles.TITLE)
        self.speed_lbl.setStyleSheet(styles.TITLE)
        self.setting_btn.setStyleSheet(styles.TOOLBUTTON)
        self.close_btn.setStyleSheet(styles.TOOLBUTTON)
        self.reset_labels()
        self.close_btn.clicked.connect(self.toggle_headerbar)

        self.zoom_wdg.zoom_area_btn.hide()
        self.setting_btn.hide()

    def reset_labels(self):
        self.fname_lbl.setText("No Image")
        self.iso_lbl.setText("ISO: -- ")
        self.fnumber_lbl.setText("f:-- ")
        self.speed_lbl.setText("1/--")

    def set_data(self, image):
        self.fname_lbl.setText(image.base)
        if image.metadata is not None:
            values = image.metadata.get_exposure_values()
            if values["iso"] is not None:
                self.iso_lbl.setText("ISO: %s " % values["iso"])
            if values["fn"] is not None:
                self.fnumber_lbl.setText("f:%s " % values["fn"])
            speed = values["exp"]
            if speed is not None:
                if speed.denominator == 1:
                    self.speed_lbl.setText("%s sec." % speed.numerator)
                else:
                    self.speed_lbl.setText(("/".join(["1", 
                            str(int(speed.denominator / speed.numerator))])))

    def toggle_headerbar(self):
        self.setVisible(not self.isVisible())





