# -*- coding: utf-8 -*-

# colortogreybox.py
# this file is part of Oqapy serie 3
#
# Author: Vincent Vande Vyvre <vincent.vandevyvre@oqapy.eu>
# Home: http://www.oqapy.eu
# Copyright: 2014-2017 Vincent Vande Vyvre
# Licence: GPL3
#
# Convert colored image to grey map tool box

from PyQt5.QtCore import Qt, pyqtSignal, QSize, QCoreApplication
from PyQt5.QtGui import QPixmap, QIcon
from PyQt5.QtWidgets import (QWidget, QGridLayout, QVBoxLayout, QHBoxLayout,
                             QSpacerItem, QSizePolicy, QCheckBox, QPushButton,
                             QSlider, QLabel, QProgressBar)

import stylesheets as styles

class ColorToGreyBox(QWidget):
    getFocus = pyqtSignal(QWidget)
    def __init__(self, ui, parent=None):
        super().__init__(parent)
        self.name = 'ColorToGreyBox'
        self.ui = ui
        self.core = ui.core
        self.gridLayout_2 = QGridLayout(self)
        self.gridLayout_2.setContentsMargins(0, 2, 8, 2)
        self.gridLayout_2.setVerticalSpacing(0)
        self.verticalLayout = QVBoxLayout()
        self.gridLayout = QGridLayout()
        sp = QSpacerItem(20, 20, QSizePolicy.Fixed, QSizePolicy.Minimum)
        self.gridLayout.addItem(sp, 0, 0, 1, 1)
        self.standard_chb = QCheckBox(self)
        self.gridLayout.addWidget(self.standard_chb, 0, 1, 1, 1)
        sp1 = QSpacerItem(40, 20, QSizePolicy.Expanding, QSizePolicy.Minimum)
        self.gridLayout.addItem(sp1, 0, 2, 1, 1)
        sp2 = QSpacerItem(20, 20, QSizePolicy.Fixed, QSizePolicy.Minimum)
        self.gridLayout.addItem(sp2, 1, 0, 1, 1)
        self.contrast_chb = QCheckBox(self)
        self.gridLayout.addWidget(self.contrast_chb, 1, 1, 1, 1)
        self.horizontalLayout = QHBoxLayout()
        self.ico_lbl = QLabel(self)
        self.ico_lbl.setPixmap(QPixmap("medias/contrast.png"))
        self.horizontalLayout.addWidget(self.ico_lbl)
        self.contrast_sld = QSlider(self)
        self.contrast_sld.setMaximum(5)
        self.contrast_sld.setPageStep(1)
        self.contrast_sld.setOrientation(Qt.Horizontal)
        self.horizontalLayout.addWidget(self.contrast_sld)
        self.ico_2_lbl = QLabel(self)
        self.ico_2_lbl.setPixmap(QPixmap("medias/contrast_2.png"))
        self.horizontalLayout.addWidget(self.ico_2_lbl)
        self.gridLayout.addLayout(self.horizontalLayout, 1, 2, 1, 1)
        self.verticalLayout.addLayout(self.gridLayout)
        self.horizontalLayout_2 = QHBoxLayout()
        sp4 = QSpacerItem(40, 20, QSizePolicy.Expanding, QSizePolicy.Minimum)
        self.horizontalLayout_2.addItem(sp4)
        self.progress = QProgressBar(self)
        self.progress.setMaximumSize(QSize(100, 12))
        self.horizontalLayout_2.addWidget(self.progress)
        self.cancel_btn = QPushButton(self)
        icon = QIcon('medias/edit-undo.png')
        self.cancel_btn.setIcon(icon)
        self.cancel_btn.setIconSize(QSize(18, 18))
        self.horizontalLayout_2.addWidget(self.cancel_btn)
        self.apply_btn = QPushButton(self)
        icon1 = QIcon('medias/apply.png')
        self.apply_btn.setIcon(icon1)
        self.apply_btn.setIconSize(QSize(18, 18))
        self.horizontalLayout_2.addWidget(self.apply_btn)
        self.verticalLayout.addLayout(self.horizontalLayout_2)
        self.gridLayout_2.addLayout(self.verticalLayout, 0, 0, 1, 1)

        self.cancel_btn.setEnabled(False)
        self.apply_btn.setEnabled(False)
        self.contrast_sld.setEnabled(False)

        self.standard_chb.setText(_("Standard"))
        self.standard_chb.setStatusTip(_("Use the common conversion to grey map"))
        self.contrast_chb.setText(_("Contrast"))
        self.contrast_chb.setStatusTip(_("Use a contrasted grey map"))
        self.cancel_btn.setText(_("Cancel"))
        self.cancel_btn.setStatusTip(_("Cancel the grey map conversion"))
        self.apply_btn.setText(_("Apply"))
        self.apply_btn.setStatusTip(_("Apply the grey map conversion"))
        self.set_style()
        self.progress.hide()

        self.standard_chb.stateChanged.connect(self.on_standard_toggled)
        self.contrast_chb.stateChanged.connect(self.on_contrast_toggled)
        self.contrast_sld.valueChanged.connect(self.on_value_changed)
        self.cancel_btn.clicked.connect(self.on_cancel_clicked)
        self.apply_btn.clicked.connect(self.on_apply_clicked)

    def on_standard_toggled(self, state):
        if state:
            self.getFocus.emit(self)
            self.unchek_contrast()
            self.apply_btn.setEnabled(True)
            self.core.set_filter_previewing_mode(True)
            self.core.filtering.show_toning_preview('greymap', 0)

        else:
            self.core.filtering.cancel_toning_preview()
            self.apply_btn.setEnabled(False)

    def on_contrast_toggled(self, state):
        if state:
            self.getFocus.emit(self)
            self.unchek_standard()
            self.apply_btn.setEnabled(True)
            self.contrast_sld.setEnabled(True)
            self.core.set_filter_previewing_mode(True)
            self.core.filtering.show_toning_preview('greyscale', 
                                self.contrast_sld.value())

        else:
            self.core.filtering.cancel_toning_preview()
            self.core.set_filter_previewing_mode(False)
            self.contrast_sld.blockSignals(True)
            self.contrast_sld.setEnabled(False)
            self.contrast_sld.setValue(0)
            self.contrast_sld.blockSignals(False)
            self.apply_btn.setEnabled(False)

    def on_value_changed(self, value):
        self.core.filtering.show_toning_preview('greyscale', 
                                                self.contrast_sld.value())

    def on_cancel_clicked(self):
        self.cancel_btn.setEnabled(False)
        self.core.filtering.undo_last_change()

    def on_apply_clicked(self):
        self.cancel_btn.setEnabled(True)
        tone = 'greymap'
        contrast = 0
        if self.contrast_chb.isChecked():
            tone = 'greyscale'
            contrast = self.contrast_sld.value()

        self.set_progress_bar(True)
        self.core.state.disable_undo_buttons(self)
        self.core.set_filtered_mode(True)
        self.core.filtering.grayscale_filter(tone, contrast, self)
        self.uncheck_all()
        self.apply_btn.setEnabled(False)

    def unchek_contrast(self):
        self.contrast_chb.blockSignals(True)
        self.contrast_sld.blockSignals(True)
        self.contrast_chb.setChecked(False)
        self.contrast_sld.setEnabled(False)
        self.contrast_sld.setValue(0)
        self.contrast_chb.blockSignals(False)
        self.contrast_sld.blockSignals(False)

    def unchek_standard(self):
        self.standard_chb.blockSignals(True)
        self.standard_chb.setChecked(False)
        self.standard_chb.blockSignals(False)

    def uncheck_all(self):
        self.unchek_contrast()
        self.unchek_standard()
        self.apply_btn.setEnabled(False)

    def unset_focus(self):
        self.uncheck_all()

    def set_progress_bar(self, b):
        if b:
            self.progress.setRange(0, 0)
            self.progress.show()
            QCoreApplication.processEvents()

        else:
            self.progress.setRange(0, 10)
            self.progress.hide()

    def reset(self):
        self.unchek_contrast()
        self.unchek_standard()
        self.set_progress_bar(False)

    def set_style(self):
        self.standard_chb.setStyleSheet(styles.CHECKBOX)
        self.contrast_chb.setStyleSheet(styles.CHECKBOX)
        self.contrast_sld.setStyleSheet(styles.SLIDER)
        self.progress.setStyleSheet(styles.PROGRESSBAR)
        self.cancel_btn.setStyleSheet(styles.PUSHBUTTON)
        self.apply_btn.setStyleSheet(styles.PUSHBUTTON)
