# -*- coding: utf-8 -*-

# contraststretchingbox.py
# this file is part of Oqapy serie 3
#
# Author: Vincent Vande Vyvre <vincent.vandevyvre@oqapy.eu>
# Home: http://www.oqapy.eu
# Copyright: 2013-2017 Vincent Vande Vyvre
# Licence: GPL3
#
# Contrast stretching tool box.

from PyQt5.QtCore import Qt, QSize, QCoreApplication
from PyQt5.QtGui import QIcon
from PyQt5.QtWidgets import (QWidget, QGridLayout, QVBoxLayout, QHBoxLayout,
                             QSpacerItem, QSizePolicy, QLabel, QDoubleSpinBox,
                             QPushButton, QProgressBar)

import stylesheets as styles

class ContrastStretchingBox(QWidget):
    def __init__(self, core, parent=None):
        super().__init__(parent)
        self.name = 'ContrastStretchingBox'
        self.i18n_name = 'Contrast stretching'
        self.core = core
        self.glayout_2 = QGridLayout(self)
        self.glayout_2.setContentsMargins(0, 2, 8, 2)
        self.vlayout = QVBoxLayout()
        self.vlayout.setSpacing(0)
        self.hlayout_6 = QHBoxLayout()
        spacer = QSpacerItem(20, 20, QSizePolicy.Expanding, QSizePolicy.Minimum)
        self.hlayout_6.addItem(spacer)
        self.glayout = QGridLayout()
        self.glayout.setHorizontalSpacing(6)
        self.label = QLabel(self)
        self.label.setAlignment(Qt.AlignRight|Qt.AlignVCenter)
        self.glayout.addWidget(self.label, 0, 0, 1, 1)
        self.hlayout = QHBoxLayout()
        self.hlayout.setSpacing(8)
        self.label_2 = QLabel(self)
        self.label_2.setAlignment(Qt.AlignRight|Qt.AlignVCenter)
        self.hlayout.addWidget(self.label_2)
        self.upper_spb = QDoubleSpinBox(self)
        self.upper_spb.setSuffix(' %')
        self.upper_spb.setMaximum(10.0)
        self.upper_spb.setSingleStep(0.1)
        self.upper_spb.setProperty("value", 0.0)
        self.hlayout.addWidget(self.upper_spb)
        self.glayout.addLayout(self.hlayout, 0, 1, 1, 1)
        spacer2 = QSpacerItem(40, 20, QSizePolicy.Preferred, QSizePolicy.Minimum)
        self.glayout.addItem(spacer2, 1, 0, 1, 1)
        self.hlayout_2 = QHBoxLayout()
        self.hlayout_2.setSpacing(8)
        self.label_3 = QLabel(self)
        self.label_3.setAlignment(Qt.AlignRight|Qt.AlignVCenter)
        self.hlayout_2.addWidget(self.label_3)
        self.lower_spb = QDoubleSpinBox(self)
        self.lower_spb.setSuffix(' %')
        self.lower_spb.setMaximum(10.0)
        self.lower_spb.setSingleStep(0.1)
        self.lower_spb.setProperty("value", 0.0)
        self.hlayout_2.addWidget(self.lower_spb)
        self.glayout.addLayout(self.hlayout_2, 1, 1, 1, 1)
        self.hlayout_6.addLayout(self.glayout)
        self.vlayout.addLayout(self.hlayout_6)

        self.hlayout_3 = QHBoxLayout()
        self.hlayout_3.setContentsMargins(20, 0, 5, 0)
        self.progress = QProgressBar(self)
        self.progress.setSizePolicy(QSizePolicy.Expanding, QSizePolicy.Fixed)
        self.progress.setMaximumSize(QSize(1000, 12))
        self.hlayout_3.addWidget(self.progress)
        self.vlayout.addLayout(self.hlayout_3)

        self.hlayout_4 = QHBoxLayout()
        spacer4 = QSpacerItem(40, 20, QSizePolicy.Expanding, QSizePolicy.Minimum)
        self.hlayout_4.addItem(spacer4)
        self.preview_btn = QPushButton(self)
        self.preview_btn.setEnabled(False)
        icon = QIcon('medias/eye_red.png')
        self.preview_btn.setIcon(icon)
        self.preview_btn.setIconSize(QSize(30, 14))
        self.hlayout_4.addWidget(self.preview_btn)
        self.cancel_btn = QPushButton(self)
        self.cancel_btn.setEnabled(False)
        icon = QIcon('medias/edit-undo.png')
        self.cancel_btn.setIcon(icon)
        self.cancel_btn.setIconSize(QSize(18, 18))
        self.hlayout_4.addWidget(self.cancel_btn)
        self.apply_btn = QPushButton(self)
        self.apply_btn.setEnabled(False)
        icon = QIcon('medias/apply.png')
        self.apply_btn.setIcon(icon)
        self.apply_btn.setIconSize(QSize(18, 18))
        self.hlayout_4.addWidget(self.apply_btn)

        self.vlayout.addLayout(self.hlayout_4)
        self.glayout_2.addLayout(self.vlayout, 0, 0, 1, 1)

        self.progress.hide()
        self.upper_spb.valueChanged.connect(self.on_value_changed)
        self.lower_spb.valueChanged.connect(self.on_value_changed)
        self.preview_btn.clicked.connect(self.on_preview_clicked)
        self.apply_btn.clicked.connect(self.apply_contrast)
        self.cancel_btn.clicked.connect(self.undo)

        self.translate()
        self.set_style()

    def get_values(self):
        return self.lower_spb.value(), self.upper_spb.value()

    def on_value_changed(self, val):
        b = val != 0.0 or self.lower_spb.value() != 0.0
        self.apply_btn.setEnabled(b)
        self.preview_btn.setEnabled(b)

    def on_preview_clicked(self):
        """Call the preview window.

        """
        self.preview_btn.setEnabled(False)
        self.core.set_preview_window(self)

    def apply_contrast(self):
        self.core.state.disable_undo_buttons(self)
        self.core.close_preview_window()
        self.set_progress_bar(True)
        self.core.set_filtered_mode(True)
        QCoreApplication.processEvents()
        self.core.filtering.apply_contrast_stretching(self.get_values(), self)
        self.apply_btn.setEnabled(False)
        self.preview_btn.setEnabled(False) 
        self.reset()

    def undo(self):
        self.core.filtering.undo_last_change()
        self.cancel_btn.setEnabled(False)

    def reset(self):
        self.upper_spb.blockSignals(True)
        self.lower_spb.blockSignals(True)
        self.upper_spb.setValue(0.0)
        self.lower_spb.setValue(0.0)
        self.upper_spb.blockSignals(False)
        self.lower_spb.blockSignals(False)
        self.set_progress_bar(False)

    def set_progress_bar(self, b):
        if b:
            self.progress.setRange(0, 0)
            self.progress.show()

        else:
            self.progress.setRange(0, 10)
            self.progress.hide()

        QCoreApplication.processEvents()

    def set_style(self):
        self.upper_spb.setStyleSheet(styles.DSPINBOX)
        self.lower_spb.setStyleSheet(styles.DSPINBOX)
        self.progress.setStyleSheet(styles.PROGRESSBAR)
        self.preview_btn.setStyleSheet(styles.PUSHBUTTON)
        self.apply_btn.setStyleSheet(styles.PUSHBUTTON)
        self.cancel_btn.setStyleSheet(styles.PUSHBUTTON)

    def translate(self):
        self.label.setText(_("Tresholds:"))
        self.label_2.setText(_("Upper:"))
        self.label_3.setText(_("Lower:"))
        self.preview_btn.setStatusTip(_("Show the preview"))
        self.apply_btn.setText(_("Apply"))
        self.cancel_btn.setText(_("Cancel"))
        self.apply_btn.setStatusTip(_("Apply the contrast"))
        self.cancel_btn.setStatusTip(_("Cancel the contrast"))

