# editingarea.py
# this file is part of Oqapy serie 3
#
# Author: Vincent Vande Vyvre <vincent.vandevyvre@oqapy.eu>
# Home: http://www.oqapy.eu
# Copyright: 2013-2016 Vincent Vande Vyvre
# Licence: GPL3
#
# Main widget for the image editing

import os
import logging
lgg = logging.getLogger(__name__)

from PyQt5.QtCore import Qt, pyqtSignal, QCoreApplication
from PyQt5.QtGui import QIcon, QPixmap, QImage
from PyQt5.QtWidgets import (QWidget, QGridLayout, QVBoxLayout, QHBoxLayout,
                             QToolButton, QSpacerItem, QSizePolicy, QLabel,
                             QGraphicsView, QGraphicsScene, QMenu, QComboBox,
                             QAction, QActionGroup)

#import stylesheets as styles

from gui.graphicsarea import GraphicsArea
#from ioimage import IOImage

class EditingArea(QWidget):
    """Define the widgets for the image transformation mode.

    """
    def __init__(self, ui, parent=None):
        super(EditingArea, self).__init__(parent)
        self.ui = ui
        self.main = ui.main
        self.layout = QVBoxLayout(self)
        self.layout.setSpacing(0)
        self.layout.setContentsMargins(0, 0, 0, 0)
        self.hl = QHBoxLayout()
        self.hl.setSpacing(2)
        self.hl.setContentsMargins(0, 0, 0, 0)
        self.layout.addLayout(self.hl)
        self.vl = QVBoxLayout()
        self.vl.setSpacing(0)
        self.vl.setContentsMargins(0, 0, 0, 0)
        self.header = MainHeader(self)
        self.vl.addWidget(self.header)
        self.raw_header = RawMainHeader(self)
        self.vl.addWidget(self.raw_header)
        self.raw_header.hide()
        self.viewer = GraphicsArea(self.ui, self)
        self.viewer.set_scene()
        self.vl.addWidget(self.viewer)
        self.hl.addLayout(self.vl)
        self.alt_view = None
        self.layout_mode = 0
        self.backups = []
        self.viewer.is_active = True
        self.views_are_sync = True
        self.sync_on_zoom = True
        self.is_auto_update = False

        self.viewer.sizeChanged.connect(self.on_viewer_size_changed)
        self.viewer.zoomFactorChanged.connect(self.on_main_zoom_changed)
        self.viewer.horizontalScrollBar().valueChanged.connect(
                                            self.on_h_scroll_event)
        self.viewer.verticalScrollBar().valueChanged.connect(
                                            self.on_v_scroll_event)
        self.header.exit_btn.clicked.connect(self.back_to_viewing)

        color = self.main.cfg[1010]
        self.viewer_style = "QWidget {background-color: %s}" % color
        #self.viewer.setStyleSheet(self.viewer_style)
        #self.viewer.horizontalScrollBar().setStyleSheet(styles.HSCROLLBAR)
        #self.viewer.verticalScrollBar().setStyleSheet(styles.VSCROLLBAR)
        self.set_scrollBars(self.main.cfg[1026])

    def change_layout_mode(self, mode):
        """Change the layout used for the QGraphicsView.

        There're three modes: 0 -- 0ne View
                              1 -- two horizontal views
                              2 -- two vertical views

        Args:
        mode -- int(index of mode)
        """
        lgg.info('Change layout mode: %s' % mode)
        if self.layout_mode:
            self.remove_alt_view()

        self.layout_mode = mode
        if not mode:
            self.update_header()
            QCoreApplication.processEvents()
            # This slot is not called when the layout_mode is changed to 0
            self.on_viewer_size_changed()
            #self.viewer.fit_in_view()
            #self.viewer.refresh_qpixmap()
            return

        self.alt_view = QWidget(self)
        self.vl1 = QVBoxLayout(self.alt_view)
        self.vl1.setSpacing(0)
        self.vl1.setContentsMargins(0, 0, 0, 0)
        self.alt_header = AltHeader(self.views_are_sync, self)
        self.vl1.addWidget(self.alt_header)
        self.alt_viewer = GraphicsArea(self.ui, self)
        self.alt_viewer.set_scene()
        self.vl1.addWidget(self.alt_viewer)
        if mode == 1:
            self.hl.addWidget(self.alt_view)

        else:
            self.layout.addWidget(self.alt_view)

        #self.alt_viewer.setStyleSheet(self.viewer_style)
        self.set_scrollBars(self.main.cfg[1026])
        self.update_header()
        self.alt_viewer.horizontalScrollBar().valueChanged.connect(
                                            self.on_alt_h_scroll_event)
        self.alt_viewer.verticalScrollBar().valueChanged.connect(
                                            self.on_alt_v_scroll_event)
        QCoreApplication.processEvents()
        #self.alt_viewer.horizontalScrollBar().setStyleSheet(styles.HSCROLLBAR)
        #self.alt_viewer.verticalScrollBar().setStyleSheet(styles.VSCROLLBAR)
        self.update_alt_view(1)
        self.viewer.refresh_qpixmap()
        self.alt_viewer.zoomFactorChanged.connect(self.on_alt_zoom_changed)
        self.alt_header.update_backup_list()

    def set_raw_mode(self, raw):
        self.header.setVisible(not raw)
        self.raw_header.setVisible(raw)
        self.raw_header.set_folders(self.main.recents.paths)

    def clean(self):
        """Set the mode 'one view' and clear the main view.

        """
        if self.alt_view is not None:
            self.remove_alt_view()

        self.layout_mode = 0
        self.header.update_buttons_state(0)
        self.backups = []
        self.imgchanges.clean_historic()
        self.viewer.clear()

    def set_origin(self, imgchanges):
        self.imgchanges = imgchanges
        self.backups = [imgchanges.origin]

    def remove_alt_view(self):
        """Remove the second view.

        """
        self.alt_viewer.clear()
        if self.layout_mode == 1:
            self.hl.removeWidget(self.alt_view)

        else:
            self.layout.removeWidget(self.alt_view)

        self.alt_view.setParent(None)
        self.alt_view.deleteLater()
        self.alt_view = None

    def update_header(self):
        """Update the backup name into the main header.

        """
        name = ""
        if self.backups:
            name = os.path.basename(self.backups[0])
            self.header.backup_lbl.setText(_(u"Backup: %s" % name))

    def set_scrollBars(self, b):
        """Show or hide the scrollBars.

        Args:
        b -- boolean
        """
        value = (Qt.ScrollBarAlwaysOff, Qt.ScrollBarAsNeeded)[b]
        self.viewer.setVerticalScrollBarPolicy(value)
        self.viewer.setHorizontalScrollBarPolicy(value)

        if self.layout_mode != 0:
            self.alt_viewer.setVerticalScrollBarPolicy(value)
            self.alt_viewer.setHorizontalScrollBarPolicy(value)
            
    def make_backup(self, fname):
        """Save the last transformed image.

        Args:
        fname -- the full path of the backup
        """
        lgg.info(u'Save backup: %s' % fname)
        iproc = IOImage(self)
        iproc.set_filename_to_writer(fname)
        reply = iproc.writer.save(self.viewer.qimage, 'tiff', 0)
        if reply[0]:
            self.add_backup(fname)

        return reply

    def add_backup(self, backup):
        """Insert a new backup at the top of the backup list.

        Args:
        backup -- the full path of the backup
        """
        if backup in self.backups:
            # The backup has been already created by toning effect
            return

        lgg.info('Insert new backup %s' % backup)
        # backups list is a stack
        self.backups.insert(0, backup)
        self.update_header()
        if self.alt_view is not None:
            self.alt_header.update_backup_list()
            self.update_alt_view(1)

        self.check_backups()

    def remove_last_backup(self):
        """Remove the last created backup.

        """
        lgg.info('Remove last backup, %s' % self.backups)
        self.backups.pop(0)
        self.check_backups()
        if self.alt_view is not None:
            self.alt_header.update_backup_list()

        if self.backups:
            self.reload_views()

    def remove_backups(self, numb):
        """Remove one or more backup.

        """
        lgg.info('Remove %s backups' % numb)
        l = len(self.backups)
        if numb >= l:
            self.back_to_viewing()
            return

        self.backups = self.backups[numb:]
        self.check_backups()
        if self.alt_view is not None:
            self.alt_header.update_backup_list()

        self.reload_views()

    def reload_views(self):
        """Reload and show a backup. 

        Called when the last backup has been removed
        """
        lgg.info('reload_views mode: {0}'.format(self.layout_mode))
        self.viewer.set_qimage_from_backup(self.backups[0], 
                                           self.main.cur_img.oriented)
        self.viewer.refresh_qpixmap()
        self.update_header()
        if self.layout_mode > 0:
            self.update_alt_view(1)

        self.imgchanges.on_last_backup_changed(self.viewer.qimage)

    def update_alt_view(self, idx):
        """Refresh the second view.

        Args:
        idx -- the index of the backup
        """
        lgg.info('Update alt view idx: {0}'.format(idx))
        if self.alt_view is not None:
            self.alt_viewer.set_qimage_from_backup(self.backups[idx])
            if len(self.backups) == 2:
                oriented = self.imgchanges.source_image.oriented
                if oriented:
                    self.alt_viewer.rotate_image(oriented)

            if self.views_are_sync:
                self.resize_alt_view()

            else:
                self.alt_viewer.refresh_qpixmap()

    def resize_main_view(self):
        """Resize the main view.

        This called when the alt view is zoomed and the views are linked.
        """
        if self.sync_on_zoom:
            val = self.alt_viewer.get_zoom_factor()

        else:
            val = self.alt_viewer.get_qpixmap_size()

        self.viewer.refresh_qpixmap(val)

    def resize_alt_view(self):
        """Resize the second image.

        """
        if self.sync_on_zoom:
            val = self.viewer.get_zoom_factor()

        else:
            mw, mh = self.viewer.get_qpixmap_size()
            aw, ah = self.alt_viewer.get_qimage_size()
            if self.layout_mode == 1:
                # Compute the same height
                dh = mh
                dw = dh * (float(aw) / ah)

            else:
                # Compute the same width
                dw = mw
                dh = dw * (float(ah) / aw)

            val = dw, dh

        self.alt_viewer.refresh_qpixmap(val)
        self.update_alt_zoom_info()
        val = self.alt_viewer.get_zoom_factor()

    def check_backups(self):
        """Verify the number of backups.

        """
        if not self.backups:
            # Historic is empty, exit the transformation state
            self.back_to_viewing()
            return

        elif len(self.backups) == 1:
            self.header.change_mode(0)
            enable = False

        else:
            enable = True

        self.header.mode_buttons[1].setEnabled(enable)
        self.header.mode_buttons[2].setEnabled(enable)

    def enable_headers(self, b):
        """Enable or disable the header bars.

        Args:
        b -- bool
        """
        self.header.setEnabled(b)
        if self.alt_view is not None:
            self.alt_header.setEnabled(b)

    def update_alt_zoom_info(self, val=None):
        if val is None:
            val = self.alt_viewer.get_zoom_factor()

        zoom = '%.3f' % (val * 100)
        self.alt_header.zoom_lbl.setText('Zoom: {0} %'.format(zoom))

    #===============================================================
    # Slots
    #===============================================================
    def on_viewer_size_changed(self):
        """Called by the QGraphicsView.resizeEvent.

        """
        width = self.width()
        height = self.height() - self.header.height()
        if not self.layout_mode:
            self.viewer.fit_in_view()

        elif self.layout_mode == 1:
            width = width / 2
            self.alt_viewer.setMaximumSize(width, height)

        elif self.layout_mode == 2:
            height = height / 2
            self.alt_viewer.setMaximumSize(width, height)

    def on_main_zoom_changed(self, factor):
        """Called when the main view is zoomed.

        Args:
        factor -- the zoom factor
        """
        zoom = '%.3f' % (factor * 100)
        self.header.zoom_lbl.setText('Zoom: {0} %'.format(zoom))
        self.main.set_zoom(factor)
        self.main.update_navigation()
        if self.layout_mode > 0 and self.views_are_sync:
            self.alt_viewer.blockSignals(True)
            self.resize_alt_view()
            self.alt_viewer.blockSignals(False)

    def on_alt_zoom_changed(self, factor):
        """Called when the second view is zoomed.

        Args:
        factor -- the zoom factor
        """
        self.update_alt_zoom_info(factor)
        if self.views_are_sync:
            self.viewer.blockSignals(True)
            self.resize_main_view()
            self.viewer.blockSignals(False)

    def on_h_scroll_event(self, value):
        """Called when the horizontal scroll bar of the main view is moved.

        Args:
        value -- the current value of the scroll bar
        """
        if self.alt_view is not None and self.views_are_sync:
            self.alt_viewer.blockSignals(True)
            self.alt_viewer.horizontalScrollBar().setValue(value)
            self.alt_viewer.blockSignals(False)

    def on_v_scroll_event(self, value):
        """Called when the vertical scroll bar of the main view is moved.

        Args:
        value -- the current value of the scroll bar
        """
        if self.alt_view is not None and self.views_are_sync:
            self.alt_viewer.blockSignals(True)
            self.alt_viewer.verticalScrollBar().setValue(value)
            self.alt_viewer.blockSignals(False)

    def on_alt_h_scroll_event(self, value):
        """Called when the horizontal scroll bar of the alt view is moved.

        Args:
        value -- the current value of the scroll bar
        """
        if self.views_are_sync:
            self.viewer.blockSignals(True)
            self.viewer.horizontalScrollBar().setValue(value)
            self.viewer.blockSignals(False)

    def on_alt_v_scroll_event(self, value):
        """Called when the vertical scroll bar of the alt view is moved.

        Args:
        value -- the current value of the scroll bar
        """
        if self.views_are_sync:
            self.viewer.blockSignals(True)
            self.viewer.verticalScrollBar().setValue(value)
            self.viewer.blockSignals(False)

    def back_to_viewing(self):
        """Close the editing mode.

        """
        self.clean()
        self.imgchanges.close_changes()
        self.main.set_transform_state(False)

    def on_quick_save_clicked(self):
        dest = unicode(self.raw_header.folders_cmb.currentText())
        self.main.raw_viewer.save_current_image(dest)

class RawMainHeader(QWidget):
    """Define the header bar for the main view.

    """
    def __init__(self, parent=None):
        super(RawMainHeader, self).__init__(parent)
        self.parent = parent
        self.setMaximumSize(99999, 24)
        self.gridLayout = QGridLayout(self)
        self.gridLayout.setContentsMargins(12, 2, 12, 2)
        self.hLayout = QHBoxLayout()
        self.hLayout.setSpacing(12)
        sp = QSpacerItem(50, 20, QSizePolicy.Fixed, QSizePolicy.Minimum)
        self.hLayout.addItem(sp)
        self.preview_lbl = QLabel(self)
        self.hLayout.addWidget(self.preview_lbl)
        self.fname_lbl = QLabel(self)
        self.hLayout.addWidget(self.fname_lbl)
        self.unsaved_lbl = QLabel(self)
        self.hLayout.addWidget(self.unsaved_lbl)
        sp1 = QSpacerItem(40, 20, QSizePolicy.Fixed, QSizePolicy.Minimum)
        self.hLayout.addItem(sp1)
        self.save_btn = QToolButton(self)
        icon = QIcon()
        icon.addPixmap(QPixmap("medias/export_th.png"), 
                                    QIcon.Normal, QIcon.Off)
        self.save_btn.setIcon(icon)
        self.save_btn.setStatusTip(_("Quick save"))
        self.hLayout.addWidget(self.save_btn)
        self.folders_cmb = QComboBox(self)
        self.hLayout.addWidget(self.folders_cmb)
        sp2 = QSpacerItem(50, 20, QSizePolicy.Expanding, QSizePolicy.Minimum)
        self.hLayout.addItem(sp2)
        self.gridLayout.addLayout(self.hLayout, 0, 0, 1, 1)
        self.preview_lbl.setText(_('Preview of:'))
        self.unsaved_lbl.setText(_('(unsaved)'))
        self.set_style()
        self.set_preview_state(False, False)

        self.save_btn.clicked.connect(self.parent.on_quick_save_clicked)

    def set_preview_state(self, prev, unsaved=False):
        ics = ["medias/export_th.png", "medias/export_th_active.png"]
        icon = QIcon()
        icon.addPixmap(QPixmap(ics[unsaved]))
        self.preview_lbl.setVisible(prev)
        self.unsaved_lbl.setVisible(unsaved)
        self.save_btn.setEnabled(unsaved)
        self.save_btn.setIcon(icon)
        self.folders_cmb.setEnabled(unsaved)

    def set_folders(self, flds):
        self.folders_cmb.clear()
        self.folders_cmb.addItems(flds)

    def set_style(self):
        self.save_btn.setStyleSheet(styles.TOOLBUTTON)
        self.folders_cmb.setStyleSheet(styles.COMBO)

class MainHeader(QWidget):
    """Define the header bar for the main view.

    """
    def __init__(self, parent=None):
        super(MainHeader, self).__init__(parent)
        self.imgs = (("medias/one_view.png", 
                        "medias/one_view_active.png",
                        "medias/one_view_hover.png"), 
                     ("medias/v_two_views.png",
                        "medias/v_two_views_active.png",
                        "medias/v_two_views_hoover.png"), 
                    ("medias/h_two_views.png",
                        "medias/h_two_views_active.png",
                        "medias/h_two_views_hoover.png"))
        self.parent = parent
        self.setMaximumSize(99999, 24)
        self.gridLayout = QGridLayout(self)
        self.gridLayout.setContentsMargins(12, 2, 12, 2)
        self.horizontalLayout_2 = QHBoxLayout()
        self.horizontalLayout = QHBoxLayout()
        self.horizontalLayout.setSpacing(2)
        button0 = Button(0, self)
        self.horizontalLayout.addWidget(button0)
        button1 = Button(1, self)
        self.horizontalLayout.addWidget(button1)
        button1.setEnabled(False)
        button2 = Button(2, self)
        self.horizontalLayout.addWidget(button2)
        button2.setEnabled(False)
        self.horizontalLayout_2.addLayout(self.horizontalLayout)
        sp = QSpacerItem(80, 20, QSizePolicy.MinimumExpanding, 
                                    QSizePolicy.Minimum)
        self.horizontalLayout_2.addItem(sp)
        self.backup_lbl = QLabel(self)
        self.horizontalLayout_2.addWidget(self.backup_lbl)
        self.zoom_lbl = QLabel(self)
        self.horizontalLayout_2.addWidget(self.zoom_lbl)
        sp1 = QSpacerItem(40, 20, QSizePolicy.Expanding, QSizePolicy.Minimum)
        self.horizontalLayout_2.addItem(sp1)
        self.exit_btn = QToolButton(self)
        icon3 = QIcon()
        icon3.addPixmap(QPixmap("medias/exit.png"), 
                                    QIcon.Normal, QIcon.Off)
        self.exit_btn.setIcon(icon3)
        self.horizontalLayout_2.addWidget(self.exit_btn)
        self.gridLayout.addLayout(self.horizontalLayout_2, 0, 0, 1, 1)
        self.mode_buttons = (button0, button1, button2)
        self.mode = 0
        self.mode_buttons[0].set_state(1)

        self.retranslateUi()
        #self.set_style()

    def change_mode(self, mode):
        lgg.info('Main header set mode: %s to %s' % (self.mode, mode))
        if mode != self.mode:
            self.update_buttons_state(mode)
            self.parent.change_layout_mode(mode)

    def update_buttons_state(self, mode):
        self.mode_buttons[self.mode].set_state(0)
        self.mode_buttons[mode].set_state(1)
        self.mode = mode

    def retranslateUi(self):
        self.mode_buttons[0].setStatusTip(_("One view"))
        self.mode_buttons[1].setStatusTip(_("Two vertical views"))
        self.mode_buttons[2].setStatusTip(_("Two horizontal views"))
        self.exit_btn.setStatusTip(_("Return to viewing mode"))
        self.exit_btn.setToolTip(_("Exit the editing mode"))

    def set_style(self):
        self.mode_buttons[0].setStyleSheet(styles.TOOLBUTTON)
        self.mode_buttons[1].setStyleSheet(styles.TOOLBUTTON)
        self.mode_buttons[2].setStyleSheet(styles.TOOLBUTTON)
        self.exit_btn.setStyleSheet(styles.TOOLBUTTON)


class AltHeader(QWidget):
    """Define the header bar for the second view.

    """
    def __init__(self, sync, parent=None):
        super(AltHeader, self).__init__(parent)
        self.parent = parent
        self.origin = parent.imgchanges.origin
        self.setMaximumSize(99999, 24)
        self.grid = QGridLayout(self)
        self.grid.setContentsMargins(12, 2, 12, 2)
        self.layout = QHBoxLayout()
        self.layout.setContentsMargins(0, 0, 0, 0)
        self.sync_btn = QToolButton(self)
        self.sync_btn.setCheckable(True)
        self.sync_btn.setChecked(sync)
        self.sync_btn.setPopupMode(QToolButton.MenuButtonPopup)
        self.layout.addWidget(self.sync_btn)
        sp = QSpacerItem(40, 20, QSizePolicy.Expanding, QSizePolicy.Minimum)
        self.layout.addItem(sp)
        self.backup_lbl = QLabel(self)
        self.backup_lbl.setText(_('Backup:'))
        self.layout.addWidget(self.backup_lbl)
        self.backup_cmb = QComboBox(self)
        self.backup_cmb.setStatusTip(_("Choose the backup"))
        self.layout.addWidget(self.backup_cmb)
        self.zoom_lbl = QLabel(self)
        self.layout.addWidget(self.zoom_lbl)
        sp1 = QSpacerItem(40, 20, QSizePolicy.Expanding, QSizePolicy.Minimum)
        self.layout.addItem(sp1)
        self.grid.addLayout(self.layout, 0, 0, 1, 1)

        self.sync_menu = QMenu(self.sync_btn)
        group = QActionGroup(self.sync_btn)
        self.zoom_act = QAction(self.sync_btn)
        self.zoom_act.setText("Zoom")
        self.zoom_act.setCheckable(True)
        self.zoom_act.setChecked(True)
        self.size_act = QAction(self.sync_btn)
        self.size_act.setText("Size")
        self.size_act.setCheckable(True)
        self.size_act.setChecked(False)
        self.sync_menu.addAction(self.zoom_act)
        self.sync_menu.addAction(self.size_act)
        group.addAction(self.zoom_act)
        group.addAction(self.size_act)
        self.sync_btn.setMenu(self.sync_menu)
        icon = QIcon()
        icon.addPixmap(QPixmap("medias/unlinked.png"), QIcon.Normal, QIcon.Off)
        icon1 = QIcon()
        icon1.addPixmap(QPixmap("medias/linked.png"), QIcon.Normal, QIcon.Off)

        self.icons = (icon, icon1)
        self.sync_btn.setIcon(self.icons[sync])
        self.sync_btn.toggled.connect(self.on_sync_toggled)
        self.backup_cmb.currentIndexChanged.connect(self.on_backup_selected)
        self.zoom_act.toggled.connect(self.on_setting_changed)
        #self.set_style()

    def update_backup_list(self):
        self.backup_cmb.blockSignals(True)
        self.backup_cmb.clear()
        for bck in self.parent.backups[1:]:
            item = os.path.basename(bck)
            self.backup_cmb.addItem(item)

        self.backup_cmb.blockSignals(False)

    def on_sync_toggled(self, state):
        self.parent.views_are_sync = state
        self.sync_btn.setIcon(self.icons[state])
        self.parent.resize_alt_view()

    def on_setting_changed(self, state):
        self.parent.sync_on_zoom = state

    def on_backup_selected(self, idx):
        self.parent.update_alt_view(idx+1)

    def set_style(self):
        self.sync_btn.setStyleSheet(styles.BUTTONPOPUP)
        self.sync_menu.setStyleSheet(styles.MENU)
        self.backup_cmb.setStyleSheet(styles.COMBO)


class Button(QToolButton):
    """Define the button used for the mode selection.

    """
    def __init__(self, idx, parent=None):
        super(Button, self).__init__(parent)
        self.parent = parent
        self.idx = idx
        imgs = parent.imgs[idx]
        ic = QIcon()
        ic.addPixmap(QPixmap(imgs[0]))
        ic1 = QIcon()
        ic1.addPixmap(QPixmap(imgs[1]))
        ic2 = QIcon()
        ic2.addPixmap(QPixmap(imgs[2]))
        self.icons = (ic, ic1, ic2)
        self.set_state(0)
        
    def set_state(self, state):
        self.state = state
        self.setIcon(self.icons[state])

    def mousePressEvent(self, event):
        QToolButton.mousePressEvent(self, event)
        #self.set_state(1)
        self.parent.change_mode(self.idx)

    def enterEvent(self, event):
        QToolButton.enterEvent(self, event)
        #self.setIcon(self.icons[2])

    def leaveEvent(self, event):
        QToolButton.leaveEvent(self, event)
        #self.setIcon(self.icons[self.state])

