# -*- coding: utf-8 -*-

# g9nbox.py
# This file is part of Oqapy serie 3
#
# Author: Vincent Vande Vyvre <vincent.vandevyvre@oqapy.eu>
# Home: http://www.oqapy.eu
# Copyright: 2013-2017 Vincent Vande Vyvre
# Licence: GPL3
#
# Geolocation tool box
#
# St Python '50.19345734865621', '3.479976593078728'


from PyQt5.QtCore import Qt, QSize, pyqtSignal
from PyQt5.QtGui import QIcon, QPixmap, QFont
from PyQt5.QtWidgets import (QWidget, QGridLayout, QVBoxLayout, QHBoxLayout,
                             QSpacerItem, QLabel, QSizePolicy, QPushButton,
                             QComboBox, QDialog, QLineEdit, QTableWidget,
                             QTableWidgetItem, QToolButton, )

import stylesheets as styles

from g9n.mapviewer import MapViewer
from g9n.expandedmap import ExpandedMap


class G9nBox(QWidget):
    """Define the geolocation tool box.

    """
    def __init__(self, ui, parent=None):
        super().__init__(parent)
        self.name = 'G9nBox'
        self.ui = ui
        self.core = ui.core
        self.cfg = ui.core.cfg
        self.vlayout = QVBoxLayout(self)
        self.vlayout.setSpacing(2)
        self.vlayout.setContentsMargins(0, 0, 0, 0)
        self.webView = MapViewer(self)
        self.webView.setMaximumSize(300, 9999)
        self.vlayout.addWidget(self.webView)
        self.glayout = QGridLayout()
        self.lat_lbl = QLabel(self)
        font = QFont()
        font.setWeight(50)
        font.setBold(False)
        self.lat_lbl.setFont(font)
        self.lat_lbl.setText("Lat.: ")
        self.lat_lbl.setAlignment(Qt.AlignRight|Qt.AlignTrailing|
                                    Qt.AlignVCenter)
        self.glayout.addWidget(self.lat_lbl, 0, 0, 1, 1)
        self.lat_value_lbl = QLabel(self)
        self.lat_value_lbl.setText("0")
        self.glayout.addWidget(self.lat_value_lbl, 0, 1, 1, 1)
        self.lon_lbl = QLabel(self)
        self.lon_lbl.setFont(font)
        self.lon_lbl.setText("Long.: ")
        self.lon_lbl.setAlignment(Qt.AlignRight|Qt.AlignTrailing|
                                    Qt.AlignVCenter)
        self.glayout.addWidget(self.lon_lbl, 0, 2, 1, 1)
        self.lon_value_lbl = QLabel(self)
        self.lon_value_lbl.setText("0")
        self.glayout.addWidget(self.lon_value_lbl, 0, 3, 1, 1)
        self.vlayout.addLayout(self.glayout)
        self.hlayout = QHBoxLayout()
        self.hlayout.setSpacing(6)
        space = QSpacerItem(40, 20, QSizePolicy.Expanding, QSizePolicy.Minimum)
        self.hlayout.addItem(space)
        self.hlayout1 = QHBoxLayout()
        self.hlayout1.setSpacing(0)       
        self.lock_btn = QPushButton(self)
        self.lock_btn.setEnabled(False)
        self.lock_btn.setMaximumSize(QSize(25, 25))
        icon1 = QIcon('medias/unlock')
        self.lock_btn.setIcon(icon1)
        self.lock_btn.setIconSize(QSize(20, 20))
        self.hlayout1.addWidget(self.lock_btn)
        self.apply_geo_btn = QPushButton(self)
        self.apply_geo_btn.setEnabled(False)
        self.apply_geo_btn.setMaximumSize(QSize(120, 25))
        icon = QIcon('medias/apply.png')
        self.apply_geo_btn.setIcon(icon)
        self.apply_geo_btn.setIconSize(QSize(20, 20))
        self.hlayout1.addWidget(self.apply_geo_btn)
        self.hlayout.addLayout(self.hlayout1)
        self.expand_btn = QPushButton(self)
        self.expand_btn.setMaximumSize(QSize(120, 25))
        self.hlayout.addWidget(self.expand_btn)
        space1 = QSpacerItem(6, 20, QSizePolicy.Fixed, QSizePolicy.Minimum)
        self.hlayout.addItem(space1)
        self.vlayout.addLayout(self.hlayout)
        self.map_server = 0
        self.icons = ((QIcon('medias/osm_th.png'), 
                       QIcon('medias/osm_th_active.png')),
                      (QIcon('medias/google_th.png'), 
                       QIcon('medias/google_th_active.png')))
        self.tips = ((_("Change to Open Street Map"), _("Open Street Map")),
                     (_("Change to Google Map"), _("Google Map")))
        self.zoom = 16

        self.webView.mapServerChanged.connect(self.on_map_server_changed)
        self.webView.coordinateChanged.connect(self.on_mapViewer_updated)
        #self.ui.openstreetmap_btn.clicked.connect(self.on_openstreetmap_clicked)
        #self.ui.googlemap_btn.clicked.connect(self.on_googlemap_clicked)
        self.lock_btn.clicked.connect(self.on_unlock)
        self.apply_geo_btn.clicked.connect(self.apply_new_location)
        self.expand_btn.clicked.connect(self.expand_map_viewer)

        self.retranslate()
        self.set_style()

    def on_map_server_changed(self, server):
        """Update the icon of the map server.

        Args:
        server -- int 0: Open Street Map
                      1: Google Map
        """
        self.map_server = server
        #self.ui.openstreetmap_btn.setIcon(self.icons[0][not server])
        #self.ui.openstreetmap_btn.setStatusTip(self.tips[0][not server])
        #self.ui.googlemap_btn.setIcon(self.icons[1][server])
        #self.ui.googlemap_btn.setStatusTip(self.tips[1][server])

    def on_mapViewer_updated(self, coord):
        """The map in the webView has been changed. If zoom factor is up to
        value 13, the button 'Apply' is set enabled.

        Args:
        coord -- list(lat, long, zoom)
        """
        self.zoom = int(coord[2])
        coord = [str(c) for c in coord]
        self.lat_value_lbl.setText(coord[0][:8])
        self.lon_value_lbl.setText(coord[1][:8])
        if not self.lock_btn.isEnabled():
            self.apply_geo_btn.setEnabled(self.zoom > 13)

    def on_openstreetmap_clicked(self):
        self.change_map_server(0)

    def on_googlemap_clicked(self):
        self.change_map_server(1)

    def change_map_server(self, server):
        """Toggle the map server.

        Args:
        server -- int 0: Open Street Map
                      1: Google Map
        """
        if server == self.map_server:
            return

        self.webView.change_server()

    def set_locked(self, b):
        """Set the lock button.

        The geolocation is locked when the current image is already geolocated.

        Args:
        b -- boolean
        """
        if b:
            icon = QIcon('medias/lock.png')
            st = _('The current image is already localised, click to unlock.')

        else:
            icon = QIcon('medias/unlock.png')
            st = _('The current image is not localised.')

        self.lock_btn.setIconSize(QSize(20, 18))
        self.lock_btn.setIcon(icon)
        self.lock_btn.setStatusTip(st)
        self.lock_btn.setEnabled(b)
        self.apply_geo_btn.setEnabled(not b)

    def on_unlock(self):
        self.set_locked(False)

    def apply_new_location(self):
        lat = str(self.lat_value_lbl.text())
        lon = str(self.lon_value_lbl.text())
        self.core.apply_geotagging(lat, lon)
        self.set_locked(True)
        
    def expand_map_viewer(self):
        """Call the expanded map dialog.

        """
        lat = str(self.lat_value_lbl.text())
        lon = str(self.lon_value_lbl.text())
        urls = self.webView.urls
        self.em = ExpandedMap(self, self.cfg, self.map_server, urls)
        self.em.configure(float(lat), float(lon), self.webView.zoom)
        reply = self.em.exec_()

    def reset(self):
        """Unusued method, needed for toolboxes normalisation.

        """
        pass

    def retranslate(self):
        self.apply_geo_btn.setText(_('Apply'))
        self.apply_geo_btn.setStatusTip(_('Apply the current coordinates to '\
                                          'the current image.'))
        self.expand_btn.setText(_('Expand'))
        self.expand_btn.setStatusTip(_('Expand the map'))

    def set_style(self):
        self.lock_btn.setStyleSheet(styles.PUSHBUTTON)
        self.apply_geo_btn.setStyleSheet(styles.PUSHBUTTON)
        self.expand_btn.setStyleSheet(styles.PUSHBUTTON)



