# -*- coding: utf-8 -*-

# graphicsarea.py
# this file is part of Oqapy serie 3
#
# Author: Vincent Vande Vyvre <vincent.vandevyvre@oqapy.eu>
# Home: http://www.oqapy.eu
# Copyright: 2014-2017 Vincent Vande Vyvre
# Licence: GPL3
#
# QGraphicsView subclass

import logging
lgg = logging.getLogger(__name__)

from PyQt5.QtCore import Qt, pyqtSignal, QSize, QLineF
from PyQt5.QtGui import (QImage, QPixmap, QCursor, QPen, QColor, QPainter,
                         QBrush, QTransform)
from PyQt5.QtWidgets import (QGraphicsView, QFrame, QApplication,
                             QGraphicsScene, QGraphicsPixmapItem, QShortcut)

from tools.rectsize import RectSize

class GraphicsArea(QGraphicsView):
    zoomFactorChanged = pyqtSignal(float)
    sizeChanged = pyqtSignal()
    def __init__(self, ui, parent=None):
        super().__init__(parent)
        self.ui = ui
        self.core = ui.core
        self.source_image = None
        self.qimage = None
        self.qpixmap = None
        self.last_backup = None
        self.setFrameShape(QFrame.StyledPanel)
        self.setFrameShadow(QFrame.Raised)
        self.setDragMode(QGraphicsView.ScrollHandDrag)
        if not self.core.cfg.get(('style', 'scrlbars')):
            # No scrollBars
            self.setVerticalScrollBarPolicy(Qt.ScrollBarAlwaysOff)
            self.setHorizontalScrollBarPolicy(Qt.ScrollBarAlwaysOff)

        self.width_ = 0
        self.height_ = 0
        self.start_point = None
        self.current_zoom = 1
        self.step_zoom = self.core.cfg.get(('zoom', 'step'))
        self.button = None
        self.aerial_mod = Qt.NoModifier
        self.select_mod = Qt.MetaModifier
        self.aerial_shrt = None
        self.rubber_band = False
        self.origin_point = False
        self.is_active = False
        self.is_sync = True
        self.is_auto_scroll = False
        self.zoom_area = False
        self.straighten_mode = False
        self.crop = None
        self.current_pixmap_item = None
        self.mask = False
        self.label_preview = False

        #self.set_extra_shortcuts()
        self.old_shape = self.cursor().shape()
        self.verticalScrollBar().valueChanged.connect(self.on_scrolling)
        self.horizontalScrollBar().valueChanged.connect(self.on_scrolling)

    def on_scrolling(self):
        if not self.is_auto_scroll:
            self.core.update_navigation()

    def set_extra_shortcuts(self):
        """Sets the shortcuts needed into the graphics views.

        These shortcuts are defined here because all the views are not created
        at startup.
        """
        m = ["Shift", "Ctrl", "Alt", "Meta"]
        mods = [Qt.ShiftModifier, Qt.ControlModifier,
                Qt.AltModifier, Qt.MetaModifier]
        aerial = None

        try:
            aerial = self.core.cfg.get((2525))
        except:
            pass

        if aerial is None:
            # use 'A' as default
            self.aerial_shrt = 65

        else:
            a = aerial.split("+")
            if len(a) == 1:
                try:
                    self.aerial_shrt = ord(a[0])
                except:
                    # Invalid shortcut use default
                    self.aerial_shrt = 65

            else:
                if a[0] in m:
                    self.aerial_mod = mods[m.index(a[0])]
                try:
                    self.aerial_shrt = ord(a[1])
                except:
                    self.aerial_shrt = 65

        select = None
        try:
            select = self.core.cfg[2514]
        except:
            pass

        if select is None:
            # use default
            select = "Meta"

        if select in m:
            self.select_mod = mods[m.index(select)]

        try:
            ks = self.core.cfg[2550]
        except:
            ks = u'+'
        zi = QShortcut(ks, self)
        zi.activated.connect(self.core.zoom_up)

        try:
            ks = self.core.cfg[2551]
        except:
            ks = u'-'
        zo = QShortcut(ks, self)
        zo.activated.connect(self.core.zoom_down)

        try:
            ks = self.core.cfg[2552]
        except:
            ks = u'Home'
        fv = QShortcut(ks, self)
        fv.activated.connect(self.core.adjust_size)

        try:
            ks = self.core.cfg[2553]
        except:
            ks = u'End'
        fs = QShortcut(ks, self)
        fs.activated.connect(self.core.set_full_size)

    def size_(self):
        return (self.width(), self.height())

    def set_zoom_area(self, b):
        self.zoom_area = b
        if b:
            self.setDragMode(QGraphicsView.RubberBandDrag)
            self.setCursor(Qt.CrossCursor)

        else:
            self.setDragMode(QGraphicsView.ScrollHandDrag)
            cursor = QCursor(Qt.OpenHandCursor)
            self.setCursor(cursor)

    def set_scene(self):
        lgg.info('Set scene')
        self.scene_ = QGraphicsScene()
        self.setScene(self.scene_)

    def get_qimage(self):
        return self.qimage

    def get_qpixmap(self):
        return self.qpixmap

    def get_qpixmap_item(self):
        return self.current_pixmap_item

    def get_qimage_size(self):
        return self.qimage.width(), self.qimage.height()

    def get_qpixmap_size(self):
        return self.qpixmap.width(), self.qpixmap.height()

    def get_zoom_factor(self):
        return self.current_zoom

    def set_source_image(self, picture):
        self.source_image = picture

    def set_qimage_from_source(self):
        img = QImage(self.source_image.uri)
        if img.isNull():
            lgg.info('set qimage from source is null image')
            lgg.info('source: %s' % self.source_image.uri)
            return False

        self.qimage = img
        orientation = self.source_image.orientation
        if orientation in (3, 6, 8):
            angle = {3: 180, 6: 90, 8: -90}[orientation]
            self.rotate_image(angle)
            self.source_image.change_orientation(angle, self.qimage.size())

        return True

    def set_qimage_from_backup(self, backup, angle=0):
        lgg.info('set image from backup: %s' % backup)
        img = QImage(backup)
        if img.isNull():
            lgg.info('set qimage from source is null image')
            lgg.info('source: %s' % backup)
            return

        self.qimage = img
        if angle:
            self.rotate_image(angle)

    def resize_qimage(self, width, height, keep_ratio=True):
        if keep_ratio:
            self.qimage = self.qimage.scaled(width, height, 
                                             Qt.KeepAspectRatio, 
                                             Qt.SmoothTransformation)
        else:
            self.qimage = self.qimage.scaled(width, height, 
                                             Qt.IgnoreAspectRatio,
                                             Qt.SmoothTransformation)

    def crop_qimage(self, rect):
        self.qimage = self.qimage.copy(rect)

    def rotate_image(self, angle):
        """Change the orientation of the QImage.

        This called only when the orientation of the original image has been
        changed.

        Args:
        angle -- degrees
        """
        matrix = QTransform()
        self.qimage = self.qimage.transformed(matrix.rotate(angle), 
                                        Qt.SmoothTransformation)

    def set_mask(self, rect):
        """Creates the cropping mask for the rotation tool.

        If a mask has previously been created, it will be removed.

        Args:
        rect -- tuple(w, y, width, height) or False for remove the old mask only
        """
        if self.mask:
            try:
                lgg.info('Delete mask: %s' % self.mask)
                self.scene().removeItem(self.mask)
            except  Exception as why:
                # Already removed by the underlying C++
                lgg.info("Can't remove mask item: %s" % why)

        self.mask = False

        if rect:
            self.mask_rect = rect
            self.mask = Mask(self, rect, scene=self.scene())
            lgg.info('Mask created: %s' % self.mask)

    def refresh_qpixmap(self, zoom=None):
        lgg.info('Refresh current view, zoom: {0}'.format(self.current_zoom))
        w, h = self.get_qimage_size()
        if zoom is None:
            if RectSize(w, h) > RectSize(self.size_()):
                # The image is not a "No preview" thumbnail
                self.fit_in_view()
                return

            else:
                zoom = 1.0

        elif isinstance(zoom, tuple):
            self.fix_pixmap_size(zoom)
            return

        elif zoom == 0:
            zoom = self.current_zoom

        elif zoom < 0.002:
            return

        self.current_zoom = zoom
        w, h, = w * zoom, h * zoom
        self.qpixmap = QPixmap.fromImage(self.qimage.scaled(w, h, 
                                         Qt.KeepAspectRatio, 
                                         Qt.SmoothTransformation))
        self.show_pixmap()

    def show_pixmap(self):
        lgg.info('Show pixmap %s' % self.qpixmap)
        w, h = self.get_qpixmap_size()
        self.core.hide_proxy_tool()
        self.scene_.clear()
        self.scene_.setSceneRect(0, 0, w, h)
        self.current_pixmap_item = self.scene_.addPixmap(self.qpixmap)
        self.zoomFactorChanged.emit(self.current_zoom)

        if self.mask:
            geometry = self.mask.geometry
            self.set_mask(geometry)

        self.core.reload_proxy_tool()

    def fit_in_view(self):
        iw, _ = self.get_qimage_size()
        if not iw:
            lgg.warning('Fit in view, size is null')
            return

        vw, vh = self.size_()
        self.qpixmap = QPixmap.fromImage(self.qimage.scaled(vw, vh, 
                                         Qt.KeepAspectRatio, 
                                         Qt.SmoothTransformation))
        pw, _ = self.get_qpixmap_size()
        self.current_zoom = pw / float(iw)
        self.show_pixmap()

    def fix_pixmap_size(self, size):
        iw, _ = self.get_qimage_size()
        self.qpixmap = QPixmap.fromImage(self.qimage.scaled(size[0], size[1], 
                                         Qt.KeepAspectRatio, 
                                         Qt.SmoothTransformation)) 
        pw, _ = self.get_qpixmap_size()
        self.current_zoom = pw / float(iw)
        self.show_pixmap()       

    def clear(self):
        self.scene_.clear()
        self.source_image = None
        self.qimage = None
        self.qpixmap = None
        self.set_mask(False)
        self.last_backup = None
        self.current_zoom = 1

    def zoom_in(self):
        self.current_zoom += self.step_zoom
        self.refresh_qpixmap(0)

    def zoom_out(self):
        self.current_zoom -= self.step_zoom
        self.refresh_qpixmap(0)

    def zoom_selection(self, start, end):
        start = start.toPoint()
        end = end.toPoint()
        lgg.info('Zoom on area: {0}, {1}'.format(start, end))
        px, py = self.get_qimage_size()
        # rubberBand geometry (startX, startY, width, height)
        geom = self.normalise_rubber_band(start, end)
        center = self.get_selection_center(geom)
        vw, vh = self.size_()
        try:
            fx, fy = float(vw) / geom[2], float(vh) /geom[3]
        except ZeroDivisionError:
            return
        
        if fx < fy:
            # Real width
            rss = geom[2] * (1 / self.current_zoom)
            # New zoom factor
            # Viewer size is reduced (10 pxl) for better result
            z = float(vw-10) / rss

        else:
            # Real height
            rss = geom[3] * (1 / self.current_zoom)
            z = float(vh-10) / rss

        self.refresh_qpixmap(z)
        self.centerOn(center[0] * z, center[1] * z)
        #self._reset_tools('zoom')

    def normalise_rubber_band(self, s, e):
        cx = [s.x(), e.x()]
        cy = [s.y(), e.y()]
        sx, sy, ex, ey = min(cx), min(cy), max(cx), max(cy)
        # Use minimum 20 pixels
        w, h = ex - sx, ey -sy

        if w < 20:
            sx -= 20 - w
            if sx < 0:
                w += abs(sx)
                sx = 0

        if h < 20:
            sy -= 20 - h
            if sy < 0:
                h += abs(sy)
                sy = 0

        return (sx, sy, w, h)

    def get_new_zoom_factor(self):
        # FIXME Unusued !
        vwr = self.vwng
        if self.state.context in (3, 4):
            vwr = self.state.current_viewer

        oz = 1 / vwr.current_zoom
        zoom = vwr.current_zoom + vwr.step_zoom
        return oz * zoom

    def get_selection_center(self, sel):
        cx, cy = sel[0] + (sel[2] / 2), sel[1] + (sel[3] / 2)
        z = self.current_zoom
        return cx * (1 / z), cy * (1 / z)

    def full_size(self):
        self.refresh_qpixmap(1)

    def display_preview_label(self, b):
        """Show or hide a label 'preview' on the current image.

        Args:
        b -- boolean
        """
        lgg.info('display preview label: %s' % b)
        try:
            self.scene_.removeItem(self.label_preview)
        except:
            pass
        self.label_preview = False

        if b:
            origin = self.mapToScene(self.width()-180, 55)
            x, y = origin.x(), origin.y()
            #w = x - 195
            #h = y + 35
            self.label_preview = QGraphicsPixmapItem()
            self.label_preview.setPixmap(QPixmap("medias/preview.png"))
            self.scene_.addItem(self.label_preview)
            self.label_preview.setPos(x, y)

    def set_freehand_linedrawer(self, callback):
        """Enter in free hand line drawer.

        This mode is used with the straighten tool.

        Args:
        callback -- the method to call whenever the line is finished
        """
        self.callback = callback
        self.straighten_mode = True
        self.setDragMode(QGraphicsView.RubberBandDrag)
        self.setCursor(Qt.CrossCursor)
        self.setMouseTracking(True)

    def exit_tool_mode(self):
        """Close the current tool.

        """
        self.crop = False
        self.straighten_mode = False
        self.setDragMode(QGraphicsView.ScrollHandDrag)
        self.setCursor(Qt.OpenHandCursor)
        self.setMouseTracking(False)
        
            
    #--------------------------------------------------------

    def wheelEvent(self, event):
        step = event.angleDelta().y() / 120
        pos = event.pos()
        ipos = self.mapToScene(event.pos())
        self.core.remove_magnifier()

        if self.core.state.context in (3, 4):
            old_zoom = self.current_zoom
            new_zoom = old_zoom + step * self.step_zoom
            if new_zoom < 0.002:
                return
            imw, imh = self.get_qimage_size()
            self.current_zoom = new_zoom
            self.refresh_qpixmap(0)
            # is_sync is needed only for the alt_viewer
            self.is_sync = False

        else:
            old_zoom = self.current_zoom
            new_zoom = old_zoom + step * self.step_zoom
            if new_zoom < 0.002:
                return
            imw, imh = self.get_qimage_size()
            #self.core.zoom_image(step, event.pos())
            self.current_zoom = new_zoom
            self.refresh_qpixmap(0)

        delta_x = delta_y = 0
        if imw * new_zoom > self.width():
            delta_x = pos.x() - (ipos.x() / old_zoom) * new_zoom

        if imh * new_zoom > self.height():
            delta_y = pos.y() - (ipos.y() / old_zoom) * new_zoom

        self.centerOn(-delta_x+self.width()/2, -delta_y+self.height()/2)

    def mousePressEvent(self, event):
        if self.qimage is None:
            # No image
            return

        self.button = event.button()
        mod = event.modifiers()
        try:
            if self.core.magnifier:
                # Magnifying glass is shown
                QGraphicsView.mousePressEvent(self, event)
                return
        except AttributeError:
            # No magnifier
            pass

        if self.button == 1 and self.crop:
            self.crop.set_selection_area(self.mapToScene(event.pos()))
            event.accept()

        elif self.button == 1 and (mod == Qt.ControlModifier or self.zoom_area):
            # Zooming by dragging rectangle
            self.setDragMode(QGraphicsView.RubberBandDrag)
            # start_pos = QPointF
            self.start_pos = self.mapToScene(event.pos())
            self.rubber_band = True
            QGraphicsView.mousePressEvent(self, event)

        elif self.button == 1 and self.straighten_mode:
            # Drawing a line for the staighten tool
            if self.origin_point:
                angle = self.straight_line.line().angle()
                lgg.info('End of straighten line, angle: %s' %angle)
                self.scene().removeItem(self.straight_line)
                self.origin_point = False
                self.straighten_mode = False
                self.callback(angle)

            else:
                self.origin_point = self.mapToScene(event.pos())

        elif self.button == 4 and self.crop:
            QGraphicsView.mousePressEvent(self, event)

        else:
            QGraphicsView.mousePressEvent(self, event)

    def mouseMoveEvent(self, event):
        if not self.button:
            if self.straighten_mode and self.origin_point:
                self.draw_line(self.mapToScene(event.pos()))
            QGraphicsView.mouseMoveEvent(self, event)
            event.ignore()

        elif self.button == 4 and self.crop:
            QGraphicsView.mouseMoveEvent(self, event)

        elif self.crop:
            self.crop.draw_selection_area(self.mapToScene(event.pos()))
            self.ensureVisible(self.crop.area)
            event.accept()

        else:
            self.core.update_navigation()
            QGraphicsView.mouseMoveEvent(self, event)
            event.ignore()

    def mouseReleaseEvent(self, event):
        QGraphicsView.mouseReleaseEvent(self, event)
        self.button = None
        if self.crop:
            # already implemented in cropTool
            self.crop.fix_selection_area()
            event.accept()

        elif self.rubber_band:
            self.zoom_selection(self.start_pos, self.mapToScene(event.pos()))
            self.set_zoom_area(False)

        self.start_pos = None
        self.rubber_band = False

    def contextMenuEvent(self, event):
        # Avoid recursion after click on an item in menu
        self.button = None
        if self.scene is None or self.qpixmap is None:
            # No image
            return

        bypassed = False
        item = self.scene().itemAt(self.mapToScene(event.pos()), QTransform())
        if item is not None:
            try:
                if item is self.core.magnifier.pixmapitem:
                    QGraphicsView.contextMenuEvent(self, event)
                    bypassed = True
                    event.ignore()
            except AttributeError:
                pass

            try:
                if item is self.core.crop.area:
                    QGraphicsView.contextMenuEvent(self, event)
                    bypassed = True
                    event.ignore()
            except AttributeError:
                pass

        if not bypassed:
            self.core.on_viewer_context_menu(event.globalPos(), 
                                             self.mapToScene(event.pos()))

    def keyPressEvent(self, event):
        mod = QApplication.keyboardModifiers()
        # Added for crop tool
        QGraphicsView.keyPressEvent(self, event)

        if event.key() == Qt.Key_Escape:
            # Escape 16777216
            if self.core.state.context == 2:
                #Exit full screen mode
                self.core.exit_fullscreen()

            else:
                self.core._escape()

            event.accept()

        elif mod == self.select_mod:
            if event.key() in range(65, 91):
                self.ui.select_target_with_first_letter(event.key())
                event.accept()

        try:
            self.core.reset_aerial_view()
        except:
            # aerial view not already open
            pass

        try:
            if self.core.magnifier:
                QGraphicsView.keyPressEvent(self, event)
                event.ignore()
        except:
            # No magnifier
            pass

    def keyReleaseEvent(self, event):
        mod = QApplication.keyboardModifiers()
        if mod == self.aerial_mod and event.key() == self.aerial_shrt:
            # Show-hide shortcut for aerial view
            if event.isAutoRepeat():
                event.ignore()
                return

            else:
                if self.core.aerial_view:
                    self.core.aerial_view.hide()
                    event.accept()
                    return

    def resizeEvent(self, event):
        QGraphicsView.resizeEvent(self, event)
        return
        # FIXME
        self.core.replace_aerial_view()
        self.core.reset_aerial_view()
        self.sizeChanged.emit()

    def draw_line(self, pos):
        try:
            self.scene().removeItem(self.straight_line)
        except:
            pass

        line = QLineF(self.origin_point, pos)
        pen = QPen(QColor(0, 0, 255, 255), 1, Qt.SolidLine, Qt.RoundCap, 
                                                        Qt.MiterJoin)
        self.straight_line = self.scene().addLine(line, pen)

    def set_background(self):
        pix = QPixmap('medias/pix_viewer_290x266.png')
        lgg.info('Set background: %s' % (not pix.isNull()))
        try:
            scene = self.scene_
        except AttributeError:
            # No scene at startup
            self.set_scene()
            scene = self.scene_

        scene.setSceneRect(0, 0, 290, 266)
        scene.addPixmap(pix)


class Mask(QGraphicsPixmapItem):
    def __init__(self, view, geometry, parent=None, scene=None):
        super(Mask, self).__init__(parent)
        if scene is not None: scene.addItem(self)
        self.geometry = geometry
        left, up, right, down = [g * view.current_zoom for g in geometry]
        color = QColor()
        color.setRgb(100, 100, 150, 100)
        pix = QPixmap(QSize(scene.width(), scene.height()))
        pix.fill(color)
        painter = QPainter()
        painter.begin(pix)
        painter.setCompositionMode(QPainter.CompositionMode_SourceOut)
        color = QColor()
        color.setRgb(0, 0, 0, 0)
        brush = QBrush(color)
        painter.setBrush(brush)
        painter.fillRect(left, up, right, down, color)
        pen = QPen(QColor(0, 0, 255, 255), 1, Qt.SolidLine, Qt.RoundCap, 
                                                        Qt.MiterJoin)
        painter.setPen(pen)
        painter.drawRect(left-3, up-3, right+6, down+6)
        pen.setColor(QColor(255, 255, 255, 255))
        painter.setPen(pen)
        painter.drawRect(left-2, up-2, right+4, down+4)
        painter.end()
        self.setPixmap(pix)
        self.setPos(0, 0)
        self.show()
