# -*- coding: utf-8 -*-

# histogrambox.py
# this file is part of Oqapy serie 3
#
# Author: Vincent Vande Vyvre <vincent.vandevyvre@oqapy.eu>
# Home: http://www.oqapy.eu
# Copyright: 2013-2016 Vincent Vande Vyvre
# Licence: GPL3
#
# Histogram tool box.

from PyQt5.QtCore import Qt, QSize
from PyQt5.QtGui import QImage, QPixmap, QPen, QColor
from PyQt5.QtWidgets import (QWidget, QGridLayout, QGraphicsView,
                             QGraphicsScene, QGraphicsLineItem)

#import stylesheets as styles

class HistogramBox(QWidget):
    def __init__(self, parent=None):
        super().__init__(parent)
        self.name = 'HistogramBox'
        self.setMaximumSize(QSize(9999, 110))
        self.gridLayout = QGridLayout(self)
        self.gridLayout.setContentsMargins(2, 0, 2, 0)
        self.viewer = HistogramViewer(self)
        self.gridLayout.addWidget(self.viewer, 0, 0, 1, 1)
        self.viewer.setStyleSheet("""QWidget {background-color: #080C14;
                                        border: 1px solid #8A8E96;
                                        border-radius: 2px;}""")
        
    def get_histogram_size(self):
        """Returns the histogram viewer's size.

        Returns:
        tuple(width, height)
        """
        return self.viewer.get_size()

    def clear(self):
        self.viewer.clear_viewer()

    def show_histo(self, img):
        self.viewer.image = img
        self.clear()
        self.viewer.refresh_view()

    def reset(self):
        self.clear()

    def set_no_histogram(self):
        self.viewer.image = "medias/no_histogram.png"
        self.clear()
        self.viewer.refresh_view(False)

class HistogramViewer(QGraphicsView):
    def __init__(self, parent=None):
        super().__init__(parent)
        self.setObjectName("histo_vwr")
        self.setMinimumSize(QSize(256, 80))
        self.setMaximumSize(QSize(9999, 80))
        self.setHorizontalScrollBarPolicy(Qt.ScrollBarAlwaysOff)
        self.setVerticalScrollBarPolicy(Qt.ScrollBarAlwaysOff)
        self.image = None

    def clear_viewer(self):
        try:
            self.scene().clear()
        except AttributeError:
            # There's no scene at startup
            pass

    def refresh_view(self, resize=True):
        if self.image is None:
            return
        img = QImage(self.image)
        if resize:
            wdh, hgt = self.get_size()
            pix = QPixmap.fromImage(img.scaled(wdh, hgt,
                                        Qt.IgnoreAspectRatio, 
                                        Qt.SmoothTransformation))

        else:
            wdh, hgt = img.width(), img.height()
            pix = QPixmap.fromImage(img)

        if pix.isNull():
            return

        scene = QGraphicsScene()
        scene.setSceneRect(0, 0, wdh, hgt)
        scene.addPixmap(pix)
        #self.set_line(scene)
        self.setScene(scene)

    def set_line(self, scene):
        self.line = Line(self.get_height(), scene)

    def get_size(self):
        return (self.get_width(), self.get_height())

    def get_width(self):
        return (self.width())

    def resizeEvent(self, e):
        self.refresh_view()

    def get_height(self):
        return (self.height())

class Line(QGraphicsLineItem):
    def __init__(self, height_, scene):
        """Instance of movable vertical line used into histogram viewer.

        Args:
        height_ -- histogram's height
        scene -- QScene instance
        """
        super().__init__(None)
        if scene is not None: 
            scene.addItem(self)
        pen = QPen(QColor(255, 255, 0, 100), 1,
                          Qt.SolidLine, Qt.RoundCap, Qt.MiterJoin)
        self.setPen(pen)
        self.setLine(1.0, 0.0, 1.0, height_)

    def move_to(self, x):
        self.setPos(x, 0.0)
