# -*- coding: utf-8 -*-

# historicbox.py
# this file is part of Oqapy serie 3
#
# Author: Vincent Vande Vyvre <vincent.vandevyvre@oqapy.eu>
# Home: http://www.oqapy.eu
# Copyright: 2013-2016 Vincent Vande Vyvre
# Licence: GPL3
#
# Transformation's historic tool box.

import os

from PyQt5.QtCore import Qt, pyqtSignal, QSize
from PyQt5.QtGui import QIcon, QFont
from PyQt5.QtWidgets import (QWidget, QGridLayout, QHBoxLayout, QVBoxLayout,
                             QSpacerItem, QSizePolicy, QPushButton, QTreeWidget,
                             QFrame, QTreeWidgetItem, QAbstractItemView, )

import stylesheets as styles

class HistoricBox(QWidget):
    def __init__(self, ui, parent=None):
        super().__init__(parent)
        self.name = 'HistoricBox'
        self.ui = ui
        self.core = ui.core
        self.gridLayout = QGridLayout(self)
        self.gridLayout.setContentsMargins(2, 0, 2, 0)
        self.verticalLayout = QVBoxLayout()
        self.verticalLayout.setSpacing(2)
        self.list_wdg = ChangesList(self)
        self.verticalLayout.addWidget(self.list_wdg)
        self.horizontalLayout = QHBoxLayout()
        spacer = QSpacerItem(40, 20, QSizePolicy.Expanding, QSizePolicy.Minimum)
        self.horizontalLayout.addItem(spacer)
        self.cancel_btn = QPushButton(self)
        icon = QIcon('medias/edit-undo.png')
        self.cancel_btn.setIcon(icon)
        self.cancel_btn.setIconSize(QSize(18, 18))
        self.cancel_btn.setEnabled(False)
        self.horizontalLayout.addWidget(self.cancel_btn)
        self.unselect_btn = QPushButton(self)
        self.unselect_btn.setEnabled(False)
        icon1 = QIcon('medias/clean_24x27.png')
        self.unselect_btn.setIcon(icon1)
        self.unselect_btn.setIconSize(QSize(18, 18))
        self.horizontalLayout.addWidget(self.unselect_btn)
        spacer1 = QSpacerItem(8, 20, QSizePolicy.Fixed, QSizePolicy.Minimum)
        self.horizontalLayout.addItem(spacer1)
        self.verticalLayout.addLayout(self.horizontalLayout)
        self.gridLayout.addLayout(self.verticalLayout, 0, 0, 1, 1)
        self.retranslateUi()
        self.set_style()

        self.unselect_btn.clicked.connect(self.list_wdg.clearSelection)
        self.cancel_btn.clicked.connect(self.list_wdg.cancel_changes)
        self.list_wdg.changeCancelled.connect(self.undo_changes)

    def undo_changes(self, idx):
        self.core.filtering.undo_changes(idx)

    def reset(self):
        self.list_wdg.clear_changes()

    def set_style(self):
        self.list_wdg.setStyleSheet(styles.DATATREE)
        self.list_wdg.header().setStyleSheet(styles.DATATREEHEADER)
        self.list_wdg.horizontalScrollBar().setStyleSheet(styles.HSCROLLBAR)
        self.list_wdg.verticalScrollBar().setStyleSheet(styles.VSCROLLBAR)
        self.cancel_btn.setStyleSheet(styles.PUSHBUTTON)
        self.unselect_btn.setStyleSheet(styles.PUSHBUTTON)

    def retranslateUi(self):
        self.cancel_btn.setToolTip(_("Cancel the selected changes"))
        self.cancel_btn.setStatusTip(_("Cancel the selected changes"))
        self.cancel_btn.setText(_("Cancel"))
        self.unselect_btn.setToolTip(_("Unselect all"))
        self.unselect_btn.setStatusTip(_("Unselect all selection"))
        self.unselect_btn.setText(_("Unselect"))


class ChangesList(QTreeWidget):
    changeCancelled = pyqtSignal(int)
    def __init__(self, parent=None):
        super(ChangesList, self).__init__(parent)
        self.parent = parent
        font = QFont()
        font.setFamily("Ubuntu")
        font.setPointSize(9)
        self.setFont(font)
        self.setFrameShape(QFrame.NoFrame)
        self.setFrameShadow(QFrame.Plain)
        self.setIndentation(16)
        self.setUniformRowHeights(True)
        self.setColumnCount(2)
        self.headerItem().setText(0, "Name")
        self.headerItem().setText(1, "Value")
        self.setColumnWidth(0, 100)
        self.header().setStretchLastSection(True)
        self.setSelectionMode(QAbstractItemView.MultiSelection)
        self.itemClicked.connect(self.on_item_clicked)
        self.itemSelectionChanged.connect(self.on_selection_changed)
        sizePolicy = QSizePolicy(QSizePolicy.Preferred, QSizePolicy.Expanding)
        self.setSizePolicy(sizePolicy)

        self.changes = []
        # dict types describe the type of changes
        # Type name: (Readable name, (readable parameter's names))
        self.types = {'resize': (_("Resize"), (_("Width: "), _("Height: "))),
                      'contrast': (_("Contrast - Brightness"), 
                                    ("Gain: ", "Bias: ")),
                      'laplace': ("Laplace", ("Aperture: ",)),
                      'stretch': (_("Contrat stretching"), 
                                    (_("Lower: "), _("Upper: "))),
                      'crop': (_("Crop"), (_("Width: "), _("Height: "))),
                      'straight': (_("Rotation"), (_("Angle: "), 
                                    _("Width: "), _("Height: "))),
                      'greymap': (_("Grey map"), (_("Method: "),)),
                      'toning': (_("Toning"), (_("color: "), _("contrast: "))),
                      'flip': (("Flip"), ("Direction: ",)),
                      'denoise': (_("Denoising"), (_("Luminance: "), 
                                 _("Luminance colour: "), _("Template: "),
                                 _("Sample: ")))}

    def clear_changes(self):
        self.clear()
        self.changes = []

    def add_change(self, change):
        """Add a change item into the list.

        Args:
        item -- the change as list [type, (param,), backup filename]
        """
        type_ = self.types[change[0]][0]
        params = self.types[change[0]][1]
        vals = change[1]
        item = QTreeWidgetItem(self)
        item.setText(0, type_)
        item.setFlags(Qt.ItemIsSelectable | Qt.ItemIsEnabled)
        self.changes.append(item)

        for i, par in enumerate(params):
            child = QTreeWidgetItem(item)
            child.setText(0, par)
            child.setText(1, str(vals[i]))

        child = QTreeWidgetItem(item)
        child.setText(0, u'Backup:')
        child.setText(1, os.path.basename(change[-1]))

    def on_item_clicked(self, item):
        if item.parent() is not None:
            item.setSelected(False)
            return

        idx = self.changes.index(item)
        self.clearSelection()
        for ch in self.changes[idx:]:
            ch.setSelected(True)

    def on_selection_changed(self):
        if len(self.selectedItems()):
            self.parent.unselect_btn.setEnabled(True)
            self.parent.cancel_btn.setEnabled(True)

        else:
            self.parent.unselect_btn.setEnabled(False)
            self.parent.cancel_btn.setEnabled(False)

    def cancel_changes(self):
        sel = len(self.selectedItems())
        self.changeCancelled.emit(sel)

    def cancel_last_change(self):
        self.takeTopLevelItem(len(self.changes) - 1)
        self.changes.pop()
