# -*- coding: utf-8 -*-

# laplacebox.py
# this file is part of Oqapy serie 3
#
# Author: Vincent Vande Vyvre <vincent.vandevyvre@oqapy.eu>
# Home: http://www.oqapy.eu
# Copyright: 2014-2016 Vincent Vande Vyvre
# Licence: GPL3
#
# Laplacian transform tool box

from PyQt5.QtCore import QSize, QCoreApplication
from PyQt5.QtGui import QIcon
from PyQt5.QtWidgets import (QWidget, QVBoxLayout, QHBoxLayout, QLabel,
                             QSpacerItem, QPushButton, QSizePolicy,
                             QProgressBar)

import stylesheets as styles

from commonwidgets.horizontalslider import HorizontalSlider

class LaplaceBox(QWidget):
    def __init__(self, ui, parent=None):
        super().__init__ (parent)
        self.name = 'LaplaceBox'
        self.core = ui.core
        self.verticalLayout = QVBoxLayout(self)
        self.verticalLayout.setContentsMargins(4, 0, 8, 2)
        self.h_l = QHBoxLayout()
        self.label = QLabel(self)
        self.h_l.addWidget(self.label)
        self.val_lbl = QLabel(self)
        self.val_lbl.setText('0')
        self.h_l.addWidget(self.val_lbl)
        self.aperture_wdg = HorizontalSlider(self)
        self.aperture_wdg.set_layout_style('right')
        self.aperture_wdg.set_range((0, 31))
        self.h_l.addWidget(self.aperture_wdg)
        self.verticalLayout.addLayout(self.h_l)
        self.h_l1 = QHBoxLayout()
        spc = QSpacerItem(40, 20, QSizePolicy.Expanding, QSizePolicy.Minimum)
        self.h_l1.addItem(spc)
        self.progress = QProgressBar(self)
        self.progress.setMaximumSize(QSize(100, 12))
        self.h_l1.addWidget(self.progress)
        self.cancel_btn = QPushButton(self)
        self.cancel_btn.setEnabled(False)
        icon = QIcon('medias/edit-undo.png')
        self.cancel_btn.setIcon(icon)
        self.cancel_btn.setIconSize(QSize(18, 18))
        self.h_l1.addWidget(self.cancel_btn)
        self.apply_btn = QPushButton(self)
        self.apply_btn.setEnabled(False)
        icon1 = QIcon('medias/apply.png')
        self.apply_btn.setIcon(icon1)
        self.apply_btn.setIconSize(QSize(18, 18))
        self.h_l1.addWidget(self.apply_btn)
        self.verticalLayout.addLayout(self.h_l1)

        self.aperture = 0
        self.translate()
        self.set_style()
        self.progress.hide()

        self.aperture_wdg.valueChanged.connect(self.on_value_changed)
        self.cancel_btn.clicked.connect(self.on_cancel_clicked)
        self.apply_btn.clicked.connect(self.on_apply_clicked)

    def on_value_changed(self, val):
        if not val % 2:
            if val > self.aperture:
                val += 1

            elif val:
                val -= 1

            self.aperture_wdg.set_value(val, True)

        self.aperture = val
        self.apply_btn.setEnabled(val)
        self.val_lbl.setText(str(val))

    def on_cancel_clicked(self):
        self.cancel_btn.setEnabled(False)
        self.core.filtering.undo_last_change()

    def on_apply_clicked(self):
        self.core.state.disable_undo_buttons(self)
        self.set_progress_bar(True)
        value = self.aperture_wdg.get_value()
        self.cancel_btn.setEnabled(True)
        self.apply_btn.setEnabled(False)
        self.aperture_wdg.set_value(0, True)
        self.val_lbl.setText('0')
        self.core.set_filtered_mode(True)
        self.core.filtering.apply_laplacian_filter(value, self)

    def set_progress_bar(self, b):
        if b:
            self.progress.setRange(0, 0)
            self.progress.show()
            QCoreApplication.processEvents()

        else:
            self.progress.setRange(0, 10)
            self.progress.hide()

    def reset(self):
        self.aperture_wdg.set_value(0, True)
        self.val_lbl.setText('0')
        self.set_progress_bar(False)

    def translate(self):
        self.label.setText(_('Aperture'))
        self.aperture_wdg.minus.setStatusTip(_('Aperture -2'))
        self.aperture_wdg.plus.setStatusTip(_('Aperture +2'))
        self.aperture_wdg.reset.setStatusTip(_("Reset to 0"))
        self.cancel_btn.setText(_('Cancel'))
        self.apply_btn.setText(_('Apply'))
        self.apply_btn.setStatusTip(_("Apply the Laplacian filter"))
        self.cancel_btn.setStatusTip(_("Cancel the Laplacian filter"))

    def set_style(self):
        self.aperture_wdg.minus.setStyleSheet(styles.TBUTTON)
        self.aperture_wdg.slider.setStyleSheet(styles.SLIDER)
        self.aperture_wdg.plus.setStyleSheet(styles.TBUTTON)
        self.aperture_wdg.reset.setStyleSheet(styles.TBUTTON)
        self.progress.setStyleSheet(styles.PROGRESSBAR)
        self.apply_btn.setStyleSheet(styles.PUSHBUTTON)
        self.cancel_btn.setStyleSheet(styles.PUSHBUTTON)


