# -*- coding: utf-8 -*-

# main_ui.py
# This file is part of Oqapy serie 3
#
# Author: Vincent Vande Vyvre <vincent.vandevyvre@oqapy.eu>
# Home: http://www.oqapy.eu
# Copyright: 2009-2018 Vincent Vande Vyvre
# Licence: GPL3
#
# Main window

import sys
import os
import re
import shutil
import logging
lgg = logging.getLogger(__name__)

from PyQt5.QtCore import QCoreApplication, Qt, QSize, QRect
from PyQt5.QtGui import QIcon, QPixmap, QColor, QPainter, QPen, QCursor
from PyQt5.QtWidgets import (QMainWindow, QSizePolicy, QWidget, QVBoxLayout,
                             QGridLayout, QStackedWidget, QTabWidget, QMenu, 
                             QAction, QActionGroup, QDialog, QToolButton,
                             QHBoxLayout, QLabel, QListWidget, QListView,
                             QSpacerItem, QAbstractItemView, QListWidgetItem,
                             QPushButton, QFrame, QLineEdit, QComboBox,
                             QSpinBox, QGraphicsView, QGraphicsScene)

import stylesheets as styles

from data import VERSION_STR
from gui.toolbar import ToolBar
from gui.statusbar import StatusBar
from gui.graphicsarea import GraphicsArea
from gui.editingarea import EditingArea
from gui.trailer import Trailer
from gui.toolpanel import ToolPanel
from gui.targetarea import TargetArea
from commonwidgets.horizontalslider import HorizontalSlider
from keyboard import Keyboard


class MainUi(QMainWindow):
    """Define the main window.

    """
    def __init__(self, core, parent=None):
        """Instanciate the main window.

        Args:
        core -- instance of Core class
        parent -- None always
        """
        super().__init__(parent)
        lgg.info('Init main window')
        self.core = core
        w, h = core.cfg.get(('startup', 'size'))
        self.resize(w, h)
        self.fixed_size = w, h
        sizePolicy = QSizePolicy(QSizePolicy.Preferred, QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.sizePolicy().hasHeightForWidth())
        self.setSizePolicy(sizePolicy)
        icon = QIcon()
        icon.addPixmap(QPixmap("medias/oqapy_ic_48.png"),
                        QIcon.Normal, QIcon.Off)
        self.setWindowIcon(icon)
        self.setWindowTitle("".join(["Oqapy-", VERSION_STR]))
        self.mainwidget = QWidget()
        self.vl_1 = QVBoxLayout(self.mainwidget)
        self.vl_1.setSpacing(0)
        self.vl_1.setContentsMargins(0, 0, 0, 0)
        self.stackedviews = StackedViews(self, self.mainwidget)
        self.vl_1.addWidget(self.stackedviews)
        self.stackedtrailers = StackedTrailers(self, self.mainwidget)
        self.vl_1.addWidget(self.stackedtrailers)
        self.targetarea = TargetArea(self, self.mainwidget)
        self.vl_1.addWidget(self.targetarea)
        self.toolbar = ToolBar(self, self.mainwidget)
        self.vl_1.addWidget(self.toolbar.widget)
        self.setCentralWidget(self.mainwidget)
        # Status bar
        self.statusbar = StatusBar(self)
        self.setStatusBar(self.statusbar.widget)
        lgg.info('Build tool boxes panel')
        self.build_tool_panel()
        
        self.stackedviews.setCurrentIndex(0)
        self.stackedtrailers.hide()
        self.set_style()

    def build_tool_panel(self):
        """Build the tool panel as docked widget.

        """
        tool_side = self.core.cfg.get(('startup', 'panel'))
        self.toolpanel = ToolPanel(self)
        self.addDockWidget(Qt.DockWidgetArea(tool_side), self.toolpanel)

    def configure(self):
        """Set the configuration of some widgets.

        """
        sv = self.stackedviews.viewing_area
        sv.zoomFactorChanged.connect(self.core.on_zoom_factor_changed)
        sv.sizeChanged.connect(self.core.on_viewer_size_changed)
        sv.step_zoom = self.core.cfg.get(('zoom', 'step'))
        self.update_recent_folders()
        self.set_shortcuts()

    def set_window_title(self, title=""):
        """Set the name of source folder into the title bar.

        """
        # Dont compare instance of Path with string !
        if title and not isinstance(title, str):
            title = "%s (%s) " % (title.path.fname, title.path.parents)

        self.setWindowTitle("%s - Oqapy-%s" %(title, VERSION_STR))

    def set_shortcuts(self):
        """Apply all shortcuts.

        """
        self.kb = Keyboard(self)

    def set_source_folder_name(self, fld):
        """Set the source folder name.

        Args:
        fld -- the SourceFolder instance
        """
        self.stackedviews.sorting_tab.setTabText(0, fld.name)
        self.toolpanel.src_tool.source_led.setText(fld.where)
        self.set_window_title(fld)

    def set_target_button_enabled(self, button, enable):
        """Set enabled or disabled a target button.

        A target button is disabled when it is linked to the source folder.

        Args:
        button -- the instance of the button
        enable -- boolean
        """
        button.setEnabled(enable)

    def update_recent_folders(self):
        """Update the recent folder's list into some widgets.

        """
        folders = self.core.recents.path_strs
        tp = self.toolpanel
        tp.export.folder_led.add_historic_entries(folders, True)
        tp.src_tool.source_led.add_historic_entries(folders, True)
        tp.src_tool.launch_with_led.add_historic_entries(folders, True)
        tp.targets_tool.path_led.add_historic_entries(folders, True)


    #------------------------------------------------------------------------
    # Slots
    #------------------------------------------------------------------------

    def toggle_trailer(self, state=None):
        """Show-hide the trailer.

        Args:
        state -- if None toggle the visibility, if True show the trailer, 
                 otherwise hide it
        """
        if state is None:
            state = not self.stackedtrailers.isVisible()
            self.core.globalmenu.widget.trailer.setChecked(state)

        lgg.info('%s trailer' % ['Hide', 'Show'][state])
        self.stackedtrailers.setVisible(state)
        self.core.cfg.update(('startup', 'trailer'), state)

    def toggle_scroll_bars(self, state=None):
        """Show-hide the scrollbars.

        Args:
        state -- if None toggle the visibility, if True show the scroll bars, 
                 otherwise hide it
        """
        if state is None:
            state = not self.core.cfg.get(('style', 'scrlbars'))

        self.stackedviews.toggle_scroll_bars(state)
        self.core.cfg.update(('style', 'scrlbars'), state)
        self.core.globalmenu.widget.scrollbars.setChecked(state)

    def toggle_tool_panel(self, state=None):
        """Show-hide the tool panel.

        Args:
        state -- if None toggle the visibility, if True show the tool panel, 
                 otherwise hide it
        """
        if state is None:
            state = not self.toolpanel.isVisible()
            self.core.globalmenu.widget.panel.setChecked(state)

        self.toolpanel.setVisible(state)

    def on_target_tab_changed(self, old, new):
        """Slot called whenever the current tab has changed.

        Args:
        old -- the index of the old tab
        new -- the index of the new tab
        """
        logging.info('Target tab changed: %s to %s' % (old, new))
        tabs = self.stackedviews.sorting_tab
        if 0 < old < tabs.count()-1:
            fld = tabs.widget(old).dir_
            self.set_target_button_enabled(fld.button, True)

        if 0 < new < tabs.count()-1:
            fld = tabs.widget(new).dir_
            self.set_target_button_enabled(fld.button, False)

    def closeEvent(self, event):
        lgg.info('Close event %s' % event)
        if event is not None:
            self.core.close_app()


    #------------------------------------------------------------------------

    def get_histogram_size(self):
        """Return the size of the histogram tool box.

        """
        return self.toolpanel.histogram.get_histogram_size()

    def get_current_list_widget(self):
        """Return the instance of the current QListView.

        """
        if not self.stackedviews.sorting_tab.currentIndex():
            return 0

        return self.stackedviews.sorting_tab.currentWidget().list_lvw

    def set_style(self):
        self.setStyleSheet(styles.MAINWINDOW)
        self.mainwidget.setStyleSheet(styles.WIDGET)
        self.toolpanel.set_style()
        

class StackedViews(QStackedWidget):
    def __init__(self, ui, parent=None):
        super().__init__(parent)
        self.ui = ui
        self.core = ui.core
        self.viewing_page = QWidget(self)
        self.vl_1 = QVBoxLayout(self.viewing_page)
        self.vl_1.setSpacing(0)
        self.vl_1.setContentsMargins(0, 0, 0, 0)
        self.viewing_header = ViewingHeader(self)
        self.vl_1.addWidget(self.viewing_header)
        self.raw_header = RawHeader(self)
        self.vl_1.addWidget(self.raw_header)
        self.raw_header.hide()
        self.viewing_area = GraphicsArea(ui, self)
        self.vl_1.addWidget(self.viewing_area)
        self.addWidget(self.viewing_page)
        self.viewing_area.set_background()

        self.sorting_page = QWidget(self)
        self.vl_2 = QVBoxLayout(self.sorting_page)
        self.vl_2.setSpacing(0)
        self.vl_2.setContentsMargins(0, 0, 0, 0)
        self.sorting_tab = SortingTabWidget(ui, self.sorting_page)
        self.vl_2.addWidget(self.sorting_tab)
        self.addWidget(self.sorting_page)

        self.dual_view_page = QWidget(self)
        self.vl_3 = QVBoxLayout(self.dual_view_page)
        self.vl_3.setSpacing(0)
        self.vl_3.setContentsMargins(0, 0, 0, 0)
        self.dual_view = DualView(self)
        self.vl_3.addWidget(self.dual_view)
        self.addWidget(self.dual_view_page)

        self.multiviews_page = QWidget(self)
        self.vl_4 = QVBoxLayout(self.multiviews_page)
        self.vl_4.setSpacing(0)
        self.vl_4.setContentsMargins(0, 0, 0, 0)
        self.multiviews_header = MultiviewsHeader(self)
        self.vl_4.addWidget(self.multiviews_header)
        self.multiviews = MultiViewers(self)
        self.vl_4.addWidget(self.multiviews)
        self.addWidget(self.multiviews_page)

        self.viewing_header.quick_save_btn.clicked.connect(self.on_quick_save)
        self.viewing_header.fname_led.textChanged.connect(self.on_name_changed)
        self.viewing_header.exit_filter_btn.clicked.connect(self.on_exit_filter)
        self.multiviews_header.apply_btn.clicked.connect(
                                                self.core.configure_multiviews)
        self.multiviews_header.unselect_btn.clicked.connect(
                                                self.core.unselect_multiviews)
        self.target_name = ''
        self.current_header = self.viewing_header
        self.viewing_area.setStyleSheet(styles.VIEWER)
        self.viewing_area.horizontalScrollBar().setStyleSheet(styles.HSCROLLBAR)
        self.viewing_area.verticalScrollBar().setStyleSheet(styles.VSCROLLBAR)

    def on_state_changed(self, state):
        if not state:
            self.setCurrentIndex(0)
            self.viewing_header.reset()

    def get_current_viewer(self):
        viewers = [self.viewing_area, self.viewing_area]
        return viewers[self.currentIndex()]

    def set_dual_view(self, dual, src, new):
        lgg.info('Set dual view, current index: %s' % self.currentIndex())

        if dual:
            if self.currentIndex() == 2:
                self.dual_view.update(new)
                return
            self.move_viewing_area(2)
            self.dual_view.set_alt_view(src, new)
            self.setCurrentIndex(2)
            self.viewing_header.enable_layout_buttons(True)
            self.viewing_header.layout_buttons[0].setEnabled(False)

        elif self.currentIndex() == 2:
            self.dual_view.close()
            self.move_viewing_area(0)
            self.setCurrentIndex(0)

    def update_dualview_zoom(self, zoom):
        lgg.info('Update alt view from main, zoom: %s' % zoom)
        if self.currentIndex() != 2:
            return

        self.dual_view.sync_from_main_viewer(zoom)

    def change_layout(self, idx):
        """Change the dual view layout.

        Args:
        idx -- 0 simple view
               1 dual view horizontal
               2 dual view vertical
        """
        lgg.info("Change viewer layout: %s" % idx)
        if not idx:
            self.dual_view.alt_wdg.hide()

        else:
            self.dual_view.set_orientation(idx-1)

    def set_raw_header(self, b):
        """Show or hide the RAW header bar.

        Args:
        b -- boolean, if True set the RAW header bar, otherwise reset set the
             viewing header bar
        """
        if b:
            self.viewing_header.hide()
            self.raw_header.show()
            self.current_header = self.raw_header

        else:
            self.raw_header.hide()
            self.viewing_header.show()
            self.current_header = self.viewing_header

    def move_viewing_area(self, place):
        if place == self.currentIndex():
            return

        self.remove_viewing_widgets()
        if not place:
            lgg.info('Move viewing area to main widget')
            self.vl_1.addWidget(self.viewing_header)
            self.vl_1.addWidget(self.viewing_area)

        elif place == 1:
            lgg.info('Move viewing area to tab widget')
            self.sorting_tab.vl_1.addWidget(self.viewing_header)
            self.sorting_tab.vl_1.addWidget(self.viewing_area)

        elif place == 2:
            self.dual_view.build_views()

    def remove_viewing_widgets(self):
        if not self.currentIndex():
            # Move from viewing to sorting tab
            self.vl_1.removeWidget(self.viewing_header)
            self.vl_1.removeWidget(self.viewing_area)

        elif self.currentIndex() == 1:
            # Move from sorting tab to viewing
            self.sorting_tab.vl_1.removeWidget(self.viewing_header)
            self.sorting_tab.vl_1.removeWidget(self.viewing_area)

        elif self.currentIndex() == 2:
            self.dual_view.close()

    def set_sortingtab_multiviewing_mode(self):
        lgg.info('Set multiviewing in tab widget')
        self.remove_viewing_widgets()
        self.vl_1.addWidget(self.viewing_header)
        self.vl_1.addWidget(self.viewing_area)
        self.vl_4.removeWidget(self.multiviews_header)
        self.vl_4.removeWidget(self.multiviews)
        self.sorting_tab.vl_1.addWidget(self.multiviews_header)
        self.sorting_tab.vl_1.addWidget(self.multiviews)

    def unset_sortingtab_multiviewing_mode(self):
        lgg.info('Unset multiviewing in tab widget')
        self.sorting_tab.vl_1.removeWidget(self.multiviews_header)
        self.sorting_tab.vl_1.removeWidget(self.multiviews)
        self.vl_4.addWidget(self.multiviews_header)
        self.vl_4.addWidget(self.multiviews)
        self.vl_1.removeWidget(self.viewing_header)
        self.vl_1.removeWidget(self.viewing_area)
        self.sorting_tab.vl_1.addWidget(self.viewing_header)
        self.sorting_tab.vl_1.addWidget(self.viewing_area)

    def toggle_scroll_bars(self, state=True):
        view = self.get_current_viewer()
        if state:
            view.setVerticalScrollBarPolicy(Qt.ScrollBarAsNeeded)
            view.setHorizontalScrollBarPolicy(Qt.ScrollBarAsNeeded)

        else:
            view.setVerticalScrollBarPolicy(Qt.ScrollBarAlwaysOff)
            view.setHorizontalScrollBarPolicy(Qt.ScrollBarAlwaysOff)

    def change_file_name(self, name):
        if not self.currentIndex() or not self.sorting_tab.currentIndex():
            self.viewing_header.title_lbl.setText(name)
            self.target_name = name

    def update_file_name(self, modified):
        lgg.info('Update file name %s' % self.currentIndex())
        if not self.currentIndex() or not self.sorting_tab.currentIndex():
            self.viewing_header.set_file_modified(modified)

    def update_target_name(self, name):
        if name.startswith('.'):
            # Case of first RAW image
            return

        lgg.info('Update target name: %s' % name)
        self.current_header.fname_led.blockSignals(True)
        self.current_header.fname_led.setText(name)
        self.current_header.fname_led.blockSignals(False)
        self.target_name = name

    def on_quick_save(self):
        if self.viewing_header.fname_led.isVisible():
            self.ui.core.save_from_export()

        else:
            self.ui.core.quick_save()

    def on_exit_filter(self):
        self.ui.core.filtering.undo_all_changes()
        self.ui.core.set_filtered_mode(False)
        self.dual_view.close()

    def on_name_changed(self, text):
        try:
            name, ext = os.path.splitext(text)
        except:
            self.update_target_name(self.target_name)
            return

        self.ui.toolpanel.export.update_filename(name)
        self.ui.toolpanel.export.update_format(ext)
        self.ui.toolpanel.export.on_extension_changed(ext, False)

    def update_from_export_tool(self, export):
        name = export.name_led.text() + export.format_cmb.currentText()
        self.update_target_name(name)
        valid = export.save_btn.isEnabled()
        self.current_header.quick_save_btn.setEnabled(valid)

    def on_exit_multiview(self):
        self.core.close_multiviewing(update=True)


class MultiViewers(QWidget):
    def __init__(self, parent=None):
        super().__init__(parent)
        self.parent = parent
        self.ui = parent.ui
        self.layout = QGridLayout(self)
        self.layout.setContentsMargins(0, 0, 0, 0)
        self.layout.setSpacing(1)

    def resizeEvent(self, event):
        if self.isVisible():
            try:
                self.ui.core.multiviewing.on_resize_event(event)
            except AttributeError:
                pass


class DualView(QWidget):
    def __init__(self, parent=None):
        super().__init__(parent)
        self.parent = parent
        self.ui = parent.ui
        self.orientation = 0
        self.glayout = QGridLayout(self)
        self.glayout.setContentsMargins(0, 0, 0, 0)
        self.glayout.setHorizontalSpacing(0)
        self.glayout.setVerticalSpacing(0)
        self.builded = False
        self.backups = []
        self.views_are_sync = True
        self.sync_on_zoom = True
        self.on_sync = False

    def build_views(self):
        self.main_wdg = QWidget(self)
        self.vl_1 = QVBoxLayout(self.main_wdg)
        self.vl_1.setContentsMargins(0, 0, 0, 0)
        self.vl_1.addWidget(self.parent.viewing_header)
        self.vl_1.addWidget(self.parent.viewing_area)
        self.alt_wdg = QWidget(self)
        self.vl_2 = QVBoxLayout(self.alt_wdg)
        self.vl_2.setContentsMargins(0, 0, 0, 0)
        self.alt_header = AltViewingHeader(True, self)
        self.alt_view = GraphicsArea(self.ui, self.alt_wdg)
        self.alt_view.set_scene()
        self.vl_2.addWidget(self.alt_header)
        self.vl_2.addWidget(self.alt_view)
        self.glayout.addWidget(self.main_wdg, 0, 0, 1, 1)
        self.glayout.addWidget(self.alt_wdg, 0, 1, 1, 1)
        self.alt_wdg.hide()
        self.builded = True
        self.alt_view.horizontalScrollBar().valueChanged.connect(
                                            self.on_alt_h_scroll_event)
        self.alt_view.verticalScrollBar().valueChanged.connect(
                                            self.on_alt_v_scroll_event)
        self.alt_view.zoomFactorChanged.connect(self.on_alt_zoom_changed)
        self.parent.viewing_area.horizontalScrollBar().valueChanged.connect(
                                            self.on_main_h_scroll_event)
        self.parent.viewing_area.verticalScrollBar().valueChanged.connect(
                                            self.on_main_v_scroll_event)
        self.alt_view.setStyleSheet(styles.VIEWER)
        self.alt_view.horizontalScrollBar().setStyleSheet(styles.HSCROLLBAR)
        self.alt_view.verticalScrollBar().setStyleSheet(styles.VSCROLLBAR)

    def close(self):
        lgg.info("Dual view close request")
        self.alt_view.clear()
        self.alt_wdg.hide()
        self.backups = []

    def set_alt_view(self, src, bckp):
        lgg.info('set alt view with: %s' % src)
        self.backups = [bckp, src]
        self.alt_header.update_backup_list()
        self.update_view()

    def set_orientation(self, orientation):
        lgg.info('Change dual view orientation: %s' % orientation)
        if not self.builded:
            self.build_views()

        if orientation != self.orientation:
            row, col = int(orientation), int(not orientation)
            self.glayout.removeWidget(self.alt_wdg)
            self.glayout.addWidget(self.alt_wdg, row, col, 1, 1)
            self.orientation = orientation
        self.alt_wdg.show()

    def update(self, backup):
        self.backups.insert(0, backup)
        self.alt_header.update_backup_list()
        self.update_view()

    def update_view(self, idx=0):
        src = self.backups[idx]
        zoom = self.parent.viewing_area.get_zoom_factor()
        self.alt_view.set_source_image(src)
        self.alt_view.set_qimage_from_source()
        self.alt_view.refresh_qpixmap(zoom)
        self.update_alt_zoom_info()
        self.on_sync = False

    def undo_backup(self):
        self.backups.pop(0)
        if not self.backups:
            self.close()

        self.alt_header.update_backup_list()
        self.update_view()

    def resize_alt_view(self):
        """Resize the second image.

        """
        lgg.info('Resize alt view, on zoom: %s' % self.sync_on_zoom)
        if self.sync_on_zoom:
            val = self.parent.viewing_area.get_zoom_factor()

        else:
            mw, mh = self.parent.viewing_area.get_qpixmap_size()
            aw, ah = self.alt_view.get_qpixmap_size()
            if not self.orientation:
                # Compute the same height
                dh = mh
                dw = dh * (float(aw) / ah)

            else:
                # Compute the same width
                dw = mw
                dh = dw * (float(ah) / aw)

            val = dw, dh

        self.alt_view.refresh_qpixmap(val)
        self.update_alt_zoom_info(self.alt_view.get_zoom_factor())

    def update_alt_view(self, idx):
        self.update_view(idx)
        if self.views_are_sync and not self.sync_on_zoom:
            self.alt_view.blockSignals(True)
            self.resize_alt_view()
            self.alt_view.blockSignals(False)

    def update_alt_zoom_info(self, val=None):
        """Refresh the zoom info into the alt view header bar.

        Args:
        val -- the value of the zoom in decimal
        """
        if val is None:
            val = self.alt_view.get_zoom_factor()

        zoom = '%.3f' % (val * 100)
        self.alt_header.zoom_lbl.setText('Zoom: {0} %'.format(zoom))

    def on_alt_h_scroll_event(self, value):
        """Called when the horizontal scroll bar of the alt view is moved.

        Args:
        value -- the current value of the scroll bar
        """
        if self.views_are_sync:
            self.parent.viewing_area.blockSignals(True)
            self.parent.viewing_area.horizontalScrollBar().setValue(value)
            self.parent.viewing_area.blockSignals(False)

    def on_alt_v_scroll_event(self, value):
        """Called when the vertical scroll bar of the alt view is moved.

        Args:
        value -- the current value of the scroll bar
        """
        if self.views_are_sync:
            self.parent.viewing_area.blockSignals(True)
            self.parent.viewing_area.verticalScrollBar().setValue(value)
            self.parent.viewing_area.blockSignals(False)

    def on_main_h_scroll_event(self, value):
        """Called when the horizontal scroll bar of the main view is moved.

        Args:
        value -- the current value of the scroll bar
        """
        if self.views_are_sync:
            self.alt_view.blockSignals(True)
            self.alt_view.horizontalScrollBar().setValue(value)
            self.alt_view.blockSignals(False)

    def on_main_v_scroll_event(self, value):
        """Called when the vertical scroll bar of the main view is moved.

        Args:
        value -- the current value of the scroll bar
        """
        if self.views_are_sync:
            self.alt_view.blockSignals(True)
            self.alt_view.verticalScrollBar().setValue(value)
            self.alt_view.blockSignals(False)

    def on_alt_zoom_changed(self, zoom):
        lgg.info('alt view zoom changed: %s' % zoom)
        self.update_alt_zoom_info(zoom)
        if self.views_are_sync:
            if self.sync_on_zoom:
                val = zoom
                
            else:
                val = self.get_zoom_from_size()

            self.on_sync = True
            self.parent.viewing_area.refresh_qpixmap(val)

    def get_zoom_from_size(self):
        w, h = self.alt_view.get_qpixmap_size()
        w1, h1 = self.parent.viewing_area.get_qimage_size()
        if self.orientation:
            # compute zoom with width
            return w / w1

        return h / h1

    def sync_from_main_viewer(self, zoom):
        if self.on_sync:
            self.on_sync = False
            return

        if self.alt_wdg.isVisible():
            if self.views_are_sync:
                if self.sync_on_zoom:
                    val = zoom
                else:
                    val = self.get_size_from_main_view()

                self.alt_view.blockSignals(True)
                self.alt_view.refresh_qpixmap(val)
                self.alt_view.blockSignals(False)
                self.update_alt_zoom_info()

    def get_size_from_main_view(self):
        w, h = self.parent.viewing_area.get_qpixmap_size()
        w1, h1 = self.alt_view.get_qimage_size()
        if self.orientation:
            # compute zoom with width
            return w / w1

        return h / h1


class AltViewingHeader(QWidget):
    def __init__(self, sync, dview):
        super().__init__(dview.alt_wdg)
        self.dview = dview
        self.origin = None
        self.setMaximumSize(QSize(16777215, 40))
        self.grid = QGridLayout(self)
        self.grid.setContentsMargins(12, 2, 4, 2)
        self.layout = QHBoxLayout()
        self.layout.setContentsMargins(0, 0, 0, 0)
        self.sync_btn = QToolButton(self)
        self.sync_btn.setCheckable(True)
        self.sync_btn.setChecked(sync)
        self.sync_btn.setPopupMode(QToolButton.MenuButtonPopup)
        self.layout.addWidget(self.sync_btn)
        self.zoom_lbl = QLabel(self)
        self.layout.addWidget(self.zoom_lbl)
        sp = QSpacerItem(40, 20, QSizePolicy.Expanding, QSizePolicy.Minimum)
        self.layout.addItem(sp)
        self.backup_lbl = QLabel(self)
        self.backup_lbl.setText(_('Backup:'))
        self.layout.addWidget(self.backup_lbl)
        self.backup_cmb = QComboBox(self)
        self.backup_cmb.setStatusTip(_("Choose the backup"))
        self.layout.addWidget(self.backup_cmb)
        sp1 = QSpacerItem(40, 20, QSizePolicy.Expanding, QSizePolicy.Minimum)
        self.layout.addItem(sp1)
        self.grid.addLayout(self.layout, 0, 0, 1, 1)

        self.sync_menu = QMenu(self.sync_btn)
        group = QActionGroup(self.sync_btn)
        self.zoom_act = QAction(self.sync_btn)
        self.zoom_act.setText("Zoom")
        self.zoom_act.setCheckable(True)
        self.zoom_act.setChecked(True)
        self.size_act = QAction(self.sync_btn)
        self.size_act.setText("Size")
        self.size_act.setCheckable(True)
        self.size_act.setChecked(False)
        self.sync_menu.addAction(self.zoom_act)
        self.sync_menu.addAction(self.size_act)
        group.addAction(self.zoom_act)
        group.addAction(self.size_act)
        self.sync_btn.setMenu(self.sync_menu)
        icon = QIcon()
        icon.addPixmap(QPixmap("medias/unlinked.png"), QIcon.Normal, QIcon.Off)
        icon1 = QIcon()
        icon1.addPixmap(QPixmap("medias/linked.png"), QIcon.Normal, QIcon.Off)

        self.icons = (icon, icon1)
        self.sync_btn.setIcon(self.icons[sync])
        self.sync_btn.toggled.connect(self.on_sync_toggled)
        self.backup_cmb.currentIndexChanged.connect(self.on_backup_selected)
        self.zoom_act.toggled.connect(self.on_setting_changed)
        self.set_style()

    def update_backup_list(self):
        self.backup_cmb.blockSignals(True)
        self.backup_cmb.clear()
        for bck in self.dview.backups[1:]:
            self.backup_cmb.addItem(bck.base)

        self.backup_cmb.blockSignals(False)

    def on_sync_toggled(self, state):
        """Called whenever the sync button is clicked.

        Args:
        state -- the stae (True, False) of the button
        """
        self.dview.views_are_sync = state
        self.sync_btn.setIcon(self.icons[state])
        if state:
            self.dview.resize_alt_view()

    def on_setting_changed(self, state):
        lgg.info('Sync setting changed, on zoom: %s' % state)
        self.dview.sync_on_zoom = state
        if self.dview.views_are_sync:
            self.dview.resize_alt_view()

    def on_backup_selected(self, idx):
        self.dview.update_alt_view(idx+1)

    def set_style(self):
        self.sync_btn.setStyleSheet(styles.BUTTONPOPUP)
        self.sync_menu.setStyleSheet(styles.MENU)
        self.backup_cmb.setStyleSheet(styles.COMBO)

class ViewingHeader(QWidget):
    def __init__(self, parent=None):
        super().__init__(parent)
        self.main_wdg = parent
        self.imgs = (("medias/one_view.png", 
                        "medias/one_view_active.png",
                        "medias/one_view_hover.png"), 
                     ("medias/v_two_views.png",
                        "medias/v_two_views_active.png",
                        "medias/v_two_views_hoover.png"), 
                    ("medias/h_two_views.png",
                        "medias/h_two_views_active.png",
                        "medias/h_two_views_hoover.png"))
        self.hlayout = QHBoxLayout(self)
        self.hlayout.setSpacing(0)
        self.hlayout.setContentsMargins(20, 2, 4, 0)
        self.setMaximumSize(QSize(16777215, 38))
        self.hlayout1 = QHBoxLayout()
        self.hlayout1.setSpacing(4)
        self.hlayout1.setContentsMargins(0, 0, 20, 0)
        button0 = Button(0, self)
        self.hlayout1.addWidget(button0)
        button0.setEnabled(False)
        button1 = Button(1, self)
        self.hlayout1.addWidget(button1)
        button1.setEnabled(False)
        button2 = Button(2, self)
        self.hlayout1.addWidget(button2)
        button2.setEnabled(False)
        self.layout_buttons = [button0, button1, button2]
        self.hlayout.addLayout(self.hlayout1)
        self.title_lbl = QLabel(self)
        self.title_lbl.setText("Main Viewer")
        self.hlayout.addWidget(self.title_lbl)
        sp = QSpacerItem(30, 20, QSizePolicy.Fixed, QSizePolicy.Minimum)
        self.hlayout.addItem(sp)
        self.fname_led = QLineEdit(self)
        self.fname_led.setMinimumSize(QSize(100, 28))
        self.fname_led.hide()
        self.hlayout.addWidget(self.fname_led)
        sp1 = QSpacerItem(10, 20, QSizePolicy.Fixed, QSizePolicy.Minimum)
        self.hlayout.addItem(sp1)
        self.quick_save_btn = QToolButton(self)
        icon = QIcon()
        icon.addPixmap(QPixmap("medias/save.svg"), QIcon.Normal, QIcon.Off)
        self.quick_save_btn.setIcon(icon)
        self.quick_save_btn.setStatusTip(_("Quick save"))
        self.quick_save_btn.hide()
        self.quick_save_btn.setEnabled(False)
        self.hlayout.addWidget(self.quick_save_btn)
        sp2 = QSpacerItem(60, 20, QSizePolicy.Expanding, QSizePolicy.Minimum)
        self.hlayout.addItem(sp2)
        self.exit_filter_btn = QToolButton(self)
        #self.exit_filter_btn.setMaximumSize(36, 36)
        icon1 = QIcon()
        icon1.addPixmap(QPixmap("medias/exit.png"), QIcon.Normal, QIcon.Off)
        self.exit_filter_btn.setIcon(icon1)
        self.exit_filter_btn.setStatusTip(_("Exit filtering state"))
        self.exit_filter_btn.hide()
        self.hlayout.addWidget(self.exit_filter_btn)
        self.display_layout_buttons(False)
        self.set_style()

    def reset(self):
        self.title_lbl.setText("")
        self.set_filtered_mode(False)

    def set_filtered_mode(self, mode):
        self.fname_led.setVisible(mode)
        self.quick_save_btn.setVisible(mode)
        self.quick_save_btn.setEnabled(False)
        self.exit_filter_btn.setVisible(mode)
        self.display_layout_buttons(mode)

    def set_title(self, title):
        self.title_lbl.setText(title)
        self.quick_save_btn.setEnabled(False)

    def set_file_modified(self, state):
        """Add an asterisk to the file name when the image was modified.

        Args:
        state -- boolean, if True, the image was modified
        """
        title = self.title_lbl.text().rstrip('*')
        title = title + ('*' * state)
        self.title_lbl.setText(title)
        self.quick_save_btn.setVisible(state)
        self.quick_save_btn.setEnabled(state)

    def display_layout_buttons(self, show):
        for button in self.layout_buttons:
            button.setVisible(show)

    def enable_layout_buttons(self, enable):
        for button in self.layout_buttons:
            button.setEnabled(enable)

    def set_quick_save_enabled(self, enable):
        self.quick_save_btn.setEnabled(enable)

    def change_layout(self, idx):
        self.display_layout_buttons(True)
        self.enable_layout_buttons(True)
        self.layout_buttons[idx].setEnabled(False)
        self.main_wdg.change_layout(idx)

    def set_style(self):
        for b in self.layout_buttons:
            b.setStyleSheet(styles.HEADERBUTTON)
        self.title_lbl.setStyleSheet(styles.TITLE)
        self.fname_led.setStyleSheet(styles.LINEEDIT)
        self.quick_save_btn.setStyleSheet(styles.HEADERBUTTON)
        self.exit_filter_btn.setStyleSheet(styles.HEADERBUTTON)


class RawHeader(QWidget):
    def __init__(self, parent=None):
        super().__init__(parent)
        self.main_wdg = parent
        self.hlayout = QHBoxLayout(self)
        self.hlayout.setSpacing(0)
        self.hlayout.setContentsMargins(20, 2, 4, 0)
        self.setMaximumSize(QSize(16777215, 38))
        self.preview_lbl = QLabel(self)
        self.preview_lbl.setText(_("Preview of"))
        self.hlayout.addWidget(self.preview_lbl)
        sp = QSpacerItem(20, 20, QSizePolicy.Fixed, QSizePolicy.Minimum)
        self.hlayout.addItem(sp)
        self.fname_lbl = QLabel(self)
        self.hlayout.addWidget(self.fname_lbl)
        sp1 = QSpacerItem(30, 20, QSizePolicy.Fixed, QSizePolicy.Minimum)
        self.hlayout.addItem(sp1)
        self.fname_led = QLineEdit(self)
        self.fname_led.setMinimumSize(QSize(100, 28))
        self.fname_led.setEnabled(False)
        self.hlayout.addWidget(self.fname_led)
        self.quick_save_btn = QToolButton(self)
        icon = QIcon()
        icon.addPixmap(QPixmap("medias/save.svg"), QIcon.Normal, QIcon.Off)
        self.quick_save_btn.setIcon(icon)
        self.quick_save_btn.setStatusTip(_("Quick save"))
        self.quick_save_btn.setEnabled(False)
        self.hlayout.addWidget(self.quick_save_btn)
        sp2 = QSpacerItem(60, 20, QSizePolicy.Expanding, QSizePolicy.Minimum)
        self.hlayout.addItem(sp2)
        self.set_style()

    def set_file_name(self, name, preview=True):
        self.fname_lbl.setText(name)

    def set_preview_state(self, preview, saved=False):
        self.fname_led.clear()
        if preview:
            self.preview_lbl.setText(_("Preview of"))
            self.enable_quick_save(False)

        else:
            if saved:
                self.set_file_modified(0)
                self.enable_quick_save(False)

            else:
                self.preview_lbl.setText(_("Image:"))
                self.fname_led.setText(self.fname_lbl.text())
                self.set_file_modified(1)
                self.enable_quick_save(True)

    def set_file_modified(self, state):
        """Add an asterisk to the file name when the image was modified.

        Args:
        state -- boolean, if True, the image was modified
        """
        name = self.fname_lbl.text().rstrip('*')
        name = name + ('*' * state)
        self.fname_lbl.setText(name)

    def enable_quick_save(self, state):
        self.fname_led.setEnabled(state)
        self.quick_save_btn.setEnabled(state)

    def set_style(self):
        self.preview_lbl.setStyleSheet(styles.TITLE)
        self.fname_lbl.setStyleSheet(styles.TITLE)
        self.fname_led.setStyleSheet(styles.LINEEDIT)
        self.quick_save_btn.setStyleSheet(styles.TOOLBUTTON)


class MultiviewsHeader(QWidget):
    def __init__(self, parent=None):
        super().__init__(parent)
        self.main_wdg = parent
        self.hlayout = QHBoxLayout(self)
        self.hlayout.setSpacing(6)
        self.hlayout.setContentsMargins(20, 2, 4, 0)
        self.setMaximumSize(QSize(16777215, 38))
        self.fname_lbl = QLabel(self)
        self.hlayout.addWidget(self.fname_lbl)
        sp = QSpacerItem(20, 20, QSizePolicy.Expanding, QSizePolicy.Minimum)
        self.hlayout.addItem(sp)
        self.select_lbl = QLabel(self)
        self.select_lbl.setText(_("No selection"))
        self.hlayout.addWidget(self.select_lbl)
        self.unselect_btn = QToolButton(self)
        self.unselect_btn.setText(_("Unselect"))
        icon = QIcon()
        icon.addPixmap(QPixmap("medias/save.svg"), QIcon.Normal, QIcon.Off)
        self.unselect_btn.setIcon(icon)
        self.unselect_btn.setStatusTip(_("Unselect all images selected"))
        self.unselect_btn.setEnabled(False)
        self.hlayout.addWidget(self.unselect_btn)
        sp1 = QSpacerItem(30, 20, QSizePolicy.Fixed, QSizePolicy.Minimum)
        self.rows_lbl = QLabel(self)
        self.rows_lbl.setText(_("Rows"))
        self.rows_lbl.setStatusTip(_("Number of rows"))
        self.hlayout.addWidget(self.rows_lbl)
        self.rows_spb = QSpinBox(self)
        self.rows_spb.setMinimum(1)
        self.hlayout.addWidget(self.rows_spb)
        sp3 = QSpacerItem(10, 20, QSizePolicy.Fixed, QSizePolicy.Minimum)
        self.hlayout.addItem(sp3)
        self.columns_lbl = QLabel(self)
        self.columns_lbl.setText(_("Columns"))
        self.columns_lbl.setStatusTip(_("Number of columns"))
        self.hlayout.addWidget(self.columns_lbl)
        self.columns_spb = QSpinBox(self)
        self.columns_spb.setMinimum(2)
        self.hlayout.addWidget(self.columns_spb)
        self.apply_btn = QToolButton(self)
        icon1 = QIcon()
        icon1.addPixmap(QPixmap("medias/dialog-apply.svg"), QIcon.Normal, QIcon.Off)
        self.apply_btn.setIcon(icon1)
        self.apply_btn.setEnabled(False)
        self.hlayout.addWidget(self.apply_btn)
        self.set_style()
        self.rows_spb.valueChanged.connect(self.on_rows_changed)
        self.columns_spb.valueChanged.connect(self.on_columns_changed)

    def configure_multiviewing(self, values):
        self.rows_spb.blockSignals(True)
        self.columns_spb.blockSignals(True)
        self.rows_spb.setValue(values["rows"])
        self.columns_spb.setValue(values["columns"])
        self.rows_spb.blockSignals(False)
        self.columns_spb.blockSignals(False)
        self.rows_numb = values["rows"]
        self.columns_numb = values["columns"]
        self.apply_btn.setEnabled(False)

    def set_maxima(self, rows, columns):
        self.rows_spb.setMaximum(rows)
        self.columns_spb.setMaximum(columns)

    def on_rows_changed(self, value):
        self.apply_btn.setEnabled(not self.rows_numb == value)

    def on_columns_changed(self, value):
        self.apply_btn.setEnabled(not self.columns_numb == value)

    def update_title(self, name):
        self.fname_lbl.setText(name)

    def update_selection(self, number):
        if not number:
            self.select_lbl.setText(_("No selection"))

        elif number == 1:
            self.select_lbl.setText(_("1 file selected"))

        else:
            self.select_lbl.setText(_("{} file selected".format(number)))

        self.unselect_btn.setEnabled(1 and number)

    def get_config(self):
        return self.rows_spb.value(), self.columns_spb.value()

    def set_style(self):
        self.fname_lbl.setStyleSheet(styles.TITLE)
        self.select_lbl.setStyleSheet(styles.TITLE)
        self.rows_lbl.setStyleSheet(styles.TITLE)
        self.rows_spb.setStyleSheet(styles.SPINBOX)
        self.columns_lbl.setStyleSheet(styles.TITLE)
        self.columns_spb.setStyleSheet(styles.SPINBOX)


class Button(QToolButton):
    """Define the button used for the mode selection.

    """
    def __init__(self, idx, parent=None):
        super().__init__(parent)
        self.parent = parent
        self.idx = idx
        imgs = parent.imgs[idx]
        ic = QIcon()
        ic.addPixmap(QPixmap(imgs[0]))
        ic1 = QIcon()
        ic1.addPixmap(QPixmap(imgs[1]))
        ic2 = QIcon()
        ic2.addPixmap(QPixmap(imgs[2]))
        self.icons = (ic, ic1, ic2)
        self.set_state(0)
        
    def set_state(self, state):
        self.state = state
        self.setIcon(self.icons[state])

    def mousePressEvent(self, event):
        QToolButton.mousePressEvent(self, event)
        #self.set_state(1)
        self.parent.change_layout(self.idx)

    def enterEvent(self, event):
        QToolButton.enterEvent(self, event)

    def leaveEvent(self, event):
        QToolButton.leaveEvent(self, event)


class SortingTabWidget(QTabWidget):
    def __init__(self, ui, parent=None):
        super().__init__(parent)
        self.ui = ui
        self.core = ui.core
        self.source_tab = QWidget()
        self.vl_1 = QVBoxLayout(self.source_tab)
        self.vl_1.setSpacing(0)
        self.vl_1.setContentsMargins(0, 0, 0, 0)
        self.addTab(self.source_tab, "Source")
        self.trash_tab = TrashCanTab(self.core.workspace['trash'], self.core)
        self.addTab(self.trash_tab, "Trash")
        self.current_tab = 0
        self.currentChanged.connect(self.on_tab_changed)
        self.indexes = [0, 25]
        self.setStyleSheet(styles.TAB)

    def insert_tab(self, index, folder):
        lgg.info('Create tab %s' % index)
        self.indexes.append(index)
        self.indexes.sort()
        idx = self.indexes.index(index)
        tab = TargetTab(folder, self.core)
        self.insertTab(idx, tab, tab.title)
        folder.tab = tab

    def set_temp_folder(self, index, tmppath):
        #FIXME Necessary ?
        try:
            idx = self.indexes.index(index)
        except IndexError:
            lgg.warning('Tab index error: %s' % idx)
            return

        self.widget(idx).list_lvw.temp = tmppath

    def on_tab_changed(self, idx):
        self.ui.on_target_tab_changed(self.current_tab, idx)
        self.current_tab = idx

    def remove_tab(self, pos):
        try:
            idx = self.indexes.index(pos)
        except IndexError:
            lgg.warning('Tab index error: %s' % pos)
            return

        self.removeTab(idx)
        self.indexes.remove(pos)

    def show_trash_tab(self):
        self.setCurrentWidget(self.trash_tab)

    def get_trash_tab(self):
        """Return the trash tab instance.

        """
        return self.trash_tab

    def shift_to_next_tab(self):
        lgg.info("Shift to next tab")
        if self.core.state.context != 2:
            return

        if self.current_tab == self.count() - 1:
            self.setCurrentIndex(0)

        else:
            self.setCurrentIndex(self.current_tab + 1)

    def shift_to_previous_tab(self):
        lgg.info("Shift to previous tab")
        if self.core.state.context != 2:
            return

        if self.current_tab == 0:
            self.setCurrentIndex(self.count() - 1)

        else:
            self.setCurrentIndex(self.current_tab - 1)


class TargetTab(QWidget):
    def __init__(self, dir_, core):
        super().__init__()
        self.dir_ = dir_
        self.core = core
        self.utils = core.utils
        self.ui = core.ui
        self.path = dir_.path
        self.title = self.path.name
        lgg.info('Tab title: %s' % self.title)
        self.glayout = QGridLayout(self)
        self.glayout.setContentsMargins(0, 0, 0, 0)
        self.glayout.setSpacing(0)
        self.vlayout = QVBoxLayout()
        self.list_lvw = TargetListWidget(core, self)
        self.list_lvw.setSelectionMode(QAbstractItemView.SingleSelection)
        self.vlayout.addWidget(self.list_lvw)
        self.hlayout = QHBoxLayout()
        sp = QSpacerItem(60, 20, QSizePolicy.Expanding, QSizePolicy.Minimum)
        self.hlayout.addItem(sp)
        self.resizer_wdg = HorizontalSlider(self)
        self.resizer_wdg.reset.hide()
        self.resizer_wdg.set_layout_style('right')
        self.resizer_wdg.set_range((100, 400))
        self.resizer_wdg.set_single_step(50)
        self.hlayout.addWidget(self.resizer_wdg)
        self.vlayout.addLayout(self.hlayout)
        self.glayout.addLayout(self.vlayout, 0, 0, 1, 1)

        self.list_lvw.resizer = self.resizer_wdg
        self.list_lvw.set_thumbnail_size()
        self.resizer_wdg.valueChanged.connect(self.list_lvw.set_thumbnail_size)
        self.resizer_wdg.slider.setStyleSheet(styles.SLIDER)
        self.resizer_wdg.minus.setStyleSheet(styles.TBUTTON)
        self.resizer_wdg.plus.setStyleSheet(styles.TBUTTON)

    def add_item(self, item):
        """Add an item into the list.

        Args:
        item -- dict of the file description's data
        """
        name = os.path.basename(item['fpath'])
        self.list_lvw.add_item(item, name)

    def remove_item(self, item):
        idx = self.list_lvw.row(item)
        lgg.info('Remove at row: %s' % idx)
        self.core.shredder.add_file_from_target(item)
        self.dir_.files.pop(idx)
        item = self.list_lvw.takeItem(idx)
        del item
        #self.set_info_text(self.list_lvw.count())

    def restore_item(self, item):
        idx = self.list_lvw.row(item)
        data = self.dir_.files.pop(idx)
        fpath = data['fpath']
        fld, fname = os.path.split(fpath)
        lgg.info('Restore file %s from %s' % (fname, os.path.basename(fld)))
        fdest = os.path.dirname(item.data['orig'])
        dest = self.check_name(item.data['orig'], fdest)
        sthumb = self.utils.get_thumb_name(dest)
        self.move_file(fpath, dest)
        self.move_file(item.data['thumb'], sthumb)
        if self.core.source_fld.where == fdest:
            self.core.insert_image(dest, sthumb)

        item = self.list_lvw.takeItem(idx)
        del item
        #self.set_info_text(self.list_lvw.count())

    def remove_items(self, item):
        pass

    def restore_items(self, item):
        # Restore more than one items
        pass

    def set_selected_text(self, num):
        # TODO self.selection_lbl.setText(_("%s files selected" % num))
        pass

    def check_name(self, n, fld=None):
        """Check if file exists already into the destination folder.

        If the file name exists, a numerical suffix is added to the file name.

        Args:
        n -- basename of the file
        """
        fld = fld or self.folder
        name, ext = os.path.splitext(n)
        count = 1
        # Check for existing suffix
        suff = re.compile('(?<=_\()(\d+)(?=\)%s)' % ext)
        match = suff.search(n)
        if match is not None:
            count = int(match.group(0)) + 1
            name = "".join(name.split("_")[:-1])

        while 1:
            path = os.path.join(fld, n)
            if os.path.isfile(path):
                n = "".join([name, "_(", str(count), ")", ext])
                count += 1
            else:
                break

        return n

    def move_file(self, src, target):
        try:
            shutil.move(src, target)
        except Exception as why:
            lgg.warning("Unable to move file, reason:\n\t{0}".format(why))


class TrashCanTab(QWidget):
    def __init__(self, dir_, core):
        super().__init__()
        self.dir_ = dir_
        self.core = core
        self.path = dir_.path
        self.title = dir_.name
        self.glayout = QGridLayout(self)
        self.glayout.setContentsMargins(0, 0, 0, 0)
        self.glayout.setSpacing(0)
        self.vlayout = QVBoxLayout()
        self.header = QWidget(self)
        self.header.setMaximumHeight(36)
        self.hlayout = QHBoxLayout(self.header)
        self.hlayout.setContentsMargins(50, 2, 10, 2)
        self.infos_lbl = QLabel(self)
        self.hlayout.addWidget(self.infos_lbl)
        self.remove_all_btn = QPushButton(self)
        self.hlayout.addWidget(self.remove_all_btn)
        sp = QSpacerItem(60, 20, QSizePolicy.Expanding, QSizePolicy.Minimum)
        self.hlayout.addItem(sp)
        self.select_lbl = QLabel(self)
        self.hlayout.addWidget(self.select_lbl)
        self.remove_selected_btn = QPushButton(self)
        self.hlayout.addWidget(self.remove_selected_btn)
        self.vlayout.addWidget(self.header)
        self.list_lvw = TargetListWidget(core, self)
        self.vlayout.addWidget(self.list_lvw)
        self.hlayout1 = QHBoxLayout()
        sp1 = QSpacerItem(60, 20, QSizePolicy.Expanding, QSizePolicy.Minimum)
        self.hlayout1.addItem(sp1)
        self.resizer_wdg = HorizontalSlider(self)
        self.resizer_wdg.reset.hide()
        self.resizer_wdg.set_layout_style('right')
        self.resizer_wdg.set_range((100, 400))
        self.resizer_wdg.set_single_step(50)
        self.hlayout1.addWidget(self.resizer_wdg)
        self.vlayout.addLayout(self.hlayout1)
        self.glayout.addLayout(self.vlayout, 0, 0, 1, 1)

        self.list_lvw.resizer = self.resizer_wdg
        self.list_lvw.set_thumbnail_size()
        self.resizer_wdg.valueChanged.connect(self.list_lvw.set_thumbnail_size)

        self.set_info_text(0)
        self.remove_all_btn.setText(_("Remove all"))
        self.set_selected_text(0)
        self.remove_selected_btn.setText(_("Remove selection"))
        self.remove_all_btn.clicked.connect(self.remove_all)
        self.remove_selected_btn.clicked.connect(self.remove_items)
        self.remove_all_btn.setStyleSheet(styles.PUSHBUTTON)
        self.remove_selected_btn.setStyleSheet(styles.PUSHBUTTON)
        self.resizer_wdg.slider.setStyleSheet(styles.SLIDER)
        self.resizer_wdg.minus.setStyleSheet(styles.TBUTTON)
        self.resizer_wdg.plus.setStyleSheet(styles.TBUTTON)

    def set_info_text(self, num):
        if num <= 1:
            txt = _("{} file ".format(num))

        else:
            txt = _("{} files ".format(num))

        self.infos_lbl.setText(_("{}into the trash can".format(txt)))
        self.remove_all_btn.setEnabled(num)
        self.resizer_wdg.setEnabled(num)

    def set_selected_text(self, num):
        if num <= 1:
            txt = _("{} file ".format(num))

        else:
            txt = _("{} files ".format(num))

        self.select_lbl.setText(_("{}selected".format(txt)))
        self.remove_selected_btn.setEnabled(num)

    def add_item(self, index, item):
        """Add an item into the list.

        Args:
        index -- the index of file
        item -- dict of the file description's data
        """
        name = os.path.basename(item['orig'])
        self.list_lvw.add_item(item, name)
        name = os.path.basename(item['fpath'])
        lgg.info('Tab Trash can add item %s' % name)
        self.set_info_text(self.core.shredder.count())

    def remove_item(self, item):
        lgg.info('Remove at row: %s' % self.list_lvw.row(item))
        idx = self.list_lvw.row(item)
        self.core.shredder.remove_item(idx)
        item = self.list_lvw.takeItem(idx)
        del item
        self.set_info_text(self.list_lvw.count())

    def restore_item(self, item):
        idx = self.list_lvw.row(item)
        self.core.shredder.restore_item(item, idx)
        item = self.list_lvw.takeItem(idx)
        del item
        self.set_info_text(self.list_lvw.count())

    def remove_items(self, item):
        for s in self.list_lvw.selection:
            self.remove_item(s)

    def restore_items(self, item):
        for s in self.list_lvw.selection:
            self.restore_item(s)

    def remove_all(self):
        for i in range(self.list_lvw.count()):
            self.remove_item(self.list_lvw.item(0))


class StackedTrailers(QStackedWidget):
    def __init__(self, ui, parent=None):
        super().__init__(parent)
        self.ui = ui
        self.setMaximumSize(QSize(16777215, 85))
        self.trailer_page = QWidget(self)
        self.vl_1 = QVBoxLayout(self.trailer_page)
        self.vl_1.setSpacing(0)
        self.vl_1.setContentsMargins(0, 0, 0, 0)
        self.trailer = Trailer(ui, self)
        self.vl_1.addWidget(self.trailer)
        self.addWidget(self.trailer_page)
        self.raw_trailer_page = QWidget(self)
        self.vl_2 = QVBoxLayout(self.raw_trailer_page)
        self.vl_2.setSpacing(0)
        self.vl_2.setContentsMargins(0, 0, 0, 0)
        self.raw_trailer = Trailer(ui, self)
        self.vl_2.addWidget(self.raw_trailer)
        self.addWidget(self.raw_trailer_page)

    def clean_all(self):
        self.trailer.clear_all()
        self.raw_trailer.clear_all()

    def scroll_to_right(self):
        widget = self.trailer
        if self.currentIndex():
            widget = self.raw_trailer

        widget.scroll_to(1)

    def scroll_to_left(self):
        widget = self.trailer
        if self.currentIndex():
            widget = self.raw_trailer

        widget.scroll_to(-1)

    def scroll_to_right2(self):
        widget = self.trailer
        if self.currentIndex():
            widget = self.raw_trailer

        widget.scroll_to(5)

    def scroll_to_left2(self):
        widget = self.trailer
        if self.currentIndex():
            widget = self.raw_trailer

        widget.scroll_to(-5)


class TargetListWidget(QListWidget):
    def __init__(self, core, parent=None):
        super().__init__(parent)
        self.core = core
        self.folder = parent.dir_
        self.temp = None
        self.resizer = None
        self.selection = 0
        self.setViewMode(QListView.IconMode)
        self.setDragEnabled(True)
        self.setDragDropMode(QAbstractItemView.DragOnly)
        self.setDefaultDropAction(Qt.MoveAction)
        self.setSelectionMode(QAbstractItemView.ExtendedSelection)
        self.setFlow(QListView.LeftToRight)
        self.setViewMode(QListView.IconMode)
        self.currentItemChanged.connect(self.item_changed)
        self.itemSelectionChanged.connect(self.selection_changed)
        self.horizontalScrollBar().setStyleSheet(styles.HSCROLLBAR)
        self.verticalScrollBar().setStyleSheet(styles.VSCROLLBAR)

    def add_item(self, item, basename):
        """Add an item into the list.

        Args:
        item -- dict of the file description's data
        basename -- the basename of the file, if the tab is the trash can tab,
                    the name is the original file name
        """
        new = TargetItem(item, basename, self)
        self.addItem(new)

    def remove_item(self, item):
        if not isinstance(item, int):
            item = self.row(item)
        obj = self.list_lvw.takeItem(item)
        del obj

    def set_thumbnail_size(self, size=None):
        if size is None:
            size = self.core.cfg.get(('style', 'tab_icon_size'))

        else:
            if size == self.iconSize().width():
                return
            self.core.cfg.update(('style', 'tab_icon_size'), size)

        self.setIconSize(QSize(size, size))
        self.resizer.set_value(size)

    def item_changed(self, new, old):
        pass

    def selection_changed(self):
        self.selection = self.selectedItems()
        self.parent().set_selected_text(len(self.selection))

    def set_background_color(self, color=None):
        # Selection colors: #3C3C50 or #46465A or #1E1E37 or #1E1E28
        chain = "".join(["QWidget { color: #FFFFFF;"
                        "background-color: #242424;"
                        "selection-background-color: #3C3C50}"])
        self.list_view.setStyleSheet(chain)

    def contextMenuEvent(self, event):
        item = self.itemAt(event.pos())
        if item:
            item.show_context_menu(event.pos(), len(self.selection))

    def mousePressEvent(self, event):
        QListWidget.mousePressEvent(self, event)

class TargetItem(QListWidgetItem):
    def __init__(self, data, name, parent=None):
        super().__init__(parent)
        self.data = data
        self.view = parent
        #path = data['fpath']
        #self.index = index
        ic = self.create_icon(data['thumb'])
        #t = self.create_text(txt)
        tip = self.create_tooltip(data)
        self.setIcon(QIcon(ic))
        self.setText(name)
        self.setTextAlignment(Qt.AlignHCenter)
        self.setFlags(Qt.ItemIsSelectable | Qt.ItemIsEnabled)
        self.setToolTip(tip)  

    def create_icon(self, i):
        """Create thumbnail.

        Create thumbnail for target's tabs.
        Thumbnail is a square transparent background icon_size x icon_size,
        the image is paste at vertical center and at down edge, and a
        black border is painted around the image with a space of 2 pixels.
      
        Args:
        i -- name of image (full path)
        fn -- /tmp/oqapy/xx_yy.jpg
                where xx -- nummer of target (same as button)
                and yy... -- index's image in this target directory list. 
        """
        img = QPixmap(i).scaled(400, 400, Qt.KeepAspectRatio,
                                        Qt.SmoothTransformation)
        w = float(img.width())
        h = float(img.height())
        #coordinates top left of thumbnail
        bw, bh = 204 - (w/2), 408 - h - 4
        #painting's zone
        rect = QRect(bw, bh, w - 1, h - 1)
        #border = image size + (2*4)
        cw, ch = w + 4, h + 4
        #black border
        brect = QRect(bw-3, bh-3, cw, ch)
        color = QColor()
        color.setRgb(0, 0, 0, 0)
        bckgrnd = QPixmap(QSize(408, 408))
        bckgrnd.fill(color)
        painter = QPainter()
        painter.begin(bckgrnd)
        painter.setPen(QPen(Qt.white, 1, Qt.SolidLine, 
                                  Qt.RoundCap, Qt.MiterJoin))
        painter.drawRect(brect)
        irect = QRect(0.0, 0.0, w - 1, h - 1)
        painter.drawPixmap(rect, img, irect)
        painter.end()
        return bckgrnd

    def create_tooltip(self, infos):
        name = os.path.basename(infos['orig'])
        added = infos['added']
        return "<p style='white-space:pre'><b>{0}</b><br>Added: <b>{1}</b>"\
                .format(name, added)

    def get_infos(self):
        # FIXME unusued ?
        w = os.path.getsize(f)
        language = QtCore.QLocale(LOC)
        weight = language.toString(w)
        i_reader = QtGui.QImageReader(f)
        X = i_reader.size().width()
        Y = i_reader.size().height()
        dim = str(X) + " x " + str(Y)
        chain = u"<b>{0}</b>\n{1} pixels\n{2} bytes".format(name, dim, weight)
        return chain

    def on_restore_clicked(self):
        self.view.parent().restore_item(self)
        self.pop_menu.deleteLater()

    def on_delete_clicked(self):
        self.view.parent().remove_item(self)
        self.pop_menu.deleteLater()

    def on_restore_multi_clicked(self):
        self.view.parent().restore_items(self)
        self.pop_menu.deleteLater()

    def on_delete_multi_clicked(self):
        self.view.parent().remove_items(self)
        self.pop_menu.deleteLater()

    def show_context_menu(self, pos, num):
        lgg.info('Show_context menu at: %s' %pos)
        name = os.path.basename(self.data["fpath"])
        size = "%s x %s" %(self.data["width"], self.data["height"])
        undo = "medias/edit-undo.png"
        clean = "medias/clean_24x27.png"
        alignment = Qt.AlignRight | Qt.AlignTrailing | Qt.AlignVCenter
        self.pop_menu = QMenu()
        self.layout = QVBoxLayout(self.pop_menu)
        self.layout.setContentsMargins(2, 2, 2, 2)
        self.layout.setSpacing(2)
        self.glayout = QGridLayout()
        lbl = QLabel(_("File: "), self.pop_menu)
        lbl.setAlignment(alignment)
        self.glayout.addWidget(lbl, 0, 0, 1, 1)
        lbl1 = QLabel("<b>%s</b>" % name, self.pop_menu)
        self.glayout.addWidget(lbl1, 0, 1, 1, 1)
        lbl2 = QLabel(_("Origin: "), self.pop_menu)
        lbl2.setAlignment(alignment)
        self.glayout.addWidget(lbl2, 1, 0, 1, 1)
        lbl3 = QLabel("<b>%s</b>" % self.data["orig"], self.pop_menu)
        self.glayout.addWidget(lbl3, 1, 1, 1, 1)
        lbl4 = QLabel(_("Size: "), self.pop_menu)
        lbl4.setAlignment(alignment)
        self.glayout.addWidget(lbl4, 2, 0, 1, 1)
        lbl5 = QLabel("<b>%s</b>" % size, self.pop_menu)
        self.glayout.addWidget(lbl5, 2, 1, 1, 1)
        lbl6 = QLabel(_("ISO: "), self.pop_menu)
        lbl6.setAlignment(alignment)
        self.glayout.addWidget(lbl6, 3, 0, 1, 1)
        lbl7 = QLabel("<b>%s</b>" % self.data["iso"], self.pop_menu)
        self.glayout.addWidget(lbl7, 3, 1, 1, 1)
        lbl8 = QLabel(_("Speed Shutter: "), self.pop_menu)
        lbl8.setAlignment(alignment)
        self.glayout.addWidget(lbl8, 4, 0, 1, 1)
        lbl9 = QLabel("<b>%s</b>" % self.data["speed"], self.pop_menu)
        self.glayout.addWidget(lbl9, 4, 1, 1, 1)
        lbl10 = QLabel(_("FNumber: "), self.pop_menu)
        lbl10.setAlignment(alignment)
        self.glayout.addWidget(lbl10, 5, 0, 1, 1)
        lbl11 = QLabel("<b>%s</b>" % self.data["fnumber"], self.pop_menu)
        self.glayout.addWidget(lbl11, 5, 1, 1, 1)
        self.layout.addLayout(self.glayout)
        line = Separator(self.pop_menu)
        self.layout.addWidget(line)
        self.restore = MenuItem(_("Restore"), undo, self.pop_menu)
        self.layout.addWidget(self.restore)
        self.delete = MenuItem(_("Delete"), clean, self.pop_menu)
        self.layout.addWidget(self.delete)
        self.restore.clicked.connect(self.on_restore_clicked)
        self.delete.clicked.connect(self.on_delete_clicked)
        if num > 1:
            line1 = Separator(self.pop_menu)
            self.layout.addWidget(line1)
            self.restore_multi = MenuItem(_("Restore %s files selected" % num), 
                                            undo, self.pop_menu)
            self.layout.addWidget(self.restore_multi)
            self.delete_multi = MenuItem(_("Delete %s files selected" % num), 
                                            clean, self.pop_menu)
            self.layout.addWidget(self.delete_multi)
            self.restore_multi.clicked.connect(self.on_restore_multi_clicked)
            self.delete_multi.clicked.connect(self.on_delete_multi_clicked)
        self.pop_menu.exec_(QCursor.pos())


class MenuItem(QToolButton):
    """Define a menu item.

    """
    def __init__(self, text, pixmap=None, parent=None):
        super().__init__(parent)
        #self.setMinimumSize(QSize(80, 48))
        icon = QIcon()
        icon.addPixmap(QPixmap(pixmap))
        self.setIcon(icon)
        self.setIconSize(QSize(24, 24))
        self.setText(text)
        self.setToolButtonStyle(Qt.ToolButtonTextBesideIcon)
        self.setAutoRaise(True)


class Separator(QFrame):
    """Define a separator of items.

    """
    def __init__(self, parent=None):
        super().__init__(parent)
        sizePolicy = QSizePolicy(QSizePolicy.Expanding, QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(1)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.sizePolicy().hasHeightForWidth())
        self.setSizePolicy(sizePolicy)
        self.setFrameShape(QFrame.HLine)
        self.setFrameShadow(QFrame.Sunken)





