# -*- coding: utf-8 -*-

# navigationbox.py
# this file is part of Oqapy serie 3
#
# Author: Vincent Vande Vyvre <vincent.vandevyvre@oqapy.eu>
# Home: http://www.oqapy.eu
# Copyright: 2013-2017 Vincent Vande Vyvre
# Licence: GPL3
#
# Images navigation tool box.

from PyQt5.QtCore import Qt, QObject, QSize, QRectF, pyqtSignal
from PyQt5.QtGui import (QPixmap, QImage, QTransform, QPen, QCursor,
                         QColor, QPainter)
from PyQt5.QtWidgets import (QWidget, QGridLayout, QVBoxLayout, QHBoxLayout,
                             QGraphicsView, QToolButton, QGraphicsPixmapItem,
                             QSizePolicy, QGraphicsScene, QPushButton,
                             QGraphicsItem, )

import stylesheets as styles

class NavigationBox(QWidget):
    def __init__(self, core, parent):
        super().__init__(parent)
        self.name = 'PreviewBox'
        self.core = core
        self.gridLayout = QGridLayout(self)
        self.gridLayout.setContentsMargins(2, 0, 2, 0)
        self.viewer = GraphicsView(core, self)
        self.gridLayout.addWidget(self.viewer, 0, 0, 1, 1)
        self.set_style()

    def get_viewer_size(self):
        """Returns the size of the graphics view.

        """
        return (self.viewer.width(), self.viewer.height())

    def set_image(self, viewer, img):
        """Change the image into the view.

        args:
        viewer -- the current graphics area instance
        img -- the current QImage instance, if None the view is just cleared
        """
        if img is not None:
            self.viewer.display_image(viewer, img)

        else:
            self.clear()

    def update_(self):
        """Update the mask.

        """
        if self.isVisible():
            self.viewer.reset_window()

    def clear(self):
        """Clear the view.

        """
        self.viewer.set_background()

    def reset(self):
        """Clear the view.

        """
        self.clear()

    def set_main_viewer(self, viewer):
        self.viewer.main_viewer = viewer

    def set_style(self):
        self.viewer.setStyleSheet(styles.VIEWER)

class GraphicsView(QGraphicsView):
    def __init__(self, core, parent=None):
        super().__init__(parent)
        self.core = core
        self.setMinimumSize(200, 120)
        self.setMaximumSize(999, 160)
        self.setVerticalScrollBarPolicy(Qt.ScrollBarAlwaysOff)
        self.setHorizontalScrollBarPolicy(Qt.ScrollBarAlwaysOff)
        self.scene_ = QGraphicsScene()
        self.setScene(self.scene_)
        self.is_horizontal = True
        self.src_image = None
        self.old_geometry = QRectF(0.0, 0.0, 1000.0, 563.0)
        # editing_mode is True when the viewer is the EditingArea instance
        self.editing_mode = False
        self.main_viewer = None

    def get_geometry(self):
        try:
            self.p_rect = self.main_viewer.scene().sceneRect()
        except RuntimeError:
            # This can happens when the entire main window is resized
            self.p_rect = self.old_geometry

        if self.p_rect.width() < 5:
            return

        self.old_geometry = self.p_rect
        s_rect = self.scene_.sceneRect()
        self.factor = s_rect.width() / self.p_rect.width()
        u_offset = self.main_viewer.mapFromScene(float(self.p_rect.x()),
                                            float(self.p_rect.y()))

        x = u_offset.x()
        if x >= 0:
            x = 0

        y = u_offset.y()
        if y >= 0:
            y = 0

        width = self.main_viewer.width() * self.factor
        if width > s_rect.width():
            width = s_rect.width()

        height = self.main_viewer.height() * self.factor
        if height > s_rect.height():
            height = s_rect.height()

        # window
        g = [abs(x) * self.factor, abs(y) * self.factor, width, height]
        return g
            
    def display_image(self, viewer, qimage):
        self.main_viewer = viewer
        self.src_image = qimage
        w, h = self.width(), self.height()
        self.base = w
        if qimage.width() < qimage.height():
            self.is_horizontal = False
            self.base = h

        self.pixmap = QPixmap(qimage.scaled(w, h, Qt.KeepAspectRatio, 
                                    Qt.SmoothTransformation))

        self.change_preview()

    def change_preview(self):
        self.scene_.clear()
        self.scene_.setSceneRect(QRectF(self.pixmap.rect()))
        self.scene_.addPixmap(self.pixmap)
        self.set_mask(self.get_geometry())
        
    def set_mask(self, r):
        self.window = Window(self, r, None, self.scene_)
        s_rect = self.scene_.sceneRect()
        self.window.w_width = s_rect.width()
        self.window.w_height = s_rect.height()
        self.window.positionChanged.connect(self._update_viewer)

    def _update_viewer(self, value):
        valX = value[0] / self.factor
        valY = value[1] / self.factor
        self.block_sync(True)
        self.core.center_views(valX, valY)
        self.block_sync(False)

    def reset_window(self):
        if self.src_image is not None:
            try:
                self.scene_.removeItem(self.window)
            except:
                pass
            self.window = None
            self.set_mask(self.get_geometry())

    def resizeEvent(self, event):
        if self.src_image is not None:
            self.display_image(self.main_viewer, self.src_image)
        QGraphicsView.resizeEvent(self, event)

    def block_sync(self, b):
        self.main_viewer.is_auto_scroll = b

    def set_background(self):
        self.src_image = None
        pix = QPixmap('medias/pix_viewer_87x80.png')
        self.scene_.clear()
        self.scene_.setSceneRect(QRectF(pix.rect()))
        self.scene_.addPixmap(pix)


class Window(QGraphicsPixmapItem):
    def __init__(self, main, rect, parent=None, scene=None):
        super().__init__(parent)
        if scene is not None: 
            scene.addItem(self)
        self.main = main
        self.viewer = main.main_viewer
        self.posX = rect[0]
        self.posY = rect[1]
        self.width = rect[2]
        self.height = rect[3]
        self.w_width = 0
        self.w_height = 0
        self.setFlags(QGraphicsItem.ItemIsMovable | 
                        QGraphicsItem.ItemSendsScenePositionChanges |
                        QGraphicsItem.ItemIsFocusable)
        self.setAcceptHoverEvents(True)
        color = QColor()
        color.setRgb(100, 220, 255, 130)
        pix = QPixmap(QSize(self.width, self.height))
        pix.fill(color)
        color.setRgb(230, 240, 255, 255)
        painter = QPainter()
        painter.begin(pix)
        painter.setPen(QPen(color, 1, Qt.SolidLine, Qt.RoundCap, Qt.MiterJoin))
        painter.drawRect(0.0, 0.0, self.width - 2, self.height - 2)
        painter.end()
        self.setPixmap(pix)
        self.setPos(self.posX, self.posY)
        self._moved = Moved()

    def positionChanged():
        def fget(self):
            return self._moved.positionChanged
        return locals()
    positionChanged = property(**positionChanged())

    def mousePressEvent(self, event):
        pos = event.scenePos()
        if event.button() == 1:
            self.old_X, self.old_Y = pos.x(), pos.y()
        event.accept()

    def mouseMoveEvent(self, event):
        pos = event.scenePos()
        if event.button() == 2:
            return

        X, Y = pos.x(), pos.y()
        m_X, m_Y = X - self.old_X, Y - self.old_Y
        newX, newY = self.posX + m_X, self.posY + m_Y
        if newX <= 0 or newX + self.width >= self.w_width + 2:
            m_X = 0

        if newY <= 0 or newY + self.height >= self.w_height + 2:
            m_Y = 0
       
        self.old_X, self.old_Y = X, Y
        self.moveBy(m_X, m_Y)
        self._update()
        self.positionChanged.emit([self.posX + (self.width / 2), 
                                   self.posY + self.height / 2])
        event.accept()

    def hoverEnterEvent(self, event):
        self.old_shape = self.viewer.cursor().shape()
        shape = QPixmap("medias/cross_cursor_32.png")
        cursor = QCursor(shape, -1, -1)
        self.setCursor(cursor)

    def hoverLeaveEvent(self, event):
        cursor = QCursor(self.old_shape)
        self.viewer.setCursor(cursor)

    def _update(self):
        self.posX, self.posY = self.pos().x(), self.pos().y()


class Moved(QObject):
    """Signal used by aerial view when window's position changed.

    """
    positionChanged = pyqtSignal(list)
    def __init__(self):
        super(Moved, self).__init__()
