# -*- coding: utf-8 -*-

# sourcefolderbox.py
# this file is part of Oqapy serie 3
#
# Author: Vincent Vande Vyvre <vincent.vandevyvre@oqapy.eu>
# Home: http://www.oqapy.eu
# Copyright: 2014-2017 Vincent Vande Vyvre
# Licence: GPL3
#
# Source folder tool box

import os
import logging
lgg = logging.getLogger(__name__)

from PyQt5.QtCore import Qt, QSize
from PyQt5.QtGui import QIcon, QPixmap
from PyQt5.QtWidgets import (QWidget, QGridLayout, QHBoxLayout, QVBoxLayout,
                             QLabel, QSpacerItem, QToolButton, QPushButton,
                             QRadioButton, QCheckBox, QSizePolicy, QButtonGroup)

import stylesheets as styles

from VWidgets_PyQt5.vlineedit import VLineEdit


class SourceFolderBox(QWidget):
    def __init__(self, ui, parent=None):
        super().__init__(parent)
        self.name = 'SourceFolderBox'
        self.ui = ui
        self.core = ui.core
        cfg = self.core.cfg
        self.vlayout = QVBoxLayout(self)
        self.vlayout.setContentsMargins(0, 2, 8, 2)
        self.gridLayout = QGridLayout()
        spacer = QSpacerItem(20, 20, QSizePolicy.Fixed, QSizePolicy.Minimum)
        self.gridLayout.addItem(spacer, 0, 0, 1, 1)
        self.source_lbl = QLabel(self)
        self.source_lbl.setAlignment(Qt.AlignRight|Qt.AlignVCenter)
        self.gridLayout.addWidget(self.source_lbl, 0, 1, 1, 1)
        self.source_led = VLineEdit(self)
        self.source_led.setResetButton(True)
        self.source_led.setHistoricButton(True)
        self.gridLayout.addWidget(self.source_led, 0, 2, 1, 1)
        self.ignore_raw_chb = QCheckBox(self)
        self.gridLayout.addWidget(self.ignore_raw_chb, 1, 2, 1, 1)
        self.hlayout = QHBoxLayout()
        self.browse_btn = QToolButton(self)
        self.browse_btn.setText("...")
        self.hlayout.addWidget(self.browse_btn)
        self.open_btn = QPushButton(self)
        icon = QIcon()
        icon.addPixmap(QPixmap("medias/apply.png"), QIcon.Normal, QIcon.Off)
        self.open_btn.setIcon(icon)
        self.open_btn.setIconSize(QSize(18, 18))
        self.hlayout.addWidget(self.open_btn)
        space1 = QSpacerItem(40, 20, QSizePolicy.Expanding, QSizePolicy.Minimum)
        self.hlayout.addItem(space1)
        self.gridLayout.addLayout(self.hlayout, 2, 2, 1, 1)
        self.vlayout.addLayout(self.gridLayout)
        self.hlayout_1 = QHBoxLayout()
        spacer_1 = QSpacerItem(20, 20, QSizePolicy.Fixed, QSizePolicy.Minimum)
        self.hlayout_1.addItem(spacer_1)
        self.label = QLabel(self)
        self.hlayout_1.addWidget(self.label)
        space2 = QSpacerItem(40, 20, QSizePolicy.Preferred, QSizePolicy.Minimum)
        self.hlayout_1.addItem(space2)
        self.vlayout.addLayout(self.hlayout_1)
        self.hlayout_2 = QHBoxLayout()
        space3 = QSpacerItem(20, 20, QSizePolicy.Fixed, QSizePolicy.Minimum)
        self.hlayout_2.addItem(space3)
        self.last_open_rdo = QRadioButton(self)
        self.hlayout_2.addWidget(self.last_open_rdo)
        space4 = QSpacerItem(40, 20, QSizePolicy.Preferred, QSizePolicy.Minimum)
        self.hlayout_2.addItem(space4)
        self.vlayout.addLayout(self.hlayout_2)
        self.hlayout_3 = QHBoxLayout()
        space5 = QSpacerItem(20, 20, QSizePolicy.Fixed, QSizePolicy.Minimum)
        self.hlayout_3.addItem(space5)
        self.selected_folder_rdo = QRadioButton(self)
        self.selected_folder_rdo.setText(' ')
        self.hlayout_3.addWidget(self.selected_folder_rdo)
        self.launch_with_led = VLineEdit(self)
        self.launch_with_led.setResetButton(True)
        self.launch_with_led.setHistoricButton(True)
        self.hlayout_3.addWidget(self.launch_with_led)
        self.selected_browse_btn = QToolButton(self)
        self.selected_browse_btn.setText("...")
        self.hlayout_3.addWidget(self.selected_browse_btn)
        self.vlayout.addLayout(self.hlayout_3)
        self.hlayout_4 = QHBoxLayout()
        space6 = QSpacerItem(20, 20, QSizePolicy.Fixed, QSizePolicy.Minimum)
        self.hlayout_4.addItem(space6)
        self.no_folder_rdo = QRadioButton(self)
        self.hlayout_4.addWidget(self.no_folder_rdo)
        space7 = QSpacerItem(40, 20, QSizePolicy.Preferred, QSizePolicy.Minimum)
        self.hlayout_4.addItem(space7)
        self.vlayout.addLayout(self.hlayout_4)

        self.retranslateUi()
        self.radio_buttons = QButtonGroup()
        self.radio_buttons.addButton(self.no_folder_rdo, 0)
        self.radio_buttons.addButton(self.last_open_rdo, 1)
        self.radio_buttons.addButton(self.selected_folder_rdo, 2)
        self.set_style()
        self.open_btn.setEnabled(False)
        self.ignore_raw_chb.setChecked(cfg.get(('rawfiles', 'ignore')))
        self.source_led.textChanged.connect(self.on_source_edited)
        self.browse_btn.clicked.connect(self.browse_to_source_folder)
        self.ignore_raw_chb.stateChanged.connect(self.core.set_raw_files_handling)
        self.open_btn.clicked.connect(self.open_source_folder)

    def set_config(self, cfg):
        self.no_folder_rdo.setChecked(True)
        if cfg['open']:
            self.last_open_rdo.setChecked(True)

        elif cfg['always_open']:
            self.selected_folder_rdo.setChecked(True)
            self.launch_with_led.setText(cfg['always_open'])

        self.radio_buttons.buttonClicked[int].connect(self.change_config)
        self.launch_with_led.editingFinished.connect(self.set_folder_name)

    def on_source_edited(self, path):
        """Invoked when the folder line edit is edited.

        Args:
        path -- the new path edited
        """
        self.open_btn.setEnabled(False)
        if not os.path.isdir(path):
            color = '#FF0000'

        else:
            color = '#000000'
            if not self.core.is_source_folder(path):
                self.open_btn.setEnabled(True)

        self.source_led.setStyleSheet("""QLineEdit {border: 2px solid #A0A5B2;
                                    border-radius: 6px;
                                    background: white;
                                    color: %s;
                                    selection-background-color: darkgray;}"""
                                    %(color))

    def browse_to_source_folder(self):
        fld = self.core.browse_to_source_folder()
        if os.path.isdir(fld):
            self.source_led.setText(fld)

        else:
            self.open_btn.setEnabled(False)

    def open_source_folder(self):
        fld = self.source_led.text()
        if os.path.isdir(fld):
            self.core.open_source_folder(fld)
            idx = self.radio_buttons.checkedId()
            if idx == -1:
                idx = 0
            self.change_config(idx)

        self.open_btn.setEnabled(False)

    def change_config(self, idx=0):
        lgg.info('Change startup config: %s' % idx)
        self.idx = idx or 1
        open_ = always = ''
        if idx == 1:
            open_ = self.source_led.text()

        elif idx == 2:
            always = self.launch_with_led.text()

        self.core.cfg.block_update = True
        self.core.cfg.update(('startup', 'open'), open_)
        self.core.cfg.block_update = False
        self.core.cfg.update(('startup', 'always_open'), always)

    def set_folder_name(self):
        if self.selected_folder_rdo.isChecked():
            self.core.cfg.block_update = True
            self.core.cfg.update(('startup', 'open', ''))
            self.core.conf.block_update = False
            self.core.cfg.update(('startup', 'always_open'),
                                 self.launch_with_led.text())

    def reset(self):
        pass
        
    def set_style(self):
        self.source_led.setStyleSheet(styles.LINEEDIT)
        self.source_led.informative_text.setStyleSheet(
                                            styles.LINEEDITPLACEHOLDER)
        self.source_led.reset_button.setStyleSheet(styles.LINEEDITBUTTON)
        self.source_led.historic_button.setStyleSheet(styles.LINEEDITBUTTON)
        self.ignore_raw_chb.setStyleSheet(styles.CHECKBOX)
        self.browse_btn.setStyleSheet(styles.TBUTTON)
        self.open_btn.setStyleSheet(styles.PUSHBUTTON)
        self.last_open_rdo.setStyleSheet(styles.RADIO)
        self.selected_folder_rdo.setStyleSheet(styles.RADIO)
        self.launch_with_led.setStyleSheet(styles.LINEEDIT)
        self.launch_with_led.informative_text.setStyleSheet(
                                            styles.LINEEDITPLACEHOLDER)
        self.launch_with_led.reset_button.setStyleSheet(styles.LINEEDITBUTTON)
        self.launch_with_led.historic_button.setStyleSheet(
                                            styles.LINEEDITBUTTON)
        self.selected_browse_btn.setStyleSheet(styles.TBUTTON)
        self.no_folder_rdo.setStyleSheet(styles.RADIO)

    def retranslateUi(self):
        self.source_lbl.setText(_("Source:"))
        self.source_led.setInformativeText(_("Path of source folder"))
        self.ignore_raw_chb.setText(_("Ignore RAW files"))
        self.browse_btn.setStatusTip(_("Browse"))
        self.open_btn.setStatusTip(_("Open the source folder"))
        self.open_btn.setText(_("Open"))
        self.label.setText(_("Always launch Oqapy with:"))
        self.last_open_rdo.setText(_("Last open folder"))
        self.launch_with_led.setStatusTip(_("Enter the path of the folder"))
        self.launch_with_led.setInformativeText(_("Path of folder"))
        self.selected_browse_btn.setStatusTip(_("Browse"))
        self.no_folder_rdo.setText(_("Don't open any folder"))


