# -*- coding: utf-8 -*-

# statusbar.py
# This file is part of Oqapy serie 3
#
# Author: Vincent Vande Vyvre <vincent.vandevyvre@oqapy.eu>
# Home: http://www.oqapy.eu
# Copyright: 2009-2018 Vincent Vande Vyvre
# Licence: GPL3
#
# Status bar

import logging
lgg = logging.getLogger(__name__)

from PyQt5.QtCore import QObject, Qt, QSize
from PyQt5.QtGui import QIcon, QPixmap
from PyQt5.QtWidgets import (QStatusBar, QGridLayout, QToolButton, QFrame,
                             QSizePolicy, QLabel, QVBoxLayout, QWidget,
                             QHBoxLayout, QSpacerItem)

import stylesheets as styles


class StatusBar(QObject):
    """Define the status bar.

    """
    def __init__(self, ui):
        """Instanciate the status bar.

        Args:
        ui -- the main window instance
        """
        super().__init__()
        self.ui = ui
        self.core = ui.core
        self.widget = UiStatusBar(self)
        self.widget.view_state_btn.clicked.connect(self.on_viewing_clicked)
        self.widget.sort_state_btn.clicked.connect(self.on_sorting_clicked)
        self.widget.full_state_btn.clicked.connect(self.on_fullscreen_clicked)
        self.widget.raw_state_btn.clicked.connect(self.on_rawview_clicked)
        self.set_current_mode(0)
        self.total_images = 0

    def on_viewing_clicked(self):
        """Slot of the viewing state button.

        """
        self.set_current_mode(1)
        self._check_multiviewing()
        self.core.set_viewing_state()

    def on_sorting_clicked(self):
        """Slot of the sorting state button.

        """
        self.set_current_mode(2)
        self._check_multiviewing()
        self.core.set_sorting_state()

    def on_fullscreen_clicked(self):
        """Slot of the full screen state button.

        """
        self.set_current_mode(3)
        self._check_multiviewing()
        self.core.state.set_current_state(3)

    def on_rawview_clicked(self):
        """Slot of the RAW images viewing state button.

        """
        self.set_current_mode(4)
        self._check_multiviewing()
        self.core.set_raw_files_viewing()

    def _check_multiviewing(self):
        if self.core.multiviewing:
            self.core.close_multiviewing(update=True)
            return False

        return True

    def set_current_mode(self, mode):
        """Set the current mode.

        Args:
        mode -- int()
                0: disable all buttons
                1: viewing state
                2: sorting state
                3: full screen state
                4: RAW files editing
        """
        lgg.info('Change mode to: %s' % mode)
        buttons = [self.widget.view_state_btn,
                   self.widget.sort_state_btn,
                   self.widget.full_state_btn,
                   self.widget.raw_state_btn,]
        if not mode:
            for b in buttons:
                b.setEnabled(False)
            self.current_mode_button = False
            self.set_image_index()
            return

        for b in buttons:
            b.setEnabled(True)

        self.enable_raw_viewing(True)
        b = buttons[mode-1]
        b.setEnabled(False)
        self.current_mode_button = b

    def display_message(self, what, timer=0):
        """Display an informative message.

        Args:
        what -- the message
        timer -- the duration in seconds
        """
        if timer:
            self.widget.showMessage(what, timer)

        else:
            self.widget.showMessage(what)

    def set_image_index(self, index=None, total=0):
        """Update the image index.

        Args:
        index -- the index of the current image, if None the label is cleared
        total -- the number of the images into the current folder
        """
        label = self.widget.image_idx_lbl
        if index is None:
            label.setText('')

        else:
            if total:
                self.total_images = total

            label.setText('Picture %s/%s' %(index, self.total_images))

    def enable_index(self, enable):
        """Set enabled or not the image index.

        Args:
        enable -- boolean
        """
        if not enable:
            self.widget.image_idx_lbl.setText('')

        self.widget.image_idx_lbl.setEnabled(enable)

    def enable_raw_viewing(self, enable):
        """Set enabled or not the raw viewing button.

        Args:
        enable -- boolean
        """
        state = bool(enable and self.core.source_fld.raws)
        self.widget.raw_state_btn.setEnabled(state)

    def set_raw_only(self, raw):
        self.widget.view_state_btn.setEnabled(not raw)
        self.widget.sort_state_btn.setEnabled(not raw)
        self.widget.full_state_btn.setEnabled(not raw)


class UiStatusBar(QStatusBar):
    def __init__(self, driver, parent=None):
        super().__init__(parent)
        self.driver = driver
        # widget in status bar
        # Status button: mode sorting, viewing and full screen
        self.icon0 = QIcon()
        self.icon0.addPixmap(QPixmap("medias/sort_btn_off_15x12.png"), 
                                        QIcon.Normal, QIcon.Off)
        self.icon1 = QIcon()
        self.icon1.addPixmap(QPixmap("medias/sort_btn_on_32x17.png"), 
                                        QIcon.Normal, QIcon.Off)
        self.icon2 = QIcon()
        self.icon2.addPixmap(QPixmap("medias/view_btn_off_20x13.png"), 
                                        QIcon.Normal, QIcon.Off)
        self.icon3 = QIcon()
        self.icon3.addPixmap(QPixmap("medias/view_btn_on_32x17.png"), 
                                        QIcon.Normal, QIcon.Off)
        self.icon4 = QIcon()
        self.icon4.addPixmap(QPixmap("medias/full_btn_35x20.png"), 
                                        QIcon.Normal, QIcon.Off)
        self.icon5 = QIcon()
        self.icon5.addPixmap(QPixmap("medias/raw_btn_on.png"), 
                                        QIcon.Normal, QIcon.Off)
        self.icon6 = QIcon()
        self.icon6.addPixmap(QPixmap("medias/raw_btn.png"), 
                                        QIcon.Normal, QIcon.Off)
        self.view_state_btn = QToolButton(self)
        self.view_state_btn.setMaximumSize(35, 20)
        self.view_state_btn.setAutoRaise(True)
        self.view_state_btn.setIcon(self.icon3)
        self.view_state_btn.setIconSize(QSize(35, 18))
        self.addPermanentWidget(self.view_state_btn)
        self.sort_state_btn = QToolButton(self)
        self.sort_state_btn.setMaximumSize(31, 20)
        self.sort_state_btn.setAutoRaise(True)
        self.sort_state_btn.setIcon(self.icon0)
        self.sort_state_btn.setIconSize(QSize(35, 18))
        self.addPermanentWidget(self.sort_state_btn)
        self.full_state_btn = QToolButton(self)
        self.full_state_btn.setMaximumSize(35, 20)
        self.full_state_btn.setAutoRaise(True)
        self.full_state_btn.setIcon(self.icon4)
        self.full_state_btn.setIconSize(QSize(35, 18))
        self.addPermanentWidget(self.full_state_btn)
        self.raw_state_btn = QToolButton(self)
        self.raw_state_btn.setMaximumSize(60, 20)
        self.raw_state_btn.setAutoRaise(True)
        self.raw_state_btn.setIcon(self.icon6)
        self.raw_state_btn.setEnabled(False)
        self.raw_state_btn.setIconSize(QSize(50, 18))
        self.addPermanentWidget(self.raw_state_btn)

        # Image index
        self.image_idx_lbl = QLabel()
        self.image_idx_lbl.setMinimumSize(QSize(150, 0))
        self.image_idx_lbl.setMaximumSize(QSize(150, 20))
        self.image_idx_lbl.setFrameShape(QFrame.Box)
        self.image_idx_lbl.setFrameShadow(QFrame.Sunken)
        self.image_idx_lbl.setAlignment(Qt.AlignRight)
        self.image_idx_lbl.setStyleSheet("QWidget { background-color: white }")
        self.addPermanentWidget(self.image_idx_lbl)
        self.sort_state_btn.setStatusTip(_("Mode sorting."))
        self.view_state_btn.setStatusTip(_("Mode viewing."))
        self.full_state_btn.setStatusTip(_("Mode full screen."))
        self.raw_state_btn.setStatusTip(_("Show RAW files"))

        self.set_style()

    def set_mode(self, mode):
        lgg.info("Change context: %s" % mode)
        self.sort_state_btn.blockSignals(True)
        self.view_state_btn.blockSignals(True)

        if mode == "viewing":
            self.sort_state_btn.setIcon(self.icon0)
            self.view_state_btn.setIcon(self.icon3)

        else:
            self.sort_state_btn.setIcon(self.icon1)
            self.view_state_btn.setIcon(self.icon2)

        self.sort_state_btn.blockSignals(False)
        self.view_state_btn.blockSignals(False)

    def set_raw_mode(self, mode):
        icons = (self.icon6, self.icon5)
        tips = (_("Show RAW files"), _("Hide RAW files"))
        self.raw_state_btn.setIcon(icons[mode])
        self.raw_state_btn.setStatusTip(tips[mode])

    def set_style(self):
        self.setStyleSheet(styles.STATUSBAR)
        self.view_state_btn.setStyleSheet(styles.STATUSBUTTON)
        self.sort_state_btn.setStyleSheet(styles.STATUSBUTTON)
        self.full_state_btn.setStyleSheet(styles.STATUSBUTTON)
        self.raw_state_btn.setStyleSheet(styles.STATUSBUTTON)

