# targetarea.py
# This file is part of Oqapy serie 3
#
# Author: Vincent Vande Vyvre <vincent.vandevyvre@oqapy.eu>
# Home: http://www.oqapy.eu
# Copyright: 2009-2017 Vincent Vande Vyvre
# Licence: GPL3
#
# The target folders widget

import logging
lgg = logging.getLogger(__name__)

from PyQt5.QtCore import Qt, pyqtSignal, QLocale
from PyQt5.QtGui import QFont, QCursor, QColor, QPen, QPainter
from PyQt5.QtWidgets import (QWidget, QGridLayout, QSizePolicy, QToolButton, 
                             QMenu, QAction)

LOCALE = QLocale.system().name()


class TargetArea(QWidget):
    def __init__(self, ui, parent):
        super().__init__(parent)
        self.ui = ui
        self.core = ui.core
        self.gl_1 = QGridLayout(self)
        self.gl_1.setContentsMargins(0, 0, 0, 0)
        self.gl_1.setSpacing(0)
        self.count = 6
        self.target_selected = None

    def build_targets(self):
        """Update the target zone.

        The target zone is the buttons of destination directories.
        This function is running only at startup.
        The number of buttons is 6 to 24 and maybe change in the future,
        if number of buttons is less than 13, one line is created, otherwise
        two lines are created.
        """
        lgg.info('Build targets buttons')
        self.targets_buttons = []
        def _generate_button(beg, nb, line, col, hb, w):
            """Generate buttons in target zone.

            This function is called for each line of button.

            Keyword arguments :
            beg -- nummer of directory at the begin of button's line.
            nb -- number of buttons.
            line -- line in the layout.
            col -- column in the layout.
            hb -- height of buttons
            w -- width of button
            """
            col = 0
            for i in range(beg, nb):
                # folder's name
                txt = "..."
                fld = self.core.targets[i]
                # button
                b = Target(self, i, w, hb, self)
                sp = QSizePolicy(QSizePolicy.Expanding, QSizePolicy.Fixed)
                b.setSizePolicy(sp)
                if fld is not None:
                    txt = fld.path.name
                    fld.button = b
                b.set_text(txt)
                b.pressed.connect(self.core.on_target_clicked)
                self.gl_1.addWidget(b, line, col, 1, 1)
                b.hide()
                col += 1
                self.targets_buttons.append(b)

        w = int((self.ui.stackedviews.size().width() / 12) - 2)
        _generate_button(0, 12, 0, 0, 25, w)
        _generate_button(12, 24, 1, 0, 25, w)
        for idx, rep in enumerate(self.core.targets):
            if rep is not None:
                rep.get_free_space()
        self.ui.toolpanel.targets_tool.set_config()

    def change_number_targets(self, number):
        """Set the number of the target's buttons.

        Args:
        number -- the number of buttons
        """
        lgg.info('Change targets number to %s' % number)
        if self.core.state.context == 2:
            for idx in range(0, number):
                self.targets_buttons[idx].show()

            for idx in range(number+1, 24):
                self.targets_buttons[idx].hide()

        self.count = number

    def open_selected_target(self):
        """Open a new folder on the selected button.

        """
        if self.target_selected is None or self.core.state.context != 2:
            return

        self.core.link_new_target(self.target_selected)

    def close_selected_target(self):
        """Close the folder open on the selected button.

        """
        if self.target_selected is None or self.core.state.context != 2:
            return

        self.targets_buttons[self.target_selected].disable_folder()

    def unselect_target(self):
        """Unselect the selected button.

        """
        if self.target_selected is None or self.core.state.context != 2:
            return

        self.targets_buttons[self.target_selected].set_selected(False)
        self.target_selected = None
        self.core.state.current_viewer.setFocus(True)

    def shift_selected_target_to_right(self):
        """Move the selection to the right.

        """
        if self.core.state.context != 2:
            return

        if self.target_selected is None:
            idx = 0

        else:
            old = self.targets_buttons[self.target_selected]

            try:
                b = self.targets_buttons[self.target_selected + 1]
                if b.isVisible():
                    idx = self.target_selected + 1

                else:
                    idx = 0
            except:
                # old == 24
                idx = 0

            old.set_selected(False)

        b = self.targets_buttons[idx]
        b.set_selected(True)
        self.target_selected = idx
        self.core.state.current_viewer.setFocus(True)

    def shift_selected_target_to_left(self):
        """Move the selection to the left.

        """
        if self.core.state.context != 2:
            return

        if self.target_selected is None:
            idx = self.count -1

        else:
            if self.target_selected == 0:
                idx = self.count -1

            else:
                idx = self.target_selected - 1

            self.targets_buttons[self.target_selected].set_selected(False)

        b = self.targets_buttons[idx]
        b.set_selected(True)
        self.target_selected = idx
        self.core.state.current_viewer.setFocus(True)

    def select_target_with_first_letter(self, l):
        """Select a target button with the first letter of the folder name.

        Args:
        l -- the letter
        """
        if self.core.state.context != 2:
            return

        last = self.target_selected
        if last is None:
            last = 1

        else:
            old = self.targets_buttons[last]
            old.set_selected(False)

        loop = last + 1
        while 1:
            t = self.targets_buttons[loop]
            if not t.button.isVisible():
                loop = 1

            elif t.button._text.upper().startswith(chr(l).upper()):
                t.button.set_selected(True)
                self.target_selected = loop
                break

            else:
                loop += 1

            if loop == last + 1:
                break

    def copy_to_selected_target(self):
        """Copy the current image file in selected target.

        """
        if self.target_selected is None or self.core.state.context != 2:
            return

        self.core.on_target_clicked(self.target_selected, 1)

    def move_to_selected_target(self):
        """Move the current image file to the selected target.

        This equivalent of cut-paste.
        """
        if self.target_selected is None or self.core.state.context != 2:
            return

        self.core.on_target_clicked(self.target_selected, 4)


class Target(QToolButton):
    pressed = pyqtSignal(int, int)
    menuRequested = pyqtSignal(int)
    def __init__(self, ui, idx, width, heigth, parent=None):
        super().__init__(parent)
        self.ui = ui
        self.core = ui.core
        self.idx_ = idx
        self.text_ = "..."
        self.fld_icon = None
        self.is_gauged = False
        self.max = 0.0
        self.sum = 1.0
        self.free = 0.0
        self.is_selected = False
        self.width_ = width
        self.height_ = heigth
        self.length = 0
        self.color = "#00AAAA"
        self.icon_ = None
        self.setAutoRaise(True)
        toolButton_style = """
                QToolButton {background-color: qlineargradient(
                    x1: 0, y1: 0, x2: 0, y2: 1,
                    stop: 0 #555753, stop: 0.6 #E8E8E8,
                    stop: 0.7 #FFFFFF, stop: 1.0 #555753);
                    border: 1px solid #000000;
                    border-radius: 4px;
                    min-width: 6ex;
                    padding: 2px;
                    color: #181E26;}
                QToolButton:hover {border: 2px solid #0F2076;
                    border-radius: 4px;
                    color: #0F2076;}
                QToolButton:pressed {background-color: qlineargradient(
                    x1: 0, y1: 0, x2: 0, y2: 1,
                    stop: 0 #DADBDE, stop: 1 #F6F7FA);}"""
        self.setStyleSheet(toolButton_style)
        self.create_context_menu()
        self.setText(self.text_)
        self.setStatusTip(
             _("Left click to copy the current image, right click to move it"))

    def set_text(self, text):
        self.text_ = text
        self.setText(self.text_)

    def get_text(self):
        return self.text_

    def reset_gauge(self):
        fld = self.core.targets[self.idx_]
        if fld is not None:
            fld.get_free_space()
            self.sum = fld.used
            self.free = fld.free
            self._update_gauge()

    def set_icon(self, icon):
        # Not implemented yet
        pass

    def set_selected(self, b):
        font = QFont()
        font.setFamily("Trebuchet MS")

        if b:
            font.setPointSize(11)
            font.setBold(True)

        else:
            font.setPointSize(10)
            font.setBold(False)

        self.setFont(font)
        self.is_selected = b

    def _update_gauge(self):
        if self.sum >= self.max:
            self.length = self.width_ - 3
            self.color = "#FF0000"
            return

        if not self.sum:
            # Avoid division by zero
            self.sum = 1.0

        f = self.max / self.sum
        self.length = int(self.width_ / f)

        if f > 1.1:
            self.color = "#00FF00"

        else:
            self.color = "#BBAA00"

    def disable_folder(self):
        """Close the folder and liberate the button.

        This function is called from menu or properties panel.
        """
        self._close()

    def unlink(self):
        """Close the folder and liberate the button.

        This function is called from the menu.
        """
        self._close()
        self.core.unlink_target(self.idx_)

    def mousePressEvent(self, event):
        mb = event.button()
        self.pressed.emit(self.idx_, mb)
        if mb == 2:
            self.show_context_menu()

        elif mb == 1 or mb == 4 and self.is_gauged:
            self.reset_gauge()

        else:
            QToolButton.mousePressEvent(self, event)

    def paintEvent(self, event):
        QToolButton.paintEvent(self, event)
        if self.is_gauged:
            col = QColor(self.color)
            p = QPainter(self)
            p.setPen(QPen(col, 4,
                      Qt.SolidLine, Qt.RoundCap,
                      Qt.MiterJoin))
            p.drawLine(3, self.height_-4, self.length, self.height_-4)
            p.end()

    def resizeEvent(self, event):
        self.width_ = self.width()
        self.height_ = self.height()
        self.reset_gauge()
        event.ignore()

    def create_context_menu(self):
        """Create context menu for target's buttons.

        If the option 'Limit the size of folder' is True,
        we show the total size of the files contained in the folder.
        """
        # TODO Add an option "Open as source folder"
        self.pop_menu = QMenu(self)
        self.open_act = QAction(_("Link"), self.pop_menu)
        self.pop_menu.addAction(self.open_act)
        self.open_act.triggered.connect(self._open)
        self.close_act = QAction(_("Unlink"), self.pop_menu)
        self.pop_menu.addAction(self.close_act)
        self.close_act.triggered.connect(self.unlink)
        self.select_act = QAction(_("Select"), self.pop_menu)
        self.pop_menu.addAction(self.select_act)
        self.select_act.triggered.connect(self._select)
        self.unselect_act = QAction(_("Unselect"), self.pop_menu)
        self.pop_menu.addAction(self.unselect_act)
        self.unselect_act.triggered.connect(self._unselect)
        self.separator = self.pop_menu.addSeparator()
        self.gauge_act = QAction("", self.pop_menu)
        self.pop_menu.addAction(self.gauge_act)

        self.pop_menu.setContextMenuPolicy(Qt.ActionsContextMenu)
        self.pop_menu.setStyleSheet("""
                QMenu {background: transparent;
                    border: 2px solid #8A8E96;
                    border-radius: 8px;}
                QMenu::item {background: #FFFFFF;
                    color: #000000;
                    padding: 2px 25px 2px 20px;
                    border: 1px solid transparent;}
                QMenu::item:selected {background-color: #E8E8F8;
                    border-color: #0F2D78;
                    border-radius: 4px;
                    color: #0F2D78;}
                QMenu::icon:checked {background: gray;
                    border: 1px inset gray;
                    position: absolute;
                    top: 1px;
                    right: 1px;
                    bottom: 1px;
                    left: 1px;}
                QMenu::separator {height: 2px;
                    background: #BABDD6;}""")

    def show_context_menu(self):
        language = QLocale(LOCALE)
        chain = language.toString(int(self.sum))
        self.close_act.setVisible(self.text_ != "...")
        self.select_act.setVisible(not self.is_selected and self.text_ != "...")
        self.unselect_act.setVisible(self.is_selected)

        if self.is_gauged:
            label = _("Content: ")

            label += chain
            chain = _(" bytes")
            label += chain
            self.separator.setVisible(True)
            self.gauge_act.setText(label)
            self.gauge_act.setVisible(True)

        else:
            self.separator.setVisible(False)
            self.gauge_act.setVisible(False)

        self.pop_menu.exec_(QCursor.pos())

    def _open(self):
        if self.text_ != "...":
            self._close()

        self.core.link_new_target(self.idx_)

    def _close(self):
        self.set_text("...")
        self.icon_ = None
        self.is_gauged = False
        self.max = 0.0
        self.sum = 1.0

        if self.is_selected:
            self.set_selected(False)

    def _select(self):
        self.set_selected(True)

    def _unselect(self):
        self.set_selected(False)
 
