# -*- coding: utf-8 -*-

# toolbar.py
# This file is part of Oqapy serie 3
#
# Author: Vincent Vande Vyvre <vincent.vandevyvre@oqapy.eu>
# Home: http://www.oqapy.eu
# Copyright: 2009-2018 Vincent Vande Vyvre
# Licence: GPL3
#
# Tool bar

import logging
lgg = logging.getLogger(__name__)

from PyQt5.QtCore import QObject, Qt, QSize, pyqtSignal
from PyQt5.QtGui import QIcon, QPixmap
from PyQt5.QtWidgets import (QWidget, QGridLayout, QHBoxLayout, QToolButton,
                             QSpacerItem, QSizePolicy, QSlider, QVBoxLayout,
                             QDoubleSpinBox, QAction, QActionGroup, QLabel)

import stylesheets as styles


class ToolBar(QObject):
    def __init__(self, ui, parent):
        super().__init__()
        self.ui = ui
        self.core = ui.core
        self.widget = UiToolBar(self, parent)

    def configure(self):
        lgg.info('Configure tool bar')
        self.activate_widgets(False)
        self.activate_zoom_widgets(False)
        w = self.widget
        z = self.widget.zoom_tools
        w.trailer_btn.pressed.connect(self.ui.toggle_trailer)
        w.multiviews_state_btn.activated.connect(self.on_multi_clicked)
        w.magnifier_btn.clicked.connect(self.core.toggle_magnifier)
        w.right_btn.clicked.connect(self.core.rotate_to_right)
        w.prev_btn.clicked.connect(self.core.previous_image)
        w.next_btn.clicked.connect(self.core.next_image)
        w.left_btn.clicked.connect(self.core.rotate_to_left)
        z.zoom_area_btn.clicked.connect(self.core.get_zoom_area)
        z.zoom_full_btn.clicked.connect(self.core.on_zoom_full_size)
        z.zoom_up_btn.clicked.connect(self.core.on_zoom_up)
        z.zoom_spb.editingFinished.connect(self.core.on_zoom_edited)
        z.zoom_down_btn.clicked.connect(self.core.on_zoom_down)
        z.zoom_fit_btn.clicked.connect(self.core.on_fit_in_view)
        w.trash_btn.pressed.connect(self.core.on_trashcan_clicked)
        w.global_menu_btn.clicked.connect(self.core.display_global_menu)
        w.go_first_btn.clicked.connect(self.move_to_first_image)
        w.trailer_sld.valueChanged.connect(self.on_trailer_slider_moved)
        w.go_last_btn.clicked.connect(self.move_to_last_image)

    def move_to_first_image(self):
        self.widget.trailer_sld.setValue(0)

    def move_to_last_image(self):
        val = self.widget.trailer_sld.maximum()
        self.widget.trailer_sld.setValue(val)

    def on_trailer_slider_moved(self, value):
        if self.core.state.context == 5:
            self.core.rawviewer.trailer.move_to_index(value)

        else:
            self.core.trailer.move_to_index(value)

    def on_state_changed(self, state):
        lgg.info('State changed: %s' % state)
        if not state:
            self.set_zoom_value(0)
            self.activate_widgets(False)
            self.activate_zoom_widgets(False)

        elif state == 1:
            self.activate_widgets(True)
            self.activate_zoom_widgets(True)

        elif state == 6:
            count = self.core.viewing.count()
            pos = self.core.viewing.counter
            self.set_zoom_value(0)
            self.activate_widgets(False)
            self.activate_zoom_widgets(False)
            self.activate_trailer_widgets(True)
            if pos:
                self.widget.prev_btn.setEnabled(True)

            if pos < count:
                self.widget.next_btn.setEnabled(True)

    def set_zoom_value(self, value):
        """Set the zoom spinbox value.

        Args:
        value -- the value in percent
        """
        self.widget.zoom_tools.zoom_spb.setValue(value)

    def get_zoom_value(self):
        """Return the zoom spinbox value.

        """
        return self.widget.zoom_tools.zoom_spb.value()

    def set_trailer_slider_max(self, value):
        """Set the maximum value for the trailer slider.

        Args:
        value -- int(max value)
        """
        self.widget.trailer_sld.setMaximum(value)

    def set_trailer_slider_value(self, value):
        """Set the value to the trailer slider.

        Args:
        value -- int(value)
        """
        lgg.info('Set trailer slider value: %s' % value)
        self.widget.trailer_sld.setValue(value)

    def on_multi_clicked(self, state):
        """Slot of the multi views state button.

        """
        #self.set_current_mode(5)
        if state:
            self.core.set_multi_viewing()

        else:
            self.core.close_multiviewing()

    def enable_multiviews_navigation(self, current, count):
        self.widget.prev_btn.setEnabled(1 and current)
        self.widget.next_btn.setEnabled(current < count)

    def activate_widgets(self, state):
        """Enable or disable the tool widgets.

        Args:
        state -- boolean, if True, set enabled the widgets
        """
        lgg.info('Enable tool buttons: %s' % state)
        self.widget.magnifier_btn.setEnabled(state)
        self.widget.right_btn.setEnabled(state)
        self.widget.prev_btn.setEnabled(state)
        self.widget.next_btn.setEnabled(state)
        self.widget.left_btn.setEnabled(state)
        self.widget.trash_btn.setEnabled(self.core.trash_enabled and state)
        self.activate_trailer_widgets(state)

    def activate_navigation_widgets(self, state):
        """Enable or disable the navigation and rotation buttons.

        Args:
        state -- boolean, if True, set enabled the buttons
        """
        lgg.info('Enable navigation buttons: %s' % state)
        self.widget.right_btn.setEnabled(state)
        self.widget.prev_btn.setEnabled(state)
        self.widget.next_btn.setEnabled(state)
        self.widget.left_btn.setEnabled(state)

    def activate_zoom_widgets(self, state):
        """Enable or disable the zoom widgets.

        Args:
        state -- boolean, if True, set enabled the widgets
        """
        lgg.info('Enable zoom widgets: %s' % state)
        self.widget.zoom_tools.zoom_area_btn.setEnabled(state)
        self.widget.zoom_tools.zoom_full_btn.setEnabled(state)
        self.widget.zoom_tools.zoom_up_btn.setEnabled(state)
        self.widget.zoom_tools.zoom_spb.setEnabled(state)
        self.widget.zoom_tools.zoom_down_btn.setEnabled(state)
        self.widget.zoom_tools.zoom_fit_btn.setEnabled(state)

    def activate_trailer_widgets(self, state):
        """Enable or disable the trailer widgets.

        Args:
        state -- boolean, if True, set enabled the widgets
        """
        lgg.info('Enable trailer widgets: %s' % state)
        self.widget.trailer_btn.setEnabled(state)
        self.widget.trailer_sld.setEnabled(state)

    def activate_trash_button(self, number):
        button = self.widget.trash_btn
        button.setEnabled(1 and number)
        if not number:
            button.setStatusTip(_("Move current image to trash can."))

        elif number > 1:
            button.setStatusTip(_("Move {} images selected to trash can."
                                  .format(number)))

        else:
            button.setStatusTip(_("Move 1 image selected to trash can."))


class UiToolBar(QWidget):
    def __init__(self, driver, parent=None):
        super().__init__(parent)
        self.driver = driver
        self.gridLayout = QGridLayout(self)
        self.gridLayout.setContentsMargins(5, 0, 5, 2)
        self.gridLayout.setVerticalSpacing(0)

        self.hl = QHBoxLayout()
        self.hl.setContentsMargins(0, 0, 0, 0)
        self.hl.setSpacing(0)

        self.widget = QWidget(self)
        self.hl_1 = QHBoxLayout(self.widget)
        self.hl_1.setContentsMargins(4, 2, 4, 2)
        self.hl_1.setSpacing(4)       
        btn_size = QSize(30, 25)
        icon_size = QSize(22, 22)
        # Trailer button
        self.trailer_btn = QToolButton(self.widget)
        self.trailer_btn.setMaximumSize(btn_size)
        icon8 = QIcon()
        icon8.addPixmap(QPixmap("medias/trailer2.png"), 
                        QIcon.Normal, QIcon.Off)
        self.trailer_btn.setIcon(icon8)
        self.trailer_btn.setIconSize(QSize(60, 24))
        self.trailer_btn.setAutoRaise(True)
        self.hl_1.addWidget(self.trailer_btn)
        # Multiviewing
        self.multiviews_state_btn = MultiViewsButton(self)
        self.hl_1.addWidget(self.multiviews_state_btn)
        # Magnifying glass
        self.magnifier_btn = QToolButton(self.widget)
        icon10 = QIcon()
        icon10.addPixmap(QPixmap("medias/magnifier_36x30.png"), 
                         QIcon.Normal, QIcon.Off)
        self.magnifier_btn.setIcon(icon10)
        #self.setIconSize(QSize(22, 22))
        self.magnifier_btn.setAutoRaise(True)
        #self.magnifier_btn.setMaximumSize(btn_size)
        self.hl_1.addWidget(self.magnifier_btn)
        # Aerial view, removed in vers. 2.0
        #self.aerial_btn = QToolButton(self.widget)
        #self.aerial_btn.setMaximumSize(btn_size)
        #icon5 = QIcon()
        #icon5.addPixmap(QPixmap("medias/bird_2_48x24.png"), 
        #                            QIcon.Normal, QIcon.Off)
        #self.aerial_btn.setIcon(icon5)
        #self.aerial_btn.setIconSize(icon_size)
        #self.aerial_btn.setAutoRaise(True)
        #self.hl_1.addWidget(self.aerial_btn)
        self.hl.addWidget(self.widget)

        spacer = QSpacerItem(40, 20, QSizePolicy.Maximum, 
                                    QSizePolicy.Minimum)
        self.hl.addItem(spacer)
        self.go_first_btn = QToolButton(self)
        icon18 = QIcon()
        icon18.addPixmap(QPixmap("medias/go_first.png"),
                         QIcon.Normal, QIcon.Off)
        self.go_first_btn.setIcon(icon18)
        self.go_first_btn.setIconSize(QSize(12, 12))
        self.go_first_btn.setAutoRaise(True)
        self.hl.addWidget(self.go_first_btn)
        # Trailer slider
        self.trailer_sld = QSlider(Qt.Horizontal, self)
        self.trailer_sld.setMaximumSize(350, 20)
        self.trailer_sld.setRange(0, 0)
        self.hl.addWidget(self.trailer_sld)
        self.go_last_btn = QToolButton(self)
        icon19 = QIcon()
        icon19.addPixmap(QPixmap("medias/go_last.png"),
                         QIcon.Normal, QIcon.Off)
        self.go_last_btn.setIcon(icon19)
        self.go_last_btn.setIconSize(QSize(12, 12))
        self.go_last_btn.setAutoRaise(True)
        self.hl.addWidget(self.go_last_btn)
        spacer1 = QSpacerItem(60, 20, QSizePolicy.Preferred, 
                                    QSizePolicy.Minimum)
        self.hl.addItem(spacer1)

        # rotate & navigation buttons
        self.widget_1 = QWidget(self)
        self.hl_2 = QHBoxLayout(self.widget_1)
        self.hl_2.setContentsMargins(4, 2, 4, 2)
        self.hl_2.setSpacing(4)
        self.right_btn = QToolButton(self.widget_1)
        self.right_btn.setMaximumSize(btn_size)
        icon11 = QIcon()
        icon11.addPixmap(QPixmap("medias/rotate_right_50x40.png"), 
                         QIcon.Normal, QIcon.Off)
        self.right_btn.setIcon(icon11)
        self.right_btn.setIconSize(icon_size)
        self.right_btn.setAutoRaise(True)
        self.hl_2.addWidget(self.right_btn)
        self.prev_btn = QToolButton(self.widget_1)
        self.prev_btn.setMaximumSize(btn_size)        
        icon12 = QIcon()
        icon12.addPixmap(QPixmap("medias/arrowl_silver_60x40.png"), 
                         QIcon.Normal, QIcon.Off)
        self.prev_btn.setIcon(icon12)
        self.prev_btn.setIconSize(icon_size)
        self.prev_btn.setAutoRaise(True)
        self.prev_btn.setEnabled(False)
        self.hl_2.addWidget(self.prev_btn)
        self.next_btn = QToolButton(self.widget_1)
        self.next_btn.setMaximumSize(btn_size)
        icon13 = QIcon()
        icon13.addPixmap(QPixmap("medias/arrowr_silver_60x40.png"), 
                         QIcon.Normal, QIcon.Off)
        self.next_btn.setIcon(icon13)
        self.next_btn.setIconSize(icon_size)
        self.next_btn.setAutoRaise(True)
        self.next_btn.setEnabled(False)
        self.hl_2.addWidget(self.next_btn)
        self.left_btn = QToolButton(self.widget_1)
        self.left_btn.setMaximumSize(btn_size)
        icon14 = QIcon()
        icon14.addPixmap(QPixmap("medias/rotate_left_50x40.png"), 
                         QIcon.Normal, QIcon.Off)
        self.left_btn.setIcon(icon14)
        self.left_btn.setIconSize(icon_size)
        self.left_btn.setAutoRaise(True)
        self.hl_2.addWidget(self.left_btn)
        self.hl.addWidget(self.widget_1)
        spacer2 = QSpacerItem(40, 20, QSizePolicy.Preferred, 
                                    QSizePolicy.Minimum)
        self.hl.addItem(spacer2)
        # Zoom
        self.zoom_tools = ZoomTools(self)
        self.hl.addWidget(self.zoom_tools)
        # Trash can
        self.trash_btn = QToolButton(self)
        self.trash_btn.setMaximumSize(btn_size)
        icon17 = QIcon()
        icon17.addPixmap(QPixmap("medias/user-trash.png"), 
                         QIcon.Normal, QIcon.Off)
        self.trash_btn.setIcon(icon17)
        self.trash_btn.setIconSize(icon_size)
        self.trash_btn.setAutoRaise(True)
        self.hl.addWidget(self.trash_btn)
        spacer3 = QSpacerItem(40, 20, QSizePolicy.Preferred, 
                                    QSizePolicy.Minimum)
        self.hl.addItem(spacer3)
        # Global menu
        self.global_menu_btn = QToolButton(self)
        self.global_menu_btn.setMaximumSize(btn_size)
        icon17 = QIcon()
        icon17.addPixmap(QPixmap("medias/global_menu.png"), 
                         QIcon.Normal, QIcon.Off)
        self.global_menu_btn.setIcon(icon17)
        self.global_menu_btn.setIconSize(icon_size)
        self.global_menu_btn.setAutoRaise(True)
        self.hl.addWidget(self.global_menu_btn) 
        self.gridLayout.addLayout(self.hl, 1, 0, 1, 9)

        self.retranslateUi()
        self.set_style()

    def set_style(self):
        self.widget.setStyleSheet(styles.CONTAINER)
        self.widget_1.setStyleSheet(styles.CONTAINER)
        self.zoom_tools.setStyleSheet(styles.CONTAINER)
        self.trailer_btn.setStyleSheet(styles.TOOLBUTTON)
        self.magnifier_btn.setStyleSheet(styles.BUTTONPOPUP)
        self.go_first_btn.setStyleSheet(styles.TBUTTON)
        self.trailer_sld.setStyleSheet(styles.SLIDER)
        self.go_last_btn.setStyleSheet(styles.TBUTTON)
        self.right_btn.setStyleSheet(styles.TOOLBUTTON)
        self.prev_btn.setStyleSheet(styles.TOOLBUTTON)
        self.next_btn.setStyleSheet(styles.TOOLBUTTON)
        self.left_btn.setStyleSheet(styles.TOOLBUTTON)
        self.trash_btn.setStyleSheet(styles.TOOLBUTTON)

    def retranslateUi(self):
        #self.aerial_btn.setStatusTip(_("Aerial view."))
        self.trailer_btn.setStatusTip(_("Show-hide trailer."))
        self.multiviews_state_btn.setStatusTip(_("Mode multi views."))
        self.magnifier_btn.setStatusTip(_("Magnifying glass tool."))
        self.go_first_btn.setStatusTip(_("Go to the first image."))
        self.trailer_sld.setStatusTip(_("Move the trailer."))
        self.go_last_btn.setStatusTip(_("Go to the last image."))
        self.right_btn.setStatusTip(_("Rotate to the right."))
        self.prev_btn.setStatusTip(_("Previous image."))
        self.next_btn.setStatusTip(_("Next image."))
        self.left_btn.setStatusTip(_("Rotate to the left."))
        self.trash_btn.setStatusTip(_("Move current image to trash can."))
        self.global_menu_btn.setStatusTip(_("Open the menu."))


class MultiViewsButton(QWidget):
    activated = pyqtSignal(bool)
    def __init__(self, parent=None):
        super().__init__(parent)
        self.driver = parent.driver
        self.is_activated = False
        self.layout = QHBoxLayout(self)
        self.layout.setContentsMargins(0, 0, 0, 0)
        self.state_lbl = QLabel(self)
        icon = QPixmap("medias/apply.png")
        icon1 = QPixmap("medias/exit.png")
        self.icons = (icon, icon1)
        self.state_lbl.setPixmap(icon)
        self.layout.addWidget(self.state_lbl)
        self.icon1 = QIcon()
        self.icon1.addPixmap(QPixmap("medias/multiviews.png"), 
                                    QIcon.Normal, QIcon.Off)
        self.multiviews_btn = QToolButton(self)
        self.multiviews_btn.setMaximumSize(35, 20)
        self.multiviews_btn.setAutoRaise(True)
        self.multiviews_btn.setIcon(self.icon1)
        self.multiviews_btn.setIconSize(QSize(35, 18))
        self.layout.addWidget(self.multiviews_btn)
        spacer = QSpacerItem(40, 12, QSizePolicy.Fixed, QSizePolicy.Minimum)
        self.layout.addItem(spacer)
        self.multiviews_btn.clicked.connect(self.on_button_clicked)
        self.multiviews_btn.setStyleSheet(styles.STATUSBUTTON)

    def on_button_clicked(self):
        self.update_state()
        self.activated.emit(self.is_activated)

    def update_state(self):
        new = not self.is_activated
        self.state_lbl.setPixmap(self.icons[new])
        self.is_activated = new


class ZoomTools(QWidget):
    def __init__(self, parent=None):
        super().__init__(parent)
        icon_size = QSize(22, 22)
        self.verticalLayout_20 = QVBoxLayout(self)
        self.verticalLayout_20.setSpacing(4)
        self.verticalLayout_20.setContentsMargins(4, 2, 4, 2)
        self.gridLayout_10 = QGridLayout()
        self.zoom_area_btn = QToolButton(self)
        self.zoom_area_btn.setMaximumSize(QSize(80, 20))
        icon15 = QIcon()
        icon15.addPixmap(QPixmap("medias/crop_white_24.png"), 
                         QIcon.Normal, QIcon.Off)
        self.zoom_area_btn.setIcon(icon15)
        self.zoom_area_btn.setIconSize(icon_size)
        self.gridLayout_10.addWidget(self.zoom_area_btn, 0, 0, 1, 1)
        self.zoom_full_btn = QToolButton(self)
        self.zoom_full_btn.setMaximumSize(QSize(80, 20))
        self.zoom_full_btn.setText("< >")
        self.gridLayout_10.addWidget(self.zoom_full_btn, 0, 1, 1, 1)
        self.zoom_up_btn = QToolButton(self)
        self.zoom_up_btn.setMinimumSize(QSize(25, 0))
        self.zoom_up_btn.setMaximumSize(QSize(25, 20))
        self.zoom_up_btn.setText("+")
        self.gridLayout_10.addWidget(self.zoom_up_btn, 0, 2, 1, 1)
        self.zoom_spb = QDoubleSpinBox(self)
        self.zoom_spb.setMinimumSize(QSize(100, 0))
        self.zoom_spb.setMaximumSize(QSize(1000, 20))
        self.zoom_spb.setWrapping(False)
        self.zoom_spb.setFrame(True)
        self.zoom_spb.setMinimum(0.05)
        self.zoom_spb.setMaximum(9900.0)
        self.zoom_spb.setSuffix(" %")
        self.zoom_spb.setProperty("value", 100.0)
        self.gridLayout_10.addWidget(self.zoom_spb, 0, 3, 1, 1)
        self.zoom_down_btn = QToolButton(self)
        self.zoom_down_btn.setMinimumSize(QSize(25, 0))
        self.zoom_down_btn.setMaximumSize(QSize(25, 20))
        self.zoom_down_btn.setText("-")
        self.gridLayout_10.addWidget(self.zoom_down_btn, 0, 4, 1, 1)
        self.zoom_fit_btn = QToolButton(self)
        self.zoom_fit_btn.setMaximumSize(QSize(80, 20))
        self.zoom_fit_btn.setText("> <")
        self.gridLayout_10.addWidget(self.zoom_fit_btn, 0, 5, 1, 1)
        self.verticalLayout_20.addLayout(self.gridLayout_10)
        self.zoom_area_btn.setStyleSheet(styles.TOOLBUTTON)
        self.zoom_full_btn.setStyleSheet(styles.TOOLBUTTON)
        self.zoom_up_btn.setStyleSheet(styles.TOOLBUTTON)
        self.zoom_spb.setStyleSheet(styles.DSPINBOX)
        self.zoom_down_btn.setStyleSheet(styles.TOOLBUTTON)
        self.zoom_fit_btn.setStyleSheet(styles.TOOLBUTTON)
        self.zoom_area_btn.setStatusTip(_("Zoom area."))
        self.zoom_full_btn.setStatusTip(_("Full size."))
        self.zoom_up_btn.setStatusTip(_("Zoom up one step."))
        self.zoom_spb.setStatusTip(_("Current zoom value."))
        self.zoom_down_btn.setStatusTip(_("Zoom down one step."))
        self.zoom_fit_btn.setStatusTip(_("Fit in view."))
