# -*- coding: utf-8 -*-

# toolpanel.py
# This file is part of Oqapy serie 3
#
# Author: Vincent Vande Vyvre <vincent.vandevyvre@oqapy.eu>
# Home: http://www.oqapy.eu
# Copyright: 2016-2017 Vincent Vande Vyvre
# Licence: GPL3
#
# Tool boxes's panel

import logging
lgg = logging.getLogger(__name__)

from PyQt5.QtCore import QSize
from PyQt5.QtWidgets import QSizePolicy

from gui.imageinfos import ImageInfos
from gui.metadatabox import MetadataBox
from gui.g9nbox import G9nBox
from gui.navigationbox import NavigationBox
from gui.histogrambox import HistogramBox
from gui.contrastbrightnessbox import ContrastBrightnessBox
from gui.contraststretchingbox import ContrastStretchingBox
from gui.denoisingbox import DenoisingBox
from gui.historicbox import HistoricBox
from gui.resizingbox import ResizingBox
from gui.cropbox import CropBox
from gui.straightenbox import StraightenBox
from gui.colortogreybox import ColorToGreyBox
from gui.toningbox import ToningBox
from gui.laplacebox import LaplaceBox
from gui.exportbox import ExportBox
from gui.sourcefolderbox import SourceFolderBox
from gui.recentsbox import RecentsBox
from gui.trashbox import TrashBox
from gui.targetfoldersbox import TargetFoldersBox
from gui.rawinquiriesbox import RawInquiriesBox
from gui.unrawbox import UnrawBox

from VWidgets_PyQt5.vtooldockpanel import VToolDockPanel

import stylesheets as styles


class ToolPanel(VToolDockPanel):
    def __init__(self, ui):
        super().__init__()
        self.setMinimumWidth(310)
        self.ui = ui
        self.core = ui.core
        self.setTitle('')
        self.set_tool_boxes()

    def set_tool_boxes(self):
        """Sets the tool boxes into the tool panel.

        """
        # How to add a tool box:
        #    - Create, if needed, a new tool page with:
        #            page = VToolPanel.addToolPage(name, icon)
        #    - Create a new box to this page with:
        #            myNewbox = VToolPanel.addToolBox(page)
        #    - Set the properties of the box header (title, icon, collapsible, ...)
        #    - Instanciate the tool widget with the box as parent:
        #            myWidget = MyWidget(myNewbox)
        #    - Place the tool widget into the box layout with:
        #            myNewBox.layout.addWidget(myWidget)

        self.page_icons = {0: ['medias/info_th.png',
                               'medias/info_th_hover.png',
                               'medias/info_th_active.png'],
                          1: ['medias/cisor_th.png',
                              'medias/cisor_th_hover.png',
                              'medias/cisor_th_active.png'],
                          2: ['medias/histogram_th.png',
                              'medias/histogram_th_hover.png',
                              'medias/histogram_th_active.png'],
                          3: ['medias/art_th.png',
                              'medias/art_th_hover.png',
                              'medias/art_th_active.png'],
                          4: ['medias/folder_th.png',
                              'medias/folder_th_hover.png',
                              'medias/folder_th_active.png'],
                          5: ['medias/raw_th.png',
                              'medias/raw_th.png',
                              'medias/raw_th_active.png']}
        self.tools = []
        self.page_0 = self.addToolPage('page_0', self.page_icons[0][2])
        self.page_0.button.setIconSize(QSize(18, 18))
        self.page_0.button.setStatusTip(_('Image informations page'))
        # Image infos
        self.info_bx = self.addToolBox(0)
        self.info_bx.setTitle('Picture')
        self.image_infos = ImageInfos(self.info_bx)
        self.info_bx.layout.addWidget(self.image_infos)
        self.tools.append(self.image_infos)

        self.splitter = self.add_splitter(0)
        self.splitter.setChildrenCollapsible(True)
        # Metadata
        self.data_bx = self.addToolBox(0, None, self.splitter)
        self.data_bx.setTitle(_('Metadata'))
        self.data_bx.setCollapsible(False)
        header = self.data_bx.header
        header.setMaximumSize(9999, 22)
        self.clone_infos_btn = header.add_button('medias/list_18x16.png',
                                                    'clone_infos_btn')
        self.clone_infos_btn.setStatusTip(_("Copy as model"))
        self.paste_infos_btn = header.add_button('medias/stamp_18x16.png',
                                                    'paste_infos_btn')
        self.paste_infos_btn.setStatusTip(_("Paste from model"))
        self.data_view = MetadataBox(self.data_bx)
        self.data_bx.layout.addWidget(self.data_view)
        self.tools.append(self.data_view)
        #self.clone_infos_btn.setStyleSheet(theme.PUSHBUTTON)
        #self.paste_infos_btn.setStyleSheet(theme.PUSHBUTTON)
        # Geolocation
        self.g9n_bx = self.addToolBox(0, splitter=self.splitter)
        self.g9n_bx.setTitle(_('Geolocation'))
        self.g9n_bx.setCollapsible(False)
        header = self.g9n_bx.header
        header.setMaximumSize(9999, 22)
        #self.openstreetmap_btn = header.add_button('medias/osm_th.png', 
                                                    #'openstreetmap_btn')
        #self.openstreetmap_btn.setIconSize(QSize(16, 16))
        #self.googlemap_btn = header.add_button('medias/google_th.png',
                                                    #'googlemap_btn')
        #self.googlemap_btn.setIconSize(QSize(16, 16))
        self.g9n_view = G9nBox(self, self.g9n_bx)
        self.g9n_bx.layout.addWidget(self.g9n_view)
        #self.openstreetmap_btn.setStyleSheet(theme.PUSHBUTTON)
        #self.googlemap_btn.setStyleSheet(theme.PUSHBUTTON)
        self.tools.append(self.g9n_view)

        # Tool page 1: Geometry transformations
        self.page_1 = self.addToolPage('page_1', self.page_icons[1][0])
        self.page_1.button.setIconSize(QSize(18, 18))
        self.page_1.button.setStatusTip(_('Physical transformation tools'))
        # Image navigation, zoom
        self.navigation_bx = self.addToolBox(1)
        self.navigation_bx.setTitle(_('Image preview'))
        header = self.navigation_bx.header
        header.setMaximumSize(9999, 22)
        self.floating_view_btn = header.add_button('medias/float_th.png',
                                                    'floating_view_btn')
        sizePolicy = QSizePolicy(QSizePolicy.Preferred, 
                                        QSizePolicy.Fixed)
        self.navigation = NavigationBox(self.core, self.navigation_bx)
        self.navigation.setSizePolicy(sizePolicy)
        self.navigation_bx.layout.addWidget(self.navigation)
        #self.floating_view_btn.setStyleSheet(theme.PUSHBUTTON)
        self.tools.append(self.navigation)
        # Resizing
        self.resize_bx = self.addToolBox(1)
        self.resize_bx.setTitle(_('Resizing'))
        header = self.resize_bx.header
        header.setMaximumSize(9999, 22)
        self.resizing = ResizingBox(self, self.resize_bx)
        self.resize_bx.layout.addWidget(self.resizing)
        self.tools.append(self.resizing)
        # Crop
        self.crop_bx = self.addToolBox(1)
        self.crop_bx.setTitle(_('Crop'))
        header = self.crop_bx.header
        header.setMaximumSize(9999, 22)
        self.crop = CropBox(self, self.crop_bx)
        self.crop_bx.layout.addWidget(self.crop)
        self.tools.append(self.crop)
        # Rotation - straighten
        self.straighten_bx = self.addToolBox(1)
        self.straighten_bx.setTitle(_('Rotation - Straighten'))
        header = self.straighten_bx.header
        header.setMaximumSize(9999, 22)
        self.straighten = StraightenBox(self, self.straighten_bx)
        self.straighten_bx.layout.addWidget(self.straighten)
        self.tools.append(self.straighten)
        # Historic of transformations
        self.historic_bx = self.addToolBox(1)
        self.historic_bx.setTitle(_('Historic'))
        header = self.historic_bx.header
        header.setMaximumSize(9999, 22)
        self.historic = HistoricBox(self, self.historic_bx)
        self.historic_bx.layout.addWidget(self.historic)
        sizePolicy = QSizePolicy(QSizePolicy.Preferred, 
                                        QSizePolicy.Expanding)
        self.historic.setSizePolicy(sizePolicy)
        self.tools.append(self.historic)

        # Tool page 2: Histogram, color enhancement
        self.page_2 = self.addToolPage('page_2', self.page_icons[2][0])
        self.page_2.button.setIconSize(QSize(18, 18))
        self.page_2.button.setStatusTip(_('Color transformation tools'))
        # Navigation (clone)
        self.clone_tool_box(self.navigation_bx, 2)
        # Histogram
        self.histogram_bx = self.addToolBox(2)
        self.histogram_bx.setTitle(_('Histogram'))
        header = self.histogram_bx.header
        header.setMaximumSize(9999, 22)
        self.histogram = HistogramBox(self.histogram_bx)
        self.histogram.setSizePolicy(sizePolicy)
        self.histogram_bx.layout.addWidget(self.histogram)
        self.tools.append(self.histogram)
        # Contrast brightness
        self.contrast_bx = self.addToolBox(2)
        self.contrast_bx.setTitle(_('Contrast - Brightness'))
        header = self.contrast_bx.header
        header.setMaximumSize(9999, 22)
        self.contrast = ContrastBrightnessBox(self.core, self.contrast_bx)
        self.contrast_bx.layout.addWidget(self.contrast)
        self.tools.append(self.contrast)
        # Color balance
        self.stretching_bx = self.addToolBox(2)
        self.stretching_bx.setTitle(_('Contrast stretching'))
        header = self.stretching_bx.header
        header.setMaximumSize(9999, 22)
        self.stretching = ContrastStretchingBox(self.core, self.stretching_bx)
        self.stretching_bx.layout.addWidget(self.stretching)
        self.tools.append(self.stretching)
        # Denoising
        self.denoising_bx = self.addToolBox(2)
        self.denoising_bx.setTitle(_('Denoising'))
        header = self.denoising_bx.header
        header.setMaximumSize(9999, 22)
        self.denoising = DenoisingBox(self.core, self.denoising_bx)
        self.denoising_bx.layout.addWidget(self.denoising)
        self.tools.append(self.denoising)
        # Historic (clone)
        self.clone_tool_box(self.historic_bx, 2)

        # Tool page 3: Artistic transformations
        self.page_3 = self.addToolPage('page_3', self.page_icons[3][0])
        self.page_3.button.setIconSize(QSize(18, 18))
        self.page_3.button.setStatusTip(_('Artistic transformation tools'))
        # Navigation (clone)
        self.clone_tool_box(self.navigation_bx, 3)
        # Convert to grey map
        self.grey_map_bx = self.addToolBox(3)
        self.grey_map_bx.setTitle(_('Convert to grey map'))
        header = self.grey_map_bx.header
        header.setMaximumSize(9999, 22)
        self.grey_map = ColorToGreyBox(self, self.grey_map_bx)
        self.grey_map_bx.layout.addWidget(self.grey_map)
        self.tools.append(self.grey_map)
        # Toning
        self.toning_bx = self.addToolBox(3)
        self.toning_bx.setTitle(_('Toning'))
        header = self.toning_bx.header
        header.setMaximumSize(9999, 22)
        self.toning = ToningBox(self, self.toning_bx)
        self.toning_bx.layout.addWidget(self.toning)
        self.tools.append(self.toning)
        # Laplacian
        self.laplace_bx = self.addToolBox(3)
        self.laplace_bx.setTitle(_('Laplacian filter'))
        header = self.laplace_bx.header
        header.setMaximumSize(9999, 22)
        self.laplace = LaplaceBox(self, self.laplace_bx)
        self.laplace_bx.layout.addWidget(self.laplace)
        self.tools.append(self.laplace)
        # Historic (clone)
        self.clone_tool_box(self.historic_bx, 3)

        # Tool page 4: Export
        self.page_4 = self.addToolPage('page_4', self.page_icons[4][0])
        self.page_4.button.setIconSize(QSize(18, 18))
        self.page_4.button.setStatusTip(_('Folder page'))
        # Export
        self.export_bx = self.addToolBox(4)
        self.export_bx.setTitle(_('Export'))
        header = self.export_bx.header
        header.setMaximumSize(9999, 22)
        self.export = ExportBox(self, self.export_bx)
        self.export_bx.layout.addWidget(self.export)
        self.tools.append(self.export)
        # Source folder
        self.source_bx = self.addToolBox(4)
        self.source_bx.setTitle(_('Source folder'))
        header = self.source_bx.header
        header.setMaximumSize(9999, 22)
        self.src_tool = SourceFolderBox(self, self.source_bx)
        self.source_bx.layout.addWidget(self.src_tool)
        self.tools.append(self.src_tool)
        # Recent folders
        self.recents_bx = self.addToolBox(4)
        self.recents_bx.setTitle(_('Recent folders'))
        header = self.recents_bx.header
        header.setMaximumSize(9999, 22)
        self.recents = RecentsBox(self, self.recents_bx)
        self.recents_bx.layout.addWidget(self.recents)
        self.tools.append(self.recents)
        # Trash can folder
        self.trashcan_bx = self.addToolBox(4)
        self.trashcan_bx.setTitle(_('Trash can'))
        header = self.trashcan_bx.header
        header.setMaximumSize(9999, 22)
        self.trash_tool = TrashBox(self, self.trashcan_bx)
        self.trashcan_bx.layout.addWidget(self.trash_tool)
        self.tools.append(self.trash_tool)
        # Target folders
        self.targets_bx = self.addToolBox(4)
        self.targets_bx.setTitle(_('Target folders'))
        header = self.targets_bx.header
        header.setMaximumSize(9999, 22)
        self.targets_tool = TargetFoldersBox(self, self.targets_bx)
        self.targets_bx.layout.addWidget(self.targets_tool)
        self.tools.append(self.targets_tool)

        # Tool page 5: RAW files handling
        self.page_5 = self.addToolPage('page_5', self.page_icons[5][0])
        self.page_5.button.setIconSize(QSize(18, 18))
        self.page_5.button.setStatusTip(_('Raw files page'))
        # Inquiries
        self.raw_inq_bx = self.addToolBox(5)
        self.raw_inq_bx.setTitle(_('File infos'))
        header = self.raw_inq_bx.header
        header.setMaximumSize(9999, 22)
        self.raw_inquiries = RawInquiriesBox(self, self.raw_inq_bx)
        self.raw_inq_bx.layout.addWidget(self.raw_inquiries)
        self.tools.append(self.raw_inquiries)
        # Histogram
        self.raw_histogram_bx = self.addToolBox(5)
        self.raw_histogram_bx.setTitle(_('Histogram'))
        header = self.raw_histogram_bx.header
        header.setMaximumSize(9999, 22)
        self.raw_histogram = HistogramBox(self.raw_histogram_bx)
        self.raw_histogram.setSizePolicy(sizePolicy)
        self.raw_histogram_bx.layout.addWidget(self.raw_histogram)
        self.tools.append(self.raw_histogram)
        # Unraw
        self.unraw_bx = self.addToolBox(5)
        self.unraw_bx.setTitle(_('Parameters'))
        header = self.unraw_bx.header
        header.setMaximumSize(9999, 22)
        self.raw_param = UnrawBox(self, self.unraw_bx)
        self.unraw_bx.layout.addWidget(self.raw_param)
        self.tools.append(self.raw_param)
        self.clone_raw_settings_btn = header.add_button('medias/list_18x16.png',
                                                    'clone_settings_btn')
        self.clone_raw_settings_btn.setStatusTip("Copy the settings")
        self.paste_raw_settings_btn = header.add_button('medias/stamp_18x16.png',
                                                    'paste_settings_btn')
        self.paste_raw_settings_btn.setStatusTip("Paste the settings")

        self.setCurrentPage(0)
        self.currentToolPageChanged.connect(self.on_tool_page_changed)
        self.set_raw_page_enabled(False)

    def configure_toolboxes(self):
        cfg = self.core.cfg
        self.src_tool.set_config(cfg.get(('startup', )))
        self.crop.configure(cfg.get(('crop', )))
        self.trash_tool.configure()

    def configure_signals(self):
        self.floating_view_btn.clicked.connect(self.core.display_aerial_view)
        self.clone_raw_settings_btn.clicked.connect(
                                        self.core.rawviewer.copy_settings)
        self.paste_raw_settings_btn.clicked.connect(
                                        self.core.rawviewer.set_old_settings)

    def set_empty_state(self):
        self.reset_all_toolboxes()
        self.enable_all_toolboxes(False)
        self.navigation.clear()

    def set_multiviewing_state(self, raw=False):
        """Disable the tool boxes which are not usable in multiviewing mode.

        args:
        raw -- boolean, if True the multiview show the raw files
        """
        self.enable_all_toolboxes(False)
        if not raw:
            self.image_infos.setEnabled(True)
            self.data_view.setEnabled(True)
            self.g9n_view.setEnabled(True)

        else:
            self.raw_inquiries.setEnabled(True)
            self.raw_histogram.setEnabled(True)
            self.raw_param.setEnabled(True)

    def reset_all_toolboxes(self):
        for tool in self.tools:
            tool.reset()

        self.info_bx.setTitle('Picture')

    def enable_all_toolboxes(self, enable):
        for tool in self.tools:
            tool.setEnabled(enable)

        for tool in [self.src_tool, self.recents, self.trash_tool, 
                     self.targets_tool]:
            tool.setEnabled(True)

        if self.core.state.context != 5:
            self.set_raw_page_enabled(False)

    def update_with_picture(self, picture):
        """Update the tool box when a new image is show in the viewer.

        Args:
        picture -- the Picture instance
        """
        self.image_infos.set_image_data(picture)
        self.resizing.setup(picture)
        self.g9n_view.webView.set_coordinate_from_picture(picture)
        self.export.set_data(picture)

    def browse_to_source_folder(self):
        pass

    def clean_recents_list(self):
        pass

    def on_tool_page_changed(self, old, new):
        self.header.change_button_icon(self.pages[old].button, 
                                       self.page_icons[old][0])
        self.header.change_button_icon(self.pages[new].button, 
                                       self.page_icons[new][2])
        #self.fix_toolheader_icon_size()

    def set_raw_page_enabled(self, value):
        lgg.info('RAW tools enabled: %s' % value)
        self.raw_inquiries.setEnabled(value)
        self.raw_histogram.setEnabled(value)
        self.raw_param.setEnabled(value)
        if value:
            # If there's only raw images, theses toolboxes are disabled
            self.image_infos.setEnabled(True)
            self.data_view.setEnabled(True)
            self.g9n_view.setEnabled(True)

    def resizeEvent(self, event):
        if not self.core.on_startup:
            w, h = event.size().width(), event.size().height()
            self.core.cfg.update(('toolpanel', 'size'), (w, h))

    def set_style(self):
        self.setStyleSheet(styles.WIDGET)
        self.header.setStyleSheet(styles.HEADER.replace('color: #FFFFFF;}', 
                                        'padding: 2px;color: #FFFFFF;}'))
        self.scrollArea.setStyleSheet(styles.SCROLLAREA)
        self.scrollArea.horizontalScrollBar().setStyleSheet(
                                                    styles.HSCROLLBAR)
        self.scrollArea.verticalScrollBar().setStyleSheet(
                                                    styles.VSCROLLBAR)
        boxes = self.get_tool_boxes()
        for tb in boxes:
            if tb == self.splitter:
                boxes.extend(tb.boxes)
                continue
            tb.header.setStyleSheet(styles.HEADER)
            tb.header.label.setStyleSheet(styles.TITLE)
            try:
                tb.header.collapse_btn.setStyleSheet(styles.EXPAND)
                tb.header.set_expanding_buttons(
                                        'medias/collapse_white_down.png',
                                        'medias/collapse_white_left.png',
                                        'medias/collapse_white_right.png')
            except:
                # Not all tool boxes have a collapse button
                pass
            tb.header.layout.itemAt(0).widget().setStyleSheet(styles.TITLE)


