# trailer.py
# This file is part of Oqapy serie 3
#
# Author: Vincent Vande Vyvre <vincent.vandevyvre@oqapy.eu>
# Home: http://www.oqapy.eu
# Copyright: 2009-2016 Vincent Vande Vyvre
# Licence: GPL3
#
# Trailer

import os
import time
import logging
lgg = logging.getLogger(__name__)

from PyQt5.QtCore import Qt, QPointF, pyqtSignal
from PyQt5.QtGui import QImage, QColor, QPen, QPainter
from PyQt5.QtWidgets import (QGraphicsView, QGraphicsScene, QGraphicsPixmapItem,
                             QMenu, QAction)


class Trailer(QGraphicsView):
    trailerReady = pyqtSignal()
    thumbnailRemoved = pyqtSignal(int)
    thumbnailInserted = pyqtSignal(int)
    selectionChanged = pyqtSignal(int, int)
    itemDoubleClicked = pyqtSignal(int)
    trailerMoved = pyqtSignal(int)

    def __init__(self, ui, parent=None):
        super().__init__(parent)
        self.ui = ui
        self.core = ui.core
        self.scene_length = 40.0
        self.items = []
        self.flow = "horizontal"
        self.current = 0
        self.centered = 0
        self.style = 3
        self.heigth_ = 100
        self.background = "#242424"
        self.is_active = False
        self.setMinimumSize(100, 100)
        self.setMaximumSize(16777215, 100)
        self.setVerticalScrollBarPolicy(Qt.ScrollBarAlwaysOff)
        self.setHorizontalScrollBarPolicy(Qt.ScrollBarAlwaysOff)
        self.set_styleSheet(self.background)
        self.scene = QGraphicsScene()
        self.setScene(self.scene)
        self.set_scene_size()

    def clear_all(self):
        self.scene.clear()
        self.scene_length = 40.0
        self.images = []
        self.items = []
        self.current = 0
        self.centered = 0

    def config(self, cfg):
        self.height_ = cfg['size'] + 8
        self.style = cfg['style']
        if self.style in (1, 2):
            self.height_ += 16

        elif self.style == 3:
            self.height_ += 32
        
        self.setMinimumSize(100, self.height_)
        self.setMaximumSize(16777215, self.height_)
        self.set_scene_size()
        self.ui.stackedtrailers.setMaximumSize(16777215, self.height_)

    def set_scene_size(self):
        self.scene.setSceneRect(0.0, 0.0, self.width(), self.height())

    def set_styleSheet(self, color):
        self.background = color
        t =  "".join(["QWidget { background-color: " , color, "}"])
        self.setStyleSheet(t) 

    def add_item(self, item):
        """Append a new item to the trailer.

        Args:
        item -- instance of TrailerItem
        """
        self.items.append(item)
        w, h = item.get_size()
        pos = (self.scene_length, 0.0)
        self.scene_length += w + 4.0
        self.scene.setSceneRect(0.0, 0.0, 
                        self.scene_length + self.width(), self.height())
        self.scene.addItem(item)
        item.setPos(pos[0], pos[1])
        self.thumbnailInserted.emit(len(self.items)-1)

    def insert_item(self, item, index):
        """Insert a new thumbnail into the trailer.

        Args:
        item -- instance of TrailerItem
        index -- index to insert the new item
        """
        lgg.info('Insert item at index %s' % index)
        self.unselect_item()
        if index is None:
            index = self.current + 1

        if not index:
            pos = QPointF(40.0, 0.0)

        elif index >= len(self.items):
            index = len(self.items)
            pos = QPointF(self.items[-1].pos().x() + 
                                    self.items[-1].get_size()[0] + 4,
                                    self.items[-1].pos().y())

        else:
            pos = self.items[index].pos()

        w, h = item.get_size()
        self.scene_length += w + 4.0
        self.scene.setSceneRect(-self.width() / 2, 0.0, 
                        self.scene_length + self.width(), self.height())

        for i in self.items[index:]:
            i.moveBy(w + 4.0, 0.0)

        self.scene.addItem(item)
        item.setPos(pos)
        self.items.insert(index, item)
        self.thumbnailInserted.emit(index)
        if index >= self.current:
            self.set_current(self.current)

        else:
            self.set_current(self.current+1)

    def set_current(self, idx):
        """Change the current image.

        Args:
        idx -- index of image
        """
        lgg.info('Change current item %s to %s' %(self.current, idx))
        self.unselect_item()
        if idx >= len(self.items):
            idx = len(self.items) - 1

        if self.items:
            try:
                self.items[self.current].setOpacity(0.9)
                self.items[self.current].setSelected(False)
            except IndexError:
                self.current = len(self.items)

            self.current = idx
            self.items[idx].set_frame()
            self.centerOn(self.items[idx])
            self.items[idx].setOpacity(1.0)
            self.selectionChanged.emit(self.current, idx)
            self.current = self.centered = idx
            self.fix_slider_pos()

    def set_centered(self, idx):
        """Move a thumbnail to the center of the trailer.

        Args:
        idx -- index of image
        """
        if not self.items:
            return

        if self.current != self.centered:
            self.items[self.centered].setOpacity(0.9)

        self.centerOn(self.items[idx])
        self.items[idx].setOpacity(1.0)
        self.centered = idx
        self.trailerMoved.emit(idx)

    def unselect_item(self, idx=None):
        lgg.info('Unselect item %s' % idx)
        if idx is None:
            idx = self.current

        try:
            # At startup the list may be empty
            #frame = self.items[idx].frame
            #frame.setParentItem(None)
            #self.scene.removeItem(frame)
            for child in self.items[idx].childItems():
                child.setParentItem(None)
                self.scene.removeItem(child)
        except Exception as why:
            lgg.info("Can't unselect: %s" % why)

    def scroll_to(self, val):
        idx = self.centered + val
        if not idx in range(len(self.items)):
            idx = abs(abs(idx) - len(self.items))

        self.set_centered(idx)

    def scroll_to_item(self, item):
        if abs(item) < len(self.items):
            self.set_centered(item)

    def scroll_to_left(self):
        idx = self.centered - 1
        if not self.centered:
            idx = len(self.items) - 1

        self.set_centered(idx)

    def scroll_to_right(self):
        idx = self.centered + 1
        if self.centered == len(self.items) - 1:
            idx = 0

        self.set_centered(idx)

    def go_first(self):
        self.set_centered(0)
        self.fix_slider_pos()

    def go_last(self):
        self.set_centered(len(self.items) - 1)
        self.fix_slider_pos()

    def remove_current(self):
        """Remove current thumbnail from the trailer.

        """
        lgg.info("Remove the current thumbnail")
        self.remove_at(self.current)
        return self.current

    def remove_at(self, idx):
        """Remove indexed thumbnail from the trailer.

        Args:
        idx -- index of thumbnail
        """
        if not self.items:
            # At startup, an image may be rejected before the trailer building
            return
        lgg.info("Remove the thumbnail %s, current: %s" % (idx, self.current))
        self.unselect_item()
        w = self.items[idx].get_size()[0] + 4.0
        self.items[idx].setParentItem(None)
        self.scene.removeItem(self.items[idx])
        item = self.items.pop(idx)
        self.thumbnailRemoved.emit(idx)
        del item
        if not self.items:
            return

        for i in self.items[idx:]:
            i.moveBy(-w, 0.0)

        if self.current >= len(self.items):
            self.current = 0

        self.set_current(self.current)

    def rebuild_item(self, idx):
        """Rebuild a thumbnail.

        This called by the context menu of an item.

        Args:
        idx -- Index of the item
        """
        img = self.core.folders[0].files[idx]
        fld = self.core.folders[0].thumbs_fld
        th = self.utils.source_thumb_name(fld, img)
        try:
            os.remove(th)
        except Exception as why:
            lgg.info("Can't delete thumbnail {0}".format(th))

        self.remove_at(idx)
        self.insert_thumbnail(img, th, idx)
        
    def wheelEvent(self, event):
        """Handle the mouse wheel event.

        """
        pos = self.items[-1].pos()
        vpos = self.mapFromScene(pos)
        step = event.angleDelta().y() / 2
        sv = self.horizontalScrollBar().value()
        if step > 0:
            if vpos.x() < self.width() - 200:
                return

        self.horizontalScrollBar().setValue(sv + step)
        self.fix_slider_pos()

    def set_trailer_button(self):
        """The trailer is complete, we can enable the previous button.

        """
        # FIXME unusued ?
        # Signal connected to ui.on_trailer_complete()
        lgg.info('Trailer completed at: %s sec.' % (time.time() - self.begin))
        self.trailerReady.emit()
        tt_build = ToolTipBuilder()
        tt_build.items = self.items
        tt_build.start()

    def fix_slider_pos(self):
        """Update the slider position.

        """
        w, h = self.get_viewer_center()
        item = self.get_item_at(w, h)
        if item is not None:
            try:
                idx = self.items.index(item)
            except ValueError:
                # Not the item but his frame
                idx = -1
            self.trailerMoved.emit(idx)

    def get_viewer_center(self):
        return self.width() / 2, self.height() / 2

    def get_item_at(self, posx, posy):
        """Return the instance of the item at the given position.

        Args:
        posx -- coordinate x
        posy -- coordinate y

        Returns:
        instance of PixmapItem
        """
        return self.itemAt(int(posx), int(posy))

    def hide_(self):
        self.ui.show_trailer()

