# -*- coding: utf-8 -*-

# helpviewer.py
# This file is part of Oqapy serie 3
#
# Author: Vincent Vande Vyvre <vincent.vandevyvre@oqapy.eu>
# Home: http://www.oqapy.eu
# Copyright: 2014-2018 Vincent Vande Vyvre
# Licence: GPL3
#
# User guide viewer

# Config
# "manual": {"size": tuple   Dialog's size,
#            "opened_tabs": list    List of unclosed tabs (page names)}


import sys
import os
import glob
import json
import tarfile
import urllib.request
import shutil
import webbrowser
from threading import Thread
import logging
lgg = logging.getLogger(__name__)

from PyQt5 import QtCore, QtWidgets

from PyQt5.QtWebKit import QWebSettings
from PyQt5.QtWebKitWidgets import QWebView, QWebPage, QWebFrame

from helpviewer.ui_helpviewer import Ui_HelpViewer
from helpviewer.data import LANGUAGES


class HelpViewer:
    def __init__(self, core):
        self.core = core
        self.user_fld = core.workspace['root']
        self.ui = Ui_HelpViewer(self, self.core.cfg.get(("manual", "size")))
        self.ui.treeWidget.itemClicked.connect(self.on_item_tree_selected)
        self.tabs = [self.ui.tabWidget.widget(0)]
        self.current_pages = [None]
        self.installed = False
        self.setup_doc()

    def setup_doc(self):
        loc = str(QtCore.QLocale.system().name())
        self.doc_path = self.core.workspace["docs"].path

        if not self.doc_is_installed(self.doc_path):
            # First use, no doc installed
            lgg.info("Install doc to: %s" % self.doc_path)
            if not self.install_docs(loc, True):
                return

        else:
            self.installed = True

        self.get_translations()
        lgg.info("Found languages: %s, looking for: %s" 
                        % (", ".join(self.translations), loc))

        self.find_translation(loc)
        # XXX Devel lang
        #self.current_language = 'en_UK'
        idx = self.translations.index(self.current_language)
        self.ui.lang_cmb.setCurrentIndex(idx)
        self.ui.lang_cmb.currentIndexChanged.connect(self.on_language_changed)
        self.load_docs()

    def find_translation(self, loc):
        if loc in self.translations:
            self.current_language = loc

        else:
            lng = loc.split('_')[0]
            for l in self.translations:
                if lng in l:
                    self.current_language = l
                    return

        self.current_language = 'en_UK'

    def load_docs(self):
        doc_trans = os.path.join(self.doc_path, self.current_language)
        if not os.path.isdir(doc_trans):
            doc_trans = os.path.join(self.doc_path, 'en_UK')
            if not os.path.isdir(doc_trans):
                self.set_update_widgets(True)
                return

        self.docs_fld = doc_trans
        self.tab_names = self.core.cfg.get(("manual", "opened_tabs"))
        self.search_kwords = None
        self.recents = None
        self.model = {}
        self.get_doc_tree()
        self.set_index(self.index_tree, self.ui.treeWidget)
        self.setup_pages()
        self.check_update()

    def reload_docs(self):
        lgg.info("Reload: %s" % self.new_update)
        if not self.new_update:
            return

        for idx in range(self.ui.tabWidget.count()-1):
            self.ui.tabWidget.removeTab(0)
        self.ui.treeWidget.clear()
        self.current_pages = [None]
        self.tabs = [self.ui.tabWidget.widget(0)]
        self.current_language = self.new_update
        self.load_docs()

    def on_language_changed(self, idx):
        self.new_update = self.translations[idx]
        self.reload_docs()
        

    def get_translations(self):
        """Check for the translations available.

        """
        self.translations = []
        docs = self.core.workspace["docs"].path + "/*"
        for d in glob.glob(docs):
            if os.path.isdir(d):
                self.translations.append(os.path.basename(d))

        for tr in self.translations:
            try:
                self.ui.lang_cmb.addItem(LANGUAGES[tr])
            except KeyError:
                self.ui.lang_cmb.addItem(tr)

    def get_doc_tree(self):
        """Un-json the index of the documentation.

        """
        fname = os.path.join(self.docs_fld, 'index.json')
        self.index_tree = self.read_json(fname)

    def set_index(self, index, parent):
        """This recursive function is the index tree iterator.

        Args:
        index -- nested dictionnary
        parent -- the QtreeWidget at the initial call, the parent 
                  QTreeWidgetItem for the nested call
        """
        def set_item(item, data):
            twi = QtWidgets.QTreeWidgetItem(item)
            twi.setText(0, value[0])
            self.model[data[0]] = os.path.join(self.docs_fld, data[1])
            return twi

        items = sorted(index.keys())

        for item in items:
            value = index[item]
            print("%s - %s" %(value[0], value[1]))
            # Some entries are not linked to a html file
            if value[1]:
                # Check if the html is really provided
                if not os.path.isfile(os.path.join(self.docs_fld, value[1])):
                    continue

            new = set_item(parent, value)
            if value[2]:
                self.set_index(value[2], new)

    def setup_pages(self):
        """Open the last unclosed pages at startup.

        """
        if not self.recents:
            self.set_page(0, self.model['Main Window'])

    def show_page(self, where, link):
        page = link.path()
        if '#' in link.toString() or os.path.isfile(page):
            # Called from the webView
            page = page.split('#')[0]
            self.set_page(where, page, link)

        else:
            # Called from the index tree context menu
            try:
                page = self.model[page]
                self.set_page(where, page)
            except:
                # missing page
                pass

    def set_page(self, idx, uri, link=None):
        """Open a new page.

        Args:
        idx -- the index of the tab, if None the current tab is used, else if 
               idx is "new" a new tab is created
        uri -- the uri of the html file
        """
        anchor = False
        try:
            if link.fragment():
                anchor = True
        except: 
            pass

        if not self.valid_page(uri, anchor):
            return

        name = self.get_page_name(uri)
        if idx is None:
            # Open the page into the current tab
            idx = self.ui.tabWidget.currentIndex()

        if isinstance(idx, int):
            self.update_tree_index(idx)

        elif idx == 'new':
            # Open the page into a new tab
            idx = len(self.tabs)
            self.tabs.append(self.ui.add_tab(name))
            self.current_pages.append(uri)

        self.current_pages[idx] = uri

        if link is None:
            link = QtCore.QUrl.fromLocalFile(uri)

        self.tabs[idx].update_page(name, link)
        self.ui.tabWidget.setCurrentIndex(idx)
        self.ui.treeWidget.reset_displayed()

    def on_item_tree_selected(self, item, col, tab=None):
        if tab is None:
            if self.ui.treeWidget.button == 4:
                tab = 'new'

        page = item.text(0)
        uri = self.model[page]
        if self.valid_page(uri):
            lgg.info("set page: %s" % uri)
            self.set_page(tab, uri)

    def call_external_link(self, url):
        webbrowser.open_new_tab(url)

    def valid_page(self, uri, anchor=False):
        if not os.path.isfile(uri):
            return False

        elif not anchor and uri in self.current_pages:
            self.ui.tabWidget.setCurrentIndex(self.current_pages.index(uri))
            return False

        return True

    def update_tree_index(self, idx):
        try:
            self.ui.treeWidget.reset_displayed()
            self.ui.treeWidget.reset()
            self.ui.treeWidget.reset_expanded()
        except:
            # At startup the list is empty
            pass

    def get_page_name(self, where):
        """Return the name of page associate to a file path.

        Args:
        where -- the html file path

        Returns:
        str(name)
        """
        for k, v in self.model.items():
            print(k, v)
            if v == where:
                return k
        print("%s Not found" % where)

    def install_docs(self, lang, new=False):
        temp = self.core.workspace['temp'].path
        update = self.fetch_update(temp)
        if not update:
            langs = False

        else:
            self.unpack_json_tar(temp)
            js = os.path.join(temp, "DOCVERSIONS.json")
            update = self.read_json(js)
            langs = list(update.keys())
            langs.remove("en_UK")
            langs.insert(0, "en_UK")
            if lang in langs:
                langs.remove(lang)
                langs.insert(0, lang)

        self.new_update = False
        from helpviewer.docinstaller import DocInstaller
        self.di = DocInstaller(self, new)
        self.di.setup(langs, self.user_fld.path, update)
        return self.di.exec_()

    def update_doc(self):
        if self.install_docs(self.current_language):
            self.reload_docs()

    def fetch_update(self, tmp):
        """Download the list of the last doc version published.

        Args:
        tmp -- the path of the temp dir, usually ~/.oqapy/temp
        """
        url = 'http://oqapy.eu/releases/DOCVERSIONS.tar.gz'
        user_agent = 'Mozilla/5.0 (X11; Ubuntu; Linux x86_64; rv:49.0)'\
                      ' Gecko/20100101 Oqapy/3.0'
        req = urllib.request.Request(url, data=None,
                                     headers={"User-Agent": user_agent})
        lgg.info('Get update ...')
        try:
            content = urllib.request.urlopen(req, timeout=5)
        except Exception as why:
            lgg.info("Unable to find the doc versions at oqapy.eu")
            lgg.info(" %s" % str(why))
            return False

        tar = os.path.join(tmp, "DOCVERSIONS.tar.gz")
        try:
            with open(tar, 'wb') as outf:
                outf.write(content.read())
        except Exception as why:
            lgg.warning("Unable to write: %s\n  \%s" % (tmp, why))
            self.display_error(_("Unable to write:"), tmp, why)

            return False

        return True

    def unpack_json_tar(self, fld):
        def doc_files(members):
            for tarinfo in members:
                if tarinfo.name.startswith('DOCVERSIONS'):
                    yield tarinfo

        old_dir = os.getcwd()
        os.chdir(fld)
        gz = os.path.join(fld, 'DOCVERSIONS.tar.gz')
        tar = tarfile.open(gz)
        tar.extractall(members=doc_files(tar))
        tar.close()
        os.chdir(old_dir)

    def fetch_doc(self, lang):
        tmp = self.core.workspace['temp'].path
        if self.fetch_update(tmp):
            self.unpack_json_tar(tmp)
            doc_vers = self.read_json(os.path.join(tmp, 'DOCVERSIONS.json'))
            langs = doc_vers.keys()
            langs.remove("en_UK")
            langs.insert(0, "en_UK")
            if lang in langs:
                langs.remove(lang)
                langs.insert(0, lang)

    def read_json(self, filename):
        with open(filename, 'r') as inf:
            content = inf.read()
            return json.loads(content)

    def doc_is_installed(self, path):
        def install():
            self.copy_docs(path)

        if not os.path.isdir(path):
            install()
            return True

        dirs = os.listdir(path)
        if not dirs:
            install()

        return True

    def check_update(self):
        if not self.installed:
            return

        tmp = self.core.workspace['temp'].path
        if self.fetch_update(tmp):
            self.unpack_json_tar(tmp)
            old = os.path.join(self.user_fld.path, "docs", 'DOCVERSIONS.json')
            new = os.path.join(tmp, 'DOCVERSIONS.json')
            lang = self.current_language
            try:
                old_vers = self.read_json(old)
                new_vers = self.read_json(new)
                lgg.info("Current doc vers: %s, update: %s" 
                            % (old_vers[lang]["version"], 
                            new_vers[lang]["version"]))
                if old_vers[lang]["version"] < new_vers[lang]["version"]:
                    self.set_update_widgets(True)
            except:
                return
            # check for localisation
            loc = str(QtCore.QLocale.system().name())
            if not loc in self.translations and loc in new_vers:
                self.set_update_widgets(True)

    def set_update_widgets(self, state):
        self.ui.update_btn.setVisible(state)

    def display_error(self, error, fld, reason):
        from helpviewer.warning import Warning_
        self.warn = Warning_(error, fld, reason)
        self.warn.exec_()

    def copy_docs(self, path, force=False):
        root = os.path.dirname(os.path.abspath(__file__))
        if not root:
            root = os.getcwd()

        docs = os.path.join(root, 'docs')
        if not os.path.isdir(path) or force:
            lgg.info('Install docs: %s' % path)
            try:
                shutil.copytree(docs, path)
                return
            except Exception as why:
                lgg.warning('Unable to install the docs: %s' % why)
                return

        for obj in os.listdir(docs):
            dest = os.path.join(path, obj)
            if obj == 'DOCVERSIONS.json':
                if not os.path.isfile(dest):
                    orig = os.path.join(docs, obj)
                    shutil.copy(orig, dest)

            elif '_' in obj:
                if not os.path.isdir(dest):
                    orig = os.path.join(docs, obj)
                    shutil.copytree(orig, dest)

# ------------------------------------------------------------
#  Devel usage
#-------------------------------------------------------------

    def print_(self):
        for idx, tab in enumerate(self.tabs):
            print(idx, tab.title)


