# histogram.py
# 
# This file is part of Oqapy serie 3
#
# Author: Vincent Vande Vyvre <vincent.vandevyvre@oqapy.eu>
# Home: http://www.oqapy.eu
# Copyright: 2009-2016 Vincent Vande Vyvre
# Licence: GPL3
#
# Histogram builder

import logging
lgg = logging.getLogger(__name__)

from PIL import Image

from PyQt5.QtCore import QObject, Qt, pyqtSignal, QSize
from PyQt5.QtGui import (QColor, QImage, QPainter, QPen, QPainterPath, QBrush, )

class Histogram(QObject):
    histogramCreated = pyqtSignal(str)
    def __init__(self, filename, fhisto, size):
        """Instanciate an object histogram.

        Args:
        filename -- image file path
        fhisto -- file name used for saving the histogram image
        size -- histogram viewer size
        """
        super().__init__()
        self.filename = filename
        self.fhisto = fhisto
        self.width_ = size[0]
        self.height_ = 150
        self.hist_size = 256
        self.brightness = False
        self.is_grey = False
        self.back = None
        self.back_color = QColor(10, 20, 30, 255).rgb()  #142332FF
        self.stop_color = QColor(150, 150, 150, 255)
        self.grvb = []


    def build_histogram(self):
        """Build the histogram.

        """
        try:
            img = Image.open(self.filename)
            lgg.info('PIL image open, size: %s, mode: %s' %(str(img.size),
                                                            img.mode))
        except Exception as why:
            lgg.warning("PIL can't read the file: %s" % self.filename)
            lgg.warning("Reason: %s" % why)
            self.histogramCreated.emit('')

        if self.is_grey:
            # The image is black & white
            self.load_grey_image(img)

        else:
            self.load_color_image(img)

    def load_color_image(self, image):
        """Extract the color and brightness histogram of the image.

        args:
        image -- instance of PIL.Image
        """
        cvals = image.histogram()
        greymap = image.convert("L")
        lvals = greymap.histogram()
        self.draw_histogram(cvals, lvals)

    def load_grey_image(self, image):
        """Extract the brightness histogram of the image.

        args:
        image -- instance of PIL.Image
        """
        greymap = image.convert("L")
        lvals = greymap.histogram()
        self.draw_histogram(False, lvals)


    def draw_histogram(self, colors, greys):
        """Create the histograms.

        Args:
        colors -- list of colors value, may be empty if the image is a greymap
        greys -- list of greys values, may be empty if no brightness histogram
                 is required
        """
        self.back = self.create_background()
        if greys:
            self.draw_grey_curve(greys)
            
        if colors:
            # Some black & white images may have only 256 values
            rvals = colors[:256] or [1]
            gvals = colors[256:512] or [1]
            bvals = colors[512:769] or [1]
            ratio = self.get_ratio(rvals, gvals, bvals)
            self.draw_color_curves(rvals, gvals, bvals, ratio)

        self.back.save(self.fhisto)
        self.histogramCreated.emit(self.fhisto)

    def draw_grey_curve(self, values):
        ratio = max(values) / 150.0
        border = QColor(100, 105, 110, 255)
        filler = QColor(50, 55, 60, 180)
        painter = QPainter()
        painter.begin(self.back)
        path = QPainterPath()
        path.moveTo(1, self.height_)

        for i, val in enumerate(values):
            y = self.height_ - (val / ratio) + 2
            path.lineTo(i, y)

        path.lineTo(256, self.height_)
        path.closeSubpath()
        painter.setPen(QPen(border, 2, Qt.SolidLine, 
                                  Qt.RoundCap, Qt.MiterJoin))
        painter.drawPath(path)
        brush = QBrush(filler)
        painter.fillPath(path, brush)
        painter.end()

    def draw_color_curves(self, red, green, blue, ratio):
        colors = (QColor(255, 0, 0, 255),
                  QColor(0, 255, 0, 255),
                  QColor(0, 0, 255, 255))

        if self.back == None:
            self.back = self.create_background()

        painter = QPainter()
        painter.begin(self.back)

        for idx, vals in enumerate((red, green, blue)):
            path = QPainterPath()
            path.moveTo(1, self.height_)
            
            for i, val in enumerate(vals):
                y = self.height_ - (val / ratio) + 2
                path.lineTo(i + 1, y)

            painter.setPen(QPen(colors[idx], 1,
                              Qt.SolidLine, Qt.RoundCap,
                              Qt.MiterJoin))
            painter.drawPath(path)

    def create_background(self):
        back_color = QColor(10, 20, 30, 255).rgb()
        stop_color = QColor(150, 150, 150, 255)
        back = QImage(QSize(258, 150), 
                            QImage.Format_ARGB32_Premultiplied)
        back.fill(back_color)
        painter = QPainter()
        painter.begin(back)     
        # Draw stop lines
        painter.setPen(QPen(stop_color, 1, Qt.SolidLine,
                              Qt.RoundCap, Qt.MiterJoin))
        painter.drawLine(51, 0, 51, 150)
        painter.drawLine(101, 0, 101, 150)
        painter.drawLine(151, 0, 151, 150)
        painter.drawLine(201, 0, 201, 150)
        painter.end()

        return back

    def get_ratio(self, r, v, b):
        max_ = max(max(r), max(v), max(b))
        return max_ / 150.0


