# -*- coding: utf-8 -*-

# ioimage.py
# This file is part of Oqapy serie 3
#
# Author: Vincent Vande Vyvre <vincent.vandevyvre@oqapy.eu>
# Home: http://www.oqapy.eu
# Copyright: 2009-2017 Vincent Vande Vyvre
# Licence: GPL3
#
# Qt class for images support

import os

from PyQt5.QtCore import QByteArray
from PyQt5.QtGui import QImageReader, QImageWriter

class IOImage:
    def __init__(self):
        self.reader = Reader()
        self.writer = Writer()
        self.get_supported_formats()

    def get_supported_formats(self):
        read = self.reader.get_formats_supported()
        # Change QBytesArray to str()
        self.read_support = [str(f, encoding='utf-8') for f in read]
        write = self.writer.get_formats_supported()
        self.write_support = [str(f, encoding='utf-8') for f in write]

    def get_read_format_support(self):
        return self.read_support

    def get_write_format_support(self):
        return self.write_support

    def set_filename_to_reader(self, path):
        self.reader.set_filename(path)

    def set_filename_to_writer(self, path):
        self.writer.set_filename(path)

    def is_readable(self, img=None):
        return self.reader.is_readable()

    def is_writable(self):
        return self.writer.is_writable()

    def create_image(self):
        return self.reader.create_image()

    def write_image(self, format=None, quality=None):
        return self.writer.save(format, quality)
    
    def set_image_to_writer(self, qimg):
        self.writer.image = qimg

class Reader(QImageReader):
    def __init__(self):
        super().__init__()
        self.supportedFormat = self.supportedImageFormats()
        self.filename = None
        self.format = None

    def get_formats_supported(self):
        return self.supportedFormat

    def set_filename(self, path):
        self.setFileName(path)
        self.format = os.path.splitext(path)[1]

    def get_filename(self):
        return self.fileName()

    def get_format(self):
        return self.format

    def is_readable(self, img=None):
        return self.canRead()

    def create_image(self):
        img = self.read()
        if not img.isNull():
            return img
        return (self.error(), self.errorString())

class Writer(QImageWriter):
    def __init__(self):
        super().__init__()
        self.supportedFormat = self.supportedImageFormats()
        self.image = None

    def get_formats_supported(self):
        return self.supportedFormat

    def set_filename(self, path):
        fname , ext = os.path.splitext(path)
        self.setFileName(path)
        if ext:
            ext = ext.lstrip('.')
            try:
                self.setFormat(ext)
            except TypeError:
                # extension must be QByteArray since Qt-5.?
                try:
                    self.setFormat(QByteArray(ext))
                except:
                    # QByteArray from string is deprecated in PyQt v5.4 and will 
                    # be removed in v5.5
                    a = QByteArray()
                    for i in ext:
                        a.append(i)
                    self.setFormat(a)

    def get_filename(self):
        return self.fileName()

    def get_format(self):
        return self.format()

    def is_writable(self):
        return self.canWrite()

    def save(self, format=None, quality=None):
        if format is not None:
            self.setFormat(format.lstrip('.'))

        if not self.format():
            return (False, 'Write error:', 'No format defined.')

        if not self.fileName():
            return (False, 'Write error:', 'No file name defined.')

        if quality is not None:
            self.setQuality(quality)

        try:
            reply = self.write(self.image)
        except Exception as why:
            return (False, why, '')

        if not reply:
            return (False, self.error(), self.errorString())

        return (True,)
