# -*- coding: utf-8 -*-

# keyboard.py
# This file is part of Oqapy serie 3
#
# Author: Vincent Vande Vyvre <vincent.vandevyvre@oqapy.eu>
# Home: http://www.oqapy.eu
# Copyright: 2009-2017 Vincent Vande Vyvre
# Licence: GPL3
#
# Key binding handler

# Shortcuts with numerical keypad issue:
# https://bugreports.qt.io/browse/QTBUG-33093

from PyQt5.QtCore import Qt
from PyQt5.QtGui import QKeySequence
from PyQt5.QtWidgets import QShortcut

from data import SHORTCUTS

class Keyboard:
    def __init__(self, ui, parent=None):
        """Instanciate the key binding handler.

        """
        # Note: some shortcuts are not defined here, see graphicsarea.py
        self.ui = ui
        self.core = ui.core
        #self.get_config()
        self.build_shortcuts()

    def get_config(self):
        """Get the user's preferences.

        """
        self.cfg = self.core.cfg

    def build_shortcuts(self):
        """Create all shortcuts.

        """
        d = SHORTCUTS["viewing"]
        s = Shortcut(self.ui.mainwidget, d["next_image"], 1)
        s.activated.connect(self.core.next_image)
        s = Shortcut(self.ui.mainwidget, d["prev_image"], 1)
        s.activated.connect(self.core.previous_image)
        s = Shortcut(self.ui.mainwidget, d["rotate_r"], 1)
        s.activated.connect(self.core.rotate_to_right)
        s = Shortcut(self.ui.mainwidget, d["rotate_l"], 1)
        s.activated.connect(self.core.rotate_to_left)
        s = Shortcut(self.ui.mainwidget, "Del", 1)
        s.activated.connect(self.core.on_trashcan_clicked)
        d = SHORTCUTS["sorting"]
        parent = self.ui.stackedviews.sorting_tab
        s = Shortcut(parent, d["next_right_tab"], 1)
        s.activated.connect(parent.shift_to_next_tab)
        s = Shortcut(parent, d["next_left_tab"], 1)
        s.activated.connect(parent.shift_to_previous_tab)
        #parent = self.ui.stackedviews
        driver = self.ui.targetarea
        s = Shortcut(parent, d["next_right_button"], 1)
        s.activated.connect(driver.shift_selected_target_to_right)
        s = Shortcut(parent, d["next_left_button"], 1)
        s.activated.connect(driver.shift_selected_target_to_left)
        s = Shortcut(parent, d["unselect"], 1)
        s.activated.connect(driver.unselect_target)
        s = Shortcut(parent, d["copy"], 1)
        #s.activated.connect(self.ui.copy_to_selected_target)
        s = Shortcut(parent, d["move"], 1)
        #s.activated.connect(self.ui.move_to_selected_target)
        s = Shortcut(parent, d["close_folder"], 1)
        #s.activated.connect(self.ui.close_selected_target)
        s = Shortcut(parent, d["open_folder"], 1)
        #s.activated.connect(self.ui.open_selected_target)
        d = SHORTCUTS["trailer"]
        parent = self.ui.stackedtrailers
        s = Shortcut(self.ui.mainwidget, d["show"], 1)
        s.activated.connect(self.ui.toggle_trailer)
        s = Shortcut(parent, d["forward"], 2)
        s.activated.connect(parent.scroll_to_right)
        s = Shortcut(parent, d["reverse"], 2)
        s.activated.connect(parent.scroll_to_left)
        s = Shortcut(parent, d["fast_forward"], 2)
        s.activated.connect(parent.scroll_to_right2)
        s = Shortcut(parent, d["fast_reverse"], 2)
        s.activated.connect(parent.scroll_to_left2)
        d = SHORTCUTS["aerial_view"]
        s = Shortcut(self.ui.mainwidget, d["show_hide"], 1)
        s.setAutoRepeat(False)
        s.activated.connect(self.core.display_aerial_view)
        s = Shortcut(self.ui.mainwidget, d["permanent"], 1)
        s.activated.connect(self.core.display_aerial_view)
        d = SHORTCUTS["crop_tool"]
        s = Shortcut(self.ui.mainwidget, d["activate"], 1)
        s.activated.connect(self.core.activate_crop_tool)
        d = SHORTCUTS["magnifier"]
        s = Shortcut(self.ui.mainwidget, d["show_hide"], 1)
        s.setAutoRepeat(False)
        s.activated.connect(self.core.toggle_magnifier)
        s = Shortcut(self.ui.mainwidget, d["set_focus"], 1)
        s.activated.connect(self.core.set_focus_on_magnifier)
        d = SHORTCUTS["grid"]
        s = Shortcut(self.ui.mainwidget, d["show_hide"], 1)
        s.activated.connect(self.core.toggle_grid)
        d = SHORTCUTS["zoom"]
        s = Shortcut(self.ui.stackedviews, d["plus"], 1)
        s.activated.connect(self.core.on_zoom_up)
        s = Shortcut(self.ui.stackedviews, d["minus"], 1)
        s.activated.connect(self.core.on_zoom_down)
        s = Shortcut(self.ui.stackedviews, d["fit_in_view"], 1)
        s.activated.connect(self.core.on_fit_in_view)
        s = Shortcut(self.ui.stackedviews, d["full_size"], 1)
        s.activated.connect(self.core.on_zoom_full_size)
        d = SHORTCUTS["global_menu"]
        s  = Shortcut(self.ui.mainwidget, d["show_hide"], 1)
        s.activated.connect(self.core.display_global_menu)


class Shortcut(QShortcut):
    def __init__(self, parent, ks=None, ct=None):
        super().__init__(parent)
        self._owner = parent

        if ks is not None:
            self.set_key(ks)

        if ct is not None:
            self.set_context(ct)

    def set_key(self, seq):
        """Sets sequence of keys for shortcut.

        Args:
        seq -- key's sequence
        """
        ks = QKeySequence(seq)
        self.setKey(ks)

    def set_context(self, context):
        """Sets shortcut's context.

        Args:
        context -- context, an int in range 0-3
                0 is active when its parent widget has focus
                1 is active when its parent widget is a logical subwidget of 
                    the active top-level window
                2 is active when one of the applications windows are active
                3 is active when its parent widget, or any of its children has
                    focus
        """
        contexts = [Qt.WidgetShortcut, Qt.WindowShortcut, 
                    Qt.ApplicationShortcut, 
                    Qt.WidgetWithChildrenShortcut]
        self.setContext(contexts[context])
