/*
 * oqapy-iproc
 * Image processor using openCV
 *
 * This lib is part of Oqapy serie 3
 *
 * Author: Vincent Vande Vyvre <vincent.vandevyvre@oqapy.eu>
 * Home: http://www.oqapy.eu
 * Copyright: 2009-2017 Vincent Vande Vyvre
 * Licence: GPL3
 *
 *
 * This lib is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This lib is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * <http://www.gnu.org/licenses/>.
 * 
 */

#include "/usr/include/opencv2/highgui/highgui.hpp"
#include "/usr/include/opencv2/imgproc/imgproc.hpp"
#include "/usr/include/opencv2/photo/photo.hpp"
#include <iostream>
#include <stdio.h>
#include <fstream>
#include <math.h>
#include "oqapy-iproc.h"

#define OQAPY_IPROC_VERSION_HEX    0x000200
#define OQAPY_IPROC_VERSION_STR    "0.2.0"

using namespace std;
using namespace cv;

OqapyIProc::OqapyIProc()
{
    bool isLoaded_ = false;
}

const char* OqapyIProc::version()
{
    return OQAPY_IPROC_VERSION_STR;
}

const int OqapyIProc::version_hex()
{
    return OQAPY_IPROC_VERSION_HEX;
}

int OqapyIProc::loadImage(const char *filename, int mode)
{
    src = imread(filename, mode);

    if( !src.data )
    { 
        cout << "Image " << filename << " not loaded\n";
        isLoaded_ = false;
        return 0; 
    }

    else
    {
        isLoaded_ = true;
        return 1;
    }
}

bool OqapyIProc::isLoaded()
{
    if (!isLoaded_)
    {
        return false;
    }
    return true;
}

void OqapyIProc::getColorHistogram()
{
    vector<Mat> bgr_planes;
    split(src, bgr_planes);
    int histSize = 256;
    float range[] = {0, 256};
    const float* histRange = { range };

    bool uniform = true; bool accumulate = false;

    calcHist(&bgr_planes[0], 1, 0, Mat(), b_hist, 1, &histSize, &histRange, uniform, accumulate);
    calcHist(&bgr_planes[1], 1, 0, Mat(), g_hist, 1, &histSize, &histRange, uniform, accumulate);
    calcHist(&bgr_planes[2], 1, 0, Mat(), r_hist, 1, &histSize, &histRange, uniform, accumulate);
}

/*
 * Contrast stretching
 *
 * This Filter is based on the work of Image Processing On Line
 * http://www.ipol.im/pub/art/2011/llmps-scb/
 *
 * Copyright: 2009-2014, IPOL Image Processing On Line & the authors
 */
int OqapyIProc::applyContrastStretching(float lower, float upper, const char* outf)
{
    if (!isLoaded_)
    {
        cout << "ImgProcess::applyContrastStretching error: image not loaded !";
        return 1;
    }

    getColorHistogram();

    float c_hist[256];

    /* compute the cumulative blue histogram */
    c_hist[0] = b_hist.at<float>(0);
    for (int i = 1 ; i < 255 ; i++)
    {
        int j = i - 1;
        c_hist[i] = (c_hist[j] + b_hist.at<float>(i));
    }

    /* compute the lower blue treshold */
    int bmin = 0;
    int size = 255;
    while (bmin < size && c_hist[bmin] <= lower)
    {
        bmin += 1;
    }

    /* compute the upper blue treshold */
    int bmax = 254;
    while (c_hist[bmax] > upper)
    {
        bmax -= 1;
    }
    bmax += 1;

    /* compute the cumulative green histogram */
    c_hist[0] = g_hist.at<float>(0);
    for (int i = 1 ; i < 255 ; i++)
    {
        int j = i - 1;
        c_hist[i] = (c_hist[j] + g_hist.at<float>(i));
    }
    
    /* compute the lower green treshold */
    int gmin = 0;
    while (gmin < size && c_hist[gmin] <= lower)
    {
        gmin += 1;
    }

    /* compute the upper green treshold */
    int gmax = 254;
    while (c_hist[gmax] > upper)
    {
        gmax -= 1;
    }
    gmax += 1;

    /* compute the cumulative red histogram */
    c_hist[0] = r_hist.at<float>(0);
    for (int i = 1 ; i < 255 ; i++)
    {
        int j = i - 1;
        c_hist[i] = (c_hist[j] + r_hist.at<float>(i));
    }
    
    /* compute the lower red treshold */
    int rmin = 0;
    while (rmin < size && c_hist[rmin] <= lower)
    {
        rmin += 1;
    }

    /* compute the upper red treshold */
    int rmax = 254;
    while (c_hist[rmax] > upper)
    {
        rmax -= 1;
    }
    rmax += 1;

    /* src.copyTo( target ) */

    setContrastStretching( bmin, bmax, 0 );
    setContrastStretching( gmin, gmax, 1 );
    setContrastStretching( rmin, rmax, 2 );

    imwrite( outf, src );
    
    return 0;
}

void OqapyIProc::setContrastStretching(int min, int max, int index)
{
    int norm [256];
    if (max <= min)
    {
        for( int y = 0 ; y < src.rows ; y++ )
        { 
            for( int x = 0 ; x < src.cols ; x++ )
            {
                src.at<Vec3b>(y,x)[index] = 127;
            }
        }
    }

    else
    {
        int i;
        for (i = 0 ; i < min ; i++)
            norm[i] = 0;

        for (i = min ; i < max ; i++)
            norm[i] = ((i - min) * 255) / ((max - min) + 0.5);

        for(i = max ; i < 255 + 1 ; i++)
            norm[i] = 255;

        for( int y = 0 ; y < src.rows ; y++ )
        { 
            for( int x = 0 ; x < src.cols ; x++ )
            {
                src.at<Vec3b>(y,x)[index] = norm[src.at<Vec3b>(y,x)[index]];
            }
        }
    }
}

int OqapyIProc::applyContrastBrightness(float gain, float bias, float lower, 
                                        float upper, const char* outf)
{
    int counter = 0;

    for (int idx = 0 ; idx < 3 ; idx++)
    {
        for( int y = 0 ; y < src.rows ; y++ )
        { 
            for( int x = 0 ; x < src.cols ; x++ )
            {
                float val = src.at<Vec3b>(y,x)[idx];
                if (val > lower && val < upper)
                {
                    val = val * gain + bias;
                    counter += 1;

                    if (val > 255)
                        val = 255.0;

                    if (val < 0)
                        val = 0;                 

                    src.at<Vec3b>(y,x)[idx] = val;
                }
            }
        }
    }

    imwrite( outf, src );
    return 0;
}

int OqapyIProc::convertToGreyMap(const char* outf)
/* Convert the color image to grey map.  This use the desaturation method 
 *      R = G = B = (min(R, G, B) + max(R, G, B)) / 2                     
 */
{
    float min, max;
    for( int y = 0 ; y < src.rows ; y++ )
    { 
        for( int x = 0 ; x < src.cols ; x++ )
        {
            float b = src.at<Vec3b>(y,x)[0];
            float g = src.at<Vec3b>(y,x)[1];
            float r = src.at<Vec3b>(y,x)[2];

            min = fmin(b, fmin(g, r));
            max = fmax(b, fmax(g, r));
            float s = (min + max) / 2;

            src.at<Vec3b>(y,x)[0] = s;
            src.at<Vec3b>(y,x)[1] = s;
            src.at<Vec3b>(y,x)[2] = s;
        }
    }
            
    imwrite( outf, src );
    return 0;
}

int OqapyIProc::convertToBrightGreyMap(int level, const char* outf)
/* Convert the color image to grey map with a brightness level
 * level must be in range 0 to 5
 */

{
    Mat dst(src);
    Matx33f m33;
    switch (level)
    {   case 0 : m33 = Matx33f(0.34, 0.7, 0.02, 0.34, 0.7, 0.02,
                                    0.34, 0.7, 0.02);
                 break;
        case 1 : m33 = Matx33f(0.02, 0.7, 0.34, 0.02, 0.7, 0.34, 
                                    0.02, 0.7, 0.34);
                 break;
        case 2 : m33 = Matx33f(0.14, 0.67, 0.38, 0.14, 0.67, 
                                    0.38, 0.14, 0.67, 0.38);
                 break;
        case 3 : m33 = Matx33f(0.32, 0.57, 0.38, 0.32, 0.57, 
                                    0.38, 0.32, 0.57, 0.38);
                 break;
        case 4 : m33 = Matx33f(0.05, 0.6, 0.6, 0.05, 0.6, 0.6, 
                                    0.05, 0.6, 0.6);
                 break;
        default : m33 = Matx33f(0.03, 0.76, 0.54, 0.03, 0.76, 
                                    0.54, 0.03, 0.76, 0.54);
    }

    transform(src, dst, m33);

    imwrite( outf, dst );
    return 0;
}

int OqapyIProc::applyToningSepia(int level, const char* outf)
/* Convert the color image to sepia with a brightness level
 * level must be in range 0 to 4
 */

{
    Mat dst(src);
    Matx33f m33;
    switch (level)
    {   case 0 : m33 = Matx33f(0.27, 0.53, 0.13, 0.35, 0.69, 
                                    0.17, 0.4, 0.77, 0.19);
                 break;
        case 1 : m33 = Matx33f(0.29, 0.55, 0.15, 0.4, 0.73, 
                                    0.22, 0.5, 0.77, 0.29);
                 break;
        case 2 : m33 = Matx33f(0.32, 0.58, 0.18, 0.4, 0.73, 
                                    0.22, 0.45, 0.82, 0.24);
                 break;
        case 3 : m33 = Matx33f(0.34, 0.6, 0.2, 0.45, 0.78, 
                                    0.27, 0.5, 0.77, 0.29);
                 break;
        default : m33 = Matx33f(0.38, 0.64, 0.24, 0.49, 0.82, 
                                    0.31, 0.54, 0.81, 0.33);
    }

    transform(src, dst, m33);

    imwrite( outf, dst );
    return 0;
}

int OqapyIProc::applyToningBlue(int level, const char* outf)
/* Convert the color image to blue with a brightness level
 * level must be in range 0 to 4
 */

{
    Mat dst(src);
    Matx33f m33;
    switch (level)
    {   case 0 : m33 = Matx33f(0.8, 0.69, 0.56, 0.46, 0.37, 
                                    0.22, 0.45, 0.4, 0.25);
                 break;
        case 1 : m33 = Matx33f(0.85, 0.78, 0.6, 0.51, 0.42, 
                                    0.32, 0.43, 0.41, 0.31);
                 break;
        case 2 : m33 = Matx33f(0.79, 0.72, 0.62, 0.6, 0.51, 
                                    0.4, 0.48, 0.42, 0.36);
                 break;
        case 3 : m33 = Matx33f(0.83, 0.72, 0.62, 0.63, 0.54, 
                                    0.43, 0.48, 0.41, 0.36);
                 break;
        default : m33 = Matx33f(0.86, 0.75, 0.65, 0.65, 0.56, 
                                    0.45, 0.51, 0.45, 0.39);
    }

    transform(src, dst, m33);

    imwrite( outf, dst );
    return 0;
}

int OqapyIProc::applyLaplacianFilter(int ksize, const char* outf)
/* Apply a Laplacian computation
 * ksize  Aperture size used to compute the second-derivative filters
 * outf   Full path of the destination file
 */
{
    Mat dst(src);
    Mat src_gray, abs_dst;
    int scale = 1;
    int delta = 0;
    int ddepth = CV_16S;

    /* Remove noise by blurring with a Gaussian filter */
    GaussianBlur( src, src, Size(3,3), 0, 0, BORDER_DEFAULT );

    /* Convert the image to grayscale */
    cvtColor( src, src_gray, CV_BGR2GRAY );

    Laplacian( src_gray, dst, ddepth, ksize, scale, delta, BORDER_DEFAULT );
    convertScaleAbs( dst, abs_dst );

    imwrite( outf, dst );
    return 0;
}

int OqapyIProc::denoiseGreyMap(float strength, int patch, int window,
                                const char* outf)
/* Denoise a B/W image
 * strength Is the parameter regulating the filter strength 
 * patch    Size in pixels of the template patch that is used to compute weights
 * window   Size in pixels of the window that is used to compute weighted 
 *          average for given pixel. Should be odd
 * outf     Full path of the denoised file
 */
{
    Mat dst(src);
    fastNlMeansDenoising( src, dst, strength, patch, window );

    imwrite( outf, dst );
    return 0;
}

int OqapyIProc::denoiseColorMap(float strength, int color, int patch,
                                int window, const char* outf)
/* Denoise a B/W image
 * strength Is the parameter regulating the filter strength 
 * patch    Size in pixels of the template patch that is used to compute weights
 * color    The same as patch but for color components
 * window   Size in pixels of the window that is used to compute weighted 
 *          average for given pixel. Should be odd
 * outf     Full path of the denoised file
 */
{
    Mat dst(src);
    fastNlMeansDenoisingColored(src, dst, strength, color, patch, window);

    imwrite( outf, dst );
    return 0;
}

OqapyIProc::~OqapyIProc()
{
}

