# -*- coding: utf-8 -*-

# magnifier.py
# This file is part of Oqapy serie 3
#
# Author: Vincent Vande Vyvre <vincent.vandevyvre@oqapy.eu>
# Home: http://www.oqapy.eu
# Copyright: 2009-2016 Vincent Vande Vyvre
# Licence: GPL3
#
# Magnifying glass widget

import os
import logging
lgg = logging.getLogger(__name__)

from math import cos, radians

from PyQt5 import QtCore, QtGui, QtWidgets


class Magnifier:
    def __init__(self, core, view, img, zoom):
        self.core = core
        self.view = view
        self.cfg = core.cfg.get(('magnifier', ))
        self.qimage = img
        self.zoom = zoom
        self.pixmapitem = PixmapItem(self)
        self.pixmapitem.setup()
        view.scene().addItem(self.pixmapitem)

    def close(self):
        self.pixmapitem.quit()

    def on_settings_changed(self, settings):
        fact, size, shape = settings
        lgg.info("Settings update %s, %s, %s" %(fact, size, shape))
        self.cfg = {'factor': fact, 'size': size, 'shape': shape}
        self.core.cfg.update(('magnifier', ), self.cfg)

    def on_close(self):
        self.pixmapitem = False
        self.core.remove_magnifier()


class PixmapItem(QtWidgets.QGraphicsPixmapItem):
    def __init__(self, driver):
        super().__init__(None)
        self.driver = driver
        self.shape = driver.cfg['shape']
        self.size = driver.cfg['size']
        self.factor = driver.cfg['factor']
        self.image = driver.qimage
        self.graphic = driver.view
        self._scene = self.graphic.scene()
        self.cur_pos = None
        self.zoom = driver.zoom
        self.background_color = (255, 255, 255, 255)
        self.old_X = None
        self.old_Y = None
        self.setFlags(QtWidgets.QGraphicsItem.ItemIsMovable | 
                        QtWidgets.QGraphicsItem.ItemSendsScenePositionChanges |
                        QtWidgets.QGraphicsItem.ItemIsFocusable)
        self.setAcceptHoverEvents(True)

    def set_graphic(self, graphic):
        """Sets the QGraphicsView.

        Args:
        graphic -- QGraphicsView instance
        """
        if not isinstance(graphic, QtWidgets.QGraphicsView):
            lgg.error("set_graphic arg not QGraphicView,"
                            " got {0}".format(graphic))
            return

        self.graphic = graphic

    def set_scene(self, scene):
        """Sets the QGraphicsScene.

        Args:
        scene -- QGraphicsScene instance
        """
        if not isinstance(scene, QtWidgets.QGraphicsScene):
            lgg.error("set_scene arg not QGraphicscene,"
                            " got {0}".format(scene))
            return

        self._scene = scene
        scene.addItem(self)

    def set_image(self, img):
        """Sets the QImage.

        Args:
        img -- a QImage scaled 1:1 or a str(path of image file)
        """
        if isinstance(img, QtGui.QImage):
            self.image = img

        elif isinstance(img, str):
            if not os.path.isfile(img):
                lgg.error("set_image no such file: {0}"
                                .format(img))
                return

            try:
                image = QtGui.QImage(img)
            except Exception as why:
                lgg.error("set_image unable to open {0}"
                                "\n\tReason: {1}".format(img, why))
                return

            if image.isNull:
                lgg.error("set_image can't create a valid"
                                " QImage with file {0}".format(img))
                return

            self.image = img

        else:
            lgg.error("set_image arg not valid: {0}"
                                .format(img))

    def set_zoom(self, zoom):
        """Set the zoom factor of the current view.

        Args:
        zoom -- float
        """
        if not isinstance(zoom, float):
            lgg.error("set_zoom arg not float, got {0}"
                                .format(zoom))
            return

        self.zoom = zoom

    def set_size(self, size):
        """Set the size of the magnifying glass.

        The size represent the side of the shape 'square' or the diameter of
        the shape 'circle'

        Args:
        size -- integer
        """
        if not isinstance(size, int):
            lgg.error("set_size arg not int, got {0}".format(size))
            return

        self.size = size
        if self.cur_pos:
            self._redraw()

    def set_shape(self, shape):
        """Set the shape of the magnifying glass.

        Args:
        shape -- str() "square" or "circle"
        """
        if not shape in ["square", "circle"]:
            lgg.error("set_shape arg must be 'square' or 'circle',"
                                " got {0}".format(shape))
            return

        self.shape = shape
        if self.cur_pos:
            self._redraw()

    def set_factor(self, fact):
        """Set the magnification factor.

        Args:
        fact -- float
        """
        if not isinstance(fact, float) and not fact == 1:
            lgg.error("set_factor arg must be float or 1, got {0}"
                                .format(fact))
            return

        self.factor = fact * 1.0
        if self.cur_pos:
            p = self.pos()
            self.setup()
            p2 = self.pos()
            self.setPos(p)
            x, y = p.x() - p2.x(), p.y() - p2.y()
            self._update(x, y)

    def set_background_color(self, color):
        """Set the background color of the magnifying glass.

        Set the background color of the magnifying glass with the background
        color of the QGraphicsView.
        When the magnifying glass leave the image we fill the texture with 
        this color.
        This only needed with the shape 'circle'.

        Args:
        color -- tuple or list, if rgb is given, alpha = 255 is append.
        """
        try:
            r, v, b = color[0:3]
            c = QtGui.QColor()
            c.setRgb(r, v, b)
        except:
            lgg.error("set_background_color value error, got {0}"
                                .format(color))
            return

        if len(color) == 3:
            # change rgb to rgba
            color = [c for c in color[:]]
            color.append(255)

        self.background_color = color
        self._set_brush()

    def _set_brush(self):
        # Qcolor don't like tuple
        r, v, b, a = self.background_color
        color = QtGui.QColor()
        color.setRgb(r, v, b, a)
        self.brush = QtGui.QBrush(color, QtCore.Qt.SolidPattern)

    def setup(self):
        """setup the magnifier.

        """
        self.cur_pos = self._get_center()
        self.setPos(self.cur_pos[2], self.cur_pos[3])
        self._set_brush()
        self.multi = self.factor * (1 / self.zoom)
        self._create_magnifier()

        if not self.old_X:
            self.recenter_magnifying_glass()
        self.old_X, self.old_Y = self.pos().x(), self.pos().y()
        #self._scene.addItem(self)

    def _create_magnifier(self):
        """Call functions to build the magnifying glass.

        """
        self._set_pixmap()
        self.orig = self._get_orig_center()
        if self.shape == "square":
            self._set_square()

        else:
            self._set_circle()

    def _set_pixmap(self):
        """Resize the image with magnification factor.

        """
        w, h = self.image.width(), self.image.height()
        self.base = self.image.scaled(w * self.factor, h * self.factor,
                                    QtCore.Qt.KeepAspectRatio, 
                                    QtCore.Qt.SmoothTransformation)

    def _set_pix(self):
        """Create the pixmap support for the texture.

        Returns:
        QPixmap square filled transparent
        """
        color = QtGui.QColor()
        color.setRgb(0, 0, 0, 0)
        pix = QtGui.QPixmap(QtCore.QSize(self.size + 4, self.size + 4))
        pix.fill(color)
        return pix        

    def _set_square(self):
        """Build a squared magnifying glass.

        """
        self.pix = self._set_pix()
        brect = QtCore.QRect(0, 0, self.size + 4, self.size + 4)
        painter = QtGui.QPainter()
        painter.begin(self.pix)
        painter.setPen(QtGui.QPen(QtCore.Qt.gray, 2,
                                  QtCore.Qt.SolidLine, QtCore.Qt.RoundCap,
                                  QtCore.Qt.MiterJoin))
        painter.drawRect(brect)
        painter.end()
        self._fill_square()
        self._set_buttons_sqr()

    def _fill_square(self):
        """Draw the texture into the shape 'square'.

        """
        painter = QtGui.QPainter()
        painter.begin(self.pix)
        rect = QtCore.QRect(2, 2, self.size, self.size)
        painter.eraseRect(rect)
        irect = QtCore.QRect(self.orig[2], self.orig[3], self.size, self.size)
        painter.drawImage(rect, self.base, irect)
        painter.end()
        self.setPixmap(self.pix)

    def _set_circle(self):
        """Build a circular magnifying glass.

        """
        self.pix = self._set_pix()
        brect = QtCore.QRect(0, 0, self.size + 4, self.size + 4)
        painter = QtGui.QPainter()
        painter.begin(self.pix)
        painter.setRenderHints(QtGui.QPainter.Antialiasing, True)
        painter.setPen(QtGui.QPen(QtCore.Qt.gray, 1,
                                  QtCore.Qt.SolidLine, QtCore.Qt.RoundCap,
                                  QtCore.Qt.MiterJoin))
        painter.drawEllipse(brect)
        rect = QtCore.QRectF(2, 2, self.size, self.size)
        self.path = QtGui.QPainterPath()
        self.path.addEllipse(rect)
        painter.drawPath(self.path)
        painter.end()
        self._fill_circle()
        self._set_buttons_crc()

    def _fill_circle(self):
        """Draw the texture into the shape 'circle'.

        """
        painter = QtGui.QPainter()
        brush = QtGui.QBrush()
        brush.setTexture(self._set_texture())
        painter.begin(self.pix)     
        painter.fillPath(self.path, brush)
        painter.end()
        self.setPixmap(self.pix)

    def _set_buttons_sqr(self):
        """Build the settings buttons for the shape square.

        """
        buttons = ButtonsKit(0, self, self._scene)
        buttons.setPos(-22, self.size - 75)
        buttons = ButtonsKit(1, self, self._scene)
        buttons.setPos(11, self.size)

    def _set_buttons_crc(self):
        """Build the settings buttons for the shape circle.

        """
        s = (self.size / 2) * cos(radians(45))
        x = (self.size / 2) - s - 33
        y = (self.size / 2) + s - 33
        buttons = ButtonsKit(2, self, self._scene)
        buttons.setPos(x + 2, y - 1)
        buttons = ButtonsKit(3, self, self._scene)
        buttons.setPos(y, y)

    def _update(self, x, y):
        """Update the original coordinates when the magnifying glass is moved.

        """
        self.orig[2] += x * self.multi
        self.orig[3] += y * self.multi
        if self.shape == "square":
            self._fill_square()

        else:
            self._fill_circle()

    def move_by(self, x, y):
        """Move the magnifying glass.

        Args:
        x -- number of pixels on x axis.
        y -- number of pixels on y axis.
        """
        if not isinstance(x, int) or not isinstance(y, int):
            lgg.error("move_by values not integer, got {0}, {1}"
                                .format(x, y))
            return

        self.moveBy(x, y)
        self._update(x, y)

    def _redraw(self):
        """Redraw magnifying glass when parameters are changed.

        """
        painter = QtGui.QPainter()
        painter.begin(self.pix)
        rect = QtCore.QRect(0, 0, self.size + 4, self.size + 4)
        painter.eraseRect(rect)
        painter.end()
        self.setPixmap(self.pix)
        self._set_pixmap()
        self.remove_buttons()
        if self.shape == "square":
            self._set_square()

        else:
            self._set_circle()        

    def _get_center(self):
        """Returns the center of the scene.

        This assume both scene and image have the same geometry.

        Returns:
        list[center X, center Y, top left x, top left Y]
        """
        cX, cY = int(self._scene.width() /2), int(self._scene.height() /2)
        posX, posY = cX - int(self.size / 2), cY - int(self.size / 2)
        return [cX, cY, posX, posY]

    def _set_texture(self):
        """Returns the sub-region of image at the current coordinate.

        Returns:
        QPixmap
        """
        texture = QtGui.QPixmap(QtCore.QSize(self.size + 1, self.size + 1))
        irect = QtCore.QRect(self.orig[2], self.orig[3], self.size + 1, self.size + 1)
        prect = QtCore.QRect(0, 0, self.size + 1, self.size + 1)
        painter = QtGui.QPainter()
        painter.begin(texture)
        painter.fillRect(prect, self.brush)
        painter.drawImage(prect, self.base, irect)
        painter.end()
        return texture

    def get_pos(self):
        """Returns the position of magnifying glass.

        Returns:
        QPointF(float, float)
        """
        return self.pos()

    def set_pos(self, pos):
        """Move magnifying glass to pos.

        Args:
        pos -- QPointF or tuple or list
        """
        if isinstance(pos, QtCore.QPointF):
            X, Y = pos.x(), pos.y()

        else:
            X, Y = pos[0], pos[1]

        cur = self.pos()
        m_X, m_Y = X - self.pos().x(), Y - self.pos().y()
        self.moveBy(m_X, m_Y)
        self._update(m_X, m_Y)
        #self.positionChanged.emit([cur.x(), cur.y(), X, Y])

    def _get_orig_center(self):
        """Returns the original position at the creation of the magnifier.

        Returns:
        list -- center of pixmap magnified and corner top left.
        """
        cX, cY = int(self.base.width() /2), int(self.base.height() /2)
        posX, posY = cX - int(self.size / 2), cY - int(self.size / 2)
        return [cX, cY, posX, posY]

    def recenter_magnifying_glass2(self):
        """Move the magnifying glass from center image to center graphicsView.

        This function was written before the function below.
        She's handle the texture changes into the movement, but the quality
        is too dependent of the server x resources
        """
        c = self.size / 2.0
        orig_x, orig_y = self.pos().x() + c, self.pos().y() + c
        g_size = self.graphic.size()
        g_center_x, g_center_y = g_size.width() / 2, g_size.height() / 2
        i_center = self.graphic.mapFromScene(orig_x, orig_y)
        delta = g_center_y - i_center.y()
        step = 1
        if delta < 0:
            step = -1

        for i in range(abs(delta)):
            # Vertical move
            self.move_by(0, step)
            QtCore.QCoreApplication.processEvents()

        delta = g_center_x - i_center.x()
        step = 1
        if delta < 0:
            step = -1

        for i in range(abs(delta)):
            # Horizontal move
            self.move_by(step, 0)
            QtCore.QCoreApplication.processEvents()

    def recenter_magnifying_glass(self):
        c = self.size / 2.0
        dest = self.graphic.mapToScene(self.graphic.width() / 2 - c, 
                                        self.graphic.height() / 2 - c)
        self.set_pos(dest)
        return
        timer = QtCore.QTimeLine(1000)
        timer.setFrameRange(0, 50)
        # FIXME QGraphicsItemAnimation class is no longer supported.
        self.animation = QtGui.QGraphicsItemAnimation()
        self.animation.setItem(self);
        self.animation.setTimeLine(timer)

        c = self.size / 2.0
        dest = self.graphic.mapToScene(self.graphic.width() / 2 - c, 
                                        self.graphic.height() / 2 - c)
        deltaX = dest.x() - self.pos().x()
        deltaY = dest.y() - self.pos().y()
        j, k, l, m = deltaX / 50, deltaY / 50, self.pos().x(), self.pos().y()

        for i in range (50):
            l, m = l + j, m + k
            self.animation.setPosAt(i / 50.0, QtCore.QPointF(l, m))
            self._update(j, k)

        timer.start()

    def remove_buttons(self):
        for child in self.childItems():
            child.setParentItem(None)
            self._scene.removeItem(child)


    #--------------------------------
    # Events
    #--------------------------------

    def mousePressEvent(self, event, auto=0):
        # auto = 0 if event is caused by self
        # auto = 1 if event is given by other, i.e. event filter of main
        if auto:
            pos = self.graphic.mapToScene(event.pos())

        else:
            pos = event.scenePos()

        if event.button() == 1:
            self.old_X, self.old_Y = pos.x(), pos.y()
            event.accept()

        else:
            event.ignore()

    def mouseMoveEvent(self, event, auto=0):
        # auto = 0 if event is caused by self
        # auto = 1 if event is given by other, i.e. event filter of main
        if auto:
            pos = self.graphic.mapToScene(event.pos())

        else:
            pos = event.scenePos()

        cur = self.pos()
        X, Y = pos.x(), pos.y()
        m_X, m_Y = X - self.old_X, Y - self.old_Y
        self.old_X, self.old_Y = X, Y
        self.moveBy(m_X, m_Y)
        self._update(m_X, m_Y)
        #self.positionChanged.emit([cur.x(), cur.y(), 
                                    #self.pos().x(), self.pos().y()])
        event.accept()

    def hoverEnterEvent(self, event):
        self.old_shape = self.graphic.cursor().shape()
        shape = QtGui.QPixmap("medias/cross_cursor_32.png")
        cursor = QtGui.QCursor(shape, -1, -1)
        self.setCursor(cursor)

    def hoverLeaveEvent(self, event):
        cursor = QtGui.QCursor(self.old_shape)
        self.graphic.setCursor(cursor)

    def contextMenuEvent(self, event):
        self._show_menu(event.screenPos())
        event.accept()

    def keyPressEvent(self, event):
        if event.key() == QtCore.Qt.Key_Escape:
            self.quit()

        elif event.key() == 16777234:
            # arrow LEFT
            self.move_by(-5, 0)
            #self.positionChanged.emit([self.pos().x() + 5, self.pos().y(), 
                                    #self.pos().x(), self.pos().y()])

        elif event.key() == 16777235:
            # arrow UP
            self.move_by(0, -5)
            #self.positionChanged.emit([self.pos().x(), self.pos().y() + 5, 
                                    #self.pos().x(), self.pos().y()])

        elif event.key() == 16777236:
            # arrow RIGHT
            self.move_by(5, 0)
            #self.positionChanged.emit([self.pos().x() - 5, self.pos().y(), 
                                    #self.pos().x(), self.pos().y()])

        elif event.key() == 16777237:
            # arrow DOWN
            self.move_by(0, 5)
            #self.positionChanged.emit([self.pos().x(), self.pos().y() - 5, 
                                    #self.pos().x(), self.pos().y()])

    def change_settings(self, s):
        """Apply new settings when a button is pressed.

        Args:
        s -- index of button
        """
        if not s and self.size < 400:
            self.size += 10
            self._redraw()

        elif s == 1 and self.size > 140:
            self.size -= 10
            self._redraw()

        elif s == 2 and self.factor < 5:
            self.set_factor(self.factor + 0.10)

        elif s == 3 and self.factor > 0.25:
            self.set_factor(self.factor - 0.10)

        self.driver.on_settings_changed([self.factor, self.size, self.shape])

    def _show_menu(self, pos):
        """Build and show context menu.

        Args:
        pos -- event.pos()
        """
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setWeight(75)
        font.setBold(True)
        self.menu = QtWidgets.QMenu()
        self.act_shape = QtWidgets.QAction(None)
        self.act_shape.setText(_("Shape"))
        self.act_shape.setFont(font)
        self.menu.addAction(self.act_shape)
        self.shape_group = QtWidgets.QActionGroup(None)
        square = QtWidgets.QAction(self.shape_group)
        square.setText(_("Square"))
        square.setCheckable(True)
        self.shape_group.addAction(square)
        self.menu.addAction(square)
        circle = QtWidgets.QAction(self.shape_group)
        circle.setText(_("Circle"))     
        circle.setCheckable(True)
        circle.setChecked(True)
        self.shape_group.addAction(circle)
        self.menu.addAction(circle)

        if self.shape == "square":
            square.setChecked(True)
        else:
            circle.setChecked(True)

        self.menu.addSeparator()
        self.act_close = QtWidgets.QAction(None)
        self.act_close.setText(_("Close"))
        self.menu.addAction(self.act_close)
        square.triggered.connect(self.change_to_square)
        circle.triggered.connect(self.change_to_circle)
        self.act_close.triggered.connect(self.quit)
        self.menu.popup(pos)

    def change_to_square(self):
        self.set_shape("square")
        self.driver.on_settings_changed([self.factor, self.size, self.shape])

    def change_to_circle(self):
        self.set_shape("circle")
        self.driver.on_settings_changed([self.factor, self.size, self.shape])

    def quit(self):
        """Quiet exit magnifying glass.

        """ 
        self._scene.removeItem(self)
        self.driver.on_close()

class ButtonsKit(QtWidgets.QGraphicsPixmapItem):
    def __init__(self, idx, parent=None, scene=None):
        super().__init__(parent)
        d = dict(idx = idx,
                    parent = parent,
                    _scene = scene,
                    img = None,
                    plus =None,
                    minus = None)
        for key, value in d.items():
            setattr(self, key, value)

        self.setFlags(QtWidgets.QGraphicsItem.ItemStacksBehindParent)

        images = ["medias/magn_but_s_l.png", "medias/magn_but_s_r.png",
                    "medias/magn_but_c_l.png", "medias/magn_but_c_r.png"]
        pix = QtGui.QPixmap(images[idx])
        self.setPixmap(pix)

        class Button(QtWidgets.QGraphicsPixmapItem):
            def __init__(self, idx, parent=None, scene=None):
                super().__init__(parent)
                d = dict(idx = idx, parent = parent, _scene = scene)
                for key, value in d.items():
                    setattr(self, key, value)
                self.setAcceptHoverEvents(True)

                if not idx % 2.0:
                    self.normale = QtGui.QPixmap("medias/plus.png")
                    self.active = QtGui.QPixmap("medias/plus_hover.png")

                else:
                    self.normale = QtGui.QPixmap("medias/minus.png")
                    self.active = QtGui.QPixmap("medias/minus_hover.png")

                self.setPixmap(self.normale)

            def hoverEnterEvent(self, event):
                self.old_shape = self.parent.cursor().shape()
                self.setCursor(QtCore.Qt.ArrowCursor)
                self.setPixmap(self.active)
                event.accept()

            def hoverLeaveEvent(self, event):
                cursor = QtGui.QCursor(self.old_shape)
                self.setCursor(cursor)
                self.setPixmap(self.normale)
                event.accept()

            def mousePressEvent(self, event):
                if event.button() == 1:
                    self.parent.on_button_pressed(self.idx)
                    event.accept()

        if not idx:
            # size +
            button = Button(0, self, scene)
            button.setPos(2.0, 6.7)
            button = Button(1, self, scene)
            button.setPos(2.0, 40)

        elif idx == 1:
            # size -
            button = Button(2, self, scene)
            button.setPos(7.0, 3.6)
            button = Button(3, self, scene)
            button.setPos(40.0, 3.6)

        elif idx == 2:
            # factor +
            button = Button(0, self, scene)
            button.setPos(2.0, 21)
            button = Button(1, self, scene)
            button.setPos(25.5, 45.0)

        elif idx == 3:
            # factor -
            button = Button(2, self, scene)
            button.setPos(46.0, 21.0)
            button = Button(3, self, scene)
            button.setPos(22.5, 45)


    def on_button_pressed(self, b):
        self.parent.change_settings(b)
