# -*- coding: utf-8 -*-

# metadata.py
# This file is part of Oqapy serie 3
#
# Author: Vincent Vande Vyvre <vincent.vandevyvre@oqapy.eu>
# Home: http://www.oqapy.eu
# Copyright: 2011-2017 Vincent Vande Vyvre
# Licence: GPL3
#
# Metadata handler
#
# http://www.sno.phy.queensu.ca/~phil/exiftool/TagNames/XMP.html
# www.photometadata.org/META-Resources-Field-Guide-to-Metadata

import os
import datetime
import math
import re
import logging
lgg = logging.getLogger(__name__)

from fractions import Fraction

from PyQt5.QtCore import QObject

import pyexiv2

from utils import Utils
from data import VERSION_STR
from metadata.misc import EDITABLES, EDITABLES_DESC, XMP
from metadata.misc import FLASH_VALUES as FLK
from metadata.misc import LIGHTSOURCE_VALUES as LSK
from warningmessages import WarningMessages

class Metadata():
    """Define the metadata handler.

    """
    def __init__(self, filename):
        """Initialisation.

        Args:
        filename -- full path of the image file
        """
        self.filename = filename
        self.last_error = None
        self.data = None
        self.utils = Utils(self)

    def get_data(self):
        """Load the metadata from the image file.

        Returns:
        True if the metada are loaded, False otherwise
        """
        lgg.info('Read metadata: .../%s' % os.path.basename(self.filename))
        # Avoid inappropriate error
        if not os.path.isfile(self.filename):
            self.last_error = "No such file %s" % self.filename
            return False

        try:
            self.data = pyexiv2.ImageMetadata(self.filename)
            self.data.read()
        except Exception as why:
            self.last_error = "Read error %s" % why
            return False

        return True

    def get_general_infos(self):
        """Returns common infos.

        Common infos are Author, Copyright, Creation Date, Image Title,
        Comments (keywords) and Unique Identifier.
        """
        tags = ['Iptc.Application2.Headline', 'Exif.Photo.DateTimeOriginal', 
                'Exif.Image.Artist', 'Exif.Image.Copyright', 
                'Exif.Photo.UserComment', 'Exif.Photo.ImageUniqueID']
        inf = []
        for k in tags:
            if k in self.data:
                dat = self.clean_data(self.data[k].value)
                inf.append(dat)
            else:
                inf.append('    --')

        # Check if iptc or xmp contains other keywords
        comment = self.get_keywords()
        if comment:
            inf.pop(4)
            inf.insert(4, comment)

        return inf

    def get_date_and_id(self):
        try:
            date = self.data['Exif.Photo.DateTimeOriginal'].value
        except:
            try:
                date = self.data['Xmp.exif.DateTimeOriginal'].value
            except:
                try:
                    date = self.data['Xmp.xmp.CreateDate'].value
                except:
                    date = ""
        if date:
            date = date.strftime('%a %d %b %Y, %H:%M')

        try:
            uid = self.data['Exif.Photo.ImageUniqueID'].value
        except:
            uid = ""
        return date, uid

    def get_creator_infos(self):
        tags = ['Xmp.dc.creator', 'Exif.Image.Artist',
                'Xmp.dc.rights', 'Exif.Image.Copyright',
                'Xmp.xmpRights.Marked', 'Xmp.xmpRights.UsageTerms',
                'Xmp.iptc.CreatorContactInfo/Iptc4xmpCore:CiAdrExtadr',
                'Xmp.iptc.CreatorContactInfo/Iptc4xmpCore:CiAdrCity',
                'Xmp.iptc.CreatorContactInfo/Iptc4xmpCore:CiAdrRegion',
                'Xmp.iptc.CreatorContactInfo/Iptc4xmpCore:CiAdrPcode',
                'Xmp.iptc.CreatorContactInfo/Iptc4xmpCore:CiAdrCtry',
                'Xmp.iptc.CreatorContactInfo/Iptc4xmpCore:CiTelWork',
                'Xmp.iptc.CreatorContactInfo/Iptc4xmpCore:CiEmailWork',
                'Xmp.iptc.CreatorContactInfo/Iptc4xmpCore:CiUrlWork']
        inf = []
        for k in tags:
            if k in self.data:
                inf.append(self.data[k].value)
            else:
                inf.append("")
        return inf

    def get_image_infos(self):
        """Returns the image infos, size, resolution, colorspace and software.

        """
        inf = []
        width, height = self.get_image_size()
        inf.append(' x '.join([str(self.utils.format_number(width)), 
                               str(self.utils.format_number(height))]))

        try:
            x = self.data['Exif.Image.XResolution'].value
            y = self.data['Exif.Image.YResolution'].value
            if x == y:
                inf.append(str(x))
            else:
                inf.append(' x '.join([str(x), str(y)]))
        except:
            inf.append('    --')

        cs = self.get_color_space()
        if cs is None:
            cs = 'Undefined'
        inf.append(cs)

        for i in ('Exif.Photo.SceneType', 'Exif.Image.ProcessingSoftware'):
            try:
                # These tags may content two null bytes
                if len(self.data[i]) > 2:
                    inf.append(str(self.data[i]))
                else:
                    inf.append(False)
            except:
                inf.append(False)

        inf.append(self.get_headline())
        return inf

    def get_image_size(self):
        """Return the image size (width, height).

        """
        w = h = 0
        try:
            w = self.data['Exif.Image.ImageWidth'].value
            h = self.data['Exif.Image.ImageLength'].value
        except:
            try:
                w = self.data['Exif.Photo.PixelXDimension'].value
                h = self.data['Exif.Photo.PixelYDimension'].value
            except Exception as why:
                lgg.info('Image size not found: %s' % why)
        return w, h

    def get_color_space(self):
        """Returns the image color space.

        """
        try:
            icc = self.data['Exif.Image.InterColorProfile'].value
            if 'Adobe RGB (1998)' in icc:
                return 'Adobe RGB (1998)'
        except:
            pass

        try:
            icc = self.data['Exif.Photo.ColorSpace'].value
            if icc == 1:
                return 'sRGB'
            return None
        except:
            return None

    def get_exposure_values(self):
        """Returns the exposure time, diaph., ISO, and orientation values.

        """
        values = {'exp': None, 'fn': None, 'iso': None, 'orient': 1}
        if self.data is not None:
            try:
                values['exp'] = self.data["Exif.Photo.ExposureTime"].value
            except:
                pass

            try:
                values['fn'] = self.data["Exif.Photo.FNumber"].value
            except:
                pass

            try:
                values['iso'] = self.data["Exif.Photo.ISOSpeedRatings"].value
            except:
                pass

            values['orient'] = self.get_orientation()

        return values

    def get_orientation(self):
        try:
            return self.data["Exif.Image.Orientation"].value
        except:
            return 1

    def get_exposure_infos(self):
        """Returns all the exposure infos.

        """
        exposures = ['Not defined', 'Manual', 'Normal program', 
                    'Aperture priority', 'Shutter priority',
                    'Creative (Slow speed)', 'Action (High speed)',
                    'Portrait', 'Landscape']
        expomodes = ['Auto exposure', 'Manual exposure', 'Auto bracket']
        metringmodes = ['    --', 'Average', 'CenterWeightedAverage', 'Spot',
                        'MultiSpot', 'Multi-segment', 'Partial', 'Other']
        exp = []
        try:
            exp.append(str(self.data['Exif.Photo.ISOSpeedRatings'].value))
        except:
            exp.append('    --')

        try:
            s = self.data['Exif.Photo.ExposureTime'].value
            if s.denominator == 1:
                exp.append("%s sec." % s.numerator)
            else:
                exp.append("/".join(["1", str(int(s.denominator / s.numerator))]))
        except:
            exp.append('    --')

        try:
            f = self.data['Exif.Photo.FNumber'].value
            exp.append(str(round(f.numerator / f.denominator, 2)))
        except:
            exp.append('    --')

        try:
            # Exposure correction
            b = self.data['Exif.Photo.ExposureBiasValue'].value
            exp.append(str(round(b.numerator / b.denominator, 2)))
        except:
            exp.append('    --')

        try:
            f = self.data['Exif.Photo.FocalLength'].value
            exp.append(str(round(f.numerator / f.denominator, 2)))
        except:
            exp.append('    --')

        try:
            val = self.data['Exif.Photo.Flash'].value
            # Flash spec is imported from data.py
            exp.append(FLK[val])
        except:
            exp.append('    --')

        try:
            val = self.data['Exif.Photo.ExposureProgram'].value
            exp.append(exposures[val])
        except:
            exp.append('    --')

        try:
            val = self.data['Exif.Photo.MeteringMode'].value
            exp.append(metringmodes[val])
        except:
            exp.append('    --')

        try:
            val = self.data['Exif.Photo.ExposureMode'].value
            exp.append(expomodes[val])
        except:
            exp.append('    --')

        # Following data are optional
        try:
            val = self.data['Exif.Photo.WhiteBalance'].value
            exp.append(['Auto', 'Manual'][val])
        except:
            exp.append(False)

        try:
            val = self.data['Exif.Photo.LightSource'].value
            exp.append(LSK[val])
        except:
            exp.append(False)

        try:
            val = self.data['Exif.Photo.SubjectDistanceRange'].value
            exp.append(str(val))
        except:
            exp.append(False)

        try:
            val = self.data['Exif.Photo.Contrast'].value
            exp.append(['Normal', 'Soft', 'Hard'][val])
        except:
            exp.append(False)

        try:
            val = self.data['Exif.Photo.Saturation'].value
            exp.append(['Normal', 'Low', 'High'][val])
        except:
            exp.append(False)

        try:
            val = self.data['Exif.Photo.Sharpness'].value
            exp.append(['Normal', 'Soft', 'Hard'][val])
        except:
            exp.append(False)

        return exp

    def get_thumbnails(self):
        """Read thumbnails into metadata.

        """
        self.thumbnails = self.data.previews

    def get_thumbnails_size(self):
        """Returns a list of thumbnail(s) size.

        """
        if self.thumbnails:
            sizes = []
            for thumb in self.thumbnails:
                sizes.append(thumb.dimensions)
        else:
            sizes = None

        return sizes

    def export_thumbnail(self, path, quality=0):
        """Save a preview embedded into the image.

        Args:
        path -- full path of the file without the extension
        quality -- the quality of the image, 0 is the smallest one

        Returns:
        The path with extension of the file or False if failed
        """
        self.get_thumbnails()
        if self.thumbnails:
            idx = -1
            num = len(self.thumbnails)
            if not quality or num == 1:
                idx = 0
                
            elif quality < num:
                idx = quality

            th = pyexiv2.preview.Preview(self.thumbnails[idx])
            ext = th.extension
            lgg.info('Save thumbnail: %s' % path)
            try:
                th.write_to_file(path)
            except Exception as why:
                lgg.warning('Save error: %s' % why)
                return False
            return path + ext

        return False

    def get_data_for_thumbnail(self):
        """Return the data needed for the thumbnail of an image.

        Returns:
        dict with keys: 'exp'           exposure time
                        'fn'            Fnumber
                        'iso'           ISO value
                        'title'         IPTC headline
                        'date'          creation date
                        'orientation'   orientation
        """
        tags = ['Iptc.Application2.Headline', 'Exif.Photo.DateTimeOriginal',
                'Exif.Image.Orientation']
        keys = ['title', 'date', 'orientation']
        data = {}
        for key, tag in zip(keys, tags):
            try:
                data[key] = self.data[tag].value
            except KeyError:
                data[key] = None

        for key, value in self.get_exposure_values().items():
            data[key] = value

        return data

    def write_thumbnail_data(self, data):
        """Write metadata into a thumbnail.

        Args:
        data -- dict
        """
        keys = ['exp', 'fn', 'iso', 'title', 'date', 'orientation']
        tags = ["Exif.Photo.ExposureTime", "Exif.Photo.FNumber",
                "Exif.Photo.ISOSpeedRatings", 'Iptc.Application2.Headline',
                'Exif.Photo.DateTimeOriginal', 'Exif.Image.Orientation']
        for key, tag in zip(keys, tags):
            if data[key] is not None:
                try:
                    self.data[tag].value = data[key]
                except KeyError:
                    if tag.startswith("Iptc"):
                        self.data[tag] = pyexiv2.iptc.IptcTag(tag, data[key])

                    else:
                        self.data[tag] = pyexiv2.exif.ExifTag(tag, data[key])

                #except Exception as why:
                    #lgg.info("Thumbnail data set error, %s, %s" %(key, why))

        try:
            self.data.write()
        except Exception as why:
            lgg.warning('Thumbnail data write error: %s' % why)

    def get_thumbnails_infos(self):
        """Returns thumbnail(s) infos.

        """
        self.get_thumbnails()
        th = []
        for t in self.thumbnails:
            s = t.size
            if s:
                s = self.utils.format_number(s)
                s += ' bytes'
            else:
                s = '    --'

            try:
                w, h = t.dimensions
                w = self.utils.format_number(w)
                h = self.utils.format_number(h)
                d = ' x '.join([str(w), str(h)])
            except:
                d = '    --'

            th.append((t.mime_type, s, d))
            # extension (t.extension) is usefull ?

        return th

    def get_camera_infos(self):
        """Returns the camera's infos.

        """
        try:
            cam = self.data['Exif.Image.Make'].value
            cam = self.clean_data(cam)
        except:
            cam = '    --'

        try:
            md = self.data['Exif.Image.Model'].value
            md = self.clean_data(md)
        except:
            md = '    --'

        try:
            sw = self.data['Exif.Image.Software'].value
            sw = self.clean_data(sw)
        except:
            sw = '    --'

        try:
            sn = self.data['Exif.Image.CameraSerialNumber'].value
            sn = self.clean_data(sn)
        except:
            try:
                sn = self.data['Exif.Photo.BodySerialNumber'].value
                sn = self.clean_data(sn)
            except:
                sn = None

        try:
            ow = self.data['Exif.Photo.CameraOwnerName'].value
            ow = self.clean_data(ow)
        except:
            try:
                ow = self.data['Exif.Canon.OwnerName'].value
                ow = self.clean_data(ow)
            except:
                ow = '    --'

        if sn is None:
            sn = self.get_serial_number(cam.lower())

        return (cam, md, sw, sn, ow)

    def get_serial_number(self, make):
        if make == '    --':
            return make

        if 'nikon' in make:
            try:
                sn = self.data['Exif.Nikon3.SerialNumber'].value
                return sn
            except Exception as why:
                return '    --'

        elif 'canon' in make:
            try:
                return str(self.data['Exif.Canon.SerialNumber'].value)
                #fn = self.data['Exif.Canon.SerialNumberFormat'].value
            except Exception as why:
                return '    --'

        elif 'fuji' in make:
            try:
                return self.data['Exif.Fujifilm.SerialNumber'].value
            except Exception as why:
                return '    --'

        elif 'pentax' in make:
            try:
                return self.data['Exif.Pentax.SerialNumber'].value
            except Exception as why:
                return '    --'

        elif 'olympus' in make:
            try:
                return self.data['Exif.Olympus.SerialNumber2'].value
            except Exception as why:
                try:
                    return self.data['Exif.OlympusEq.SerialNumber'].value
                except Exception as why:
                    return '    --'

        elif 'sigma' in make:
            try:
                return self.data['Exif.Sigma.SerialNumber'].value
            except Exception as why:
                return '    --'

        return '    --'

    def get_lens_infos(self, make=None):
        """Returns the len's infos.

        """
        # http://www.rottmerhusen.com/objektives/lensid/nikkor.html
        # http://owl.phy.queensu.ca/~phil/exiftool/TagNames/Nikon.html#LensID
        if make is not None:
            return self.get_constructor_lens_infos(make.lower())

        try:
            mk = self.data['Exif.Photo.LensMake'].value
            mk = self.clean_data(mk)
        except:
            mk = None

        try:
            md = self.data['Exif.Photo.LensModel'].value
            md = self.clean_data(md)
        except:
            md = None

        try:
            ls = self.data['Exif.Photo.LensSpecification'].value
            ls = self.format_len_spec(ls)
        except:
            ls = None

        try:
            ma = self.data['Exif.Photo.MaxApertureValue'].value
            ma = str(round(ma.numerator / ma.denominator, 2))
        except:
            ma = None

        try:
            sn = self.data['Exif.Photo.LensSerialNumber'].value
            sn = self.clean_data(sn)
        except:
            sn = None

        return ['common', mk, md, ls, ma, sn]

    def get_constructor_lens_infos(self, make):
        if 'canon' in make:
            #Exif.Canon.LensModel 	    Ascii
            #Exif.CanonCs.Lens 	        Short
            #Exif.Canon.FocalLength 	Short
            #Exif.CanonCs.MaxAperture 	SShort
            #Exif.CanonCs.MinAperture 	SShort
            try:
                model = self.data['Exif.Canon.LensModel'].value
                model = self.clean_data(model)
                # some models as DMARK3 contents extra characters
                model = model.replace(r'\ufffd', '')
            except Exception as why:
                model = None
            try:
                lens = self.data['Exif.CanonCs.Lens'].value
                # 'long' and 'short' focal length of lens (in 'focal units') 
                # and 'focal units' per mm
                lens[2] = lens[2] or 1
                lens = [lens[1]/lens[2], lens[0]/lens[2]]
            except Exception as why:
                lens = [None]
            try:
                fl = self.data['Exif.Canon.FocalLength'].value
                # [units, focale, n, d] focale * (n/d) = 35 mm equivalent
                fl[0] = fl[0] or 1
                fl = "%s mm" %str(fl[1] / fl[0])
            except Exception as why:
                fl = None
            try:
                mxa = self.data['Exif.CanonCs.MaxAperture'].value
                mxa = self.clean_data(mxa)
            except Exception as why:
                mxa = None
            try:
                mna = self.data['Exif.CanonCs.MinAperture'].value
                mna = self.clean_data(mna)
            except Exception as why:
                mna = None
            try:
                sn = self.data['Exif.Photo.LensSerialNumber'].value
                sn = self.clean_data(sn)
            except Exception as why:
                sn = None
            # Max and Min aperture seems incorrect for several models
            return ["Canon", model, lens, fl, sn]

        elif "casio" in make:
            # Add the generic data
            lens = self.get_lens_infos()
            lens.insert(0, "Casio")
            try:
                fl = self.data['Exif.Casio2.FocalLength'].value
                fl = str(round(fl.numerator / fl.denominator, 2))
                lens[4] = '%s mm' %fl
            except Exception as why:
                lens[4] = None
            return lens

        elif "fuji" in make:
            lens = ["Fuji"]
            lens.extend(self.get_lens_infos())
            try:
                amn = self.data['Exif.Fujifilm.MaxApertureAtMinFocal'].value
                lens[5] = str(round(amn.numerator / amn.denominator, 2))
            except Exception as why:
                pass
            return lens

        elif "minolta" in make:
            lens = ["Minolta"]
            lens.extend(self.get_lens_infos())
            try:
                fl = self.data['Exif.MinoltaCsNew.FocalLength'].value
                fl = self.clean_data(fl)
                lens.insert(4, "%s mm" % fl)
            except Exception as why:
                lens.insert(5, None)
            try:
                amx = self.data['Exif.MinoltaCsNew.MaxAperture'].value
                amx = self.clean_data(amx)
            except:
                amx = None
            lens.insert(6, amx)
            return lens

        elif "nikon" in make:
            return self.get_nikon_lens_data()

        return self.get_lens_infos()

    def get_nikon_lens_data(self):
        # The Nikon LensID is constructed as a Composite tag from the raw hex 
        # values of 8 other tags: LensIDNumber, LensFStops, MinFocalLength, 
        # MaxFocalLength, MaxApertureAtMinFocal, MaxApertureAtMaxFocal, 
        # MCUVersion and LensType, in that order.
        # source: http://www.rottmerhusen.com/objektives/lensid/thirdparty.html.
        # The data LensType don't exists with NikonLd1 and NikonLd2, so we don't
        # use it. The seven first data are sufficient to identify the lens.
        # Focal = round(5.0 * pow(2.0, value/24.0), 0)
        # Aperture = round(pow(2.0, 44.0/24.0), 1)

        try:
            # Try to define the exif familly
            vers = self.data['Exif.NikonLd1.Version']
            #Exif.NikonLd1.LensIDNumber 	        Lens ID number
            #Exif.NikonLd1.LensFStops 	            Lens F-stops
            #Exif.NikonLd1.MinFocalLength 	        Min focal length
            #Exif.NikonLd1.MaxFocalLength 	        Max focal length
            #Exif.NikonLd1.MaxApertureAtMinFocal    Max aperture at min focal
            #Exif.NikonLd1.MaxApertureAtMaxFocal    Max aperture at max focal
            #Exif.NikonLd1.MCUVersion               MCUVersion

            lens = ['nikonld1']
            try:
                lens.append(self.data['Exif.NikonLd1.LensIDNumber'].value)
            except Exception as why:
                lens.append(None)
            try:
                lens.append(self.data['Exif.NikonLd1.LensFStops'].value)
            except:
                lens.append(None)
            try:
                lens.append(self.data['Exif.NikonLd1.MinFocalLength'].value)
            except:
                lens.append(None)
            try:
                lens.append(self.data['Exif.NikonLd1.MaxFocalLength'].value)
            except:
                lens.append(None)
            try:
                lens.append(
                       self.data['Exif.NikonLd1.MaxApertureAtMinFocal'].value)
            except:
                lens.append(None)
            try:
                lens.append(
                       self.data['Exif.NikonLd1.MaxApertureAtMaxFocal'].value)
            except:
                lens.append(None)

            return lens

            try:
                lens.append(self.data['Exif.NikonLd1.MCUVersion'].value)
            except:
                lens.append(None)
            return lens
        except:
            try:
                vers = self.data['Exif.NikonLd2.Version']
                lens = ['nikonld2']
                #Exif.NikonLd2.LensIDNumber 	        Lens ID number
                #Exif.NikonLd2.LensFStops 	            Lens F-stops
                #Exif.NikonLd2.MinFocalLength 	        Min focal length
                #Exif.NikonLd2.MaxFocalLength 	        Max focal length
                #Exif.NikonLd2.MaxApertureAtMinFocal    Max aperture at min focal
                #Exif.NikonLd2.MaxApertureAtMaxFocal    Max aperture at max focal
                #Exif.NikonLd2.MCUVersion               MCUVersion
                #Exif.NikonLd2.AFAperture 	            AF aperture
                #Exif.NikonLd2.FocusPosition 	        Focus position
                #Exif.NikonLd2.FocusDistance 	        Focus distance
                #Exif.NikonLd2.FocalLength 	            Focal length
                #Exif.NikonLd2.EffectiveMaxAperture 	Effective max aperture

                try:
                    lens.append(self.data['Exif.NikonLd2.LensIDNumber'].value)
                except Exception as why:
                    lens.append(None)
                try:
                    lens.append(self.data['Exif.NikonLd2.LensFStops'].value)
                except:
                    lens.append(None)
                try:
                    lens.append(self.data['Exif.NikonLd2.MinFocalLength'].value)
                except:
                    lens.append(None)
                try:
                    lens.append(self.data['Exif.NikonLd2.MaxFocalLength'].value)
                except:
                    lens.append(None)
                try:
                    lens.append(
                        self.data['Exif.NikonLd2.MaxApertureAtMinFocal'].value)
                except:
                    lens.append(None)
                try:
                    lens.append(
                        self.data['Exif.NikonLd2.MaxApertureAtMaxFocal'].value)
                except:
                    lens.append(None)
                try:
                    lens.append(self.data['Exif.NikonLd2.MCUVersion'].value)
                except:
                    lens.append(None)
                try:
                    lens.append(self.data['Exif.NikonLd2.AFAperture'].value)
                except:
                    lens.append(None)
                try:
                    lens.append(self.data['Exif.NikonLd2.FocusPosition'].value)
                except:
                    lens.append(None)
                try:
                    lens.append(self.data['Exif.NikonLd2.FocusDistance'].value)
                except:
                    lens.append(None)
                try:
                    lens.append(self.data['Exif.NikonLd2.FocalLength'].value)
                except:
                    lens.append(None)
                try:
                    lens.append(
                        self.data['Exif.NikonLd2.EffectiveMaxAperture'].value)
                except:
                    lens.append(None)
            except:
                try:
                    vers = self.data['Exif.NikonLd3.Version']
                    lens = ['nikonld3']
                except Exception as why:
                    return self.get_lens_infos()

                #Exif.NikonLd3.LensIDNumber 	        Lens ID number
                #Exif.NikonLd3.LensFStops 	            Lens F-stops
                #Exif.NikonLd3.MinFocalLength 	        Min focal length
                #Exif.NikonLd3.MaxFocalLength 	        Max focal length
                #Exif.NikonLd3.MaxApertureAtMinFocal    Max ap. at min focal
                #Exif.NikonLd3.MaxApertureAtMaxFocal    Max ap. at max focal
                #Exif.NikonLd3.MCUVersion               MCUVersion
                #Exif.Nikon3.LensType                   Lens type
                #Exif.NikonLd3.AFAperture 	            AF aperture
                #Exif.NikonLd3.FocusPosition 	        Focus position
                #Exif.NikonLd3.FocusDistance 	        Focus distance
                #Exif.NikonLd3.FocalLength 	            Focal length
                #Exif.NikonLd3.EffectiveMaxAperture 	Effective max aperture
                try:
                    lens.append(self.data['Exif.NikonLd3.LensIDNumber'].value)
                except:
                    lens.append(None)
                try:
                    lens.append(self.data['Exif.NikonLd3.LensFStops'].value)
                except:
                    lens.append(None)
                try:
                    lens.append(self.data['Exif.NikonLd3.MinFocalLength'].value)
                except:
                    lens.append(None)
                try:
                    lens.append(self.data['Exif.NikonLd3.MaxFocalLength'].value)
                except:
                    lens.append(None)
                try:
                    lens.append(
                        self.data['Exif.NikonLd3.MaxApertureAtMinFocal'].value)
                except:
                    lens.append(None)
                try:
                    lens.append(
                        self.data['Exif.NikonLd3.MaxApertureAtMaxFocal'].value)
                except:
                    lens.append(None)
                try:
                    lens.append(self.data['Exif.NikonLd3.MCUVersion'].value)
                except:
                    lens.append(None)
                try:
                    lens.append(self.data['Exif.Nikon3.LensType'].value)
                except:
                    lens.append(None)
                try:
                    lens.append(self.data['Exif.NikonLd3.AFAperture'].value)
                except:
                    lens.append(None)
                try:
                    lens.append(self.data['Exif.NikonLd3.FocusPosition'].value)
                except:
                    lens.append(None)
                try:
                    lens.append(self.data['Exif.NikonLd3.FocusDistance'].value)
                except:
                    lens.append(None)
                try:
                    lens.append(self.data['Exif.NikonLd3.FocalLength'].value)
                except:
                    lens.append(None)
                try:
                    lens.append(
                        self.data['Exif.NikonLd3.EffectiveMaxAperture'].value)
                except:
                    lens.append(None)

            return lens

    def get_location(self):
        """Returns the image location used for the map viewer.

        Returns:
        tuple (float(lat), float(long)) or None
        """
        try:
            la = self.data['Exif.GPSInfo.GPSLatitude'].value
            lar = self.data['Exif.GPSInfo.GPSLatitudeRef'].value
            dla = self.dms_to_decimal(la)
            lat = dla * (1 if re.match(r'[Nn]', lar) else -1)
            lo = self.data['Exif.GPSInfo.GPSLongitude'].value
            lor = self.data['Exif.GPSInfo.GPSLongitudeRef'].value
            dlo = self.dms_to_decimal(lo)
            lon = dlo * (1 if re.match(r'[Ee]', lor) else -1)
        except:
            lat = lon = '    --'

        try:
            alt = self.data['Exif.GPSInfo.GPSAltitude'].value
            alt = round(alt.numerator / alt.denominator, 2)
            altr = self.data['Exif.GPSInfo.GPSAltitudeRef'].value
            if eval(altr):
                alt *= -1
        except:
            alt = '    --'

        return (lat, lon, alt)

    def get_subject_location(self):
        try:
            la = self.data['Exif.GPSInfo.GPSDestLatitude'].value
            lar = self.data['Exif.GPSInfo.GPSDestLatitudeRef'].value
            dla = self.dms_to_decimal(la)
            lat = dla * (1 if re.match(r'[Nn]', lar) else -1)
            lo = self.data['Exif.GPSInfo.GPSDestLongitude'].value
            lor = self.data['Exif.GPSInfo.GPSDestLongitudeRef'].value
            dlo = self.dms_to_decimal(lo)
            lon = dlo * (1 if re.match(r'[Ee]', lor) else -1)
        except:
            lat = lon = None

        # It seems there's no subject altitude
        return lat, lon

    def get_author_infos(self):
        """Returns the author's infos.

        """
        try:
            au = self.data['Exif.Image.Artist'].value
            au = self.clean_data(au)
        except:
            try:
                au = self.data['Iptc.Application2.Byline'].value[0]
            except:
                au = ""

        try:
            pr = self.data['Iptc.Application2.BylineTitle'].value[0]
        except Exception as why:
            #lgg.info('Iptc.Application2.BylineTitle {0}'.format(why))
            pr = ""

        try:
            cp = self.data['Exif.Image.Copyright'].value
            cp = self.clean_data(cp)
        except Exception as why:
            #lgg.info('Exif.Image.Copyright {0}'.format(why))
            try:
                cp = self.data['Xmp.dc.rights'].value
                cp = cp['x-default']
                # TODO support other langages
            except Exception as why:
                #lgg.info('Xmp.dc.rights {0}'.format(why))
                cp = None

        try:
            ad = self.data['Iptc.Application2.0x00d0'].value[0]
        except:
            ad = None

        try:
            vl = self.data['Iptc.Application2.0x00d1'].value[0]
        except:
            vl = None

        try:
            ep = self.data['Iptc.Application2.0x00d2'].value[0]
        except:
            ep = None

        try:
            zc = self.data['Iptc.Application2.0x00d3'].value[0]
        except:
            zc = None

        try:
            cn = self.data['Iptc.Application2.0x00d4'].value[0]
        except:
            cn = None

        try:
            tl = self.data['Iptc.Application2.0x00d5'].value[0]
        except:
            tl = None

        try:
            ml = self.data['Iptc.Application2.0x00d6'].value[0]
        except:
            ml = None

        try:
            st = self.data['Iptc.Application2.0x00d7'].value[0]
        except:
            st = ""

        return au, pr, cp, ad, vl, ep, zc, cn, tl, ml, st

    def get_contact_infos(self):
        infos = {}
        data = self.get_author_infos()
        infos['author'] = data[0]
        infos['rights'] = data[2]
        keys = ('addr', 'city', 'state', 'pocode', 'country', 'tel', 
                'mail', 'url')
        for key, dat in zip(keys, data[3:]):
            infos[key] = dat
        return infos

    def get_headline(self):
        """Returns the title of the image.

        """
        try:
            hl = self.data['Iptc.Application2.Headline'].value
            return hl[0]
        except:
            try:
                h = self.data['Xmp.photoshop.Headline'].value
                return h['x-default']
            except:
                return None

    def get_all_titles(self):
        """Returns the XMP title and the IPTC headline.

        Used instead of get_headline() by the new metadata editor.
        """
        try:
            title = self.data['Xmp.dc.title'].value
        except:
            title = ""
        try:
            hl = self.data['Iptc.Application2.Headline'].value
            hl = " ".join(hl)
        except:
            hl = ""

        return title, hl

    def get_content_infos(self):
        """Returns the IPTC content infos.

        """
        hl = self.get_headline()
        try:
            cp = self.data['Iptc.Application2.Caption'].value[0]
        except:
            try:
                c = self.data['Xmp.dc.description'].value
                cp = c['x-default']
                # TODO support other language
            except:
                cp = None

        try:
            ct = self.data['Iptc.Application2.Category'].value[0]
        except:
            try:
                c = self.data['Xmp.photoshop.Category'].value
                ct = c['x-default']
            except:
                ct = None

        try:
            wr = self.data['Iptc.Application2.Writer'].value[0]
        except:
            try:
                w = self.data['Xmp.photoshop.CaptionWriter'].value
                wr = w['x-default']
            except:
                wr = None

        return hl, cp, ct, wr

    def get_iptc_values(self, cont):
        try:
            val = self.data[cont[0]].value
            cont[1] = self.clean_data(val)
        except Exception as why:
            if "%s" % why != "'Tag not set'":
                lgg.info('Error key: %s, %s' %(cont[0], why))

    def get_xmp_values(self, cont):
        try:
            val = self.data[cont[0]].value
            if self.data[cont[0]].type == 'bag Integer':
                
                cont[1] = val
            else:
                cont[1] = self.clean_data(val)
        except Exception as why:
            if "%s" % why != "'Tag not set'":
                lgg.info('Error key: %s, %s' %(cont[0], why))

    def get_geolocation_values(self, cont):
        try:
            val = self.data[cont[0]].value
            cont[1] = val
        except Exception as why:
            if "%s" % why != "'Tag not set'":
                lgg.info('Error key: %s, %s' %(cont[0], why))

    def get_iptc_context(self):
        try:
            hl = self.data['Iptc.Application2.Headline'].value
            if hl:
                hl = hl[0]
            else:
                hl = ""
        except:
            hl = ""
        try:
            cp = self.data['Iptc.Application2.Caption'].value
            if cp:
                cp = cp[0]
            else:
                cp = ""
        except:
            cp = ""
        try:
            cw = self.data['Iptc.Application2.Writer'].value
            if cw:
                cw = cw[0]
            else:
                cw = ""
        except:
            cw = ""
        return hl, cp, cw

    def get_scene_infos(self):
        """Returns the IPTC Scene infos.

        """
        try:
            lc = self.data['Iptc.Application2.SubLocation'].value[0]
        except:
            try:
               lc = self.data['Xmp.iptc.Location'].value
            except:
                lc = None

        try:
            ct = self.data['Iptc.Application2.City'].value[0]
        except:
            try:
               ct = self.data['Xmp.photoshop.City'].value
            except:
                ct = None

        try:
            ps = self.data['Iptc.Application2.ProvinceState'].value[0]
        except:
            try:
               ps = self.data['Xmp.photoshop.State'].value
            except:
                ps = None

        try:
            cn = self.data['Iptc.Application2.CountryName'].value[0]
        except:
            try:
               cn = self.data['Xmp.photoshop.Country'].value
            except:
                cn = None

        try:
            cc = self.data['Iptc.Application2.CountryCode'].value[0]
        except:
            try:
               cc = self.data['Xmp.iptc.CountryCode'].value
            except:
                cc = None

        return lc, ct, ps, cn, cc

    def get_iptc_scene(self):
        """Returns the IPTC Scene infos.

        """
        # TODO add the XMP.Iptc4xmpExt location's tags
        # http://www.exiv2.org/tags-xmp-iptcExt.html
        scene = ["", "", "", "", ""]
        for idx, key in enumerate(['Iptc.Application2.SubLocation',
                                   'Iptc.Application2.City',
                                   'Iptc.Application2.ProvinceState',
                                   'Iptc.Application2.CountryName',
                                   'Iptc.Application2.CountryCode']):
            try:
                val = self.data[key].value
                scene[idx] = val[0]
            except:
                pass

        return scene

    def get_keywords(self):
        """Returns the keywords.

        """
        try:
            uc = self.data['Exif.Photo.UserComment'].value
            uc = self.clean_data(uc)
        except:
            uc = ""

        try:
            k = self.data['Iptc.Application2.Keywords'].value
            kw = [self.clean_data(w) for w in k]
            kw = " ".join(kw)
        except:
            kw = ""

        try:
            s = self.data['Xmp.dc.subject'].value
            sb = [self.clean_data(w) for w in s]
            sb = " ".join(sb)
        except:
            sb = ""

        return max(uc, kw, sb, key=len)

    def get_all_comments(self):
        """Return the keywords found in EXIF, IPTC and XMP.

        Used instead of get_keywords() by the new metadata editor.
        """
        try:
            uc = self.data['Exif.Photo.UserComment'].value
            uc = self.clean_data(uc)
        except:
            uc = ""

        try:
            k = self.data['Iptc.Application2.Keywords'].value
            kw = [self.clean_data(w) for w in k]
            kw = u" ".join(kw)
        except Exception as why:
            kw = ""

        try:
            s = self.data['Xmp.dc.subject'].value
            sb = [self.clean_data(w) for w in s]
            sb = u" ".join(sb)
        except:
            sb = ""

        return uc, kw, sb

    def get_xmp(self):
        """Returns Xmp data.

        """
        xmp = []
        for key in XMP:
            try:
                l = []
                v = self.data[key].value
                format = self.get_xmp_type(key)
                if format in ('Text', 'ProperName'):
                    l = [v]

                elif format == 'Lang Alt':
                    for i, val in v.items():
                        l.append(val)

                elif format in ('bag Text', 'seq ProperName'):
                    for val in v:
                        l.append(val)

                xmp.append((key, " ".join(l)))
            except Exception as why:
                if '{0}'.format(why) != "'Tag not set'":
                    lgg.info('Get xmp returns: {0}'.format(why))

        return xmp

    def get_xmp_description(self):
        try:
            return self.data['Xmp.dc.description'].value
        except Exception as why:
            if "%s" % why != "'Tag not set'":
                lgg.info("Xmp.dc.description error:", why)
            return ""

    def get_model_infos(self):
        try:
            models = self.data['Xmp.iptcExt.PersonInImage'].value
            models = u", ".join(models)
        except:
            models = ""
        try:
            age = self.data['Xmp.iptcExt.ModelAge'].value 
            age = u", ".join(age)
        except:
            age = ""
        try:
            minor = self.data['Xmp.plus.MinorModelAgeDisclosure'].value 
        except:
            minor = ""
        try:
            addl = self.data['Xmp.iptcExt.AddlModelInfo'].value
        except:
            addl = ""
        return models, age, minor, addl

    def get_artwork_infos(self):
        try:
            artw = self.data['Xmp.iptcExt.ArtworkOrObject'].value
            artw = u", ".join(artw)
        except:
            artw = ""
        try:
            rights = self.data['Xmp.iptcExt.AOCopyrightNotice'].value
        except:
            rights = u""
        return artw, rights

    def get_gettyimage_infos(self):
        values = []
        for key in ['OriginalFileName', 'ParentMEID', 'Personality',
                    'ImageRank', 'OriginalCreateDateTime', 'CameraFilename']:
            try:
                values.append(self.data['Xmp.GettyImagesGIFT.%s' % key].value)
            except:
                values.append('')

        return values

    def get_common_data(self):
        """Returns common infos.

        These infos are used by the image finder dialog.
        """
        gnrl = []
        for g in self.get_general_infos():
            if g == '    --':
                g = False
            gnrl.append(g)

        scn = self.get_scene_infos()
        loc = self.get_location()
        if loc[0] == '    --':
            loc = False
        exp = self.get_exposure_values()

        return (gnrl, scn, loc, exp)

    def modify_by_oqapy(self):
        """Change the name of processing software and update the date of 
        changes.
        
        """
        lgg.info("Change processing software")
        d = 'Exif.Image.DateTime'
        p = 'Exif.Image.ProcessingSoftware'
        soft = "Oqapy-{0}".format(VERSION_STR)
        if self.data:
            try:
                self.data[d].value = datetime.datetime.today()
            except:
                self.data[d] = pyexiv2.ExifTag(d, datetime.datetime.today())

            try:
                self.data[p].value = soft
            except:
                self.data[p] = pyexiv2.ExifTag(p, soft)
            
    def modify_size(self, size):
        """Update the size when the image has been modified.

        Args:
        size -- tuple(width, height)
        """
        lgg.info("Change image size")
        tags = "Exif.Image.ImageWidth", "Exif.Image.ImageLength"
        for tag, value in zip(tags, size):
            try:
                self.data[tag].value = value
            except:
                self.data[tag] = pyexiv2.ExifTag(tag, value)

        tags = "Exif.Photo.PixelXDimension", "Exif.Photo.PixelYDimension"
        for tag, value in zip(tags, size):
            try:
                self.data[tag].value = value
            except:
                self.data[tag] = pyexiv2.ExifTag(tag, value)

    def modify_orientation(self, orientation=1):
        """Update the orientation when the image has been rotated.

        Args:
        orientation -- 1 = 0°, 3 = 180, 6 = 90, 8 = -90
        """
        o = "Exif.Image.Orientation"
        try:
            self.data[o] = orientation
        except:
            self.data[o] = pyexiv2.ExifTag(o, orientation)

    def update_orientation(self, orientation, width, height):
        tag = "Exif.Image.Orientation"
        try:
            self.data[tag] = orientation
        except:
            self.data[tag] = pyexiv2.ExifTag(tag, orientation)

        if width:
            self.modify_size((width, height))

    def update_oqapy_data(self, change=None):
        # TODO
        pass

    def rewrite_data(self, news, save=False):
        """Apply the changes to the current image.

        Args:
        news -- news value (dict)
        save -- if True, the metadata will be write into the file.
        """
        lgg.info("Modify data, save: {0}".format(save))
        for k, v in news.items():
            if k == 100:
                # Geolocation
                self.edit_geotags(v)
                continue

            keys = EDITABLES[k]
            for key in keys:
                if not key in self.data:
                    try:
                        if key.startswith("Exif"):
                            lgg.info('Need to create key EXIF')
                            self.data[key] = pyexiv2.exif.ExifTag(key)

                        elif key.startswith("Iptc"):
                            lgg.info('Need to create key IPTC')
                            self.data[key] = pyexiv2.iptc.IptcTag(key)

                        elif 'oqapy' in key:
                            self.data[key] = pyexiv2.xmp.XmpTag(key)

                        else:
                            # FIXME xmp are not writable because crash of exiv2
                            lgg.info('XMP are not writable.')
                            continue

                    except Exception as why:
                        lgg.warning('Key {0} not created reason: {1}'
                                    .format(key, why))
                        continue

                value = self.get_tag_type(key, v)
                try:
                    self.data[key].value = value
                except Exception as why:
                    lgg.info("Key error <{0}> value '{1}' \n\tError: {2}"
                                    .format(key, value, why))

                    try:
                       self.data[key].raw_value = value
                    except Exception as why:
                        lgg.warning("Value <{0}> rejected for key {1}, "
                                    "reason:\n{2}".format(value, key, why))
                    else:
                        lgg.info('Raw value accepted for key <{0}>'
                                    .format(key))

        lgg.info("Rewrite metadata complete")
        if save:
            self.save_data()

    def set_value(self, key, val):
        """Set a new value, if the key doesn't exist, she's created.

        Args:
        key -- The key name
        val -- The new value
        """
        lgg.info('Set value %s: %s' %(key, val))
        if not key in self.data:
            try:
                if key.startswith("Exif"):
                    lgg.info('Create key EXIF')
                    self.data[key] = pyexiv2.exif.ExifTag(key)

                elif key.startswith("Iptc"):
                    lgg.info('Create key IPTC')
                    self.data[key] = pyexiv2.iptc.IptcTag(key)

                elif 'oqapy' in key:
                    self.data[key] = pyexiv2.xmp.XmpTag(key)

                else:
                    # xmp are not writable because crash of exiv2
                    lgg.info('Create key XMP')
                    self.create_xmp_tag(key)
                    #self.data[key] = pyexiv2.xmp.XmpTag(key)

            except Exception as why:
                lgg.info('Key {0} not created reason: {1}'
                            .format(key, why))
                return

        value = self.get_tag_type(key, val)
        try:
            self.data[key].value = value
        except Exception as why:
            lgg.info("Key error <{0}> value '{1}' \n\tError: {2}"
                            .format(key, value, why))
            try:
               self.data[key].raw_value = value
            except Exception as why:
                lgg.info("Value <{0}> rejected for key {1}, "
                            "reason:\n{2}".format(value, key, why))
            else:
                lgg.info('Raw value accepted for key <{0}>'
                            .format(key))

    def get_tag_type(self, key, tag):
        """Returns the value in the appropriate type of key.

        Args:
        key -- the complete key name
        tag -- the value
        """
        lgg.info("Get the type of tag <{0}>".format(key))
        if key.startswith('Exif'):
            if 'UserComment' in key:
                # No change, unicode accepted
                return tag

            t = pyexiv2.exif.ExifTag(key).type
            if t in ('Ascii', 'String', 'Byte', 'SByte'):
                return tag
                # FIXME not needed with python3-exiv2
                # Must be changed to ASCII
                return self.utils.unicode_to_string(tag)

            elif t in ('Long', 'SLong', 'Short', 'SShort'):
                # Shouldn't appears since there's no writable tag with
                # numeric value handled by Oqapy.
                if not isinstance(tag, int):
                    try:
                        return int(tag)
                    except:
                        return False
                return tag

        elif key.startswith('Iptc'):
            t = pyexiv2.iptc.IptcTag(key).type
            # Values are always into a list
            return [tag]

        if 'oqapy' in key:
            return tag

        t = pyexiv2.xmp.XmpTag(key).type
        if t in ('Text', 'ProperName', 'URL', 'bag Integer'):
            # No change
            return tag

        elif t == 'Lang Alt':
            # Change to dict
            return {'x-default': tag}

        # Change to list
        return [tag]

    def get_tag_description(self, tag):
        """Returns the name and the long description of the given tag.

        Args:
        tag -- index of tag in EDITABLES list

        Returns:
        tuple(name, description)
        """
        lgg.info("Get description for tag <{0}>".format(tag)) 
        if isinstance(tag, int):
            if tag in range(3, 11):
                # Theses tags are recents, descriptions are provided by
                # Oqapy, see data.py
                return EDITABLES_DESC[tag]

            tag = EDITABLES[tag][0]

        if tag.startswith('Exif'):
            lab = pyexiv2.exif.ExifTag(tag).label
            dsc = pyexiv2.exif.ExifTag(tag).description

        elif tag.startswith('Iptc'):
            lab = pyexiv2.iptc.IptcTag(tag).title
            dsc = pyexiv2.iptc.IptcTag(tag).description

        elif tag.startswith('Xmp'):
            if 'oqapy' in tag:
                lab = "Author of the last changes"
                dsc = "Identification of the name of the person who has "\
                      "made the last modifications of the image or import "\
                      "it from a raw file."
            else:
                lab = pyexiv2.xmp.XmpTag(tag).title
                dsc = pyexiv2.xmp.XmpTag(tag).description

        return lab, dsc

    def create_xmp_tag(self, tag):
        """Create a Xmp tag.

        Args:
        tag -- the name of the key
        """
        t = self.get_xmp_type(tag)
        lgg.info("Create Xmp tag <{0}> Type: {1}".format(tag, t))
        if t in ('Text', 'ProperName'):
            self.data[tag] = pyexiv2.xmp.XmpTag(tag, 'value')

        elif t == 'Lang Alt':
            self.data[tag] = pyexiv2.xmp.XmpTag(tag, {'x-default': 'value'})

        elif t in ('bag Text', 'seq ProperName'):
            # Xmp.iptcExt.ArtworkOrObject is always rejected, why ?
            # http://www.exiv2.org/tags-xmp-iptcExt.html
            self.data[tag] = pyexiv2.xmp.XmpTag(tag, ['value'])

        elif t == 'URL':
            self.data[tag] = pyexiv2.xmp.XmpTag(tag, 
                                        'http://ns.useplus.org/ldf/vocab/')

        elif t == 'bag Integer':
            self.data[tag] = pyexiv2.xmp.XmpTag(tag, [1])

    def get_xmp_type(self, tag):
        """Return the type of a given Xmp tag.

        Args:
        tag -- the Xmp key
        """
        return pyexiv2.xmp.XmpTag(tag).type

    def get_xmp_oqapy_tags(self):
        """Return the xmp tags registered with oqapy domain name.

        www.oqapy.eu/xmp/1.0/
        Xmp.oqapy.oqapyVersion:     2.1.0
        Xmp.oqapy.unrawDate:	    2015 06 04 03 47 26
        Xmp.oqapy.rawSourceFile:	_DSC0046.NEF
        Xmp.oqapy.colorSpace:	    sRGB D65
        Xmp.oqapy.lastEditDate:	    2015 06 04 03 47 26
        Xmp.oqapy.lastEditorName:
        Xmp.oqapy.lastChanges
        Xmp.oqapy.cleanupPasses:	0
        Xmp.oqapy.version:	        1.0
        Xmp.oqapy.imageIndex:	    0
        Xmp.oqapy.whiteBalance:	    default
        Xmp.oqapy.gamma:	        BT.709
        Xmp.oqapy.interpolation
        Xmp.oqapy.substractDarkFrame
        """
        try:
            vals = [self.data['Xmp.oqapy.version'].value]
        except KeyError:
            # There's no Xmp.oqapy tags
            return None

        tags = ['oqapyVersion', 'lastEditDate', 'lastEditorName', 'lastChanges',
                'rawSourceFile', 'unrawDate', 'whiteBalance', 'interpolation', 
                'colorSpace', 'gamma', 'cleanupPasses', 'imageIndex', 
                'substractDarkFrame']
        for tag in tags:
            try:
                vals.append(self.data['Xmp.oqapy.%s' % tag].value)
            except KeyError:
                vals.append('')

        return vals

    def edit_geotags(self, values):
        """Set the correct values for the geolocation data.

        Args:
        values -- 2-tuple or 3-tuple (latitude, longitude[, altitude])
                  if there's no geotags, the values are '  --'
        """
        try:
            lat = float(values[0])
            lon = float(values[1])
        except:
            return

        try:
            elev = float(values[2])
        except:
            elev = 0

        self.modify_geotags(lat, lon, elev)

    def modify_geotags(self, lat, lon, elev=None):
        """Change geographical data.

        If the values are provided by a map site, only latitude and
        longitude are used here.

        Args:
        lat -- float(latitude)
        lon -- float(longitude)
        elev -- float(altitude)
        """
        lgg.info("Modify geodata lat: %s lon: %s alt: %s" %(lat, lon, elev))
        latref = ("N" if lat >= 0 else "S")
        lonref = ("E" if lon >= 0 else "W")
        lat = self.decimal_to_dms(abs(lat))
        dat = [self.int_to_rational(l) for l in lat]
        tag = 'Exif.GPSInfo.GPSLatitudeRef'
        try:
            self.data[tag].value = latref
        except KeyError:
            self.data[tag] = pyexiv2.ExifTag(tag, latref)

        tag = 'Exif.GPSInfo.GPSLatitude'
        try:
            self.data[tag].value = dat
        except KeyError:
            self.data[tag] = pyexiv2.ExifTag(tag, dat)

        lon = self.decimal_to_dms(abs(lon))
        dat = [self.int_to_rational(l) for l in lon]
        tag = 'Exif.GPSInfo.GPSLongitudeRef'
        try:
            self.data[tag].value = lonref
        except KeyError:
            self.data[tag] = pyexiv2.ExifTag(tag, lonref)

        tag = 'Exif.GPSInfo.GPSLongitude'
        try:
            self.data[tag].value = dat
        except KeyError:
            self.data[tag] = pyexiv2.ExifTag(tag, dat)

        if elev is not None:
            tag = 'Exif.GPSInfo.GPSAltitudeRef'
            dat = b'1' if elev < 0 else b'0'
            try:
                self.data[tag].value = dat
            except KeyError:
                self.data[tag] = pyexiv2.ExifTag(tag, dat)

            tag = 'Exif.GPSInfo.GPSAltitude'
            dat = self.int_to_rational(elev)
            try:
                self.data[tag].value = dat
            except KeyError:
                self.data[tag] = pyexiv2.ExifTag(tag, dat)

    def modify_thumbnail(self, thumb):
        if not os.path.isfile(thumb):
            lgg.debug("Update thumbnail in metadata error: "
                            "No such file '{0}'".format(thumb))
            return

        try:
            t = self.data.exif_thumbnail
        except Exception as why:
            lgg.warning("reading thumbnail error {0}".format(why))
            return

        try:
            t.set_from_file(thumb)
        except Exception as why:
            lgg.warning("modifying thumbnail error: {0}".format(why))

    def clean_data(self, d):
        """Remove space and chr(Null) from a given string.

        Args:
        d -- data
        """
        if isinstance(d, dict):
            return d

        if isinstance(d, list):
            if not d:
                return ""
            #d = d[0]
            # Changed for the new metadata handler
            d = ', '.join(d)

        if isinstance(d, str):
            return d.replace('\x00', '').strip()

        try:
            return str(d)
        except:
            return ""

    def format_len_spec(self, spec):
        """Exif.Photo.LensSpecification 0xa432 type: Rational

        This tag notes minimum focal length, maximum focal length, minimum 
        F number in the minimum focal length, and minimum F number in the 
        maximum focal length, which are specification information for the lens 
        that was used in photography."""
        # TODO Need examples of this tag
        return None

    def save_data(self):
        """Write metadata in image file.

        """
        lgg.info("Write metadata")
        if self.data is not None:
            try:
                self.data.write()
            except Exception as why:
                lgg.warning("Write error: %s" % why)
                message = {'context': _('Save metadata'),
                    'warn': _("Write error in:"),
                    'data': self.pict,
                    'msg': _('Reason:'),
                    'data1': str(why),
                    'btn': _('Close')}
                warn = WarningMessages(0, message)
                warn.build_dialog()

            else:
                lgg.info("Metadata written")

    def export_data(self, dest):
        """Copy metadata to other image file.

        Args:
        dest -- destination file
        """
        lgg.info("Export metadata to {0}".format(dest))
        try:
            new = pyexiv2.ImageMetadata(dest)
            new.read()
        except Exception as why:
            lgg.debug("Read error: {0}".format(why))
            message = {'context': _('Read metadata'),
                'warn': _("Read error in:"),
                'data': dest,
                'msg': _('Reason:'),
                'data1': str(why),
                'btn': _('Close')}
            warn = WarningMessages(0, message)
            warn.build_dialog()
            return False

        # FIXME TIFF file don't support comment
        cm = not os.path.splitext(dest)[1] in ['.tif', '.TIF', '.tiff', '.TIFF']
        self.data.copy(new, comment=cm)
        try:
            new.write()
        except Exception as why:
            lgg.debug(u"Export error: {0}".format(why))
            message = {'context': _('Save metadata'),
                'warn': _("Write error in:"),
                'data': dest,
                'msg': _('Reason:'),
                'data1': str(why),
                'btn': _('Close')}
            warn = WarningMessages(0, message)
            warn.build_dialog()
            return False

        return True

    def decimal_to_dms(self, d):
        """Convert decimal degrees into degrees, minutes, seconds.

        """
        i, d = math.modf(d)
        j, m = math.modf(i*60)
        s = j*60
        return [int(d), int(abs(m)), abs(s)]

    def dms_to_decimal(self, frcts):
        vals = [(f.numerator / f.denominator) for f in frcts]
        return (vals[0] + (vals[1] / 60.0) + (vals[2] / 3600.0))

    def int_to_rational(self, val):
        """Convert integer to rational.
    
        """
        return Fraction(val).limit_denominator(1000)

    def get_formatted_data(self):
        """Returns metadata for the warning dialog box.

        returns:
        Formated text
        """
        if not self.exif:
            return ""

        text = ""
        
        try:
            t = self.data['Exif.Image.ImageDescription'].value
            t = t.lstrip()
            t = t.lstrip('\0x00')
            t = t.rstrip()
            text = text + t + "\n"
        except:
            pass

        try:
            text = text + _("Author:\n")
            text = text + self.data['Exif.Image.Artist'].value + "\n"
        except:
            pass
        try:
            text = text + self.data['Exif.Image.Copyright'].value + "\n"
        except:
            pass

        try:         
            t = self.data['Exif.Photo.UserComment'].value + "\n"
            t = t.lstrip()
            t = t.lstrip('\0x00')
            t = t.rstrip()
            if len(t)>1:
                text = text + _("Comments:\n")
                text = text + t + "\n"
        except:
            pass

        try:
            text = text + _("Created:\n")
            t = self.data['Exif.Photo.DateTimeOriginal'].value
            t = t.strftime('%a %d %b %Y, %H:%M')
            text = text + t + " \n"
        except:
            pass

        try:
            text = text + self.data['Exif.Image.Make'].value.lstrip()
        except:
            pass

        try:
            text = text + " " + self.data['Exif.Image.Model'].value  + "\n"
        except:
            text = text + u"\n"

        try:
            text = text + _("Digitalised:\n")
            t = self.data['Exif.Photo.DateTimeDigitized'].value
            t = t.strftime('%a %d %b %Y, %H:%M')
            text = text + t + " \n"
        except:
            pass

        try:
            text = text + _("Modified le:\n")
            t = self.data['Exif.Image.DateTime'].value
            t = t.strftime('%a %d %b %Y, %H:%M')
            text = text + t + "\n"
        except:
            pass

        try:
            text = text + self.data['Exif.Image.ProcessingSoftware'].value
            text = text + "\n"
        except:
            pass

        if 'Exif.Image.XResolution' in self.data and \
                    'Exif.Image.YResolution' in self.data and \
                    'Exif.Image.ResolutionUnit' in self.data:
            text =  text + _("Resolution: {0} x {1}".format
                    (str(self.data['Exif.Image.XResolution'].value),
                    str(self.data['Exif.Image.YResolution'].value)))
            if self.data['Exif.Image.ResolutionUnit'].value == 3:
                text = text + " dpi"
          
        return text


class GeoTag(object):
    """Class to handle the image's geolocation.

    """
    def __init__(self, img):
        """Init class GeoTag

        Args:
        img -- path of image
        """
        self.img = img
        self.obj = pyexiv2.ImageMetadata(img)
        self.obj.read()
        self.lk = self.obj.exif_keys
        self.keys_ = ['Exif.Photo.DateTimeOriginal', 'Exif.GPSInfo.GPSLatitude',
                    'Exif.GPSInfo.GPSLatitudeRef', 'Exif.GPSInfo.GPSLongitude',
                    'Exif.GPSInfo.GPSLongitudeRef','Exif.GPSInfo.GPSAltitude',
                    'Exif.GPSInfo.GPSAltitudeRef', 'Exif.Photo.UserComment']

    def get_geo_data(self):
        """Get the metadata used for geolocation.

        Returns:
        pack -- list of metadata [datetime, latitude, latitude ref,
                                    longitude, longitude ref, altitude,
                                    altitude ref, user comments]
        """
        pack = [None] * 8
        for i, k in enumerate(self.keys_[:]):
            if k in self.lk:
                pack[i] = self.obj[self.keys_[i]].value

        return pack

    def write_geo_data(self, dat):
        """Write new geo data.

        Args:
        dat -- list of data [latitude, latitude ref,
                                longitude, longitude ref, altitude,
                                altitude ref, date, time]
                All values are mandatory but may be None.

        Returns:
        False if no error, error infos otherwise
        """
        lgg.info("Write geodata")
        try:
            for idx, k in enumerate(self.keys_[1:]):
                if dat[idx] is not None:
                    self.obj[k] = dat[idx]
        except Exception as why:
            lgg.warning("Value {0} rejected reason: {1}".format(dat[idx],
                            why))
            return (k, dat[idx])

        if dat[6] is not None:
            self.obj['Exif.GPSInfo.GPSDateStamp'] = dat[6]

        if dat[7] is not None:
            self.obj['Exif.GPSInfo.GPSTimeStamp'] = dat[7]

        try:
            self.obj.write()
        except Exception as why:
            message = {'context': _('Save geodata'),
                'warn': _("Write error in:"),
                'data': self.img,
                'msg': _('Reason:'),
                'data1': str(why),
                'btn': _('Close')}
            warn = WarningMessages(0, message)
            warn.build_dialog()
            return why

        return False
    
    def valid_coords(self, lat, lon):
        """Determine the validity of coordinates."""
        if not isinstance(lat, float) or not isinstance(lon, float):
            return False
        return abs(lat) <= 90 and abs(lon) <= 180
