# -*- coding: utf-8 -*-

# metadatahandler.py
# This file is part of Oqapy serie 3
#
# Author: Vincent Vande Vyvre <vincent.vandevyvre@oqapy.eu>
# Home: http://www.oqapy.eu
# Copyright: 2012-2016 Vincent Vande Vyvre
# Licence: GPL3
#
# Metadata editor.

import os
import pickle
import logging
lgg = logging.getLogger(__name__)

from math import pow

from PyQt5 import QtCore

LANG = str(QtCore.QLocale.system().name())

from metadata.metadata import Metadata
from metadata.uimetadataeditor import Ui_MetadataEditor
from metadata.misc import COUNTRIES


class MetadataHandler:
    """Define the metadata editor.

    """
    def __init__(self, edit, image, thumbnail, sizes=None):
        """Instanciate the metadata editor.

        args:
        edit -- MetadataEditor instance
        image -- full path of the image file
        thumbnail -- full path of the thumbnail
        """
        self.edit = edit
        core = edit.core
        self.image = image
        self.sizes = sizes
        self.temp = core.workspace['temp']
        if not os.path.isfile(image):
            return

        self.ui = Ui_MetadataEditor(self)
        self.ui.show()
        QtCore.QCoreApplication.processEvents()
        self.ui.fix_splitter_size()
        self.completers_file = core.workspace['root'] / 'completers'
        self.completers = self.get_completers()
        self.ui.completers = self.completers
        self.data_ = Metadata(self.image)
        self.data_.get_data()
        self.ui.display_preview(thumbnail, self.data_.get_orientation())
        self.set_metadata()
        self.ui.ignore_signal = False

    def set_metadata(self): 
        self.on_init = True
        prefix = 'Xmp.iptc.CreatorContactInfo/Iptc4xmpCore:'
        self.data = {'iptc': {'headline': ['Iptc.Application2.Headline',
                                            '', ''],
                              'desc': ['Iptc.Application2.Caption',
                                            '', ''],
                              'writer': ['Iptc.Application2.Writer',
                                            '', ''],
                              'kwords': ['Iptc.Application2.Keywords',
                                            '', ''],
                              'place': ['Iptc.Application2.SubLocation',
                                            '', ''],
                              'city': ['Iptc.Application2.City',
                                            '', ''],
                              'state': ['Iptc.Application2.ProvinceState',
                                            '', ''],
                              'country': ['Iptc.Application2.CountryName',
                                            '', '']},
                     'xmp': {'author': ['Xmp.dc.creator', 
                                        '', ''],
                             'marked': ['Xmp.xmpRights.Marked',
                                        '', ''],
                             'rights': ['Xmp.dc.rights', 
                                        '', ''],
                             'terms': ['Xmp.xmpRights.UsageTerms',
                                        '', ''],
                             'addr': ['%sCiAdrExtadr' % prefix,
                                        '', ''],
                             'city': ['%sCiAdrCity' % prefix,
                                        '', ''],
                             'state': ['%sCiAdrRegion' % prefix,
                                        '', ''],
                             'pocode': ['%sCiAdrPcode' % prefix,
                                        '', ''],
                             'country': ['%sCiAdrCtry' % prefix,
                                        '', ''],
                             'tel': ['%sCiTelWork' % prefix,
                                        '', ''],
                             'mail': ['%sCiEmailWork' % prefix,
                                        '', ''],
                             'url': ['%sCiUrlWork' % prefix,
                                        '', ''],
                             'title': ['Xmp.dc.title',
                                        '', ''],
                             'subject': ['Xmp.dc.description',
                                        '', ''],
                             'kwords': ['Xmp.dc.subject',
                                        '', ''],
                             'people': ['Xmp.iptcExt.PersonInImage',
                                        '', ''],
                             'ages': ['Xmp.iptcExt.ModelAge',
                                        '', ''],
                             'minors': ['Xmp.plus.MinorModelAgeDisclosure',
                                        0, 0],
                             'others': ['Xmp.iptcExt.AddlModelInfo',
                                        '', ''],
                             'artworks': ['Xmp.iptcExt.ArtworkOrObject',
                                        '', ''],
                             'artrights': ['Xmp.iptcExt.AOCopyrightNotice',
                                        '', '']},
                     'oqapy': {'editor': ['Xmp.oqapy.lastEditorName',
                                        '', '']},
                     'geoloc': {'c_lat': ['Exif.GPSInfo.GPSLatitude',
                                        '', ''],
                                'c_long': ['Exif.GPSInfo.GPSLongitude',
                                        '', ''],
                                'alt':['Exif.GPSInfo.GPSAltitude',
                                        '', ''],
                                's_lat': ['Exif.GPSInfo.GPSDestLatitude',
                                        '', ''],
                                's_long': ['Exif.GPSInfo.GPSDestLongitude',
                                        '', '']}}

        for key in self.data['iptc'].keys():
            self.data_.get_iptc_values(self.data['iptc'][key])

        for key in self.data['xmp'].keys():
            self.data_.get_xmp_values(self.data['xmp'][key])

        self.data_.get_xmp_values(self.data['oqapy']['editor'])
        self.clean_data()
        self.normalise_data()
        date, uid = self.data_.get_date_and_id()
        self.ui.set_filename(os.path.basename(self.image), date)
        camera = self.data_.get_camera_infos()
        self.ui.set_camera_data(camera)
        lensdata = self.data_.get_lens_infos(camera[0])
        if 'nikon' in camera[0].lower():
            lensdata = self.format_nikon_lens_data(lensdata)

        self.ui.set_lens_data(lensdata)
        self.ui.set_exposure_settings(self.data_.get_exposure_infos())
        infs = self.data_.get_image_infos()
        if self.sizes is not None:
            # FIXME get_image_infos() return the first thumbnail size instead
            # of the image size in case is a raw file
            infs[0] = "%s x %s" % (self.sizes[0], self.sizes[1])

        self.ui.set_image_data(infs, self.data_.get_orientation(), uid)
        self.ui.set_preview_infos(self.data_.get_thumbnails_infos())
        self.ui.set_keywords(self.data['iptc']['kwords'][1],
                             self.data['xmp']['kwords'][1])
        self.ui.set_title(self.data['iptc']['headline'][1], 
                          self.data['xmp']['title'][1])

        self.ui.set_iptc_data((self.data['iptc']['headline'][1], 
                               self.data['iptc']['desc'][1], 
                               self.data['iptc']['writer'][1]),
                              (self.data['iptc']['place'][1],
                               self.data['iptc']['city'][1],
                               self.data['iptc']['state'][1],
                               self.data['iptc']['country'][1]))

        self.ui.set_xmp(self.data['xmp'])
        tags = self.data_.get_xmp_oqapy_tags()
        if tags is None:
            self.ui.set_oqapy_data(None)

        else:
            tags[3] = self.data['oqapy']['editor'][1]
            self.ui.set_oqapy_data(tags, )

        self.on_init = False

    def format_nikon_lens_data(self, values):
        if None in values:
            # Old lens without specifications
            return values

        key = ' '.join(values[1:8])
        from metadata.lens import NIKON_LENS
        try:
            lens = NIKON_LENS[key]
        except KeyError as why:
            vals = ' '.join(values)
            lgg.warning('Nikon lens error: %s\n\tValues: %s' % (why, vals))
        else:
            # Focal = round(5.0 * pow(2.0, value/24.0), 0)
            # Aperture = round(pow(2.0, value/24.0), 1)
            fmin = round(5.0 * pow(2, int(values[3])/24.0), 0)
            fmax = round(5.0 * pow(2, int(values[4])/24.0), 0)
            amin = str(pow(2, int(values[5])/24.0)).split('.')
            amax = str(pow(2, int(values[6])/24.0)).split('.')
            # The rounded value is always different of the lens spec, so we
            # prefer the formating with the first decimal
            amin = '.'.join([amin[0], amin[1][0]])
            amax = '.'.join([amax[0], amax[1][0]])
            return 'Nikon', lens, fmin, fmax, amin, amax

    def clean_data(self):
        author = self.data['xmp']['author']
        if len(author) > 4:
            if not author[1]:
                author[1] = author[4]

        if isinstance(author[1], list):
            author[1] = ", ".join(author[1])

        rights = self.data['xmp']['rights']
        if len(rights) > 4:
            if not rights[1]:
                rights[1] = rights[4]
        if isinstance(rights[1], dict):
            rights[1] = self.choose_language(rights[1])

        usage = self.data['xmp']['terms']
        if usage[1] and isinstance(usage[1], dict):
            usage[1] = self.choose_language(usage[1])

        kwords = self.data['xmp']['kwords']
        if len(kwords) > 4:
            if not kwords[1]:
                kwords[1] = kwords[4]

        title = self.data['xmp']['title']
        if title[1]:
           title[1] = self.choose_language(title[1])

        subj = self.data['xmp']['subject']
        if subj[1] and isinstance(subj[1], dict):
            subj[1] = self.choose_language(subj[1])

        art = self.data['xmp']['artworks']
        if isinstance(art[1], list):
            art[1] = ', '.join(art[1])

        ages = self.data['xmp']['ages']
        if ages[1]:
            ages[1] = ', '.join([str(i) for i in ages[1]])

        minor = self.data['xmp']['minors']
        if minor[1]:
            minor[1] = self.handle_minor_model_age(minor[1])

    def choose_language(self, dct):
        try:
            keys = dct.keys()
        except AttributeError:
            # Bad formatting
            return ''

        if LANG in keys:
            return dct[LANG]

        lng = LANG.replace('_', '-')
        if lng in keys:
            return dct[lng] 

        lng = LANG.split('_')[0]
        for k in keys:
            if lng in k:
                return dct[k]

        try:
            return dct[u'x-default']
        except KeyError:
            return dct[keys[0]]

    def handle_minor_model_age(self, tag=None, val=None):
        """Returns the value of the tag Xmp.plus.MinorModelAgeDisclosure.

        Args:
        tag -- the string of the tag
        val -- the index of the age

        Returns:
        The index of the age if the tag is provided or the string value if
        the age's index is provided.
        """
        url = 'http://ns.useplus.org/ldf/vocab/'
        ages = ('AG-UNK', 'AG-A25', 'AG-A24', 'AG-A23', 'AG-A22', 'AG-A21',
                'AG-A20', 'AG-A19', 'AG-A18', 'AG-A17', 'AG-A16', 'AG-A15',
                'AG-U14')
        if tag is not None:
            if not tag.startswith(url):
                return 0
            try:
                return ages.index(tag[32:])
            except Exception as why:
                lgg.info("Minor model age error: %s\n\tTag: %s" % (why, tag))
                return 0

        else:
            return ''.join([url, ages[val]])

    def normalise_data(self):
        """Join the data same iptc and xmp.

        """
        infos = self.data_.get_contact_infos()
        is_changed = False
        # Don't merge 'desc' and 'subject', they're not really the same things
        for k in ('author', 'rights', 'addr', 'city', 'state', 'country', 
                    'pocode', 'tel', 'mail', 'url'):
            if infos[k] == self.data['xmp'][k][1]:
                # Same value, no change
                continue

            if infos[k] is not None and not self.data['xmp'][k][1]:
                self.data['xmp'][k][1] = infos[k]
                self.data_.set_value(self.data['xmp'][k][0], infos[k])
                is_changed = True

        if is_changed:
            self.data_.save_data()

    def get_tag_description(self, domain, tag):
        label, desc =  self.data_.get_tag_description(self.data[domain][tag][0])
        return desc

    def update_data(self, domain, tag, data):
        if not self.on_init:
            lgg.info("Update: %s-%s: %s" %(domain, tag, data))
            if tag == 'ages' or isinstance(data, int):
                self.data[domain][tag][2] = data
       
            else:
                self.data[domain][tag][2] = data

            self.ui.apply_btn.setEnabled(True)

    def on_close(self):
        lgg.info('Close metadata editor')
        self.edit.wdgt.expand_btn.setEnabled(True)

    def apply_data_changes(self):
        lgg.info('Apply new metadata')
        self.check_values()
        for domain in ('iptc', 'xmp'):
            for key in self.data[domain].keys():
                tag = self.data[domain][key]
                if tag[2]:
                    if tag[1] != tag[2]:
                        self.data_.set_value(tag[0], tag[2])
                        self.update_completers(key, tag[2])

        dat = self.data['oqapy']['editor']
        if dat[2] and dat[2] != dat[1]:
            self.data_.set_value(dat[0], dat[2])

        self.data_.save_data()
        self._save_completers()
        self.edit.reload_data(self.data_)

    def check_values(self):
        tag = self.data['xmp']['minors']
        if tag[1] != tag[2]:
            tag[2] = self.handle_minor_model_age(None, tag[2])

    def cancel_changes(self):
        self.set_metadata()

    def update_completers(self, tag, value):
        lgg.info('Update completer: %s, %s' %(tag, value))
        if tag == 'url':
            tag = 'site'

        if tag not in self.completers:
            return

        if tag == 'rights':
            value = value.replace('\n', ' ')
            for i in value.split(' '):
                if len(i) > 3:
                    self.completers['copy'].add(i)
                    return

        if value not in self.completers[tag]:
            self.completers[tag].append(value)

    def get_completers(self):
        try:
            with open(self.completers_file.path, 'rb') as inf:
                completers = pickle.load(inf)
            for key in ('editor', 'addr', 'city', 'state', 'pocode', 'tel'):
                if not key in completers:
                    # New in 2.2
                    completers[key] = []
            if completers['copy'] and not isinstance(completers['copy'], set):
                # Change 2.3, the rights editor is now a QTexEdit
                words = []
                for str_ in completers['copy']:
                    items = str_.split(' ')
                    words.extend([i for i in items if len(i) > 3])
                completers['copy'] = set(words)
        except Exception as why:
            lgg.info('Get completer error: %s, use default' % why)
            completers = {'author': [], 'prof': [], 'copy': [], 'mail': [], 
                          'site': [], 'kw': [], 'editor': [], 'addr': [],
                          'city': [], 'state': [], 'pocode': [], 'tel': []}
            self._save_completers(completers)

        return completers

    def _save_completers(self, data=None):
        if data is None:
            data = self.completers

        try:
            with open(self.completers_file.path, 'wb') as objf:
                pickle.dump(data, objf)
        except Exception as why:
            lgg.warning("Unable to save completers: %s" % why)

