# -*- coding: utf-8 -*-

# uimetadataeditor.py
# This file is part of Oqapy serie 3
#
# Author: Vincent Vande Vyvre <vincent.vandevyvre@oqapy.eu>
# Home: http://www.oqapy.eu
# Copyright: 2012-2017 Vincent Vande Vyvre
# Licence: GPL3
#
# Metadata editor dialog box.

import sys
import logging
lgg = logging.getLogger(__name__)

from PyQt5.QtCore import Qt
from PyQt5.QtGui import (QFont, QIcon, QPixmap, QImage, QTransform)
from PyQt5.QtWidgets import (QDialog, QGridLayout, QSplitter, QWidget, 
                             QVBoxLayout, QGraphicsView, QGraphicsScene, 
                             QHBoxLayout, QLabel, QSpacerItem, QSizePolicy, 
                             QStackedWidget, QPushButton, QCompleter, 
                             QTreeWidgetItem, QApplication, QListWidget,
                             QListWidgetItem, QLineEdit, QItemDelegate)

from metadata.camerapage import CameraPage
from metadata.shotsettings import ShotSettings
from metadata.imagepage import ImagePage
from metadata.iptcpage import IPTCPage
from metadata.xmppage import XMPPage
from tools.rectsize import RectSize
from metadata.misc import COUNTRIES
import stylesheets as style

class Ui_MetadataEditor(QDialog):
    def __init__(self, main):
        super().__init__()
        self.main = main
        self.resize(680, 488)
        self.setSizeGripEnabled(True)
        self.gridLayout = QGridLayout(self)
        self.gridLayout.setContentsMargins(2, 2, 2, 12)
        self.splitter = QSplitter(self)
        self.splitter.setOrientation(Qt.Horizontal)
        self.leftWidget = QWidget(self.splitter)
        self.vlayout = QVBoxLayout(self.leftWidget)
        self.vlayout.setContentsMargins(0, 0, 0, 0)
        self.preview = QGraphicsView(self)
        self.preview.setVerticalScrollBarPolicy(Qt.ScrollBarAlwaysOff)
        self.preview.setHorizontalScrollBarPolicy(Qt.ScrollBarAlwaysOff)
        self.scene = QGraphicsScene()
        self.preview.setScene(self.scene)
        self.vlayout.addWidget(self.preview)
        self.pagelist = PageList(self)
        self.vlayout.addWidget(self.pagelist)
        self.widget = QWidget(self.splitter)
        self.verticalLayout = QVBoxLayout(self.widget)
        self.verticalLayout.setContentsMargins(0, 0, 0, 0)
        self.horizontalLayout = QHBoxLayout()
        font = QFont()
        font.setPointSize(11)
        font.setBold(True)
        font.setItalic(True)
        font.setWeight(75)
        self.filename_lbl = QLabel(self.widget)
        self.filename_lbl.setFont(font)
        self.filename_lbl.setText("File name")
        self.horizontalLayout.addWidget(self.filename_lbl)
        spacerItem2 = QSpacerItem(40, 20, QSizePolicy.Expanding, 
                                        QSizePolicy.Minimum)
        self.horizontalLayout.addItem(spacerItem2)
        self.date_lbl = QLabel(self.widget)
        self.date_lbl.setFont(font)
        self.date_lbl.setText("Creation date")
        self.horizontalLayout.addWidget(self.date_lbl)
        spacerItem3 = QSpacerItem(10, 20, QSizePolicy.Fixed, 
                                        QSizePolicy.Minimum)
        self.horizontalLayout.addItem(spacerItem3)
        self.verticalLayout.addLayout(self.horizontalLayout)
        self.stackedWidget = QStackedWidget(self.widget)
        self.page = CameraPage(self)
        self.stackedWidget.addWidget(self.page)
        self.page_1 = ShotSettings(self)
        self.stackedWidget.addWidget(self.page_1)
        self.page_2 = ImagePage(self)
        self.stackedWidget.addWidget(self.page_2)
        self.page_3 = IPTCPage(self)
        self.stackedWidget.addWidget(self.page_3)
        self.page_4 = XMPPage(self)
        self.stackedWidget.addWidget(self.page_4)
        #self.page_5 = G9nPage(self)
        #self.stackedWidget.addWidget(self.page_5)
        self.verticalLayout.addWidget(self.stackedWidget)
        self.horizontalLayout_2 = QHBoxLayout()
        spacerItem1 = QSpacerItem(40, 20, QSizePolicy.Expanding, 
                                        QSizePolicy.Minimum)
        self.horizontalLayout_2.addItem(spacerItem1)
        self.cancel_btn = QPushButton(self.widget)
        icon = QIcon()
        icon.addPixmap(QPixmap("medias/edit-undo.png"), 
                                        QIcon.Normal, QIcon.Off)
        self.cancel_btn.setIcon(icon)
        self.horizontalLayout_2.addWidget(self.cancel_btn)
        self.apply_btn = QPushButton(self.widget)
        icon1 = QIcon()
        icon1.addPixmap(QPixmap("medias/apply.png"), 
                                        QIcon.Normal, QIcon.Off)
        self.apply_btn.setIcon(icon1)
        self.apply_btn.setObjectName("apply_btn")
        self.horizontalLayout_2.addWidget(self.apply_btn)
        self.close_btn = QPushButton(self.widget)
        icon2 = QIcon()
        icon2.addPixmap(QPixmap("medias/close.png"), 
                                        QIcon.Normal, QIcon.Off)
        self.close_btn.setIcon(icon2)
        self.horizontalLayout_2.addWidget(self.close_btn)
        self.verticalLayout.addLayout(self.horizontalLayout_2)
        self.gridLayout.addWidget(self.splitter, 0, 0, 1, 1)

        self.retranslateUi()
        self.cancel_btn.setEnabled(False)
        self.apply_btn.setEnabled(False)
        self.cancel_btn.clicked.connect(self.on_cancel_clicked)
        self.apply_btn.clicked.connect(self.on_apply_clicked)
        self.close_btn.clicked.connect(self.on_close_clicked)
        self.apply = False
        self.page_4.contact_copy_btn.setFocus(False)
        self.set_style()
        self.current_line_edit = None
        self.contact_editors = []
        self.completers = None
        self.ignore_signal = True
        self.preview.resizeEvent = self.on_viewer_size_changed
        self.set_connections()

    def fix_splitter_size(self):
        size = sum(self.splitter.sizes())
        left = size / 10.0 * 2
        right = size - left
        self.splitter.setSizes([left, right])
        self.preview.setMaximumSize(1000, left)
        self.pagelist.set_items()

    def display_preview(self, path, orientation):
        self.preview_path = path
        self.scene.clear()
        gsize = RectSize(self.preview.size())
        qimage = QImage(path)
        if qimage.isNull():
            lgg.info('Unreadable thumbnail: %s' % path)
            return

        # Thumbnails are already oriented ?
        #if orientation != 1:
            #matrix = QTransform()
            #angles = {3: 180, 6: 90, 8: -90}
            #qimage = qimage.transformed(matrix.rotate(angles[orientation]), 
                                        #Qt.SmoothTransformation)

        isize = RectSize(qimage.size())
        if isize.is_highest(gsize.height) or isize.is_widest(gsize.width):
            pix = QPixmap.fromImage(qimage.scaled(gsize.width-2, 
                                          gsize.height-2,
                                          Qt.KeepAspectRatio, 
                                          Qt.SmoothTransformation))

        else:
            pix = QPixmap.fromImage(qimage)

        self.scene.setSceneRect(0, 0, pix.width(), pix.height())
        self.scene.addPixmap(pix)

    def on_viewer_size_changed(self, e):
        if self.ignore_signal:
            # startup case
            return

        self.display_preview(self.preview_path, 1)

    def set_filename(self, name, date):
        self.filename_lbl.setText(name)
        self.date_lbl.setText(date)

    def display_page(self, idx):
        self.stackedWidget.setCurrentIndex(idx)

    def set_camera_data(self, camera):
        cp = self.page
        items = [cp.maker_value_lbl, cp.model_value_lbl, cp.firmware_value_lbl,
                 cp.serial_value_lbl, cp.owner_value_lbl]
        for label, value in zip(items, camera):
            label.setText(value)

    def set_lens_data(self, values):
        cp = self.page
        alignment = Qt.AlignRight|Qt.AlignTrailing|Qt.AlignVCenter
        if values is None:
            lbl = QLabel(cp)
            lbl.setAlignment(alignment)
            lbl.setText('Constructor:')
            cp.gridLayout_2.addWidget(lbl, 0, 0, 1, 1)
            lbl = QLabel(cp)
            lbl.setText('Undefined')
            cp.gridLayout_2.addWidget(lbl, 0, 1, 1, 1)
            return
           
        if values[1] == 'common':
            values.pop(0)

        if values[0] == 'common':
            values.pop(0)
            items = ['Constructor:', 'Model:', 'Specifications:', 
                     'Maximum aperture:', 'Serial number:']

        elif values[0] == 'Canon':
            # The short and long focale are already into the model name
            values.pop(2)
            items = ['Constructor:', 'Model:', 'Focal length:', 
                     'Serial number:']

        elif values[0] == 'Nikon':
            self.set_nikon_lens_data(values)
            return

        row = 0
        for label, value in zip(items, values):
            lbl = QLabel(cp)
            lbl.setAlignment(alignment)
            lbl.setText(label)
            cp.gridLayout_2.addWidget(lbl, row, 0, 1, 1)
            if value is None:
                value = '    --'

            lbl = QLabel(cp)
            lbl.setText(value)
            cp.gridLayout_2.addWidget(lbl, row, 1, 1, 1)
            row += 1

    def set_nikon_lens_data(self, values):
        if len(values[1]) == 2:
            items = ['Constructor:', 'Model:']

        else:
            items = ['Constructor:', 'Model:', 'Or:', 'Model:']

        cp = self.page
        alignment = Qt.AlignRight|Qt.AlignTrailing|Qt.AlignVCenter
        row = 0
        for label, value in zip(items, values[1]):
            lbl = QLabel(cp)
            lbl.setAlignment(alignment)
            lbl.setText(label)
            cp.gridLayout_2.addWidget(lbl, row, 0, 1, 1)
            lbl = QLabel(cp)
            lbl.setText(value)
            cp.gridLayout_2.addWidget(lbl, row, 1, 1, 1)
            row += 1

        items = ['Short focal:', 'Long Focal:', 'Aperture at short focal:',
                'Aperture at long focal:']
        for label, value in zip(items, values[2:]):
            lbl = QLabel(cp)
            lbl.setAlignment(alignment)
            lbl.setText(label)
            cp.gridLayout_2.addWidget(lbl, row, 0, 1, 1)
            lbl = QLabel(cp)
            lbl.setText(str(value))
            cp.gridLayout_2.addWidget(lbl, row, 1, 1, 1)
            row += 1  

    def set_exposure_settings(self, values):
        ep = self.page_1
        items = [ep.iso_value_lbl, ep.shutter_value_lbl, ep.fnumber_value_lbl,
                 ep.correction_value_lbl, ep.focal_value_lbl,
                 ep.flash_value_lbl, ep.program_value_lbl, ep.measure_value_lbl,
                 ep.exposure_value_lbl, ep.whitebalance_value_lbl,
                 ep.lightsource_value_lbl, ep.subjectdis_value_lbl,
                 ep.contrast_value_lbl, ep.saturation_value_lbl,
                 ep.sharpness_value_lbl]
        for label, value in zip(items, values):
            if not value or value is None:
                value = '    --'
            label.setText(value)

    def set_image_data(self, data, orientation, uid):
        self.page_2.size_value_lbl.setText(data[0])
        self.page_2.resolution_value_lbl.setText(data[1])
        self.page_2.colorspace_value_lbl.setText(data[2])
        self.page_2.orientation_value_lbl.setText(str(orientation))
        self.page_2.uid_value_lbl.setText(uid)

    def set_preview_infos(self, previews):
        count = 0
        layout = self.page_2.gridLayout_2
        while 1:
            item = layout.takeAt(0)
            if not item:
                break

            item.widget().deleteLater()

        if not previews:
            label = QLabel("No previews", self.page_2)
            layout.addWidget(label, count, 0, 1, 1)
            return

        alignment = Qt.AlignRight|Qt.AlignTrailing|Qt.AlignVCenter
        while previews:
            prev = previews.pop(0)
            label = QLabel("Type:", self.page_2)
            label.setAlignment(alignment)
            layout.addWidget(label, count, 0, 1, 1)
            label = QLabel(self.page_2)
            label.setText(prev[0])
            layout.addWidget(label, count, 1, 1, 1)
            count += 1
            label = QLabel("Size:", self.page_2)
            label.setAlignment(alignment)
            layout.addWidget(label, count, 0, 1, 1)
            label = QLabel(self.page_2)
            label.setText(prev[1])
            layout.addWidget(label, count, 1, 1, 1)            
            count += 1
            label = QLabel("Dimensions:", self.page_2)
            label.setAlignment(alignment)
            layout.addWidget(label, count, 0, 1, 1)
            label = QLabel(self.page_2)
            label.setText(prev[2])
            layout.addWidget(label, count, 1, 1, 1)            
            count += 1

    def clear_all(self):
        gl = self.page.gridLayout_2
        for c in range(gl.count()):
            it = gl.itemAt(c)
            it.widget().deleteLater()

        self.page_3.desc_ted.clear()
        self.page_4.rights_cmb.setCurrentIndex(0)
        self.page_4.rights_cmb.setEnabled(True)
        self.page_4.usage_ted.clear()
        self.page_4.rights_ted.clear()
        self.page_4.author_contact_trw.clear()
        self.page_3.headline_led.clear()
        self.page_3.desc_ted.clear()
        self.page_3.writer_led.clear()
        self.page_3.keywords_ted.clear()
        self.page_4.subject_ted.clear()
        self.page_4.keywords_ted.clear()
        self.page_4.names_ted.clear()
        self.page_4.ages_led.clear()
        self.page_4.artwork_desc_ted.clear()
        self.page_4.artwork_copyright_ted.clear()
        self.page_4.getty_tbl.clear()
        self.page_4.oqapy_tbl.clear()

    def set_keywords(self, iptc, xmp):
        self.page_3.keywords_ted.appendPlainText(iptc)
        self.page_4.keywords_ted.appendPlainText(xmp)

    def set_title(self, headline, title):
        self.page_2.title_value_lbl.setText(headline)
        self.page_4.title_led.setText(title)

    def set_iptc_data(self, data, scene):
        ip = self.page_3
        ip.headline_led.setText(data[0])
        ip.desc_ted.appendPlainText(data[1])
        ip.writer_led.setText(data[2])
        completer = QCompleter(self.completers['author'])
        completer.setCaseSensitivity(Qt.CaseInsensitive)
        ip.writer_led.setCompleter(completer)
        ip.place_led.setText(scene[0])
        ip.city_led.setText(scene[1])
        ip.state_led.setText(scene[2])
        ip.country_led.setText(scene[3])
        completer = QCompleter(COUNTRIES)
        completer.setCaseSensitivity(Qt.CaseInsensitive)
        ip.country_led.setCompleter(completer)

    def set_xmp(self, xmp):
        xp = self.page_4
        self.author_name = xmp['author'][1]
        if xmp['marked'][1]:
            xp.rights_cmb.setCurrentIndex(1)
            xp.rights_cmb.setEnabled(False)
            xp.author_led.hide()
            xp.author_value_lbl.setText(xmp['author'][1])
            xp.author_value_lbl.show()
            xp.rights_ted.setReadOnly(True)

        else:
            xp.author_value_lbl.hide()
            xp.author_led.show()
            xp.author_led.setText(xmp['author'][1])
            completer = QCompleter(self.completers['author'])
            completer.setCaseSensitivity(Qt.CaseInsensitive)
            xp.author_led.setCompleter(completer)

        xp.rights_ted.setPlainText(xmp['rights'][1])
        # completer for copyright is a set
        xp.rights_ted.setCompleter(list(self.completers['copy']))
        xp.usage_ted.appendPlainText(xmp['terms'][1])
        self.contact_editors = []
        labels = ['Address', 'City', 'State', 'PO code', 'Country',
                  'Telephone', 'Mail', 'URL']
        keys = ['addr', 'city', 'state', 'pocode', 'country', 'tel', 'mail',
                'url']

        for label, key in zip(labels, keys):
            twi = QTreeWidgetItem(xp.author_contact_trw)
            twi.setText(0, label)
            dedit = DataEditor(self, key, xmp[key][1], self.on_data_changed)
            xp.author_contact_trw.setItemWidget(twi, 1, dedit)
            if key != 'country':
                if key == 'url':
                    key = 'site'
                dedit.set_completer(self.completers[key])
            else:
                dedit.set_completer(COUNTRIES)
            self.contact_editors.append(dedit)

        xp.subject_ted.appendPlainText(xmp['subject'][1])
        xp.names_ted.appendPlainText(xmp['people'][1])
        xp.ages_led.setText(xmp['ages'][1])
        xp.minor_ages_cmb.setCurrentIndex(xmp['minors'][1])
        xp.add_infos_ted.appendPlainText(xmp['others'][1])
        xp.artwork_desc_ted.appendPlainText(xmp['artworks'][1])
        xp.artwork_copyright_ted.appendPlainText(xmp['artrights'][1])

    def copy_contact_infos(self):
        infos = [str(i.text()) for i in self.contact_editors]
        txt = "\n".join([i for i in infos if i])
        if self.author_name:
            txt = "%s\n%s" %(self.author_name, txt)
        clip = QApplication.clipboard()
        clip.setText(txt)

    def set_gettyimage_data(self, data):
        labels = ['Original file name:', 'Parent MEID:', 'Personality:',
                  'Image rank:', 'Creation date:', 'Camera file name:']
        for idx, label in enumerate(labels):
            twi = QTreeWidgetItem(self.page_4.getty_tbl)
            twi.setText(0, label)
            twi.setText(1, data[idx])

    def set_oqapy_data(self, xmp):
        """'oqapyVersion', 'lastEditDate', 'lastEditorName', 'lastChanges',
                'rawSourceFile', 'unrawDate', 'whiteBalance', 'interpolation', 
                'colorSpace', 'gamma', 'cleanupPasses', 'imageIndex', 
                'substractDarkFrame'"""
        if xmp is None:
            # FIXME
            #self.page_4.oqapy_tbl.setEnabled(False)
            return

        # Don't use the namespace version
        data = xmp[1:]
        labels = ['Oqapy version:', 'Edition date:']
        for label in labels:
            twi = QTreeWidgetItem(self.page_4.oqapy_tbl)
            twi.setText(0, label)
            twi.setText(1, data.pop(0))
        twi = QTreeWidgetItem(self.page_4.oqapy_tbl)
        twi.setText(0, 'Editor name:')
        dedit = DataEditor(self, 2, data.pop(0), self.on_editor_changed)
        dedit.set_completer(self.completers['author'])
        self.page_4.oqapy_tbl.setItemWidget(twi, 1, dedit)
        labels = ['Last changes:', 'Raw source file:', 'Unraw date:',
                  'White balance:', 'Interpolation:', 'Color space:',
                  'Gamma curve:', 'Cleanup pass:', 'Image index:',
                  'Dark frame file:']
        for label in labels:
            twi = QTreeWidgetItem(self.page_4.oqapy_tbl)
            twi.setText(0, label)
            twi.setText(1, data.pop(0))

    def set_camera_location(self, loc):
        self.page_5.set_dec_latitude(loc[0])
        self.page_5.set_dec_longitude(loc[1])
        self.page_5.set_elevation(loc[2])

    def set_subject_location(self, loc):
        self.page_5.set_dec_latitude_2(loc[0])
        self.page_5.set_dec_longitude_2(loc[1])

    def set_connections(self):
        self.page_3.connect_signals()
        self.page_4.connect_signals()

    def on_data_changed(self, ed, data):
        self.main.update_data('xmp', ed.idx, data)

    def on_editor_changed(self, ed, data):
        self.main.update_data('oqapy', 'editor', data)

    def display_description(self, context):
        if context == 2:
            txt = "The name of the last editor."
        else:
            txt = "Set the contact informations for the image."
        self.page_4.show_description(txt)

    def on_cancel_clicked(self):
        lgg.info("Metadata editor cancel changes")
        self.apply = False
        self.clear_all()
        self.main.cancel_changes()
        self.cancel_btn.setEnabled(False)

    def on_apply_clicked(self):
        lgg.info("Metadata editor apply changes")
        self.apply = True
        self.cancel_btn.setEnabled(True)
        self.apply_btn.setEnabled(False)

    def on_close_clicked(self):
        self.closeEvent(None)

    def closeEvent(self, event):
        lgg.info("Metadata editor close, apply changes: %s" % self.apply)
        if self.apply:
            self.main.apply_data_changes()
        self.main.on_close()
        self.done(1)

    def retranslateUi(self):
        self.setWindowTitle(_("Metadata Editor - Oqapy"))
        self.cancel_btn.setText(_("Cancel"))
        self.apply_btn.setText(_("Apply"))
        self.close_btn.setText(_("Close"))

    def set_style(self):
        self.preview.setStyleSheet("QWidget { background-color: #242424}")
        self.pagelist.setStyleSheet("""QWidget 
                                    {background-color: qlineargradient(
                                    x1: 0, y1: 0, x2: 0, y2: 1,
                                    stop: 0 #F4F4F4, stop: 0.4 #E8E8E8,
                                    stop: 0.5 #E8E8E8, stop: 1.0 #F4F4F4);
                                    border: 2px solid #8A8E96;
                                    border-radius: 4px;
                                    color: #000000;}""")
        self.filename_lbl.setStyleSheet(style.SUBTITLE)
        self.date_lbl.setStyleSheet(style.SUBTITLE)


class PageList(QListWidget):
    def __init__(self, parent=None):
        super().__init__(parent)
        self.ui = parent
        self.setSelectionMode(self.SingleSelection)
        self.currentRowChanged.connect(self.on_item_changed)

    def set_items(self):
        pages = [_("Camera"), _("Settings"), _("Image"),
                 _("IPTC"), _("XMP")]
        for page in pages:
            item = QListWidgetItem(page, self)

        self.setCurrentRow(0)

    def on_item_changed(self, row):
        self.ui.display_page(row)
            
        
class DataEditor(QLineEdit):
    def __init__(self, med, idx, txt=None, slot=None, parent=None):
        """Instanciate a QLineEdit delegate for the treeWidget.

        Args:
        med -- MetadataEditor instance
        idx -- index
        txt -- text
        slot -- function called when editing
        parent -- parent
        """
        super().__init__(parent)
        self.med = med
        self.idx = idx
        self.slot = slot

        if txt is not None:
            self.setText(txt)
            if len(txt) > 30:
                self.setToolTip(txt)

        #self.textChanged.connect(self.on_editing)
        self.editingFinished.connect(self.on_editing_finished)

    @staticmethod
    def paintForDelegate(delegate, painter, option, index):
        QItemDelegate.paint(delegate, painter, option, index)

    def set_completer(self, words):
        """Set the completer.

        """
        completer = QCompleter(words)
        completer.setCaseSensitivity(Qt.CaseInsensitive)
        self.setCompleter(completer)

    def get_text(self):
        return str(self.text())

    def on_editing(self, text):
        if len(text) > 20:
            self.setToolTip(text)

        self.slot(self, str(text).strip())

    def on_editing_finished(self):
        self.on_editing(self.text())

    def focusInEvent(self, e):
        self.med.display_description(self.idx)
        QLineEdit.focusInEvent(self, e)

    def mousePressEvent(self, e):
        if e.button() == 1:
            self.med.current_line_edit = (self, self.idx)
        QLineEdit.mousePressEvent(self, e)

if __name__ == "__main__":
    app = QApplication(sys.argv)
    ui = Ui_MetadataEditor(None)
    ui.show()
    QtCore.QCoreApplication.processEvents()
    ui.fix_splitter_size()
    sys.exit(app.exec_())

