# -*- coding: utf-8 -*-

# metadataeditor.py
# This file is part of Oqapy serie 3
#
# Author: Vincent Vande Vyvre <vincent.vandevyvre@oqapy.eu>
# Home: http://www.oqapy.eu
# Copyright: 2012-2017 Vincent Vande Vyvre
# Licence: GPL3
#
# Dialog box for metadata


# TODO set completer for keywords


import os
import pickle
import logging
lgg = logging.getLogger(__name__)

from PyQt5.QtCore import QObject, Qt, QTimer
from PyQt5.QtWidgets import (QTreeWidgetItem, QLineEdit, QItemDelegate, QDialog, 
                             QGridLayout, QVBoxLayout, QHBoxLayout, QLabel, 
                             QSpacerItem, QSizePolicy, QPlainTextEdit, 
                             QCheckBox, QToolButton, QPushButton, QCompleter, 
                             QWidget, QFrame,)


from metadata.misc import COUNTRIES_ABBR_2, COUNTRIES
from utils import Utils
from metadata.metadata import Metadata
from metadata.metadatahandler import MetadataHandler
from metadata.multicloningdata import MultiCloningData


class MetadataEditor(QObject):
    def __init__(self, core):
        """Instanciate the metadata editor.

        Metadata editor is part of propertie's panel.

        Args:
        core -- Oqapy main class
        """
        super().__init__()
        self.core = core
        # Properties panel
        self.wdgt = core.ui.toolpanel.data_view
        self.editor = core.ui.toolpanel.data_view.data_tree
        self.utils = Utils(self)
        self.tags_changed = {}
        self.current_line_edit = False
        self.show_description = True
        self.clipboard_state = False
        self.clipboard_model = 'Empty'
        self.current_image = None
        self.items = []
        self.get_expandeds()
        self.model_file = self.core.workspace['root'] / 'metadataModel'
        self.model = self.get_model()
        self.comp_file = self.core.workspace['root'] / 'completers'
        self.get_completers()
        panel = core.ui.toolpanel
        panel.clone_infos_btn.pressed.connect(self.copy_data_to_clipboard)
        panel.paste_infos_btn.setEnabled(False)
        panel.paste_infos_btn.pressed.connect(self.clone_data)
        self.wdgt.apply_btn.setEnabled(False)
        self.wdgt.apply_btn.clicked.connect(self.apply_new_data)
        self.wdgt.expand_btn.setEnabled(False)
        self.wdgt.expand_btn.clicked.connect(self.expand_editor)
        self.editor.itemCollapsed.connect(self.update_expanded)
        self.editor.itemExpanded.connect(self.update_expanded)

    def clear_editor(self):
        """Clear the QListWidget.

        """
        self.editor.clear()
        self.wdgt.expand_btn.setEnabled(False)

    def get_expandeds(self):
        """Get the expanded state of item in the user config.

        """
        try:
            self.expandeds = self.core.cfg.get(('dataeditor', 'expandeds'))
        except KeyError:
            self.expandeds = [True, False, False, False, False, False, True,
                                True, True, True]
            self.core.cfg.update(('dataeditor', 'expandeds'), self.expandeds)

    def update_expanded(self, item):
        """Update the expanded state for an item.

        args:
        item -- the listWidgetItem instance
        """
        try:
            idx = self.items.index(item)
        except:
            return

        self.expandeds[idx] = item.isExpanded()
        # FIXME the list is referenced
        #self.core.cfg.update(('dataeditor', 'expandeds'), self.expandeds)

    def set_data(self, data):
        """Populate the metadata's treeWidget.

        Args:
        data -- the metadata instance of the current view
        """
        self.data = data
        self.editor.blockSignals(True)
        self.clear_editor()
        self.items = []

        if data is None:
            lgg.info("Image {0} hasn't metadata".format(self.current_image))
            return

        lgg.info("Set data for {0}".format(self.current_image))

        # General infos
        inf = self.data.get_general_infos()
        twi = QTreeWidgetItem(self.editor)
        twi.setText(0, _(" General"))
        labels = [_("Title"), _("Date"), _("Author"), _("Copyright"),
                    _("Comment"), _("Unique Identifier")]

        for idx, l in enumerate(labels):
            child = QTreeWidgetItem(twi)
            child.setText(0, l)
            child.setText(1, inf[idx])
            if len(inf[idx]) > 25:
                child.setToolTip(1,inf[idx])

        self.items.append(twi)

        self.editor.expandItem(self.items[0])

        # Image infos
        inf = self.data.get_image_infos()
        twi = QTreeWidgetItem(self.editor)
        twi.setText(0, _(" Image"))
        child = QTreeWidgetItem(twi)
        child.setText(0, _("Size"))
        child.setText(1, inf[0])
        child = QTreeWidgetItem(twi)
        child.setText(0, _("Resolution"))
        child.setText(1, inf[1])
        child = QTreeWidgetItem(twi)
        child.setText(0, _("Color space"))
        child.setText(1, inf[2])

        if inf[3]:
            child = QTreeWidgetItem(twi)
            child.setText(0, _("Scene type"))
            child.setText(1, inf[3])

        if inf[4]:
            child = QTreeWidgetItem(twi)
            child.setText(0, _("Software"))
            child.setText(1, inf[4])

        if inf[5]:
            child = QTreeWidgetItem(twi)
            child.setText(0, _("Description"))
            child.setText(1, inf[5])

            if len(inf[5]) > 25:
                child.setToolTip(1, inf[5])

        self.items.append(twi)
        #self.editor.expandItem(self.items[1])

        # Exposure infos
        inf = self.data.get_exposure_infos()
        twi = QTreeWidgetItem(self.editor)
        twi.setText(0, _(" EXIF - Exposure"))
        labels = [_("ISO"), _("Exposure time"), _("FNumber"), 
                    _("Exposure bias"), _("Focal length"), _("Flash"),
                    _("Exposure program"), _("Metering mode"), 
                    _("Exposure mode")]

        for idx, l in enumerate(labels):
            child = QTreeWidgetItem(twi)
            child.setText(0, l)
            child.setText(1, inf[idx])

            if len(inf[idx]) > 25:
                child.setToolTip(1, inf[idx])

        labels = [_("White balance"), _("Light source"), _("Subject distance"), 
                    _("Contrast"), _("Saturation"), _("Sharpness")]

        for idx, l in enumerate(labels):
            if inf[idx+9]:
                child = QTreeWidgetItem(twi)
                child.setText(0, l)
                child.setText(1, inf[idx+9])

                if len(inf[idx+9]) > 25:
                    child.setToolTip(1, inf[idx+9])

        #self.editor.expandItem(twi)

        # Thumbnail(s) infos
        inf = self.data.get_thumbnails_infos()
        twi = QTreeWidgetItem(self.editor)
        twi.setText(0, _(" EXIF - Thumbnails"))
        labels = [_("Type mime"), _("Size"),
                    _("Dimensions")]

        for idx, t in enumerate(inf):
            order = "   {0}".format(str(idx + 1))
            child = QTreeWidgetItem(twi)
            child.setText(0, order)

            for i in range(3):
                subchild = QTreeWidgetItem(child)
                subchild.setText(0, labels[i])
                subchild.setText(1, t[i])

        self.items.append(twi)
        #self.editor.expandItem(twi)

        # Camera infos
        inf = self.data.get_camera_infos()
        twi = QTreeWidgetItem(self.editor)
        twi.setText(0, _(" EXIF - Camera"))
        labels = [_("Constructor"), _("Model"), _("Firmware"), 
                    _("Serial number"), _("Owner")]

        for idx, l in enumerate(labels):
            child = QTreeWidgetItem(twi)
            child.setText(0, l)
            child.setText(1, inf[idx])

            if len(inf[idx]) > 25:
                child.setToolTip(1, inf[idx])

        self.items.append(twi)
        self.editor.expandItem(twi)

        # Lens infos
        inf = self.data.get_lens_infos()
        twi = QTreeWidgetItem(self.editor)
        twi.setText(0, _(" EXIF - Lens"))
        labels = [_("Constructor"), _("Model"), "Spec.",
                    _("Aperture max."), _("Serial number")]

        for idx, l in enumerate(labels):
            if inf[idx] is not None:
                child = QTreeWidgetItem(twi)
                child.setText(0, l)
                child.setText(1, inf[idx])

        # Geolocation
        inf = self.data.get_location()
        twi = QTreeWidgetItem(self.editor)
        twi.setText(0, _(" EXIF - Geolocation"))
        self.geoloc_items = []

        if any(inf):
            labels = [_("Latitude"), _("Longitude"), _("Altitude")]

            for idx, l in enumerate(labels):
                if inf[idx] is not None:
                    child = QTreeWidgetItem(twi)
                    child.setText(0, l)
                    child.setText(1, str(inf[idx]))
                    self.geoloc_items.append(child)

        self.items.append(twi)

        # Editables
        # Author QLineEdit indexed 0 ~ 19
        self.data_editors = []
        inf = self.data.get_author_infos()
        twi = QTreeWidgetItem(self.editor)
        twi.setText(0, _(" IPTC - Author"))
        labels = [_("Author"), _("Title"), _("Copyright")]
        for idx, l in enumerate(labels):
            child = QTreeWidgetItem(twi)
            child.setText(0, l)
            dedit = DataEditor(self, idx, inf[idx], self.on_data_changed)
            self.editor.setItemWidget(child, 1, dedit)
            self.data_editors.append(dedit)

        labels = [_("Address"), _("City"), _("State"), _("Zip code"),
                    _("Country"), _("Telephone"), _("Mail address"), 
                    _("Site web")]

        for idx, l in enumerate(labels):
            child = QTreeWidgetItem(twi)
            child.setText(0, l)
            dedit = DataEditor(self, idx+3, inf[idx+3], self.on_data_changed)
            self.editor.setItemWidget(child, 1, dedit)
            self.data_editors.append(dedit)
            if idx == 4:
                dedit.set_completer(COUNTRIES)

        for comp in [(0, 'author'), (1, 'prof'), (2, 'copy'), 
                        (9, 'mail'), (10, 'site')]:
            self.data_editors[comp[0]].set_completer(self.completers[comp[1]])

        self.items.append(twi)

        # Content QLineEdit indexed 20 ~ 29
        inf = self.data.get_content_infos()
        twi = QTreeWidgetItem(self.editor)
        twi.setText(0, _(" IPTC - Content"))
        labels = [_('Title'), _('Description'),
                    _('Category'), _('Description writer')]

        for idx, l in enumerate(labels):
            child = QTreeWidgetItem(twi)
            child.setText(0, l)
            dedit = DataEditor(self, idx+20, inf[idx], self.on_data_changed)
            self.editor.setItemWidget(child, 1, dedit)
            self.data_editors.append(dedit)

        self.items.append(twi)

        # Scene QLineEdit indexed 30 ~ 39
        inf = self.data.get_scene_infos()
        twi = QTreeWidgetItem(self.editor)
        twi.setText(0, _(" IPTC - Scene"))
        labels = [_('Place'), _('City'), _('State'), _('Country'),
                    _('ISO country code')]
        for idx, l in enumerate(labels):
            child = QTreeWidgetItem(twi)
            child.setText(0, l)
            dedit = DataEditor(self, idx+30, inf[idx], self.on_data_changed)
            self.editor.setItemWidget(child, 1, dedit)
            self.data_editors.append(dedit)

            if idx == 3:
                dedit.set_completer(COUNTRIES)

            elif idx == 4:
                self.country_code_edit = dedit

        self.data_editors.pop()
        self.items.append(twi)     

        #IPTC-Status
        # Keywords QLineEdit indexed 50
        inf = self.data.get_keywords()
        twi = QTreeWidgetItem(self.editor)
        twi.setText(0, _(" IPTC - Keywords"))
        child = QTreeWidgetItem(twi)
        child.setText(0, _('Keywords'))
        dedit = DataEditor(self, 50, inf, self.on_data_changed)
        self.editor.setItemWidget(child, 1, dedit)
        self.data_editors.append(dedit)

        self.items.append(twi)

        #XMP
        inf = self.data.get_xmp()
        twi = QTreeWidgetItem(self.editor)
        twi.setText(0, _(" XMP"))
        for i in inf:
            name = i[0].split('.')[-1]
            child = QTreeWidgetItem(twi)
            child.setText(0, name)
            child.setText(1, i[1])

            if len(i[1]) > 25:
                child.setToolTip(1, i[1])

        self.items.append(twi)
        lgg.info("Metadata completed")
        self.set_expanded()
        self.editor.blockSignals(False)
        self.wdgt.expand_btn.setEnabled(True)

    def reload_data(self, data):
        self.set_data(self.data)

    def update_data(self, txt):
        """Update the current lineEdit when the expanded lineEdit is used.

        Args:
        txt -- text
        """
        lgg.warning('update_data is obsolete')
        #self.current_line_edit[0].setText(txt)

    def on_data_changed(self, ed, val):
        """Store the new data in a dictionnary and set enabled the 'Apply'
        button.

        Args:
        ed -- lineEdit instance
        val -- new text
        """
        self.tags_changed[ed.idx] = val
        self.wdgt.apply_btn.setEnabled(True)
        self.wdgt.expand_btn.setEnabled(True)

        if ed.idx == 33:
            # Update ISO country code
            self.add_country_code(val)

    def copy_data_to_clipboard(self):
        """Call the metadata's clipboard dialog.

        """
        lgg.info("Metadata clipboard called")
        self.settings = None
        clone = CloneMetadataDialog(self)
        clone.set_origin(self.current_image)
        clone.set_values(None)
        reply = clone.exec_()

        if reply:
            self._update_model()

    def clone_data(self):
        """Copy the data from the model to the current view.

        """
        lgg.info("Clone data from model")
        if self.core.multiviewing:
            self.clone_data_to_multifiles()
            return

        change = 0
        if self.model is not None:
            for edit in self.data_editors:
                if self.model[edit.idx]:
                    edit.setText(self.model[edit.idx])
                    change = 1

        self.wdgt.apply_btn.setEnabled(change)

    def clone_data_to_multifiles(self):
        """Copy the data from the model to the selected images.

        """
        news = dict([(k, v) for k, v in self.model.items() if v])
        print(news)
        if not news:
            return

        files = self.get_file_names()
        if not files:
            return

        origin = self.core.current_picture
        cloning = MultiCloningData(self, files, self.model, origin)
        ret = cloning.ui.exec_()
        if ret:
            for f in files:
                self.copy_from_model(f, news)

    def get_file_names(self):
        """Return the list of selected files into the multiviewers.

        Only the jpeg and tiff are returned
        """
        paths = self.core.multiviewing.selected
        if self.core.multiviewing.is_raw_files:
            # Assume all RAW image formats have metadata
            return paths

        valids = []
        for path in paths:
            if self.utils.valid_image(path) in ("jpg", "tiff"):
                valids.append(path)

        return valids

    def apply_new_data(self):
        """Apply the new data to the current image.

        """
        lgg.info("Apply changes to current image")
        self.data.rewrite_data(self.tags_changed, save=True)
        self.wdgt.apply_btn.setEnabled(False)
        self._update_completers()
        self.tags_changed = {}
        self.set_data(self.data)

    def copy_from_model(self, fname, changes):
        """Copy the new data to an image.

        Args:
        fname -- path of the image file
        changes -- the new data as dict
        """
        if not os.path.isfile(fname):
            return

        data = Metadata(fname)
        if data.get_data():
            data.rewrite_data(changes, save=True)

    def expand_editor(self, led):
        """call the expanded lineEdit.

        """
        self.editor.clearSelection()
        self.wdgt.expand_btn.setEnabled(False)
        self.display_edit_dialog()

    def display_edit_dialog(self):
        lgg.info("Show metadata dialog")
        fld = self.core.source_fld.thumbs_fld
        if self.core.state.context == 5:
            # RAW files edition
            ci = self.current_image
            try:
                sizes = self.core.rawviewer.current_preview.unraw_settings[
                                                                "out_size"]
            except:
                sizes = None

        else:
            ci = self.core.current_picture.uri
            sizes = None

        self.current_image = ci
        th = self.utils.get_thumb_name(ci, fld.path)
        handler = MetadataHandler(self, ci, th, sizes)

    def add_country_code(self, country):
        """Set the ISO country code when country has been changed.

        Args:
        country -- country
        """
        try:
            idx = COUNTRIES.index(country)
            self.country_code_edit.setText(COUNTRIES_ABBR_2[idx])
        except:
            self.country_code_edit.setText("")

    def _update_model(self):
        """Create and save a metadata model used for metadata's cloning.

        """
        self.model = {}
        for de in self.data_editors:
            if self.settings[de.idx]:
                self.model[de.idx] = str(de.text())
            else:
                self.model[de.idx] = False

        self.core.ui.toolpanel.paste_infos_btn.setEnabled(True)
        self.save_model

    def save_model(self):
        try:
            with open(self.model_file.path, 'wb') as objf:
                pickle.dump(self.model, objf)
            lgg.info("Model of metadata saved")
        except Exception as why:
            lgg.debug("Unable to save model of metadata.\nReason: {0}"
                            .format(why))

    def model_is_empty(self):
        for k, v in self.model.items():
            if v:
                return False

        return True

    def get_model(self):
        """Load the metadata's model.

        """
        if not self.model_file.exists:
            lgg.info("No metadata model found")
            return

        try:
            with open(self.model_file.path, 'r') as objf:
                return pickle.load(objf)
        except Exception as why:
            lgg.debug("Unable to load model of metadata.\nReason: {0}"
                            .format(why))

    def _update_completers(self):
        name = ['author', 'prof', 'copy', 'mail', 'site']
        indexes = [0, 1, 2, 9, 10]
        for k in self.tags_changed:
            try:
                i = indexes.index(k)
            except:
                continue

            if self.tags_changed[k] not in self.completers[name[i]]:
                self.completers[name[i]].append(self.tags_changed[k])

        self._save_completers()

    def get_completers(self):
        try:
            with open(self.comp_file.path, 'rb') as inf:
                self.completers = pickle.load(inf)
        except:
            self.completers = {'author': [], 'prof': [], 'copy': [], 'mail': [], 
                        'site': [], 'kw': []}
            self._save_completers()

    def _save_completers(self):
        try:
            with open(self.comp_file.path, 'wb') as objf:
                pickle.dump(self.completers, objf)
                lgg.info("Completers saved")
        except Exception as why:
            lgg.debug("Unable to save completers.\n\t\tReason: {0}"
                            .format(why))

    def set_expanded(self):
        for idx, val in enumerate(self.expandeds):
            if val:
                self.editor.expandItem(self.items[idx])


class DataEditor(QLineEdit):
    def __init__(self, med, idx, txt=None, slot=None, parent=None):
        """Instanciate a QLineEdit delegate for the treeWidget.

        Args:
        med -- MetadataEditor instance
        idx -- index
        txt -- text
        slot -- function called when editing
        parent -- parent
        """
        super().__init__(parent)
        self.med = med
        self.wdgt = med.wdgt
        self.idx = idx
        self.slot = slot

        if txt is not None:
            self.setText(txt)
            if len(txt) > 20:
                self.setToolTip(txt)

        self.textChanged.connect(self.on_editing)
        self.editingFinished.connect(self.on_editing_finished)
        self.setStyleSheet("""
                QLineEdit {border-top: 1px solid #000000;
                    border-right: 1px solid #282E36;
                    border-bottom: 1px solid #282E36;
                    border-left: 1px solid #282E36;
                    background: #282E36;
                    color: #FFFFFF;
                    selection-background-color: #A0A0A0;}""")

    @staticmethod
    def paintForDelegate(delegate, painter, option, index):
        QItemDelegate.paint(delegate, painter, option, index)

    def set_completer(self, words):
        """Set the countries completer.

        """
        completer = QCompleter(words)
        completer.setCaseSensitivity(Qt.CaseInsensitive)
        self.setCompleter(completer)

    def get_text(self):
        return str(self.text())

    def on_editing(self, text):
        if len(text) > 20:
            self.setToolTip(text)

        self.slot(self, str(text))

    def on_editing_finished(self):
        # If the user click on the 'Expand' button, the signal 'editingFinished'
        # is emitted first. So, we need to temporise it.
        timer = QTimer()
        timer.singleShot(300, self.update_button)
        timer.start()

    def mousePressEvent(self, e):
        if e.button() == 1:
            self.wdgt.expand_btn.setEnabled(True)
            self.med.current_line_edit = (self, self.idx)

    def update_button(self):
        if self.med.current_line_edit == self.idx:
            self.wdgt.expand_btn.setEnabled(False)
            self.med.current_line_edit = False

class ExpandedLineEdit(QDialog):
    def __init__(self, med, edit, title, desc=None, parent=None):
        """Instanciate the expanded lineEdit.

        Args:
        med -- instance of MetadataEditor
        val -- current text
        title -- the label of the metadata
        desc -- long description
        """
        super().__init__(parent)
        self.med = med
        self.edit = edit
        self.completer = None
        self.completers = self.med.completers
        text = edit.text()
        self.setWindowTitle(_("Metadata Editor - Oqapy"))
        self.setModal(True)
        self.resize(437, 100)
        self.setFocusPolicy(Qt.StrongFocus)
        self.setSizeGripEnabled(True)
        self.gridLayout = QGridLayout(self)
        self.gridLayout.setContentsMargins(2, 2, 2, 2)
        self.gridLayout.setVerticalSpacing(0)
        self.verticalLayout = QVBoxLayout()
        self.verticalLayout.setSpacing(2)
        self.horizontalLayout = QHBoxLayout()
        self.name_lbl = QLabel(self)
        self.name_lbl.setText(title)
        self.horizontalLayout.addWidget(self.name_lbl)
        spacerItem = QSpacerItem(40, 20, QSizePolicy.Expanding, 
                                                    QSizePolicy.Minimum)
        self.horizontalLayout.addItem(spacerItem)
        self.verticalLayout.addLayout(self.horizontalLayout)

        if edit.idx in [21, 50]:
            self.edit_led = QPlainTextEdit(self)
            if text is not None:
                self.edit_led.insertPlainText(text)

        else:
            self.edit_led = QLineEdit(self)
            if text is not None:
                self.edit_led.setText(text)

        self.edit_led.textChanged.connect(self.on_editing)
        self.verticalLayout.addWidget(self.edit_led)
        self.horizontalLayout_2 = QHBoxLayout()
        self.desc_chb = QCheckBox(self)
        self.desc_chb.setText(_("Show description"))
        self.horizontalLayout_2.addWidget(self.desc_chb)
        spacerItem1 = QSpacerItem(40, 20, QSizePolicy.Expanding, 
                                                    QSizePolicy.Minimum)
        self.horizontalLayout_2.addItem(spacerItem1)
        self.ok_btn = QToolButton(self)
        self.ok_btn.setText(_("Close"))
        self.horizontalLayout_2.addWidget(self.ok_btn)
        self.verticalLayout.addLayout(self.horizontalLayout_2)
        self.gridLayout.addLayout(self.verticalLayout, 0, 0, 1, 1)
        self.ext = Extension(desc, self)
        self.setExtension(self.ext)
        self.setOrientation(Qt.Vertical)

        self.ok_btn.clicked.connect(self.close_)
        self.edit_led.textChanged.connect(self.on_editing)
        self.desc_chb.stateChanged.connect(self.show_description)

        if self.med.show_description:
            self.desc_chb.setChecked(True)

        edindex = [0, 1, 2, 9, 10]
        if title.startswith('Country'):
            self.set_country_completer()
        
        elif edit.idx in edindex:
            idx = edindex.index(edit.idx)
            name = ['author', 'prof', 'copy', 'mail', 'site']
            self.completer = name[idx]
            self.set_completer()

    def set_country_completer(self):
        completer = QCompleter(COUNTRIES)
        completer.setCaseSensitivity(Qt.CaseInsensitive)
        self.edit_led.setCompleter(completer)

    def set_completer(self):
        completer = QCompleter(self.completers[self.completer])
        completer.setCaseSensitivity(Qt.CaseInsensitive)
        self.edit_led.setCompleter(completer)        

    def on_editing(self, txt=None):
        if txt is None:
            txt = self.get_text()
        self.med.update_data(txt)

    def show_description(self, state):
        self.showExtension(state)
        self.med.show_description = state

    def get_text(self):
        t = self.edit_led.toPlainText()
        return t.replace('\n', ' ')

    def closeEvent(self, e):
        self.close_()

    def close_(self):
        if self.completer is not None:
            text = str(self.edit_led.text())
            if text and text not in self.completers[self.completer]:
                self.completers[self.completer].append(text)

        self.accept()


class Extension(QWidget):
    def __init__(self, txt, parent=None):
        """Instanciate the long description extension for the expanded lineEdit.

        Args:
        txt -- text
        """
        super().__init__(parent)
        self.layout = QVBoxLayout(self)
        self.layout.setContentsMargins(0, 0, 0, 0)
        self.desc_edit = QPlainTextEdit(self)
        self.layout.addWidget(self.desc_edit)

        if txt is None:
            txt = 'No description'

        self.desc_edit.setPlainText(txt)


class CloneMetadataDialog(QDialog):
    _items = [(0, 'Image Author'), (1, "Author's Title"), (2, 'Copyright'),
            (3, 'Address'), (4, 'City'), (5, 'State'), (6, 'Zip Code'),
            (7, 'Country'), (8, 'Telephone'), (9, 'Mail Address'),
            (10, 'Site Web'), (20, 'Content Title'),
            (21, 'Content Description'), (22, 'Content Category'),
            (23, 'Content Description Writer'), (30, 'Scene Place'),
            (31, 'Scene City'), (32, 'Scene State'), (33, 'Scene Country'),
            (50, 'Keywords')]
    def __init__(self, med, parent=None):
        """Instanciate the dialog to copy the metadata into a model.

        Args:
        med -- MetadataEditor instance
        """
        super().__init__(parent)
        self.med = med
        self.setWindowTitle(_("Metadata's Clipboard - Oqapy"))
        self.resize(360, 167)
        self.setSizeGripEnabled(True)
        self.setModal(True)
        self.gridLayout_2 = QGridLayout(self)
        self.gridLayout_2.setContentsMargins(4, 4, 4, -1)
        self.gridLayout_2.setHorizontalSpacing(4)
        self.verticalLayout_2 = QVBoxLayout()
        self.horizontalLayout = QHBoxLayout()
        self.label = QLabel(self)
        self.label.setText(_("Copy from: "))
        self.horizontalLayout.addWidget(self.label)
        self.origin_lbl = QLabel(self)
        self.horizontalLayout.addWidget(self.origin_lbl)
        spacerItem = QSpacerItem(138, 20, QSizePolicy.Expanding, 
                                                    QSizePolicy.Minimum)
        self.horizontalLayout.addItem(spacerItem)
        self.verticalLayout_2.addLayout(self.horizontalLayout)
        self.frame = QFrame(self)
        self.frame.setFrameShape(QFrame.StyledPanel)
        self.frame.setFrameShadow(QFrame.Raised)
        self.verticalLayout = QVBoxLayout(self.frame)
        self.verticalLayout.setContentsMargins(2, 2, 2, 2)

        self.gridLayout = QGridLayout()
        self.checkBoxes = []
 
        for idx, item, in enumerate(self._items):
            spacer1 = QSpacerItem(40, 20, QSizePolicy.Fixed, 
                                                    QSizePolicy.Minimum)
            self.gridLayout.addItem(spacer1, idx, 0, 1, 1)
            checkBox = QCheckBox(self.frame)
            checkBox.setText(item[1])
            self.checkBoxes.append(checkBox)
            self.gridLayout.addWidget(checkBox, idx, 1, 1, 1)
            spacer2 = QSpacerItem(178, 20, QSizePolicy.Expanding, 
                                                    QSizePolicy.Minimum)
            self.gridLayout.addItem(spacer2, idx, 2, 1, 1)

        self.verticalLayout.addLayout(self.gridLayout)
        self.verticalLayout_2.addWidget(self.frame)

        self.horizontalLayout_2 = QHBoxLayout()
        spacerItem5 = QSpacerItem(40, 20, QSizePolicy.Expanding,
                                                    QSizePolicy.Minimum)
        self.horizontalLayout_2.addItem(spacerItem5)
        self.cancel_btn = QPushButton(self)
        self.cancel_btn.setText(_("Cancel"))
        self.horizontalLayout_2.addWidget(self.cancel_btn)
        self.close_btn = QPushButton(self)
        self.close_btn.setText(_("Copy"))
        self.horizontalLayout_2.addWidget(self.close_btn)
        self.verticalLayout_2.addLayout(self.horizontalLayout_2)
        self.gridLayout_2.addLayout(self.verticalLayout_2, 0, 0, 1, 1)

        self.cancel_btn.clicked.connect(self.reject)
        self.close_btn.clicked.connect(self.on_close_clicked)

    def set_origin(self, txt):
        self.origin_lbl.setText(txt)

    def set_values(self, val):
        if val is None:
            for b in self.checkBoxes:
                b.setChecked(True)

        else:
            del val['name']
            for k, v in val.iteritems():
                self.checkBoxes[k].setChecked(v)

    def on_close_clicked(self):
        settings = {}
        for idx, box in enumerate(self.checkBoxes):
            settings[self._items[idx][0]] = box.isChecked()

        self.med.settings = settings
        self.accept()
