# -*- coding: utf-8 -*-

# multiviews.py
# This file is part of Oqapy serie 3
#
# Author: Vincent Vande Vyvre <vincent.vandevyvre@oqapy.eu>
# Home: http://www.oqapy.eu
# Copyright: 2018 Vincent Vande Vyvre
# Licence: GPL3

# Multi views handling

import os
import sys
import glob
import logging
lgg = logging.getLogger(__name__)

from fractions import Fraction

from PyQt5.QtWidgets import (QApplication, QWidget, QVBoxLayout, QHBoxLayout,
                             QGraphicsView, QSpacerItem, QSizePolicy, QLabel, 
                             QGraphicsProxyWidget, QGraphicsScene, QToolButton,
                             QCheckBox, QMenu, QAction)
from PyQt5.QtCore import (QSize, Qt, QTimer, QCoreApplication, QRectF, QPoint,
                          pyqtSignal)
from PyQt5.QtGui import (QImage, QPixmap, QCursor, QPen, QColor, QPainter,
                         QBrush, QTransform, QIcon)

from tools.rectsize import RectSize
import stylesheets as styles
from metadata.metadata import Metadata
from picture import Picture


class MultiViews:
    def __init__(self, core, cfg):
        self.core = core
        self.ui = core.ui
        self.header = core.ui.stackedviews.multiviews_header
        self.on_resize = -1
        self.current_index = 0
        self.lock_selection = False
        self.is_raw_files = False
        self.selected = []
        self.qimages = []
        self.viewer_items = []
        # instance of Picture which is diplayed into the info's tool boxes
        self.current_picture = None
        self.configure(cfg)

    def configure(self, cfg):
        """Fix the parameters of the multiviewing mode.

        Args:
        cfg -- dictionnary
            {"images": the ordered list of the image files from source folder
             "config": the number of rows and columns
             "idx": the index of the current picture
             "graphics": instance of the QGraphicsView
             "trailer": instance of the trailers handler}
        """
        self.pictures = cfg["images"]
        self.config = cfg["config"]
        self.current_index = cfg["idx"]
        self.viewer = cfg["graphics"]
        self.trailer = cfg["trailer"]
        self.read_only = cfg["read_only"]
        self.raw_previews = cfg["raw_views"]
        self.get_reasonable_size()
        if self.raw_previews:
            self.set_preview_list()

    def set_preview_list(self):
        self.raws_items = self.pictures[:]
        self.pictures = []
        for item in self.raws_items:
            self.pictures.append(item.preview)

    def build_views(self):
        """Build the items of the multiview window.

        """
        # TODO Display a buzzy cursor
        # TODO If image are read-only dont display a checkBox
        self.set_buzzy_widget(True)
        view_w, view_h, item_w, item_h = self.get_sizes()
        numb = self.config["rows"] * self.config["columns"]
        if self.current_index + numb >= len(self.pictures):
            self.current_index = len(self.pictures) - numb
        # Free memory
        self.clean_view()
        self.qimages = self.get_qimages(numb)
        for i in range(numb):
            path = self.pictures[self.current_index + i]
            self.viewer_items.append(ViewerItem(self, i, path, item_w, item_h))

        self.install_items()

    def install_items(self):
        """Display the items into the view.

        """
        top, left = 0, 0
        h_number = self.config["columns"]
        layout = self.ui.stackedviews.multiviews.layout
        for idx, item in enumerate(self.viewer_items):
            layout.addWidget(item, top, left, 1, 1)
            left += 1
            if not (idx + 1) % h_number:
                top += 1
                left = 0

        QCoreApplication.processEvents()
        for i in self.viewer_items:
            i.display_image()

        self.on_resize = 0
        self.update_file_name()
        self.set_buzzy_widget(False)

    def set_config(self, config):
        """Set the layout parameters.

        Args:
        config -- dictionnary
            {"rows": number of rows
             "columns": number of columns}
        """
        for k in config:
            if self.config[k] != config[k]:
                self.update_views(config)

    def get_current_picture(self):
        path = self.pictures[self.current_index]
        return path, self.current_index

    def update_views(self, views):
        """Update the layout of the window.

        Called whenever the layout has changed.
        """
        n_views = views["rows"] * views["columns"]
        numb = self.config["rows"] * self.config["columns"]
        self.config = views
        if n_views > numb:
            news = n_views - numb
            self.add_views(news, numb)

        elif n_views < numb:
            self.remove_views(n_views)

        self.resize_items()

    def add_views(self, views, last):
        """Add some views into the layout.

        Args:
        views -- number of the new items
        last -- number of the items currently into the window
        """
        self.set_buzzy_widget(True)
        index = self.current_index + last
        # Check if there's sufficient images
        if index + views <  len(self.pictures):
            for i in range(views):
                path = self.pictures[index+i]
                self.qimages.append(self.get_qimage(path))
                idx = len(self.qimages) - 1
                self.viewer_items.append(ViewerItem(self, idx, path, 0, 0))

        if index + views >=  len(self.pictures):
            self.current_index = len(self.pictures) - (views + last)
            self.qimages = self.get_qimages(views + last)
            self.viewer_items = []
            for i in range(views + last):
                path = self.pictures[self.current_index+i]
                self.viewer_items.append(ViewerItem(self, i, path, 0, 0))

        self.clean_layout()
        self.install_items()
            
    def remove_views(self, views):
        """Remove some items from the list of QImages.

        Args:
        views -- number of items to keep into the list
        """
        self.set_buzzy_widget(True)
        self.clean_layout()
        while len(self.viewer_items) > views:
            w = self.viewer_items.pop()
            w.deleteLater()

        self.qimages = self.qimages[:views]
        self.install_items()

    def remove_selection(self):
        items = self.selected
        if items:
            for idx in reversed(items):
                print(idx, self.pictures[idx])
                #self.picture.pop(idx)
                #self.build_views()

    def move_to_left(self):
        if self.current_index:
            self.set_buzzy_widget(True)
            new = self.current_index - 1
            numb = self.config["rows"] * self.config["columns"]
            self.qimages.pop()
            end = new + numb
            qi = self.get_qimage(self.pictures[new])
            if not qi.isNull():
                self.qimages.insert(0, qi)

            self.shift_images(new)
            self.current_index = new
            self.update_file_name()

        self.core.update_multiviews_navigation(self.current_index,
                                               len(self.pictures))
        self.set_buzzy_widget(False)

    def move_to_right(self):
        new = self.current_index + 1
        numb = self.config["rows"] * self.config["columns"]
        end = new + numb
        if end <= len(self.pictures):
            self.set_buzzy_widget(True)
            self.qimages.pop(0)
            qi = self.get_qimage(self.pictures[end-1])
            if not qi.isNull():
                self.qimages.append(qi)

            self.shift_images(new)
            self.current_index = new
            self.update_file_name()

        self.core.update_multiviews_navigation(end, len(self.pictures))
        self.set_buzzy_widget(False)

    def set_current(self, idx):
        numb = self.config["rows"] * self.config["columns"]
        if idx + numb >= len(self.pictures):
            idx = len(self.pictures) - numb

        self.current_index = idx
        self.build_views()

    def shift_images(self, start):
        for idx, img in enumerate(self.qimages):
            item = self.viewer_items[idx]
            item.change_image(self.pictures[start+idx], img)

        self.trailer.set_current_item(start)
        self.update_check_boxes()

    def reset_layout(self):
        # Called whenever the number of rows or columns changed
        r, c = self.header.get_config()
        self.update_views({"rows": r, "columns": c})
        self.header.configure_multiviewing(self.config)

    def update_check_boxes(self):
        if not self.selected:
            return

        self.lock_selection = True
        for i in self.viewer_items:
            i.check_chb.setChecked(False)
            if i.picture in self.selected:
                i.check_chb.setChecked(True)

        self.lock_selection = False

    def resize_items(self):
        #print(self.viewer_items[0].size(), self.viewer_items[-1].size())
        view_w, view_h, item_w, item_h = self.get_sizes()
        #print(view_w, view_h, item_w, item_h)
        for i in self.viewer_items:
            i.setMaximumSize(item_w, item_h)
            i.resize(item_w-2, item_h-2)

        QCoreApplication.processEvents()
        # Need to separate these operations to avoid "staccato" effect
        for i in self.viewer_items:
            i.resize_picture()
        #print(self.viewer_items[0].size(), self.viewer_items[-1].size())

    def on_resize_event(self, event):
        if self.on_resize == -1:
            return

        self.on_resize += 1
        timer = QTimer.singleShot(200, self.resizing)

    def resizing(self):
        if self.on_resize:
            self.on_resize -= 1

            if not self.on_resize:
                self.resize_items()

    def clean_view(self):
        """Destroy all view items.

        Needed because the items are double referenced.
        """
        lgg.info("Delete viewer items")
        self.qimages = []
        layout = self.ui.stackedviews.multiviews.layout
        while self.viewer_items:
            widget = self.viewer_items.pop(0)
            layout.removeWidget(widget)
            widget.deleteLater()

        QCoreApplication.processEvents()

    def clean_layout(self):
        layout = self.ui.stackedviews.multiviews.layout
        for i in self.viewer_items:
            layout.removeWidget(i)

    def get_qimages(self, number):
        qimages = []
        num = self.current_index + number
        for p in self.pictures[self.current_index:num]:
            qi = self.get_qimage(p)
            if qi.isNull():
                continue

            qimages.append(qi)

        return qimages

    def get_qimage(self, path):
        return QImage(path)

    def get_sizes(self):
        v_width = self.viewer.width()
        v_height = self.viewer.height()
        it_width = int(v_width / self.config["columns"]) - 1
        it_height = int(v_height / self.config["rows"]) - 1
        return v_width, v_height, it_width, it_height

    def get_reasonable_size(self):
        """Define the maximal values for rows and columns.

        Since the multiviewing is not intended to replace the thumbnail trailer,
        the minimal size of items displayed into the multiviewer is fixed at
        300 x 300 pixels.
        """
        rows_max = int(self.viewer.height() / 300)
        columns_max = int(self.viewer.width() / 300)
        if self.config["rows"] > rows_max:
            self.config["rows"] = rows_max

        if self.config["columns"] > columns_max:
            self.config["columns"] = columns_max

        self.header.configure_multiviewing(self.config)
        self.header.set_maxima(rows_max, columns_max)

    def update_file_name(self):
        name = ""
        if self.viewer_items:
            item = self.viewer_items[0]
            date = item.data["frmt_date"]
            name = "%s  -  (%s)" %(os.path.basename(item.picture), date)

        self.header.update_title(name)

    def show_in_main_viewer(self, index):
        idx, img, name = self.find_picture(index)
        lgg.info("Main view request for image: %s" % name)
        self.current_index = idx
        QCoreApplication.processEvents()
        self.core.close_multiviewing(update=True)

    def select_image(self, index, checked):
        if self.lock_selection:
            return

        #idx, img, name = self.find_picture(index)
        if checked:
            self.selected.append(index)

        else:
            self.selected.remove(index)

        print('Selected: %s' % index)
        self.header.update_selection(len(self.selected))
        self.ui.toolbar.activate_trash_button(len(self.selected))
        self.selected.sort()

    def show_metadata(self, index):
        idx, img, name = self.find_picture(index)
        lgg.info("Display metadata for %s" % name)
        self.current_picture = Picture(self.core, img, idx)
        self.core.metadataeditor.set_data(self.current_picture.metadata)
        self.ui.toolpanel.update_with_picture(self.current_picture)

    def clear_selection(self):
        self.lock_selection = True
        for item in self.viewer_items:
            item.check_chb.setChecked(False)
            item.is_selected = False

        self.selected = []
        self.lock_selection = False
        self.header.update_selection(0)

    def get_images_selected(self):
        images = []
        for i in self.selected:
            images.append(self.find_picture(i))

        return images

    def find_picture(self, index):
        idx = self.current_index + index
        img = self.pictures[idx]
        name = os.path.basename(img)
        return idx, img, name

    def set_buzzy_widget(self, state):
        cursors = (Qt.ArrowCursor, Qt.BusyCursor)
        self.viewer.setCursor(cursors[state])
        QCoreApplication.processEvents()

class ViewerItem(QWidget):
    def __init__(self, driver, idx, pict, width, height):
        super().__init__()
        self.driver = driver
        self.index = idx
        self.picture = pict
        self.width_ = width
        self.height_ = height
        self.setMaximumSize(width, height)
        self.data = None
        self.layout = QVBoxLayout(self)
        self.layout.setContentsMargins(0, 0, 0, 0)
        self.layout.setSpacing(0)
        self.header = QWidget(self)
        self.header.setMaximumSize(QSize(2222, 18))
        name = os.path.basename(pict)
        self.hlayout = QHBoxLayout(self.header)
        self.hlayout.setContentsMargins(4, 0, 2, 0)
        self.filename_lbl = QLabel(name, self.header)
        self.hlayout.addWidget(self.filename_lbl)
        spacer0 = QSpacerItem(20, 20, QSizePolicy.Expanding, QSizePolicy.Minimum)
        self.hlayout.addItem(spacer0)
        self.view_btn = QToolButton(self.header)
        icon = QIcon()
        icon.addPixmap(QPixmap("medias/eye_red.png") , QIcon.Normal, QIcon.Off)
        self.view_btn.setIcon(icon)
        self.view_btn.setIconSize(QSize(14, 14))
        self.view_btn.setToolTip(_("View in main viewer"))
        self.hlayout.addWidget(self.view_btn)
        self.info_btn = QToolButton(self.header)
        icon1 = QIcon()
        icon1.addPixmap(QPixmap("medias/infos.png") , QIcon.Normal, QIcon.Off)
        self.info_btn.setIcon(icon1)
        self.info_btn.setIconSize(QSize(14, 14))
        self.info_btn.setToolTip(_("Show metadata"))
        self.hlayout.addWidget(self.info_btn)
        self.check_chb = QCheckBox(self.header)
        self.check_chb.setToolTip(_("Select the image"))
        self.hlayout.addWidget(self.check_chb)
        self.layout.addWidget(self.header)
        self.graphics = Graphics(self)
        self.layout.addWidget(self.graphics)
        self.footer = QWidget(self)
        self.footer.setMaximumSize(QSize(2222, 18))
        self.hlayout1 = QHBoxLayout(self.footer)
        self.hlayout1.setContentsMargins(4, 0, 2, 0)
        spacer = QSpacerItem(20, 20, QSizePolicy.Expanding, QSizePolicy.Minimum)
        self.iso_lbl = QLabel("ISO", self.footer)
        spacer1 = QSpacerItem(20, 20, QSizePolicy.Expanding, QSizePolicy.Minimum)
        self.fnumber_lbl = QLabel("fNumber", self.footer)
        spacer2 = QSpacerItem(20, 20, QSizePolicy.Expanding, QSizePolicy.Minimum)
        self.speed_lbl = QLabel("Shutter", self.footer)
        spacer3 = QSpacerItem(20, 20, QSizePolicy.Expanding, QSizePolicy.Minimum)
        self.hlayout1.addItem(spacer)
        self.hlayout1.addWidget(self.iso_lbl)
        self.hlayout1.addItem(spacer1)
        self.hlayout1.addWidget(self.fnumber_lbl)
        self.hlayout1.addItem(spacer2)
        self.hlayout1.addWidget(self.speed_lbl)
        self.hlayout1.addItem(spacer3)
        self.layout.addWidget(self.footer)
        self.header.setStyleSheet(styles.HEADER)
        self.footer.setStyleSheet(styles.HEADER)
        self.graphics.setStyleSheet(styles.VIEWER)
        self.view_btn.setStyleSheet(styles.TOOLBUTTON)
        self.check_chb.setStyleSheet(styles.TOOLBUTTON)
        self.info_btn.setStyleSheet(styles.TOOLBUTTON)

        self.is_selected = False
        self.graphics.qimage = driver.qimages[idx]
        self.view_btn.clicked.connect(self.on_view_action)
        self.check_chb.stateChanged.connect(self.on_check_action)
        self.info_btn.clicked.connect(self.on_info_action)
        if driver.read_only:
            self.check_chb.hide()
        self.get_metadata()

    def get_metadata(self):
        data = Metadata(self.picture)
        if data.get_data():
            self.data = data.get_data_for_thumbnail()
            self.data["frmt_date"] = ""

        self.display_infos()

    def display_image(self):
        self.graphics.refresh_view()

    def resize_picture(self):
        self.graphics.refresh_view()

    def change_image(self, path, image):
        self.data = None
        self.picture = path
        self.get_metadata()
        name = os.path.basename(path)
        self.filename_lbl.setText(name)
        self.graphics.qimage = image
        self.graphics.refresh_view()

    def on_view_action(self):
        QCoreApplication.processEvents()
        self.driver.show_in_main_viewer(self.index)
        
    def on_check_action(self, state):
        self.driver.select_image(self.index, state)
        self.is_selected = state

    def on_check_menu_action(self, state):
        self.check_chb.click()

    def on_info_action(self):
        self.driver.show_metadata(self.index)

    def display_infos(self):
        iso = "ISO: --"
        fnumber = "f: --"
        shutter = "-- sec."
        if self.data is not None:
            fn, sh, date = self.format_data()
            fnumber = fnumber.replace("--", fn)
            shutter = shutter.replace("--", sh)
            if self.data["iso"] is not None:
                iso = iso.replace("--", str(self.data["iso"]))

            self.filename_lbl.setText(os.path.basename(self.picture))
            self.data["frmt_date"] = date

        self.iso_lbl.setText(iso)
        self.fnumber_lbl.setText(fnumber)
        self.speed_lbl.setText(shutter)

    def format_data(self):
        """Returns the exposure infos as string.

        """
        fnumb, speed, date = "--", "--", None
        di = self.data['fn']
        if di is not None:
            if isinstance(di, Fraction):
                n, d = di.numerator * 1.0, di.denominator
                try:
                    fn = round(n / d, 2)
                    if fn != 0:
                        fnumb = str(fn)
                except ZeroDivisionError:
                    pass

        exp = self.data['exp']
        if exp is not None:
            if isinstance(exp, Fraction):
                if exp.denominator == 1:
                    speed = str(exp.numerator)
                else:
                    n, d = exp.numerator * 1.0, exp.denominator
                    try:
                        sp = int(d / n)
                        if sp != 0:
                            speed = "/".join(["1", str(sp)])
                    except ZeroDivisionError:
                        pass

        dt = self.data['date']
        date = ""
        if dt is not None:
            date = dt.strftime('%d %b %Y %H:%M:%S')

        return fnumb, speed, date


class Graphics(QGraphicsView):
    def __init__(self, parent=None):
        super().__init__(parent)
        self.item = parent
        self.qimage = None
        self.setVerticalScrollBarPolicy(Qt.ScrollBarAlwaysOff)
        self.setHorizontalScrollBarPolicy(Qt.ScrollBarAlwaysOff)
        self.scene_ = QGraphicsScene()
        self.setScene(self.scene_)

    def get_size(self):
        return RectSize(self.size())

    def refresh_view(self, oriented=True):
        if self.qimage is None:
            return

        if oriented:
            self.check_orientation()
        size = RectSize(self.qimage.size())
        v_size = self.get_size()
        if size.is_include(v_size):
            self.qpixmap = QPixmap.fromImage(self.qimage)

        else:
            self.qpixmap = QPixmap.fromImage(self.qimage.scaled(v_size.width,
                                         v_size.height,
                                         Qt.KeepAspectRatio, 
                                         Qt.SmoothTransformation))

        self.scene_.clear()
        self.scene_.setSceneRect(0, 0, self.qpixmap.width(), 
                                 self.qpixmap.height())
        self.scene_.addPixmap(self.qpixmap)

    def check_orientation(self):
        if self.item.data is not None:
            orn = self.item.data["orientation"]
            if orn in [3, 6, 8]:
                matrix = QTransform()
                angles = {3: 180, 6: 90, 8: -90}
                self.qimage = self.qimage.transformed(matrix.rotate(
                                    angles[orn]), 
                                    Qt.SmoothTransformation)

    def rotate_to_left(self):
        matrix = QTransform()
        self.qimage = self.qimage.transformed(matrix.rotate(-90), 
                                    Qt.SmoothTransformation)
        self.refresh_view(False)

    def rotate_to_right(self):
        matrix = QTransform()
        self.qimage = self.qimage.transformed(matrix.rotate(90), 
                                    Qt.SmoothTransformation)
        self.refresh_view(False)

    def mousePressEvent(self, event):
        if event.button() == 2:
            self.show_menu(event)

    def show_menu(self, event):
        self.pop_menu = QMenu(self)
        self.mainview = QAction(self.pop_menu)
        self.mainview.setText(_("Show in main viewer"))
        self.pop_menu.addAction(self.mainview)
        self.data = QAction(self.pop_menu)
        self.data.setText(_("Show metadata"))
        self.pop_menu.addAction(self.data)
        self.selected = QAction(self.pop_menu)
        self.selected.setCheckable(True)
        if self.item.is_selected:
            self.selected.setChecked(True)
            self.selected.setText(_("Unselect"))

        else:
            self.selected.setChecked(False)
            self.selected.setText(_("Select"))

        self.pop_menu.addAction(self.selected)
        self.pop_menu.addSeparator()
        self.lrotate = QAction(self.pop_menu)
        self.lrotate.setText(_("Rotate to Left"))
        self.pop_menu.addAction(self.lrotate)
        self.rrotate = QAction(self.pop_menu)
        self.rrotate.setText(_("Rotate to Right"))
        self.pop_menu.addAction(self.rrotate)

        self.pop_menu.setContextMenuPolicy(Qt.ActionsContextMenu)
        self.pop_menu.setStyleSheet("""
                QMenu {background: transparent;
                    border: 2px solid #8A8E96;
                    border-radius: 8px;}
                QMenu::item {background: #FFFFFF;
                    color: #000000;
                    padding: 2px 25px 2px 20px;
                    border: 1px solid transparent;}
                QMenu::item:selected {background-color: #E8E8F8;
                    border-color: #0F2D78;
                    border-radius: 4px;
                    color: #0F2D78;}
                QMenu::icon:checked {background: gray;
                    border: 1px inset gray;
                    position: absolute;
                    top: 1px;
                    right: 1px;
                    bottom: 1px;
                    left: 1px;}
                QMenu::separator {height: 2px;
                    background: #BABDD6;}""")
        
        self.mainview.triggered.connect(self.item.on_view_action)
        self.data.triggered.connect(self.item.on_info_action)
        self.selected.toggled.connect(self.item.on_check_menu_action)
        self.lrotate.triggered.connect(self.rotate_to_left)
        self.rrotate.triggered.connect(self.rotate_to_right)
        offset = QPoint(10, 10)
        pos = QCursor.pos() - event.pos() + offset
        self.pop_menu.exec_(pos)

