# -*- coding: utf-8 -*-

# nogui.py
# This file is part of Oqapy serie 3
#
# Author: Vincent Vande Vyvre <vincent.vandevyvre@oqapy.eu>
# Home: http://www.oqapy.eu
# Copyright: 2009-2017 Vincent Vande Vyvre
# Licence: GPL3
#
#
# Oqapy is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Oqapy is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Oqapy.  If not, see <http://www.gnu.org/licenses/>.
#
# Run Oqapy in command line only

import os
import sys
import logging
import datetime

from data import VERSION_STR

def show_versions():
    import subprocess
    from PyQt5.QtCore import Qt, QT_VERSION_STR, PYQT_VERSION_STR
    import platform
    from sip import SIP_VERSION_STR
    PY_VERSION = sys.version.split()[0]
    PLATFORM = platform.platform()
    try:
        import pyexiv2
        EXIV2 = pyexiv2.version_info
        del pyexiv2
    except:
        EXIV2 = "< 2.x"
    rep = subprocess.Popen(["gphoto2", "--version"], universal_newlines=True,
                                stdout=subprocess.PIPE).communicate()
    GPHOTO2_VERS = rep[0].split('\n')[0]

    print("\n-------------------------------------------")
    print("  Oqapy-{0}".format(VERSION_STR))
    print("  Python version: {0}".format(PY_VERSION))
    print("  on: {0}".format(PLATFORM))
    print("  Qt version: {0}".format(QT_VERSION_STR))
    print("  Sip version: {0}".format(SIP_VERSION_STR))
    print("  PyQt4 version: {0}".format(PYQT_VERSION_STR))
    print("  Pyexiv2 version: {0}".format(EXIV2))
    print("  gPhoto2 version: {0}".format(GPHOTO2_VERS))
    print("------------------------------------------\n")

def show_infos():
    """Show info and exit."""
    # http://misc.flogisoft.com/bash/tip_colors_and_formatting
    print("""
    Oqapy-{0}

    Oqapy is photographic workflow software.

    This version is running with python >= 3.3

    Copyright (C) 2010-2017 Vincent Vande Vyvre <vincent.vandevyvre@oqapy.eu>
    This is free software; see the source for copying conditions.  There is NO
    warranty; not even for MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
\n""".format(VERSION_STR))
    print("""  Usage: oqapy [FILENAME][OPTION]

  FILENAME  Path of an image file or a folder that contains image files.
            If a file name is provided, the parent folder is open as
            source folder, the list of all images is created and the 
            viewer will show the file provided in first place.
  Options:
  -v --version\t Show version and exit.
  -h --help\t Show this info and exit.
  -d --debug\t Launch Oqapy in debugging mode
  -s --size FILENAME VALUE -o FILENAME [-m]
            Create a resized copy of an image.
            VALUE is 'max=x' or 'width=x' or 'heigth=x' where x is the value in pixel
            -o FILENAME is the output file
            -m Copy the metadata into the output file, default: False
  -r --rotate FILENAME SIDE [-o FILENAME] [-m]
            Rotate an image
            SIDE must be 'right' or 'left'
            -o FILENAME is the output file, if not provided, the original file
                will be overridden.
            -m Copy the metadata into the output file, default: False 
  -i --infos FILENAME [-o FILENAME]
           Display the essential metadata of FILENAME.
           -o FILENAME is the output text file.


  Report bugs to <[2;34mhttps://bugs.launchpad.net/oqapy/+filebug[0;39m>\n
""")

class Oqapy:
    def __init__(self, args):
        if '-v' in args or '--version' in args:
            show_versions()
            sys.exit()

        if '-h' in args or '--help' in args:
            show_infos()
            sys.exit()

        infile = False
        outfile = False
        copy_metadata = False
        for a in ['-d', '--debug']:
            if a in args:
                # Not used in command line mode
                args.remove(a)

        if '-o' in args:
            idx = args.index('-o')
            try:
                path = args[idx+1]
            except IndexError:
                print("The argument '-o' must be followed by a file name\n")
                sys.exit(1)

            args.remove('-o')
            outfile = self.get_full_path(path)
            if not self.check_permission(outfile):
                sys.exit(1)

            args.remove(path)

        if '-m' in args:
            copy_metadata = True
            args.remove('-m')

        for a in ['-s', '--size']:
            size = inf = False
            if a in args:
                idx = args.index(a)
                args = args[idx+1:]
                if not args:
                    self.unvalid_length()

                num = len(args)
                i = 0
                while i < num:
                    if '=' in args[i]:
                        size = args[i]
                    else:
                        inf = args[i]
                    i += 1

                if not all((size, inf, outfile)):
                    self.unvalid_length()

                try:
                    side, value = size.split('=')
                    if side not in ('max', 'width', 'height'):
                        raise ValueError
                    v = int(value)
                except ValueError:
                    print("Unable to read the size, enter oqapy -h for help\n")
                    sys.exit(1)

                infile = self.check_path(inf)
                if not infile:
                    print("No such file: %s\n" % inf)
                    sys.exit(1)

                new_size = self.resize_image(infile, outfile, size)
                if copy_metadata:
                    self.write_metadata(infile, outfile, new_size)

                sys.exit()

        for a in ['-r', '--rotate']:
            if a in args:
                side = infile = False
                args.remove(a)
                if not args:
                    self.unvalid_length()

                for arg in args:
                    if 'side=' in arg:
                        side = arg.split('=')[1]
                        if side not in ['left', 'right']:
                            print("The second argument of rotate must "\
                                        "be 'left' or 'right'\n")
                            sys.exit(1)

                    else:
                        infile = self.check_path(arg)
                        if not infile:
                            print("No such file: %s\n" % arg)
                            sys.exit(1)

                outfile = outfile or infile
                if all((infile, side)):
                    new_size = self.rotate_image(infile, outfile, side)

                else:
                    self.unvalid_length()

                if copy_metadata:
                    self.write_metadata(infile, outfile, new_size, 1)

                sys.exit()

        for a in ['-i', '--infos']:
            if a in args:
                idx = args.index(a)
                try:
                    path = args[idx+1]
                except IndexError:
                    print("The argument '%s' must be followed by a "\
                                        "file name\n" % a)
                    sys.exit(1)

                infile = self.check_file(path)
                if not infile:
                    print("No such file: %s\n" % args[idx+1])
                    sys.exit(1)

                self.get_metadata(infile, outfile)
                sys.exit()

    def get_full_path(self, path):
        if not os.path.isabs(path):
            path = os.path.normpath(os.path.join(os.getcwd(), path))

        return path

    def check_path(self, path):
        path = self.get_full_path(path)
        if os.path.exists(path):
            return path

        return False

    def check_permission(self, path):
        parent = os.path.dirname(path)
        if not os.access(parent, os.W_OK):
            print('Destination dir `%s` is read only\n' % parent)
            return False

        return True

    def unvalid_length(self):
        print("Not enought arguments, enter oqapy -h for help\n")
        sys.exit(1)

    def resize_image(self, infile, outfile, size):
        from PyQt5.QtGui import QImage
        from PyQt5.QtCore import Qt
        side, value = size.split('=')
        try:
            img = QImage(infile)
        except Exception as why:
            print("Read image error:\n  %s" % why)
            sys.exit(1)

        if side == 'max':
            img = img.scaled(int(value), int(value), Qt.KeepAspectRatio, 
                                Qt.SmoothTransformation)
        elif side == 'height':
            img = img.scaledToHeight(int(value), Qt.SmoothTransformation)
        else:
            img = img.scaledToWidth(int(value), Qt.SmoothTransformation)

        self.save_image(img, outfile)
        return img.width(), img.height()

    def write_metadata(self, source, dest, size=None, orientation=None):
        _, ext = os.path.splitext(source)
        if not ext.lower() in ('.tif', '.tiff', '.jpg', '.jpeg'):
            print("Format %s not supported for metadata, ignoring\n" % ext)
            return

        data = self.read_metadata(source)
        if not data:
            print("No metadata to copy")
            return

        self.update_processing_software(data)
        if size is not None:
            self.update_image_size(data, size)

        if orientation is not None:
            self.update_image_orientation(data, orientation)

        new = self.read_metadata(dest)
        # TIFF file don't support comment
        cm = not os.path.splitext(dest)[1] in ['.tif', '.TIF', '.tiff', '.TIFF']
        data.copy(new, comment=cm)
        try:
            new.write()
        except Exception as why:
            print("Metadata writing error: %s\n" % why)

    def read_metadata(self, fname):
        try:
            import pyexiv2
        except ImportError:
            print("pyexiv2 not found, ignoring\n")
            return

        data = pyexiv2.ImageMetadata(fname)
        data.read()
        return data

    def rotate_image(self, infile, outfile, angle):
        if angle == "left":
            angle = -90

        else:
            angle = 90
        from PyQt5.QtGui import QImage, QTransform
        from PyQt5.QtCore import Qt
        try:
            img = QImage(infile)
        except Exception as why:
            print("Read image error:\n  %s" % why)
            sys.exit(1)

        matrix = QTransform()
        img = img.transformed(matrix.rotate(angle), Qt.SmoothTransformation)
        self.save_image(img, outfile)
        return img.width(), img.height()

    def get_metadata(self, src, outf):
        data = self.read_metadata(src)
        if data is not None:
            infos = {'Date': '', 'Author': '', 'Copyright': '', 'Comment': '', 
                     'Exposure': '', 'FNumber': '', 'ISO': '',
                     'Width': '', 'Height': '', 'Orientation': '', 'Longitude': '',
                     'Latitude': '', 'Elevation': '', 'File': src}
            self.set_infos_values(data, infos)
            self.write_infos(infos, outf)
            return

        print("No metadata found in %s\n" % src)

    def update_processing_software(self, data):
        import pyexiv2
        d = 'Exif.Image.DateTime'
        p = 'Exif.Image.ProcessingSoftware'
        soft = "Oqapy-{0}".format(VERSION_STR)
        try:
            data[d].value = datetime.datetime.today()
        except:
            data[d] = pyexiv2.ExifTag(d, datetime.datetime.today())

        try:
            data[p].value = soft
        except:
            data[p] = pyexiv2.ExifTag(p, soft)

    def update_image_size(self, data, size):
        w = "Exif.Image.ImageWidth"
        l = "Exif.Image.ImageLength"
        w1 = 'Exif.Photo.PixelXDimension'
        l1 = 'Exif.Photo.PixelYDimension'
        try:
            data[w].value = size[0]
        except:
            pass

        try:
            data[l].value = size[1]
        except:
            pass

        try:
            data[w1].value = size[0]
        except:
            pass

        try:
            data[l1].value = size[1]
        except:
            pass

    def update_image_orientation(self, data, orientation):
        o = "Exif.Image.Orientation"
        try:
            data[o].value = orientation
        except:
            pass

    def set_infos_values(self, data, dct):
        long_, lat, alt = self.get_geolocation(data)
        data_keys = {'Date': 'Exif.Photo.DateTimeOriginal', 
                     'Author': 'Exif.Image.Artist', 
                     'Copyright': 'Exif.Image.Copyright', 
                     'Comment': 'Exif.Photo.UserComment',
                     'Width': "Exif.Photo.PixelXDimension", 
                     'Height': "Exif.Photo.PixelYDimension",
                     'Orientation': "Exif.Image.Orientation",
                     'Exposure': "Exif.Photo.ExposureTime", 
                     'FNumber': "Exif.Photo.FNumber", 
                     'ISO': "Exif.Photo.ISOSpeedRatings"}

        for key in data_keys:
            try:
                dct[key] = data[data_keys[key]].value
            except:
                pass

        dct['Longitude'] = long_
        dct['Latitude'] = lat
        dct['Elevation'] = alt

    def get_geolocation(self, data):
        import re
        lon, lat, elev = '', '', ''
        try:
            lo = data['Exif.GPSInfo.GPSLongitude'].value
            lor = data['Exif.GPSInfo.GPSLongitudeRef'].value
            dlo = dms_to_decimal(lo)
            lon = dlo * (1 if lor in ('E', 'e') else -1)
        except:
            pass

        try:
            la = data['Exif.GPSInfo.GPSLatitude'].value
            lar = data['Exif.GPSInfo.GPSLatitudeRef'].value
            dla = dms_to_decimal(la)
            lat = dla * (1 if lar in ('N', 'n') else -1)
        except:
            pass

        try:
            alt = data['Exif.GPSInfo.GPSAltitude'].value
            alt = round(alt.numerator / alt.denominator, 2)
            altr = data['Exif.GPSInfo.GPSAltitudeRef'].value
            if eval(altr):
                alt *= -1
        except:
            pass

        return lon, lat, elev

    def write_infos(self, infos, dest):
        out = sys.stdout
        out.write('\n')
        if dest:
            out = open(dest, 'w')

        for key in ('File', 'Date', 'Author', 'Copyright', 'Width', 'Height',  
                     'Orientation', 'Exposure', 'FNumber', 'ISO', 'Longitude',  
                     'Latitude', 'Elevation', 'Comment'):
            out.write('%10s:\t%s\n' % (key, infos[key]))
        out.write('\n')

        if dest:
            out.close()

    def dms_to_decimal(self, frcts):
        vals = [(f.numerator / f.denominator) for f in frcts]
        return (vals[0] + (vals[1] / 60.0) + (vals[2] / 3600.0))

    def check_dir(self, path):
        path = self.get_full_path(path)
        if os.path.isdir(path):
            return path
        return False

    def check_file(self, path):
        path = self.get_full_path(path)
        if os.path.isfile(path):
            return path
        return False

    def save_image(self, image, filename):
        tmp = self.get_temp_filename(filename)
        try:
            image.save(tmp)
        except IOError as why:
            print('Write image error: %s' % why)
            if os.path.isfile(tmp):
                os.unlink(tmp)
        
        else:
            os.rename(tmp, filename)

    def get_temp_filename(self, fname):
        import time
        fld = os.path.dirname(fname)
        ext = os.path.splitext(fname)[1]
        n = str(time.time()).replace('.', '')
        target = os.path.join(fld, n + ext)
        count = 1
        while 1:
            if os.path.isfile(target):
                target = os.path.join(fld, n + '(%s)' % count + ext)
                count += 1

            else:
                break

        return target



    
