# -*- coding: utf-8 -*-

# picture.py
# This file is part of Oqapy serie 3
#
# Author: Vincent Vande Vyvre <vincent.vandevyvre@oqapy.eu>
# Home: http://www.oqapy.eu
# Copyright: 2011-2017 Vincent Vande Vyvre
# Licence: GPL3
#
# All images are instances of this class


import os
import logging
lgg = logging.getLogger(__name__)

from threading import Thread

from histogram import Histogram
from metadata.metadata import Metadata


class Picture:
    """Define an image object, all images shown in the viewer are instance
    of this class.

    """
    def __init__(self, core, path, idx, histo=True):
        """Instanciate an image object.

        Args:
        core -- instance of OqapyCore
        path -- full path of the image file
        idx -- index of image into the current list
        """
        self.core = core
        f, b = os.path.split(path)
        n, e = os.path.splitext(b)
        self.uri = path
        self.folder = f
        self.base = b
        self.name = n
        self.date = None
        self.image = None
        self.ext = e
        self.weight = os.path.getsize(path)
        self.width = 0
        self.height = 0
        self.metadata = None
        self.camera = None
        self.orientation = 1
        self.oriented = 0
        self.locate = False
        self.histogram = None
        self.rotation = 0
        lgg.info("Instanciate Picture %s, histo: %s" %(b, histo))
        if self.ext.lower() in [".jpg", ".jpeg", ".tif", ".tiff"]:
            self.load_metadata()

        if histo:
            self.create_histogram(path)

    def load_metadata(self):
        self.metadata = Metadata(self.uri)
        if self.metadata.get_data():
            orient = self.metadata.get_orientation()
            if orient not in [1, 3, 6, 8]:
                orient = 1
            lgg.info('Orientation: %s' % orient)
            self.orientation = orient
            self.locate = self.metadata.get_location()
            if not self.locate[0]:
                # some cameras (Leica, ...) write 0.0 or -0.0
                self.locate = False
            self.date, uid = self.metadata.get_date_and_id()
            self.width, self.height = self.metadata.get_image_size()

        else:
            lgg.info('No data: %s' % self.metadata.last_error)

    def set_size(self, s):
        self.width, self.height = s

    def get_size(self):
        if self.image:
            self.width = self.image.width()
            self.height = self.image.height()

    def set_rotation(self, angle):
        self.rotation = (self.rotation + angle) % 360

    def change_orientation(self, angle=None, size=None, metadata=False):
        """Fix the attribute 'oriented' when the orientation has been changed.

        Args:
        angle -- int(degrees), if None, the metadata 'Exif.Image.Orientation'
                 will be used to choose the angle.
        size -- the new size of the image
        metadata -- if True, the metadata will be updated
        """
        if angle is None:
            angle = {1: 0, 3: 180, 6: 90, 8: -90}[self.orientation]

        self.oriented += angle
        if size is None:
            if abs(angle) == 90:
                self.width, self.height = self.height, self.width

        else:
            self.width, self.height = size.width(), size.height()

        self.core.ui.toolpanel.update_with_picture(self)
        if metadata:
            self.update_orientation_data()

    def write_geotag(self, lat, lon):
        """Modify geographical values in metadata, values are send by the
        webView.

        Args:
        lat -- float(latitude)
        lon -- float(longitude)
        """
        lgg.info('Write geo tags to {0}'.format(self.base))
        try:
            lat = float(lat)
            lon = float(lon)
        except ValueError:
            lgg.error('write_geotag unexpected values: {0}, {1}'
                        .format(lat, lon))
            return

        # Check values
        if abs(lat) <= 90 and abs(lon) <= 180:
            if self.metadata is not None:
                self.metadata.modify_geotags(lat, lon)
                self.apply_new_data()


    def update_orientation_data(self):
        if self.metadata is None:
            return

        self.metadata.update_orientation(1, self.width, self.height)
        self.metadata.modify_by_oqapy()
        self.metadata.update_oqapy_data(change="Rotation: %s deg." % self.rotation)
        self.apply_new_data()

    def apply_new_data(self):
        self.metadata.save_data()
        self.data = self.metadata.get_data()
        lgg.info('Metadata updated')
        self.core.metadataeditor.set_data(self.metadata)

    def create_histogram(self, img):
        def draw_histo():
            histogram.build_histogram()
        size = self.core.ui.get_histogram_size()
        fname = "".join(["histo_", self.name, '.png'])
        self.file_histo = self.core.workspace['histo'] + fname

        if not os.path.isfile(self.file_histo):
            histogram = Histogram(self.uri, self.file_histo, size)
            histogram.histogramCreated.connect(self.core.display_histogram)
            histogram.brightness = True
            if self.ext.lower() == '.pgm':
                histogram.is_grey = True

            Thread(target=draw_histo).start()

        else:
            self.core.display_histogram(self.file_histo)

    def get_rgb_values(self, level):
        """Return the RGB values for the given level.

        histogram attribute is a list of [gray value], [red values],
        [green values] and [blue values].

        Args:
        level -- color value (0~255)
        
        Returns:
        tuple (red, green, blue)
        """
        return (self.histogram[1][level], self.histogram[2][level], 
                self.histogram[3][level])

        

