# -*- coding: utf-8 -*-

# previewer.py
# this file is part of Oqapy serie 3
#
# Author: Vincent Vande Vyvre <vincent.vandevyvre@oqapy.eu>
# Home: http://www.oqapy.eu
# Copyright: 2014-2017 Vincent Vande Vyvre
# Licence: GPL3
#
# Preview window

import os
import logging
lgg = logging.getLogger(__name__)

from PyQt5 import QtGui, QtCore, QtWidgets

from PyQt5.QtCore import Qt, pyqtSignal

from tools.rectsize import RectSize
from VWidgets_PyQt5.vtoolheader import VToolHeader
#from stylesheets import VIEWER, HEADER, TITLE


class PreviewHandler:
    def __init__(self, core, size, viewer, img, zoom, toolbox):
        self.core = core
        self.ui = core.ui
        self.temp_fld = core.workspace['temp'].path
        self.size = size
        self.viewer = viewer
        self.image = img
        self.toolbox = toolbox
        self.sample_file = os.path.join(self.temp_fld, 'preview.TIFF')
        self.filtered = os.path.join(self.temp_fld, 'filtered.TIFF')
        self.previewer = Previewer(self, size, viewer, img)
        vw, vh = self.viewer.width(), self.viewer.height()
        vx, vy = vw/2, vh/2
        offset = self.viewer.mapFromScene(0, 0)
        center = vx - offset.x(), vy - offset.y()
        posx, posy = center[0] - self.size/2, center[1] - self.size/2 - 22
        self.proxy = ProxyPreview(self, self.previewer)
        self.previewer.set_config(posx, posy, self.proxy, zoom, offset, center)
        self.previewer.update_sample()
        self.viewer.scene().addItem(self.proxy)
        self.proxy.setPos(posx, posy)

    def config_filter(self, cfg):
        """Configure the filter.

        Args:
        cfg -- dict 'name': the filter name
                    'params': list of parametres
        """
        self.config = cfg
        self.previewer.header.setTitle(u"%s" % self.config['name'])

    def update(self):
        """Apply the filter.

        """
        lgg.info('Update filter: %s' % self.config['name'])
        if self.config['name'] == 'Denoising':
            self.denoise_sample()

        elif self.config['name'] == 'Contrast-Brightness':
            self.contrast_sample()

        elif self.config['name'] == 'Contrast stretching':
            self.contrast_stretching_sample()

    def update_filtered(self):
        """Display the filtered image.

        """
        lgg.info('Display: %s' % self.filtered)
        if os.path.isfile(self.filtered):
            im = QtGui.QImage(self.filtered)
            self.previewer.display_image(im)

    def close(self):
        self.proxy.close()

    def _close(self):
        # The close action must be called from core
        self.core.close_preview_window()

    def denoise_sample(self):
        """Apply the denoising filter.

        """
        from filters.denoisingfilter import DenoisingFilter
        flt = DenoisingFilter(bytes(self.sample_file, 'utf-8'), 
                              bytes(self.filtered, 'utf-8'))
        l, lc, t, r = self.config['params']
        flt.configure(l_bw=l, l_color=lc, template=t, sample=r)
        if not flt.load_image():
            lgg.info('denoise sample loading image error: %s' 
                        % flt.last_error)
            return

        ret = flt.apply_filter()
        if ret:
            self.update_filtered()

    def contrast_sample(self):
        """Apply the contrast-brightness filter.

        """
        from filters.contrastbrightnessfilter import ContrastBrightnessFilter
        flt = ContrastBrightnessFilter(bytes(self.sample_file, 'utf-8'), 
                                       bytes(self.filtered, 'utf-8'))
        l, u, c, b = self.config['params']
        flt.configure(cnt=c, brg=b, low=l, up=u)
        if not flt.load_image():
            lgg.info('contrast sample loading image error: %s' 
                        % flt.last_error)
            return

        ret = flt.apply_filter()
        if ret:
            self.update_filtered()

    def contrast_stretching_sample(self):
        """Apply the contrast stretching filter.

        """
        from filters.contraststretchingfilter import ContrastStretchingFilter
        flt = ContrastStretchingFilter(bytes(self.sample_file, 'utf-8'), 
                                       bytes(self.filtered, 'utf-8'))
        pixels = self.size * self.size
        upper = (1 - self.config['params'][1] / 100) * pixels
        lower = pixels * self.config['params'][0] / 100
        flt.configure(min_=lower, max_=upper)
        if not flt.load_image():
            lgg.info('contrast stretching sample loading image error: %s'
                        % flt.last_error)
            return

        ret = flt.apply_filter()
        if ret:
            self.update_filtered()

class Previewer(QtWidgets.QWidget):
    positionChanged = pyqtSignal(int, int)
    def __init__(self, handler, size, viewerarea, img, parent=None):
        super().__init__(parent)
        self.handler = handler
        self.size = size
        rsize = RectSize(size, size)
        self.viewerarea = viewerarea
        self.image = img
        self.sample = None
        self.filter_cfg = None
        self.zoom = 1
        self.center = None
        self.glayout = QtWidgets.QGridLayout(self)
        self.glayout.setContentsMargins(2, 2, 2, 2)
        self.vlayout = QtWidgets.QVBoxLayout()
        self.vlayout.setContentsMargins(0, 0, 0, 0)
        self.vlayout.setSpacing(0)
        self.header = VToolHeader(self)
        self.header.setTitle(u"Preview")
        self.header.setMaximumSize(size, 22)
        self.vlayout.addWidget(self.header)
        self.view = QtWidgets.QGraphicsView(self)
        self.view.setMinimumSize(rsize.as_qsize)
        self.vlayout.addWidget(self.view)
        self.glayout.addLayout(self.vlayout, 0, 0, 1, 1)
        self.close_btn = self.header.add_button('medias/close.png', 'close_btn')
        self.close_btn.setToolTip(u'Close the preview')

        self.view.setVerticalScrollBarPolicy(QtCore.Qt.ScrollBarAlwaysOff)
        self.view.setHorizontalScrollBarPolicy(QtCore.Qt.ScrollBarAlwaysOff)
        self.scene = QtWidgets.QGraphicsScene()
        self.view.setScene(self.scene)
        self.moved_to = False
        self.close_btn.clicked.connect(self.on_close_preview)
        self.header.setCollapseButton(False)
        #self.setStyleSheet(VIEWER)
        #self.header.setStyleSheet(HEADER)
        #self.header.label.setStyleSheet(TITLE)
        #self.header.layout.itemAt(0).widget().setStyleSheet(TITLE)

    def set_config(self, posx, posy, proxy, zoom, limits, center):
        self.old_posx = posx
        self.old_posy = posy
        self.proxy = proxy
        self.zoom = zoom
        self.limits = limits
        self.center = center[0] / zoom, center[1] / zoom

    def update_sample(self):
        self.sample = self.crop_image()
        self.display_image(self.sample)

    def crop_image(self):
        px = int(self.center[0] - (self.size / 2.0))
        py = int(self.center[1] - (self.size / 2.0))
        orect = QtCore.QRect(px, py, self.size, self.size)
        c = self.image.copy(orect)
        c.save(self.handler.sample_file)
        return c

    def display_image(self, im):
        self.scene.clear()
        self.scene.setSceneRect(0, 0, self.size, self.size)
        self.scene.addPixmap(QtGui.QPixmap.fromImage(im))

    def refresh_template(self, pos):
        x = y = 0
        pixmap = self.viewerarea.get_qpixmap()
        w, h = pixmap.width(), pixmap.height()
        if pos[0] < 0:
            x = abs(pos[0])

        elif pos[0] + self.size > w:
            x = w - (pos[0] + self.size)

        if pos[1] < -28:
            y = 28

        elif pos[1] + self.size + 28 > h:
            y = h - (pos[1] + self.size + 28)

        if x or y:
            self.proxy.setPos(pos[0]+x, pos[1]+y)

        pos = [self.pos().x(), self.pos().y()]
        w, h = self.image.width(), self.image.height()
        left = ((pos[0] + self.size / 2) / self.zoom) - self.size / 2

        if left < 0:
            left = 0

        elif left > w - self.size:
            left =  w - self.size

        up = ((pos[1] + 22 + self.size / 2) / self.zoom) - self.size / 2
        if up < 0:
            up = 0

        elif up > h - self.size:
            up = h - self.size

        rect = QtCore.QRect(left , up, self.size, self.size)
        self.moved_to = False
        self.sample = self.image.copy(rect)
        self.sample.save(self.handler.sample_file)
        self.display_image(self.sample)
        self.handler.update()

    def closeEvent(self, event):
        self.parent().preview_window = False

    def moveEvent(self, event):
        self.moved_to = event.pos().x(), event.pos().y()

    def hideEvent(self, event):
        pass

    def mouse_press_event(self, event):
        pos = event.pos()
        child = self.childAt(pos.toPoint())
        if child == self.close_btn:
            event.accept()
            self.on_close_preview()

        if event.button() == 1:
            self.old_x, self.old_y = pos.x(), pos.y()

    def mouse_move_event(self, event):
        pos = event.pos()
        cur = self.pos()
        x, y = pos.x(), pos.y()
        deltax, deltay = x - self.old_x, y - self.old_y
        self.old_posx += deltax
        self.old_posy += deltay
        self.proxy.setPos(self.old_posx, self.old_posy)

    def mouse_release_event(self, event):
        if self.moved_to:
            self.refresh_template(self.moved_to)

    def on_close_preview(self):
        self.handler._close()


class ProxyPreview(QtWidgets.QGraphicsProxyWidget):
    def __init__(self, handler, widget):
        super().__init__()
        self.handler = handler
        self.setWidget(widget)
        self.setAcceptHoverEvents(True)

    def mousePressEvent(self, event):
        self.widget().mouse_press_event(event)

    def mouseMoveEvent(self, event):
        self.widget().mouse_move_event(event)

    def mouseReleaseEvent(self, event):
        self.widget().mouse_release_event(event)

    def hoverEnterEvent(self, event):
        self.old_shape = self.handler.viewer.cursor().shape()
        shape = QtGui.QPixmap("medias/cross_cursor_32.png")
        cursor = QtGui.QCursor(shape, -1, -1)
        self.widget().view.setCursor(cursor)

    def hoverLeaveEvent(self, event):
        cursor = QtGui.QCursor(self.old_shape)
        self.handler.viewer.setCursor(cursor)



