# -*- coding: utf-8 -*-

# printing.py
# this file is part of Oqapy serie 3
#
# Author: Vincent Vande Vyvre <vincent.vandevyvre@oqapy.eu>
# Home: http://www.oqapy.eu
# Copyright: 2010-2018 Vincent Vande Vyvre
# Licence: GPL3
#
# Dialog box for printing.


import sys
import os
import shutil

from PyQt5 import QtCore, QtGui, QtWidgets, QtPrintSupport

from printing.ui_printing import Ui_Printing

class Printing:
    def __init__(self, main, pictures, index):
        self.main = main
        self.pictures = pictures
        self.index = index
        self.picture = pictures[index]
        self.mode = 'single'
        self.units = ["mm", "Point", "in", "Pica", "Didot", "Cicero", 
                        "Device pixel"]
        self.unit = 0
        self.tmp_fld = main.workspace['temp']
        self.is_layouted = False

        self.ui = Ui_Printing(self)
        self.ui.show()
        QtCore.QCoreApplication.processEvents()
        self.viewer = self.ui.viewer
        self.scene = self.ui.scene
        self.layout = self.ui.layout
        self.page = Page(self, self.ui)
        self.printer = Printer(self)
        self.render = Render(self)

        self.ui.print_btn.clicked.connect(self.send_to_printer)
        self.ui.close_btn.clicked.connect(self.ui.accept)
        self.ui.preview.select_btn.clicked.connect(self.insert_image)
        self.ui.printer.printer_sett_btn.clicked.connect(
                                            self.get_printer_settings)
        self.ui.printer.orientation_cmb.currentIndexChanged.connect(
                                            self.set_orientation)
        self.ui.printer.color_cmb.currentIndexChanged.connect(self.set_color)
        self.ui.printer.apply_resol_btn.clicked.connect(self.set_resolution)
        self.layout.preset_size_cmb.currentIndexChanged.connect(
                                            self.render.resize_image)
        self.layout.width_spb.valueChanged.connect(self.on_width_changed)
        self.layout.height_spb.valueChanged.connect(self.on_height_changed)
        self.layout.apply_resize_btn.clicked.connect(self.apply_resize)
        self.layout.preset_pos_cmb.currentIndexChanged.connect(
                                                self.preset_location_image)
        self.layout.left_spb.valueChanged.connect(self.set_left_marge)
        self.layout.top_spb.valueChanged.connect(self.set_top_marge)
        self.layout.right_spb.valueChanged.connect(self.set_right_marge)
        self.layout.bottom_spb.valueChanged.connect(self.set_bottom_marge)
        self.ui.preview.configure(pictures, index)
        self.get_printer_settings()

    def get_printer_settings(self):
        reply = self.printer.get_settings()
        if not reply:
            return

        self.scene.set_page()
        self.is_layouted = True
        self.set_image()

    def insert_image(self):
        self.set_image()

    def set_image(self):
        """Copy the current image in /temp.

        """
        if not self.is_layouted:
            # There's no printer settings
            return

        self.image = self.pictures[self.ui.preview.current]
        # FIXME
        dest = os.path.join(self.tmp_fld.path, os.path.basename(self.picture))
        #dest = os.path.join(self.tmp_fld, os.path.basename(self.picture))
        shutil.copy(self.image, dest)
        self.render.show_image(dest)
        self.ui.preview.select_btn.setEnabled(False)
        self.ui.print_btn.setEnabled(True)
        self.layout.set_adjustment(0)

    def update_margins(self, x, y, w, h):
        """Update the margins spinBox.

        """
        self.page.margins = [x, y, w, h]
        self.layout.set_margin_values(x, y, w, h)

    def update_sizers(self):
        """Update the image size spinBox.

        """
        self.layout.set_size_values(self.render.render_size)

    def set_orientation(self, idx):
        """Change orientation.

        Args:
        idx -- 0 = portrait, 1 = landscape
        """
        self.render.change_orientation(idx)

    def set_color(self, idx):
        """Change color.

        Args:
        idx -- 0 = colors, 1 = grayscale
        """
        if not idx:
            self.set_image()

        else:
            img = QtGui.QImage(self.image)
            if self.ui.printer.orientation_cmb.currentIndex() == 1:
                matrix = QtGui.QTransform()       
                img = img.transformed(matrix.rotate(90), 
                                QtCore.Qt.SmoothTransformation)

            fn = os.path.splitext(os.path.split(self.image)[1])[0] + ".pgm" 
            dest = os.path.join(self.tmp_fld.path, fn)
            img.save(dest, "PGM")
            self.render.show_image(dest)

    def set_resolution(self):
        """Change resolution.

        """
        if not self.is_layouted:
            return

        self.printer.set_resolution(self.ui.printer.resol_spb.value())
        self.render.show_image(self.render.path)
        self.ui.printer.apply_resol_btn.setEnabled(False)

    def on_width_changed(self, val):
        """Change the value of image heigth spinbox.

        Args:
        val -- value of width spinbox
        """
        if not self.layout.proportion_chk.isChecked() or not self.is_layouted:
            return

        w, h = self.render.render_size
        if val == int(w):
            nh = h

        else:
            ratio = val / w
            nh = h * ratio
            if nh < 2:
                return
            self.layout.apply_resize_btn.setEnabled(True)

        self.layout.block_sizers_signal(True)
        self.layout.height_spb.setValue(nh)
        self.layout.block_sizers_signal(False)
        

    def on_height_changed(self, val):
        """Change the value of image width spinbox.

        Args:
        val -- value of heigth spinbox
        """
        if not self.layout.proportion_chk.isChecked() or not self.is_layouted:
            # Not proportionnal resize
            return

        w, h = self.render.render_size
        if val == int(h):
            # No change
            nw = w

        else:
            ratio = val / h
            nw = w * ratio
            if nw < 2:
                return
            self.layout.apply_resize_btn.setEnabled(True)

        self.layout.block_sizers_signal(True)
        self.layout.width_spb.setValue(nw)
        self.layout.block_sizers_signal(False)

    def apply_resize(self):
        """Call render.resize() func from user's entry.

        """
        self.render.resize_image(-1)

    def preset_location_image(self, idx):
        """Call render.move_to_preset() func from preset location.

        Args:
        idx -- current index of preset location comboBox
        """
        self.render.move_to_preset(idx)

    def set_left_marge(self, val):
        """Change left marge value.

        Args:
        val -- marge value 
        """
        if not self.is_layouted:
            return

        self.layout.block_margins_signal(True)
        dif = val - self.page.margins[0]
        self.layout.right_spb.setValue(self.layout.right_spb.value() - dif) 
        self.render.move_image(1, val)
        self.layout.block_margins_signal(False)
        self.page.margins[0] = val
        self.page.margins[2] -= dif  

    def set_top_marge(self, val):
        """Change top marge value.

        Args:
        val -- marge value
        """
        if not self.is_layouted:
            return

        self.layout.block_margins_signal(True)
        dif = val - self.page.margins[1]
        self.layout.bottom_spb.setValue(self.layout.bottom_spb.value() - dif) 
        self.render.move_image(2, val)
        self.layout.block_margins_signal(False)
        self.page.margins[1] += dif
        self.page.margins[3] -= dif

    def set_right_marge(self, val):
        """Change right margin value.

        Args:
        val -- marge value 
        """
        if not self.is_layouted:
            return

        self.layout.block_margins_signal(True)
        dif = val - self.page.margins[2]
        self.layout.left_spb.setValue(self.layout.left_spb.value() - dif) 
        self.layout.block_margins_signal(False)
        self.page.margins[2] += dif
        self.page.margins[0] -= dif
        self.render.move_image(1, self.page.margins[0])

    def set_bottom_marge(self, val):
        """Change bottom margin value.

        Args:
        val -- marge value 
        """
        if not self.is_layouted:
            return

        self.layout.block_margins_signal(True)
        dif = val - self.page.margins[3]
        self.layout.top_spb.setValue(self.layout.top_spb.value() - dif) 
        self.layout.block_margins_signal(False)
        self.page.margins[3] += dif
        self.page.margins[1] -= dif
        self.render.move_image(2, self.page.margins[1])

    def send_to_printer(self):
        """Send the image to the printer

        """
        self.printer.print_page()


class Page(object):
    def __init__(self, main, ui):
        dct = dict(main = main,
                    ui = ui,
                    size = (0, 0),         # Printable area
                    paper_size = (0, 0),   # Paper format not used ?
                    orientation = 0,       # Portrait
                    color = 0,             # Colors
                    resolution = 100,
                    units = main.units,
                    printer_name = None,   # not used ?
                    orig_size = (0, 0),    # Size of the original image in pixels
                    margins = [0, 0, 0, 0])

        for key, value in dct.items():
            setattr(self, key, value)

    def get_res_metric(self, inch=0):
        """Return value pixels/mm for the printer resolution

        """
        return inch / 25.4

    def set_page_settings(self):
        self.orientation = 0
        if self.paper_size[0] > self.paper_size[1]:
            self.orientation = 1
        self.ui.printer.orientation_cmb.blockSignals(True)
        self.ui.printer.orientation_cmb.setCurrentIndex(self.orientation)
        self.ui.printer.orientation_cmb.blockSignals(False)
        self.ui.printer.output_cmb.setCurrentIndex(0)
        self.ui.layout.set_param(self.main.unit, self.margins, self.paper_size)


class Render(object):
    def __init__(self, main):
        self.main = main
        self.scene = main.ui.scene
        self.Xorig = self.Yorig = 0
        shape = QtGui.QPixmap("medias/cross_cursor_32.png")
        self.cursor = QtGui.QCursor(shape, -1, -1)
        self.render_size = (0, 0)       # Size in unit of printed image
        self.orientation = 0             # XXX = page.orientation
        self.path = None
        # Convert :
        #   dpi to mm -- dpmm = scene.get_ratio()
        #   page (mm) to scene (pxl) -- pxl = mm * scene.get_zoom()
        #   scene (pxl) to page (mm) -- mm = pxl / scene.get_zoom() 
        

    def show_image(self, path=None):
        """Show current image.

        Args:
        path -- full path of image
        """
        if path:
            self.path = path

        self.i_read = QtGui.QImageReader(self.path)
        self.Xi = self.i_read.size().width()
        self.Yi = self.i_read.size().height()
        self.original_sizes = (self.Xi, self.Yi)
        self.Xorig = self.Yorig = 0
        r = self.scene.get_ratio()

        try:
            self.scene.removeItem(self.g_pix)
            del self.g_pix
        except:
            pass

        self.g_pix = ImageObject(self.main, path, None, self.scene)
        self.g_pix.setPos(0, 0)
        self.g_pix.setCursor(self.cursor)
        self.render_size = self.scene.render_size = (self.Xi / r, self.Yi / r)
        rg = self.main.page.size[0] - self.Xi / r
        bt = self.main.page.size[1] - self.Yi / r

        self.main.update_margins(0, 0, rg, bt)
        self.main.update_sizers()
        self.main.ui.layout.apply_resize_btn.setEnabled(False)
        self.main.ui.layout.preset_pos_cmb.blockSignals(True)
        self.main.ui.layout.preset_pos_cmb.setCurrentIndex(1)
        self.main.ui.layout.preset_pos_cmb.blockSignals(False)

    def move_image(self, direct, val):
        """Move image.

        Args:
        direct -- 1 = horizontal
                  2 = vertical
        val -- value in page units
        """
        val *= self.scene.get_zoom()
        self.Xorig, self.Yorig = self.g_pix.x(), self.g_pix.y()

        if direct == 1:
            self.Xorig = val

        elif direct == 2:
            self.Yorig = val

        self.g_pix.setPos(self.Xorig, self.Yorig)
        self.main.layout.preset_pos_cmb.setCurrentIndex(0)

    def move_to_preset(self, mode):
        """Move the image to a preset place.

        Args:
        mode -- place
        """
        if mode == 0:
            # Undefined
            return

        Xdelta = self.main.page.size[0] - self.render_size[0]
        Ydelta = self.main.page.size[1] - self.render_size[1]

        if mode == 1:
            # Top left
            x = y = 0

        elif mode == 2:
            # Top right
            x = Xdelta
            y = 0

        elif mode == 3:
            # Center
            x = Xdelta / 2
            y = Ydelta / 2

        elif mode == 4:
            # Bottom left
            x = 0
            y = Ydelta

        else:
            # Bottom right
            x = Xdelta 
            y = Ydelta

        self.main.layout.left_spb.setValue(x)
        self.main.layout.top_spb.setValue(y)
        self.main.layout.preset_pos_cmb.blockSignals(True)
        self.main.layout.preset_pos_cmb.setCurrentIndex(mode)
        self.main.layout.preset_pos_cmb.blockSignals(False)            

    def change_orientation(self, orient):
        """Change the orientation of the image.

        Args:
        orient -- 0 landscape
                  1 portrait
        """
        matrix = QtGui.QTransform()

        if self.orientation:
            deg = -90

        else:
            deg = 90

        i = QtGui.QImage(self.path)
        self.orientation = orient
        img = i.transformed(matrix.rotate(deg), QtCore.Qt.SmoothTransformation)
        img.save(self.path)
        self.show_image(self.path)

    def resize_image(self, mode):
        """Resize the image.

        Args:
        mode -- -1 values in spinBox
                 0 original sizes
                 1 full page
                 2 full width
                 3 full height
        """
        if self.path is None:
            return

        if mode == 0:
            try:
                self.main.scene.removeItem(self.g_pix)
                del self.g_pix
            except:
                pass

            self.main.set_image()
            if self.orientation:
                self.orientation = 0
                self.change_orientation(1)

            return

        r = self.main.scene.get_ratio()
        p_w = self.main.page.size[0] * r
        p_h = self.main.page.size[1] * r
        r_w, r_h = p_w / self.Xi, p_h / self.Yi

        if mode == -1:
            Xi = self.main.layout.width_spb.value()
            Yi = self.main.layout.height_spb.value()
            width, height = Xi * r, Yi * r

        elif mode == 1:
            if r_w > r_h:
                width, height = self.Xi * r_w, self.Yi * r_w

            else:
                width, height = self.Xi * r_h, self.Yi * r_h

        elif mode == 2:
            width, height = self.Xi * r_w, self.Yi * r_w
        
        else:
            width, height = self.Xi * r_h, self.Yi * r_h

        img = QtGui.QImage(self.main.image)

        if self.orientation:
            matrix = QtGui.QMatrix()
            img = img.transformed(matrix.rotate(90), 
                                    QtCore.Qt.SmoothTransformation)
                
        pix = img.scaled(width, height, 
                            QtCore.Qt.KeepAspectRatio, 
                            QtCore.Qt.SmoothTransformation)
        pix.save(self.path)
        self.main.layout.preset_pos_cmb.blockSignals(True)
        self.main.layout.preset_pos_cmb.setCurrentIndex(0)
        self.main.layout.preset_pos_cmb.blockSignals(False)
        self.show_image(self.path)


class ImageObject(QtWidgets.QGraphicsPixmapItem):
    def __init__(self, main, path, parent=None, scene=None):
        super().__init__(parent)
        self.main = main
        i_read = QtGui.QImageReader(path)
        Xi, Yi = i_read.size().width(), i_read.size().height()
        r, z = self.main.scene.get_ratio(), self.main.scene.get_zoom()
        self.Xprint_size, self.Yprint_size = Xi / r, Yi / r
        self.Xrender, self.Yrender = (Xi / r) * z, (Yi / r) * z
        img = QtGui.QImage(path)
        pix = QtGui.QPixmap.fromImage(img.scaled(self.Xrender, self.Yrender, 
                            QtCore.Qt.KeepAspectRatio, 
                            QtCore.Qt.SmoothTransformation))
        self.setPixmap(pix)

        self.setFlags(QtWidgets.QGraphicsItem.ItemIsMovable | 
                        QtWidgets.QGraphicsItem.ItemIsSelectable)
        self.setTransformationMode(QtCore.Qt.SmoothTransformation)
        self.r, self.z = r, z
        self.moved = False
        scene.addItem(self)

    def mousePressEvent(self, event):
        if event.button() != QtCore.Qt.LeftButton:
            event.ignore()
            return

        self.oldX = event.scenePos().x()
        self.oldY = event.scenePos().y()
        event.accept()

    def mouseReleaseEvent(self, event):
        if not self.moved:
            event.ignore()
            return

        self.update_marges()
        self.moved = False
        event.accept()

    def update_marges(self):
        l, t = (self.x() / self.z), (self.y() / self.z)
        r = self.main.page.size[0] - (l + self.Xprint_size)
        b = self.main.page.size[1] - (t + self.Yprint_size)
        self.main.page.margins = [l, t, r, b]
        self.main.layout.block_margins_signal(True)
        self.main.layout.left_spb.setValue(l)
        self.main.layout.top_spb.setValue(t)
        self.main.layout.right_spb.setValue(r)
        self.main.layout.bottom_spb.setValue(b)
        self.main.layout.block_margins_signal(False)

    def mouseMoveEvent(self, event):
        posX, posY = event.scenePos().x(), event.scenePos().y() 

        if self.x() < -(self.Xrender - 10):
            mX = 2

        elif self.x() + 10 > self.main.scene.sceneRect().width():
            mX = -2

        else:
            mX = posX - self.oldX

        if self.y() < -(self.Yrender - 10):
            mY = 2

        elif self.y() + 10 > self.main.scene.sceneRect().height():
            mY = -2

        else:
            mY = posY - self.oldY

        self.oldX, self.oldY = posX, posY
        self.moveBy(mX, mY)
        self.moved = True
        self.main.layout.preset_pos_cmb.setCurrentIndex(0)
        event.accept()


class Printer(QtPrintSupport.QPrinter):
    def __init__(self, main):
        super(Printer, self).__init__(QtPrintSupport.QPrinter.ScreenResolution)
        self.main = main
        self.setResolution(100)
        self.copies = 1
        self.infos = QtPrintSupport.QPrinterInfo(self)

        if self.infos.isNull():
            # XXX Show warning ?
            main.ui.printer.output_cmb.insertItem(0, 'No printer')
            self.cur_printer = None
            return 

        printers = [p for p in self.infos.availablePrinters()]
        self.printers_name = [p.printerName() for p in printers]
        self.cur_printer = self.infos.defaultPrinter().printerName()
        self.printers_name.remove(self.cur_printer)
        self.printers_name.insert(0, self.cur_printer)
        self.setCreator("www.oqapy.eu")
        main.ui.printer.output_cmb.insertItems(0, self.printers_name)

    def get_settings(self):
        """Open page setup dialog.

        """
        self.margins = []
        units = ["mm", "Point", "in", "Pica", "Didot", "Cicero", 
                        "Device pixel"]
        # XXX It seems the unit is always in locale units
        p_dialog = QtPrintSupport.QPageSetupDialog(self)
        reply = p_dialog.exec_()

        if not reply:
            return

        else:
            self.unit = self.Unit()   # XXX = self.main.unit

            if self.unit == 2:
                r, s = 2, 0.01

            else:
                r, s = 0, 1.0

            w_page = round(self.paperSize(self.unit).width(), r)
            h_page = round(self.paperSize(self.unit).height(), r)
            
            for m in self.getPageMargins(self.unit):
                self.margins.append(m)

        self.main.page.size = (round(self.pageRect(self.unit).width(), r),
                        round(self.pageRect(self.unit).height(), r))
        self.main.page.paper_size = (w_page, h_page)
        self.main.unit = self.unit
        self.main.page.set_page_settings()

        return True

    def set_resolution(self, val):
        """Set the printer's resolution.

        Args:
        val -- resolution in dpi
        """
        self.setResolution(val)
        self.main.page.resolution = val

    def set_disc(self, p):
        """Set parametres for print on cd, dvd.

        Args:
        p -- 0 = paper
             1 = disc 5 inch 116 x 116 mm
             2 = disc 3 inch 65 x 65 mm
        """
        if not p:
            self.main.get_page_sizes()

        elif p == 1:
            pass

        else:
            pass

    def print_page(self):
        """Send the image to printer.

        """
        if self.main.ui.printer.color_cmb.currentIndex():
            self.setColorMode(QtPrintSupport.QPrinter.GrayScale)

        self.copies = self.main.ui.copies_spb.value()

        # Define the source rectangle
        image = QtGui.QImage(self.main.render.path)
        r = self.main.scene.get_ratio()
        Xs, Ys, Ws, Hs = 0, 0, self.main.render.Xi, self.main.render.Yi
        
        if self.main.page.margins[0] < 0:
            Xs = abs(self.main.page.margins[0]) * r

        if self.main.page.margins[1] < 0:
            Ys = abs(self.main.page.margins[1]) * r

        if self.main.page.margins[2] < 0:
            Ws += self.main.page.margins[2] * r

        if self.main.page.margins[3] < 0:
            Hs += self.main.page.margins[3] * r

        Ws -= Xs
        Hs -= Ys
        source = QtCore.QRectF(Xs, Ys, Ws, Hs)

        # Define the point origin of destination rectangle
        Xd, Yd = 0.0, 0.0
        if self.main.page.margins[0] > 0:
            Xd = self.main.page.margins[0] * r

        if self.main.page.margins[1] > 0:
            Yd = self.main.page.margins[1] * r

        target = QtCore.QRectF(Xd, Yd, Ws, Hs)

        out = self.main.ui.printer.output_cmb.currentIndex()
        if out == 0:
            if self.cur_printer is None:
                return

            self.setOutputFormat(QtPrintSupport.QPrinter.NativeFormat)
            self.setOutputFileName('')

        elif out == 1:
            reply = self.get_file_name(".pdf")
            if not reply:
                return

            self.setOutputFormat(QtPrintSupport.QPrinter.PdfFormat)
            self.setOutputFileName(reply)

        elif out == 2:
            reply = self.get_file_name(".ps")
            if not reply:
                return

            self.setOutputFormat(QtPrintSupport.QPrinter.PostScriptFormat)
            self.setOutputFileName(reply)

        for i in range(self.copies):
            painter = QtGui.QPainter()
            painter.begin(self)
            painter.drawImage(target, image, source)
            painter.end()

            # Avoid unusued last blank sheet
            if not i == self.copies - 1:
                self.newPage()

    def get_file_name(self, format):
        f = os.path.join(os.path.splitext(self.main.image)[0], format)
        name = QtWidgets.QFileDialog.getSaveFileName(
                        None, "Save File", f,
                        "Files (*.%s)" % format)[0]

        if os.path.isfile(name):
            if not self.ask_replace(name):
                return False

        return name

    def ask_replace(self, f):
        """Show message if the file already exists in destination's folder.

        Args:
        f -- file name

        Return:
        True if file must be rewrite, otherwise False
        """
        mssg = QtWidgets.QMessageBox()
        mssg.setIcon(QtWidgets.QMessageBox.Question)
        mssg.setText(_("A file with the same name already exists in the" 
                    " destination folder."))
        mssg.setInformativeText(_("Do you want to replace the existing file?"))
        can = QtWidgets.QPushButton(_("Cancel"))
        mssg.addButton(can, QtWidgets.QMessageBox.ActionRole)
        rmp = QtWidgets.QPushButton(_("Replace"))
        mssg.addButton(rmp, QtWidgets.QMessageBox.ActionRole)
        mssg.setDefaultButton(can)
        reply = mssg.exec_()

        if reply == 0:
            return False

        return True


if __name__ == "__main__":
    app = QtGui.QApplication(sys.argv)
    pr = Printing()
    sys.exit(app.exec_())
