# -*- coding: utf-8 -*-

# ui_printing.py
# this file is part of Oqapy serie 3
#
# Author: Vincent Vande Vyvre <vincent.vandevyvre@oqapy.eu>
# Home: http://www.oqapy.eu
# Copyright: 2010-2017 Vincent Vande Vyvre
# Licence: GPL3
#
# Main gui for printing dialog box.

import sys

from PyQt5.QtCore import Qt, QMetaObject, QSize
from PyQt5.QtGui import (QIcon, QPixmap, QPen, QBrush, QColor, QImage, 
                         QImageReader)
from PyQt5.QtWidgets import (QDialog, QGridLayout, QVBoxLayout, QHBoxLayout,
                             QSplitter, QSpacerItem, QSizePolicy, QGraphicsView,
                             QLabel, QSpinBox, QPushButton, QGraphicsScene,
                             QWidget, QGraphicsPixmapItem, QGraphicsItem)
import stylesheets as styles

from VWidgets_PyQt5.vtoolpanel import VToolPanel
from printing.previewpage import PreviewPage
from printing.printerpage import PrinterPage
from printing.layoutpage import LayoutPage


class Ui_Printing(QDialog):
    def __init__(self, main, parent=None):
        super(Ui_Printing, self).__init__(parent)
        self.setObjectName("self")
        self.resize(1000, 700)
        self.gridLayout = QGridLayout(self)
        self.gridLayout.setContentsMargins(0, 2, 2, 2)
        self.splitter = QSplitter(self)
        self.splitter.setOrientation(Qt.Horizontal)
        self.splitter.setObjectName("splitter")
        self.wdg = QWidget(self.splitter)
        self.vlayout = QVBoxLayout(self.wdg)
        self.viewer = QGraphicsView(self)
        self.viewer.setObjectName("viewer")
        self.vlayout.addWidget(self.viewer)
        self.hlayout = QHBoxLayout()
        spc = QSpacerItem(40, 20, QSizePolicy.Expanding, QSizePolicy.Minimum)
        self.hlayout.addItem(spc)
        self.copies_lbl = QLabel(self)
        self.hlayout.addWidget(self.copies_lbl)
        self.copies_spb = QSpinBox(self)
        self.copies_spb.setMaximum(9999)
        self.copies_spb.setValue(1)
        self.copies_spb.setObjectName("copies_spb")
        self.hlayout.addWidget(self.copies_spb)
        self.print_btn = QPushButton(self)
        icon = QIcon()
        icon.addPixmap(QPixmap("medias/dialog-apply.svg"), 
                        QIcon.Normal, QIcon.Off)
        self.print_btn.setIcon(icon)
        self.print_btn.setEnabled(False)
        self.print_btn.setObjectName("print_btn")
        self.hlayout.addWidget(self.print_btn)
        self.close_btn = QPushButton(self)
        icon1 = QIcon()
        icon1.addPixmap(QPixmap("medias/close.png"), 
                        QIcon.Normal, QIcon.Off)
        self.close_btn.setIcon(icon1)
        self.close_btn.setObjectName("close_btn")
        self.hlayout.addWidget(self.close_btn)
        self.vlayout.addLayout(self.hlayout)
        self.tools = VToolPanel(self.splitter)
        self.tools.setMaximumSize(QSize(300, 16777215))
        self.tools.setTitle("Settings")
        self.tools.setObjectName("tools")
        self.gridLayout.addWidget(self.splitter, 0, 0, 1, 1)
        self.setSizeGripEnabled(True)

        self.retranslateUi()
        QMetaObject.connectSlotsByName(self)
        self.main = main
        #self.set_theme()
        self.set_tool_pages()
        self.scene = GraphicsScene(self, self.viewer)
        self.viewer.setScene(self.scene)

    def set_tool_pages(self):
        """Sets the tool pages into the tool panel.

        How to add a tool page:
            - Create, if needed, a new tool group with:
                    group = VToolPanel.addToolGroup(name, icon)
            - Create a new page to this group with:
                    myNewPage = VToolPanel.addToolPage(group)
            - Set the properties of the page (title, icon, collapsible, ...)
            - Instanciate the tool widget with the page as parent:
                    myWidget = MyWidget(myNewPage)
            - Place the tool widget into the page layout with:
                    myNewPage.layout.addWidget(myWidget)
        """
        self.grp_0 = self.tools.addToolPage('grp_0')
        self.preview_pg = self.tools.addToolBox(0)
        self.preview_pg.setTitle(_('Image preview'))
        self.preview_pg.header.setStyleSheet(styles.HEADER)
        self.preview_pg.header.label.setStyleSheet(styles.TITLE)
        self.preview_pg.header.collapse_btn.setStyleSheet(styles.EXPAND)
        self.preview_pg.header.set_expanding_buttons(
                                        'medias/collapse_white_down.png',
                                        'medias/collapse_white_left.png',
                                        'medias/collapse_white_right.png')
        self.preview = PreviewPage(self.preview_pg)
        self.preview_pg.layout.addWidget(self.preview)

        self.printer_pg = self.tools.addToolBox(0)
        self.printer_pg.setTitle(_('Printer'))
        self.printer_pg.header.setStyleSheet(styles.HEADER)
        self.printer_pg.header.label.setStyleSheet(styles.TITLE)
        self.printer_pg.header.collapse_btn.setStyleSheet(styles.EXPAND)
        self.printer_pg.header.set_expanding_buttons(
                                        'medias/collapse_white_down.png',
                                        'medias/collapse_white_left.png',
                                        'medias/collapse_white_right.png')
        self.printer = PrinterPage(self.printer_pg)
        self.printer_pg.layout.addWidget(self.printer)

        self.layout_pg = self.tools.addToolBox(0)
        self.layout_pg.setTitle(_('Page layout'))
        self.layout_pg.header.setStyleSheet(styles.HEADER)
        self.layout_pg.header.label.setStyleSheet(styles.TITLE)
        self.layout_pg.header.collapse_btn.setStyleSheet(styles.EXPAND)
        self.layout_pg.header.set_expanding_buttons(
                                        'medias/collapse_white_down.png',
                                        'medias/collapse_white_left.png',
                                        'medias/collapse_white_right.png')
        self.layout = LayoutPage(self.layout_pg)
        self.layout_pg.layout.addWidget(self.layout)
        self.tools.setCurrentPage(0)

    def get_viewer_size(self):
        return self.viewer.width(), self.viewer.height()

    def retranslateUi(self):
        self.setWindowTitle(_("Printing - Oqapy"))
        self.copies_lbl.setText(_('Copies:'))
        self.print_btn.setText(_('Print'))
        self.close_btn.setText(_('Close'))

    def set_theme(self):
        self.setStyleSheet("""QWidget {background: #181E26;
                        color: #FFFFFF;}""")
        self.viewer.setStyleSheet(styles.VIEWER)
        self.viewer.horizontalScrollBar().setStyleSheet(styles.HSCROLLBAR)
        self.viewer.verticalScrollBar().setStyleSheet(styles.VSCROLLBAR)
        self.copies_spb.setStyleSheet(styles.SPINBOX)
        self.print_btn.setStyleSheet(styles.PUSHBUTTON)
        self.close_btn.setStyleSheet(styles.PUSHBUTTON)
        self.tools.scrollArea.setStyleSheet(styles.SCROLLAREA)
        self.tools.header.setStyleSheet(styles.HEADER)
        self.tools.header.label.setStyleSheet(styles.TITLE)
        self.tools.scrollArea.horizontalScrollBar().setStyleSheet(
                                                    styles.HSCROLLBAR)
        self.tools.scrollArea.verticalScrollBar().setStyleSheet(
                                                    styles.VSCROLLBAR)

class GraphicsScene(QGraphicsScene):
    def __init__(self, ui, view, parent=None):
        super().__init__(parent)
        self.ui = ui
        self.viewer = view
        # Size in unit of printed image
        self.render_sizes = (0, 0)
        self._sizes = (0, 0)

    def set_page(self):
        #XXX page size may be in inch
        self.clear()
        ratio = self.get_ratio()
        v_size = self.ui.get_viewer_size()
        p_size = self.ui.main.page.size
        if p_size[0] <= v_size[0] / ratio:
            if p_size[1] <= v_size[1] / ratio:
                Xs, Ys = p_size[0] * ratio, p_size[1] * ratio

            else:
                Ys = v_size[1]
                Xs = p_size[0] * (v_size[1] / p_size[1])

        else:
            if (v_size[0] / p_size[0]) <= (v_size[1] / p_size[1]):
                Xs = v_size[0]
                Ys = p_size[1] * (v_size[0] / p_size[0])

            else:
                Ys = v_size[1]
                Xs = p_size[0] * (v_size[1] / p_size[1])

        # Lets 5 pixels free around the frame
        Xs -= 10
        Ys -= 10
        self.setSceneRect(0.0, 0.0, Xs, Ys)
        pen = QPen(QColor("white"))
        brush = QBrush(QColor("white"), Qt.SolidPattern)
        self.addRect(0.0, 0.0, Xs, Ys, pen, brush)

        try:
            self.removeItem(self.border)
            del self.g_pix
        except:
            pass

        pix = QPixmap("medias/page_frame_256.png").scaled(Xs, Ys)
        self.border = QGraphicsPixmapItem(None)
        self.addItem(self.border)
        self.border.setPixmap(pix)
        self.border.setZValue(99)
        self._sizes = (Xs, Ys)

    def get_size(self):
        """Return size of graphics scene

        """
        return self.sceneRect()

    def get_ratio(self):
        """Return value pixels/mm for the printer resolution

        """
        return self.ui.main.page.resolution / 25.4

    def get_zoom(self):
        """Return current zoom for the page

        """
        return self.sceneRect().width() / self.ui.main.page.size[0]


class ImageObject(QGraphicsPixmapItem):
    def __init__(self, main, path, parent=None, scene=None):
        super().__init__(parent)
        if scene is not None: 
            scene.addItem(self)

        self.main = main
        i_read = QImageReader(path)
        Xi, Yi = i_read.size().width(), i_read.size().height()
        r, z = self.main.scene.get_ratio(), self.main.scene.get_zoom()
        self.Xprint_size, self.Yprint_size = Xi / r, Yi / r
        self.Xrender, self.Yrender = (Xi / r) * z, (Yi / r) * z
        img = QImage(path)
        pix = QPixmap.fromImage(img.scaled(self.Xrender, self.Yrender, 
                                Qt.KeepAspectRatio, 
                                Qt.SmoothTransformation))
        self.setPixmap(pix)

        self.setFlags(QGraphicsItem.ItemIsMovable | 
                      QGraphicsItem.ItemIsSelectable)
        self.setTransformationMode(Qt.SmoothTransformation)
        self.r, self.z = r, z
        self.moved = False

    def mousePressEvent(self, event):
        if event.button() != Qt.LeftButton:
            event.ignore()
            return

        self.oldX = event.scenePos().x()
        self.oldY = event.scenePos().y()
        event.accept()

    def mouseReleaseEvent(self, event):
        if not self.moved:
            event.ignore()
            return

        self.update_marges()
        self.moved = False
        event.accept()

    def mouseMoveEvent(self, event):
        posX, posY = event.scenePos().x(), event.scenePos().y() 

        if self.x() < -(self.Xrender - 10):
            mX = 2

        elif self.x() + 10 > self.main.scene.sceneRect().width():
            mX = -2

        else:
            mX = posX - self.oldX

        if self.y() < -(self.Yrender - 10):
            mY = 2

        elif self.y() + 10 > self.main.scene.sceneRect().height():
            mY = -2

        else:
            mY = posY - self.oldY

        self.oldX, self.oldY = posX, posY
        self.moveBy(mX, mY)
        self.moved = True
        self.main.preset_pos_cmb.setCurrentIndex(0)
        event.accept()
