# -*- coding: utf-8 -*-

# settings.py
# This file is part of Oqapy serie 3
#
# Author: Vincent Vande Vyvre <vincent.vandevyvre@oqapy.eu>
# Home: http://www.oqapy.eu
# Copyright: 2009-2017 Vincent Vande Vyvre
# Licence: GPL3

# Settings dialog


import sys
import os
import json
import logging
logger = logging.getLogger(__name__)

from PyQt5.QtCore import QCoreApplication, QSize, pyqtSignal, Qt
from PyQt5.QtGui import QIcon, QFont, QPixmap
from PyQt5.QtWidgets import (QDialog, QGridLayout, QHBoxLayout, QVBoxLayout, 
                             QLabel, QSpacerItem, QSizePolicy, QPushButton,
                             QFileDialog, QToolButton, QWidget)

import stylesheets as style

from settings.startpage import StartPage
from settings.viewingpage import ViewingPage
from settings.sortingpage import SortingPage
from settings.zoompage import ZoomPage
from settings.trailerpage import TrailerPage
from settings.croppage import CropPage
from settings.magnifierpage import MagnifierPage
from settings.g9npage import G9nPage
from settings.gridpage import GridPage
from data import CONFIG_DEFAULT
from warningmessages import WarningMessages


PAGES = [StartPage, ViewingPage, SortingPage, TrailerPage, ZoomPage,
            CropPage, GridPage, MagnifierPage, G9nPage]

CATEGORIES = ["start", "viewing", "sorting", "trailer", "zoom",
                "crop", "grid", "magnifier", "Geolocation"]

ICONS = ["oqapy_ic_48.png", "viewing_mode_128x64.png", 
            "multifolder_128x111.png", "trailer_297x117.png",
            "telescope_64.png", "easel_128.png",
            "grid_46.png", "magnifier_36x30.png", "world_54.png"]


class Configuration(QDialog):
    def __init__(self, core, parent=None):
        super().__init__(parent)
        logger.info('Init config dialog')
        self.core = core
        self.resize(780, 680)
        self.setSizeGripEnabled(True)
        pal = style.get_palette()
        self.setPalette(pal)
        self.gridLayout = QGridLayout(self)
        self.h_layout = QHBoxLayout()
        self.menu_area_widget = QWidget(self)
        self.menu_area_widget.setMinimumSize(QSize(210, 16777215))
        self.v_layout = QVBoxLayout(self.menu_area_widget)

        font = QFont()
        font.setPointSize(12)
        font.setWeight(75)
        font.setBold(True)
        font.setItalic(True)

        self.buttons = []

        self.general_lbl = QLabel(self.menu_area_widget)
        self.general_lbl.setFont(font)
        self.v_layout.addWidget(self.general_lbl)

        for idx, cat in enumerate(CATEGORIES[:4]):
            button = Button(idx, self.menu_area_widget)
            self.v_layout.addWidget(button)
            self.buttons.append(button)
            button.selected.connect(self.change_page)

        self.tools_lbl = QLabel(self.menu_area_widget)
        self.tools_lbl.setFont(font)
        self.v_layout.addWidget(self.tools_lbl)

        for idx, cat in enumerate(CATEGORIES[4:9]):
            button = Button(idx + 4, self.menu_area_widget)
            self.v_layout.addWidget(button)
            self.buttons.append(button)
            button.selected.connect(self.change_page)

        spacerItem5 = QSpacerItem(20, 40, QSizePolicy.Minimum,
                                        QSizePolicy.Expanding)
        self.v_layout.addItem(spacerItem5)
        self.h_layout.addWidget(self.menu_area_widget)
        self.verticalLayout_4 = QVBoxLayout()

        self.horizontalLayout_2 = QHBoxLayout()
        spacerItem6 = QSpacerItem(40, 20, QSizePolicy.Expanding,
                                        QSizePolicy.Minimum)
        self.horizontalLayout_2.addItem(spacerItem6)
        self.reset_btn = QPushButton(self)
        icon1 = QIcon()
        icon1.addPixmap(QPixmap("medias/gtk-refresh.png"), 
                                        QIcon.Normal, QIcon.Off)
        self.reset_btn.setIcon(icon1)
        self.reset_btn.setEnabled(False)
        self.horizontalLayout_2.addWidget(self.reset_btn)
        self.apply_btn = QPushButton(self)
        icon2 = QIcon()
        icon2.addPixmap(QPixmap("medias/apply.svg"), 
                                        QIcon.Normal, QIcon.Off)
        self.apply_btn.setIcon(icon2)
        self.apply_btn.setEnabled(False)
        self.horizontalLayout_2.addWidget(self.apply_btn)
        self.close_btn = QPushButton(self)
        icon3 = QIcon()
        icon3.addPixmap(QPixmap("medias/close.png"), QIcon.Normal, 
                                        QIcon.Off)
        self.close_btn.setIcon(icon3)
        self.horizontalLayout_2.addWidget(self.close_btn)

        self.pages = []
        for p in PAGES:
            page = p(self)
            self.verticalLayout_4.addWidget(page)
            self.pages.append(page)
            page.hide()

        self.pages[0].show()
        self.current_page = 0
        self.verticalLayout_4.addLayout(self.horizontalLayout_2)
        self.h_layout.addLayout(self.verticalLayout_4)

        self.gridLayout.addLayout(self.h_layout, 0, 0, 1, 1)

        self.settings_changed = False
        self.reset_btn.clicked.connect(self.reset)
        self.apply_btn.clicked.connect(self.apply_settings)
        self.close_btn.clicked.connect(self._close)
        self._update_settings()
        self.retranslateUi()
        self.set_style()
        self.exec_()

    def _update_settings(self):
        self.cfg = self.core.cfg.load_config()
        self.pages[3].temp =  self.core.workspace['temp']
        self.pages[8].temp = self.core.workspace['g9n']
        from data import SHORTCUTS
        for page in self.pages:
            page.set_config_values(self.cfg, SHORTCUTS)

    def _close(self):
        QCoreApplication.processEvents()
        if self.settings_changed:
            #self.debug()
            jsn = json.dumps(self.cfg, sort_keys=True, indent=4, 
                         separators=(',', ': '), ensure_ascii=False)
            try:
                with open(self.core.cfg.cfg_file, 'wb') as outfile:
                    outfile.write(jsn.encode('utf-8', 'replace'))
                    logger.info("Configuration saved")
            except Exception as why:
                logger.warning('Unable to save config file.\n\tReason: {0}'
                                .format(why))
                message = {'context': _('Save settings'),
                    'warn': _("Unable to save file."),
                    'data': self.core.cfg.cfg_file,
                    'msg': _('Reason:'),
                    'data1': str(why),
                    'btn': _('Close')}
                warn = WarningMessages(0, message)
                warn.build_dialog()

            else:
                self.core.reload_config()
                self.done(True)

        self.done(False)

    def debug(self):
        # Debugging function
        for key in self.cfg:
            print("%s\t%s" %(key, self.cfg[key]))

    def apply_settings(self):
        for page in self.pages:
            page.apply_change()
        self.settings_changed = True
        self.reset_btn.setEnabled(True)

    def reset(self):
        self._update_settings()
        self.settings_changed = False
        self.reset_btn.setEnabled(False)        

    def change_page(self, idx):
        self.pages[self.current_page].hide()
        self.buttons[self.current_page].set_selected(False)
        self.pages[idx].show()
        self.buttons[idx].set_selected(True)
        self.current_page = idx
        # Need to show the map viewer to avoid Java crash
        QCoreApplication.processEvents()
        if idx == 8:
            logger.info('Show the map viewer')
            self.pages[8].locate()

    def value_changed(self):
        self.apply_btn.setEnabled(True)

    def get_dir(self, folder=None):
        """Launch QFileDialog to open a directory. 

        Returns:
        directory's path
        """
        if folder is None:
            folder = self.core.last_open

        d = QFileDialog.getExistingDirectory(None, 
                                _("Open source folder"),
                                folder, 
                                QFileDialog.DontResolveSymlinks
                                | QFileDialog.ShowDirsOnly)
        return d

    def resizeEvent(self, event):
        """Set the width of the menu using the Geolocation button width.

        """
        w = self.buttons[8].size().width()
        if self.menu_area_widget.width() < w:
            self.menu_area.setMaximumSize(QSize(w + 20, 16777215))

        QDialog.resizeEvent(self, event)

    def retranslateUi(self):
        self.setWindowTitle(_("Oqapy - Configuration"))
        self.general_lbl.setText(_("General"))
        self.buttons[0].setText(_("&Startup"))
        self.buttons[0].setShortcut(_("S"))
        self.buttons[1].setText(_("&Viewing"))
        self.buttons[1].setShortcut(_("V"))
        self.buttons[2].setText(_("S&orting"))
        self.buttons[2].setShortcut(_("O"))
        self.buttons[3].setText(_("&Trailer"))
        self.buttons[3].setShortcut(_("Z"))
        self.buttons[4].setText(_("&Zoom"))
        self.buttons[4].setShortcut(_("T"))
        self.tools_lbl.setText(_("Tools"))
        #self.buttons[5].setText(_("Aeria&l view"))
        #self.buttons[5].setShortcut(_("L"))
        self.buttons[5].setText(_("&Crop"))
        self.buttons[5].setShortcut(_("C"))
        self.buttons[6].setText(_("G&rid"))
        self.buttons[6].setShortcut(_("R"))
        self.buttons[7].setText(_("&Magnifier"))
        self.buttons[7].setShortcut(_("M"))
        self.buttons[8].setText(_("&Geolocation"))
        self.buttons[8].setShortcut(_("G"))
        self.reset_btn.setToolTip(_("Reset to default values."))
        self.reset_btn.setText(_("&Reset"))
        self.reset_btn.setShortcut(_("Alt+R"))
        self.apply_btn.setToolTip(_("Apply new config."))
        self.apply_btn.setText(_("&Apply"))
        self.apply_btn.setShortcut(_("Alt+A"))
        self.close_btn.setToolTip(_("Close config dialog."))
        self.close_btn.setText(_("&Close"))
        self.close_btn.setShortcut(_("Alt+C"))

    def set_style(self):
        self.menu_area_widget.setStyleSheet("""QWidget 
                        {background-color: qlineargradient(
                        x1: 0, y1: 0, x2: 0, y2: 1,
                        stop: 0 #F4F4F4, stop: 0.4 #E8E8E8,
                        stop: 0.5 #E8E8E8, stop: 1.0 #F4F4F4);
                        border: 2px solid #8A8E96;
                        border-radius: 4px;
                        color: #000000;}""")
        label = """QLabel {color: #555555;
                background-color: transparent;
                border-color: transparent;}"""
        self.general_lbl.setStyleSheet(label)
        self.tools_lbl.setStyleSheet(label)
        self.reset_btn.setStyleSheet(style.PUSHBUTTONDIAL)
        self.apply_btn.setStyleSheet(style.PUSHBUTTONDIAL)
        self.close_btn.setStyleSheet(style.PUSHBUTTONDIAL)


class Button(QToolButton):
    selected = pyqtSignal(int)
    def __init__(self, idx, parent=None):
        super().__init__(parent)
        self.idx = idx
        font = QFont()
        font.setPointSize(11)
        sizePolicy = QSizePolicy(QSizePolicy.Preferred, 
                                    QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(1)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(False)
        btn_style = Qt.ToolButtonTextBesideIcon
        size = QSize(0, 30)
        p = QPixmap(os.path.join("medias", ICONS[idx]))
        icon = QIcon()
        icon.addPixmap(p)
        name = "".join([CATEGORIES[idx], "_btn"])

        self.setSizePolicy(sizePolicy)
        self.setMinimumSize(size)
        self.setFont(font)
        self.setIcon(icon)
        self.setIconSize(QSize(36, 36))
        self.setToolButtonStyle(btn_style)
        self.static_style = """ 
                QToolButton {
                border: 2px solid #aac5e0;
                border-radius: 6px;
                background-color: qlineargradient(x1: 0, y1: 0, x2: 0, y2: 1,
                                           stop: 0 #f0f0f0, stop: 1 #babdb6);
                min-width: 80px;
                }

                QToolButton:pressed {
                 background-color: qlineargradient(x1: 0, y1: 0, x2: 0, y2: 1,
                                           stop: 0 #babdb6, stop: 1 #f0f0f0);
                }

                QToolButton:hover {
                 background-color: qlineargradient(x1: 0, y1: 0, x2: 0, y2: 1,
                                           stop: 0 #f4f4f4, stop: 1 #eeeeee);
                }

                QToolButton:default {
                 border-color: navy; /* make the default button prominent */
                }"""
        self.selected_style = """ QToolButton {
                border: 2px solid #7799e0;
                border-radius: 6px;
                background-color: qlineargradient(x1: 0, y1: 0, x2: 0, y2: 1,
                                           stop: 0 #eeeeee, stop: 1 #c8c8c8);
                color: #0000ff;
                min-width: 80px;
                }"""
        self.setStyleSheet(self.static_style)
        self.setAutoRaise(True)
        self.setObjectName(name)
        self.pressed.connect(self.on_mouse_pressed)

    def set_selected(self, b):
        """Change the button appearance when (un)selected.

        Args:
        b -- boolean
        """
        if b:
            self.setStyleSheet(self.selected_style)

        else:
            self.setStyleSheet(self.static_style)

    def on_mouse_pressed(self):
        self.selected.emit(self.idx)

