# -*- coding: utf-8 -*-

# shredder.py
# This file is part of Oqapy serie 3
#
# Author: Vincent Vande Vyvre <vincent.vandevyvre@oqapy.eu>
# Home: http://www.oqapy.eu
# Copyright: 2011-2016 Vincent Vande Vyvre
# Licence: GPL3
#
# Trash can handler

import os
import shutil
import glob
import json
import sqlite3
import re
import datetime
import logging
lgg = logging.getLogger(__name__)

from PyQt5.QtGui import QImageReader

from warningmessages import WarningMessages
from tools.path import Path


class Shredder:
    def __init__(self, core):
        super().__init__()
        self.core = core
        self.ui = core.ui
        self.tab_widget = core.ui.stackedviews.sorting_tab.trash_tab
        self.toolbox = core.ui.toolpanel.trash_tool
        self.utils = core.utils
        self.folder = core.workspace['trash'].path
        self.thumbs_folder = core.workspace['trash_thumbs'].path
        self.findex = os.path.join(self.folder, 'index.json')
        self.files = []
        self.folder_size = 0
        self.tab_widget.dir_ = self.folder

    def setup(self, clean=False):
        if not self.set_folders():
            # No trash can feature
            self.core.trash_enabled = False
            return

        if clean:
            self.clear()

        else:
            self.get_file_list()

    def set_folders(self):
        if not os.path.isdir(self.folder):
            try:
                os.makedirs(self.folder)
            except Exception as why:
                lgg.warning("Can't create trash folder %s" % why)
                self.show_setting_error(self.folder, why)
                return False

        if not os.path.isdir(self.thumbs_folder):
            try:
                os.mkdir(self.thumbs_folder)
            except Exception as why:
                lgg.warning("Can't create thumbnails trash folder %s" % why)
                self.show_setting_error(self.thumbs_folder, why)
                return False

        return True

    def get_file_list(self):
        index = self.load_index()
        if not index:
            self.create_index()

        if index:
            self.populate_view(index)

    def load_index(self):
        lgg.info('Load index from: %s' % self.findex)
        if not os.path.isfile(self.findex):
            return False

        try:
            with open(self.findex, 'r') as objfile:
                cnt = objfile.read()
                return json.loads(cnt)
        except Exception as why:
            lgg.warning("Read error from: {0},".format(self.findex))
            lgg.warning("reason: {0}.".format(why))
            return False

    def save_index(self):
        lgg.info('Save trash can index')
        jsn = json.dumps(self.files, sort_keys=True, indent=4, 
                         separators=(',', ': '), ensure_ascii=False)
        try:
            with open(self.findex, 'wb') as outfile:
                outfile.write(jsn.encode('utf-8', 'replace'))
                lgg.info("Trash can index saved")
        except Exception as why:
            lgg.warning("Write error file: {0}".format(self.findex))
            lgg.warning("reason: {0}".format(why))

    def populate_view(self, entries):
        for idx, entry in enumerate(entries):
            if os.path.isfile(entry["fpath"]):
                self.files.append(entry)
                self.tab_widget.add_item(idx, entry)

        self.save_index()

    def show_setting_error(self, fld, err):
        message = {'context': _('Trash folder creation'),
            'warn': _("Unable to create folder:"),
            'data': fld,
            'msg': _('Reason:'),
            'data1': str(err),
            'btn': _('Close')}
        warn = WarningMessages(0, message)
        warn.build_dialog()

    def create_index(self):
        """Create the index of the files found into the trash folder.

        This happens when the index was not found or when this is the 
        first use of Oqapy-3.
        """
        old = os.path.join(self.folder, "shredder.db")
        if os.path.isfile(old):
            self.files = self.get_old_db(old)
            # Old Oqapy-2 version
            try:
                os.unlink(old)
            except Exception as why:
                lgg.warning("Can't remove 'shredder.db' %s" % why)

        if self.files:
            self.save_index()

    def get_data_from_picture(self, fname, tname, pict):
        data = {'date': None, 'speed': None, 'fnumber': None, 'iso': None}
        data['fpath'] = fname
        data['orig'] = pict.uri
        data['added'] = datetime.datetime.now().strftime('%a %d %b %Y, %H:%M')
        data['thumb'] = tname
        data['width'] = pict.width
        data['height'] = pict.height
        data['weight'] = pict.weight
        data['orient'] = pict.orientation
        if pict.metadata is not None:
            data['date'] = pict.metadata.get_date_and_id()[0]
            values = pict.metadata.get_exposure_infos()
            data['iso'] = values[0]
            data['speed'] = values[1]
            data['fnumber'] = values[2]

        return data

    def count(self):
        return len(self.files)

    def add_file_from_source(self, f):
        """Add the current image into the trash can.

        Args:
        f -- instance of Picture
        """
        lgg.info('Move file to trash can:\n\t{0}'.format(f.uri))
        base = self.check_name(os.path.basename(f.uri))
        dest = os.path.join(self.folder, base)
        self.move_file(f.uri, dest)
        th = self.utils.get_thumb_name(dest)
        sthumb = self.utils.get_thumb_name(f.uri)
        self.move_file(sthumb, th)
        data = self.get_data_from_picture(dest, th, f)
        self.add_entry(data)

    def add_file_from_target(self, item):
        """Remove an image file from a target folder.

        Args:
        item -- instance of TargetItem
        """
        data = item.data
        orig = data["fpath"]
        base = self.check_name(os.path.basename(orig))
        dest = os.path.join(self.folder, base)
        self.move_file(orig, dest)
        data["fpath"] = dest
        th = self.utils.get_thumb_name(dest)
        orig = data["thumb"]
        self.move_file(orig, th)
        data["thumb"] = th
        data['added'] = datetime.datetime.now().strftime('%a %d %b %Y, %H:%M')
        self.add_entry(data)

    def move_file(self, src, target):
        try:
            shutil.move(src, target)
        except Exception as why:
            lgg.warning("Unable to move file in the trash can, reason:\n\t"
                            "{0}".format(why))

    def add_entry(self, entry):
        self.files.append(entry)
        self.save_index()
        idx = self.count() - 1
        # Create thumbnail for basket tab
        self.tab_widget.add_item(idx, entry)

    def check_name(self, n, fld=None):
        """Check if file exists already into the destination folder.

        If the file name exists, a numerical suffix is added to the file name.

        Args:
        n -- basename of the file
        """
        fld = fld or self.folder
        name, ext = os.path.splitext(n)
        count = 1
        # Check for existing suffix
        suff = re.compile('(?<=_\()(\d+)(?=\)%s)' % ext)
        match = suff.search(n)
        if match is not None:
            count = int(match.group(0)) + 1
            name = "".join(name.split("_")[:-1])

        while 1:
            path = os.path.join(fld, n)
            if os.path.isfile(path):
                n = "".join([name, "_(", str(count), ")", ext])
                count += 1
            else:
                break

        return n

    def restore_item(self, item, index):
        fpath = item.data['fpath']
        fld, fname = os.path.split(fpath)
        lgg.info('Restore file %s from trash can' % fname)
        fdest = os.path.dirname(item.data['orig'])
        dest = self.check_name(item.data['orig'], fdest)
        sthumb = self.utils.get_thumb_name(dest)
        self.move_file(fpath, dest)
        self.move_file(item.data['thumb'], sthumb)
        self.files.pop(index)
        if self.core.source_fld.where == fdest:
            self.core.insert_image(dest, sthumb)

    def remove_item(self, idx):
        item = self.files.pop(idx)
        self.remove_file(item['fpath'])
        self.remove_file(item['thumb'])
        self.save_index()

    def remove_file(self, f):
        """Delete an image file from trash folder.

        Args:
        f -- file name
        """
        if not os.path.isfile(f):
            # Case of first use
            return

        weight = os.path.getsize(f)
        try:
            os.remove(f)
            self.folder_size -= weight
        except Exception as why:
            lgg.warning("Unable to remove {0}, reason: {1}".format(f, why))

    def clear(self):
        """Remove all files from the trash can.

        This function can be called only at startup.
        """
        removables = glob.glob(os.path.join(self.folder, "*.*"))
        removables.extend(glob.glob(os.path.join(self.thumbs_folder, "*.*")))
        lgg.info('Remove {0} file(s) from trash can'.format(len(removables)))
        for r in removables:
            self.remove_file(r)

        self.files = []

    def move_folder(self, p):
        """Change trash folder with user's pref.

        Args:
        p -- new folder path, must exists
        """
        try:
            os.renames(self.folder, p)
            self.folder = p
            self.thumbs_folder = os.path.join(p, "thumbnails")
            self.core.workspace['trash'] = Path(p)
            self.core.workspace['trash_thumbs'] = Path(self.thumbs_folder)
            self.findex = os.path.join(p, 'index.json')
        except Exception as why:
            lgg.warning("Moving folder has failed! Reason: {0}".format(why))
            return False

        return True

    def get_old_db(self, db):
        """Load the old dbase from Oqapy-2.

        Args:
        db -- DBase file path
        """
        conn = sqlite3.connect(db)
        conn.row_factory = sqlite3.Row
        conn.isolation_level = None
        cur = conn.cursor()
        cur.execute("select * from trash order by file")
        items = []
        while 1:
            r = cur.fetchone()
            if r is None:
                break

            fpath = os.path.join(self.folder, r["file"])
            if os.path.isfile(fpath):
                item = {'thumb': None, 'iso': None, 'speed': None,
                        'date': None, 'fnumber': None, 'orient': 1}
                item['fpath'] = fpath
                item['added'] = r['date']
                item['orig'] = r['orig']
                item['width'] = r['width']
                item['height'] = r['height']
                item['weight'] = r['weight']
                items.append(item)

        cur.close()
        conn.close()
        return items

    def get_sqlite_version(self):
        conn = sqlite3.connect(self.db)
        conn.isolation_level = None
        cur = conn.cursor()
        cur.execute('SELECT SQLITE_VERSION()')
        data = cur.fetchone()
        cur.close()
        conn.close()
        return data[0]
