# -*- coding: utf-8 -*-

# imagefinder.py
# This file is part of Oqapy serie 3
#
# Author: Vincent Vande Vyvre <vincent.vandevyvre@oqapy.eu>
# Home: http://www.oqapy.eu
# Copyright: 2012-2017 Vincent Vande Vyvre
# Licence: GPL3
#
# Image finder dialog

import os
import sys
import math
import glob
import time
import datetime
from threading import Thread
import logging
lgg = logging.getLogger(__name__)

from PyQt5.QtCore import QObject, pyqtSignal, QCoreApplication, Qt
from PyQt5.QtGui import QImageReader, QImage

from sortingtools.ui_imagefinder import UiImageFinder
from metadata.metadata import Metadata

class ImageFinder(QObject):
    scanningFinished = pyqtSignal()
    def __init__(self, core, root):
        super().__init__()
        lgg.info("Call image finder dialog")
        self.core = core
        self.root = root
        self.ui = UiImageFinder(self)
        self.ui.folder_led.setText(self.root.where)
        #self.ui.folder_led.historic.entries = FOLDERS
        self.ui.folder_led._update_historic()
        self.scanningFinished.connect(self.on_scanning_finished)
        reply = self.ui.exec_()

    def set_searching(self, path):
        """Compile the criteria and run the research.

        Args:
        path -- Folder path
        """
        lgg.info("Set searching's parametres")
        lgg.info("Beggining in root: {0}".format(path))
        self.ui.tab_wdg.setCurrentIndex(2)
        self.ui.prog.show_progress(True)
        self.ui.prog.show()
        QCoreApplication.processEvents()
        self.criteria = {}
        self.selection = []
        self.files_count = 0
        self.folders_count = 0
        self.begin = time.clock()
        self.cancel = False

        if self.ui.uid_chb.isChecked():
            uid = str(self.ui.identifier_led.text())
            lgg.info("Searching for identifier: {0}".format(uid))
            Thread(target=self.find_identifier, args=(path, uid)).start()
            return

        chain = self.ui.fname.led.text()
        if chain:
            if not self.ui.case_1_chb.isChecked():
                chain = chain.lower()
            self.criteria['fname'] = self.get_pattern(chain)
            self.criteria['fname'].append(self.ui.case_1_chb.isChecked())

        if self.ui.create_chb.isChecked():
            begin = self.ui.date_create_from_ded.date().toPyDate()
            if not self.ui.unique_day_chb.isChecked():
                end = self.ui.date_create_to_ded.date().toPyDate()

            else:
                end = begin
            self.criteria["cdates"] = (begin, end)

        elif self.ui.modif_chb.isChecked():
            begin = self.ui.date_modif_from_ded.date().toPyDate()
            if not self.ui.unique_day_chb.isChecked():
                end = self.ui.date_modif_to_ded.date().toPyDate()

            else:
                end = begin
            self.criteria["mdates"] = (begin, end)

        if self.ui.fsize_chb.isChecked():
            self.criteria["fsize"] = (self.ui.min_file_spb.value(),
                                        self.ui.max_file_spb.value())

        if self.ui.psize_chb.isChecked():
            self.criteria["psize"] = (self.ui.min_pict_spb.value(),
                                        self.ui.max_pict_spb.value())

        chain = self.ui.title.led.text()
        if chain:
            self.criteria["title"] = self.get_pattern(chain)

        chain = self.ui.author.led.text()
        if chain:
            self.criteria["author"] = self.get_pattern(chain)

        chain = self.ui.tag.led.text()
        if chain:
            self.criteria["tag"] = self.get_pattern(chain)

        chain = self.ui.place.led.text()
        if chain:
            self.criteria["place"] = self.get_pattern(chain)

        if self.ui.use_coord_chb.isChecked():
            lo, la, ac = self.ui.get_geo_coord() 
            acc = self.get_accuracy(la, ac)
            lm, mm = acc[0] / 60, acc[1] / 60
            self.criteria["gps"] = (lo - lm, lo + lm, la - mm, la + mm)

        self.print_()
        if not self.criteria:
            self.ui.tab_wdg.setCurrentIndex(0)
            self.ui.prog.show_progress(False)
            lgg.info("criteria is empty, nothing to do")
            return

        if self.ui.subfolder_chb.isChecked():
            thumbs = self.ui.ignore_thumbs_chb.isChecked()
            Thread(target=self._get_folders, args=(path, thumbs)).start()

        else:
            Thread(target=self._get_files, args=(path,)).start()

    def on_scanning_finished(self):
        """Called when the research is finished.

        """
        end = time.clock()
        lgg.info("Found {0} files, selected: {1} in {2} sec.".format( 
                    self.files_count, len(self.selection), end-self.begin))
        self.ui.prog.bar.setMinimum(0)
        self.ui.prog.bar.setMaximum(len(self.selection))
        self.set_result()
        self.ui.table.column_resized(0, 0, 0)
        lgg.info("Table complete: {0}, Done at: {1}".format(
                    time.clock()-end, time.clock()-self.begin))


    def _get_list_files(self, fld):
        """Return the list of files found into the folder fld.

        Args:
        fld-- folder path

        Returns:
        list(full path of files)
        """
        lgg.info('get files in {0}'.format(fld))
        lfls = glob.glob(os.path.join(fld, '*.*'))
        exc = ('doc', 'txt', 'pdf', 'tar', '.gz', '.py', '.pl', '.db', 'pps',
                'tml', '.ui', 'exe', 'asc', 'bin', 'zip', 'odt', 'xcf', 'mov',
                'mp4', 'avi', 'iiq')

        for i in range(len(lfls) - 1, -1, -1):
            if lfls[i][-3:].lower() in exc:
                lfls.pop(i)

        lfls = self.sort_(lfls)
        self.ui.prog.set_text(fld, len(lfls))
        self.files_count += len(lfls)

        return lfls

    def _get_files(self, fld):
        """Call the research process.

        This called for each folder in case of the research must include the
        sub-folders.
        Emit the signal scanningFinished when the last folder is complete.

        Args:
        fld -- folder path
        """
        fls = self._get_list_files(fld)

        if self.ui.all_crit_chb.isChecked():
            files = self._select_files_all_crit(fls)

        else:
            files = self._select_files_any_crit(fls)

        self.selection.extend(files)

        if not self.folders_count:
            self.scanningFinished.emit()

    def _get_folders(self, root, ignore, feedback=False):
        """Return the sub-folders of the root.

        Args:
        root -- root folder path
        ignore -- if True, ignore the sub-folder thumbnails
        feedback -- True only in the case of the research criterion is the
                    unique identifier of the picture.

        Returns:
        The list of the sub-folders only if feedback is True, otherwise None
        """
        folders = []
        for fld, ds, fs in os.walk(root):
            if ignore and fld.endswith('thumbnails'):
                continue
            folders.append(fld)

        folders = self.sort_(folders)
        self.folders_count = len(folders)

        if feedback:
            return folders

        for fold in folders:
            self.folders_count -= 1
            self._get_files(fold)

    def _select_files_all_crit(self, items):
        """Research for the files if all criteria are mandatory.

        Args:
        items -- file's paths list
        """
        def discard(elem):
            try:
                selected.remove(elem)
            except:
                pass

        selected = set()
        crt = self.criteria
        for item in items:
            if self.cancel:
                break

            keys = list(self.criteria.keys())
            if 'fname' in keys:
                keys.remove('fname')
                name = os.path.splitext(os.path.basename(item))[0]
                if crt['fname'][2]:
                    if self._compare_str_strict(crt['fname'], name):
                        selected.add(item)

                    else:
                        continue

                else:
                    if self._compare_str(crt['fname'], name):
                        selected.add(item)

                    else:
                        continue

            if 'fsize' in keys:
                keys.remove('fsize')
                size = os.path.getsize(item) / 1024
                mn, mx = crt['fsize']
                if size < mn or size > mx:
                    discard(item)
                    continue

                else:
                    selected.add(item)

            if 'psize' in keys:
                keys.remove('psize')
                size = self.get_picture_size(item)
                # if the file is not an image file, size = (-1, -1)
                if min(crt['psize']) > min(size) or \
                            max(crt['psize']) < max(size):
                    discard(item)
                    continue

                else:
                    selected.add(item)

            if 'mdates' in keys:
                keys.remove('mdates')
                date = crt['mdates']
                fdate = datetime.date.fromtimestamp(os.path.getmtime(item))

                if date[1] and fdate - date[0] >= 0 and date[1] - fdate >= 0:
                    selected.add(item)

                elif not date[1] and date[0] == fdate:
                    selected.add(item)

                else:
                    discard(item)
                    continue

            if not keys:
                continue

            data = self.get_metadata(item)
            if not data:
                discard(item)
                continue

            if 'cdates' in keys:
                if not data[0][1]:
                    discard(item)
                    continue

                try:
                    d = [int(v) for v in data[0][1].split()[0].split('-')]
                except ValueError:
                    discard(item)
                    continue

                cdate = datetime.date(d[0], d[1], d[2])
                date = crt['cdates']
                if date[1] and date[0] <= cdate and cdate <= date[1]:
                    selected.add(item)

                elif not date[1] and date[0] == cdate:
                    selected.add(item)

                else:
                    discard(item)
                    continue

            selected.add(item)
            if 'title' in keys:
                if not data[0][0] or not self._compare_str(crt['title'], 
                                                            data[0][0]):
                    discard(item)
                    continue

            if 'author' in keys:
                if not data[0][2] or not self._compare_str(crt['author'], 
                                                            data[0][2]):
                    discard(item)
                    continue

            if 'tag' in keys:
                if not data[0][4] or not self._compare_str(crt['tag'], 
                                                            data[0][4]):
                    discard(item)
                    continue

            if 'place' in keys:
                if not self._compare_place(crt['place'], data[1]):
                    discard(item)
                    continue                   

            if 'gps' in keys:
                if not data[2] or not self._compare_gps(crt["gps"], data[2]):
                    discard(item)

        return list(selected)

    def _select_files_any_crit(self, items):
        """Research for the files.

        Args:
        items -- file's paths list
        """
        def discard(elem):
            try:
                selected.remove(elem)
            except:
                pass

        selected = set()
        crt = self.criteria
        for item in items:
            if self.cancel:
                break 
            keys = self.criteria.keys()
            if 'fname' in keys:
                keys.remove('fname')
                name = os.path.splitext(os.path.basename(item))[0]
                if crt['fname'][2]:
                    if self._compare_str_strict(crt['fname'], name):
                        selected.add(item)
                        continue

                else:
                    if self._compare_str(crt['fname'], name):
                        selected.add(item)
                        continue

            if 'fsize' in keys:
                keys.remove('fsize')
                size = os.path.getsize(item) / 1024
                mn, mx = crt['fsize']
                if size >= mn and size <= mx:
                    selected.add(item)
                    continue

            if 'psize' in keys:
                keys.remove('psize')
                size = self.get_picture_size(item)
                # if the file is not an image file, size = (-1, -1)
                if min(crt['psize']) <= min(size) and \
                            max(crt['psize']) >= max(size):
                    selected.add(item)
                    continue

            if 'mdates' in keys:
                keys.remove('mdates')
                date = crt['mdates']
                fdate = datetime.date.fromtimestamp(os.path.getmtime(item))

                if date[1] and fdate - date[0] >= 0 and date[1] - fdate >= 0:
                    selected.add(item)
                    continue

                elif not date[1] and date[0] == fdate:
                    selected.add(item)
                    continue

            if not keys:
                continue

            data = self.get_metadata(item)
            if not data:
                continue

            if 'cdates' in keys and data[0][1]:

                try:
                    d = [int(v) for v in data[0][1].split()[0].split('-')]
                except ValueError:
                    pass

                else:
                    cdate = datetime.date(d[0], d[1], d[2])
                    date = crt['cdates']
                    if date[1] and date[0] <= cdate and cdate <= date[1]:
                        selected.add(item)
                        continue

                    elif not date[1] and date[0] == cdate:
                        selected.add(item)
                        continue

            if 'title' in keys:
                if data[0][0] and self._compare_str(crt['title'], data[0][0]):
                    selected.add(item)
                    continue

            if 'author' in keys:
                if data[0][2] and self._compare_str(crt['author'], data[0][2]):
                    selected.add(item)
                    continue

            if 'tag' in keys:
                if data[0][4] and self._compare_str(crt['tag'], data[0][4]):
                    selected.add(item)
                    continue

            if 'place' in keys:
                if self._compare_place(crt['place'], data[1]):
                    selected.add(item)
                    continue                   

            if 'gps' in keys:
                if data[2] and self._compare_gps(crt["gps"], data[2]):
                    selected.add(item)

        return list(selected)

    def find_identifier(self, root, ident):
        """Search for an image with his unique identifier.

        Args:
        root -- folder path
        ident -- identifier
        """
        # Testing: 54F56A50F8C1488A974221475E8A8652
        if self.ui.subfolder_chb.isChecked():
            thumbs = self.ui.ignore_thumbs_chb.isChecked()
            flds = self._get_folders(root, thumbs, True)

        else:
            flds = [root]
        
        all_ = self.ui.all_occur.isChecked()
        found = False
        for fold in flds:
            lgg.info("Enter in folder {0}".format(fold))
            files = self._get_list_files(fold)

            for fl in files:
                data = self.get_metadata(fl)
                if not data:
                    continue

                if data[0][-1] == ident:
                    self.selection.append(fl)
                    if not all_:
                        found = True
                        break

            if found:
                break

        self.scanningFinished.emit()

    def get_pattern(self, arg):
        """Return the pattern as list of list.

        Ex.: arg = 'italy ombria + 2010 - roma'
         pattern = [[['italy', 'ombria'], ['2010']], ['roma']]
         meaning:      word  OR  word   AND  word   NOT  word
                      |_________accepted_________|  |_rejected_|

        Args:
        arg -- str()

        Returns:
        list()
        """ 
        groups = arg.split('-')
        acc = groups[0].strip()
        exc = False

        if len(groups) > 1:
            grp = groups[1].strip().replace('+', '').replace('-', '')
            exc = [a.strip() for a in grp.split()]

        acc = [a.strip() for a in acc.split('+')]
        adm = [a.split() for a in acc]

        return [adm, exc]

    def get_accuracy(self, angle, kmeter):
        k = abs(((6378 * math.cos(angle)) * math.pi * 2) / 21600)

        lam = kmeter / 1.855
        try:
            lom = kmeter / k
        except ValueError:
            lom = lam

        return lam, lom

    def _compare_str(self, patt, chain):
        """Compare the criterion string, ignoring the case.

        Args:
        patt -- pattern
        chain -- the string that will be evaluate with the pattern

        Returns:
        True if the string is conform to the pattern
        """
        # patt = [[['aout', 'sept'], ['italy']], ['rome', 'milan'], False]
        #           -elem----elem-    -elem--     -elem----elem--
        #          ------adm-------  ---adm---
        #          -----------patt[0]----------  ----patt[1]------
        found = False
        chain = chain.lower()
        for adm in patt[0]:
            for elem in adm:
                found = False
                if elem.lower() in chain:
                    found = True
                    break

            if not found:
                break

        if not found:
            return False

        if patt[1]:
            for elem in patt[1]:
                if elem.lower() in chain:
                    found = False
                    break

        return found

    def _compare_str_strict(self, patt, chain):
        """Compare the criterion string, case is sensitive.

        Args:
        patt -- pattern
        chain -- the string that will be evaluate with the pattern

        Returns:
        True if the string is conform to the pattern
        """
        # patt = [[['aout', 'sept'], ['italy']], ['rome', 'milan'], True]
        #           -elem----elem-    -elem--     -elem----elem--
        #          ------adm-------  ---adm---
        #         ------------patt[0]----------  ----patt[1]------ 
        found = False
        for adm in patt[0]:
            for elem in adm:
                found = False
                if elem in chain:
                    found = True
                    break
            if not found:
                break

        if not found:
            return False

        if patt[1]:
            for elem in patt[1]:
                if elem in chain:
                    found = False
                    break

        return found

    def _compare_place(self, patt, data):
        """Compare the IPTC-Scene data with the pattern.

        Args:
        patt -- pattern
        chain -- the string that will be evaluate with the pattern

        Returns:
        True if the string is conform to the pattern
        """
        for d in data:
            if d is not None:
                if self._compare_str(patt, d):
                    return True

        return False

    def _compare_gps(self, patt, data):
        """Compare the EXIF-GPS data with the pattern.

        Args:
        patt -- (longitude min., longitude max., latitude min., latitude max.)
        chain -- the coordinates of the picture

        Returns:
        True if the coordinates are valid
        """
        if not data[0] >= patt[0] or not data[0] <= patt[1]:
            return False

        if not data[1] >= patt[2] or not data[1] <= patt[3]:
            return False

        return True

    def get_picture_size(self, path):
        """Return the size of a picture.

        Args:
        path -- image's path

        Returns:
        tuple(width, height)
        """
        size = QImageReader(path).size()
        return size.width(), size.height()

    def get_metadata(self, path):
        meta = Metadata(path)
        data = meta.get_data()

        if not data:
            return False

        return meta.get_common_data()
        

    # QTableWidget -------------------------------------------------------

    def set_result(self):
        """Set the selected files into the table view.

        """
        lgg.info('Set the selected files into the table widget')
        selected = list(self.selection)
        selected = self.sort_(selected)
        self.items = []
        count = 0
        for select in selected:
            item = SelectedItem(self, select)
            self.ui.table.add_item(item)
            self.items.append(item)
            self.ui.prog.bar.setValue(count)
            count += 1

        self.ui.prog.hide()
        self.ui.set_files_number_label(len(self.items))
        self.ui.open_btn.setEnabled(len(self.items))

    def remove_from_selection(self, removed):
        for r in removed:
            self.items.pop(r)

        self.ui.set_files_number_label(len(self.items))
        self.ui.open_btn.setEnabled(len(self.items))

    def send_selected_files(self, arg):
        self.core.selection = [i.path for i in self.items]
        self.ui.accept()

    def print_(self):
        lgg.info("criteria:")
        for w in ["fname", "cdates", "mdates", "fsize", "psize", "title",
                     "author", "tag", "place", "gps"]:
            try:
                val = self.criteria[w]
            except KeyError:
                val = "Nihil"

            lgg.info("   %s : %s" %(w, val))

    def sort_(self, lst):
        if sys.version_info[0] < 3:
            return sorted(lst, cmp=lambda x, y: cmp(x.lower(), y.lower()))
        else:
            return sorted(lst, key=str.lower)


class SelectedItem:
    """Define a file object wich has been selected by the research.

    Attributes:
    path -- full path
    fld -- folder path
    name -- basename
    img -- QImage instance of the thumbnail
    icon -- ?
    fsize -- file's size in bytes
    psize -- picture size in pixels
    data -- set of common metadata of the picture
    """
    def __init__(self, main, path):
        self.main = main
        dct = dict(path = path,
                    fld = os.path.dirname(path),
                    name = os.path.basename(path),
                    img = None,
                    icon = None,
                    fsize = None,
                    psize = None,
                    data = None)

        for key, value in dct.items():
            setattr(self, key, value)

        self.set_image()
        self.get_sizes()
        self.data = self.main.get_metadata(path)

    def set_image(self):
        thpath = self.get_thumbnail_path()
        if thpath is None:
            thpath = self.path

        img = QImage(thpath)
        self.img = img.scaledToHeight(160, Qt.SmoothTransformation)

    def get_sizes(self):
        self.fsize = os.path.getsize(self.path) / 1024
        size = QImageReader(self.path).size()
        self.psize = size.width(), size.height()

    def get_thumbnail_path(self):
        th = os.path.join(self.fld, "thumbnails", "thumb_" + self.name)
        if os.path.exists(th):
            return th
