# -*- coding: utf-8 -*-

# state.py
# This file is part of Oqapy serie 3
#
# Author: Vincent Vande Vyvre <vincent.vandevyvre@oqapy.eu>
# Home: http://www.oqapy.eu
# Copyright: 2014-2018 Vincent Vande Vyvre
# Licence: GPL3
#
# Define the context of the application


import threading
import logging
lgg = logging.getLogger(__name__)

from PyQt5.QtCore import QObject, Qt, QCoreApplication, pyqtSignal, QSize

class State(QObject):
    _instance = None
    _states = ['empty', 'viewing', 'sorting', 'fullscreen', 'filtering',
               'raw_viewing', 'multi_viewing']
    stateChanged = pyqtSignal(int)
    def __new__(cls, *args, **kwargs):
        if cls._instance is None:
            cls._instance = super(State, cls).__new__(cls, *args, **kwargs)
        return cls._instance

    def __init__(self, core):
        super().__init__()
        self.core = core
        self.ui = core.ui
        self.statusbar = core.ui.statusbar
        self.toolbar = core.ui.toolbar
        self.toolpanel = core.ui.toolpanel
        self.viewers = self.ui.stackedviews
        self.trailers = self.ui.stackedtrailers
        self.context = 0
        self.old_state = 0
        self.dock_visibility = self.toolpanel.isVisible()
        self.trailer_visibility = False
        self.filter_preview = False
        self.is_dual_screen = False
        self.spy = 0

    @property
    def current_viewer(self):
        #if self.context == 6:
            #return self.viewers.multiviews
        return self.viewers.viewing_area

    def set_current_state(self, state, force=False):
        """Sets the current state.

        Args:
        state -- index of the state;
                    0 No viewing, empty list of images
                    1 Mode viewing
                    2 Mode sorting
                    3 Mode full screen
                    4 Mode filtering
                    5 Mode RAW image edition
                    6 Mode multi views
        force -- if true reset the current state, needed at startup
        """
        lgg.info("Set current state: %s - %s" %(state, self._states[state]))
        if state == self.context and not force:
            return

        if self.context == 5 and state != 6:
            self.close_raw_editing()

        self.old_state = self.context
        if not state:
            self.set_no_images_state()

        elif state == 1:
            self.set_viewing_state()

        elif state == 2:
            self.set_sorting_state()

        elif state == 3:
            self.set_full_screen_state()

        elif state == 4:
            self.set_filtering_state(True)

        elif state == 5:
            self.set_raw_viewing_state()

        elif state == 6:
            self.set_multiviewing_state()

        self.context = state
        self.core.globalmenu.set_context(self.context)

    def set_no_images_state(self):
        lgg.info("Close source folder")
        self.trailers.clean_all()
        self.trailers.hide()
        self.toolbar.on_state_changed(0)
        self.viewers.on_state_changed(0)
        self.toolpanel.set_empty_state()
        self.statusbar.set_current_mode(0)
        self.viewers.viewing_area.clear()
        self.viewers.viewing_area.set_background()

    def set_viewing_state(self):
        """Change to the viewing state.

        """
        lgg.info('Set viewing state')
        self.viewers.move_viewing_area(0)
        self.viewers.setCurrentIndex(0)
        self.trailers.setCurrentIndex(0)
        self.show_trailer(self.trailer_visibility)
        self.toolbar.on_state_changed(1)
        self.statusbar.set_current_mode(1)
        self.set_target_bar(False)
        self.toolpanel.enable_all_toolboxes(True)

    def set_sorting_state(self):
        """Change to the sorting mode.

        """
        lgg.info('Set sorting state')
        self.viewers.move_viewing_area(1)
        self.viewers.setCurrentIndex(1)
        self.trailers.setCurrentIndex(0)
        self.show_trailer(self.trailer_visibility)
        self.statusbar.set_current_mode(2)
        self.set_target_bar(True)

    def set_full_screen_state(self):
        """Change to the full screen view.

        """
        self.old_state = self.context
        if not self.old_state in (1, 5):
            # Need to change to viewing state before
            self.set_viewing_state()

        lgg.info('Set full screen state, old state: %s' % self.old_state)
        self.set_widgets(False)
        self.request_scrollBar(False)
        self.ui.showFullScreen()
        QCoreApplication.processEvents()
        self.current_viewer.setFocus(True)
        threading.Timer(0.2, self.fit_view).start()

    def exit_fullscreen(self):
        """Back from full screen mode

        """
        lgg.info('Exit full screen state, old state: %s' % self.old_state)
        self.ui.showNormal()
        self.set_widgets(True)
        QCoreApplication.processEvents()
        if self.old_state == 2:
            self.set_sorting_state()
            self.context = 2

        elif self.old_state == 5:
            self.core.set_raw_files_viewing()
            self.context = 5

        else:
            self.context = 1
        self.core.globalmenu.set_context(self.context)
        self.statusbar.set_current_mode(self.context)
        threading.Timer(0.2, self.fit_view).start()

    def set_filtering_state(self, state):
        """Sets or cancel the filtering state.

        Args:
        state -- boolean
        """
        self.enable_navigation_button(not state)
        if not state:
            lgg.info('Exit filtering state, old state: %s', self.old_state)
            self.viewers.viewing_header.set_filtered_mode(False)
            self.set_current_state(self.old_state)

        else:
            lgg.info('Set filtering state, current state: %s', self.context)
            self.old_state = self.context
            self.viewers.viewing_header.set_filtered_mode(True)

    def set_raw_viewing_state(self):
        lgg.info('Set raw editing state')
        self.old_state = self.context
        self.trailer_visibility = self.trailers.isVisible()
        if self.context == 2:
            self.viewers.move_viewing_area(0)
            self.viewers.setCurrentIndex(0)
            self.set_target_bar(False)

        self.trailers.setCurrentIndex(1)
        self.enable_tool_boxes(False)
        self.viewers.set_raw_header(True)
        self.toolpanel.set_raw_page_enabled(True)

    def close_raw_editing(self):
        lgg.info('Close raw editing state')
        self.core.rawviewer.close_rawfiles_viewing()
        self.enable_tool_boxes(True)
        self.toolpanel.raw_inquiries.clear()
        self.toolpanel.set_raw_page_enabled(False)
        self.core.trailer.is_raw_viewing = False
        self.core.trailer.on_trailer_completed()
        self.viewers.set_raw_header(False)
        # Avoid perpetual movement
        self.context = 0
        self.set_current_state(self.old_state)

    def set_raw_only(self, raw_only):
        self.toolpanel.set_raw_page_enabled(True)
        self.statusbar.set_raw_only(raw_only)
        self.core.globalmenu.set_raw_only(raw_only)
        if raw_only:
            self.enable_navigation_button(True)

    def set_multiviewing_state(self):
        lgg.info('Set multi viewing state')
        self.old_state = self.context
        if self.context == 2:
            self.viewers.set_sortingtab_multiviewing_mode()

        else:
            self.viewers.setCurrentIndex(3)

        self.trailer_visibility = self.trailers.isVisible()
        #self.set_target_bar(False)
        self.toolbar.on_state_changed(6)
        self.toolpanel.set_multiviewing_state()

    def close_multiviewing(self):
        lgg.info('Close multi viewing state, old state: %s' % self.old_state)
        self.enable_tool_boxes(True)
        self.core.multiviewing.clean_view()
        if self.old_state == 2:
            self.viewers.unset_sortingtab_multiviewing_mode()

    def enable_tool_boxes(self, enable):
        self.toolpanel.resize_bx.setEnabled(enable)
        self.toolpanel.crop_bx.setEnabled(enable)
        self.toolpanel.straighten_bx.setEnabled(enable)
        self.toolpanel.historic_bx.setEnabled(enable)
        self.toolpanel.contrast_bx.setEnabled(enable)
        self.toolpanel.stretching_bx.setEnabled(enable)
        self.toolpanel.denoising_bx.setEnabled(enable)
        self.toolpanel.grey_map_bx.setEnabled(enable)
        self.toolpanel.toning_bx.setEnabled(enable)
        self.toolpanel.laplace_bx.setEnabled(enable)

    def close_source_folder(self):
        if self.context == 6:
            self.close_multiviewing()
        self.set_no_images_state()

    def show_trailer(self, state=None):
        """Show - hide the trailer.

        Args:
        state -- if None, the old state is used, if True, the trailer is shown
                 otherwise it is hidden
        """
        if self.context == 4:
            self.show_raw_trailer(state)
            return

        lgg.info('Show trailer state: %s current: %s' 
                    % (state, self.context))
        if state is None:
            state = not self.trailers.isVisible()

        self.trailers.setVisible(state)

    def set_target_bar(self, visible):
        for i in range(self.core.cfg.get(('sorting', 'number'))):
            self.ui.targetarea.targets_buttons[i].setVisible(visible)

    def set_widgets(self, b):
        """Hide or show widgets when set or unset the full screen mode.

        Args:
        b -- True or False
        """
        self.toolpanel.setVisible(b)
        self.statusbar.widget.setVisible(b)
        self.toolbar.widget.setVisible(b)
        self.viewers.viewing_header.setVisible(b)
        if b:
            b = self.trailer_visibility

        self.trailers.setVisible(b)

    def enable_navigation_button(self, mode):
        """Enables the navigation and rotation buttons.

        Args:
        mode -- boolean, if True the buttons are enabled
        """
        self.toolbar.activate_navigation_widgets(mode)

    def freeze_tool_boxes(self, caller, freeze=True):
        lgg.info('Freeze tool boxes, caller: %s' %(caller))
        boxes = self.toolpanel.get_tool_boxes()
        if caller is not None:
            caller = caller.parent()

        for box in boxes:
            if box != caller:
                box.setEnabled(not freeze)

    def disable_undo_buttons(self, tool):
        lgg.info('Disable cancel buttons, caller: {0}'.format(tool))
        # Disable all undo buttons
        boxes = self.toolpanel.get_tool_boxes()
        for box in boxes:
            try:
                widget = box.layout.itemAt(1).widget()
            except AttributeError:
                # Case of box is splitter
                continue

            try:
                widget.cancel_btn.setEnabled(False)
            except AttributeError:
                # No cancel button
                continue

        # Enable undo button for the last change
        if tool is not None:
            tool.cancel_btn.setEnabled(True)

    def enable_raw_tools(self, enable):
        pass

    def request_scrollBar(self, b=None):
        """Show - hide scrollBars.

        Args:
        b -- boolean
        """
        if b is None:
            b = self.show_scrollbars

        if b:
            self.current_viewer.setVerticalScrollBarPolicy(Qt.ScrollBarAsNeeded)
            self.current_viewer.setHorizontalScrollBarPolicy(Qt.ScrollBarAsNeeded)

        else:
            self.current_viewer.setVerticalScrollBarPolicy(Qt.ScrollBarAlwaysOff)
            self.current_viewer.setHorizontalScrollBarPolicy(Qt.ScrollBarAlwaysOff)

        self.show_scrollbars = b

    def fit_view(self):
        """Fit the current image to the viewer size.

        """
        # We can't call directly the function because we are into a thread
        self.toolbar.widget.zoom_tools.zoom_fit_btn.click()
        
