# -*- coding: utf-8 -*-

# counter.py
# This file is part of Oqapy serie 3
#
# Author: Vincent Vande Vyvre <vincent.vandevyvre@oqapy.eu>
# Home: http://www.oqapy.eu
# Copyright: 2009-2017 Vincent Vande Vyvre
# Licence: GPL3
#
# Perpetual counter

class Counter:
    """Define a perpetual counter.

    The counter count between a range of two values in a perpetual loop.
    When the count beyound the maximum value, the current value become the start 
    value and, in case of reverse counting, the count become lower than the
    minimum value then the current value is set to the maximum.

    The value is incremented with Counter.inc() and is decremented with
    Counter.dec().

    The step value may be any integer upper than 0.

    The counter may be restarted with Counter.reset().
    """
    def __init__(self, minimum=0, maximum=0, step=1, start=None):
        """Instanciate a Counter object.

        Args:
        minimum -- The minimum value of the counter, it must be +-int().
        maximum -- The maximum value of the counter, it must be +-int() and 
                 must be upper than the start value.
        step --  The value of the step, it must be positive int(). Default: 1
        start -- Set the beginning value, it must be +-int() between the range
                 of start and end values. Default: None (minimum value will be
                 used)
        """
        if minimum > maximum:
            raise ValueError('Counter.__init__() maximum is lower than minimum')

        # Avoid NameError in set_minimum()
        self.maximum = False
        self.set_minimum(minimum)
        self.set_maximum(maximum)
        self.set_step(step)
        self.value = self.start = minimum
        if start is not None:
            self.start = start
            self.set_value(start)

    def __repr__(self):
        return "<Counter instance id: '%s'>" % id(self)

    def inc(self):
        """Increment the current value with the step value.

        Returns:
        The new value of the counter
        """
        self.value += self.step
        while self.value > self.maximum:
            # Need a loop if the step value is upper than the end value
            # Don't use modulo because start or end value may be 0
            self.value = self.value - 1 - self.maximum + self.minimum

        return self.value

    def dec(self):
        """Decrement the current value with the step value.

        Returns:
        The new value of the counter
        """
        self.value -= self.step
        while self.value < self.minimum:
            # Need a loop if the step value is upper than the end value
            # Don't use modulo because start or end value may be 0
            diff = self.minimum - self.value
            self.value = self.maximum - diff + 1

        return self.value

    def set_minimum(self, mnm):
        """Sets the minimum value.

        Args:
        mnm -- [+-]int(), must be lower than the maximum value.
        """
        if not isinstance(mnm, int):
            raise TypeError('Counter.set_minimum(int) arg has type: %s' 
                            % type(mnm))

        if self.maximum and mnm > self.maximum:
            raise ValueError('Counter.set_minimum(int) minimum must be lower '
                            'than maximum value.')

        self.minimum = mnm

    def set_maximum(self, mxm):
        """Sets the maximum value.

        Args:
        mxm -- [+-]int(), must be upper than the minimum value.
        """
        if not isinstance(mxm, int):
            raise TypeError('Counter.set_maximum(int) arg has type: %s' 
                            % type(mxm))

        if mxm < self.minimum:
            raise ValueError('Counter.set_maximum(int) maximum must be upper '
                            'than minimum value.')

        self.maximum = mxm

    def set_step(self, step):
        """Sets the step value.

        Args:
        step -- positive int()
        """
        if not isinstance(step, int):
            raise TypeError('Counter.set_step(int) arg has type: %s' 
                            % type(step))

        if step < 1:
            raise TypeError('Counter.set_step(int) must be upper than 0 got: %s' 
                            % step)

        self.step = step

    def set_value(self, val):
        """Sets the current value.

        Args:
        val -- [+-]int() between the minimum and maximum values.
        """
        if not isinstance(val, int):
            raise TypeError('Counter.set_value(int) arg has type: %s' 
                            % type(val))

        if val < self.minimum or val > self.maximum:
            raise ValueError('Counter.set_value(int) arg is out of range')

        self.value = val

    def reset(self):
        """Sets the current value to the start value.

        """
        self.value = self.start
