# path.py
# This file is part of Oqapy serie 3
#
# Author: Vincent Vande Vyvre <vincent.vandevyvre@oqapy.eu>
# Home: http://www.oqapy.eu
# Copyright: 2009-2018 Vincent Vande Vyvre
# Licence: GPL3
#
# File path object

import os
import logging
lgg = logging.getLogger(__name__)


class Path:
    def __init__(self, path):
        """Define a file path object.

        Args:
        path -- absolute path of the file
        """
        # Since version 3.4 python implement pathlib and the
        # attribute PurePath.path since version 3.4.5.
        # So, at the moment we keep our class Path
        #lgg.info('Define Path: %s' % path)
        path = path.rstrip('/\\')
        self.path = path
        self.set_properties()

    def set_properties(self):
        self.is_dir = os.path.isdir(self.path)
        self.is_file = os.path.isfile(self.path)
        self.fname = os.path.basename(self.path)
        self.name, self.ext = os.path.splitext(self.fname)
        self.parents = os.path.dirname(self.path)
        self.parent = os.path.basename(self.parents)
        self.parents_exists = os.path.exists(self.parents)
        self.is_thumbnail = self.name.startswith('thumb_')
        self.is_writable = self.check_writable()

    @property
    def exists(self):
        return os.path.exists(self.path)

    def size(self):
        if not self.exists:
            return 0

        return os.path.getsize(self.path)

    def check_writable(self):
        if not self.exists:
            parent = self.find_existing_parent()
            if not parent:
                return False

            return os.access(parent, os.W_OK)

        return os.access(self.path, os.W_OK)

    def ensure_folder_exists(self, parent=False):
        """Check if the directory exists, if not, create it.

        Args:
        parent -- If True, create only the parent
        """
        if not self.exists:
            if not self.check_writable():
                lgg.warning('File: %s is read-only!' % self.path)
                return False

            path = self.path
            if parent:
                path = self.parents

            try:
                os.makedirs(path)
            except Exception as why:
                lgg.warning("Can't create %s " % path)
                lgg.warning("reason: %s" % why)
                return False

        return True

    def find_existing_parent(self):
        path = self.path
        while 1:
            asc = os.path.dirname(path)
            if os.path.exists(asc):
                return asc

            if asc == path:
                # No parent
                return False

            path = asc

    def __eq__(self, other):
        # Some builtins can use this, i.e. method remove of list
        if isinstance(other, Path):
            other = other.path

        if not os.path.exists(other) or not self.exists:
            return self.path == other

        return os.path.samefile(self.path, other)

    def __add__(self, tail):
        if not isinstance(tail, str):
            raise TypeError('Path.__add__ expect string got: %s' % type(tail))

        return os.path.join(self.path, tail)

    def __iadd__(self, tail):
        if not isinstance(tail, str):
            raise TypeError('Path.__iadd__ expect string got: %s' % type(tail))

        self.path = os.path.join(self.path, tail)
        self.set_properties()
        return self

    def __truediv__(self, tail):
        return Path(os.path.join(self.path, tail))

    def __repr__(self):
        return "{}({!r})".format(self.__class__.__name__, self.path)
