# -*- coding: utf-8 -*-

# rectsize.py
# This file is part of Oqapy serie 3
#
# Author: Vincent Vande Vyvre <vincent.vandevyvre@oqapy.eu>
# Home: http://www.oqapy.eu
# Copyright: 2016-2017 Vincent Vande Vyvre
# Licence: GPL3
#
# Rectangular size object

from functools import wraps

from PyQt5.QtCore import QSize, QSizeF


class RectSize:
    """Define a rectangular size.

    A rectangular size is a pair (width, height), it can be instanciated with
    a list or a tuple of two integers or floats or with a QSize instance.
    """
    def __init__(self, *size):
        """Instanciate the RectSize object.

        Args:
        *size -- tuple(or list)(int(or float), int(or float)) or
                 QSize or QSizeF instance or
                 int(width), int(height) or
                 float(width), float(height)
        """
        if isinstance(size[0], (tuple, list)):
            self.width = size[0][0]
            self.height = size[0][1]

        elif isinstance(size[0], (QSize, QSizeF)):
            self.width = size[0].width()
            self.height = size[0].height()

        elif len(size) == 2:
            self.width = size[0]
            self.height = size[1]

        else:
            raise ValueError('RectSize error: unexpected argument %s' 
                             % str(size))

        for value in (self.width, self.height):
            if not isinstance(value, (int, float)):
                raise ValueError('RectSize.__init__ expected number, got %s' 
                                 % type(value))

    def check_value(func):
        @wraps(func)
        def _wrapped_func(*args):
            for arg in args[1:]:
                if not isinstance(arg, (int, float)):
                    raise ValueError('RectSize.%s expected int or float, got %s' 
                                     % (func.__name__, type(arg)))
            return func(*args)
        return _wrapped_func

    def check_other(func):
        @wraps(func)
        def _wrapped_func(*args):
            if not isinstance(args[1], RectSize):
                raise ValueError('RectSize.%s expected RectSize instance, got %s' 
                                 % (func.__name__, type(args[1])))
            return func(*args)
        return _wrapped_func

    @property
    def values(self):
        return self.width, self.height

    @property
    def is_square(self):
        return self.width == self.height

    @property
    def is_horizontal(self):
        return self.width > self.height

    @property
    def is_vertical(self):
        return self.width < self.height

    def reverse(self):
        self.width, self.height = self.height, self.width

    @property
    def reversed(self):
        return self.height, self.width

    def to_int(self):
        self.width = int(self.width)
        self.height = int(self.height)

    def to_float(self):
        self.width = float(self.width)
        self.height = float(self.height)

    @property
    def as_int(self):
        return int(self.width), int(self.height)

    @property
    def as_float(self):
        return float(self.width), float(self.height)

    @property
    def as_qsize(self):
        return QSize(self.width, self.height)

    @property
    def as_qsizef(self):
        return QSizeF(float(self.width), float(self.height))

    @property
    def as_dict(self):
        return {'width': self.width, 'height': self.height}

    @property
    def as_string(self):
        return '%s x %s' % self.values

    @check_value
    def is_widest(self, width):
        return self.width > width

    @check_value
    def is_highest(self, height):
        return self.height > height

    @check_other
    def is_include(self, other):
        if self.is_widest(other.width) or self.is_highest(other.height):
            return False

        return True

    @check_other
    def __eq__(self, other):
        return self.width == other.width and self.height == other.height

    @check_other
    def __ne__(self, other):
        return not self == other

    @check_other
    def __lt__(self, other):
        return self.width < other.width and self.height < other.height

    @check_other
    def __le__(self, other):
        return self == other or self < other

    @check_other
    def __gt__(self, other):
        return self.width > other.width and self.height > other.height

    @check_other
    def __ge__(self, other):
        return self == other or self > other

    @check_other
    def is_equivalent(self, other):
        return self == other or RectSize(self.reversed) == other

    @check_other
    def is_proportionnal(self, other, epsilon=0.001):
        if not self.height:
            return False

        factor = self.width / float(self.height)
        return abs(other.width * factor - other.height) < epsilon

    def __repr__(self):
        return '<RectSize(%s, %s) object at %s>' %(self.width, self.height, 
                                                   id(self))

    @check_other
    def __add__(self, other):
        return self.width + other.width, self.height + other.height

    @check_other
    def __sub__(self, other):
        return self.width - other.width, self.height - other.height

    @check_other
    def __iadd__(self, other):
        self.resize(other.width, other.height)
        return self

    @check_other
    def __isub__(self, other):
        self.resize(-other.width, -other.height)
        return self

    def resize(self, width, height):
        self.width += width
        self.height += height

    def rescale(self, factor=0, percent=0):
        if factor:
            self.width *= factor
            self.height *= factor

        else:
            self.width = self.width / 100.0 * percent
            self.height = self.height / 100.0 * percent

