# -*- coding: utf-8 -*-

# traileritemdrawer.py
# This file is part of Oqapy serie 3
#
# Author: Vincent Vande Vyvre <vincent.vandevyvre@oqapy.eu>
# Home: http://www.oqapy.eu
# Copyright: 2016 Vincent Vande Vyvre
# Licence: GPL3
#
# Thumbnail drawer for the trailer

import os
import logging
logger = logging.getLogger(__name__)

from fractions import Fraction

from PyQt5.QtCore import QObject, pyqtSignal, Qt, QSize, QRectF
from PyQt5.QtGui import QColor, QImage, QPainter, QPen, QFont

from metadata.metadata import Metadata
from tools.rectsize import RectSize


class TrailerItemDrawer(QObject):
    itemCreated = pyqtSignal(str, dict, str)
    def __init__(self, fld):
        """thumbnail builder used by the trailer.

        Args:
        fld -- thumbnails folder, usually ~/.oqapy/temp/trailer/

        Attributes:
            style -- render style
            size -- size of image, biggest side
            Fnumber -- aperture value
            speed -- exposure time
            iso -- iso value
            tmp -- path of temporary folder where saving the thumbnail
            width_ -- width of bounding rectangle
            height_ -- height of bounding rectangle, same as scene
        """ 
        super().__init__()
        self.items_fld = fld
        self.back_color = QColor(24, 30, 38, 255)        # Lualaba mid
        self.back_b_color = QColor(54, 59, 120, 255)     # Uele light
        self.text_color = QColor(237, 212, 0, 255)       # Ituri mid

    def configure(self, cfg):
        width = int((cfg['size'] / 90.0) * 160.0) + 1
        self.pix_size = RectSize(width, cfg['size'])
        self.height_ = cfg['size'] + 4
        self.width_ = width + 8
        self.style = cfg['style']
        if self.style in (1, 2):
            self.height_ += 16

        elif self.style == 3:
            self.height_ += 32

        self.set_background()

    def draw(self, orig, thumb, data=None):
        """Draw a thumbnail item for the trailer.

        Args:
        orig -- full path of the original image file
        thumb -- instance of Path of the thumbnail
        """
        self.orig = orig
        self.fname = thumb.fname
        self.image_data = data
        outf = self.get_item_filename(os.path.basename(orig))
        self.back = QImage(self.background)
        self.load_image(thumb)
        if data is None:
            self.image_data = self.get_metadata(thumb, orig)

        if not self.style:
            self.set_image(0)

        elif self.style == 1:
            self.set_image(8)
            self.set_title_up()

        elif self.style == 2:
            self.set_image(-8)
            if self.image_data is not None:
                self.set_title_down()

        else:
            self.set_image(0)
            self.set_title_up()
            if self.image_data is not None:
                self.set_title_down()

        self.save_item(outf)

    def get_item_filename(self, fname):
        name = os.path.splitext(fname)[0]
        return os.path.join(self.items_fld.path, name + '.png')

    def load_image(self, path):
        self.img = QImage(path.path)
        size = RectSize(self.img.size())
        if not size.is_include(self.pix_size):
            self.img = self.img.scaled(self.pix_size.width, 
                                        self.pix_size.height, 
                                        Qt.KeepAspectRatio,
                                        Qt.SmoothTransformation)

        self.i_width, self.i_height = self.img.width(), self.img.height()

    def get_metadata(self, path, orig=None):
        md = Metadata(path.path)
        if md.get_data():
            return md.get_data_for_thumbnail()

        elif orig is not None:
            md = Metadata(orig)
            if md.get_data():
                return md.get_data_for_thumbnail()

        return {'fn': None, 'exp': None, 'iso': None}

    def set_background(self):
        """Build the thumbnail's background.

        """
        self.background = QImage(QSize(self.width_, self.height_),
                            QImage.Format_ARGB32_Premultiplied)
        self.background.fill(self.back_color)
        painter = QPainter()
        painter.begin(self.background)
        painter.setPen(QPen(self.back_b_color, 1, Qt.SolidLine, 
                            Qt.RoundCap, Qt.MiterJoin))
        painter.drawRect(0.0, 0.0, self.width_ - 1, self.height_ - 1)
        painter.end()

    def set_image(self, up):
        """Apply frame and image in thumbnail with style 'outTitle'.

        Args:
        up -- offset from Y origin
        """
        ofX = (self.width_ - self.i_width) / 2
        ofY = int((self.height_ - self.i_height) / 2) + up
        painter = QPainter()
        painter.begin(self.back)
        painter.setPen(QPen(QColor(220, 220, 220, 255), 2,
                          Qt.SolidLine, Qt.RoundCap, Qt.MiterJoin))
        painter.drawRect(ofX, ofY, self.i_width + 1, self.i_height + 1)
        p_rect = QRectF(ofX, ofY, self.i_width, self.i_height)
        src = QRectF(0.0, 0.0, self.i_width, self.i_height)
        painter.drawImage(p_rect, self.img, src)
        painter.end()

    def set_title_up(self):
        """Apply text on the thumbnail's upside in style 'outTitle'.

        """
        text = self.get_name_txt()
        painter = QPainter()
        painter.begin(self.back)
        font = self.get_font()
        painter.setFont(font)
        metric = painter.fontMetrics()
        w = metric.width(text)
        offset = (self.width_ - w) / 2

        if offset < 0:
            offset = 0

        painter.setPen(QPen(self.text_color, 1,
                          Qt.SolidLine, Qt.RoundCap, Qt.MiterJoin))
        painter.drawText(offset, 12, text)
        painter.end()

    def set_title_down(self):
        """Apply text on the thumbnail's upside in style 'outTitle'.

        """
        text = self.get_infos_txt()
        painter = QPainter()
        painter.begin(self.back)
        font = self.get_font()
        painter.setFont(font)
        metric = painter.fontMetrics()
        w = metric.width(text)
        offset = (self.width_ - w) / 2

        if offset < 0:
            offset = 0

        painter.setPen(QPen(self.text_color, 1,
                          Qt.SolidLine, Qt.RoundCap, Qt.MiterJoin))
        painter.drawText(offset, self.height_ - 4, text)
        painter.end()

    def get_infos_txt(self):
        """Returns the exposure infos as string.

        """
        fnumb, speed = "", ""
        di = self.image_data['fn']
        if di is not None:
            if isinstance(di, Fraction):
                n, d = di.numerator * 1.0, di.denominator
                try:
                    fn = round(n / d, 2)
                    if fn != 0:
                        fnumb = ":".join(["f", str(fn)])
                except ZeroDivisionError:
                    pass

        exp = self.image_data['exp']
        if exp is not None:
            if isinstance(exp, Fraction):
                if exp.denominator == 1:
                    speed = "%s s." % exp.numerator
                else:
                    n, d = exp.numerator * 1.0, exp.denominator
                    try:
                        sp = int(d / n)
                        if sp != 0:
                            speed = "/".join(["1", str(sp)])
                    except ZeroDivisionError:
                        pass

        return "    ".join([fnumb, speed])

    def get_name_txt(self):
        """Returns the file name resized with the help of fontMetrics.

        """
        painter = QPainter()
        painter.begin(self.back)
        font = self.get_font()
        painter.setFont(font)
        metric = painter.fontMetrics()
        text = os.path.basename(self.orig)

        while metric.width(text) > self.width_:
            text = self.truncate_name(text)

        painter.end()
        return text

    def get_font(self):
        font = QFont()
        font.setFamily('Ubuntu')
        font.setPointSize(8)
        return font

    def save_item(self, dest):
        """Save thumbnail.

        Thumbnails are saved in ~/oqapy/tmp/thumbnails

        Args:
        name -- file name without extension
        """
        if self.image_data is None:
            self.image_data = {}
        try:
            self.back.save(dest)
            self.itemCreated.emit(dest, self.image_data, self.orig)
        except Exception as why:
            logger.warning("Can't save thumbnail, reason: {0}".format(why))

    def truncate_name(self, name):
        return name[:-5] + "..."
