# -*- coding: utf-8 -*-

# trailers.py
# This file is part of Oqapy serie 3
#
# Author: Vincent Vande Vyvre <vincent.vandevyvre@oqapy.eu>
# Home: http://www.oqapy.eu
# Copyright: 2009-2018 Vincent Vande Vyvre
# Licence: GPL3
#
# Trailer

import os
import time
import logging
lgg = logging.getLogger(__name__)

from threading import Thread

from PyQt5.QtCore import QObject, pyqtSignal
from PyQt5 import QtCore, QtGui, QtWidgets

from metadata.metadata import Metadata
from thumbnailer import Thumbnailer
from traileritemdrawer import TrailerItemDrawer
from tools.path import Path


class Trailer(QObject):
    trailerCompleted = pyqtSignal()
    def __init__(self, core, israw=False):
        super().__init__()
        self.core = core
        self.ui = core.ui
        self.utils = core.utils
        self.is_raw_viewing = israw
        self.view = self.ui.stackedtrailers.trailer
        self.temp_fld = core.workspace['trailer']
        self.current_item = False
        self.current_index = 0
        self.centered = 0
        self.stop_request = False
        self.is_complete = False
        self.to_insert = False
        self.drawer = TrailerItemDrawer(self.temp_fld)
        self.drawer.configure(core.cfg.get(('trailer', )))
        self.drawer.itemCreated.connect(self.add_item)
        self.trailerCompleted.connect(self.on_trailer_completed)
        self.view.trailerMoved.connect(self.on_trailer_moved)

    def configure(self, cfg, rebuild=False):
        self.height = cfg['size']
        self.style = cfg['style']
        self.view.config(cfg)

    def check_config(self, cfg):
        if cfg["size"] != self.height or cfg['style'] != self.style:
            self.configure(cfg, True)
            self.drawer.configure(cfg)
            return True

        return False

    def clean_view(self):
        self.view.clear_all()
        self.current_item = 0
        self.is_complete = False

    def load_images(self):
        """Build the list of the thumbnails.

        Missing thumbnails will be create.
        """
        images = self.core.source_fld.images
        self.thumbs_fld = self.core.source_fld.thumbs_fld
        self.thumbs = []
        self.clean_view()
        def load_thumbnails():
            for image in images:
                thumb = self.get_thumb_name(image)
                if not os.path.isfile(thumb):
                    self.create_thumbnail(image, thumb)

                elif os.path.splitext(image)[1].lower() in ('.tif', '.tiff',
                                                           '.jpg', '.jpeg'):
                    self.check_metadata(image, thumb)

                self.thumbs.append(thumb)

            self.build_all_items()
        Thread(target=load_thumbnails).start()

    def load_raw_images(self, raws):
        self.thumbs = []
        self.clean_view()
        def load_thumbnails():
            for raw in raws:
                self.thumbs.append(raw.thumbnail)
                self.drawer.draw(raw.source.fname, Path(raw.thumbnail))
            self.trailerCompleted.emit()
        Thread(target=load_thumbnails).start()

    def get_thumb_name(self, path):
        """Set the full path of thumbnail from the path of an image.

        i.e. path = /home/user/pictures/family/lena.jpg
        return /home/user/pictures/family/thumbnails/thumb_lena<jpg>.jpg

        Args:
        path -- full path of image.

        Returns:
        Full path of thumbnail.
        """
        caput, cauda = os.path.split(path)
        name, ext = os.path.splitext(cauda)
        ext = '<%s>' % ext[1:]
        f = "".join(["thumb_", name, ext, ".jpg"])
        return os.path.join(self.thumbs_fld.path, f)

    def create_thumbnail(self, fname, tname):
        """Create a thumbnail.

        Args:
        fname -- path of original image file
        tname -- path of the thumbnail
        """
        lgg.info('Create thumbnail: .../%s' % os.path.basename(tname))
        orient = 1
        self.data = self.get_metadata(fname)
        if self.data is not None:
            orient = self.get_orientation()
        th = Thumbnailer(fname, tname, orient)
        th.draw_and_save()
        self.copy_metadata(tname)

    def get_metadata(self, path):
        """Return the Metadata instance of an image.

        Args:
        path -- full path of an original image
        """
        md = Metadata(path)
        if md.get_data():
            return md

    def get_orientation(self):
        try:
            return self.data.data['Exif.Image.Orientation'].value
        except KeyError:
            return 1

    def check_metadata(self, fname, tname):
        """Ensure the metadata is written into the thumbnail file.

        Args:
        fname -- path of original image file
        tname -- path of the thumbnail
        """
        md = self.get_metadata(tname)
        if md is not None:
            try:
                o = md.data['Exif.Image.Orientation'].value
            except KeyError:
                self.data = self.get_metadata(fname)
                if self.data is not None:
                    self.copy_metadata(tname)

    def copy_metadata(self, dest):
        """Copy the metadata into a thumbnail file.

        Args:
        dest -- path of the thumbnail
        """
        if self.data is None:
            return

        data = self.data.get_data_for_thumbnail()
        # The thumbnail has been re-oriented if necessary
        data['orientation'] = 1
        md = Metadata(dest)
        if md.get_data():
            md.write_thumbnail_data(data)

        else:
            lgg.warning('Read thumbnail data error: %s' % md.last_error)

    def build_all_items(self):
        """Create the trailer's items.

        """
        for img, thumb in zip(self.core.source_fld.images, self.thumbs):
            if self.stop_request:
                lgg.info("Stop request")
                break

            self.drawer.draw(os.path.basename(img), Path(thumb))

        self.trailerCompleted.emit()
        self.items = self.thumbs
        
    def add_item(self, path, data, image):
        """Append an item to the trailer.

        Slot called by the item drawer.

        Args:
        path -- path of the item
        data -- metadata of the image
        image -- path of the original image
        """
        lgg.info('Add trailer item: .../%s' % os.path.basename(path))
        if self.stop_request:
            return

        item = TrailerItem(self, path, data, image)
        if self.to_insert:
            self.view.insert_item(item, self.inserting_index)
            self.to_insert = False

        else:
            self.view.add_item(item)

    def insert_thumbnail(self, image, index, thumbnail):
        self.inserting_index = index
        self.to_insert = True
        self.drawer.draw(os.path.basename(image), Path(thumbnail))

    def set_current_image(self, item):
        """Set current the item selected.

        Called by the item itself.

        Args:
        item -- instance of TrailerItem
        """
        lgg.info('Set current picture: .../%s' % os.path.basename(item.image))
        self.view.unselect_item(self.current_index)
        idx = self.view.items.index(item)
        if self.is_raw_viewing:
            self.core.rawviewer.set_current_image(idx)

        else:
            self.core.viewing.counter = idx
            self.core.viewing.load_picture()
            self.current_index = idx

    def set_current_item(self, idx):
        """Set current the item selected.

        Called by the core with the next and previous buttons or shortcuts.

        Args:
        idx -- index of TrailerItem
        """
        if self.is_complete and self.view.items:
            self.view.unselect_item(self.current_index)
            self.current_item = self.view.items[idx]
            self.current_index = idx
            self.view.set_current(idx)

    def reload_current(self, fname, thumb):
        idx = self.current_index
        self.remove_at_index(idx)
        self.insert_thumbnail(fname, idx, thumb)

    def on_item_hoover(self, item):
        item.setOpacity(0.9)

    def on_item_exited(self, item):
        if self.current_item != item:
            item.setOpacity(0.9)

    def on_trailer_completed(self):
        if self.is_raw_viewing:
            idx = 0

        else:
            idx = self.core.current_index

        lgg.info('Trailer completed, %s items' % len(self.view.items))
        if not self.view.items:
            return

        self.stop_request = False
        self.is_complete = True
        self.core.globalmenu.set_enable_trailer(True)
        self.ui.toolbar.activate_trailer_widgets(True)
        self.ui.toolbar.set_trailer_slider_max(len(self.view.items)-1)
        self.set_current_item(idx)
        self.view.set_centered(idx)
        self.ui.toggle_trailer(self.core.cfg.data['startup']['trailer'])

    def remove_at_index(self, idx):
        self.view.remove_at(idx)

    def get_current_trailer(self):
        return self.view

    def on_trailer_moved(self, idx):
        lgg.info('Trailer moved at: %s' % idx)
        if idx == -1:
            idx = self.core.current_index
        self.ui.toolbar.set_trailer_slider_value(idx)

    def move_to_index(self, index):
        lgg.info('Move trailer with slider at: %s' % index)
        self.view.blockSignals(True)
        self.view.scroll_to_item(index)
        self.view.blockSignals(False)

    def _hide(self):
        """Hide the trailer.

        Called by the item's contextual menu
        """
        self.ui.toggle_trailer()
#-------------------------------------------------------------------------------


class TrailerItem(QtWidgets.QGraphicsPixmapItem):
    """Define a thumbnail item in the trailer.

    """
    def __init__(self, driver, path, data, image):
        super().__init__()
        self.driver = driver
        self.path = path
        self.data = data
        self.image = image
        self.is_current_image = False
        self.frame = None
        self.opacity = 1
        pix = QtGui.QPixmap(path)
        if pix.isNull():
            pix = QtGui.QPixmap("medias/No_Preview_200x150.png")

        self.setPixmap(pix)
        self.setOpacity(0.8)
        self.setFlags(QtWidgets.QGraphicsItem.ItemIsSelectable)
        self.setTransformationMode(QtCore.Qt.SmoothTransformation)
        self.setAcceptHoverEvents(True)

    def get_size(self):
        return (self.boundingRect().width(), self.boundingRect().height())

    def get_pos(self):
        return (self.boundingRect().left(), self.boundingRect().top())

    def mousePressEvent(self, event):
        if event.button() == 1:
            self.change_current_image()
            self.set_frame()

    def hoverEnterEvent(self, event):
        self.driver.on_item_hoover(self)

    def hoverLeaveEvent(self, event):
        self.driver.on_item_exited(self)

    def contextMenuEvent(self, event):
        self.show_context_menu(event.screenPos())

    def set_current(self):
        self.set_frame()

    def set_frame(self):
        s = self.get_size()
        self.frame = Frame(s[0], s[1], self)

    def set_toolTip(self):
        txt = "Image: <b>%s</b>" % str(self.idx + 1)
        if self.data is not None:
            if self.data['iso'] is not None:
                txt += '<br>ISO: <b>%s</b>' % self.data['iso']
            if self.data['fn'] is not None:
                txt += '<br>Fnumber: <b>%s</b>' % self.data['fn']
            if self.data['exp'] is not None:
                txt += '<br>Speed: <b>%s</b>' % self.data['exp']

        self.setToolTip(txt)

    def change_current_image(self):
        self.driver.set_current_image(self)

    def show_context_menu(self, pos):
        self.menu = QtWidgets.QMenu()
        select = QtWidgets.QAction(self.menu)
        select.setText(_('Select the image'))
        self.menu.addAction(select)
        close_ = QtWidgets.QAction(self.menu)
        close_.setText(_('Close the trailer'))
        self.menu.addAction(close_)
        select.triggered.connect(self.change_current_image)
        close_.triggered.connect(self.driver._hide)
        self.menu.popup(pos)



class Frame(QtWidgets.QGraphicsPixmapItem):
    def __init__(self, w, h, parent=None, scene=None):
        """Frame is selection rectangle used for the current image.

        Args:
        w, h -- width & height of the thumbnail
        """
        super().__init__(parent)
        if scene is not None: 
            scene.addItem(self)
        color = QtGui.QColor()
        color.setRgb(0, 0, 0, 0)
        pix = QtGui.QPixmap(QtCore.QSize(w, h))
        pix.fill(color)
        color.setRgb(0, 0, 255, 255)
        painter = QtGui.QPainter()
        painter.begin(pix)
        painter.setPen(QtGui.QPen(color, 2,
                              QtCore.Qt.SolidLine, QtCore.Qt.RoundCap,
                              QtCore.Qt.MiterJoin))
        painter.drawRect(1.0, 1.0, w - 2, h - 2)
        painter.end()
        self.setPixmap(pix)
        self.setPos(0, 0)

class ToolTipBuilder(Thread):
    def __init__(self):
        """Initiate the toolTip builder.

        ToolTip are made in a thread after the trailer is completed.
        """
        self.items = []
        Thread.__init__(self)

    def run(self):
        for i in self.items:
            i.set_toolTip()


class BuildThumbnails(Thread):
    """Thumbnails builder

    """
    def __init__(self, trail, core):
        self.trail = trail
        self.core = core
        self.utils = core.utils
        self.thumb_fld = core.folders[0].thumbs_fld
        self.thumbBuilder = ThumbnailBuilder(core.trailer_fld)
        self.thumbBuilder.config(core.cfg[1022], core.cfg[1028])
        self.thumbBuilder.thumbSaved.connect(trail.add_new_thumb)
        self.basename = int(str(time.time()).replace('.', '')[4:])
        Thread.__init__(self)
    
    def run(self):
        for i in self.core.folders[0].files:
            data = None
            th = self.utils.source_thumb_name(self.thumb_fld, i)
            if not os.path.isfile(th):
                img = QtGui.QImage(i)
                if img.isNull():
                    img = QtGui.QImage("medias/no_icon.png")

                if img.width() > 200 or img.height() > 200:
                    img = img.scaled(200, 200, QtCore.Qt.KeepAspectRatio, 
                                    QtCore.Qt.SmoothTransformation)
                img.save(th, None, -1)

            if os.path.splitext(i)[1].lower() in [".jpg", ".jpeg", 
                                                  ".tif", ".tiff"]:
                data = self.trail.get_data(i)
            self.basename += 1
            self.thumbBuilder.build_thumb(i, th, str(self.basename), data)

        self.trail.set_trailer_button()

class BuildRawThumbnails(Thread):
    """Thumbnails builder for RAW files

    """
    def __init__(self, trail, core):
        self.trail = trail
        self.core = core
        self.utils = core.utils
        self.thumb_fld = core.folders[0].thumbs_fld
        self.thumbBuilder = ThumbnailBuilder(core.trailer_fld)
        self.thumbBuilder.config(core.cfg[1022], core.cfg[1028])
        self.thumbBuilder.thumbSaved.connect(trail.add_new_thumb)
        self.basename = int(str(time.time()).replace('.', '')[4:])
        Thread.__init__(self)
    
    def run(self):
        for i in self.core.folders[0].raws:
            th = self.utils.source_thumb_name(self.thumb_fld, i)
            if not os.path.isfile(th):
                pass
