# -*- coding: utf-8 -*-

# utils.py
# This file is part of Oqapy serie 3
#
# Author: Vincent Vande Vyvre <vincent.vandevyvre@oqapy.eu>
# Home: http://www.oqapy.eu
# Copyright: 2009-2017 Vincent Vande Vyvre
# Licence: GPL3
#
# Tool-box for Oqapy

import os
import sys
import imghdr
import shutil
import logging
import subprocess

from urllib import request

from PyQt5.QtCore import QLocale
from PyQt5.QtGui import QColor

from tools.path import Path

lgg = logging.getLogger(__name__)


class Utils:
    def __init__(self, core):
        self.core = core
        self.loc = QLocale.system().name()
        self.lang = QLocale(self.loc)

    def define_workspace(self):
        lgg.info('Define_workspace')
        root = Path(os.path.join(os.path.expanduser("~"), ".oqapy"))
        if not root.ensure_folder_exists():
            lgg.critical("Can't create workspace: %s" % root.path)
            sys.exit()

        workspace = {}
        workspace['root'] = root
        workspace['temp'] = root / "temp"
        workspace['trailer'] = root / "trailer"
        #conf = os.path.join(root, "config")
        workspace['trash'] = root / "trash"
        workspace['trash_thumbs'] = workspace['trash'] / "thumbnails"
        # Thumbnails folder, used if the source folder is read only
        workspace['thumbs'] = root / "thumbnails"
        workspace['histo'] = root / "histo"
        #workspace['logs'] = root / "logs"
        workspace['camera'] = root / "camera"
        workspace['g9n'] = root / "g9n"
        workspace['bckups'] = root / "changes"
        workspace['bckups_thumbs'] = workspace['bckups'] / "thumbnails"
        workspace['raws'] = root / "raws_tmp"
        workspace['darks'] = root / "darks_tmp"
        # RAW previews folder, used if the source folder is read only
        workspace['rprev'] = root / "raw_previews"
        workspace['docs'] = root / "docs"

        for path in workspace:
            workspace[path].ensure_folder_exists()

        return workspace

    def valid_image(self, path):
        """Verify if an image is valid.

        Tests the image data and returns a string describing the image type.

        Args:
        path -- file name (full path)

        Returns:
        The image format if valid, None otherwise
        """
        try:
            f = imghdr.what(path)
        except IOError:
            return

        # Normalise
        if f == "jpeg":
            f = "jpg"

        return f

    def valid_raw_image(self, path):
        """Verify if an image is readable by dcraw.

        Args:
        path -- file name (full path)

        Returns:
        boolean
        """
        reply = subprocess.Popen(["dcraw", "-i", path],
                                universal_newlines=True,
                                stdout=subprocess.PIPE)

        while 1:
            if reply.poll() != None:
                break

        if reply.poll() == 1:
            # Unvalid raw file
            return False

        else:
            return True

    def clean_all_temp_folders(self):
        for name in ["temp", "trailer", "thumbs", "histo", "camera", 
                     "bckups", "bckups_thumbs", "raws", "darks", "rprev"]:
            path = self.core.workspace[name].path
            self.clean_temp_folder(path)

    def clean_temp_folder(self, temp):
        for i in (os.listdir(temp)):
            fpath = os.path.join(temp, i)
            if os.path.isfile(fpath):
                try:
                    os.remove(fpath)
                except Exception as why:
                    lgg.warning("Can't remove %s %s" %(i, why))
        
    def get_thumb_name(self, path, fld=None):
        """Set the full path of thumbnail from the path of an image.

        ex.: path = /home/user/pictures/family/lena.jpg
        return: /home/user/pictures/family/thumbnails/thumb_lena<jpg>.jpg

        Args:
        path -- full path of image
        fld -- path of thumbnails folder if different dir of the image,
               i.e. the image is on unwritable support

        Returns:
        Full path of thumbnail.
        """
        caput, cauda = os.path.split(path)
        if fld is None:
            fld = os.path.join(caput, 'thumbnails')
        name, ext = os.path.splitext(cauda)
        ext = '<%s>' % ext[1:]
        f = "".join(["thumb_", name, ext, ".jpg"])
        return os.path.join(fld, f)


    def format_number(self, numb):
        """Return a number formatted according to the locale parameters.

        Args:
        numb -- int(number) or str(number)

        Returns:
        str() or numb if fail
        """
        if isinstance(numb, str):
            try:
                numb = eval(numb)
            except:
                return numb

        try:
            return self.lang.toString(numb)
        except:
            return numb

    def get_qcolor(self, c, replace=None):
        """Return a QColor instance.

        Args:
        c -- str('#c5a456') or tuple(r, v, b)
        replace -- a QColor returned if c is not valid
        """
        if isinstance(c, QColor):
            return c

        if isinstance(c, str):
            if not c.startswith('#'):
                return replace

        elif isinstance(c, tuple):
            if len(c) < 3:
                return replace

            c = "".join(['#', hex(c[0])[2:], hex(c[1])[2:], hex(c[1])[2:]])

        try:
            return QColor(c)
        except:
            return replace



