# -*- coding: utf-8 -*-

# viewing.py
# This file is part of Oqapy serie 3
#
# Author: Vincent Vande Vyvre <vincent.vandevyvre@oqapy.eu>
# Home: http://www.oqapy.eu
# Copyright: 2009-2016 Vincent Vande Vyvre
# Licence: GPL3
#
# Viewing engine

import os
import logging
lgg = logging.getLogger(__name__)

from PyQt5.QtCore import pyqtSignal
from PyQt5.QtWidgets import QGraphicsPixmapItem, QGraphicsView


class Viewing:
    loadingImage = pyqtSignal()
    imageLoaded = pyqtSignal(int, int)
    imageChanged = pyqtSignal(int, QGraphicsPixmapItem, float)
    unvalidImage = pyqtSignal(int, str)
    emptyList = pyqtSignal()

    def __init__(self, core, viewer, scene=None, first=0):
        """Initialise viewing.

        Args:
        core -- instance of OqapyCore
        viewer -- QGraphicsView instance
        scene -- QGraphicsScene, will be created if None, default = None
        first -- index of first image displayed into the viewer, default = 0

        Signals:
        loadingImage() -- emitted when the current image is loading, useful for 
                            showing a busy cursor, progressBar, etc.
        imageLoaded(int, QImage) -- emitted when the QImage is created, values
                            are index of the image in list and QImage instance.
        imageChanged(int, QGraphicsPixmapItem, float) -- emitted when an image 
                            is showed, values are index of the image in list,
                            the QGraphicsItem instance and the zoom factor.
        unvalidImage(int, str) -- emitted when an image == nullImage(), values
                            are index and full path of the image. Note: this
                            image is removed from the list.
        emptyList() -- emitted when the list of images becomes empty.
        """ 
        super().__init__()
        self.core = core
        self.viewer = viewer
        self.scene = scene
        self.images = []
        self.first = first
        self.gr_width = 0
        self.gr_height = 0
        self.is_running = False
        self.counter = 0
        self.current_zoom = 1
        self.step_zoom = 0.06
        self.zoom_min = 0.01
        self.zoom_max = 10.0
        self.current_qpixmap = None
        #if viewer is not None:
            #self.set_viewer(viewer)

    def set_viewer(self, viewer):
        self.viewer = viewer
        try:
            self.scene = viewer.scene_
        except AttributeError as why:
            lgg.info('No scene: %s' % why)
            self.viewer.set_scene()
            self.scene = self.viewer.scene_

    def run_viewing(self):
        """Run the viewing.

        """
        lgg.info('Run viewing %s images' % len(self.images))
        if not isinstance(self.viewer, QGraphicsView):
            raise TypeError("Can't run Viewing: no such QGraphicsView")

        if not self.images:
            self.emptyList.emit()
            return

        if abs(self.first) >= len(self.images):
            self.first = 0

        self.counter = self.first - 1
        self.is_running = True
        self.next_view()

    def count(self):
        return len(self.images)

    def next_view(self):
        self.increment_counter()
        self.load_picture()

    def previous_view(self):
        self.decrement_counter()
        self.load_picture()

    def load_picture(self):
        filename = self.images[self.counter]
        if not self.validate_file(filename):
            return

        lgg.info('Load file: %s' % filename)
        self.core.set_current_picture(filename, self.counter)
        self.viewer.set_source_image(self.core.current_picture)
        if self.viewer.set_qimage_from_source():
            self.viewer.refresh_qpixmap()
            qimage = self.viewer.get_qimage()
            self.core.update_navigation_viewer(qimage)

        else:
            self.core.reject_current_image()

    def validate_file(self, path):
        if not os.path.isfile(path):
            lgg.info('File %s not found' % path)
            self.core.remove_image(path)
            return False

        return True

    def rotate_to_right(self):
        self.viewer.rotate_image(90)
        self.viewer.refresh_qpixmap()

    def rotate_to_left(self):
        self.viewer.rotate_image(-90)
        self.viewer.refresh_qpixmap()

    def zoom_on_area(self):
        self.viewer.set_zoom_area(True)

    def set_full_size(self):
        self.viewer.full_size()

    def zoom_up(self):
        self.viewer.zoom_in()

    def zoom_at_value(self, zoom):
        self.viewer.current_zoom = zoom / 100.0
        self.viewer.refresh_qpixmap(0)

    def zoom_down(self):
        self.viewer.zoom_out()

    def fit_in_view(self):
        self.viewer.fit_in_view()

    def increment_counter(self):
        self.counter = (self.counter + 1) % len(self.images)

    def decrement_counter(self):
        self.counter = (self.counter - 1) % len(self.images)
