# -*- coding: utf-8 -*-

# warningmessages.py
# This file is part of Oqapy serie 3
#
# Author: Vincent Vande Vyvre <vincent.vandevyvre@oqapy.eu>
# Home: http://www.oqapy.eu
# Copyright: 2016-2017 Vincent Vande Vyvre
# Licence: GPL3
#
# Warnings handler

import os

from PyQt5.QtCore import Qt, QSize
from PyQt5.QtGui import QPixmap, QFont
from PyQt5.QtWidgets import (QApplication, QDialog, QGridLayout, QVBoxLayout, 
                             QHBoxLayout, QGraphicsView, QGraphicsScene,
                             QLabel, QSpacerItem, QSizePolicy, QPushButton, 
                             QWidget, )

import stylesheets as styles

class WarningMessages:
    def __init__(self, idx, data):
        self.idx = idx
        self.data = self.get_message()
        for key, value in data.items():
            self.data[key] = value

    def build_dialog(self):
        wd = WarningDialog(self.idx)
        wd.action_type_lbl.setText(self.data['context'])
        if self.data['object'] is not None:
            wd.action_val_lbl.setText(self.data['object'])
            wd.action_val_lbl.show()
        wd.warning_val_lbl.setText(self.data['warn'])
        keys = ('data', 'msg', 'data1', 'msg1')
        widgets = (wd.data_lbl, wd.message_lbl, wd.data_1_lbl, wd.message_1_lbl)
        for key, widget in zip(keys, widgets):
            if self.data[key] is not None:
                widget.setText(self.data[key])
                widget.show()

        if self.idx == 1:
            exist = os.path.isfile(self.data['img'])
            wd.set_thumbnail(0, self.data['img'])
            wd.set_thumbnail_data(0, self.data['img_data'])
            wd.set_thumbnail(1, self.data['img1'])
            wd.set_thumbnail_data(1, self.data['img1_data'])

        wd.accept_btn.setText(self.data['btn'])
        keys = ('btn_1', 'btn_2', 'btn_3', 'btn_4')
        buttons = (wd.reject_btn, wd.option_1_btn, 
                   wd.option_2_btn, wd.option_3_btn)
        for key, button in zip(keys, buttons):
            if self.data[key] is not None:
                button.setText(self.data[key])
                button.show()

        wd.setMaximumSize(self.data['size'])
        wd.resize(self.data['size'])
        return wd.exec_()

    def get_message(self):
        """Return a dictionnary with the components of the warning.

        Args:
        msg -- int() index of warning message

        Returns:
        dict using this model:
              {'context': None, 'object': None, 
                'warn': None,
                'data': None, 
                'msg': None, 
                'data1': None,
                'msg1': None, 
                'img': None,
                'img_data': None,
                'img1': None,
                'img1_data': None,
                'btn': _('Close'),
                'btn_1': _('Cancel'),
                'btn_2': _('Apply'),
                'btn_3': _('Extra'),
                'btn_4': _('Option'),
                'size': QSize(350, 150)}  
        """
        messages = [{'context': None, 'object': None, 
                    'warn': None,
                    'data': None, 
                    'msg': None, 
                    'data1': None,
                    'msg1': None, 
                    'img': None,
                    'img_data': None,
                    'img1': None,
                    'img1_data': None,
                    'btn': _('Close'),
                    'btn_1': None,
                    'btn_2': None,
                    'btn_3': None,
                    'btn_4': None,
                    'size': QSize(250, 150)},
                    # mssg 1
                    {'context': _('Copy of'), 
                    'object': None, 
                    'warn': _('A file with same name already exists in:'),
                    'data': None,
                    'msg': _('Do you want to replace it ?'),
                    'data1': None, 
                    'msg1': None,
                    'img': None,
                    'btn': _('Replace'),
                    'btn_1': _('Ignore'),
                    'btn_2': None,
                    'btn_3': None,
                    'btn_4': None,
                    'text': None,
                    'size': QSize(400, 500)},
                    # mssg 2
                     {'context': _('Resize image:'),
                    'object': None, 
                    'msg': _('Unable to resize the image'),
                    'msg_1': None, 'msg_2': None, 'dat': None,
                    'dat_1': None, 'img': None,
                    'btn': _('Close'),
                    'btn_1': None,
                    'btn_2': None,
                    'btn_3': None,
                    'btn_4': None,
                    'text': None,
                    'size': None},
                    # mssg 3
                    {'context': _('Select Trash can folder'), 
                    'object': None, 'msg': _("Can't create folder:"),
                    'msg_1': _('Reason:'), 
                    'msg_2': None, 'dat': None,
                    'dat_1': None, 'img': None,
                    'btn': _('Close'),
                    'btn_1': None,
                    'btn_2': None,
                    'btn_3': None,
                    'text': None,
                    'size': QSize(350, 150)},
                    # mssg 4
                    {'context': _('Create a selection list'), 
                    'object': None, 'msg': _('A list named:'),
                    'msg_1': _('already exists.'), 
                    'msg_2': _("Do you want to replace it or to complete " 
                                    "it with these new files ?"),
                    'dat': None,
                    'dat_1': None, 'img': None,
                    'btn': None,
                    'btn_1': _("Cancel"),
                    'btn_2': _("Complete"),
                    'btn_3': _("Replace"),
                    'text': None,
                    'size': QSize(350, 150)},]

        return messages[self.idx]
        
        
class WarningDialog(QDialog):
    def __init__(self, style):
        super().__init__()
        self.setWindowTitle(_("Oqapy - Warning"))
        pal = styles.get_palette()
        self.setPalette(pal)
        self.setWindowModality(Qt.WindowModal)
        self.setModal(True)
        font = QFont()
        font.setWeight(75)
        font.setBold(True)
        self.glayout = QGridLayout(self)
        self.vlayout = QVBoxLayout()
        self.glayout1 = QGridLayout()
        self.action_lbl = QLabel(self)
        self.action_lbl.setText(_("Action:"))
        self.action_lbl.setStyleSheet("QLabel {color: #758088;}")
        self.hlayout = QHBoxLayout()
        self.glayout1.addWidget(self.action_lbl, 0, 0, 1, 1)
        self.hlayout1 = QHBoxLayout()
        self.action_type_lbl = QLabel(self)
        self.action_type_lbl.setStyleSheet("QLabel {color: #758088;}")
        self.hlayout1.addWidget(self.action_type_lbl)
        self.action_val_lbl = QLabel(self)
        self.action_val_lbl.setFont(font)
        self.action_val_lbl.setStyleSheet("QLabel {color: #758088;}")
        self.hlayout1.addWidget(self.action_val_lbl)
        self.glayout1.addLayout(self.hlayout1, 0, 1, 1, 1)
        self.warning_lbl = QLabel(self)
        self.warning_lbl.setText(_("Warning:"))
        self.warning_lbl.setStyleSheet("QLabel {color: #758088;}")
        self.glayout1.addWidget(self.warning_lbl, 1, 0, 1, 1)
        self.warning_val_lbl = QLabel(self)
        self.glayout1.addWidget(self.warning_val_lbl, 1, 1, 1, 1)
        self.data_lbl = QLabel(self)
        self.data_lbl.setFont(font)
        self.data_lbl.setStyleSheet("QLabel {color: #758088;}")
        self.glayout1.addWidget(self.data_lbl, 2, 1, 1, 1)
        self.hlayout.addLayout(self.glayout1)
        sp = QSpacerItem(40, 20, QSizePolicy.Expanding, QSizePolicy.Minimum)
        self.hlayout.addItem(sp)
        self.vlayout.addLayout(self.hlayout)

        self.message_lbl = QLabel(self)
        self.message_lbl.setStyleSheet("QLabel {color: #758088;}")
        self.vlayout.addWidget(self.message_lbl)
        self.data_1_lbl = QLabel(self)
        self.data_1_lbl.setFont(font)
        self.data_1_lbl.setStyleSheet("QLabel {color: #758088;}")
        self.vlayout.addWidget(self.data_1_lbl)
        self.message_1_lbl = QLabel(self)
        self.message_1_lbl.setStyleSheet("QLabel {color: #758088;}")
        self.vlayout.addWidget(self.message_1_lbl)
        if style == 1:
            self.hlayout2 = QHBoxLayout()
            self.src_wdg = ThumbnailViewer(_('Source'), self)
            self.src_wdg.setMaximumSize(QSize(300, 600))
            self.hlayout2.addWidget(self.src_wdg)
            self.dest_wdg = ThumbnailViewer(_('Target'), self)
            self.dest_wdg.setMaximumSize(QSize(300, 600))
            self.hlayout2.addWidget(self.dest_wdg)
            self.vlayout.addLayout(self.hlayout2)
        self.hlayout3 = QHBoxLayout()
        spacer = QSpacerItem(40, 20, QSizePolicy.Expanding, QSizePolicy.Minimum)
        self.hlayout3.addItem(spacer)
        self.option_3_btn = QPushButton(self)
        self.hlayout3.addWidget(self.option_3_btn)
        self.option_2_btn = QPushButton(self)
        self.hlayout3.addWidget(self.option_2_btn)
        self.option_1_btn = QPushButton(self)
        self.hlayout3.addWidget(self.option_1_btn)
        self.reject_btn = QPushButton(self)
        self.hlayout3.addWidget(self.reject_btn)
        self.accept_btn = QPushButton(self)
        self.accept_btn.setText("Accept")
        self.hlayout3.addWidget(self.accept_btn)
        self.vlayout.addLayout(self.hlayout3)
        self.glayout.addLayout(self.vlayout, 0, 0, 1, 1)
        self.action_val_lbl.hide()
        self.data_lbl.hide()
        self.message_lbl.hide()
        self.data_1_lbl.hide()
        self.message_1_lbl.hide()
        self.option_3_btn.hide()
        self.option_2_btn.hide()
        self.option_1_btn.hide()
        self.reject_btn.hide()
        self.action_type_lbl.setText("actiontype")
        self.action_val_lbl.setText("actionvalue")
        self.warning_val_lbl.setText("warningvalue")
        self.data_lbl.setText("data")
        self.message_lbl.setText("message")
        self.data_1_lbl.setText("data")
        self.message_1_lbl.setText("message")
        self.accept_btn.clicked.connect(self.accept)
        self.reject_btn.clicked.connect(self.reject)
        self.option_1_btn.clicked.connect(self.on_option_1)
        self.option_2_btn.clicked.connect(self.on_option_2)
        self.option_3_btn.clicked.connect(self.on_option_3)

    def set_thumbnail(self, side, thumb):
        wdg = (self.src_wdg, self.dest_wdg)[side]
        if thumb is None:
            wdg.hide()
            return

        wdg.set_pixmap(thumb)

    def set_thumbnail_data(self, side, data):
        if data is None:
            return

        (self.src_wdg, self.dest_wdg)[side].set_data(data)

    def on_option_1(self):
        self.done(2)
    def on_option_2(self):
        self.done(3)
    def on_option_3(self):
        self.done(4)


class ThumbnailViewer(QWidget):
    def __init__(self, label, parent):
        super().__init__(parent)
        self.glayout = QGridLayout(self)
        self.vlayout = QVBoxLayout()
        self.orig_lbl = QLabel(self)
        self.orig_lbl.setText(label)
        self.orig_lbl.setAlignment(Qt.AlignCenter)
        self.vlayout.addWidget(self.orig_lbl)
        self.viewer = QGraphicsView(self)
        self.vlayout.addWidget(self.viewer)
        self.glayout1 = QGridLayout()
        self.date_lbl = QLabel(self)
        self.date_lbl.setText(_("Date:"))
        alignment = Qt.AlignRight | Qt.AlignTrailing | Qt.AlignVCenter
        self.date_lbl.setAlignment(alignment)
        self.glayout1.addWidget(self.date_lbl, 0, 0, 1, 1)
        self.date_val_lbl = QLabel(self)
        self.glayout1.addWidget(self.date_val_lbl, 0, 1, 1, 1)
        self.size_lbl = QLabel(self)
        self.size_lbl.setText(_("Size:"))
        self.size_lbl.setAlignment(alignment)
        self.glayout1.addWidget(self.size_lbl, 1, 0, 1, 1)
        self.size_val_lbl = QLabel(self)
        self.glayout1.addWidget(self.size_val_lbl, 1, 1, 1, 1)
        self.iso_lbl = QLabel(self)
        self.iso_lbl.setText("ISO:")
        self.iso_lbl.setAlignment(alignment)
        self.glayout1.addWidget(self.iso_lbl, 2, 0, 1, 1)
        self.iso_val_lbl = QLabel(self)
        self.glayout1.addWidget(self.iso_val_lbl, 2, 1, 1, 1)
        self.speed_lbl = QLabel(self)
        self.speed_lbl.setText(_("Shutter speed:"))
        self.speed_lbl.setAlignment(alignment)
        self.glayout1.addWidget(self.speed_lbl, 3, 0, 1, 1)
        self.speed_val_lbl = QLabel(self)
        self.glayout1.addWidget(self.speed_val_lbl, 3, 1, 1, 1)
        self.fnumber_lbl = QLabel(self)
        self.fnumber_lbl.setText(_("Fnumber:"))
        self.fnumber_lbl.setAlignment(alignment)
        self.glayout1.addWidget(self.fnumber_lbl, 4, 0, 1, 1)
        self.fnumber_val_lbl = QLabel(self)
        self.glayout1.addWidget(self.fnumber_val_lbl, 4, 1, 1, 1)
        self.other_lbl = QLabel(self)
        self.other_lbl.setText(_("Other:"))
        self.other_lbl.setAlignment(alignment)
        self.glayout1.addWidget(self.other_lbl, 5, 0, 1, 1)
        self.other_val_lbl = QLabel(self)
        self.glayout1.addWidget(self.other_val_lbl, 5, 1, 1, 1)
        self.other_1_lbl = QLabel(self)
        self.other_1_lbl.setText(_("Other 1:"))
        self.other_1_lbl.setAlignment(alignment)
        self.glayout1.addWidget(self.other_1_lbl, 6, 0, 1, 1)
        self.other_1_val_lbl = QLabel(self)
        self.glayout1.addWidget(self.other_1_val_lbl, 6, 1, 1, 1)
        self.vlayout.addLayout(self.glayout1)
        self.glayout.addLayout(self.vlayout, 0, 0, 1, 1)

        self.date_val_lbl.setText("    --")
        self.size_val_lbl.setText("    --")
        self.iso_val_lbl.setText("    --")
        self.speed_val_lbl.setText("    --")
        self.fnumber_val_lbl.setText("    --")
        self.other_val_lbl.setText("    --")
        self.other_1_val_lbl.setText("    --")

    def set_pixmap(self, path):
        pix = QPixmap(path).scaled(200, 200, Qt.KeepAspectRatio, 
                                   Qt.SmoothTransformation)
        w, h = pix.width(), pix.height()
        self.viewer.setMaximumSize(QSize(w+4, h+4))
        scene = QGraphicsScene()
        scene.setSceneRect(0.0, 0.0, float(w), float(h))
        scene.addPixmap(pix)
        self.viewer.setScene(scene)

    def set_data(self, data):
        widgets = (self.date_val_lbl, self.size_val_lbl,
                   self.iso_val_lbl, self.speed_val_lbl,
                   self.fnumber_val_lbl)
        keys = ('date','size', 'iso', 'speed', 'fnumber')
        for key, widget in zip(keys, widgets):
            if data[key] is not None:
                widget.setText(data[key])

        
if __name__ == '__main__':
    _ = str
    app = QApplication([])
    data = {'context': 'Copy of:', 'object': 'malta_7813_(5).TIFF', 
                'warn': 'A file with same name already exists in:',
                'data': '/home/vincent/Images/testing/', 
                'msg': 'Do you want to replace it ?', 
                'dat1': None,
                'msg1': None, 
                'img': '/home/vincent/Images/testing/thumbnails/thumb_malta_7813_(5)<TIFF>.jpg',
                'img_data': {'date': 'June-18-2004', 'size': '3143 x 2056',
                             'iso': '100', 'speed': '1/125', 'fnumber': '5.6'},
                'img1': None,
                'img1_data': None,
                'btn': _('Replace'),
                'btn_1': _('Ignore'),
                'btn_2': _('Apply'),
                'btn_3': _('Extra'),
                'btn_4': _('Option'),
                'size': QSize(450, 500)}
    warn = WarningMessages(1, data)
    print(warn.build_dialog())



